package AccessRanking;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# アクセスランキングモジュール
# AccessRanking.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-07-24
######################################################################
# メイン
sub run ($) {
    my $this = shift;

    #ランキング設定
    $this->use_deco;
    $this->ranking_setting;

    #ヘッダ
    $this->set_og_url( $this->my_url );
    $this->print_page_header('ranking');
    $this->contents_header;

    #カウント
    $this->count;

    #結果報告
    $this->print_report;

    #フッタ
    $this->print_tool_icon if $this->is_secure('tools');
    $this->print_page_footer('ranking');

}

######################################################################
# カウント
sub count ($) {
    my $this = shift;
    my ( @localtime, $month, $date, $date2 );

    @localtime = localtime(time);
    $month = ( $localtime[5] + 1900 ) . sprintf( "%.2d", ++$localtime[4] );
    $date = $month . sprintf( "%.2d", $localtime[3] );

    @localtime = localtime( time - 86400 );
    $date2
        = ( $localtime[5] + 1900 )
        . sprintf( "%.2d", ++$localtime[4] )
        . sprintf( "%.2d", $localtime[3] );

    # アクセスカウント情報読み書き
    my (%counter);
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie( %counter, $env::dbm_file,
        $default::access_counter . '/access_counter',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %counter ) {
        while ( my ( $location, $data ) = each %counter ) {
            utf8::decode($location) if !utf8::is_utf8($location);
            utf8::decode($data)     if !utf8::is_utf8($data);

            #解析対象ファイルチェック
            next
                if @setting::target
                && grep( eval { $location =~ /$_/ }, @setting::target ) == 0;

            my (@data);
            @data = split( /\x09/, $counter{$location} );
            $this->{'access_count'}->{$location}
                = $this->access_count( \@data, $month, $date, $date2 );
        }
    }
    untie %counter;
}

sub access_count ($$$$$) {
    my ( $this, $data, $month, $date, $date2 ) = @_;
    if ( defined $setting::ranking{'span'}
        && $setting::ranking{'span'} eq 'thismonth' )
    {
        if ( defined $$data[1] && defined $month && $$data[1] eq $month ) {
            return $$data[2];
        }
        else {
            return 0;
        }
    }
    elsif ( defined $setting::ranking{'span'}
        && $setting::ranking{'span'} eq 'today' )
    {
        if ( defined $$data[3] && defined $date && $$data[3] eq $date ) {
            return $$data[4];
        }
        else {
            return 0;
        }
    }
    elsif ( defined $setting::ranking{'span'}
        && $setting::ranking{'span'} eq 'yesterday' )
    {
        if ( defined $$data[5] && defined $date2 && $$data[5] eq $date2 ) {
            return $$data[6];
        }
        elsif ( defined $$data[3] && defined $date2 && $$data[3] eq $date2 ) {
            return $$data[4];
        }
        else {
            return 0;
        }
    }
    else {
        return $$data[0];
    }
}

######################################################################
# 結果表示
sub print_report ($) {
    my $this = shift;

    #my(@ranking,$tablewidth,$grafwidth,$lastpage,$max);
    my ( @ranking, $lastpage, $max );

    #ソート
    $setting::ranking{'sort'} = 'descend'
        if !defined $setting::ranking{'sort'}
        || $setting::ranking{'sort'} ne 'ascend';
    @ranking = sort {
        $this->sort_by(
            \$this->{'access_count'}->{$a},
            \$this->{'access_count'}->{$b},
            $setting::ranking{'sort'}
            )
            || $a cmp $b
    } keys %{ $this->{'access_count'} };

    #最大値
    $max
        = ( ref $this->{'access_count'} && @ranking )
        ? ( $setting::ranking{'sort'} eq 'ascend' )
            ? $this->{'access_count'}->{ $ranking[$#ranking] }
            : $this->{'access_count'}->{ $ranking[0] }
        : 0;

#幅
#$grafwidth = ($this->{'Table'}->{'cellpadding'} * 2) + ($this->{'Table'}->{'cellspacing'} * 2) + $this->{'Table'}->{'border'} + $this->{'BarGraph'}->{'barWidth'};
#$tablewidth = 430 + $grafwidth;

    $this->{'Table'}->set(

        #	'tablewidth' => $tablewidth,
        'styleH' =>
            'text-align: center; width: 30px, text-align: center; width: 300px, text-align: center; width: 100px, text-align: center; width: '
            . $this->{'BarGraph'}->{'barWidth'} . 'px',
        'style0' =>
            'text-align: center,text-align: center,text-align: right,text-align: left'
    );

    #ページ数
    $setting::ranking{'u'} = 40 if $this->is_nan( $setting::ranking{'u'} );
    $lastpage
        = ( $setting::ranking{'u'} )
        ? ( ( @ranking % $setting::ranking{'u'} )
        ? int( @ranking / $setting::ranking{'u'} ) + 1
        : @ranking / $setting::ranking{'u'} )
        : 1;
    if ( !$main::p || $main::p < 1 ) {
        $main::p = 1;
    }
    elsif ( $main::p > $lastpage ) {
        $main::p = $lastpage;
    }
    print '<div id="ranking-report">';
    print ' (' . $main::p . '/' . $lastpage . 'ページ)' if $lastpage > 1;
    print $this->{'Table'}->table;
    print $this->{'Table'}->tr;
    print $this->{'Table'}->td('No.');
    print $this->{'Table'}->td('URL');
    print $this->{'Table'}->td('アクセス数');
    print $this->{'Table'}->td('グラフ (件)');
    print $this->{'Table'}->_tr;

    #解析結果
    my ( $count, $ranking, $location, $prev, $pageoption );
    foreach $location (@ranking) {
        $count++;

        #ページ
        if ( $setting::ranking{'u'} > 0 ) {
            if ( $setting::ranking{'u'} * ( $main::p - 1 ) >= $count ) {
                $pageoption = 1;
                next;
            }
            elsif ( $setting::ranking{'u'} * $main::p < $count ) {
                $pageoption = 1;
                last;
            }
        }

        #ランキング
        $ranking = $count
            if ( defined $this->{'access_count'}->{$location}
            && defined $prev
            && $this->{'access_count'}->{$location} != $prev )
            || !$ranking;

        print $this->{'Table'}->tr;
        print $this->{'Table'}->td($ranking);
        print $this->{'Table'}->td( $this->ranking_key($location) );
        print $this->{'Table'}->td( ( $this->{'access_count'}->{$location} )
            ? $this->{'access_count'}->{$location}
            : 0 );
        print $this->{'Table'}->td(
            $this->{'BarGraph'}->bargraph(
                $this->{'access_count'}->{$location},
                $max, '件'
            )
        );
        print $this->{'Table'}->_tr;
        $prev = $this->{'access_count'}->{$location};
    }
    print $this->{'Table'}->_table;

    if ( defined $pageoption && $pageoption ) {
        my $form_flag = 0;
        if ( $setting::general{'postMethod'} ) {
            print $this->form_start(
                'key'      => 'ranking',
                'name'     => 'input',
                'required' => \@setting::form
            );
            $form_flag = 1;
        }
        else {
            $this->use_deco;
        }

        print '<ul class="tools">';
        if ( $main::p > 1 ) {
            print '<li>';
            if ($form_flag) {
                print $cgi::Deco->submit_button( 'prev',
                    $setting::page{'ranking'}{'prevButton'},
                    '前のページ', '', { 'p' => ( $main::p - 1 ) } );
            }
            else {
                print $cgi::Deco->button(
                    'prev',
                    $setting::page{'ranking'}{'prevButton'},
                    './start.cgi'
                        . $this->set_query( 'url', \@setting::form )
                        . $this->add_query( 'url', { 'p' => $main::p - 1 } ),
                    '前のページ'
                );
            }
            print '</li>';
        }
        if ( $main::p < $lastpage ) {
            print '<li>';
            if ($form_flag) {
                print $cgi::Deco->submit_button( 'next',
                    $setting::page{'ranking'}{'nextButton'},
                    '次のページ', '', { 'p' => ( $main::p + 1 ) } );
            }
            else {
                print $cgi::Deco->button(
                    'next',
                    $setting::page{'stat'}{'nextButton'},
                    './start.cgi'
                        . $this->set_query( 'url', \@setting::form )
                        . $this->add_query( 'url', { 'p' => $main::p + 1 } ),
                    '次のページ',
                );
            }
            print '</li>';
        }
        print '</ul>';
        print '</form>' if $form_flag;
    }
    print '</div>';
}

######################################################################
# キー項目調整
sub ranking_key ($$) {
    my ( $this, $key ) = @_;
    return
          '<a href="'
        . $key
        . '" target="_blank">'
        . $this->jfold( $this->query_decode($key), 40, 'r', 0, '<wbr />' )
        . '</a>';
}

sub query_decode ($$) {
    my ( $this, $query ) = @_;
    utf8::encode($query);
    $query =~ s/\+/ /g;
    $query =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
    return $this->jcode_convert( \$query, 'utf8' );
}

######################################################################
# コンテンツヘッダ
sub contents_header ($) {
    my $this = shift;

    #解析条件テーブル
    $this->{'infoTable'} = Table->new;
    if ( defined $setting::page{'ranking'}{'infoTable'}
        && ref $table::obj{ $setting::page{'ranking'}{'infoTable'} } )
    {
        %{ $this->{'infoTable'} }
            = %{ $table::obj{ $setting::page{'ranking'}{'infoTable'} } };
        $this->{'infoTable'}->{'fixed'} = 0;
    }
    else {
        $this->{'infoTable'}->set(
            'fixed'      => 0,
            'tableclass' => 'datatable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
        );
    }

    #解析結果テーブル
    $this->{'Table'} = Table->new;
    if ( defined $setting::page{'ranking'}{'table'}
        && ref $table::obj{ $setting::page{'ranking'}{'table'} } )
    {
        %{ $this->{'Table'} }
            = %{ $table::obj{ $setting::page{'ranking'}{'table'} } };
        $this->{'Table'}->{'fixed'} = 0;
    }
    else {
        $this->{'Table'}->set(
            'fixed'      => 0,
            'tableclass' => 'datatable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
            'thenabled'  => 'true'
        );
    }

    #グラフモジュール読み込み
    require CMONOS::CGI_Lib::BarGraph;
    $this->{'BarGraph'} = BarGraph->new(
        \%setting::deco,
        $setting::page{'ranking'}{'barColor'},
        $setting::page{'ranking'}{'bgColor'},
        $setting::page{'ranking'}{'barImage'},
        $setting::page{'ranking'}{'bgImage'},
        $setting::page{'ranking'}{'barWidth'}
    );

    #解析条件
    $this->print_ranking_env;

    #タイトル
    print '<h1>' . $this->ranking_title . '</h1>';
}

sub ranking_title ($) {
    my $this = shift;
    if ( !defined $setting::ranking{'span'} ) {
        return 'アクセス総数ランキング';
    }
    elsif ( $setting::ranking{'span'} eq 'thismonth' ) {
        return '今月のアクセス数ランキング';
    }
    elsif ( $setting::ranking{'span'} eq 'today' ) {
        return '今日のアクセス数ランキング';
    }
    elsif ( $setting::ranking{'span'} eq 'yesterday' ) {
        return '昨日のアクセス数ランキング';
    }
    else {
        return 'アクセス総数ランキング';
    }
}

######################################################################
# 解析条件
sub print_ranking_env ($) {
    my $this = shift;

    #フォーム開始
    print $this->form_start(
        'key'      => 'ranking',
        'name'     => 'input',
        'required' => \@setting::form
    );

    print $this->{'infoTable'}->table( 'id', 'accesscounter-env' );

    #ランキング対象
    my $target
        = ( defined $setting::ranking{'target'} )
        ? $setting::ranking{'target'}
        : '';
    $target =~ s/\\(?!\\)//g
        if !defined $setting::ranking{'regexp'}
        || !$setting::ranking{'regexp'};
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th(
        '<label for="setting:target">ランキング対象ファイル</label>'
    );
    print $this->{'infoTable'}->td(
              '<textarea name="setting:target" id="setting:target" wrap="off"'
            . ' title="パスの一部に設定された文字列を含むファイルのみランキングします。" cols="60" rows="4">'
            . $this->escape($target)
            . '</textarea>'

            . '<div class="external-field"><label for="setting:regexp" class="datatitle">正規表現</label>'
            . '<select name="setting:regexp" id="setting:regexp">'
            . '<option value="">不使用</option>'
            . '<option value="1"'
            . (
            ( $setting::ranking{'regexp'} ) ? ' selected="selected"' : '' )
            . '>使用</option>'
            . '</select></div>'

    );
    print $this->{'infoTable'}->_tr;

    #ランキング設定
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('ランキング設定');
    print $this->{'infoTable'}->td(
        '<label for="setting:u" class="datatitle">表示数</label><input type="text" name="setting:u" id="setting:u" value="'
            . $this->escape( $setting::ranking{'u'} )
            . '" size="3" maxlength="3" />'
            . '<select name="setting:sort">'
            . '<option value="descend"'
            . (
            (   defined $setting::ranking{'sort'}
                    && $setting::ranking{'sort'} eq 'descend'
            ) ? ' selected="selected"' : ''
            )
            . '>降順</option>'
            . '<option value="ascend"'
            . (
            (   defined $setting::ranking{'sort'}
                    && $setting::ranking{'sort'} eq 'ascend'
            ) ? ' selected="selected"' : ''
            )
            . '>昇順</option>'
            . '</select>'
    );
    print $this->{'infoTable'}->_tr;

    #ランキング対象期間
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}
        ->th('<label for="setting:span">ランキング対象期間</label>');
    print $this->{'infoTable'}->td(
              '<select name="setting:span" id="setting:span">'
            . '<option value="">全期間</option>'
            . '<option value="thismonth"'
            . (
            (   defined $setting::ranking{'span'}
                    && $setting::ranking{'span'} eq 'thismonth'
            ) ? ' selected="selected"' : ''
            )
            . '>今月</option>'
            . '<option value="today"'
            . (
            (   defined $setting::ranking{'span'}
                    && $setting::ranking{'span'} eq 'today'
            ) ? ' selected="selected"' : ''
            )
            . '>今日</option>'
            . '<option value="yesterday"'
            . (
            (   defined $setting::ranking{'span'}
                    && $setting::ranking{'span'} eq 'yesterday'
            ) ? ' selected="selected"' : ''
            )
            . '>昨日</option>'
            . '</select>'
            . ' <input type="submit" value="表示する" />'
    );
    print $this->{'infoTable'}->_tr;

    print $this->{'infoTable'}->_table;
    print '</form>';

}

######################################################################
# 設定書き込み/読み込み
sub ranking_setting ($) {
    my $this = shift;

    #書き込み/読み込み
    if (%main::setting) {
        $main::setting{'u'} = '' if $this->is_nan( $main::setting{'u'} );
        $main::setting{'target'} =~ s/[<>"]//gs
            if defined $main::setting{'target'} && $main::setting{'target'};
        $this->write_db( $default::preference_dir . '/ranking',
            \%main::setting, '', '', \%setting::ranking, 1 );

        #読み込みのみ
    }
    else {
        $this->import_db( $default::preference_dir . '/ranking',
            \%setting::ranking, 1 );
    }

    if ( defined $setting::ranking{'target'} && $setting::ranking{'target'} )
    {
        @setting::target = grep( $_ ne '',
            split( /[\x09\x0A\x0C\x0D\x20]+/, $setting::ranking{'target'} ) );
        map( $_ = quotemeta $_, @setting::target )
            if !defined $setting::ranking{'regexp'}
            || !$setting::ranking{'regexp'};
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::deco     = 1;    #テキスト装飾設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み
    $setting::table    = 1;    #テーブル設定読み込み
    $setting::page     = 1;    #ページ設定読み込み

    #設定種キー設定
    $setting::security_key
        = 'ranking';           #デフォルトセキュリティキー
    $setting::key = 'ranking'; #ページ設定キー

    #クッキー
    $cookie::id_enabled = 1;    #ID情報クッキー読み込み

    #必須フォーム
    @setting::form = ( 'k', 'm' );

    #デフォルトキー
    $main::k = 'ranking' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'ranking', 'AccessRanking' );

}

1;

