package TracerReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# 訪問者の足跡再現モジュール
# TracerReport.pm ver1.1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-12-01
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    return 0 if !defined $$data[1];
    if ( !$this->{'target_ip'} ) {
        ( $this->{'target_ip'}, $this->{'target_ua'} )
            = split( /:/, $setting::stat{'tracer'}{'target'}, 2 );
        if ( $this->{'target_ip'} =~ s/\*/\x0B/sg ) {
            $this->{'target_ip'} = quotemeta $this->{'target_ip'};
            $this->{'target_ip'} =~ s/\\\x0B/.*/sg;
            $this->{'target_ip_regexp'} = 1;
        }
        if ( $this->{'target_ua'} =~ s/\*/\x0B/sg ) {
            $this->{'target_ua'} = quotemeta $this->{'target_ua'};
            $this->{'target_ua'} =~ s/\\\x0B/.*/sg;
            $this->{'target_ua_regexp'} = 1;
        }
    }
    if ( $this->{'target_ip_regexp'} ) {
        return 0 if $$data[1] !~ /^$this->{'target_ip'}$/i;
    }
    elsif ( $$data[1] ne $this->{'target_ip'} ) {
        return 0;
    }
    if ( $this->{'target_ua'} ) {
        if ( $this->{'target_ua_regexp'} ) {
            return 0 if !$$data[3] || $$data[3] !~ /^$this->{'target_ua'}$/i;
        }
        elsif ( !$$data[3] || $$data[3] ne $this->{'target_ua'} ) {
            return 0;
        }
    }

    my $atime = timelocal(
        substr( $$data[0], 4, 2 ),
        substr( $$data[0], 2, 2 ),
        substr( $$data[0], 0, 2 ),
        substr( $date,     6, 2 ),
        ( substr( $date, 4, 2 ) - 1 ),
        substr( $date, 0, 4 )
    );
    $this->{'target_referer'}->{$atime} = $$data[5]
        if $$data[5] eq ''
        || !ref $this->{'target_atime'}
        || $$data[5] ne $this->{'target_request'}
        ->{ $this->{'target_atime'}->[ $#{ $this->{'target_atime'} } ] };
    $this->{'target_request'}->{$atime} = $$data[2];
    $this->{'target_info'}
        = [ $$data[3], $$data[4], $$data[6], $$data[7], $$data[8], $$data[9] ]
        if !ref $this->{'target_info'};
    push( @{ $this->{'target_atime'} }, $atime );

    return 1;
}

sub _stat_setting ($) {
    my $this = shift;
    use Time::Local;
}

######################################################################
# 解析環境（追加）
sub print_sub_stat_env ($) {
    my $this = shift;
    unless ( ref $cgi::UserAgent ) {
        require CMONOS::CGI_Lib::UserAgent;
        $cgi::UserAgent = UserAgent->new;
    }
    my @info
        = $cgi::UserAgent->get_user_agent_info( $this->{'target_info'}[0] );
    shift @info;
    splice( @info, 1, 2, $info[1] . ' ' . $info[2] );
    push( @info, $this->get_lang($1) )
        if $this->{'target_info'}[1]
        && $this->{'target_info'}[1] =~ /^([0-9a-zA-Z_\-]+)/;
    push( @info,
              'ウィンドウサイズ ['
            . $this->{'target_info'}[2]
            . '&times;'
            . $this->{'target_info'}[3]
            . ']' )
        if $this->{'target_info'}[2] && $this->{'target_info'}[3];
    push( @info,
              'スクリーンサイズ ['
            . $this->{'target_info'}[4]
            . '&times;'
            . $this->{'target_info'}[5]
            . ']' )
        if $this->{'target_info'}[4] && $this->{'target_info'}[5];
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('対象IP環境');
    print $this->{'infoTable'}
        ->td( join( ' / ', grep( $_ =~ /[^\x09\x0A\x0C\x0D\x20]/, @info ) ) );
    print $this->{'infoTable'}->_tr;
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('対象IP(:UA)');
    print $this->{'infoTable'}->td(
        '<input type="text" name="setting:tracer_target" size="50" value="'
            . $this->escape( $setting::stat{'tracer'}{'target'} ) . '" />'
            . $this->submit_button( $env::image_dir . '/stat/dog.gif',
            14, 14, '足跡を再現します。', '', 'm', 'TracerReport',
            'k', 'tracer' )
            . ' '
            . $this->domain_info(
            $setting::stat{'tracer'}{'target'}, '[site]'
            )
    );
    print $this->{'infoTable'}->_tr;
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return '訪問者の足跡再現';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return if defined $this->{'noLog'} && $this->{'noLog'};
    $this->tie_title_db;

    #ヘッダ
    if ( $setting::stat{'download'} eq 'csv' ) {
        print $this->jcode_convert(
            $this->stat_title
                . "\x0D\x0A"
                . $this->svjoin( ',', '状態', 'アクセス時',
                '滞在時間', 'URL' )
                . "\x0D\x0A",
            'sjis', 'utf8'
        );
    }
    elsif ( $setting::stat{'download'} eq 'tsv' ) {
        print $this->jcode_convert(
            $this->stat_title
                . "\x0D\x0A"
                . $this->svjoin(
                "\x09",            '状態',
                'アクセス時', '滞在時間',
                'URL'
                )
                . "\x0D\x0A",
            'sjis', 'utf8'
        );
    }
    else {
        print
            '<div class="note">▼=リンク元 ●=最初のページ ↓=移動</div>';
        $this->{'Table'}->set(
            'styleH' =>
                'text-align: center; width: 50px,text-align: center; width: 250px,text-align: center; width: 60px,text-align: center; width: 400px,text-align: center; width: 60px',
            'style0' =>
                'text-align: center; width: 50px,text-align: center; width: 250px,text-align: center; width: 60px,text-align: left; width: 400px,text-align: center; width: 60px'
        );
        print $this->{'Table'}->table;
        print $this->{'Table'}->tr;
        print $this->{'Table'}->td('状態');
        print $this->{'Table'}->td('アクセス時');
        print $this->{'Table'}->td('滞在時間');
        print $this->{'Table'}->td('URL');
        print $this->{'Table'}->td('クリック率');
        print $this->{'Table'}->_tr;
    }

    #解析結果
    if ( ref $this->{'target_atime'} ) {
        my ($i);
        for ( $i = 0; $i < @{ $this->{'target_atime'} }; $i++ ) {
            my ( $mark, $atime, $length );
            $atime = $this->{'target_atime'}->[$i];
            if ( ( $i + 1 ) < @{ $this->{'target_atime'} }
                && !
                defined $this->{'target_referer'}
                ->{ $this->{'target_atime'}->[ $i + 1 ] } )
            {
                $length = $this->{'target_atime'}->[ $i + 1 ] - $atime;
            }
            if ( defined $this->{'target_referer'}->{$atime} ) {
                $this->{'Table'}->{'row'}++;
                $this->print_stat_report( '▼', '',
                    $this->{'target_referer'}->{$atime},
                    '', 1 )
                    if defined $this->{'target_referer'}->{$atime}
                    && $this->{'target_referer'}->{$atime};
                $mark = '●';
            }
            else {
                $mark = '↓';
            }
            $this->print_stat_report( $mark, $atime,
                $this->{'target_request'}->{$atime}, $length )
                if defined $this->{'target_request'}->{$atime}
                && $this->{'target_request'}->{$atime};
        }
    }
    if (   $setting::stat{'download'} eq 'csv'
        || $setting::stat{'download'} eq 'tsv' )
    {
        print "\x0D\x0A";
    }
    else {
        print $this->{'Table'}->_table;
    }
    $this->untie_title_db;
}

sub print_stat_report ($$$$;$$) {
    my ( $this, $mark, $atime, $url, $length, $redirect ) = @_;
    $setting::page{'stat'}{'datetype'} = ''
        if !defined $setting::page{'stat'}{'datetype'};
    $atime = ($atime)
        ? $this->date(
        $atime,
        $setting::page{'stat'}{'datelimit'},
        'gmt' . $setting::page{'stat'}{'datetype'}
        )
        : '';
    if ( defined $length && $length ) {
        $length
            = int( $length / 3600 ) . ':'
            . sprintf( "%.2d", int( ( $length % 3600 ) / 60 ) ) . ':'
            . sprintf( "%.2d", ( $length % 3600 ) % 60 );
    }
    else {
        $length = '';
    }
    if ( $setting::stat{'download'} eq 'csv' ) {
        print $this->jcode_convert(
            $this->svjoin( ',', $mark, $atime, $length, $url ) . "\x0D\x0A",
            'sjis', 'utf8' );
    }
    elsif ( $setting::stat{'download'} eq 'tsv' ) {
        print $this->jcode_convert(
            $this->svjoin( "\x09", $mark, $atime, $length, $url )
                . "\x0D\x0A",
            'sjis', 'utf8'
        );
    }
    else {
        print $this->{'Table'}->tr;
        print $this->{'Table'}->td($mark);
        print $this->{'Table'}->td($atime);
        print $this->{'Table'}->td($length);
        print $this->{'Table'}->td(
            (   ( $url =~ m!^https?://!i )
                ? $this->url_to_title( $url, 0, $redirect )
                : $this->escape($url)
            )
        );
        if ( grep( eval { $url =~ /\Q$_\E/ }, @default::target ) ) {
            print $this->{'Table'}->td(
                $this->submit_button(
                    $env::image_dir . '/stat/node.gif',
                    14,
                    14,
                    'このファイルからのリンクのクリック率をランキングします。',
                    '',
                    'm',
                    'ClickRatioReport',
                    'k',
                    'click',
                    'setting:click_target',
                    $url
                )
            );
        }
        else {
            print $this->{'Table'}->td();
        }
        print '</tr>';
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'tracer' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'tracer', 'TracerReport' );

}

1;

