package ArticleSweeper;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 記事整理モジュール
# ArticleSweeper.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( $error, @folder, %year );

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #一括削除
    if (   defined $main::delete
        && $main::delete
        && !$this->is_nan($main::mday)
        && !$this->is_nan($main::mon)
        && !$this->is_nan($main::year)
        && !$this->is_impossible_mday( $main::year, $main::mon, $main::mday )
        )
    {

        #必要なモジュール
        use File::Path;

        use Time::Local;
        my $limit
            = timelocal( 0, 0, 0, $main::mday, $main::mon - 1, $main::year );

        #フィールド設定読み込み
        $this->use_field( 'field', $default::field_pref_dir . '/inputfield',
            2, 2, 'no' );

        my (%tree,            %thread_mtime,  %thread_ctime,
            %thread_category, %thread_count,  %thread_status,
            %db,              $id,            $field,
            $db,              @delete_thread, $delete_count,
            $delthread_count, $top
        );
        $delete_count = $delthread_count = 0;

        tie( %tree, $env::dbm_file, $default::db . '/tree',
            O_RDWR, $env::dbm_mode );
        tie( %thread_mtime, $env::dbm_file, $default::db . '/thread_mtime',
            O_RDWR, $env::dbm_mode );
        tie( %thread_ctime, $env::dbm_file, $default::db . '/thread_ctime',
            O_RDWR, $env::dbm_mode );
        tie( %thread_category, $env::dbm_file,
            $default::db . '/thread_category',
            O_RDWR, $env::dbm_mode );
        tie( %thread_count, $env::dbm_file, $default::db . '/thread_count',
            O_RDWR, $env::dbm_mode );
        tie( %thread_status, $env::dbm_file, $default::db . '/thread_status',
            O_RDWR, $env::dbm_mode );

        tie(%{ $db{'ctime'} },
            $env::dbm_file, $default::db . '/ctime',
            O_RDWR, $env::dbm_mode
        );
        tie(%{ $db{'mtime'} },
            $env::dbm_file, $default::db . '/mtime',
            O_RDWR, $env::dbm_mode
        );
        tie(%{ $db{'agent'} },
            $env::dbm_file, $default::db . '/agent',
            O_RDWR, $env::dbm_mode
        );
        tie(%{ $db{'ip'} },
            $env::dbm_file, $default::db . '/ip',
            O_RDWR, $env::dbm_mode
        );
        tie(%{ $db{'mailto'} },
            $env::dbm_file, $default::db . '/mailto',
            O_RDWR, $env::dbm_mode
        );
        tie(%{ $db{'status'} },
            $env::dbm_file, $default::db . '/status',
            O_RDWR, $env::dbm_mode
        );

        while ( ( $id, $field ) = each %field::obj ) {
            my $db_name = $field->db;
            unless ( !$db_name || ref $db{$db_name} ) {
                tie(%{ $db{$db_name} },
                    $env::dbm_file, $default::db . '/' . $db_name,
                    O_RDWR, $env::dbm_mode
                );
            }
        }

        if ( $main::by eq 'ctime' ) {
            while ( my ( $no, $ctime ) = each %thread_ctime ) {
                push( @delete_thread, $no ) if $ctime < $limit;
            }
        }
        else {
            while ( my ( $no, $mtime ) = each %thread_mtime ) {
                push( @delete_thread, $no ) if $mtime < $limit;
            }
        }

        foreach $top (@delete_thread) {
            next if !defined $top || !$top;
            $delthread_count++;

            my @delete_article = $this->get_del_tree( $top, \%tree );
            push( @delete_article, $top );

            foreach my $aid (@delete_article) {
                rmtree "$default::file/$aid"
                    if defined $default::file && $default::file ne '';
                utf8::encode($aid) if utf8::is_utf8($aid);
                delete $tree{$aid};
                while ( my ( $id, $db ) = each %db ) {
                    delete $$db{$aid};
                }
                $delete_count++;
            }

            utf8::encode($top) if utf8::is_utf8($top);
            delete $thread_mtime{$top};
            delete $thread_ctime{$top};
            delete $thread_category{$top};
            delete $thread_count{$top};
            delete $thread_status{$top};
        }

        while ( ( $id, $db ) = each %db ) {
            untie(%$db);
        }
        undef %db;
        untie(%tree);
        untie(%thread_mtime);
        untie(%thread_ctime);
        untie(%thread_category);
        untie(%thread_count);
        untie(%thread_status);
        push(
            @{ $this->{'dialog'} },
            $delthread_count
                . 'のスレッド（'
                . $delete_count
                . 'の記事）を削除しました。'
        );
    }
    elsif ($main::delete) {
        push(
            @{ $this->{'dialog'} },
            '日付の指定に問題があります。'
        );
    }

    #年
    my ( %threads, @threads, $oldest, $latest );
    tie( %threads, $env::dbm_file, $default::db . '/thread_mtime',
        O_RDONLY, $env::dbm_mode );
    @threads = sort { $a <=> $b } keys %threads;
    untie(%threads);
    if (@threads) {
        $oldest = $this->decode_utf8( substr( $threads[0],         0, 4 ) );
        $latest = $this->decode_utf8( substr( $threads[$#threads], 0, 4 ) );
    }
    undef %threads;
    undef @threads;

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print
        '<h1 title="スレッドの一括削除を行います。HTMLで書き出したデフォルトページ上の該当記事は削除されません。">記事の整理</h1>';

    if ( !defined $oldest || !$oldest ) {
        print
            '<fieldset>スレッドがみつかりませんでした。</fieldset>';
        $this->print_page_footer('tools');
        return;
    }

    if ( !defined $main::year || !$main::year ) {
        ( $main::mday, $main::mon, $main::year )
            = ( localtime(time) )[ 3 .. 5 ];
        $main::year += 1899;
        $main::mon++;
    }

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<fieldset class="delete">';
    print '<select name="by">';
    print '<option value="ctime">作成時</option>';
    print '<option value="mtime"';
    print( ( defined $main::by && $main::by eq 'mtime' )
        ? ' selected="selected"'
        : '' );
    print '>更新時</option>';
    print '</select> が ';

    print '<label><select name="year">';
    foreach ( $oldest .. $latest ) {
        print '<option value="', $_, '"';
        print( ( $main::year && $main::year == $_ )
            ? ' selected="selected"'
            : '' );
        print '>', $_, '</option>';
    }
    print '</select>年</label> ';

    print '<label><select name="mon">';
    foreach ( 1 .. 12 ) {
        print '<option value="', $_, '"';
        print( ( $main::mon && $main::mon == $_ )
            ? ' selected="selected"'
            : '' );
        print '>', $_, '</option>';
    }
    print '</select>月</label> ';

    print '<label><select name="mday">';
    foreach ( 1 .. 31 ) {
        print '<option value="', $_, '"';
        print( ( $main::mday && $main::mday == $_ )
            ? ' selected="selected"'
            : '' );
        print '>', $_, '</option>';
    }
    print '</select>日</label> ';

    print
        ' より古いスレッドを <input type="submit" name="delete=1" value="一括削除する" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\');" />';
    print '</fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# 削除ツリー取得
sub get_del_tree ($$$) {
    my ( $this, $id, $tree ) = @_;
    my ( @delete_article, @temp );

    @delete_article = split( /,/, $$tree{$id} );
    splice( @delete_article, 0, 3 );

    foreach (@delete_article) {
        my @temp2 = $this->get_del_tree( $_, $tree );
        push( @temp, @temp2 ) if @temp2;
    }
    push( @delete_article, @temp ) if @temp;
    return @delete_article;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #クラス登録
        Common::class_register( 'xxas', 'ArticleSweeper' );

        #メニュー登録
        $setting::jump_menu{'xxas'} = '記事の整理';

    }
}

1;
