package GeneralSetting;
require CMONOS::CGI_Lib::Tools::General;
@ISA = qw(General);
use strict;
use utf8;
######################################################################
# 一般設定モジュール
# GeneralSetting.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2008-06-21
######################################################################
# 設定チェック項目取得
sub get_sub_setting_items ($) {
    my $this = shift;

    #チェックボックス
    push( @check::checkbox,
        'html',              'skipConfirmation', 'siteRSSEnabled',
        'commentRSSEnabled', 'RSSEnabled',       'rssLinkToHtml' );
    push( @check::checkbox, 'masterRSSEnabled' ) if $default::subdivided;

    #タグを除去する
    push( @check::notag, 'description' );

    #数字チェック
    $check::num{'limit'}     = '記事保存数の上限';
    $check::num{'ipStopper'} = '連続投稿の制限';

    #アイコン画像
    $check::imagefile{'og'} = 'og:image';
    $check::ogimage{'og'}   = 1;
    if ($default::subdivided) {
        if ( $master::general{'subicon'} ) {
            $check::imagefile{'icon'}
                = 'カテゴリーアイコン';    #画像の名前
            $check::maxsize{'icon'}
                = ( $master::general{'iconMaxSize'} )
                ? $master::general{'iconMaxSize'}
                : 30000;    #ファイルサイズ制限
            $check::widthlimit{'icon'} = $master::general{'iconWidthLimit'}
                ;           #ファイル横幅制限
            $check::heightlimit{'icon'} = $master::general{'iconHeightLimit'}
                ;           #ファイル高さ制限
        }
        push( @check::checkbox, 'p' );
    }
    return;
}

sub is_database_cgi ($) {
    my $this = shift;
    return 1;
}

######################################################################
# 設定環境2
sub sub_set_preferences ($) {
    my $this = shift;
    $cgi::enctype_multipart = 1;

    # カテゴリーがない場合
    if ( !$default::subdivided ) {
        $default::file = '.'
            if !defined $default::file || $default::file eq '';
        $preference::filedir{'og'}
            = $default::file;    #画像保存ディレクトリ

        # カテゴリーがある場合
    }
    elsif ( $default::subdivided && defined $main::c && $main::c ne '' ) {
        if ( defined $master::general{'subicon'}
            && $master::general{'subicon'} )
        {
            $default::file = '.'
                if !defined $default::file || $default::file eq '';
            $preference::filedir{'icon'}
                = $default::file;    #画像保存ディレクトリ
        }
        $preference::filedir{'og'}
            = $default::file;        #画像保存ディレクトリ
    }
}

######################################################################
# サブ設定パネル
sub print_sub_preference_panel ($$) {
    my ( $this, $master ) = @_;

    # カテゴリーがある場合
    if ($default::subdivided) {
        print '<fieldset><legend>カテゴリー設定</legend>';

        if ( $master::general{'subicon'} ) {
            print '<fieldset><legend>カテゴリーアイコン</legend>';
            $this->print_image_upload_form( 'icon', 'icon', \%main::data );
            print '</fieldset>';
        }
        print '<fieldset><legend>og:image</legend>';
        $this->print_image_upload_form( 'og', 'og', \%main::data );
        print '</fieldset>';

        print
            '<fieldset><legend><label for="data:description">説明</label></legend>';
        print
            '<textarea name="data:description" id="data:description" cols="56" rows="6" wrap="soft" title="インデックスに表示する掲示板の説明を設定します。">'
            . ( ( defined $main::data{'description'} )
            ? $main::data{'description'}
            : '' )
            . '</textarea>';
        print
            '<div class="extra-field"><label><input type="checkbox" name="data:p" value="1"'
            . ( ( $main::data{'p'} ) ? 'checked="checked"' : '' )
            . ' /> 改行の連続を &lt;p&gt; に変換する</label></div>';
        print
            '<div class="note">インデックスに表示する掲示板の説明を設定します。</div></fieldset>';

        if (   $default::subdivided
            && $main::data{'original'}
            && $main::data{'original'} ne '' )
        {
            print '<fieldset><legend>オリジナルフォルダ</legend>';
            print '<a href="'
                . $cgi::path . '?c='
                . $main::data{'original'}
                . '" target="_blank">'
                . $main::data{'original'} . '</a>';
            print '</fieldset>';
        }

        print '</fieldset>';
    }
    else {
        print '<fieldset><legend>メタデータ</legend>';
        print '<fieldset><legend>og:image</legend>';
        $this->print_image_upload_form( 'og', 'og', \%main::data );
        print '</fieldset>';

        print
            '<fieldset><legend><label for="data:description">説明</label></legend>';
        print
            '<textarea name="data:description" id="data:description" cols="56" rows="6" wrap="soft" title="メタデータに埋め込む掲示板の説明を設定します。">'
            . ( ( defined $main::data{'description'} )
            ? $main::data{'description'}
            : '' )
            . '</textarea>';
        print
            '<div class="note">メタデータに埋め込む掲示板の説明を設定します。</div></fieldset>';
        print '</fieldset>';
    }

    print '<fieldset><legend>デフォルトページ</legend>';
    print
        '<label><input type="checkbox" name="data:html" id="data:html" value="1"'
        . ( ( $main::data{'html'} ) ? ' checked="checked"' : '' )
        . ' /> 1 ページ目を HTML ファイルに書き出す</label>';
    print
        '<div class="hints">アクセス権設定で「記事閲覧」「非公開スレッド閲覧」「非公開項目閲覧」が「チェックなし」の場合のみ有効です。</div></fieldset>';

    print '<fieldset><legend>投稿設定</legend>';
    print '<table class="field-table"><tbody>';

    print '<tr>';
    print
        '<td class="field-name"><label for="data:ipStopper">連続投稿の制限</label></td>';
    print
        '<td class="field"><input type="number" size="3" maxlength="3" name="data:ipStopper" id="data:ipStopper" value="'
        . $this->escape( $main::data{'ipStopper'} )
        . '" /> 秒間、同一IPからの投稿を禁止する</td>';
    print '</tr>';

    print
        '<tr class="tbody-end"><td class="field-name"><span class="datatitle">投稿内容の確認</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="data:skipConfirmation" id="data:skipConfirmation" value="1"'
        . ( ( $main::data{'skipConfirmation'} ) ? 'checked="checked"' : '' )
        . ' /> 投稿内容確認を省略する</label></td>';
    print '</tr>';

    print '</tbody><tbody>';

    $main::data{'limit'} = '100'
        if !defined $main::data{'limit'} || !$main::data{'limit'};
    print '<tr>';
    print
        '<td class="field-name"><label for="data:limit">記事保存数の上限</label></td>';
    print
        '<td class="field"><input type="number" size="5" maxlength="5" name="data:limit" id="data:limit" value="'
        . $this->escape( $main::data{'limit'} )
        . '" /> 件';
    print '</td></tr>';

    print
        '<tr class="tbody-end"><td class="field-name"><label for="data:action">上限を超えたときの処理</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:action',
        '記事保存数が上限を超えたときの処理を設定します。',
        '',
        $main::data{'action'},
        'byMtime' => '更新時が最も古いスレッドを削除する',
        'byCtime' => '作成時が最も古いスレッドを削除する',
        'close'   => '新しい書き込みを拒絶する',
        (     ( defined $default::subdivided && $default::subdivided )
            ? ( 'make' => '新しい掲示板を作成する' )
            : ()
        )
    );
    print '</td>';
    print '</tr>';

    print '</tbody><tbody>';

    print '<tr>';
    print
        '<td class="field-name"><label for="data:newmail">新規投稿通知</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:newmail',
        '新規投稿を管理人・共同オーナー・スタッフに通知するかどうか設定します。',
        '', $main::data{'newmail'},
        ''     => '新規投稿通知機能停止',
        'on'   => 'メールで通知（内容を含まない）',
        'data' => 'メールで通知（内容を含む）'
    );
    print '</td>';
    print '</tr>';

    print '<tr>';
    print
        '<td class="field-name"><label for="data:resmail">返信通知</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:resmail',
        '投稿したスレッドに新しいレスが投稿されたとき通知するかどうか設定します。',
        '', $main::data{'resmail'},
        ''     => '返信通知機能停止',
        'on'   => 'メールで通知（内容を含まない）',
        'data' => 'メールで通知（内容を含む）'
    );
    print '</td>';
    print '</tr>';

    print '</tbody></table>';
    print '</fieldset>';

    #RSS更新設定
    print '<fieldset><legend>RSS更新設定</legend>';
    print '<fieldset><legend>投稿時に更新するRSS</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:siteRSSEnabled" value="1"'
        . ( ( $main::data{'siteRSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( -f '../../index.xml' )
        ? '<a href="../../index.xml" target="_blank">サイト全体の新着情報(RSS)</a>'
        : 'サイト全体の新着情報(RSS)' )
        . 'を更新する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:commentRSSEnabled" value="1"'
        . ( ( $main::data{'commentRSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( -f '../../comment.xml' )
        ? '<a href="../../comment.xml" target="_blank">サイト全体のコメント新着情報(RSS)</a>'
        : 'サイト全体のコメント新着情報(RSS)' )
        . 'を更新する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:masterRSSEnabled" value="1"'
        . ( ( $main::data{'masterRSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . (
        (   $default::master_file_dir
                && -f $default::master_file_dir . '/index.xml'
        )
        ? '<a href="'
            . $default::master_file_dir
            . '/index.xml'
            . '" target="_blank">掲示板全体の新着情報(RSS)</a>'
        : '掲示板全体の新着情報(RSS)'
        )
        . 'を更新する</label></li>'
        if $default::subdivided;
    print '<li><label><input type="checkbox" name="data:RSSEnabled" value="1"'
        . ( ( $main::data{'RSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( $default::file && -f $default::file . '/index.xml' )
        ? '<a href="'
            . $default::file
            . '/index.xml'
            . '" target="_blank">この掲示板の新着情報(RSS)</a>'
        : 'この掲示板の新着情報(RSS)' )
        . 'を更新する</label></li>';
    print '</ol>';
    print '</fieldset>';
    print '<fieldset><legend>リンク先</legend>';
    print '<label><input type="checkbox" name="data:rssLinkToHtml" value="1"'
        . ( ( $main::data{'rssLinkToHtml'} ) ? ' checked="checked"' : '' )
        . ' /> デフォルトページにリンクする</label>';
    print
        '<div class="hints">アクセス権設定で「記事閲覧」「非公開スレッド閲覧」「非公開項目閲覧」が「チェックなし」の場合のみ有効です。</div></fieldset>';
    print '</fieldset>';

}

sub navidata_is_changed ($$) {
    my ( $this, $db ) = @_;
    return (   ( $db->{'html'} && !$main::data{'html'} )
            || ( !$db->{'html'} && $main::data{'html'} ) ) ? 1 : 0;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) )
    {
        #クラス登録
        Common::class_register( '0000', 'GeneralSetting' );

        #ツール登録
        $setting::jump_menu{'0000'} = '一般設定';
    }
}

1;
