package IndexPage;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
######################################################################
# 総合インデックスページ設定モジュール
# IndexPage.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox,
        'headBox', 'footBox', 'searchFrameTitle', 'tileFrameTitle' );

    #数字チェック
    $check::num{'searchWidth'} = '検索フレーム幅';
    $check::num{'tableWidth'}  = 'テーブル幅';
    $check::num{'tileWidth'}   = 'タイル横幅';
    $check::num{'tileHeight'}  = 'タイル高さ';
    $check::num{'tableCols'}   = 'テーブル列数';
    $check::num{'tableRows'}   = 'テーブル行数';
    $check::num{'upIconLimit'} = '更新アイコン対象期間';

    return;
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    if (   defined $main::data{'returnUrl'}
        && $main::data{'returnUrl'} ne ''
        && $this->is_invalid_url( $main::data{'returnUrl'} ) )
    {
        push(
            @{ $this->{'dialog'} },
            '戻る先URLを更新できませんでした。URLを確認してください。'
        );
        delete $main::data{'returnUrl'};
    }
    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::id = 'index';    # データキー
    $preference::name = '総合インデックスページレイアウト'
        ;                         # 設定パネル名
    $preference::title
        = '総合インデックスページの表示について設定します。';
}

######################################################################
# 設定パネル
sub print_sub_preference_panel ($$$) {
    my ( $this, $id, $key ) = @_;

    print '<fieldset><legend>インデックスの表示設定</legend>';

    print
        '<fieldset><legend><label for="data:index_defaultSort">デフォルトのカテゴリー表示順</label></legend>';
    print $this->mk_select_menu(
        'data:index_defaultSort', '', '', $main::data{'index_defaultSort'},
        'static' => '固定（カテゴリーフォルダ名でソート）',
        'newer'  => '更新時の新しい順',
        'older'  => '更新時の古い順'
    );
    print '</fieldset>';

    print '<fieldset><legend>検索ボックス表示位置</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label for="data:index_headBox"><input type="checkbox" name="data:index_headBox" id="data:index_headBox" value="1"'
        . (
        (   defined $main::data{'index_headBox'}
                && $main::data{'index_headBox'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 検索ボックスをページ上部に表示する</label></li>';
    print
        '<li><label for="data:index_footBox"><input type="checkbox" name="data:index_footBox" id="data:index_footBox" value="1"'
        . (
        (   defined $main::data{'index_footBox'}
                && $main::data{'index_footBox'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 検索ボックスをページ下部に表示する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print
        '<fieldset class="foldable"><legend>検索ボックスを囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:index_searchFrame', 'frame',
        $main::data{'index_searchFrame'} );
    print
        '<div class="additional-field"><label for="data:index_searchWidth">フレーム幅</label><input type="text" name="data:index_searchWidth" id="data:index_searchWidth" value="'
        . $this->escape( $main::data{'index_searchWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span></div>';
    print
        '<div class="external-field"><label for="data:index_searchFrameTitle"><input type="checkbox" name="data:index_searchFrameTitle" id="data:index_searchFrameTitle" value="1"'
        . (
        (   defined $main::data{'index_searchFrameTitle'}
                && $main::data{'index_searchFrameTitle'}
        ) ? ' checked="checked"' : ''
        )
        . ' />「検索」を検索フレームのタイトルバー部分に表示する</label></div>';
    print '</fieldset>';

    print
        '<fieldset class="foldable"><legend>タイル全体を囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:index_tableFrame', 'frame',
        $main::data{ $key . 'tableFrame' } );
    print '</fieldset>';

    print '<fieldset><legend>タイルテーブル設定</legend>';
    print '<table class="field-table">';
    print '<tbody class="foldable">';
    print '<tr>';
    print '<td class="field-name">タイルテーブルタイプ</td>';
    print '<td class="field" cols="3">';
    print $this->table_menu( 'data:index_table', $main::data{'index_table'} );
    print '</td>';
    print '</tr></tbody>';
    print '<tbody><tr>';
    print
        '<td class="field-name"><label for="data:index_tableRows">タイル行数</label></td>';
    print
        '<td class="q-field"><input type="text" name="data:index_tableRows" id="data:index_tableRows" value="'
        . $this->escape( $main::data{'index_tableRows'} )
        . '" size="7" maxlength="7" /></td>';
    print
        '<td class="field-name"><label for="data:index_tableCols">タイル列数</label></td>';
    print
        '<td class="q-field"><input type="text" name="data:index_tableCols" id="data:index_tableCols" value="'
        . $this->escape( $main::data{'index_tableCols'} )
        . '" size="7" maxlength="7" /></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';

    print '</fieldset>';

    print '<fieldset><legend>タイル設定</legend>';
    print '<fieldset><legend>タイルを囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:index_tileFrame', 'frame',
        $main::data{'index_tileFrame'} );
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:index_tileAlign" class="datatitle">タイルフレーム内行揃え</label></td>';
    print '<td class="field" cols="3">';
    print $this->mk_select_menu(
        'data:index_tileAlign', '', '', $main::data{'index_tileAlign'},
        ''       => '',
        'left'   => '左寄せ',
        'center' => '中央揃え',
        'right'  => '右寄せ'
    );
    print $this->mk_select_menu(
        'data:index_tileValign', '', '', $main::data{'index_tileValign'},
        ''       => '',
        'top'    => '上寄せ',
        'middle' => '中央揃え',
        'bottom' => '下寄せ'
    );
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:index_tileWidth">タイル横幅</label></td>';
    print
        '<td class="q-field"><input type="text" name="data:index_tileWidth" id="data:index_tileWidth" value="'
        . $this->escape( $main::data{'index_tileWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span></td>';
    print
        '<td class="field-name"><label for="data:index_tileHeight">タイル高さ</label></td>';
    print
        '<td class="q-field"><input type="text" name="data:index_tileHeight" id="data:index_tileHeight" value="'
        . $this->escape( $main::data{'index_tileHeight'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print
        '<div class="external-field"><label for="data:index_tileFrameTitle"><input type="checkbox" name="data:index_tileFrameTitle" id="data:index_tileFrameTitle" value="1"'
        . (
        (   defined $main::data{'index_tileFrameTitle'}
                && $main::data{'index_tileFrameTitle'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> カテゴリータイトルをタイルフレームのタイトルバー部分に表示する</label></div>';
    print '</fieldset>';

    my ( $template, $tool );
    require CMONOS::CGI_Lib::Template;
    $template = Template->new();
    $tool
        .= '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:index_template\').quotationByMenu(this)" title="これらのタグは、書き出し時に適切なタグに変換されます。">';
    $tool .= '<option value="">▼特殊タグを挿入する</option>';
    $tool
        .= '<option value="&lt;!--title--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/title--&gt;">カテゴリータイトル</option>';
    $tool
        .= '<option value="&lt;!--icon--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/icon--&gt;">カテゴリーアイコン</option>';
    $tool
        .= '<option value="&lt;!--description--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/description--&gt;">カテゴリーの説明</option>';
    $tool
        .= '<option value="&lt;!--master--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/master--&gt;">管理人名</option>';
    $tool
        .= '<option value="&lt;!--count--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/count--&gt;">投稿総数</option>';
    $tool
        .= '<option value="&lt;!--mtime--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/mtime--&gt;">更新時</option>';
    $tool .= '</select>';
    $template->print_template_form(
        'data:index_template',
        'タイルテンプレート（上記設定よりもテンプレートが優先されます）',
        \$main::data{'index_template'},
        \$tool
    );
    undef $tool;
    undef $template;

    print '</fieldset>';

    print '<fieldset>';
    print '<legend>ページナビゲーション</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="radio" name="data:index_pageinfo" value="1"'
        . ( ( $main::data{'index_pageinfo'} ) ? ' checked="checked"' : '' )
        . ' /> ページ位置情報のみ表示する</label></li>';
    print
        '<li><label><input type="radio" name="data:index_pageinfo" value="0"'
        . ( ( !$main::data{'index_pageinfo'} ) ? ' checked="checked"' : '' )
        . ' /> 各ページへのリンクを表示する</label>';
    print ' (<span class="additional-field">';
    print
        '<label for="data:index_pagelistmax">ページリンク最大表示数</label><input type="number" size="3" maxlength="3" name="data:index_pagelistmax" id="data:index_pagelistmax" value="'
        . $this->escape( $main::data{'index_pagelistmax'} )
        . '" /> (7以上)';
    print
        ' / <label for="data:index_pagelistmargin">前方ページリンク表示数</label><input type="number" size="3" maxlength="3" name="data:index_pagelistmargin" id="data:index_pagelistmargin" value="'
        . $this->escape( $main::data{'index_pagelistmargin'} ) . '" />';
    print '</span>)</li>';
    print '</ol>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:index_toolIcon">ツールアイコン</label></legend>';
    print $this->mk_select_menu(
        'data:index_toolIcon', '', '', $main::data{'index_toolIcon'},
        '1' => '表示',
        '0' => '非表示'
    );
    print '</fieldset>';
    $this->print_page_button_setting;
    $this->print_close_button_setting;
    $this->print_sc_button_setting;

    print '<fieldset><legend>更新アイコン</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name">';
    print '<span class="datatitle">対象</span></td>';
    print
        '<td class="field"><label><input type="text" name="data:index_upIconLimit" id="data:index_upIconLimit" value="'
        . $this->escape( $main::data{'index_upIconLimit'} )
        . '" size="3" /> 日以内に更新されたカテゴリー</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:index_upIconAlt">代替文字列</label></td>';
    print
        '<td class="field"><input type="text" name="data:index_upIconAlt" id="data:index_upIconAlt" value="'
        . $this->escape( $main::data{'index_upIconAlt'} )
        . '" size="40" /></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '<fieldset><legend>アイコン</legend>';
    print $cgi::Deco->image_menu( 'data:index_upIcon', 'new', 120,
        $main::data{'index_upIcon'} );
    print '</fieldset>';
    print '</fieldset>';

}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);
    if ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) ) {

        #クラス登録
        Common::class_register( 'b000', 'IndexPage' );

        #ツール登録
        $setting::jump_menu{'b000'}
            = '総合インデックスページレイアウト';

        #画像
        $setting::deco_dir{'new'}     = '新着アイコン';
        $setting::deco_type{'new'}    = 'image';
        $setting::deco_dir{'return'}  = '戻る ボタン';
        $setting::deco_type{'return'} = 'button';
        $setting::deco_dir{'next'}    = '「次」ボタン';
        $setting::deco_type{'next'}   = 'button';
        $setting::deco_dir{'prev'}    = '「前」ボタン';
        $setting::deco_type{'prev'}   = 'button';

        if ( defined $env::checkout && $env::checkout ) {
            $setting::deco_dir{'check'}  = 'カートを見る ボタン';
            $setting::deco_type{'check'} = 'button';
        }
    }
}

1;
