package ThreadCategoryEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# スレッドカテゴリー設定モジュール
# ThreadCategoryEditor.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this                = shift;
    my $threadcategory_pref = $default::preference_dir . '/thread_category';
    $this->use_icon( 'baobab', 'bb_icon' );

    #全体編集書き込み
    if ( defined $main::a && $main::a eq 'e' ) {
        no warnings qw(numeric);
        my ( $k, $v, %old );

        #不正なデータの除去
        while ( ( $k, $v ) = each %main::order ) {
            $main::data{ $k . '_closed' } = ''
                if !defined $main::data{ $k . '_closed' }
                || !$main::data{ $k . '_closed' };
            $main::data{ $k . '_icon' } = ''
                if !defined $main::data{ $k . '_icon' }
                || !$main::data{ $k . '_icon' };
            $this->detag( \$main::data{ $k . '_name' } );
        }

        #表示順データ読み込み
        %old = $this->import_order( $threadcategory_pref, \%main::order );

        #データ削除
        push( @main::delete, $main::delete )
            if defined $main::delete && $main::delete;
        @main::delete = grep( $_ ne '', @main::delete );
        if ( @main::delete > 0 ) {

            #順序判定への影響を回避
            foreach (@main::delete) {
                delete $old{$_};
                delete $main::order{$_};
            }
        }

        #順序判定
        my $no = 0;
        foreach (
            sort {
                       $main::order{$a} <=> $main::order{$b}
                    || $old{$b} <=> $old{$a}
                    || $main::order{$a} cmp $main::order{$b}
                    || $a <=> $b
            } keys %old
            )
        {

            $no++;
            $main::data{ 'ORDER_' . $no } = $_;
            $main::data{ $_ . '_ORDER' } = $no;
        }

        #新規作成
        $this->detag( \$main::new{'name'} );
        if ( defined $main::new{'name'} && $main::new{'name'} ne '' ) {
            my ( $k, $v, $n );
            while (1) {
                $n++;
                last if !defined $old{$n} || $old{$n} eq '';
            }
            $no++;
            $main::data{ 'ORDER_' . $no } = $n;
            $main::data{ $n . '_ORDER' } = $no;
            while ( ( $k, $v ) = each %main::new ) {
                $main::data{ $n . '_' . $k } = $v;
            }
            @main::delete = grep( $_ ne $n, @main::delete );
        }

        undef %old;
        $this->write_db( $threadcategory_pref, \%main::data, 'ORDER',
            \@main::delete, \%thread::category, 1 );

    }
    else {
        $this->import_db( $threadcategory_pref, \%thread::category, 1 );

    }
    undef(%main::data);

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');
    print '<h1>スレッドカテゴリーの設定</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="e" />';
    print '<fieldset>';

    #テーブル設定
    require CMONOS::CGI_Lib::Table;
    my $table = Table->new;
    $table->set(
        'tableclass' => 'tooltable',
        'trclass'    => 'odd,even',
        'class0'     => 'odd,even',
        'style0'     => 'white-space: nowrap;',
        'style0'     => 'text-align: center;',
        'classF'     => 'caution',
        'thenabled'  => 'true'
    );

    #表
    print $table->table;

    #ヘッダ
    print $table->tr;
    print $table->td('削除');
    print $table->td('表示順');
    print $table->td('ID');
    print $table->td('カテゴリー名');
    print $table->td('非公開');
    print $table->td('アイコン');
    print $table->_tr;

    #設定項目
    foreach ( sort { $a <=> $b } keys %{ $thread::category{'ORDER'} } ) {
        my $id = $thread::category{'ORDER'}{$_};
        print $table->tr;
        print $table->td( '<input type="checkbox" name="delete" value="'
                . $id
                . '" onclick="cmonos.form.field(this).checkboxConfirm(\''
                . $thread::category{$id}{'name'}
                . ' に関連する全てのデータが削除されます。本当に削除しますか？\')" />'
        );
        print $table->td( '<input type="number" name="order:'
                . $id
                . '" value="'
                . $_
                . '" size="2" maxlength="2" />' );
        print $table->td($id);
        print $table->td( '<input type="text" name="data:'
                . $id
                . '_name" value="'
                . $thread::category{$id}{'name'}
                . '" size="20" />' );
        print $table->td(
                  '<input type="checkbox" name="data:'
                . $id
                . '_closed" value="1"'
                . (
                ( $thread::category{$id}{'closed'} )
                ? ' checked="checked"'
                : ''
                )
                . ' />'
        );
        print $table->td(
            $cgi::icon->image_menu(
                'data:' . $id . '_icon', 'threadcategory',
                50,                      $thread::category{$id}{'icon'}
            )
        );
        print $table->_tr;
    }

    #新規作成
    print $table->tr;
    print $table->td('');
    print $table->td('');
    print $table->td('新規');
    print $table->td('<input type="text" name="new:name" size="20" />');
    print $table->td('<input type="checkbox" name="new:closed" value="1" />');

    #アイコン選択
    print $table->td(
        $cgi::icon->image_menu( 'new:icon', 'threadcategory', 50, '' ) );

    print $table->_tr;

    print $table->_table;
    undef %$table;
    undef $table;
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

sub import_order ($$$) {
    no warnings qw(untie);
    my ( $this, $db, $order ) = @_;
    my (%db);
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie( %db, $env::dbm_file, $db, O_RDONLY, $env::dbm_mode );
    if ( ref tied %db ) {
        my (%order);
        while ( my ( $key, $value ) = each %db ) {
            utf8::decode($key)   if !utf8::is_utf8($key);
            utf8::decode($value) if !utf8::is_utf8($value);
            if ( $key =~ /^ORDER_(.*)$/ ) {
                $order{$value} = $1;
                $$order{$value} = $1 unless $$order{$value} ne '';
            }
        }
        untie %db;
        return %order;
    }
    else {
        untie %db;
        my ($dbname);
        $dbname = '\n' . $1 if $db ne $env::register_db && $db =~ m!([^/]+)$!;
        push(
            @{ $this->{'dialog'} },
            'データベース読み込みに失敗しました。'
                . $dbname
        );
        return ();
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) )
    {
        #クラス登録
        Common::class_register( 'b101', 'ThreadCategoryEditor' );

        #ツール登録
        $setting::jump_menu{'b101'} = 'スレッドカテゴリー設定';
    }
}

1;
