package TreeView;
require CMONOS::Baobab::BbCommon;
@ISA = qw(BbCommon);
use strict;
use utf8;
######################################################################
# バオバブツリー表示モジュール
# TreeView.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-06-27
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( %db, @list, $is_secure );
    @list = ();

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    require CMONOS::Baobab::BbTree;
    my $view_obj = BbTree->new;

    unless ( ref tied %{ $db{'tree'} } ) {
        tie(%{ $db{'tree'} }, $env::dbm_file, "$default::db/tree",
            O_RDONLY,         $env::dbm_mode
        );
    }
    unless ( ref tied %{ $db{'thread_category'} } ) {
        tie(%{ $db{'thread_category'} },
            $env::dbm_file, "$default::db/thread_category",
            O_RDONLY, $env::dbm_mode
        );
    }

    #スレッドカテゴリーチェック
    if ( $main::n || $main::t ) {
        my ($top);
        if ( defined $main::n && $main::n ) {
            $top = $this->get_top(
                $main::n,
                $this->decode_utf8(
                    $db{'tree'}->{ $this->encode_utf8($main::n) }
                )
            );
        }
        else {
            $top = $main::t;
        }

        if ( $field::for{'category'} && %{ $thread::category{'ORDER'} } ) {
            my $category = $this->decode_utf8(
                $db{'thread_category'}->{ $this->encode_utf8($top) } );
            $is_secure = 1
                if !defined $category
                || !ref $thread::category{$category}
                || !$thread::category{$category}{'closed'}
                || $this->is_secure('closed');
        }
        else {
            $is_secure = 1;
        }

        if ( defined $is_secure && $is_secure ) {
            $main::t = $top;
            @list    = ($main::t);
            $this->set_canonical(
                $this->my_url
                    . (
                    (          $default::subdivided
                            && defined $main::c
                            && $main::c ne ''
                    ) ? '?c=' . $main::c . '&' : '?'
                    )
                    . 'm=TreeView&t='
                    . $main::t
            );
        }
    }

    $this->use_icon( 'baobab', 'bb_icon' );
    $this->use_deco();

    #ページセッティング
    $this->set_css( $view_obj->css );

    #ヘッダ
    if ( $setting::general{'lastModifiedHeaderEnabled'} ) {
        if ( !ref tied %{ $db{'thread_mtime'} } ) {
            tie( %{ $db{'thread_mtime'} },
                $env::dbm_file, "$default::db/thread_mtime", O_RDONLY,
                $env::dbm_mode );
        }
        $this->set_last_modified(
            $this->decode_utf8(
                $db{'thread_mtime'}->{ $this->encode_utf8($main::t) }
            )
        );
    }
    $this->print_page_header('tree');

    if ( defined $is_secure && $is_secure ) {

        #必要なDBを読み込む
        $this->tie_db( \%db, $view_obj->required_fields );

        #ページ数判定
        $main::p
            = $view_obj->list_count( '', \%db, \@list, $main::p, $main::t,
            $main::n );

        #記事ヘッダ
        $view_obj->print_list_header( \*STDOUT, '', $main::p );

        #記事リスト
        $view_obj->print_articles( \*STDOUT, '', \%db, \@list, $main::p );

        #記事フッタ
        $view_obj->print_list_footer( \*STDOUT, '', $main::p );
    }

    #DBの接続を解除する
    $this->untie_db( \%db );

    #フッタ
    $this->print_page_footer('tree');
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #各種キー設定
    $setting::security_key = 'browse'
        if !defined $setting::security_key
        || $setting::security_key eq
        '';    #デフォルトセキュリティキー

    #ページ設定キー
    $setting::key = 'tree' if !defined $setting::key || $setting::key eq '';

    #デフォルトキー
    $main::k = 'tree' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'tree', 'TreeView' );
}

1;
