package Base;
require CMONOS::CGI_Lib::Common;
@ISA = qw(Common);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# CGIに共通する処理を行うモジュール
# Base.pm ver1.4.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新：2015-07-25
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	obj->login
#		初期設定の読み込みやセキュリティチェックを行います。
#
#	obj->use_session;
#		セッションオブジェクト $cgi::Session を作成します。
#
# ---------------------------------------------------------------------------------------
#	obj->is_secure(key,option)
#		アクセス権設定に基づき必要なセキュリティチェックを行います。
#			key		セキュリティキー
#			option	1=IPチェックとrefererチェックも行う
#
# ---------------------------------------------------------------------------------------
#	obj->import_db(db,hash_ref,parse,@key)
#		データベースをハッシュに読み込みます。
#			db			データベースのパス
#			hash_ref	読み込み先のハッシュリファレンス
#			parse		1="_"を区切りとした構造体として読み込みます
#			key			読み込み対象データのキーワード
#
# ---------------------------------------------------------------------------------------
#	obj->write_db(db,\%data[,changekey,deletekey[,hash_ref,parse,@key]])
#		データベースに書き込みます。
#			db			データベースのパス
#			data		書き込むハッシュのリファレンス
#			changekey	changekey_...がキーのデータを書き込む前に全て削除します。
#			deletekey	deletekey_...がキーのデータを全て削除します(書き込みもしません)。
#
#			以下を設定すると書き込みと同時にデータを読み込みます。
#			hash_ref	読み込み先のハッシュリファレンス
#			parse		1="_"を区切りとした構造体として読み込みます
#			key			読み込み対象データのキーワード
#
# ---------------------------------------------------------------------------------------
#	obj->unlink_db(@db)
#		データベースを関連ファイルを含め削除します。
#			@db			データベースのパス
#
# ---------------------------------------------------------------------------------------
#	obj->delete_data(path,@pat)
#		pathにあるデータベースからパターンにマッチするキーを持つ値を削除します。
#			dir			データベースのパス
#			pat			パターン
#
#	obj->delete_all_data(dir,@keys)
#		dirにあるデータベースから@keysをキーに持つ値を削除します。
#			dir			データベースのあるディレクトリ
#			@keys		削除する値のキー
#
# ---------------------------------------------------------------------------------------
#	obj->form_start(
#		key				=>		セキュリティキー,
#		action			=>		CGI の URI,
#		name			=>		フォームの名前,
#		id				=>		フォームの ID,
#		target			=>		ターゲットウィンドウ,
#		authorization	=>		enabled/disabled,	#パスワード情報を受け渡すかどうか
#		multi			=>		1/0, 				#"1"=multipart/form-data
#		required		=>		[必須変数のリファレンス]
#	);
#
# ---------------------------------------------------------------------------------------
#	obj->feed_rss({item => value},rss_dir,link,master_rss_dir,master_link)
#
#	obj->delete_rss_item([delete],rss_dir,link,master_rss_dir,master_link)
#
######################################################################
#	特殊変数
#		$cgi::output	ファイルを書き出す位置
#
######################################################################
#	obj->login
sub login ($) {
    my $this = shift;
    if ( ref $cgi::input && ref $cgi::action ) {
        my @msg = $cgi::input->dialog;
        push( @{ $cgi::action->{'dialog'} }, @msg ) if @msg;
    }
    if ($main::use_memorized_id) {
        undef $main::pswd;
        undef $main::email;
    }
    else {
        $main::pswd = pop @main::pswd
            if ( !defined $main::pswd || $main::pswd eq '' ) && @main::pswd;
        $main::email = pop @main::email
            if ( !defined $main::email || $main::email eq '' )
            && @main::email;
        $main::email = '' if $this->is_invalid_email_address($main::email);
    }
    $default::master_preference_dir = $default::preference_dir
        if defined $default::preference_dir;
    $default::master_field_pref_dir = $default::field_pref_dir
        if defined $default::field_pref_dir;
    $default::master_file_dir = $default::file if defined $default::file;
    $default::master_db_dir   = $default::db   if defined $default::db;
    if ( $default::subdivided && defined $main::c && $main::c ne '' ) {
        my $preference_dir = (
            (   defined $default::preference_dir
                    && $default::preference_dir ne ''
            ) ? $default::preference_dir : '.'
            )
            . '/'
            . $main::c;
        if ( -d $preference_dir ) {
            if ( defined $default::field_pref_dir ) {
                unless ( $default::field_pref_dir
                    =~ s!^$default::preference_dir/!$default::preference_dir/$main::c/!
                    )
                {
                    $default::field_pref_dir = '.'
                        if !defined $default::field_pref_dir
                        || $default::field_pref_dir eq '';
                    $default::field_pref_dir .= '/' . $main::c;
                }
            }
            if ( defined $default::file ) {
                $default::file = '.' if $default::file eq '';
                $default::file .= '/' . $main::c;
            }
            if ( defined $default::db ) {
                $default::db = '.' if $default::db eq '';
                $default::db .= '/' . $main::c;
            }
            $default::preference_dir = $preference_dir;
            $this->import_db( $default::master_preference_dir . '/general',
                \%master::general, 1 )
                if !%master::general;
        }
        else {
            $this->jump( $ENV{'SCRIPT_NAME'} );
        }
    }

    #一般設定読み込み
    $this->import_db( $default::preference_dir . '/general',
        \%setting::general, 1 )
        if defined $setting::general && $setting::general;

    #POSTのみに制限されている場合
    undef %main::data
        if %main::data
        && $setting::general{'postMethod'}
        && $ENV{'REQUEST_METHOD'} ne "POST";

    #アクセス権設定読み込み
    $this->read_security_settings if $setting::security;

    #ID情報クッキー読み込み
    $this->home_location;
    if ( $cookie::id_enabled && !$main::id_disabled ) {
        if ( $env::session_enabled && $this->use_session ) {
            $session::id
                = $this->read_cookie_direct( $this->login_cookie_name );
            $this->{'session_is_unauthorized'} = 1
                if $main::email || $main::pswd;
            if ( $cgi::Session->is_available($session::id) ) {
                ( $main::email, $main::pswd )
                    = $cgi::Session->get_user_info($session::id)
                    if !$main::email && !$main::pswd;
                if (  !ref $main::pswd
                    && defined $main::pswd
                    && $main::pswd ne '' )
                {
                    $this->{'session_is_unauthorized'} = 1;
                }
                elsif ( ref $main::pswd && $main::pswd->{'new_email'} ) {
                    $main::new_email = $main::pswd->{'new_email'};
                    $main::new_pswd
                        = ( $main::pswd->{'new_claimed_id'} )
                        ? { 'claimed_id' => $main::pswd->{'new_claimed_id'} }
                        : ( $main::pswd->{'new_encrypted_password'} )
                        ? { 'encrypted_password' =>
                            $main::pswd->{'new_encrypted_password'} }
                        : {};
                }
            }
            else {
                $session::is_deleted = 1
                    if defined $session::id && $session::id ne '';
                undef $session::id;
            }
        }
        else {
            $this->read_cookie( $this->login_cookie_name );
        }
    }

    #ユーザー設定情報クッキー読み込み
    $this->read_cookie('CUSTOM')
        if defined $cookie::custom_enabled && $cookie::custom_enabled;
    $this->read_cookie($cookie::name)
        if defined $cookie::default_enabled
        && $cookie::default_enabled
        && !%main::data;

    #テーブル設定読み込み
    $this->use_table if defined $setting::table && $setting::table;

    #装飾設定読み込み
    if ( defined $setting::deco && $setting::deco ) {
        $this->import_db( $env::preference_dir . '/deco',
            \%setting::deco, 1, @setting::decotype );
        $this->use_frame('frame')
            if ref $setting::deco{'frame'} && %{ $setting::deco{'frame'} };
    }

    #ページ設定読み込み
    $this->import_db( $default::preference_dir . '/page',
        \%setting::page, 1, $setting::key, $setting::subkey )
        if defined $setting::page && $setting::page;

    #フィールド設定読み込み
    $this->use_field(
        'field',
        $default::field_pref_dir . '/' . $setting::field_db_name,
        $setting::field,
        $setting::field_style_disabled,
        $setting::field_tag_enabled,
        $setting::field_tag_name
    ) if defined $setting::field && $setting::field;

    #CGIのパス
    $cgi::path
        = ($env::cgi_server)
        ? $env::cgi_server . $ENV{'SCRIPT_NAME'}
        : $ENV{'SCRIPT_NAME'};

    #セキュリティチェック
    $this->unauthorized_alert
        if defined $setting::security_key
        && $setting::security_key ne ''
        && !$this->is_secure( $setting::security_key, 1, '', 1 );
}

sub read_security_settings ($) {
    my $this = shift;
    return if $setting::security_imported;
    $this->import_db( $default::preference_dir . '/security',
        \%setting::security, 1 );
    @setting::security_GoodIPs
        = split( /[\x09\x0A\x0C\x0D\x20]+/, $setting::security{'GoodIPs'} )
        if defined $setting::security{'GoodIPs'}
        && $setting::security{'GoodIPs'};
    @setting::security_GoodIPs
        = grep( defined $_ && $_ ne '', @setting::security_GoodIPs )
        if @setting::security_GoodIPs;
    @setting::security_BadIPs
        = split( /[\x09\x0A\x0C\x0D\x20]+/, $setting::security{'BadIPs'} )
        if defined $setting::security{'BadIPs'}
        && $setting::security{'BadIPs'};
    @setting::security_BadIPs
        = grep( defined $_ && $_ ne '', @setting::security_BadIPs )
        if @setting::security_BadIPs;
    @setting::security_GoodUA
        = split( /[\x09\x0A\x0C\x0D\x20]+/, $setting::security{'GoodUA'} )
        if defined $setting::security{'GoodUA'}
        && $setting::security{'GoodUA'};
    @setting::security_GoodUA
        = grep( defined $_ && $_ ne '', @setting::security_GoodUA )
        if @setting::security_GoodUA;
    @setting::security_BadUA
        = split( /[\x09\x0A\x0C\x0D\x20]+/, $setting::security{'BadUA'} )
        if defined $setting::security{'BadUA'} && $setting::security{'BadUA'};
    @setting::security_BadUA
        = grep( defined $_ && $_ ne '', @setting::security_BadUA )
        if @setting::security_BadUA;
    $setting::security_imported = 1;
}

sub unauthorized_alert ($) {
    my $this = shift;
}

#登録モジュール呼出
sub use_register ($;$) {
    my ( $this, $refresh ) = @_;
    return 1 if ref $cgi::Register && !$refresh;
    if ( !$env::register_db ) {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            '設定エラー！ : パスワード管理データベースディレクトリのパスが設定されていません。'
        );
        return 0;
    }
    $setting::general{'master'} = $env::master_email
        if $this->is_invalid_email_address( $setting::general{'master'} );
    $env::openid_enabled = (
        $env::openid_enabled && $env::session_enabled && eval {
            eval { require LWP::UserAgent };
        }
            && eval 'use Crypt::SSLeay;1;'
    ) ? 1 : 0;
    undef $cgi::Register if $refresh;
    require CMONOS::CGI_Lib::Register;
    $cgi::Register = Register->new(
        $env::sendmail,              $setting::general{'mastername'},
        $setting::general{'master'}, $env::register_db,
        $env::master_password,       $env::master_password_encrypted,
        $env::session_enabled,       $env::openid_enabled,
        $env::register_check,        $env::password_threshold,
        $env::password_latency
    );
    return 1;
}

sub get_user_name ($) {
    my $this = shift;
    if ( !defined $cgi::user_name ) {
        if ( $this->use_register && $this->is_registered_user ) {
            $security::registered = 1;
            $cgi::user_name
                = $cgi::Register->get_name_from_email($main::email);
        }
        else {
            $cgi::user_name = '';
        }
    }
    return $cgi::user_name;
}

#セッションモジュール呼出
sub use_session ($;$) {
    my ( $this, $refresh ) = @_;
    return 1 if ref $cgi::Session && !$refresh;
    if ( !$env::session_store ) {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            '設定エラー！ : セッション情報保存ディレクトリのパスが設定されていません。'
        );
        return 0;
    }
    return 0 if !$this->use_register($refresh);
    undef $cgi::Session if $refresh;
    require CMONOS::CGI_Lib::Session;
    $cgi::Session = Session->new( $env::session_store, $env::session_security,
        $env::cookie_lifetime );
    return 1;
}

######################################################################
#	obj->is_secure(key)
sub is_secure ($$;$$$$) {
    my ( $this, $key, $full_check, $subkey, $lock_required ) = @_;
    my $is_authorized = 0;
    $key = $subkey
        if ( !defined $setting::security{$key}{'passwordCheck'}
        || $setting::security{$key}{'passwordCheck'} eq '' )
        && $subkey;
    $setting::security{$key}{'passwordCheck'} = 'owner'
        if !defined $setting::security{$key}{'passwordCheck'}
        || $setting::security{$key}{'passwordCheck'} eq '';

    if (   $setting::security{$key}{'passwordCheck'} ne 'nocheck'
        || $setting::master_check_option{$key} )
    {
        $this->use_register;
        my $is_secure = 1;

        #パスワードなし
        if ( !defined $main::pswd
            || ( !ref $main::pswd && $main::pswd eq '' ) )
        {
            return 0;

            #パスロックチェック
        }
        elsif ( $lock_required && $cgi::Register->is_locked($main::email) ) {
            push(
                @{  ( ( ref $cgi::action ) ? $cgi::action : $this )
                        ->{'dialog'}
                },
                $cgi::Register->error_dialog
            );
            return 0;

            #管理パスワードチェック
        }
        elsif ($setting::security{$key}{'passwordCheck'} eq 'master'
            || $setting::master_check_option{$key} )
        {
            if ( $this->is_master ) {
                $is_authorized = 1;
            }
            else {
                $is_secure = 0;
            }

            #オーナーパスワードチェック
        }
        elsif ( $setting::security{$key}{'passwordCheck'} eq 'owner' ) {
            if ( $this->is_owner ) {
                $is_authorized = 1;
            }
            else {
                $is_secure = 0;
            }

            #共同オーナーパスワードチェック
        }
        elsif ( $setting::security{$key}{'passwordCheck'} eq 'owners' ) {
            if ( $this->is_one_of_owners ) {
                $is_authorized = 1;
            }
            else {
                $is_secure = 0;
            }

            #スタッフパスワードチェック
        }
        elsif ( $setting::security{$key}{'passwordCheck'} eq 'staff' ) {
            if ( $this->is_staff ) {
                $is_authorized = 1;
            }
            else {
                $is_secure = 0;
            }

            #登録パスワードチェック
        }
        elsif ( $setting::security{$key}{'passwordCheck'} eq 'member' ) {
            if ( $this->is_registered_user ) {
                $is_authorized = 1;
            }
            else {
                $is_secure = 0;
            }
        }

        if ($is_secure) {
            $cgi::Register->unlock($main::email) if $lock_required;
        }
        else {
            if ($lock_required) {
                $cgi::Register->lock($main::email);
                push(
                    @{  ( ( ref $cgi::action ) ? $cgi::action : $this )
                            ->{'dialog'}
                    },
                    $cgi::Register->error_dialog
                );
            }
            return 0;
        }
    }

    #セッションのユーザー情報を更新
    if (   $cookie::id_enabled
        && !$session::is_changed
        && $this->{'session_is_unauthorized'}
        && ( $is_authorized || $this->is_registered_user )
        && $this->use_session )
    {
        $session::id
            = $cgi::Session->set_user_info( $main::email, $main::pswd,
            $session::id );
        $session::is_changed = 1;
    }

    #パスワードのチェックのみ
    return 1 if !$full_check;

    #REFERERチェック
    $this->referer_is_matched(
        $setting::security{'refUrl'},
        $setting::security{'refJump'},
        $setting::security{'refOp'}
        )
        if defined $setting::security{$key}{'refererCheck'}
        && $setting::security{$key}{'refererCheck'};

    #IPチェック（許可）
    if ( defined $setting::security{$key}{'GoodIPCheck'}
        && $setting::security{$key}{'GoodIPCheck'} )
    {
        if (   $security::good_ip_checked
            || $this->client_ip_is_matched(@setting::security_GoodIPs) )
        {
            $security::good_ip_checked = 1;
        }
        else {
            $this->{'access_forbidden'} = 1;
            return 0;
        }
    }

    #IPチェック（拒否）
    if ( defined $setting::security{$key}{'BadIPCheck'}
        && $setting::security{$key}{'BadIPCheck'} )
    {
        if (   $security::bad_ip_checked
            || !@setting::security_BadIPs
            || !$this->client_ip_is_matched(@setting::security_BadIPs) )
        {
            $security::bad_ip_checked = 1;
        }
        else {
            $this->{'access_forbidden'} = 1;
            return 0;
        }
    }

    #UAチェック（許可）
    if ( defined $setting::security{$key}{'GoodUACheck'}
        && $setting::security{$key}{'GoodUACheck'} )
    {
        if ($security::good_ua_checked
            || grep( eval { $ENV{'HTTP_USER_AGENT'} =~ /$_/i },
                @setting::security_GoodUA )
            )
        {
            $security::good_ua_checked = 1;
        }
        else {
            $this->{'access_forbidden'} = 1;
            return 0;
        }
    }

    #UAチェック（拒否）
    if ( defined $setting::security{$key}{'BadUACheck'}
        && $setting::security{$key}{'BadUACheck'} )
    {
        if (   $security::bad_ua_checked
            || !@setting::security_BadUA
            || !grep( eval { $ENV{'HTTP_USER_AGENT'} =~ /$_/i },
                @setting::security_BadUA )
            )
        {
            $security::bad_ua_checked = 1;
        }
        else {
            $this->{'access_forbidden'} = 1;
            return 0;
        }
    }

    if ( !$env::session_enabled && $is_authorized ) {
        if (   $main::set_cookie_id
            || $cookie::read{ $this->login_cookie_name } )
        {
            $main::set_cookie_id = 1;
        }
        else {
            $this->{'authorization_required'}{$key} = 1;
        }
    }
    return 1;
}

#管理パスワードチェック
sub is_master ($) {
    my $this = shift;
    if ( !defined $security::master_password ) {
        $security::master_password = (
            (          !$env::master_email_required
                    || !$env::master_email
                    || ( $main::email && $main::email eq $env::master_email )
            )
                && $this->use_register
                && $cgi::Register->is_master( $main::pswd, 1 )
        ) ? 1 : 0;
    }
    return $security::master_password;
}

#オーナーパスワードチェック
sub is_owner ($) {
    my $this = shift;
    return 1 if $this->is_master;
    if ( defined $security::owner_password ) {
        return $security::owner_password;
    }
    elsif (defined $main::email
        && $main::email eq $this->get_address( $setting::general{'master'} )
        && $this->is_registered_email )
    {
        $security::owner_password = 1;
        return 1;
    }
    else {
        $security::owner_password = 0;
        return 0;
    }
}

#共同オーナーパスワードチェック
sub is_one_of_owners ($) {
    my $this = shift;
    return 1 if $this->is_master || $this->is_owner;
    if ( defined $security::owners_password ) {
        return $security::owners_password;
    }
    elsif ( $this->is_registered_owner ) {
        $security::owners_password = 1;
        return 1;
    }
    else {
        $security::owners_password = 0;
        return 0;
    }
}

sub is_registered_owner ($) {
    my $this = shift;
    my @addr = $this->get_address( $setting::general{'owners'} );
    return 0 unless defined $main::email && grep( $_ eq $main::email, @addr );
    return $this->is_registered_email;
}

#スタッフパスワードチェック
sub is_staff ($) {
    my $this = shift;
    return 1
        if $this->is_master || $this->is_owner || $this->is_one_of_owners;
    if ( defined $security::staff_password ) {
        return $security::staff_password;
    }
    elsif ( $this->is_registered_staff ) {
        $security::staff_password = 1;
        return 1;
    }
    else {
        $security::staff_password = 0;
        return 0;
    }
}

sub is_registered_staff ($) {
    my $this = shift;
    my @addr = $this->get_address( $setting::general{'staff'} );
    return 0 unless defined $main::email && grep( $_ eq $main::email, @addr );
    return $this->is_registered_email;
}

#登録パスワードチェック
sub is_registered_user ($) {
    my $this = shift;
    return 1
        if $this->is_master
        || $this->is_owner
        || $this->is_one_of_owners
        || $this->is_staff;
    return $this->is_registered_email;
}

sub is_registered_email ($) {
    my $this = shift;
    return 0 if !$this->use_register;
    if ( defined $security::registered ) {
        return $security::registered;
    }
    elsif ( $cgi::Register->is_registered( $main::pswd, $main::email ) ) {
        $security::registered = 1;
        return 1;
    }
    else {
        $security::registered = 0;
        return 0;
    }
}

#パスワードチェック
sub is_authorized ($;@) {
    my ( $this, @email ) = @_;
    if (   $this->is_master
        || $this->is_owner
        || $this->is_one_of_owners
        || $this->is_staff )
    {
        $main::set_cookie_id = 1
            if !$env::session_enabled
            && $cookie::read{ $this->login_cookie_name };
        return 1;
    }
    elsif (
           defined $main::email
        && $main::email
        && @email
        && grep( defined $_ && $_ eq $main::email, @email )
        && ($security::registered
            || (   $this->use_register
                && $cgi::Register->is_registered( $main::pswd, $main::email )
            )
        )
        )
    {
        $security::registered = 1;
        $main::set_cookie_id  = 1
            if !$env::session_enabled
            && $cookie::read{ $this->login_cookie_name };
        return 1;
    }
    else {
        return 0;
    }
}

sub reset_security ($) {
    my $this = shift;
    undef $security::master_password;
    undef $security::owner_password;
    undef $security::owners_password;
    undef $security::registered;
}

######################################################################
#	obj->import_db(db,package,name,hash_flag,@key)
sub import_db ($$$;$@) {
    my ( $this, $db, $hash_ref, $parse_flag, @key ) = @_;
    my (%db_hash);
    eval 'use ' . $env::dbm_file . ';';
    tie( %db_hash, $env::dbm_file, $db, O_RDONLY, $env::dbm_mode );
    if ( ref tied %db_hash ) {
        $this->read_db( \%db_hash, $hash_ref, $parse_flag, @key );
    }
    else {
        my ($dbname);
        $dbname = '\n' . $1 if $db ne $env::register_db && $db =~ m!([^/]+)$!;
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'データベース読み込みに失敗しました。'
                . $dbname
        );
    }
    untie(%db_hash);
}

sub read_db ($$$$;@) {
    no strict "refs";
    my ( $this, $database, $hash_ref, $parse_flag, @key ) = @_;
    my ($ignore_nosubkey);
    $ignore_nosubkey = 1 if grep( defined $_ && $_ eq '!', @key );
    @key = grep( defined $_ && $_ ne '' && $_ ne '!', @key );
    while ( my ( $key, $value ) = each %$database ) {
        utf8::decode($key)   if !utf8::is_utf8($key);
        utf8::decode($value) if !utf8::is_utf8($value);
        my ( $k1, $k2 );
        $key =~ s/[^0-9a-zA-Z_]//g;
        if ( $key =~ /^([^_]+)(_.+)$/ ) {
            $k1 = $1;
            $k2 = $2;
            next if @key && !grep( $_ eq $k1, @key );
            if ( $parse_flag && $k2 !~ /_$/ && $k2 !~ /__/ ) {

                # 多次元配列
                $k2 =~ s/_([^_]+)/\{'$1\'}/g;
                if ( $value =~ /\0/ ) {
                    eval '@{$hash_ref->{\'' . $k1 . '\'}' . $k2
                        . '} = split(/\0/,$value);';
                }
                else {
                    eval '$hash_ref->{\'' . $k1 . '\'}' . $k2 . ' = $value;';
                }
                next;
            }
        }
        if ( !$ignore_nosubkey || ( defined $k1 && $k1 ne '' ) ) {
            if ( $value =~ /\0/ ) {
                @{ $hash_ref->{$key} } = split( /\0/, $value );
            }
            else {
                $hash_ref->{$key} = $value;
            }
        }
    }
}

######################################################################
#	obj->write_db(db,deletekey,\%data)
sub write_db ($$;$$$$$@) {
    my ( $this, $db, $read, $change, $delete, $hash_ref, $parse_flag, @key )
        = @_;
    my (%db_hash);
    my $dir = $db;
    $dir =~ s!/[^/]+$!!;
    if ( !$this->set_dir($dir) ) {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'データベース書き込みに失敗しました。保存用ディレクトリが存在しません。'
        );
        return;
    }
    $change = '' if !defined $change;
    $delete = '' if !defined $delete;

    eval 'use ' . $env::dbm_file . ';';
    tie( %db_hash, $env::dbm_file, $db, O_RDWR | O_CREAT, $env::dbm_mode );

    if ( ref tied %db_hash ) {
        if ( $change ne '' || $delete ne '' ) {
            $change = join( '|', @$change ) if ref $change eq 'ARRAY';
            1 while $change =~ s/\|\|/\|/g;
            $delete = join( '|', @$delete ) if ref $delete eq 'ARRAY';
            1 while $delete =~ s/\|\|/\|/g;

            my (@delete);
            while ( my ($k) = each %db_hash ) {
                utf8::decode($k) if !utf8::is_utf8($k);
                push( @delete, $k )
                    if ( $change ne '' && $k =~ /^($change)_/ )
                    || ( $delete ne '' && $k =~ /^($delete)_/ )
                    || $k =~ /^_/;
            }
            foreach my $k (@delete) {
                utf8::encode($k) if utf8::is_utf8($k);
                delete $db_hash{$k};
            }
        }
        $this->_write_db( $read, \%db_hash, $delete ) if ref $read;
        $this->read_db( \%db_hash, $hash_ref, $parse_flag, @key )
            if ref $hash_ref;
    }
    else {
        my ($dbname);
        $dbname = '\n' . $1 if $db ne $env::register_db && $db =~ m!([^/]+)$!;
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'データベース書き込みに失敗しました。'
                . $dbname
        );
    }
    untie %db_hash;
}

sub _write_db ($$$;$$) {
    my ( $this, $read, $write, $delete, $key ) = @_;
    while ( my ( $k, $v ) = each %$read ) {
        next if !defined $k || $k eq '';
        $k = $key . '_' . $k if defined $key && $key;
        if ( ref $v eq 'HASH' ) {
            $this->_write_db( $v, $write, $delete, $k );
        }
        elsif ( defined $delete && $delete ne '' && $k =~ /^($delete)_/ ) {
            utf8::encode($k) if utf8::is_utf8($k);
            delete $$read{$k};
        }
        elsif ( defined $v && $v eq "\x7F" ) {
            utf8::encode($k) if utf8::is_utf8($k);
            delete $$read{$k};
            delete $$write{$k};
        }
        else {
            $v = join( "\0", @$v ) if ref $v eq 'ARRAY';
            $v = '' if !defined $v;
            utf8::encode($k) if utf8::is_utf8($k);
            utf8::encode($v) if utf8::is_utf8($v);
            $v = bytes::substr( $v, 0, $env::db_maxlength )
                if defined $env::db_maxlength
                && $env::db_maxlength >= 1000
                && bytes::length $v > $env::db_maxlength;
            eval '$$write{$k} = $v';
            push(
                @{  ( ( ref $cgi::action ) ? $cgi::action : $this )
                        ->{'dialog'}
                },
                '一部データの書き込みに失敗しました。容量オーバーです。'
            ) if $@;
        }
    }
}

######################################################################
#	obj->unlink_db(db,option)
sub unlink_db ($@) {
    my ( $this, @db ) = @_;
    my ($db);
    foreach $db (@db) {
        my ( $name, @unlink );
        if ( $db =~ s/([^\/]+)$// ) {
            $name = $1;
        }
        else {
            next;
        }
        opendir( DIR, $db );
        @unlink = map( $db . $_, grep( $_ =~ /$name(\.|$)/, readdir(DIR) ) );
        closedir(DIR);
        unlink @unlink;
    }
}

######################################################################
#	obj->delete_data($$@)
sub delete_data ($$@) {
    my ( $this, $path, @pat ) = @_;
    return (wantarray) ? () : 0 if !$path;
    my ( %db_hash, @delete, $delete_count );
    @delete       = ();
    $delete_count = 0;
    eval 'use ' . $env::dbm_file . ';';
    tie( %db_hash, $env::dbm_file, $path, O_RDWR, $env::dbm_mode );
    if ( ref tied %db_hash ) {

        while ( my ($k) = each %db_hash ) {
            utf8::decode($k) if !utf8::is_utf8($k);
            push( @delete, $k ) if grep( $k =~ /$_/, @pat ) > 0;
        }
        foreach my $k (@delete) {
            utf8::encode($k) if utf8::is_utf8($k);
            $delete_count++  if delete $db_hash{$k};
        }
        $delete_count = @delete if !$delete_count;
    }
    else {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'データ削除に失敗しました。'
        );
    }
    untie %db_hash;
    return (wantarray) ? @delete : $delete_count;
}

#	obj->delete_all_data($$@)
sub delete_all_data ($$@) {
    my ( $this, $dir, @keys ) = @_;
    return if !$dir;
    my ( @files, %database, %db_hash );
    local (*DIR);

    opendir( DIR, $dir );
    @files = readdir(DIR);
    closedir(DIR);

    foreach my $db (@files) {
        next if $db =~ /^\./ || $db =~ /\.htm[l]?$/i || !-f $dir . '/' . $db;
        $db =~ s/\.[^\/\.]+$//;
        $database{$db} = 1;
    }

    eval 'use ' . $env::dbm_file . ';';
    while ( my ($db) = each %database ) {
        tie( %db_hash, $env::dbm_file, $dir . '/' . $db,
            O_RDWR, $env::dbm_mode );
        if ( ref tied %db_hash ) {
            foreach my $k (@keys) {
                utf8::encode($k) if utf8::is_utf8($k);
                delete $db_hash{$k};
            }
        }
        untie %db_hash;
    }
}

######################################################################
# JavaScript ファイル作成
sub mk_external_javascript($$;$$$) {
    my ( $this, $external_file, $external_js, $html, $raw_script,
        $http_domain )
        = @_;
    if ( !defined $external_js || $external_js eq '' ) {
        $external_js = $external_file;
        $external_js .= '.js' unless $external_js =~ s/\.[0-9a-zA-Z_]+$/.js/;
    }
    $html = $this->read_file( $external_file, 0, 0, 1 )
        if !defined $html || $html eq '';
    $html
        = $this->html_to_javascript( $html,
        ( ($env::embed_js_on_the_same_domain) ? 0 : 1 ),
        $raw_script, $http_domain );
    utf8::encode($html) if utf8::is_utf8($html);
    local (*JS_FILE);
    if ( open( JS_FILE, ">>$external_js" ) ) {
        binmode JS_FILE;
        flock( JS_FILE, LOCK_EX );
        truncate( JS_FILE, 0 );
        seek( JS_FILE, 0, 0 );
        print JS_FILE $html;
        close(JS_FILE);
        return 1;
    }
    else {
        return 0;
    }
}

sub html_to_javascript ($$;$$$) {
    my ( $this, $html, $append_domain, $raw_script, $http_domain ) = @_;
    my ( $javascript, $domain );
    $javascript = '';
    if ($append_domain) {
        $domain
            = ( $http_domain && $http_domain =~ m!^(https?://[^/]+)! )
            ? lc $1
            : $this->http_my_domain;
        $html =~ s/(<script(?![0-9a-zA-Z_])[^>]+>)/\x0A$1/sig;
        $html =~ s!<noscript>.*?</noscript>!!sig;
        $html
            =~ s!(<[^<>]*[\x09\x0A\x0C\x0D\x20]+(href|src|background)=["']?)(/[^<>]*>)!$1$domain$3!sg;
        $html
            =~ s!([\(,]')(/[0-9a-zA-Z_/\.\-]+\.(gif|jpg|png)')!$1$domain$2!sg;
    }
HTML:
    while ( $html
        =~ /(<script(?![0-9a-zA-Z_])[^>]+>.*?<\/script>|<!--[^>]*?-->|([^\x0A\x0D]+)(?!<script(?![0-9a-zA-Z_])[^>]+>)|[\x0A\x0D]+)/isg
        )
    {
        my $temp = $1;
        next
            if $temp !~ /[^\x09\x0A\x0C\x0D\x20]/
            || $temp =~ /^<!--[^>]*?-->$/;
        if ( $temp =~ /<script(?![0-9a-zA-Z_])([^>]+)>(.*?)<\/script>/is ) {
            my ( $option, $script ) = ( $1, $2 );
            if ( $option !~ /[\x09\x0A\x0C\x0D\x20]+src=/i ) {
                if ($raw_script) {
                    $script
                        =~ s/^[\x09\x0A\x0C\x0D\x20]*<!\-+[\x09\x0A\x0C\x0D\x20]*//s;
                    $script
                        =~ s/[\x09\x0A\x0C\x0D\x20]*\-+>[\x09\x0A\x0C\x0D\x20]*$//s;
                    $script
                        =~ s/^[\x09\x0A\x0C\x0D\x20]*<!\[CDATA\[[\x09\x0A\x0C\x0D\x20]*//s;
                    $script
                        =~ s/[\x09\x0A\x0C\x0D\x20]*\]\]>[\x09\x0A\x0C\x0D\x20]*$//s;
                    $script
                        =~ s/[\x09\x0A\x0C\x0D\x20]*\/\/[\x09\x0A\x0C\x0D\x20]*$//s;
                    $script
                        =~ s!(\.src[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'])/!$1$domain/!sg
                        if $domain;
                    $javascript .= $script . "\x0A";
                    next;
                }
                else {
                    $temp
                        =~ s!(\.src[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'])/!$1$domain/!sg
                        if $domain;
                    $temp =~ s/\\/\\\\/g;
                    $temp =~ s/[\x0A\x0D]+/\\n/sg;
                }
            }
        }
        else {
            $temp =~ s/<!--[^>]*?-->//gs;
        }
        $temp =~ s/'/\\'/g;
        $javascript .= 'document.write(\'' . $temp . '\');' . "\x0A";
    }
    return $javascript;
}

######################################################################
# ファイル埋め込み
sub include_external_file ($$$;$$) {
    my ( $this, $external_file, $external_flag, $output, $abs_flag ) = @_;
    return ''
        if !defined $external_file
        || $external_file eq ''
        || !-f $external_file;
    if ( defined $external_flag && $external_flag ) {
        if ( $output && $env::ssi && $env::ssi_ext ) {
            return '<!--#include virtual="'
                . $this->path_change( $external_file, $ENV{'SCRIPT_NAME'},
                'abs' )
                . '"-->';
        }
        else {
            return $this->include_by_javascript( $external_file, $output,
                $abs_flag );
        }
    }
    else {
        return $this->read_file( $external_file, 1 );
    }
}

######################################################################
# JavaScript による埋め込み
sub include_by_javascript ($$$;$) {
    my ( $this, $external_file, $output, $abs_flag ) = @_;
    my $external_js = $external_file;
    $external_js .= '.js' unless $external_js =~ s/\.[0-9a-zA-Z_]+$/.js/;
    $this->mk_external_javascript( $external_file, $external_js )
        if !-f $external_js;
    $output = $cgi::output if !defined $output || !$output;
    return '<script type="text/javascript" language="JavaScript" src="'
        . ( ($abs_flag)
        ? $this->path_change( $external_js, $ENV{'SCRIPT_NAME'}, 'abs' )
        : ($output) ? $this->base_change( $external_js, $output )
        :             $external_js )
        . '" class="sim-ssi"></script>';
}

######################################################################
# 装飾オブジェクト
sub use_deco ($;$) {
    my ( $this, $output ) = @_;
    unless ( ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    $cgi::Deco->set_output($output);
    return $cgi::Deco;
}

sub use_icon ($$;$$$) {
    my ( $this, $icon_dir, $icon_name, $output, $refresh ) = @_;
    if ( !ref $cgi::icon || $refresh ) {
        my ( $icon_pref, $setting );
        $icon_pref = '';
        if ($icon_name) {
            $icon_pref = $env::preference_dir . '/' . $icon_name;
            $setting   = \%setting::icon;
            $this->import_db( $icon_pref, $setting, 1 ) if !%setting::icon;
        }
        require CMONOS::CGI_Lib::Deco;
        $cgi::icon = Deco->new(
            $env::image_dir . '/' . $icon_dir,
            $env::image_lib . '/' . $icon_dir,
            $output, $setting, $icon_pref
        );
    }
    else {
        $cgi::icon->set_output($output);
    }
    return $cgi::icon;
}

sub use_frame ($$) {
    my ( $this, $key ) = @_;
    $this->import_db( $env::preference_dir . '/deco', \%setting::deco, 1 )
        if !%setting::deco;
    require CMONOS::CGI_Lib::Frame;
    while ( my ($k) = each %{ $setting::deco{$key} } ) {
        unless ( ref $frame::obj{$key}{$k} ) {
            $frame::obj{$key}{$k} = Frame->new;
        }
        %{ $frame::obj{$key}{$k} } = (
            'library' => 'frame',
            'id'      => $k,
            'cssbase' => 1,
            'type'    => (
                ( $setting::deco{$key}{$k}{'type'} )
                ? $setting::deco{$key}{$k}{'type'}
                : 'frame'
            )
        );
        undef %{ $setting::deco{$key}{$k} };
    }
    undef %{ $setting::deco{$key} };
    delete $setting::deco{$key};
}

######################################################################
#	obj->use_field
sub use_field ($$$$$$;$) {
    no strict "refs";
    my ( $this, $p, $db, $setting, $field_style_disabled, $field_tag_enabled,
        $field_tag_name, $all_flag )
        = @_;
    my ( $field_obj, $field_for );
    $this->import_db( $db, \%setting::field, 1 ) if !%setting::field;
    require CMONOS::CGI_Lib::Field;
    return if !%setting::field;
    if ($all_flag) {
        $this->call_field($setting);
    }
    else {
        my %required = ();
        while ( my ( $id, $field ) = each %setting::field ) {
            next
                if $id eq ''
                || $id =~ /[^0-9a-zA-Z_]/
                || $id =~ /[^a-z0-9]/
                || !ref $field;
            $required{ $setting::field{$id}{'pkg'} }++
                if defined $setting::field{$id}{'pkg'}
                && $setting::field{$id}{'pkg'} ne '';
        }
        $this->call_field( $setting, \%required );
    }
    $p = 'field' if !$p;
    %{ $p . '::obj' } = ();
    $field_obj = \%{ $p . '::obj' };
    %{ $p . '::for' } = ();
    $field_for = \%{ $p . '::for' };
    while ( my ( $id, $field ) = each %setting::field ) {
        next if $id eq '' || $id =~ /[^0-9a-zA-Z_]/;
        if ( $id eq 'for' || $id =~ /[^a-z0-9]/ ) {
            if ( ref $field eq 'HASH' ) {
                while ( my ( $k, $v ) = each %$field ) {
                    ${ $p . '::' . $id }{$k} = $v;
                }
            }
            else {
                ${ $p . '::' . $id } = $field;
            }
        }
        else {
            next unless ref $field;
            if ( ref $field_obj->{$id} ) {
                undef %{ $field_obj->{$id} };
                delete $field_obj->{$id};
            }
            $field_obj->{$id} = Field->new(
                $field->{'class'},
                $id,
                (   ( defined $field_tag_name && $field_tag_name ne '' )
                    ? $field_tag_name
                    : 'field'
                ),
                $field_obj,
                $field_for
            );
            delete $field->{'class'};
            $field_obj->{$id}->set(
                %{$field},
                (   (   defined $field_style_disabled
                            && $field_style_disabled ne ''
                    )
                    ? ( 'field_style_disabled' => $field_style_disabled )
                    : ()
                ),
                (   (   defined $field_tag_enabled
                            && $field_tag_enabled ne ''
                    ) ? ( 'field_tag_enabled' => $field_tag_enabled ) : ()
                )
            );
        }
        undef %{$field} if ref $field;
    }
    undef %setting::field;
}

sub call_field ($$;$) {
    my ( $this, $setting, $required ) = @_;
    require CMONOS::CGI_Lib::Field;
    if ( $setting >= 1
        && ( !defined $field::enabled || $field::enabled <= $setting ) )
    {
        Common::use_modules( 'CGI_Lib/Field', \@default::field,
            \@default::ignorefield, $required );
        $field::enabled = 1;
    }
    if ( $setting >= 2 && $field::enabled <= $setting ) {
        Common::use_modules(
            $default::lib . '/Field', \@default::field,
            \@default::ignorefield,   $required
        );
        $field::enabled = 2;
    }
}

######################################################################
#	obj->use_table
sub use_table ($) {
    my $this = shift;
    my ( $k, $v );
    $this->import_db( $env::preference_dir . '/table', \%setting::table, 1 )
        if !%setting::table;
    require CMONOS::CGI_Lib::Table;
    while ( ( $k, $v ) = each %setting::table ) {
        next unless ref $v;
        $table::obj{$k} = Table->new unless ref $table::obj{$k};
        $table::obj{$k}->set( $setting::table{$k}{'setting'} );
        $table::obj{$k}->{'name'} = $setting::table{$k}{'name'};
        undef %{$v};
    }
    undef %setting::table;
}

######################################################################
#クッキー設定
sub mkcookie ($) {
    my $this = shift;
    $env::cookie_path = $this->home_directory if !$env::cookie_path;
    if ($env::session_enabled) {
        if ( $session::is_changed && $session::id ) {
            $this->set_cookie_direct( $env::cookie_lifetime,
                $this->login_cookie_name, $session::id, $env::cgi_server,
                $env::cookie_path );
        }
        elsif ($session::is_deleted) {
            $this->set_cookie_direct( -1, $this->login_cookie_name, 'null',
                $env::cgi_server, $env::cookie_path );
        }
    }
    elsif ($main::set_cookie_id) {
        $this->set_cookie(
            $env::cookie_lifetime,
            $this->login_cookie_name,
            $env::cgi_server,
            $env::cookie_path,
            { 'pswd', $main::pswd, 'email', $main::email }
        );
    }
    if ( $main::set_cookie && %cookie::data ) {
        $cookie::lifetime = $env::cookie_lifetime
            if !defined $cookie::lifetime || $cookie::lifetime eq '';
        $cookie::name = uc $main::m if $cookie::name eq '' && $main::m ne '';
        $cookie::domain = $env::cgi_server
            if !defined $cookie::domain || $cookie::domain eq '';
        $cookie::path = $env::cookie_path
            if !defined $cookie::path || $cookie::path eq '';
        $this->set_cookie(
            $cookie::lifetime, $cookie::name, $cookie::domain,
            $cookie::path,     \%cookie::data
        );
    }
}

sub log_out ($) {
    my $this = shift;
    $env::cookie_path = $this->home_directory if !$env::cookie_path;
    if ( $env::session_enabled && $this->use_session ) {
        $cgi::Session->log_out($session::id);
        undef $session::id;
    }
    $this->set_cookie_direct( -1, $this->login_cookie_name, 'null',
        $env::cgi_server, $env::cookie_path );
    return $this;
}

######################################################################
#	obj->form_start
sub form_start ($@) {
    my ( $this, %attribute ) = @_;
    my ( $out, $k, $v, $authorization_required, $required, $path );
    $authorization_required
        = (    ref $this->{'authorization_required'}
            && $setting::key
            && defined $this->{'authorization_required'}{$setting::key} )
        ? $this->{'authorization_required'}{$setting::key}
        : 0;
    if ( defined $attribute{'key'} ) {
        $authorization_required = 1
            if ref $this->{'authorization_required'}
            && defined $this->{'authorization_required'}{ $attribute{'key'} }
            && $cgi::authorization_required{ $attribute{'key'} };
        delete $attribute{'key'};
    }
    if ( defined $attribute{'authorization'} ) {
        if ( $attribute{'authorization'} eq 'required' ) {
            $authorization_required = 1;
        }
        elsif ($attribute{'authorization'} eq 'no'
            || $attribute{'authorization'} eq 'disabled' )
        {
            $authorization_required = 0;
        }
        delete $attribute{'authorization'};
    }
    if ( defined $attribute{'path'} ) {
        $path = $attribute{'path'} if $attribute{'path'} =~ m!^/!;
        delete $attribute{'path'};
    }
    $attribute{'enctype'} = 'multipart/form-data' if $attribute{'multi'};
    delete $attribute{'multi'};
    if ( ref $attribute{'required'} ) {
        $required = $attribute{'required'};
        delete $attribute{'required'};
    }
    $authorization_required = 0
        if $env::session_enabled
        || !defined $main::pswd
        || ref $main::pswd
        || $main::pswd eq '';
    if (   $setting::general{'postMethod'}
        || $authorization_required
        || ( defined $attribute{'enctype'}
            && $attribute{'enctype'} eq 'multipart/form-data' )
        )
    {
        $attribute{'method'} = 'post';
    }
    elsif ( !defined $attribute{'method'} || !$attribute{'method'} ) {
        $attribute{'method'} = 'get';
    }
    $attribute{'action'} = ($cgi::path) ? $cgi::path : $ENV{'SCRIPT_NAME'}
        if !defined $attribute{'action'} || !$attribute{'action'};
    $attribute{'action'} .= $path if $path;

    $out = '<form';
    while ( ( $k, $v ) = each %attribute ) {
        $out .= ' ' . $k . '="' . $v . '"' if defined $v && $v ne '';
    }
    $out .= '>';
    $out .= '<input type="hidden" name="_" value="＿" />';
    if ( $authorization_required
        && !$cookie::read{ $this->login_cookie_name } )
    {
        $out .= '<input type="hidden" name="pswd" value="'
            . $this->escape($main::pswd) . '" />';
        $out
            .= '<input type="hidden" name="email" value="'
            . $this->escape($main::email) . '" />'
            if defined $main::email;
    }
    $out
        .= '<input type="hidden" name="clipboard_id" value="'
        . $this->escape( $cgi::input->{'clipboard_id'} ) . '" />'
        if ref $cgi::input
        && $cgi::input->{'clipboard_id'}
        && !$cgi::input->{'clipboard_expired'};
    return ( ref $required && @$required )
        ? $out . $this->set_query( 'post', $required )
        : $out;
}

######################################################################
# ウィンドウを閉じる
sub close_button ($;$$$$) {
    my ( $this, $url, $close_image, $return_image, $closebutton_only ) = @_;
    $this->use_deco($cgi::output);
    my $is_xhtml = (
        ( $default::doctype && $default::doctype =~ /XHTML/ )
            || ( !$default::doctype
            && $env::doctype
            && $env::doctype =~ /XHTML/ )
    ) ? 1 : 0;
    my $close_button
        = '<script type="text/javascript" language="JavaScript">' . "\x0A";
    $close_button .= '<!--' . ( ($is_xhtml) ? ' <![CDATA[' : '' ) . "\x0A";
    $close_button
        .= 'if (!parent || parent == self || (self.name && self.name == \'temporary-target-iframe\')) {'
        . "\x0A";
    $close_button
        .= '	if (((!parent || parent == self) && window.opener) || (parent && parent != self && window.parent.opener)) {'
        . "\x0A";
    $close_button .= '		document.write(\''
        . $cgi::Deco->button( 'close', $close_image, '#',
        ( ($default::english) ? 'Close.' : '閉じる' ),
        '', { 'onclick' => 'window.close();return false;' } )
        . '\');' . "\x0A";

    if ( !$closebutton_only ) {
        $close_button .= '	} else {' . "\x0A";
        $close_button .= '		document.write(\''
            . $cgi::Deco->button(
            'return',
            $return_image,
            $url,
            ( ($default::english) ? 'Return.' : '戻る' ),
            '',
            ( ($url) ? '' : { 'onclick' => 'history.back();return false;' } )
            ) . '\');' . "\x0A";
    }
    $close_button .= '	}' . "\x0A";
    $close_button .= '}' . "\x0A";
    $close_button .= '// ' . ( ($is_xhtml) ? ']]> ' : '' ) . '-->' . "\x0A";
    $close_button .= '</script>' . "\x0A";
    $close_button .= '<noscript><a href="'
        . (
        ($url) ? $this->escape($url) : $cgi::path
            . (
            ( defined $main::c && $main::c ne '' ) ? '?c=' . $main::c : ''
            )
        )
        . '">'
        . ( ($default::english) ? 'Return.' : '戻る' )
        . '</a></noscript>' . "\x0A";
    return $close_button;
}

######################################################################
#タグ変換
sub item_tag_replace ($$$$$;$$) {
    my ( $this, $tag, $attributes, $content, $data, $file_flag, $nolimit )
        = @_;
    return '' if !$file_flag && ( !defined $data || $data eq '' );
    my ( $header, $footer );
    ( $header, $footer, $attributes )
        = $this->get_header_and_footer( $attributes, $content );
    $data = $this->cut_string( $this->detag_except_anchor($data), $1, 'i' )
        if defined $data
        && $data ne ''
        && !$nolimit
        && $attributes =~ /[\x09\x0A\x0C\x0D\x20]+maxlength="([0-9]+)"/s;
    return $this->make_item_tag( $tag, $attributes, $data, $header, $footer,
        $file_flag );
}

sub make_item_tag ($$$;$$$$) {
    my ( $this, $tag, $attributes, $data, $header, $footer, $file_flag ) = @_;
    return $header . $data . $footer if !$file_flag;
    if ( defined $data && $data ne '' ) {
        return
              '<!--'
            . $tag
            . $attributes . '-->'
            . ( ( $header ne '' ) ? '<!--head-->' . $header . '<!--/head-->'
            : '' )
            . '<!--data-->'
            . $data
            . '<!--/data-->'
            . ( ( $footer ne '' ) ? '<!--foot-->' . $footer . '<!--/foot-->'
            : '' )
            . '<!--/'
            . $tag . '-->';
    }
    else {
        return
              '<!--'
            . $tag
            . $attributes
            . ( ( $header ne '' )
            ? ' head="' . $this->lf_to_entity( $this->escape($header) ) . '"'
            : '' )
            . ( ( $footer ne '' )
            ? ' foot="' . $this->lf_to_entity( $this->escape($footer) ) . '"'
            : '' )
            . '--><!--/'
            . $tag . '-->';
    }
}

sub datetime_tag_replace ($$$$$$$$;$) {
    my ($this,      $tag,           $attributes,   $content,
        $datetime,  $default_limit, $default_type, $default_format,
        $file_flag, $nolimit
    ) = @_;
    my $type
        = ( $attributes
            =~ /[\x09\x0A\x0C\x0D\x20]+type[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/si
        ) ? $this->unescape( $this->entity_to_lf($1) ) : '';
    my $limit
        = ( $attributes
            =~ /[\x09\x0A\x0C\x0D\x20]+limit[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/si
        ) ? $this->unescape( $this->entity_to_lf($1) ) : '';
    my $format = '';
    if ( $content
        =~ s/[\x09\x0A\x0C\x0D\x20]*<\!--format(?![0-9a-zA-Z_])[^>]*-->(.*?)<\!--\/format(?![0-9a-zA-Z_])[^>]*-->//si
        )
    {
        $format = $1;
        $attributes
            .= ' format="'
            . $this->lf_to_entity( $this->escape($format) ) . '"'
            unless $attributes
            =~ s/([\x09\x0A\x0C\x0D\x20]+format[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*")[^"]*(")/$1 . $this->lf_to_entity($this->escape($format)) . $2/gesi;
    }
    elsif ( $attributes
        =~ /[\x09\x0A\x0C\x0D\x20]+format[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/si
        )
    {
        $format = $this->unescape( $this->entity_to_lf($1) );
    }
    $datetime = ( $type || $format )
        ? $this->date(
        $datetime,
        ( $limit  || $default_limit ),
        ( $type   || $default_type ),
        ( $format || $default_format )
        )
        : $this->date( $datetime, $default_limit, $default_type,
        $default_format );
    return $this->item_tag_replace( $tag, $attributes, $content, $datetime,
        $file_flag );
}

sub get_header_and_footer ($$$) {
    my ( $this, $attributes, $content ) = @_;
    my ( $header, $footer );
    $header = $footer = '';
    if ( $content
        =~ s/^[\x09\x0A\x0C\x0D\x20]*<\!--head(?![0-9a-zA-Z_])[^>]*-->(.*?)<\!--\/head(?![0-9a-zA-Z_])[^>]*-->//s
        )
    {
        $header = $1;
    }
    elsif ( $attributes
        =~ /[\x09\x0A\x0C\x0D\x20]+head[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s
        )
    {
        $header = $this->unescape( $this->entity_to_lf($1) );
    }
    if ( $content
        =~ s/<\!--foot(?![0-9a-zA-Z_])[^>]*-->((?:.(?!<\!--foot-->))*?)<\!--\/foot(?![0-9a-zA-Z_])[^>]*-->[\x09\x0A\x0C\x0D\x20]*\Z//s
        )
    {
        $footer = $1;
    }
    elsif ( $attributes
        =~ /[\x09\x0A\x0C\x0D\x20]+foot[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s
        )
    {
        $footer = $this->unescape( $this->entity_to_lf($1) );
    }
    $attributes
        =~ s/[\x09\x0A\x0C\x0D\x20]+head[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//sg;
    $attributes
        =~ s/[\x09\x0A\x0C\x0D\x20]+foot[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//sg;
    $attributes = '' if $attributes !~ /[^\x09\x0A\x0C\x0D\x20]/;
    return ( $header, $footer, $attributes, $content );
}

sub join_header_and_footer ($$$) {
    my ( $this, $attributes, $content ) = @_;
    my ( $header, $footer );
    $header = $footer = '';
    if ( $content
        =~ s/^[\x09\x0A\x0C\x0D\x20]*<\!--head(?![0-9a-zA-Z_])[^>]*-->(.*?)<\!--\/head(?![0-9a-zA-Z_])[^>]*-->[\x09\x0A\x0C\x0D\x20]*//s
        )
    {
        $header = $1;
    }
    elsif ( $attributes
        =~ /[\x09\x0A\x0C\x0D\x20]+head[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s
        )
    {
        $header = $this->unescape( $this->entity_to_lf($1) );
    }
    if ( $content
        =~ s/[\x09\x0A\x0C\x0D\x20]*<\!--foot(?![0-9a-zA-Z_])[^>]*-->((?:.(?!<\!--foot-->))*?)<\!--\/foot(?![0-9a-zA-Z_])[^>]*-->[\x09\x0A\x0C\x0D\x20]*\Z//s
        )
    {
        $footer = $1;
    }
    elsif ( $attributes
        =~ /[\x09\x0A\x0C\x0D\x20]+foot[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s
        )
    {
        $footer = $this->unescape( $this->entity_to_lf($1) );
    }
    return $header . $content . $footer;
}

sub get_hash_from_template ($$) {
    my ( $this, $template ) = @_;
    my $hash;
    while ( $template =~ /<!--([0-9a-zA-Z_]+)([^>]*)-->(.*?)<!--\/\1-->/sg ) {
        my ( $k, $v1, $v2 ) = ( $1, $2, $3 );
        return $v2 if $k eq 'data';
        $hash = {} if !ref $hash;
        if ( $v1 =~ /[\x09\x0A\x0C\x0D\x20]+value="(.*?)"/ ) {
            $this->_set_hash_value( $hash, $k, $1 );
        }
        else {
            $this->_set_hash_value( $hash, $k,
                $this->get_hash_from_template($v2) );
        }
    }
    return ( ref $hash ) ? $hash : $template;
}

sub _set_hash_value ($$$$) {
    my ( $this, $hash, $k, $v ) = @_;
    if ( defined $hash->{$k} ) {
        if ( ref $hash->{$k} eq 'ARRAY' ) {
            if ( ref $v eq 'ARRAY' ) {
                push( @{ $hash->{$k} }, @$v );
            }
            else {
                push( @{ $hash->{$k} }, $v );
            }
        }
        elsif ( ref $hash->{$k} eq 'HASH' ) {
            if ( ref $v eq 'ARRAY' ) {
                $this->_set_hash_value( $hash->{$k}, 'ARRAY', $v );
            }
            elsif ( ref $v eq 'HASH' ) {
                my ( $key, $value );
                while ( ( $key, $value ) = each %$v ) {
                    $this->_set_hash_value( $hash->{$k}, $key, $value );
                }
            }
            else {
                $this->_set_hash_value( $hash->{$k}, 'DATA', $v );
            }
        }
        else {
            $hash->{$k} = [ $hash->{$k}, $v ];
        }
    }
    else {
        $hash->{$k} = $v;
    }
}

sub include_header_and_footer ($$$;$) {
    my ( $this, $attributes, $content, $tag ) = @_;
    $content = '' if !defined $content;
    if (   $attributes
        && $attributes
        =~ s/[\x09\x0A\x0C\x0D\x20]+head[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//s
        )
    {
        my $head = $1;
        $content
            =~ s/^[\x09\x0A\x0C\x0D\x20]*<!--head-->.*?<!--\/head-->[\x09\x0A\x0C\x0D\x20]*//s;
        $content
            = '<!--head-->'
            . $this->unescape( $this->entity_to_lf($head) )
            . '<!--/head-->'
            . $content;
    }
    if (   $attributes
        && $attributes
        =~ s/[\x09\x0A\x0C\x0D\x20]+foot[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//s
        )
    {
        my $foot = $1;
        $content
            =~ s/[\x09\x0A\x0C\x0D\x20]*<\!--foot(?![0-9a-zA-Z_])[^>]*-->(?:.(?!<\!--foot-->))*?<\!--\/foot(?![0-9a-zA-Z_])[^>]*-->[\x09\x0A\x0C\x0D\x20]*\Z//s;
        $content
            .= '<!--foot-->'
            . $this->unescape( $this->entity_to_lf($foot) )
            . '<!--/foot-->';
    }
    if ($tag) {
        return
              '<!--'
            . $tag
            . $attributes . '-->'
            . $content . '<!--/'
            . $tag . '-->';
    }
    else {
        return $content;
    }
}

######################################################################
sub javascript_link ($$) {
    my ( $this, $uri ) = @_;
    return
          '<script type="text/javascript" language="JavaScript" src="'
        . $uri
        . '"></script>';
}

sub javascript ($$$) {
    my ( $this, $script, $is_xhtml ) = @_;
    return '' if !defined $script || $script eq '';
    $script .= "\x0A" if $script !~ /\x0A$/;
    $is_xhtml = (
        ( $default::doctype && $default::doctype =~ /XHTML/ )
            || ( !$default::doctype
            && $env::doctype
            && $env::doctype =~ /XHTML/ )
        ) ? 1 : 0
        if !defined $is_xhtml;
    return
          '<script type="text/javascript" language="JavaScript">' . "\x0A"
        . '<!--'
        . ( ($is_xhtml) ? ' <![CDATA[' : '' ) . "\x0A"
        . $script . '// '
        . ( ($is_xhtml) ? ']]> ' : '' ) . '-->' . "\x0A"
        . '</script>';
}

######################################################################
sub sim_ssi ($$) {
    my ( $this, $html_ref ) = @_;
    $$html_ref
        =~ s/<!--#include(?![0-9a-zA-Z_])[^>]*virtual="(\/[^"]+\.ssi)"[^>]*-->/$this->_sim_ssi($1)/sge;
}

sub _sim_ssi ($$) {
    my ( $this, $path ) = @_;
    my $js = $path;
    $js =~ s/\.ssi$/.js/;
    if ( -f $this->path_change( $js, '', 'rel' ) ) {
        return
              '<script type="text/javascript" language="JavaScript" src="'
            . $js
            . '" class="sim-ssi"></script>';
    }
    else {
        return $this->read_file( $path, 0, 0, 0, 'utf8' );
    }
}

######################################################################
#RSS更新
sub feed_rss ($$;$$$$$) {
    my ( $this, $item, $rss_dir, $link, $master_rss_dir, $master_link,
        $feed_podcast )
        = @_;
    return 0
        if !defined $item
        || ref $item ne 'HASH'
        || ( !$item->{'link'}
        && !ref $item->{'delete'}
        && !ref $item->{'delete_prefix'} );
    my $favicon_url = $this->full_path( $this->favicon_path );
    my $favicon
        = ($favicon_url) ? { 'value' => { 'url' => $favicon_url } } : '';
    my @podcast_type = (
        'application/pdf', 'application/epub+zip',
        'audio/mpeg',      'audio/m4a',
        'video/mp4',       'video/quicktime',
        'video/x-m4v',     'document/x-epub'
    );
    my $delete_only = (
        (   ( ref $item->{'delete'} && @{ $item->{'delete'} } )
                || ( ref $item->{'delete_prefix'}
                && @{ $item->{'delete_prefix'} } )
        )
            && !$item->{'link'}
    ) ? 1 : 0;
    require CMONOS::CGI_Lib::Feed::RSS2_0;
    if ($feed_podcast
        || (   ref $item->{'enclosure'}
            && $item->{'enclosure'}{'type'}
            && grep( $_ eq $item->{'enclosure'}{'type'}, @podcast_type ) )
        )
    {
        require CMONOS::CGI_Lib::Feed::Podcast;
        $feed_podcast = 1;
    }
    elsif ( ref $item->{'enclosure'} && !$item->{'enclosure'}{'url'} ) {
        delete $item->{'enclosure'};
    }

    #サイト全体RSS更新
    my $title
        = ( ( defined $env::title && $env::title ne '' ) ? $env::title : '' );
    my $copyright
        = ( defined $env::copyright && $env::copyright ne '' )
        ? $env::copyright
        : '';
    my $home_url = $this->full_path('../../');
    if ( $setting::general{'siteRSSEnabled'} ) {
        RSS2_0->new(
            'path'    => '../../index.xml',
            'maxitem' => ( ($env::rss_maxitem) ? $env::rss_maxitem : 15 ),
            'description_maxlength' => (
                ($env::rss_description_maxlength)
                ? $env::rss_description_maxlength
                : 128
            ),
            'delete_only' => $delete_only,
            'xmlns'       => {
                'content' => 'http://purl.org/rss/1.0/modules/content/',
                'dc'      => 'http://purl.org/dc/elements/1.1/',
                (     ($env::checkout)
                    ? ( 'g' => 'http://base.google.com/ns/1.0' )
                    : ()
                )
            },
            'utm_medium'      => 'Site',
            'channel_element' => {
                'title' => ( ( $title ne '' ) ? $title : $home_url ),
                'link' => $home_url,
                'description' => (
                    ( defined $env::description && $env::description ne '' )
                    ? $env::description
                    : $home_url
                ),
                'copyright' => $copyright,
                'image'     => $favicon,
            }
        )->update($item);
    }

    #サイト全体コメントRSS更新
    if ( $setting::general{'commentRSSEnabled'} ) {
        RSS2_0->new(
            'path'    => '../../comment.xml',
            'maxitem' => ( ($env::rss_maxitem) ? $env::rss_maxitem : 15 ),
            'description_maxlength' => (
                ($env::rss_description_maxlength)
                ? $env::rss_description_maxlength
                : 128
            ),
            'delete_only' => $delete_only,
            'xmlns'       => {
                'content' => 'http://purl.org/rss/1.0/modules/content/',
                'dc'      => 'http://purl.org/dc/elements/1.1/'
            },
            'utm_medium'      => 'Site(Comment)',
            'channel_element' => {
                'title' => (
                    ( $title ne '' ) ? $title . ':新着コメント'
                    : '新着コメント'
                ),
                'link'        => $home_url,
                'description' => '新着コメント情報です。',
                'copyright'   => $copyright,
                'image'       => $favicon,
            },
        )->update($item);
    }

    #サイト全体ポッドキャストRSS更新
    if ($feed_podcast
        && (   $setting::general{'siteRSSEnabled'}
            || $setting::general{'commentRSSEnabled'} )
        )
    {
        Podcast->new(
            'path'    => '../../podcast.xml',
            'maxitem' => ( ($env::rss_maxitem) ? $env::rss_maxitem : 15 ),
            'description_maxlength' => (
                ($env::rss_description_maxlength)
                ? $env::rss_description_maxlength
                : 128
            ),
            'delete_only' => $delete_only,
            'xmlns'       => {
                'content' => 'http://purl.org/rss/1.0/modules/content/',
                'dc'      => 'http://purl.org/dc/elements/1.1/'
            },
            'utm_medium'      => 'Site(Podcast)',
            'channel_element' => {
                'title' => ( ( $title ne '' ) ? $title : $home_url ),
                'link' => $home_url,
                'description' => (
                    ( defined $env::description && $env::description ne '' )
                    ? $env::description
                    : $home_url
                ),
                'copyright' => $copyright,
                'image'     => $favicon,
            }
        )->update($item);
    }

    #マスターRSS更新
    my $cgi_id = ($default::id) ? $default::id : '';
    $cgi_id =~ s/^_//;
    $cgi_id =~ s/_/:/g;
    if (   $master_rss_dir
        && defined $main::c
        && $main::c ne ''
        && $this->set_dir($master_rss_dir) )
    {
        my $description = '';
        my $icon        = '';
        if ( $default::subdivided && $default::master_preference_dir ) {
            $title
                .= ( ( $title ne '' ) ? ':' : '' )
                . $master::general{'cginame'}
                if defined $master::general{'cginame'}
                && $master::general{'cginame'} ne '';
            $copyright
                .= ( ( $copyright ne '' ) ? '/' : '' )
                . $master::general{'mastername'}
                if defined $master::general{'mastername'}
                && $master::general{'mastername'} ne '';
            $icon
                = ( $master::general{'icon'}
                    && -f $master_rss_dir . '/' . $master::general{'icon'} )
                ? {
                'value' => {
                    'url' => $this->full_path(
                        $master_rss_dir . '/' . $master::general{'icon'}
                    )
                }
                }
                : $favicon;
            $description = $master::general{'description'}
                if defined $master::general{'description'};
        }
        else {
            $title
                .= ( ( $title ne '' ) ? ':' : '' )
                . $setting::general{'cginame'}
                if defined $setting::general{'cginame'}
                && $setting::general{'cginame'} ne '';
            $copyright
                .= ( ( $copyright ne '' ) ? '/' : '' )
                . $setting::general{'mastername'}
                if defined $setting::general{'mastername'}
                && $setting::general{'mastername'} ne '';
            $icon
                = ( $setting::general{'icon'}
                    && -f $master_rss_dir . '/' . $setting::general{'icon'} )
                ? {
                'value' => {
                    'url' => $this->full_path(
                        $master_rss_dir . '/' . $setting::general{'icon'}
                    )
                }
                }
                : $favicon;
            $description = $setting::general{'description'}
                if defined $setting::general{'description'};
        }
        if ( $setting::general{'masterRSSEnabled'} ) {
            my $channel_element = {
                'title' => ( ( $title ne '' ) ? $title : $home_url ),
                'link' => $master_link,
                'description' =>
                    ( ( $description ne '' ) ? $description : $home_url ),
                'copyright' => $copyright,
                'image'     => ( ( ref $icon ) ? $icon : $favicon ),
            };
            RSS2_0->new(
                'path'    => $master_rss_dir . '/index.xml',
                'maxitem' => ( ($env::rss_maxitem) ? $env::rss_maxitem : 15 ),
                'description_maxlength' => (
                    ($env::rss_description_maxlength)
                    ? $env::rss_description_maxlength
                    : 128
                ),
                'delete_only' => $delete_only,
                'xmlns'       => {
                    'content' => 'http://purl.org/rss/1.0/modules/content/',
                    'dc'      => 'http://purl.org/dc/elements/1.1/',
                    (     ($env::checkout)
                        ? ( 'g' => 'http://base.google.com/ns/1.0' )
                        : ()
                    )
                },
                'utm_medium'      => $cgi_id,
                'channel_element' => $channel_element,
            )->update($item);
            if ($feed_podcast) {
                Podcast->new(
                    'path' => $master_rss_dir . '/podcast.xml',
                    'maxitem' =>
                        ( ($env::rss_maxitem) ? $env::rss_maxitem : 15 ),
                    'description_maxlength' => (
                        ($env::rss_description_maxlength)
                        ? $env::rss_description_maxlength
                        : 128
                    ),
                    'delete_only' => $delete_only,
                    'xmlns'       => {
                        'content' =>
                            'http://purl.org/rss/1.0/modules/content/',
                        'dc' => 'http://purl.org/dc/elements/1.1/'
                    },
                    'utm_medium'      => $cgi_id,
                    'channel_element' => $channel_element,
                )->update($item);
            }
        }
    }

    #RSS更新
    if (   $rss_dir
        && $setting::general{'RSSEnabled'}
        && $this->set_dir($rss_dir) )
    {
        if (   $master_rss_dir
            && defined $main::c
            && $main::c ne ''
            && !$default::subdivided )
        {
            $this->import_db( $default::preference_dir . '/category',
                \%setting::category, 1 )
                if !%setting::category;
            $title
                .= ( ( $title ne '' ) ? ':' : '' )
                . $setting::category{$main::c}{'name'}
                if ref $setting::category{$main::c}
                && defined $setting::category{$main::c}{'name'}
                && $setting::category{$main::c}{'name'} ne '';
        }
        else {
            $copyright
                .= ( ( $copyright ne '' ) ? '/' : '' )
                . $setting::general{'mastername'}
                if defined $setting::general{'mastername'}
                && $setting::general{'mastername'} ne '';
            $title
                .= ( ( $title ne '' ) ? ':' : '' )
                . $setting::general{'cginame'}
                if defined $setting::general{'cginame'}
                && $setting::general{'cginame'} ne '';
        }
        my $icon
            = ( $setting::general{'icon'}
                && -f $rss_dir . '/' . $setting::general{'icon'} )
            ? {
            'value' => {
                'url' => $this->full_path(
                    $rss_dir . '/' . $setting::general{'icon'}
                )
            }
            }
            : $favicon;
        my $utm_medium
            = $cgi_id
            . ( ( $master_rss_dir && defined $main::c && $main::c ne '' )
            ? ':' . $main::c
            : '' );
        my $channel_element = {
            'title' => ( ( $title ne '' ) ? $title : $home_url ),
            'link' => $link,
            'description' => (
                (   defined $setting::general{'description'}
                        && $setting::general{'description'} ne ''
                ) ? $setting::general{'description'} : $home_url
            ),
            'copyright' => $copyright,
            'image'     => ( ( ref $icon ) ? $icon : $favicon ),
        };
        RSS2_0->new(
            'path'    => $rss_dir . '/index.xml',
            'maxitem' => ( ($env::rss_maxitem) ? $env::rss_maxitem : 15 ),
            'description_maxlength' => (
                ($env::rss_description_maxlength)
                ? $env::rss_description_maxlength
                : 128
            ),
            'delete_only' => $delete_only,
            'xmlns'       => {
                'content' => 'http://purl.org/rss/1.0/modules/content/',
                'dc'      => 'http://purl.org/dc/elements/1.1/',
                (     ($env::checkout)
                    ? ( 'g' => 'http://base.google.com/ns/1.0' )
                    : ()
                )
            },
            'utm_medium'      => $utm_medium,
            'channel_element' => $channel_element,
        )->update($item);
        if ($feed_podcast) {
            Podcast->new(
                'path'    => $rss_dir . '/podcast.xml',
                'maxitem' => ( ($env::rss_maxitem) ? $env::rss_maxitem : 15 ),
                'description_maxlength' => (
                    ($env::rss_description_maxlength)
                    ? $env::rss_description_maxlength
                    : 128
                ),
                'delete_only' => $delete_only,
                'xmlns'       => {
                    'content' => 'http://purl.org/rss/1.0/modules/content/',
                    'dc'      => 'http://purl.org/dc/elements/1.1/'
                },
                'utm_medium'      => $utm_medium,
                'channel_element' => $channel_element,
            )->update($item);
        }
    }
}

sub delete_rss_item ($$$$;$$) {
    my ( $this, $delete, $rss_dir, $link, $master_rss_dir, $master_link,
        $feed_podcast )
        = @_;
    $this->feed_rss(
        { 'delete' => ( ( ref $delete ) ? $delete : [$delete] ) },
        $rss_dir, $link, $master_rss_dir, $master_link, $feed_podcast );
}

sub delete_rss_item_prefix ($$$$;$$) {
    my ( $this, $delete, $rss_dir, $link, $master_rss_dir, $master_link,
        $feed_podcast )
        = @_;
    $this->feed_rss(
        { 'delete_prefix' => ( ( ref $delete ) ? $delete : [$delete] ) },
        $rss_dir, $link, $master_rss_dir, $master_link, $feed_podcast );
}

sub favicon_path ($) {
    my $this         = shift;
    my $favicon_path = '';
    if ( -f '../../favicon.ico' ) {
        $favicon_path = '../../favicon.ico';
    }
    elsif ( -f '../../favicon.png' ) {
        $favicon_path = '../../favicon.png';
    }
    elsif ( -f '../../favicon.gif' ) {
        $favicon_path = '../../favicon.gif';
    }
    elsif ( -f '../../favicon.jpg' ) {
        $favicon_path = '../../favicon.jpg';
    }
    else {
        my $count = 0;
        while ( $ENV{'SCRIPT_NAME'} =~ m!\G/[^/]+!g ) {
            $count++;
            next if $count < 3;
            my $path = ( '../' x $count );
            if ( -f $path . 'favicon.ico' ) {
                $favicon_path = $path . 'favicon.ico';
                last;
            }
            elsif ( -f $path . 'favicon.png' ) {
                $favicon_path = $path . 'favicon.png';
                last;
            }
            elsif ( -f $path . 'favicon.gif' ) {
                $favicon_path = $path . 'favicon.gif';
                last;
            }
            elsif ( -f $path . 'favicon.jpg' ) {
                $favicon_path = $path . 'favicon.jpg';
                last;
            }
        }
    }
    return $favicon_path;
}

######################################################################
# 検索マッチ
sub pattern_matched ($$$;$$$) {
    my ( $this, $str, $patterns, $ignore_case, $mode, $count ) = @_;
    return 1
        if !defined $patterns
        || !ref $patterns
        || !@$patterns
        || !ref $str
        || !defined $$str;
    $mode = ( $mode && $mode eq 'and' ) ? 'and' : 'or';
    my $i            = ($ignore_case) ? '(?i)' : '';
    my $matched      = 0;
    my $current_mode = $mode;
    for ( my $n = 0; $n < @$patterns; $n++ ) {
        my $pat = $patterns->[$n];
        next if $pat eq '';
        if ( $pat =~ /^(?:and|&&)$/i ) {
            $current_mode = 'and';
            next;
        }
        elsif ( $pat =~ /^(?:or|\|+)$/i ) {
            $current_mode = 'or';
            next;
        }
        my $keep = ( $patterns->[ $n + 1 ]
                && $patterns->[ $n + 1 ] =~ /^(?:or|\|+)$/i ) ? 1 : 0;
        if ( $pat =~ s/^\0// ) {
            if ( $$str !~ /$i$pat/ ) {
                return 1 if !$count && $current_mode ne 'and';
                $matched++;
            }
            elsif ( $current_mode eq 'and' ) {
                return 0 if !$keep;
                $matched = 0;
            }
        }
        else {
            if ( $$str =~ /$i$pat/ ) {
                return 1 if !$count && $current_mode ne 'and';
                $matched++;
            }
            elsif ( $current_mode eq 'and' ) {
                return 0 if !$keep;
                $matched = 0;
            }
        }
        return 0
            if !$keep && !$matched && $current_mode eq 'or' && $mode eq 'and';
        $current_mode = $mode;
    }
    return ($count) ? $matched : ( $current_mode eq 'and' ) ? 1 : 0;
}

sub make_patterns ($$;$$) {
    my ( $this, $str, $ignore_case, $mode ) = @_;
    return () if !defined $str || !ref $str || !defined $$str;
    my $temp;
    my @patterns;
    my $quot = 0;
    while ( $$str =~ /\G([^\s\x{3000}]*)([\s\x{3000}]+|\Z)/gs ) {
        next if $1 eq '';
        my ( $v, $s ) = ( $1, $2 );
        $quot += ( $v =~ tr/"/"/ )
            if ( defined $v && $v =~ /^\-?"/ )
            || ( defined $temp && $temp =~ /^\-?"/ );
        if ( $quot && $quot % 2 > 0 ) {
            $temp .= $v . $s;
            next;
        }
        else {
            $v = $temp . $v if defined $temp;
            if ( $v !~ /^(?:and|or|&&|\|+)$/si ) {
                my $is_exception = ( $v =~ s/^\-//s ) ? 1 : 0;
                if ( $v =~ s/^"// ) {
                    $v =~ s/"$//;
                    $v =~ s/""/"/g;
                }
                undef $temp;
                $v =~ s/\0//g;
                $v = quotemeta($v);

                if ($ignore_case) {

                    #ひらがな→カタカナ+カタカナ→ひらがな
                    $v
                        =~ s/([\x{3041}-\x{3096}\x{30A1}-\x{30F6}])/$this->make_kana_pattern($1)/ges;

                    #A→Ａ+Ａ→A
                    $v
                        =~ s/([a-zA-Z0-9\x{ff00}-\x{ff5e}])/$this->make_latin_pattern($1,1)/ges;

                    #空白
                    $v =~ s/\\?[\s\x{3000}]+/[\\s\\x{3000}]+/gs;

                    #ハイフン
                    $v =~ s/\\?[\-ーー]+/[\\-ーー]+/gs;
                }

                if ( $mode && $mode eq 'A' ) {
                    $v = '\\A' . $v;
                }
                elsif ( $mode && $mode eq 'Z' ) {
                    $v .= '\\Z';
                }
                elsif ( $mode && $mode eq 'eq' ) {
                    $v = '\\A' . $v . '\\Z';
                }
                $v = "\0" . $v if $is_exception;
            }
            push( @patterns, $v );
            $quot = 0;
        }
    }
    return (@patterns);
}

sub make_kana_pattern ($$) {
    my ( $this, $w ) = @_;
    my $w2 = $w;
    $w2 =~ s/([\x{3041}-\x{3096}])/pack("U",unpack("U",$1) + 96)/ges
        || $w =~ s/([\x{30A1}-\x{30F6}])/pack("U",unpack("U",$1) - 96)/ges;
    my $w3 = $this->z_to_h( $w2, 'utf8', 1 );
    return '(?:' . $w . '|' . $w2 . '|' . $w3 . ')';
}

sub make_latin_pattern ($$;$) {
    my ( $this, $w, $i ) = @_;
    my $w2 = $w;
    $w2 =~ s/([a-zA-Z0-9])/pack("U",unpack("U",$1) + 65248)/ges
        || $w
        =~ s/([\x{ff00}-\x{ff5e}])/quotemeta(pack("U",unpack("U",$1) - 65248))/ges;
    return '[' . $w . $w2 . ']' if !$i || $w =~ /[0-9]/;
    my $w3 = ( $w =~ /[a-z]/ ) ? uc $w : lc $w;
    my $w4 = $w3;
    $w4 =~ s/([a-zA-Z0-9])/pack("U",unpack("U",$1) + 65248)/ges;
    return '[' . $w . $w2 . $w3 . $w4 . ']';
}

######################################################################
BEGIN {
    $env::dbm_file = 'AnyDBM_File'
        if !defined $env::dbm_file || !$env::dbm_file;
    $env::dbm_mode = 0666 if !defined $env::dbm_mode || !$env::dbm_mode;
}

1;
