package Calendar;
use strict;
use utf8;
######################################################################
# カレンダーモジュール
# Calendar.pm 1.0.6
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
#　support@cmonos.jp
#　http://cmonos.jp/
#  最終更新日：2010-08-20
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0)-0 オブジェクトを作る。
#		require CMONOS::CGI_Lib::Calendar;
#		obj = Calendar->new();
#
#	(0)-1 オプションを設定する。
#		obj->set(
#			'css' => '1',							# 1=cssを利用して表示する
#			'jcode' => 'utf8',						# utf8/euc/jis/sjis
#			'bg' => '1',							# 1=テーブルの背面を反映させる
#			'cellpadding' => '2',					# cellpadding
#			'cellspacing' => '2',					# cellspacing
#			'border' => '1',						# border
#			'bordercolor' => '#000000',				# bordercolor
#			'background' => './bg.gif',				# background
#			'width' => '140',						# カレンダーテーブル全体のwidth
#			'height' => '100',						# カレンダーテーブル全体のheight
#			'option' => '',							# 上記以外のカレンダーテーブル全体オプション
#			'cellwidth' => '20',					# セル幅
#			'cellheight' => '20',					# セル高さ
#			'cellalign' => 'center',				# セルalign
#			'cellvalign' => 'middle',				# セルvalign
#			'bgcolor' => '#ffffff',					# 標準bgcolor
#			'fontcolor' => '#000000',				# 標準文字色
#			'fontsize' => '2',						# 標準文字サイズ
#			'style' => 'font-size: 9pt;',			# 標準文字スタイル
#			'bold' => '1',							# 標準文字太字
#			'headerstyle' => 'font-size: 9pt;',		# 月名文字スタイル
#			'headerfontcolor' => '#000000',			# 月名文字色
#			'headerfontsize' => '2',				# 月名文字サイズ
#			'headerbold' => '1',					# 月名文字太字
#			'thbgcolor' => '#000099',				# 曜日bgcolor
#			'thfontcolor' => '#ffffff',				# 曜日文字色
#			'thfontsize' => '2',					# 曜日文字サイズ
#			'thstyle' => 'font-size: 9pt;',			# 曜日文字スタイル
#			'thbold' => '1',						# 曜日文字太字
#			'linkstyle' => 'font-size: 9pt;',		# リンク文字スタイル
#			'linkbold' => '1',						# リンク文字太字
#			'weekformat' => 'j',					# 曜日
#			#（j=日本語/e=英語/f=フル/u=ucfirst/U=大文字/L=小文字）
#			'monthformat' => 'j',					# 月
#			#（j=日本語/J=陰暦月名/e=英語/f=フル/u=ucfirst/U=大文字/L=小文字）
#			'yearformat' => 'j',					# 年
#			#（J=年号/w=西暦/j=西暦日本語表記/k=漢字表記）
#			'saturdaybgcolor' => '#ccccff',			# 土曜日bgcolor
#			'sundaybgcolor' => '#ffcccc',			# 日曜日bgcolor
#			'holidaybgcolor' => '#ffcccc',			# 休日bgcolor
#			'specialdaybgcolor' => '#ffffcc'		# 特別な日bgcolor
#			'specialwday' => '0,6'					# 特別な曜日[0-6]（カンマ区切り）
#			'specialwdayexception' => '1/1,12/31'	# 特別な曜日の例外（カンマ区切り）
#			'specialwdayextra' => '1/1,12/31'		# 特別な曜日の追加（カンマ区切り）
#			'specialwdaybgcolor' => '#ffffcc'		# 特別な曜日bgcolor
#		);
#
# ---------------------------------------------------------------------------------------
#	(1)-1 カレンダーの作成
#		obj->calendar(year,mon,[\x09\x0A\x0C\x0D\x20]pecialday,\cell);
#			year			年（西暦4桁）
#			mon				月（1-12）
#			[\x09\x0A\x0C\x0D\x20]pecialday		特別な日データHASHのリファレンス	$hash{月}{日} = 'コメント';
#							例）$special{'1'}{'1'} = '誕生日'
#			\cell			セルへの書き込みHASH	$hash{月}{日} = '<br />休み';
#
#	(1)-2 CSSの取得
#		css = obj->mk_css;
#
# ---------------------------------------------------------------------------------------
#	(2) カレンダーの文字にリンクを埋め込む
#		obj->set_link(\calendar,id,url[,target[,comment]]);
#			\calendar		カレンダーHTMLのリファレンス
#			id				年（西暦4桁）月（01-12）日（01-31）
#							例）2001年1月2日->20010102
#			url				リンク先のURL（空白の場合、リンクを削除）
#			target			ターゲットウィンドウ
#			comment			リンクについてのコメント
#
######################################################################
#	■利用規約
#	このスクリプトの著作権はCMONOSが保有しています。
#	このサブルーチン集は、以下の条件に従うかぎり自由に再配布できます。
#		・著作権表示を変更してはいけません。
#		・修正を加えた場合は、修正者名・修正者の連絡先・修正個所・修正日
#		　時を明記して下さい。
#	このスクリプトを利用したことによって生じた、いかなる損害にも
#	CMONOSは責任を負いません。
######################################################################
#オブジェクト作成
if ( !ref $cgi::Jcode ) {
    use Jcode;
    $Jcode::FALLBACK = &Jcode::FB_XMLCREF if defined $Jcode::FALLBACK;
    $cgi::Jcode = Jcode->new;
}

sub new ($@) {
    my ( $pkg, @setting ) = @_;
    my $this = {};
    bless $this;
    $this->set(@setting);
    return $this;
}

# --------------------------------------------------------------------
#	(0)-1 初期設定
sub set ($@) {
    my ( $this, @setting ) = @_;
    my ( $k, $v );

    if (@setting) {

        #文字列設定の処理
        if ( @setting == 1 ) {
            my $setting = $setting[0];
            @setting = ();
            while ( $setting
                =~ /\G[\x09\x0A\x0C\x0D\x20]*([^=\x09\x0A\x0C\x0D\x20]+)[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*("([^"]*)"|'([^']*)')/g
                )
            {
                push( @setting, $1, $3 . $4 );
            }

            #配列
        }
        else {
            while ( ( $k, $v ) = splice( @setting, 0, 2 ) ) {
                next if !defined $k || $k eq '' || !defined $v;
                if ( $k eq 'specialwday' ) {
                    @{ $this->{'specialwday'} }
                        = ( ref $v ) ? @$v : split( /,/, $v )
                        if defined $v && ( ref $v eq 'ARRAY' || $v ne '' );
                }
                elsif ( $k =~ /^specialwday(exception|extra)$/ ) {
                    while ( $v =~ /([0-9]+)\/([0-9]+),?\x0A?/g ) {
                        $this->{$k}{$1}{$2} = 1;
                    }
                }
                else {
                    $this->{$k} = $v;
                }
            }
        }
    }

 # 曜日（j=日本語/e=英語/f=フル/u=ucfirst/U=大文字/l=小文字）
    if ( defined $this->{'weekformat'} && $this->{'weekformat'} =~ /e/ ) {
        @{ $this->{'week_title'} } = (
            'Sunday',   'Monday', 'Tuesday', 'Wednesday',
            'Thursday', 'Friday', 'Saturday'
        );
        if ( $this->{'weekformat'} eq 'eI' ) {
            @{ $this->{'week'} } = ( 'S', 'M', 'T', 'W', 'T', 'F', 'S' );
        }
        else {
            @{ $this->{'week'} } = @{ $this->{'week_title'} };
            if ( $this->{'weekformat'} =~ /U/ ) {
                @{ $this->{'week'} } = map( uc $_, @{ $this->{'week'} } );
            }
            elsif ( $this->{'weekformat'} =~ /L/ ) {
                @{ $this->{'week'} } = map( lc $_, @{ $this->{'week'} } );
            }
            @{ $this->{'week'} }
                = map( substr( $_, 0, 3 ), @{ $this->{'week'} } )
                if $this->{'weekformat'} !~ /f/;
        }
    }
    else {
        @{ $this->{'week'} }
            = ( '日', '月', '火', '水', '木', '金', '土' );
        @{ $this->{'week_title'} } = (
            '日曜日', '月曜日', '火曜日', '水曜日',
            '木曜日', '金曜日', '土曜日'
        );
    }

# 月（j=日本語/J=伝統的日本語/e=英語/f=フル/u=ucfirst/U=大文字/l=小文字）
    if ( defined $this->{'monthformat'} && $this->{'monthformat'} =~ /e/ ) {
        @{ $this->{'month'} } = (
            '',       'January',   'February', 'March',
            'April',  'May',       'June',     'July',
            'August', 'September', 'October',  'November',
            'December'
        );
        if ( $this->{'monthformat'} =~ /U/ ) {
            @{ $this->{'month'} } = map( uc $_, @{ $this->{'month'} } );
        }
        elsif ( $this->{'monthformat'} =~ /L/ ) {
            @{ $this->{'month'} } = map( lc $_, @{ $this->{'month'} } );
        }
        @{ $this->{'month'} }
            = map( substr( $_, 0, 3 ), @{ $this->{'month'} } )
            if $this->{'monthformat'} !~ /f/;

    }
    elsif ( defined $this->{'monthformat'} && $this->{'monthformat'} =~ /J/ )
    {
        @{ $this->{'month'} } = (
            '',       '睦月', '如月',    '弥生',
            '卯月', '皐月', '水無月', '文月',
            '葉月', '長月', '神無月', '霜月',
            '師走'
        );

    }
    else {
        @{ $this->{'month'} } = (
            '',     '1月', '2月', '3月', '4月',  '5月',
            '6月', '7月', '8月', '9月', '10月', '11月',
            '12月'
        );
        @{ $this->{'month'} } = $this->kansuuji( $this->{'monthformat'},
            @{ $this->{'month'} } );
    }

    #フォントオプション
    $this->font_option('');
    $this->font_option('th');
    $this->font_option('header');
    $this->font_option('link');

    #セルオプション
    delete $this->{'tdoption'};
    delete $this->{'troption'};
    return $this if $this->{'css'};

    $this->{'tdoption'} .= ' width="' . $this->{'cellwidth'} . '"'
        if defined $this->{'cellwidth'} && $this->{'cellwidth'};
    $this->{'tdoption'} .= ' height="' . $this->{'cellheight'} . '"'
        if defined $this->{'cellheight'} && $this->{'cellheight'};
    $this->{'tdoption'} .= $this->{'celloption'}
        if defined $this->{'celloption'} && $this->{'celloption'};
    $this->{'troption'} .= ' align="' . $this->{'cellalign'} . '"'
        if defined $this->{'cellalign'} && $this->{'cellalign'};
    $this->{'troption'} .= ' valign="' . $this->{'cellvalign'} . '"'
        if defined $this->{'cellvalign'} && $this->{'cellvalign'};
    return $this;
}

sub font_option ($$) {
    my ( $this, $op ) = @_;
    my ($fontoption);
    $op = '' if !defined $op;
    delete $this->{ $op . 'fontoption' };
    delete $this->{ $op . 'fontoptionend' };
    return if $this->{'css'};

    if (   $this->{ $op . 'fontcolor' }
        || $this->{ $op . 'fontsize' }
        || $this->{ $op . 'fontface' }
        || $this->{'fontcolor'}
        || $this->{'fontsize'}
        || $this->{'fontface'} )
    {
        $this->{ $op . 'fontoption' } .= '<font';
        $this->{ $op . 'fontoption' }
            .= ' color="'
            . ( ( $this->{ $op . 'fontcolor' } )
            ? $this->{ $op . 'fontcolor' }
            : $this->{'fontcolor'} )
            . '"'
            if $this->{ $op . 'fontcolor' } || $this->{'fontcolor'};
        $this->{ $op . 'fontoption' }
            .= ' size="'
            . ( ( $this->{ $op . 'fontsize' } )
            ? $this->{ $op . 'fontsize' }
            : $this->{'fontsize'} )
            . '"'
            if $this->{ $op . 'fontsize' } || $this->{'fontsize'};
        $this->{ $op . 'fontoption' }
            .= ' face="'
            . ( ( $this->{ $op . 'fontface' } )
            ? $this->{ $op . 'fontface' }
            : $this->{'fontface'} )
            . '"'
            if $this->{ $op . 'fontface' } || $this->{'fontface'};
        $this->{ $op . 'fontoption' } .= '>';
        $fontoption++;
    }
    $this->{ $op . 'fontoption' } .= '<strong>'
        if defined $this->{ $op . 'bold' } && $this->{ $op . 'bold' };

    $this->{ $op . 'fontoptionend' } .= '</strong>'
        if defined $this->{ $op . 'bold' } && $this->{ $op . 'bold' };
    $this->{ $op . 'fontoptionend' } .= '</font>'
        if defined $fontoption && $fontoption;
}

# 漢字化
sub kansuuji ($$@) {
    my ( $this, $option, @num ) = @_;
    return (wantarray) ? @num : $num[0]
        if !defined $option || $option !~ /k/i;
    my (@k);
    @k = ( $option =~ /K/ )
        ? (
        '〇', '壱', '弐', '参', '四', '五',
        '六', '七', '八', '九'
        )
        : (
        '〇', '一', '二', '三', '四', '五',
        '六', '七', '八', '九'
        );
    map( $_ =~ s/([0-9])/$k[$1]/g, @num );
    return (wantarray) ? @num : $num[0];
}

# 年号化
sub nengo ($$$$) {
    my ( $this, $year, $mon, $mday ) = @_;
    my ( $id, $nengo );
    $mon++;
    $id = $year . sprintf( "%.2d", $mon ) . sprintf( "%.2d", $mday );

    if ( $id - 19890108 > 0 ) {
        $year -= 1988;
        $nengo = '平成';
    }
    elsif ( $id - 19261225 > 0 ) {
        $year -= 1925;
        $nengo = '昭和';
    }
    elsif ( $id - 19120730 > 0 ) {
        $year -= 1911;
        $nengo = '大正';
    }
    elsif ( $id - 19680908 > 0 ) {
        $year -= 1967;
        $nengo = '明治';
    }
    else {
        $nengo = '西暦';
    }
    return $nengo . ( ( $year == 1 ) ? '元' : $year );
}

#閏年判定
sub is_leap_year ($$) {
    my ( $this, $year ) = @_;
    return 1 if ( $year % 4 == 0 && $year % 100 != 0 ) || $year % 400 == 0;
}

#ツェラーの公式
sub get_wday ($$$$) {
    my ( $this, $year, $mon, $mday ) = @_;
    if ( $mon == 1 || $mon == 2 ) {
        $year--;
        $mon += 12;
    }
    return
        int( $year
            + int( $year / 4 )
            - int( $year / 100 )
            + int( $year / 400 )
            + int( 2.6 * $mon + 1.6 )
            + $mday )
        % 7;
}

######################################################################
#	(1) カレンダーの作成
sub calendar ($$$;$$$) {
    my ( $this, $year, $mon, $special, $cell, $cap_link ) = @_;
    my ( $id, $calendar, $holiday );
    return unless $year && $mon;
    $year += 0;
    $mon  += 0;

    require CMONOS::CGI_Lib::Holiday;
    $holiday = Holiday->new($year);

    $id = sprintf( "%.4d", $year ) . sprintf( "%.2d", $mon );
    $calendar = '<!--calendar id="' . $id . '"-->';

    #カレンダーテーブル
    $calendar .= '<table class="calendar"';
    if ( !$this->{'css'} ) {
        $calendar .= ' width="' . $this->{'width'} . '"'
            if defined $this->{'width'} && $this->{'width'} ne '';
        $calendar .= ' height="' . $this->{'height'} . '"'
            if defined $this->{'height'} && $this->{'height'} ne '';
        $calendar .= ' ' . $this->{'option'}
            if defined $this->{'option'} && $this->{'option'} ne '';
        $calendar .= ' style="' . $this->{'style'} . '"'
            if defined $this->{'style'} && $this->{'style'} ne '';

        #テーブル調整
        if ((      !defined $this->{'cellpadding'}
                || $this->{'cellpadding'} eq ''
                || $this->{'cellpadding'} > 0
            )
            && defined $this->{'border'}
            && $this->{'border'} > 0
            && defined $this->{'bordercolor'}
            && $this->{'bordercolor'} ne ''
            && !$this->{'bg'}
            )
        {

            $this->{'nobg'} = 1;

            #テーブルの背面を無視する
            $calendar .= ' cellpadding="0" border="0">';
            $calendar .= $this->get_caption( $id, $year, $mon, $cap_link );
            $calendar
                .= '<tbody><tr><td bgcolor="' . $this->{'bordercolor'} . '">';
            $calendar .= '<table width="100%" border="0"';
            $calendar .= ' cellpadding="' . $this->{'cellpadding'} . '"'
                if defined $this->{'cellpadding'}
                && $this->{'cellpadding'} ne '';
            $calendar .= ' background="' . $this->{'background'} . '"'
                if defined $this->{'background'}
                && $this->{'background'} ne '';
            $calendar .= ' cellspacing="' . $this->{'border'} . '"';
            $calendar .= '>';

        }
        else {
            $calendar .= ' cellpadding="' . $this->{'cellpadding'} . '"'
                if defined $this->{'cellpadding'}
                && $this->{'cellpadding'} ne '';
            $calendar .= ' cellspacing="' . $this->{'cellspacing'} . '"'
                if defined $this->{'cellspacing'}
                && $this->{'cellspacing'} ne '';
            $calendar .= ' border="' . $this->{'border'} . '"'
                if defined $this->{'border'} && $this->{'border'} ne '';
            $calendar .= ' bordercolor="' . $this->{'bordercolor'} . '"'
                if defined $this->{'bordercolor'}
                && $this->{'bordercolor'} ne '';
            $calendar
                .= ' bordercolorlight="' . $this->{'bordercolorlight'} . '"'
                if defined $this->{'bordercolorlight'}
                && $this->{'bordercolorlight'} ne '';
            $calendar
                .= ' bordercolordark="' . $this->{'bordercolordark'} . '"'
                if defined $this->{'bordercolordark'}
                && $this->{'bordercolordark'} ne '';
            $calendar .= ' background="' . $this->{'background'} . '"'
                if defined $this->{'background'}
                && $this->{'background'} ne '';
            $calendar .= '>';
            $calendar .= $this->get_caption( $id, $year, $mon, $cap_link );

        }
    }
    else {
        $calendar .= ' cellpadding="' . $this->{'cellpadding'} . '"'
            if defined $this->{'cellpadding'} && $this->{'cellpadding'} ne '';
        $calendar .= ' cellspacing="' . $this->{'cellspacing'} . '"'
            if defined $this->{'cellspacing'} && $this->{'cellspacing'} ne '';
        $calendar .= '>';
        $calendar .= $this->get_caption( $id, $year, $mon, $cap_link );
    }

    $calendar .= $this->week;
    $calendar .= '<tbody>';
    $calendar .= $this->_calendar( $year, $mon, $special, $cell, $holiday );
    $calendar .= '</tbody></table></td></tr>'
        if !$this->{'css'} && $this->{'nobg'};
    $calendar .= '</tbody></table>';
    $calendar .= '<br />' if !$this->{'css'};
    $calendar .= '<!--/calendar-->';
    if ( $this->{'jcode'} && $this->{'jcode'} ne 'utf8' ) {
        utf8::encode($calendar) if utf8::is_utf8($calendar);
        Jcode::convert( \$calendar, $this->{'jcode'}, 'utf8' );
    }
    return $calendar;
}

#月名
sub get_caption ($$$$;$) {
    my ( $this, $id, $year, $mon, $link ) = @_;
    my $caption
        = '<caption'
        . ( ( !$this->{'css'} && $this->{'headerstyle'} )
        ? ' style="' . $this->{'headerstyle'} . '"'
        : '' )
        . '>';
    $caption .= $this->{'headerfontoption'}
        if !$this->{'css'} && defined $this->{'headerfontoption'};
    $caption
        .= '<a name="cal'
        . $id
        . '" id="cal'
        . $id . '"'
        . ( ($link) ? ' href="' . $this->escape($link) . '"' : '' ) . '>';
    if ( defined $this->{'yearformat'} && $this->{'yearformat'} =~ /e/ ) {
        $caption .= $this->{'month'}[$mon] . ', ' . $year;
    }
    else {
        my ($header);
        $header
            = ( defined $this->{'yearformat'}
                && $this->{'yearformat'} =~ /J/ )
            ? $this->nengo( $year, $mon - 1, 0 )
            : $year;
        $header .= '年';
        $header .= $this->{'month'}[$mon];
        $header = $this->kansuuji( $this->{'yearformat'}, $header );
        $caption .= $header;
    }
    $caption .= '</a>';
    $caption .= $this->{'headerfontoptionend'}
        if !$this->{'css'} && defined $this->{'headerfontoptionend'};
    $caption .= '</caption>';
    return $caption;
}

sub escape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/&/&amp;/gs;
    $str =~ s/"/&quot;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    return $str;
}

sub set_caption ($$$;$) {
    my ( $this, $calendar, $monid, $link ) = @_;
    $$calendar
        =~ s/(<!--calendar(?!=[0-9a-zA-Z_])([^>]*)-->.*?<!--\/calendar-->)/$this->_set_caption($1,$2,substr($monid,0,6),$link)/sge;
    return $this;
}

sub _set_caption ($$$$;$) {
    my ( $this, $calendar, $setting, $monid, $link ) = @_;
    return $calendar
        if $setting
        !~ /[\x09\x0A\x0C\x0D\x20]*id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"[\x09\x0A\x0C\x0D\x20]*/
        || $1 != $monid;
    my $caption = $this->get_caption(
        $monid,
        (     ( $monid =~ /^([0-9]{4})([0-9]{2})/ )
            ? ( $1 + 0, $2 + 0 )
            : ( '', '' )
        ),
        $link
    );
    $calendar =~ s/(<caption(?![0-9a-zA-Z_])[^>]*>.*?<\/caption>)/$caption/sg;
    return $calendar;
}

#曜日名行
sub week ($) {
    my $this = shift;
    my ( $calendar, @week, $w );
    @week = (
        'sunday',   'monday', 'tuesday', 'wednesday',
        'thursday', 'friday', 'saturday'
    );
    $w = 0;
    $calendar .= '<thead>'
        . $this->tr(
        (   (          !$this->{'css'}
                    && defined $this->{'thstyle'}
                    && $this->{'thstyle'} ne ''
            ) ? { 'style' => $this->{'thstyle'} } : {}
        )
        );
    if ( $this->{'css'} ) {
        foreach my $wname ( @{ $this->{'week'} } ) {
            $calendar
                .= '<th class="'
                . $week[$w]
                . '" title="'
                . $this->{'week_title'}[$w] . '">'
                . $wname . '</th>';
            $w++;
        }
    }
    else {
        foreach my $wname ( @{ $this->{'week'} } ) {
            $calendar
                .= '<th class="'
                . $week[$w]
                . '" title="'
                . $this->{'week_title'}[$w] . '"'
                . ( ( $this->{'tdoption'} ) ? $this->{'tdoption'} : '' )
                . ( ( $this->{'thbgcolor'} )
                ? ' bgcolor="' . $this->{'thbgcolor'} . '"'
                : '' )
                . ' nowrap="nowrap">'
                . (
                ( $this->{'thfontoption'} ) ? $this->{'thfontoption'} : '' )
                . $wname
                . (
                ( $this->{'thfontoptionend'} ) ? $this->{'thfontoptionend'}
                : '' )
                . '</th>';
            $w++;
        }
    }
    $calendar .= '</tr></thead>';
    return $calendar;
}

#カレンダー
sub _calendar ($$$;$$$) {
    my ( $this, $year, $mon, $special, $cell, $holiday ) = @_;
    my ( @mdays, $n, $first, $calendar );
    @mdays = ( 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
    $mdays[2]++ if $this->is_leap_year($year);
    $calendar .= $this->tr;

    #空白
    $first = $this->get_wday( $year, $mon, 1 );
    for ( $n = 0; $n < $first; $n++ ) {
        $calendar .= $this->tr if $this->{'col'} >= 7;
        $calendar .= $this->cell;
        $calendar .= '</tr>'   if $this->{'col'} >= 7;
    }

    #日にち
    for ( $n = 1; $n <= $mdays[$mon]; $n++ ) {
        $calendar .= $this->tr if $this->{'col'} >= 7;
        $calendar
            .= $this->cell( $n, $mon, $year, $special, $cell, $holiday );
        $calendar .= '</tr>' if $this->{'col'} >= 7;
    }

    #空白
    for ( $n = $this->{'col'}; $n < 7; $n++ ) {
        $calendar .= $this->tr if $this->{'col'} >= 7;
        $calendar .= $this->cell;
        $calendar .= '</tr>'   if $this->{'col'} >= 7;
    }
    return $calendar;
}

#	(1)-2 CSSを取得する
sub mk_css ($) {
    my $this = shift;
    my ( $css, $space, $border );
    return '' if !$this->{'css'};

    #線
    if ( defined $this->{'border'} && $this->{'border'} > 0 ) {
        $border
            = '  border: '
            . $this->{'border'}
            . 'px solid'
            . (
            ( $this->{'bordercolor'} ) ? ' ' . $this->{'bordercolor'} : '' )
            . ';' . "\x0A";
    }
    else {
        $border = '  border: none;' . "\x0A";
    }

    #テーブル全体
    $css .= 'table.calendar {' . "\x0A";
    $css .= '  margin-bottom: 20px;' . "\x0A";
    $css .= '  empty-cells: show;' . "\x0A";
    if ( defined $this->{'cellspacing'} && $this->{'cellspacing'} > 0 ) {
        $css .= '  border-collapse: separate;' . "\x0A";
        $space
            = ( ( $this->{'border'} )      ? $this->{'border'}      : 0 ) * 2
            + ( ( $this->{'cellpadding'} ) ? $this->{'cellpadding'} : 0 ) * 2
            + $this->{'cellspacing'};
        $css .= '  border-spacing: ' . $this->{'cellspacing'} . 'px;' . "\x0A"
            if defined $this->{'cellspacing'} && $this->{'cellspacing'} ne '';
    }
    else {
        $css .= '  border-collapse: collapse;' . "\x0A";
        $space = ( ( $this->{'border'} ) ? $this->{'border'} : 0 )
            + ( ( $this->{'cellpadding'} ) ? $this->{'cellpadding'} : 0 ) * 2;
    }
    $css
        .= '  width: '
        . ( ( $this->{'border'} )
        ? $this->{'width'} - ( $this->{'border'} * 2 )
        : $this->{'width'} )
        . 'px;' . "\x0A"
        if defined $this->{'width'} && $this->{'width'} ne '';
    $css
        .= '  height: '
        . ( ( $this->{'border'} )
        ? $this->{'height'} - ( $this->{'border'} * 2 )
        : $this->{'height'} )
        . 'px;' . "\x0A"
        if defined $this->{'height'} && $this->{'height'} ne '';
    $css .= $border;
    $css
        .= '  background-image: url("'
        . $this->{'background'} . '");' . "\x0A"
        if defined $this->{'background'} && $this->{'background'} ne '';
    $css .= '}' . "\x0A";

    #標準
    $css .= 'table.calendar th, table.calendar td {' . "\x0A";
    $css .= '  text-align: ' . $this->{'cellalign'} . ';' . "\x0A"
        if defined $this->{'cellalign'} && $this->{'cellalign'} ne '';
    $css .= '  vertical-align: ' . $this->{'cellvalign'} . ';' . "\x0A"
        if defined $this->{'cellvalign'} && $this->{'cellvalign'} ne '';
    $css
        .= '  padding: '
        . ( ( $this->{'cellpadding'} ) ? $this->{'cellpadding'} . 'px' : '0' )
        . ";\x0A";
    $css .= '  width: ' . ( $this->{'cellwidth'} - $space ) . 'px;' . "\x0A"
        if defined $this->{'cellwidth'} && $this->{'cellwidth'} ne '';
    $css .= '  height: ' . ( $this->{'cellheight'} - $space ) . 'px;' . "\x0A"
        if defined $this->{'cellheight'} && $this->{'cellheight'} ne '';
    $css .= $border;
    $css .= $this->mk_style;
    $css .= '}' . "\x0A";

    #月名
    $css .= 'table.calendar caption {' . "\x0A";
    $css .= $this->mk_style('header');
    $css .= '}' . "\x0A";

    #週名
    $css .= 'table.calendar th {' . "\x0A";
    $css .= '  white-space: nowrap;' . "\x0A";
    $css .= $this->mk_style('th');
    $css .= '}' . "\x0A";

    #リンク
    $css .= 'table.calendar td a {' . "\x0A";
    $css .= '  text-decoration: none;' . "\x0A";
    $css .= '  display: block;' . "\x0A";
    $css .= $this->mk_style('link');
    $css .= '}' . "\x0A";

    #特別な日の背景色
    $css .= $this->mk_bgcolor('saturday');
    $css .= $this->mk_bgcolor('sunday');
    $css .= $this->mk_bgcolor('holiday');
    $css .= $this->mk_bgcolor('specialwday');
    $css .= $this->mk_bgcolor('specialday');

    return $css;
}

sub mk_bgcolor ($$) {
    my ( $this, $type ) = @_;
    return '' if !$this->{ $type . 'bgcolor' };
    my $css = 'table.calendar td.' . $type . ' {' . "\x0A";
    $css .= '  background-color: '
        . $this->{ $type . 'bgcolor' } . ';' . "\x0A";
    $css .= '}' . "\x0A";
    return $css;
}

sub mk_style ($;$) {
    my ( $this, $type ) = @_;
    $type = '' if !defined $type;
    my $css = $this->font_size( $this->{ $type . 'fontsize' } );
    $css
        .= '  font-weight: '
        . ( ( $this->{ $type . 'bold' } ) ? 'bold' : 'normal' ) . ';'
        . "\x0A";
    $css .= '  color: ' . $this->{ $type . 'fontcolor' } . ';' . "\x0A"
        if defined $this->{ $type . 'fontcolor' }
        && $this->{ $type . 'fontcolor' } ne '';
    $css
        .= '  background-color: '
        . $this->{ $type . 'bgcolor' } . ';' . "\x0A"
        if defined $this->{ $type . 'bgcolor' }
        && $this->{ $type . 'bgcolor' } ne '';
    $css .= $this->format_for_css( $this->{ $type . 'style' } );
    return $css;
}

sub format_for_css ($$) {
    my ( $this, $css ) = @_;
    return '' if !defined $css || !$css;
    $css =~ s/^[\x09\x0A\x0C\x0D\x20]*/  /;
    $css =~ s/;[\x09\x0A\x0C\x0D\x20]*/;\x0A  /sg;
    $css =~ s/[\x09\x0A\x0C\x0D\x20]*$/\x0A/s;
    return $css;
}

sub font_size ($$) {
    my ( $this, $size ) = @_;
    if ( !$size ) {
        return '';
    }
    elsif ( $size == 1 ) {
        return '  font-size: 0.67em;' . "\x0A";
    }
    elsif ( $size == 2 ) {
        return '  font-size: 0.83em;' . "\x0A";
    }
    elsif ( $size == 3 ) {
        return '  font-size: 1em;' . "\x0A";
    }
    elsif ( $size == 4 ) {
        return '  font-size: 1.17em;' . "\x0A";
    }
    elsif ( $size == 5 ) {
        return '  font-size: 1.5em;' . "\x0A";
    }
    elsif ( $size == 6 ) {
        return '  font-size: 2em;' . "\x0A";
    }
    elsif ( $size == 7 ) {
        return '  font-size: 3em;' . "\x0A";
    }
    else {
        return '';
    }
}

######################################################################
#	(2) カレンダーの文字にリンクを埋め込む
sub set_link ($$$;$$$) {
    my ( $this, $calendar, $id, $url, $target, $comment ) = @_;
    my ( $m, $d )
        = ( $id =~ s/^([0-9]{6})([0-9]{2}).*$/$1$2/s )
        ? ( $1 + 0, $2 + 0 )
        : ( '', '' );
    $$calendar
        =~ s/(<!--calendar(?![0-9a-zA-Z_])([^>]*)-->.*?<!--\/calendar-->)/$this->_set_link($1,$2,$id,$m,$d,$url,$target,$comment)/sge;
    return $this;
}

sub _set_link ($$$$$$;$$$) {
    my ( $this, $calendar, $setting, $id, $m, $d, $url, $target, $comment )
        = @_;
    return $calendar
        if $setting
        !~ /[\x09\x0A\x0C\x0D\x20]+id="([0-9]{6})"(?![0-9a-zA-Z_])/
        || $1 != $m;
    $calendar
        =~ s/(<td(?![0-9a-zA-Z_])([^>]*)>)(.*?)(<\/td>)/$1 . $this->link_replace($2,$3,$id,$d,$url,$target,$comment) . $4/sge;
    return $calendar;
}

sub link_replace ($$$$$;$$$) {
    my ( $this, $setting, $cell, $id, $d, $url, $target, $comment ) = @_;
    return $cell
        if $setting
        !~ /[\x09\x0A\x0C\x0D\x20]+id="d([0-9]{8})"(?![0-9a-zA-Z_])/
        || $1 != $id;

    $cell =~ s/^<a(?![0-9a-zA-Z_])[^>]+>(.*?)<\/a>/$1/s;
    $cell =~ s/^<font(?![0-9a-zA-Z_])[^>]+>(.*?)<\/font>/$1/s;
    $cell =~ s/^<strong>(.*?)<\/strong>/$1/s;

    if ( defined $url && $url ne '' ) {
        my $href = '<a href="' . $url . '"';
        $href .= ' target="' . $target . '"' if defined $target  && $target;
        $href .= ' title="' . $comment . '"' if defined $comment && $comment;
        $href .= ' style="' . $this->{'linkstyle'} . '"'
            if !$this->{'css'} && $this->{'linkstyle'};
        $href .= '>';

        if ( $this->{'css'} ) {
            $cell =~ s/^$d/$href$d<\/a>/s;
        }
        else {
            $cell
                =~ s/^$d/$href$this->{'linkfontoption'}$d$this->{'linkfontoptionend'}<\/a>/s;
        }

        #削除
    }
    elsif ( !$this->{'css'} ) {
        $cell =~ s/^$d/$this->{'fontoption'}$d$this->{'fontoptionend'}/s;
    }
    return $cell;
}

######################################################################
sub tr ($;$) {
    my ( $this, $option ) = @_;
    $this->{'col'} = 0;
    return ( $this->{'css'} )
        ? '<tr>'
        : '<tr'
        . ( ( defined $this->{'troption'} ) ? $this->{'troption'} : '' )
        . ( ( defined $option ) ? $option : '' ) . '>';
}

sub cell ($;$$$$$) {
    my ( $this, $date, $mon, $year, $special, $cell, $holiday ) = @_;
    my ( $bgcolor, $title, $comment, $date_id, $class, $style );

    #曜日色
    if ( !$this->{'col'} ) {
        $bgcolor = $this->{'sundaybgcolor'};
    }
    elsif ( $this->{'col'} == 6 ) {
        $bgcolor = $this->{'saturdaybgcolor'};
    }
    else {
        $bgcolor = $this->{'bgcolor'};
    }
    $class = (
        'sunday',   'monday', 'tuesday', 'wednesday',
        'thursday', 'friday', 'saturday'
    )[ $this->{'col'} ];

    #説明
    if ( defined $date && $date ) {
        $date_id
            = ' id="d'
            . sprintf( "%.4d", $year )
            . sprintf( "%.2d", $mon )
            . sprintf( "%.2d", $date ) . '"';
        if ( ref $holiday && $holiday->is_holiday( $mon, $date ) ) {
            $bgcolor = $this->{'holidaybgcolor'}
                if defined $this->{'holidaybgcolor'}
                && $this->{'holidaybgcolor'};
            $title .= $holiday->name( $mon, $date );
            $class .= ' holiday';
        }
        if (   $year
            && $date
            && $this->is_special_wday( $year, $mon, $date, $this->{'col'} ) )
        {
            $bgcolor = $this->{'specialwdaybgcolor'}
                if defined $this->{'specialwdaybgcolor'}
                && $this->{'specialwdaybgcolor'};
            $class .= ' specialwday';
        }
        if ( ref $special && ref $$special{$mon} && $$special{$mon}{$date} ) {
            my $special_title = $$special{$mon}{$date};
            if ( $special_title =~ s/:(.*?)$// ) {
                $bgcolor = $1;
                $style   = ' style="background-color: ' . $bgcolor . '"';
            }
            elsif ( defined $this->{'specialdaybgcolor'}
                && $this->{'specialdaybgcolor'} )
            {
                $bgcolor = $this->{'specialdaybgcolor'};
            }
            $title .= '/' if defined $title && $title;
            $title .= $special_title;
            $class .= ' specialday';
        }
        $title = ' title="' . $title . '"' if defined $title && $title;
        $comment
            = '<div class="datecomment">' . $$cell{$mon}{$date} . '</div>'
            if ref $cell && ref $$cell{$mon} && $$cell{$mon}{$date};
    }

    $this->{'col'}++;
    if ( $this->{'css'} ) {
        return
              '<td class="'
            . $class . '"'
            . ( ( defined $style )   ? $style   : '' )
            . ( ( defined $date_id ) ? $date_id : '' )
            . ( ( defined $title )   ? $title   : '' ) . '>'
            . ( ( defined $date )    ? $date    : '' )
            . ( ( defined $comment ) ? $comment : '' ) . '</td>';
    }
    else {
        my ( $font, $fontend, $tdoption );
        if ( !defined $date || $date eq '' ) {
            $date = '';
        }
        else {
            $font = ( $this->{'fontoption'} ) ? $this->{'fontoption'} : '';
            $fontend
                = ( $this->{'fontoptionend'} )
                ? $this->{'fontoptionend'}
                : '';
            $tdoption = ' nowrap="nowrap"';
        }
        $tdoption .= ' bgcolor="' . $bgcolor . '"'
            if defined $bgcolor && $bgcolor;
        return
              '<td class="'
            . $class . '"'
            . ( ( defined $this->{'tdoption'} ) ? $this->{'tdoption'} : '' )
            . ( ( defined $tdoption )           ? $tdoption           : '' )
            . ( ( defined $date_id )            ? $date_id            : '' )
            . ( ( defined $title )   ? $title   : '' ) . '>'
            . ( ( defined $font )    ? $font    : '' )
            . ( ( defined $date )    ? $date    : '' )
            . ( ( defined $fontend ) ? $fontend : '' )
            . ( ( defined $comment ) ? $comment : '' ) . '</td>';
    }
}

sub is_special_wday ($$$$;$) {
    my ( $this, $year, $mon, $mday, $wday ) = @_;
    return 0
        if (
        ( !ref $this->{'specialwday'} || !@{ $this->{'specialwday'} } )
        && (   !ref $this->{'specialwdayextra'}
            || !ref $this->{'specialwdayextra'}{$mon} )
        )
        || $this->{'specialwdayexception'}{$mon}{$mday};
    $wday = $this->get_wday( $year, $mon, $mday )
        if !defined $wday || $wday eq '';
    return ( grep( $_ == $wday, @{ $this->{'specialwday'} } )
            || $this->{'specialwdayextra'}{$mon}{$mday} ) ? 1 : 0;
}

sub is_special_wday_exception ($$$) {
    my ( $this, $mon, $mday ) = @_;
    return (   $mon
            && $mday
            && ref $this->{'specialwdayexception'}
            && ref $this->{'specialwdayexception'}{$mon}
            && $this->{'specialwdayexception'}{$mon}{$mday} ) ? 1 : 0;
}

sub date ($$$$) {
    my ( $this, $year, $mon, $mday ) = @_;
    my $wday = $this->get_wday( $year, $mon, $mday );
    my $date = '';
    if ( defined $this->{'yearformat'} && $this->{'yearformat'} =~ /e/ ) {
        $date
            = $this->{'week'}[$wday] . ' '
            . $this->{'month'}[$mon] . ' '
            . $mday . ' ';
    }
    else {
        $date .= $this->{'month'}[$mon];
        $date .= $mday . '日';
        $date .= '(' . $this->{'week'}[$wday] . ')';
        $date = $this->kansuuji( $this->{'yearformat'}, $date );
    }
    return $date;
}

1;
