package CartController;
require CMONOS::CGI_Lib::Base;
@ISA = qw(Base);
use strict;
use utf8;
use Fcntl qw(:flock :DEFAULT);
eval 'use ' . $env::dbm_file . ';';
######################################################################
# カート制御モジュール
# CartController.pm 1.0.7
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-03-31
######################################################################
#
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0) オブジェクトを作る。
#		require CMONOS::CGI_Lib::CartController;
#		obj = CartController->new;
#
# ---------------------------------------------------------------------------------------
#	(1) obj->add(\%new_item,\%item_data,\@item_order);
#		アイテムの追加。
#			new_item		新しいアイテムのハッシュのリファレンス
#			item_data		カートデータを読み込むハッシュのリファレンス
#			item_order		格納順を読み込む配列のリファレンス
#
# ---------------------------------------------------------------------------------------
#	(2) obj->change(\%quantity,\%item_data,\@item_order);
#		アイテム数の変更。
#			quantity		アイテム数のハッシュのリファレンス
#
# ---------------------------------------------------------------------------------------
#	(3) obj->cancel(\%canceled_item,\%item_data,\@item_order);
#		アイテム数の変更。
#			canceled_item	キャンセルするアイテムのリファレンス（スカラー・配列・ハッシュ）
#
# ---------------------------------------------------------------------------------------
#	(4) obj->cancel_all;
#		カートを空にする
#
# ---------------------------------------------------------------------------------------
#	(5) obj->check(\%cart,\%item_data,\@item_order,strictly);
#		アイテムの存在と在庫をチェック
#			cart			カートデータハッシュのリファレンス（空の場合セッションから読み込みます。）
#			strictly		1=データベースのフィールド設定をチェック
#
######################################################################
#オブジェクト作成
sub new ($) {
    my $pkg  = shift;
    my $this = {};
    bless $this;
    $this->fix_env;
    $this->use_session;
    if ( !$session::id && !$session::is_deleted ) {
        $session::id = $this->read_cookie_direct( $this->login_cookie_name );
        if ( !$cgi::Session->is_available($session::id) ) {
            $session::is_deleted = 1;
            undef $session::id;
        }
    }
    $this;
}

######################################################################
#アイテムの追加
sub add ($$;$$$) {
    my ( $this, $new_item, $item_data, $item_order, $strictly ) = @_;
    my $new_item_id = '';
    my %cart        = ();
    $new_item->{'quantity'} = 1 if $new_item->{'digitalcontent'};
    if ( !$new_item->{'caller'} ) {
        push(
            @{ $cgi::action->{'dialog'} },
            '呼びだし元が不明です。'
        ) if ref $cgi::action;
    }
    elsif ( !$new_item->{'quantity'}
        || $this->is_nan( $new_item->{'quantity'} ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '個数が数ではありません。'
        ) if ref $cgi::action;
    }
    elsif ( $new_item->{'quantity'} < 1 ) {
    }
    elsif ( $new_item->{'m'} ) {
        if ( !$this->_get_field_info($new_item) ) {
            push(
                @{ $cgi::action->{'dialog'} },
                'カート呼び出し元フィールド設定が取得できませんでした。'
            ) if ref $cgi::action;
        }
        elsif ( $new_item->{'digitalContentRequired'}
            && !$new_item->{'digitalcontent'} )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                'デジタルコンテンツが存在しません。'
            ) if ref $cgi::action;
        }
        else {
            my $path = '../' . $new_item->{'caller'} . '/start.cgi?';
            my $no
                = ( ( defined $new_item->{'c'} && $new_item->{'c'} ne '' )
                ? '&c=' . $new_item->{'c'}
                : '' )
                . ( ( defined $new_item->{'n'} && $new_item->{'n'} ne '' )
                ? '&n=' . $new_item->{'n'}
                : '' );
            if ( $new_item->{'lib'} && $new_item->{'lib'} eq 'Baobab' ) {
                my $query = $no . '&v=listarticle';
                $query =~ s/^&//;
                $new_item->{'path'} = $path . $query;
            }
            else {
                $new_item->{'path'}    = $path . 'm=DetailViewer' . $no;
                $new_item->{'referer'} = $path . 'm=' . $new_item->{'m'} . $no
                    if $new_item->{'m'} ne 'DetailViewer';
            }
            no warnings qw(numeric);
            $new_item_id
                = $new_item->{'caller'}
                . ( ( defined $new_item->{'c'} && $new_item->{'c'} ne '' )
                ? '_' . $new_item->{'c'}
                : '' )
                . ( ( defined $new_item->{'n'} && $new_item->{'n'} ne '' )
                ? '_' . $new_item->{'n'}
                : '' )
                . (
                ( $new_item->{'option'} )
                ? '--'
                    . (
                    ( ref $new_item->{'option'} )
                    ? join( '_',
                        sort { $a <=> $b || $a cmp $b }
                            @{ $new_item->{'option'} } )
                    : $new_item->{'option'}
                    )
                : ''
                );
        }
    }
    elsif ( $new_item->{'cartfield'} ) {
        my $setting
            = $this->read_file( '../' . $new_item->{'caller'} . '/default.pl',
            1 );
        my $lib = $1
            if $setting
            =~ /\$lib[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'](.*?)["']/;
        my $home = $this->un_base_change( eval $1, 'rel', 1 )
            if $setting
            =~ /\$home[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/;
        if ( $lib && $home ) {
            if ( $lib eq 'HomePageMaker' ) {
                $new_item->{'path'}
                    = $home
                    . ( ( defined $new_item->{'c'} && $new_item->{'c'} ne '' )
                    ? '/' . $new_item->{'c'}
                    : '' )
                    . ( ( defined $new_item->{'n'} && $new_item->{'n'} ne '' )
                    ? '/' . $new_item->{'n'}
                    : '' )
                    . '/'
                    . ( ( $new_item->{'p'} )
                    ? ( $new_item->{'p'} eq 'i' )
                            ? '1'
                            : $new_item->{'p'}
                    : 'index' )
                    . '.'
                    . (
                    ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' );
            }
            else {
                $new_item->{'path'} = $home;
            }
        }
        else {
            $new_item->{'path'}
                = '../../'
                . $new_item->{'caller'}
                . ( ( defined $new_item->{'c'} && $new_item->{'c'} ne '' )
                ? '/' . $new_item->{'c'}
                : '' )
                . ( ( defined $new_item->{'n'} && $new_item->{'n'} ne '' )
                ? '/' . $new_item->{'n'}
                : '' )
                . '/'
                . ( ( $new_item->{'p'} )
                ? ( $new_item->{'p'} eq 'i' )
                        ? '1'
                        : $new_item->{'p'}
                : 'index' )
                . '.'
                . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' );
            $new_item->{'path'}
                .= 'index.'
                . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' )
                if $new_item->{'path'} =~ m!/$!;
        }
        if (   !$new_item->{'path'}
            || !-f $new_item->{'path'}
            || !-f '../' . $new_item->{'caller'} . '/start.cgi' )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                'カート呼び出し元ページが取得できませんでした。'
            ) if ref $cgi::action;
        }
        else {
            $new_item_id = $this->item_id($new_item);
        }
    }
    else {
        push(
            @{ $cgi::action->{'dialog'} },
            'カート呼び出しデータに不足があります。'
        ) if ref $cgi::action;
    }
    $session::is_changed = 1 if !$session::id;
    $cgi::Session->read_by_uri( $session::id, $env::home_dir_id . 'cart',
        \%cart, 0, $item_order );
    if ($new_item_id) {
        my $old_quantity
            = ( ref $cart{$new_item_id} )
            ? $cart{$new_item_id}{'quantity'}
            : 0;
        $cart{$new_item_id} = $new_item;
        if ( $cart{$new_item_id}{'digitalcontent'} ) {
            $cart{$new_item_id}{'quantity'} = 1;
        }
        elsif ($old_quantity) {
            $cart{$new_item_id}{'quantity'} += $old_quantity;
        }
        $cart{$new_item_id}{'time'} = time;
    }
    $this->check( \%cart, $item_data, $item_order, $strictly );
    push( @$item_order, $new_item_id )
        if $new_item_id
        && ref $cart{$new_item_id}
        && !grep( $_ eq $new_item_id, @$item_order );
    $cgi::Session->save_by_uri( $session::id, $env::home_dir_id . 'cart',
        \%cart );
    return ( $new_item_id && ref $cart{$new_item_id} ) ? 1 : 0;
}

sub item_id ($$) {
    my ( $this, $new_item ) = @_;
    return $new_item->{'caller'}
        . ( ( defined $new_item->{'c'} && $new_item->{'c'} ne '' )
        ? '_' . $new_item->{'c'}
        : '' )
        . ( ( defined $new_item->{'n'} && $new_item->{'n'} ne '' )
        ? '_' . $new_item->{'n'}
        : '' )
        . ( ( $new_item->{'p'} ) ? '_' . $new_item->{'p'} : '' )
        . (
        ( $new_item->{'chapterid'} ) ? '_' . $new_item->{'chapterid'} : '' )
        . (
        ( $new_item->{'sectionid'} ) ? '_' . $new_item->{'sectionid'} : '' )
        . (
        ( $new_item->{'option'} )
        ? '--'
            . (
            ( ref $new_item->{'option'} )
            ? join( '_', sort @{ $new_item->{'option'} } )
            : $new_item->{'option'}
            )
        : ''
        );
}

sub _get_field_info ($$) {
    my ( $this, $item ) = @_;
    my ( $setting, $subdivided, $db, $field_pref_dir, $preference_dir,
        $is_success, $caller_path );
    $is_success  = 0;
    $caller_path = '../' . $item->{'caller'} . '/start.cgi';
    $setting
        = $this->read_file( '../' . $item->{'caller'} . '/default.pl', 1 );
    $item->{'lib'} = $1
        if $setting
        =~ /\$lib[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'](.*?)["']/;
    $db
        = ( $setting
            =~ /\$db[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'](.*?)["']/
        ) ? $this->un_base_change( $1, $caller_path, 'rel', 1 ) : '';
    $subdivided = 1
        if $setting
        =~ /\$subdivided[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*1/;
    $preference_dir = $this->un_base_change( $1, $caller_path, 'rel', 1 )
        if $setting
        =~ /\$preference_dir[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'](.*?)["']/;
    $field_pref_dir = $this->un_base_change( $1, $caller_path, 'rel', 1 )
        if $setting
        =~ /\$field_pref_dir[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'](.*?)["']/;

    if ( $db && $field_pref_dir ) {
        my (%field_setting);
        $field_pref_dir .= '/' . $item->{'c'}
            unless !$subdivided
            || $field_pref_dir
            =~ s!^$preference_dir/!$preference_dir/$item->{'c'}/!;
        $db .= '/' . $item->{'c'}
            if defined $item->{'c'} && $item->{'c'} ne '';
        $item->{'db'} = $db;
        tie( %field_setting, $env::dbm_file, $field_pref_dir . '/inputfield',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %field_setting ) {
            $item->{'keyfield'}
                = ( $field_setting{'for_key'} )
                ? $this->decode_utf8( $field_setting{'for_key'} )
                : '';
            $item->{'namefield'}
                = ( $field_setting{'for_title'} )
                ? $this->decode_utf8( $field_setting{'for_title'} )
                : ( $field_setting{'for_name'} )
                ? $this->decode_utf8( $field_setting{'for_name'} )
                : $item->{'keyfield'};
            if (   $item->{'cartfield'}
                && $field_setting{ $item->{'cartfield'} . '_pkg' }
                && $this->decode_utf8(
                    $field_setting{ $item->{'cartfield'} . '_pkg' }
                ) eq 'ShoppingCart'
                )
            {
                if ($field_setting{ $item->{'cartfield'} . '_stockview' }
                    && $this->decode_utf8(
                        $field_setting{ $item->{'cartfield'} . '_stockview' }
                    ) eq 'sync'
                    )
                {
                    $item->{'outOfStockBounds'}
                        = $this->decode_utf8(
                        $field_setting{ $item->{'cartfield'}
                                . '_outOfStockBounds' } )
                        if defined $field_setting{ $item->{'cartfield'}
                            . '_outOfStockBounds' };
                }
                else {
                    delete $item->{'outOfStockBounds'};
                }
                $item->{'maxQuantity'}
                    = $this->decode_utf8(
                    $field_setting{ $item->{'cartfield'} . '_maxQuantity' } )
                    if defined $field_setting{ $item->{'cartfield'}
                        . '_maxQuantity' };
                if ( $field_setting{ $item->{'cartfield'} . '_stockDisabled' }
                    )
                {
                    $item->{'stockDisabled'} = 1;
                    delete $item->{'outOfStockBounds'};
                }
                else {
                    delete $item->{'stockDisabled'};
                }
                if ($field_setting{ $item->{'cartfield'}
                            . '_digitalContentRequired' } )
                {
                    $item->{'digitalContentRequired'} = 1;
                    delete $item->{'outOfStockBounds'};
                }
                else {
                    delete $item->{'digitalContentRequired'};
                }
                if ($field_setting{ $item->{'cartfield'} . '_optionRequired' }
                    )
                {
                    $item->{'optionRequired'} = 1;
                }
                else {
                    delete $item->{'optionRequired'};
                }
                if ( $field_setting{ $item->{'cartfield'} . '_periodfield' } )
                {
                    $item->{'period_field'}
                        = $this->decode_utf8(
                        $field_setting{ $item->{'cartfield'}
                                . '_periodfield' } );
                }
                else {
                    delete $item->{'period_field'};
                }
                if ( $item->{'lib'} && $item->{'lib'} eq 'Calendar' ) {
                    if ( $field_setting{ $item->{'cartfield'} . '_periodEnd' }
                        )
                    {
                        $item->{'period_end'}
                            = $this->decode_utf8(
                            $field_setting{ $item->{'cartfield'}
                                    . '_periodEnd' } );
                    }
                    else {
                        delete $item->{'period_end'};
                    }
                    if ($field_setting{ $item->{'cartfield'}
                                . '_periodEndUnit' } )
                    {
                        $item->{'period_end_unit'}
                            = $this->decode_utf8(
                            $field_setting{ $item->{'cartfield'}
                                    . '_periodEndUnit' } );
                    }
                    else {
                        delete $item->{'period_end_unit'};
                    }
                    if ($field_setting{ $item->{'cartfield'}
                                . '_noPeriodForDC' } )
                    {
                        $item->{'no_period_for_dc'}
                            = $this->decode_utf8(
                            $field_setting{ $item->{'cartfield'}
                                    . '_noPeriodForDC' } );
                    }
                    else {
                        delete $item->{'no_period_for_dc'};
                    }
                }
                $is_success = 1;
            }
        }
        untie(%field_setting);
    }
    if ( $item->{'digitalcontent'} ) {
        if (   $item->{'digitalcontent'}
            && $item->{'digitalcontent'} eq '1'
            && $db
            && -d $db )
        {
            $item->{'digitalcontentpath'} = $db . '/digital-content';
        }
        elsif ($item->{'digitalcontent'}
            && $item->{'digitalcontent'} eq '2'
            && $setting
            =~ /\$file[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*["'](.*?)["']/
            )
        {
            my $files = $this->un_base_change( $1, $caller_path, 'rel', 1 );
            $item->{'digitalcontentpath'}
                = $files
                . ( ( defined $item->{'c'} && $item->{'c'} ne '' )
                ? '/' . $item->{'c'}
                : '' )
                . ( ( defined $item->{'n'} && $item->{'n'} ne '' )
                ? '/' . $item->{'n'}
                : '' )
                if $files && -d $files;
        }
    }
    return $is_success;
}

sub read_file ($$) {
    my ( $this, $path ) = @_;
    my ($data);
    local (*FILE);
    if ( open( FILE, $path ) ) {
        binmode FILE, ":utf8";
        flock( FILE, LOCK_SH );
        $data = join( '', <FILE> );
        close(FILE);
    }
    return ( defined $data ) ? $data : '';
}

######################################################################
#アイテム数の変更
sub change ($$;$$$) {
    my ( $this, $quantity, $item_data, $item_order, $strictly ) = @_;
    return 0 if !$session::id;
    my %cart = ();
    $cgi::Session->read_by_uri( $session::id, $env::home_dir_id . 'cart',
        \%cart, 0, $item_order );
    while ( my ( $id, $q ) = each %{$quantity} ) {
        if ( ref $cart{$id} ) {
            if ( !$q ) {
                $cart{$id}{'quantity'} = 0;
            }
            elsif ( $q && !$this->is_nan($q) ) {
                $cart{$id}{'quantity'} = $q;
            }
            $cart{$id}{'time'} = time;
        }
    }
    $this->check( \%cart, $item_data, $item_order, $strictly );
    $cgi::Session->save_by_uri( $session::id, $env::home_dir_id . 'cart',
        \%cart );
    return 1;
}

######################################################################
#アイテムのキャンセル
sub cancel ($$;$$$) {
    my ( $this, $canceled_item, $item_data, $item_order, $strictly ) = @_;
    return 0 if !$session::id;
    my %quantity = ();
    if ( $canceled_item && !ref $canceled_item ) {
        $quantity{$canceled_item} = 0;
    }
    elsif ( ref $canceled_item eq 'ARRAY' ) {
        foreach my $id (@$canceled_item) {
            $quantity{$id} = 0;
        }
    }
    elsif ( ref $canceled_item eq 'HASH' ) {
        %quantity = %$canceled_item;
    }
    return $this->change( \%quantity, $item_data, $item_order, $strictly );
}

######################################################################
#全アイテムのキャンセル
sub cancel_all ($) {
    my $this = shift;
    return 0 if !$session::id;
    return $cgi::Session->reset_by_uri(
        $session::id,
        $env::home_dir_id . 'cart',
        $env::home_dir_id . 'checkout'
    );
}

sub get_url_of_last_item ($) {
    my $this = shift;
    return '' if !$session::id;
    my ( %cart, @order );
    $cgi::Session->read_by_uri( $session::id, $env::home_dir_id . 'cart',
        \%cart, 0, \@order );
    return (   %cart
            && @order
            && ref $cart{ $order[$#order] }
            && $cart{ $order[$#order] }{'path'} )
        ? $cart{ $order[$#order] }{'path'}
        : '';
}

######################################################################
#アイテムのチェック
sub check ($;$$$$$) {
    my ( $this, $cart, $item_data, $item_order, $strictly,
        $digital_content_array )
        = @_;
    my %temp = ();
    my (@outofstock_items, @invalid_items, @cancelled_items,
        %total_quantity,   $changed
    );
    @outofstock_items = @invalid_items = @cancelled_items = ();
    if ( !ref $cart ) {
        $cart = \%temp;
        $cgi::Session->read_by_uri( $session::id, $env::home_dir_id . 'cart',
            $cart, 0, $item_order );
    }
    no warnings qw(numeric);
    require CMONOS::CGI_Lib::Template;
    foreach my $id (
        sort { $cart->{$a}{'time'} <=> $cart->{$b}{'time'} || $a cmp $b }
        keys %$cart )
    {
        my $item = $cart->{$id};
        if ( !$item->{'quantity'} ) {
            push( @cancelled_items, $id );
            next;
        }
        elsif ( $item->{'m'} ) {
            if ( $item->{'db'} ) {
                my ($stock_id, %db,  %temp_data, $data,
                    $pending,  $cal, $sellout,   $name
                );
                $stock_id = $id;
                $stock_id =~ s/--[0-9_]+$//;
                $total_quantity{$stock_id} = 0 if !$total_quantity{$stock_id};
                tie( %db, $env::dbm_file, $item->{'db'} . '/pending',
                    O_RDONLY, $env::dbm_mode );
                $pending
                    = $this->decode_utf8(
                    $db{ $this->encode_utf8( $item->{'n'} ) } )
                    if ref tied %db
                    && $db{ $this->encode_utf8( $item->{'n'} ) };
                untie(%db);
                if ($pending) {
                    my ( $is_pending, $start, $end )
                        = split( /\-/, $pending );
                    my $time = time;
                    if (   ( $is_pending && !$start && !$end )
                        || ( $start && $time < $start )
                        || ( $end   && $end < $time ) )
                    {
                        push( @outofstock_items, $id );
                        next;
                    }
                }
                tie( %db, $env::dbm_file, $item->{'db'} . '/showcase',
                    O_RDONLY, $env::dbm_mode );
                $data
                    = $this->decode_utf8(
                    $db{ $this->encode_utf8( $item->{'n'} ) } )
                    if ref tied %db;
                untie(%db);
                if (( $strictly && !$this->_get_field_info($item) )
                    || !$this->_is_available(
                        $data, $item,
                        (   ( ref $item_data ) ? \%{ $item_data->{$id} }
                            : \%temp_data
                        ),
                        \$total_quantity{$stock_id},
                        \$sellout
                    )
                    || !$this->_has_available_item(
                        (   ( ref $item_data ) ? \%{ $item_data->{$id} }
                            : \%temp_data
                        ),
                        $item,
                        $digital_content_array
                    )
                    )
                {
                    push( @outofstock_items, $id );
                    next;
                }
                undef $data;
                if ( $item->{'optionRequired'}
                    && ( !defined $item->{'option'}
                        || $item->{'option'} eq '' ) )
                {
                    push( @invalid_items, $id );
                    next;
                }
                else {
                    my $name;
                    if ( ref $item_data || ( ref $cgi::action && $sellout ) )
                    {
                        if ( $item->{'namefield'} ) {
                            if (   $item->{'keyfield'}
                                && $item->{'keyfield'} eq
                                $item->{'namefield'} )
                            {
                                $name = $item_data->{$id}->{'key'};
                            }
                            else {
                                tie(%db,
                                    $env::dbm_file,
                                    $item->{'db'} . '/'
                                        . $item->{'namefield'},
                                    O_RDONLY,
                                    $env::dbm_mode
                                );
                                $name = $this->decode_utf8(
                                    $db{$this->encode_utf8( $item->{'n'} )
                                    }
                                ) if ref tied %db;
                                untie(%db);
                                $name =~ s/<>([#0-9a-zA-z]{3,10})\Z//s
                                    if $name;
                            }
                        }
                    }
                    if ( ref $item_data ) {
                        $item_data->{$id}->{'name'} = $name if defined $name;
                        if ( $item->{'keyfield'} ) {
                            tie(%db,
                                $env::dbm_file,
                                $item->{'db'} . '/' . $item->{'keyfield'},
                                O_RDONLY,
                                $env::dbm_mode
                            );
                            $item_data->{$id}->{'key'}
                                = $this->decode_utf8(
                                $db{ $this->encode_utf8( $item->{'n'} ) } )
                                if ref tied %db;
                            untie(%db);
                            $item_data->{$id}->{'key'}
                                =~ s/<>([#0-9a-zA-z]{3,10})\Z//s
                                if $item_data->{$id}->{'key'};
                        }
                        if ( $item->{'lib'} && $item->{'lib'} eq 'Calendar' )
                        {
                            my ( %dt, $data );
                            tie( %dt, $env::dbm_file, $item->{'db'} . '/dt',
                                O_RDONLY, $env::dbm_mode );
                            $data
                                = $this->decode_utf8(
                                $dt{ $this->encode_utf8( $item->{'n'} ) } )
                                if ref tied %dt && $dt{ $item->{'n'} };
                            untie %dt;
                            if ($data) {
                                if ( !ref $cal ) {
                                    require CMONOS::Calendar::CalCommon;
                                    $cal = CalCommon->new_date;
                                }
                                $item_data->{$id}->{'name'} .= ' ['
                                    . $cal->get_event_date_info_from_data(
                                    $data, 0, 1 )
                                    . ']';
                            }
                        }
                        $item_data->{$id}->{'db'}       = $item->{'db'};
                        $item_data->{$id}->{'n'}        = $item->{'n'};
                        $item_data->{$id}->{'path'}     = $item->{'path'};
                        $item_data->{$id}->{'referer'}  = $item->{'referer'};
                        $item_data->{$id}->{'quantity'} = $item->{'quantity'};
                        $item_data->{$id}->{'max'}      = $item->{'max'};
                        $item_data->{$id}->{'digitalcontentpath'}
                            = $item->{'digitalcontentpath'};
                        $item_data->{$id}->{'time'} = $item->{'time'};
                    }
                    push(
                        @{ $cgi::action->{'dialog'} },
                        ( ($name) ? $name . ' は、' : '' )
                            . (
                            ( $sellout == 2 )
                            ? '在庫切れのため数が制限されています。'
                            : '一度に注文できる数が制限されています。'
                            )
                    ) if ref $cgi::action && $sellout;
                    $changed = 1 if $sellout;
                }
            }
        }
        else {
            my %data = ();
            my $this_template = Template->new( '', '', '' );
            if ( $item->{'p'} && $item->{'p'} eq 'i' ) {
                $this_template->set(
                    'chapter'        => '',
                    'section'        => '',
                    'field_tag_name' => 'info'
                );
            }
            else {
                $this_template->set(
                    'chapter'        => 'chapter',
                    'section'        => 'section',
                    'field_tag_name' => 'field'
                );
            }
            my $article = $this->read_file( $item->{'path'}, 1 );
            my $chap_id
                = ( ( defined $item->{'chapterid'} )
                ? $item->{'chapterid'}
                : '' );
            my $sect_id
                = ( ( defined $item->{'sectionid'} )
                ? $item->{'sectionid'}
                : '' );
            $this_template->scan( \$article, \$chap_id, \$sect_id, \%data );

#存在のチェック/在庫のチェック→FALSE→%cartから削除 & item_dataに読み込まない
            $item->{'digitalcontentpath'}
                = $this->un_base_change(
                $data{ $item->{'cartfield'} }{'folder'},
                $item->{'path'} )
                if $data{ $item->{'cartfield'} }{'folder'};
            $item->{'optionRequired'} = 1
                if $data{ $item->{'cartfield'} }{'optionRequired'}
                && defined $data{ $item->{'cartfield'} }{'option'}
                && $data{ $item->{'cartfield'} }{'option'} ne '';
            if (   defined $data{ $item->{'cartfield'} }{'price'}
                && !$data{ $item->{'cartfield'} }{'unavailable'}
                && (   !$item->{'periodfield'}
                    || !ref $data{ $item->{'periodfield'} }
                    || $this->_is_available_time(
                        $data{ $item->{'periodfield'} }{'str'}
                    )
                )
                && $this->_has_available_item(
                    $data{ $item->{'cartfield'} }, $item,
                    $digital_content_array
                )
                )
            {
                if ( $item->{'optionRequired'}
                    && ( !defined $item->{'option'}
                        || $item->{'option'} eq '' ) )
                {
                    push( @invalid_items, $id );
                }
                elsif ( ref $item_data ) {
                    $item_data->{$id}->{'key'}
                        = ( $item->{'keyfield'}
                            && defined $data{ $item->{'keyfield'} }{'str'} )
                        ? $data{ $item->{'keyfield'} }{'str'}
                        : '';
                    $item_data->{$id}->{'name'}
                        = ( $item->{'namefield'}
                            && defined $data{ $item->{'namefield'} }{'str'} )
                        ? $data{ $item->{'namefield'} }{'str'}
                        : '';
                    $item_data->{$id}->{'anchor'}
                        = ( $item->{'p'} && $item->{'p'} eq 'i' ) ? ''
                        : '#a'
                        . ( ( defined $item->{'p'} && $item->{'p'} ne '' )
                        ? sprintf( "%.2d", $item->{'p'} )
                        : ( $item->{'path'} =~ m!/1\.s?html?$!i ) ? '01'
                        :                                           '' )
                        . sprintf( "%.2d", $item->{'chapterid'} )
                        . sprintf( "%.2d", $item->{'sectionid'} );
                    $item_data->{$id}->{'path'}     = $item->{'path'};
                    $item_data->{$id}->{'quantity'} = $item->{'quantity'};
                    $item_data->{$id}->{'max'}      = $item->{'maxQuantity'}
                        if $item->{'maxQuantity'};
                    $item_data->{$id}->{'digitalcontentpath'}
                        = $item->{'digitalcontentpath'};
                    $item_data->{$id}->{'time'} = $item->{'time'};

                    while ( my ( $k, $v )
                        = each %{ $data{ $item->{'cartfield'} } } )
                    {
                        $item_data->{$id}->{$k} = $v
                            if $k ne 'option' && $k ne 'multiple';
                    }
                    (   $item_data->{$id}->{'option'},
                        $item_data->{$id}->{'optionprice'}
                        )
                        = $this->_option(
                        $item->{'option'},
                        $data{ $item->{'cartfield'} }{'option'},
                        $data{ $item->{'cartfield'} }{'multiple'}
                        )
                        if $item->{'option'}
                        && $data{ $item->{'cartfield'} }{'option'};
                }
            }
            else {
                push( @outofstock_items, $id );
            }
        }
    }
    if ( @outofstock_items || @invalid_items || @cancelled_items || $changed )
    {
        foreach my $id ( @outofstock_items, @invalid_items, @cancelled_items )
        {
            undef %{ $cart->{$id} };
            $cart->{$id} = "\x7F";
            @$item_order = grep( $_ ne $id, @$item_order )
                if ref $item_order eq 'ARRAY';
        }
        $cgi::Session->save_by_uri( $session::id, $env::home_dir_id . 'cart',
            \%temp )
            if %temp;
    }
    if ( @outofstock_items || @invalid_items || $changed ) {
        push(
            @{ $cgi::action->{'dialog'} },
            '取り扱いが中止されたか、在庫切れのため'
                . scalar(@outofstock_items)
                . '件のご注文をキャンセルしました。'
        ) if ref $cgi::action && @outofstock_items;
        push(
            @{ $cgi::action->{'dialog'} },
            'オプションが指定されていないため'
                . scalar(@invalid_items)
                . '件のご注文をキャンセルしました。'
        ) if ref $cgi::action && @invalid_items;
        return 0;
    }
    return 1;
}

sub _has_available_item ($$$;$) {
    my ( $this, $data, $item, $digital_content_array ) = @_;
    return (   $item->{'digitalContentRequired'}
            || $data->{'digitalContentRequired'} ) ? 0 : 1
        if !$item->{'digitalcontent'};
    $item->{'quantity'} = 1;
    if (   $data->{'digitalcontent'}
        && $data->{'foldertype'}
        && $data->{'foldertype'} eq $item->{'digitalcontent'}
        && $item->{'digitalcontentpath'}
        && !$data->{'withdrawn'} )
    {
        my $dc
            = $item->{'digitalcontentpath'} . '/'
            . ( ( $data->{'foldertype'} eq '2' )
            ? $item->{'cartfield'} . '.' . $data->{'digitalcontent'} . '.dat'
            : $data->{'digitalcontent'} );
        if ( -f $dc ) {
            push( @$digital_content_array, $dc )
                if ref $digital_content_array eq 'ARRAY';
            return 1;
        }
    }
    push(
        @{ $cgi::action->{'dialog'} },
        'デジタルコンテンツが存在しません。'
    ) if ref $cgi::action;
    return 0;
}

sub _is_available ($$$$$$) {
    no warnings qw(numeric);
    my ( $this, $str, $item, $item_data, $total_ref, $sellout_ref ) = @_;
    my ( %data, $option, $multiple );
    return 0 if !defined $str;
    $data{'price'} = $data{'condition'} = $data{'availability'}
        = $data{'stock'}     = $data{'papersize'}    = $data{'thickness'}
        = $data{'weight'}    = $data{'multiple'}     = $data{'option'}
        = $data{'withdrawn'} = $data{'discountable'} = $data{'limited'} = '';
    $data{'price'} = $1 if $str =~ s/^([0-9]*)\x0A//;
    while ( $str =~ /^([0-9a-zA-Z_]+)\:(.*)$/gm ) {
        my ( $k, $v ) = ( $1, $2 );
        chomp $v;
        $v =~ s/&#10;/\x0A/sg;
        if ( $k eq 'option' ) {
            $option = $v;
        }
        elsif ( $k eq 'multiple' ) {
            $multiple = $v;
        }
        else {
            $data{$k} = $v;
        }
    }
    delete $item->{'optionRequired'} if !defined $option || $option eq '';
    if (   !$item->{'stockDisabled'}
        && defined $data{'stock'}
        && $data{'stock'} ne '' )
    {
        $item->{'outOfStockBounds'} = 0 if !$item->{'outOfStockBounds'};
        if ( $data{'stock'} <= $item->{'outOfStockBounds'} ) {
            $item->{'max'} = 0;
            return 0;
        }
        $$sellout_ref = 0;
        $item->{'max'}
            = $data{'stock'} - $item->{'outOfStockBounds'} - $$total_ref;
        $item->{'max'} = 0 if $item->{'max'} < 0;
        if ( $item->{'max'} < $item->{'quantity'} ) {
            $item->{'quantity'} = $item->{'max'};
            $$sellout_ref = 2;
        }
        if ( $item->{'maxQuantity'} ) {
            if ( $item->{'maxQuantity'} < $item->{'quantity'} ) {
                $item->{'quantity'} = $item->{'max'} = $item->{'maxQuantity'};
                $$sellout_ref = 1;
            }
            elsif ( $item->{'maxQuantity'} < $item->{'max'} ) {
                $item->{'max'} = $item->{'maxQuantity'};
            }
        }
        $$total_ref += $item->{'quantity'};
        return 0 if !$item->{'max'};
    }
    elsif (
        !defined $item->{'outOfStockBounds'}
        && (   ( $item->{'stockDisabled'} && $data{'withdrawn'} )
            || ( !$item->{'stockDisabled'} && $data{'unavailable'} ) )
        )
    {
        return 0;
    }
    if ( $item->{'period_field'} ) {
        my ( %period, $period );
        tie( %period, $env::dbm_file,
            $item->{'db'} . '/' . $item->{'period_field'},
            O_RDONLY, $env::dbm_mode );
        $period
            = $this->decode_utf8(
            $period{ $this->encode_utf8( $item->{'n'} ) } )
            if ref tied %period && $period{ $item->{'n'} };
        untie %period;
        return 0 if $period && !$this->_is_available_time($period);
    }
    if (   $item->{'lib'}
        && $item->{'lib'} eq 'Calendar'
        && defined $item->{'period_end'}
        && $item->{'period_end'} ne ''
        && (   !$data{'digitalcontent'}
            || !$data{'foldertype'}
            || $data{'withdrawn'}
            || !$item->{'no_period_for_dc'} )
        )
    {
        my ( %dt, $dt );
        tie( %dt, $env::dbm_file, $item->{'db'} . '/dt',
            O_RDONLY, $env::dbm_mode );
        $dt = $this->decode_utf8( $dt{ $this->encode_utf8( $item->{'n'} ) } )
            if ref tied %dt && $dt{ $item->{'n'} };
        untie %dt;
        use Time::Local;
        if (   $dt
            && $dt
            =~ /^([0-9]{0,4})([0-9]{0,2})([0-9]{0,2})T?([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})/
            )
        {
            my $start = ($1)
                ? timelocal(
                ( $6 || 0 ),
                ( $5 || 0 ),
                ( $4 || 0 ),
                ( $3 || 1 ),
                ( ($2) ? $2 - 1 : 0 ), $1
                )
                : 0;
            my $gap
                = $item->{'period_end'}
                * ( ( $item->{'period_end_unit'} eq 'min' ) ? 60
                : ( $item->{'period_end_unit'} eq 'hour' ) ? 3600
                : ( $item->{'period_end_unit'} eq 'day' )  ? 86400
                : ( $item->{'period_end_unit'} eq 'week' ) ? 604800
                : ( $item->{'period_end_unit'} eq 'year' ) ? 31536000
                :                                            86400 );
            return 0 if time > $start - $gap;
        }
    }
    if ( ref $item_data ) {
        ( $data{'option'}, $data{'optionprice'} )
            = $this->_option( $item->{'option'}, $option, $multiple )
            if $option && defined $item->{'option'};
        if ( !defined $data{'option'} || $data{'option'} eq '' ) {
            delete $data{'option'};
            delete $data{'optionprice'};
            delete $item->{'option'};
        }
        %$item_data = %data;
    }
    return 1;
}

sub _is_available_time ($$$$) {
    no warnings qw(numeric);
    my ( $this, $period, $data, $item ) = @_;
    use Time::Local;
    if (   $period
        && $period
        =~ m!^([0-9]{0,4})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2}) \- ([0-9]{0,4})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})!
        )
    {
        my $start = ($1)
            ? timelocal(
            ( $6 || 0 ),
            ( $5 || 0 ),
            ( $4 || 0 ),
            ( $3 || 1 ),
            ( ($2) ? $2 - 1 : 0 ), $1
            )
            : 0;
        my $end = ($7)
            ? timelocal(
            ( $12 || 0 ),
            ( $11 || 0 ),
            ( $10 || 0 ),
            ( $9  || 1 ),
            ( ($8) ? $8 - 1 : 0 ), $7
            )
            : 0;
        my $now = time;
        if ( $start > $now || ( $end && $end < $now ) ) {
            return 0;
        }
    }
    return 1;
}

sub _option ($$$;$) {
    my ( $this, $selected_option, $option, $multiple ) = @_;
    my ( $option_name, $option_price, $id );
    while ( $option =~ /^(.*)$/mg ) {
        $id++;
        if ((      !ref $selected_option
                && $selected_option
                && $selected_option == $id
            )
            || (   $multiple
                && ref $selected_option
                && grep( $_ == $id, @$selected_option ) )
            )
        {
            my $name = $1;
            my $price = ( $name =~ s/:([\+\-]?[0-9]+)$// ) ? $1 : '';
            next if $name !~ /[^\x09\x0A\x0C\x0D\x20]/;
            $option_price += $price if $price;
            my $mark = ( $price =~ s/^([\+\-])// ) ? $1 : '+';
            $option_name
                .= ( ($option_name) ? ' ' : '' )
                . $name
                . (
                ($price) ? '(' . $mark . $this->digit($price) . '円)' : '' );
            last if !$multiple || !ref $selected_option;
        }
    }
    return ( $option_name, $option_price );
}

sub purchase ($;$$$$) {
    my ( $this, $item_data, $item_order, $strictly, $already_read ) = @_;
    my %temp_data  = ();
    my @temp_order = ();
    $item_data  = \%temp_data  if !ref $item_data;
    $item_order = \@temp_order if !ref $item_order;
    return 0
        if !$already_read
        && !$this->check( '', $item_data, $item_order, $strictly );
    my $id;
    foreach $id (@$item_order) {

        if ( $item_data->{$id}->{'db'} ) {
            my (%db);
            my $n = $this->encode_utf8( $item_data->{$id}->{'n'} );
            tie( %db, $env::dbm_file, $item_data->{$id}->{'db'} . '/showcase',
                O_RDWR, $env::dbm_mode );
            if ( ref tied %db && $db{$n} ) {
                my $data = $this->decode_utf8( $db{$n} );
                $data
                    =~ s/^stock\:([0-9]+)$/'stock:' . ($1 - $item_data->{$id}->{'quantity'})/me;
                $db{$n} = $this->encode_utf8($data);
            }
            untie(%db);
            tie( %db, $env::dbm_file, $item_data->{$id}->{'db'} . '/sales',
                O_RDWR, $env::dbm_mode );
            if ( ref tied %db && $db{$n} ) {
                $db{$n} = $this->encode_utf8(
                    $this->set_sales_data(
                        $db{$n}, $item_data->{$id}->{'quantity'}
                    )
                );
            }
            untie(%db);
        }
    }
    return 1;
}

sub set_sales_data ($$$) {
    my ( $this, $data, $quantity ) = @_;
    return $data if !$quantity;
    my ( $point, $daily, $total, $ctime, $recommend );
    ( $point, $daily, $total, $ctime ) = split( /\t/, $data, 4 );
    $point = 0 if !$point;
    $daily = 0 if !$daily;
    $daily = $total = 0 if !$total;
    $ctime = time if !$ctime;
    $recommend = ( $point == 999999 ) ? 1 : 0;
    $total += $quantity;
    $daily
        = ($total)
        ? int(
        ( ( $total / int( ( ( time - $ctime ) / 86400 ) + 1 ) ) * 1000000 )
        + 0.5 ) / 1000000
        : 0;
    return
          ( ($recommend) ? 999999 : $daily ) . "\x09"
        . $daily . "\x09"
        . $total . "\x09"
        . $ctime;
}

sub restore ($;$$) {
    my ( $this, $order_info, $dl_dir ) = @_;
    my ( $id, $item );
    my %temp = ();
    my $cart = \%temp;
    if ($order_info) {
        if ( $order_info =~ s/^(.*?)\x0A// ) {

            #($total,$tax,$fee,$carriage,$carriage_total,$download_key,$ptime)
            my @info = split( /\x09/, $1 );
            if ( $dl_dir && $info[5] && -d $dl_dir . '/' . $info[5] ) {
                use File::Path;
                rmtree $dl_dir . '/' . $info[5];
            }
        }
        my $id = 1;
        while ( $order_info =~ /^(.*)$/mg ) {
            $cgi::Session->read_data( \%{ $cart->{ $id++ } }, $1 ) if $1;
        }
        undef $order_info;
    }
    else {
        $cgi::Session->read_by_uri( $session::id, $env::home_dir_id . 'cart',
            $cart );
    }
    while ( ( $id, $item ) = each %$cart ) {
        if ( $item->{'db'} ) {
            my (%db);
            my $n = $this->encode_utf8( $item->{'n'} );
            tie( %db, $env::dbm_file, $item->{'db'} . '/showcase',
                O_RDWR, $env::dbm_mode );
            if ( ref tied %db && $db{$n} ) {
                my $data = $this->decode_utf8( $db{$n} );
                $data
                    =~ s/^stock\:([0-9]+)$/'stock:' . ($1 + $item->{'quantity'})/me;
                $db{$n} = $this->encode_utf8($data);
            }
            untie(%db);
            tie( %db, $env::dbm_file, $item->{'db'} . '/sales',
                O_RDWR, $env::dbm_mode );
            if ( ref tied %db && $db{$n} ) {
                $db{$n} = $this->set_sales_data( $db{$n},
                    ( 0 - $item->{'quantity'} ) );
            }
            untie(%db);
        }
    }
    return $this->cancel_all;
}

1;
