package DraggableMap;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（自作地図）
# DraggableMap.pm 1.0.1
# Copyright (C) 2007-2018 CMONOS Co. Ltd.
#
#　support@cmonos.jp
#　http://cmonos.jp/
#  最終更新日：2010-06-14
######################################################################
sub _set ($$) {
    my ( $this, $setting ) = @_;
    unless ( ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    $this->{'canvasWidth'}  = 400 if !$this->{'canvasWidth'};
    $this->{'canvasHeight'} = 300 if !$this->{'canvasHeight'};
    $this->{'markerTop'}    = 100 if !$this->{'markerTop'};
    $this->{'markerLeft'}   = 100 if !$this->{'markerLeft'};
    $this->{'elements'} = 1;    #複数の要素を登録
    return;
}

######################################################################
#	(1)-0 obj->match_enabled;
sub match_enabled ($) {
    my $this = shift;
    return (   !$this->{'required'}
            && defined $main::search{ $this->{'id'} }{'check'}
            && $main::search{ $this->{'id'} }{'check'} ne '' ) ? 1 : 0;
}

#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    return 1
        if $this->{'required'}
        || !defined $main::search{ $this->{'id'} }{'check'}
        || $main::search{ $this->{'id'} }{'check'} eq '';
    my %data = $this->elements($$str);
    if (   ( $main::search{ $this->{'id'} }{'check'} && $data{'enabled'} )
        || ( !$main::search{ $this->{'id'} }{'check'} && !$data{'enabled'} ) )
    {
        return 1;
    }
    else {
        return 0;
    }
}

######################################################################
#	(2) obj->form;
sub form {
    my ( $this, $status, $original, $path ) = @_;
    my ( $name, $form );

    #名前にデータIDを埋め込む
    $name = 'data:' . $this->{'id'} . ':';

    #フィールドネーム+ヘッダ
    $form = $this->field_header;
    $form
        .= '<label><input type="checkbox" name="'
        . $name
        . 'enabled" value="1"'
        . ( ( $main::data{ $this->{'id'} }{'enabled'} )
        ? ' checked="checked"'
        : '' )
        . ' onclick="cmonos.element.item(\'imap-field-'
        . $this->{'id'}
        . '\').disappear(!this.checked)" /> 使用する</label>'
        if !$this->{'required'};
    $form .= '<div id="imap-field-' . $this->{'id'} . '">';
    $form .= $this->get_draggable_map_form( 'imap-' . $this->{'id'},
        $main::data{ $this->{'id'} }, $name );
    $form .= '</div>';
    $form
        .= $this->javascript( 'cmonos.element.item(\'imap-field-'
            . $this->{'id'}
            . '\').disappear(true);' )
        if !$main::data{ $this->{'id'} }{'enabled'} && !$this->{'required'};
    $form .= $this->field_footer;
    return $form;
}

sub get_draggable_map_form ($$$$) {
    my ( $this, $canvas_id, $data_ref, $name ) = @_;
    my @maps = $this->get_all_maps;
    return '地図画像がみつかりません。' if !@maps;
    $data_ref = $this if !ref $data_ref || !%{$data_ref};
    my $javascript = '';
    my $marker_id  = '';
    if ( !$data_ref->{'mapId'} ) {
        $data_ref->{'mapId'}
            = ( $this->{'mapId'} ) ? $this->{'mapId'} : $maps[$#maps];
    }
    $data_ref->{'mapId'} = $maps[$#maps]
        if !grep( $_ eq $data_ref->{'mapId'}, @maps );
    my $form
        = '<input type="hidden" name="'
        . $name
        . 'markerTop" id="'
        . $canvas_id
        . '-marker-top" value="'
        . $this->escape( $data_ref->{'markerTop'} ) . '" />';
    $form
        .= '<input type="hidden" name="'
        . $name
        . 'markerLeft" id="'
        . $canvas_id
        . '-marker-left" value="'
        . $this->escape( $data_ref->{'markerLeft'} ) . '" />';
    $form
        .= '<input type="hidden" name="'
        . $name
        . 'mapTop" id="'
        . $canvas_id
        . '-top" value="'
        . $this->escape( $data_ref->{'mapTop'} ) . '" />';
    $form
        .= '<input type="hidden" name="'
        . $name
        . 'mapLeft" id="'
        . $canvas_id
        . '-left" value="'
        . $this->escape( $data_ref->{'mapLeft'} ) . '" />';

    $form
        .= '<div class="imap-canvas" style="width: '
        . $this->{'canvasWidth'}
        . 'px; height: '
        . $this->{'canvasHeight'} . 'px;">';
    $form
        .= '<div id="'
        . $canvas_id
        . '" class="map-'
        . $this->escape( $data_ref->{'mapId'} )
        . '" style="top: '
        . ( ( $data_ref->{'mapTop'} ) ? $data_ref->{'mapTop'} . 'px' : '0' )
        . '; left: '
        . ( ( $data_ref->{'mapLeft'} ) ? $data_ref->{'mapLeft'} . 'px' : '0' )
        . ';" title="'
        . $this->escape(
        $setting::deco{'map'}{ $data_ref->{'mapId'} }{'alt'} )
        . '">';

#$form .= '<img src="" alt="' . $this->escape($setting::deco{'map'}{$data_ref->{'mapId'}}{'alt'}) . '" />';

    if ( defined $this->{'markerId'} && $this->{'markerId'} ne '' ) {
        $form
            .= '<div id="'
            . $canvas_id
            . '-marker" class="marker-'
            . $this->{'markerId'}
            . '" style="top: '
            . ( ( $data_ref->{'markerTop'} ) ? $data_ref->{'markerTop'} . 'px'
            : ( defined $data_ref->{'markerTop'} ) ? '0'
            :   int( $this->{'canvasHeight'} / 2 ) . 'px' )
            . '; left: '
            . (
              ( $data_ref->{'markerLeft'} ) ? $data_ref->{'markerLeft'} . 'px'
            : ( defined $data_ref->{'markerLeft'} ) ? '0'
            :   int( $this->{'canvasWidth'} / 2 ) . 'px' )
            . ';" title="'
            . $this->escape( $data_ref->{'markerTitle'} ) . '">';
        $form .= '<p>' . $this->escape( $data_ref->{'markerTitle'} ) . '</p>';
        $form .= '</div>';
        $javascript
            = 'cmonos.tool.draggableMap.set(\''
            . $canvas_id . '\',\''
            . $canvas_id
            . '-marker\',\''
            . $canvas_id . '\');';
        $marker_id = ',\'' . $canvas_id . '-marker\'';
    }
    else {
        $javascript
            = 'cmonos.tool.draggableMap.set(\''
            . $canvas_id
            . '\',\'\',\''
            . $canvas_id . '\');';
    }

    $form .= '</div>';
    $form .= '</div>';
    if ( @maps > 1 ) {
        $form
            .= '<select name="'
            . $name
            . 'mapId" onchange="cmonos.tool.draggableMap.setMapClassById(\''
            . $canvas_id
            . '\',this.options[this.selectedIndex].value'
            . $marker_id . ')">';
        foreach my $mid (@maps) {
            $form
                .= '<option value="'
                . $mid . '"'
                . ( ( $data_ref->{'mapId'} eq $mid ) ? ' selected="selected"'
                : '' )
                . '>'
                . (
                (   defined $setting::deco{'map'}{$mid}{'alt'}
                        && $setting::deco{'map'}{$mid}{'alt'} ne ''
                ) ? $this->escape( $setting::deco{'map'}{$mid}{'alt'} )
                : 'map-' . $mid
                ) . '</option>';
        }
        $form .= '</select>';
    }
    else {
        $form
            .= '<input type="hidden" name="'
            . $name
            . 'mapId" value="'
            . $maps[0] . '" />';
    }
    if ( defined $this->{'markerId'} && $this->{'markerId'} ne '' ) {
        $form
            .= '<label><input type="checkbox" name="'
            . $name
            . 'markerEnabled" value="1"'
            . ( ( $data_ref->{'markerEnabled'} ) ? ' checked="checked"' : '' )
            . ' onclick="cmonos.element.item(\''
            . $canvas_id
            . '-marker\').disappear(!this.checked)" /> マーカー</label>';
        $form
            .= ' <input type="text" size="20" name="'
            . $name
            . 'markerTitle" value="'
            . $this->escape( $data_ref->{'markerTitle'} )
            . '" onchange="cmonos.element.item(\''
            . $canvas_id
            . '-marker\').setTitle(this.value)" title="マーカーのタイトル" />';
        $javascript
            .= 'cmonos.element.item(\''
            . $canvas_id
            . '-marker\').disappear(true);'
            if !$data_ref->{'markerEnabled'};
    }
    $form .= $this->javascript($javascript);
    return $form;
}

sub get_all_maps ($) {
    my $this   = shift;
    my @map_id = ();
    local (*DIR);
    if ( opendir( DIR, "$env::image_dir/map" ) ) {
        @map_id
            = sort { $a <=> $b || $a cmp $b }
            grep( -f "$env::image_dir/map/$_"
                && $_ =~ s/^([0-9a-zA-Z_]+)\.(jpg|gif|png)$/$1/i,
            readdir(DIR) );
        closedir(DIR);
    }
    return @map_id;
}

sub get_draggable_map ($$) {
    my ( $this, $canvas_id, $data_ref ) = @_;
    return '地図画像がみつかりません。'
        if !ref $setting::deco{'map'} || !%{ $setting::deco{'map'} };
    return '' if !$data_ref->{'mapId'};
    my ( $javascript, $form );
    $form
        = '<div class="imap-canvas" style="width: '
        . $this->{'canvasWidth'}
        . 'px; height: '
        . $this->{'canvasHeight'} . 'px;">';
    $form
        .= '<div id="'
        . $canvas_id
        . '" class="map-'
        . $this->escape( $data_ref->{'mapId'} )
        . '" style="top: '
        . ( ( $data_ref->{'mapTop'} ) ? $data_ref->{'mapTop'} . 'px' : '0' )
        . '; left: '
        . ( ( $data_ref->{'mapLeft'} ) ? $data_ref->{'mapLeft'} . 'px' : '0' )
        . ';" title="'
        . $this->escape(
        $setting::deco{'map'}{ $data_ref->{'mapId'} }{'alt'} )
        . '">';

#$form .= '<img src="" alt="' . $this->escape($setting::deco{'map'}{$data_ref->{'mapId'}}{'alt'}) . '" />';
    if (   defined $this->{'markerId'}
        && $this->{'markerId'} ne ''
        && $data_ref->{'markerEnabled'} )
    {
        $form
            .= '<div id="'
            . $canvas_id
            . '-marker" class="marker-'
            . $this->{'markerId'}
            . '" style="top: '
            . ( ( $data_ref->{'markerTop'} ) ? $data_ref->{'markerTop'} . 'px'
            : ( defined $data_ref->{'markerTop'} ) ? '0'
            :   int( $this->{'canvasHeight'} / 2 ) . 'px' )
            . '; left: '
            . (
              ( $data_ref->{'markerLeft'} ) ? $data_ref->{'markerLeft'} . 'px'
            : ( defined $data_ref->{'markerLeft'} ) ? '0'
            :   int( $this->{'canvasWidth'} / 2 ) . 'px' )
            . ';" title="'
            . $this->escape( $data_ref->{'markerTitle'} ) . '">';
        $form .= '<p>' . $this->escape( $data_ref->{'markerTitle'} ) . '</p>';
        $form .= '</div>';
    }
    $form .= '</div>';
    $form .= '</div>';
    $form .= $this->javascript(
        'cmonos.tool.draggableMap.set(\'' . $canvas_id . '\');' );
    return $form;
}

sub is_valid_map ($$;$$) {
    my ( $this, $data_ref, $name, $prefix ) = @_;
    if ( !ref $setting::deco{'map'} || !%{ $setting::deco{'map'} } ) {
        undef %$data_ref;
        return 1;
    }
    $prefix = '' if !$prefix;
    if (   defined $data_ref->{ $prefix . 'mapTop' }
        && $data_ref->{ $prefix . 'mapTop' } ne ''
        && $this->is_nan( $data_ref->{ $prefix . 'mapTop' } ) )
    {
        $data_ref->{ $prefix . 'mapTop' } = 0;
    }
    if (   defined $data_ref->{ $prefix . 'mapLeft' }
        && $data_ref->{ $prefix . 'mapLeft' } ne ''
        && $this->is_nan( $data_ref->{ $prefix . 'mapLeft' } ) )
    {
        $data_ref->{ $prefix . 'mapLeft' } = 0;
    }
    if (   defined $data_ref->{ $prefix . 'markerTop' }
        && $data_ref->{ $prefix . 'markerTop' } ne ''
        && $this->is_nan( $data_ref->{ $prefix . 'markerTop' } ) )
    {
        $data_ref->{ $prefix . 'markerTop' }
            = int( $this->{'canvasHeight'} / 2 );
    }
    if (   defined $data_ref->{ $prefix . 'markerLeft' }
        && $data_ref->{ $prefix . 'markerLeft' } ne ''
        && $this->is_nan( $data_ref->{ $prefix . 'markerLeft' } ) )
    {
        $data_ref->{ $prefix . 'markerLeft' }
            = int( $this->{'canvasWidth'} / 2 );
    }
    $data_ref->{ $prefix . 'markerEnabled' } = 0
        if !$data_ref->{ $prefix . 'markerEnabled' };
    my $invalid_map = 0;
    if ( !defined $data_ref->{ $prefix . 'mapId' }
        || $data_ref->{ $prefix . 'mapId' } eq '' )
    {
        $invalid_map = 1;
    }
    else {
        my @maps = $this->get_all_maps;
        if ( grep( $_ eq $data_ref->{ $prefix . 'mapId' }, @maps ) ) {
            $invalid_map = 0;
        }
        else {
            $invalid_map = 1;
        }
    }
    if ($invalid_map) {
        push(
            @{ $cgi::action->{'dialog'} },
            ( ($name) ? $name . 'の' : '' )
                . '地図画像が選択されていません。地図画像を選択してください'
        ) if ref $cgi::action;
        return 0;
    }
    return 1;
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my ( $this, $path ) = @_;
    $main::data{ $this->{'id'} }{'enabled'} = 0
        if !$main::data{ $this->{'id'} }{'enabled'};
    if ( $this->is_valid_map( $main::data{ $this->{'id'} }, $this->name ) ) {
        $this->data;
        return 1;
    }
    else {
        return 0;
    }
}

######################################################################
#	(4) obj->usual_format(str);
sub usual_format {
    my ( $this, $str, $path, $c, $n ) = @_;
    return $this->field_tag('') if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    if (%data) {
        return $this->field_tag(
            $this->data_tag_from_ref( \%data )
                . (
                ( $data{'enabled'} )
                ? $this->get_draggable_map(
                    $this->mk_user_generated_field_id( $c, $n ), \%data )
                : ''
                )
        );
    }
    else {
        return $this->field_tag('');
    }
}

sub has_content {
    my ( $this, $str ) = @_;
    return 0 if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return ( %data && $data{'enabled'} ) ? 1 : 0;
}

sub format_for_mail {
    my ( $this, $str ) = @_;
    $str = $this->format_for_text($str);
    return ($str) ? '[' . $this->name . "]\x0A" . $str . "\x0A\x0A" : '';
}

sub format_for_text {
    my ( $this, $str, $path, $c, $n ) = @_;
    return '' if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return ( %data && $data{'enabled'} )
        ? ( ($default::english) ? 'yes' : 'あり' )
        : ( ($default::english) ? 'no' : 'なし' );
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled ) = @_;

    #名前
    my $name = 'data:' . $this->{'id'};

    $this->print_field_info_form($name);

    print '<fieldset><legend>地図設定</legend>';
    print '<fieldset>';
    print '<legend>地図のサイズ</legend>';
    print '<label>幅 <input type="number" size="4" name="'
        . $name
        . '_canvasWidth" id="'
        . $name
        . '_canvasWidth" value="'
        . $this->escape( $this->{'canvasWidth'} )
        . '" /> px</label> &times; ';
    print '<label>高さ <input type="number" size="4" name="'
        . $name
        . '_canvasHeight" id="'
        . $name
        . '_canvasHeight" value="'
        . $this->escape( $this->{'canvasHeight'} )
        . '" /> px</label>';
    print '</fieldset>';
    print '<fieldset><legend>マーカー画像</legend>';
    print $cgi::Deco->image_menu( $name . '_markerId',
        'marker', 120, $this->{'markerId'} );
    print '</fieldset>';
    print '<fieldset><legend>デフォルトの設定</legend>';
    print $this->get_draggable_map_form( 'imap-default', $this, $name . '_' );
    print '</fieldset>';
    print '</fieldset>';

    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this     = shift;
    my $is_valid = $this->is_valid_map( \%main::data, 'デフォルト',
        $this->{'id'} . '_' );
    if (   $main::data{ $this->{'id'} . '_canvasWidth' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_canvasWidth' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '地図の幅を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_canvasWidth' };
        $is_valid = 0;
    }
    if (   $main::data{ $this->{'id'} . '_canvasHeight' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_canvasHeight' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '地図の高さを更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_canvasHeight' };
        $is_valid = 0;
    }
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    return $is_valid;
}

######################################################################
#	(8) obj->sort_enabled;
sub sort_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(9) obj->cookie_enabled;
sub cookie_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	obj->search_form;
sub search_form_enabled ($) {
    my $this = shift;
    return ( $this->{'required'} ) ? 0 : 1;
}

sub search_form ($) {
    my $this = shift;
    return ( $this->{'required'} )
        ? ''
        : '<tr><td class="field-name"><label for="search:'
        . $this->{'id'}
        . ':check">'
        . $this->name
        . '</label></td><td class="field">'
        . $this->mk_select_menu(
        'search:' . $this->{'id'} . ':check', '', '',
        $main::search{ $this->{'id'} }{'check'},
        ''  => '',
        '1' => ( ($default::english) ? 'ON' : '有' ),
        '0' => ( ($default::english) ? 'OFF' : '無' )
        ) . '</td></tr>';
}

######################################################################
#sub import {
BEGIN {
    if ( !$setting::field_for_mail ) {
        Field::class_register( 'select_xmap', 'DraggableMap',
            '自作地図' );
        $setting::deco_dir{'map'}     = '地図画像';
        $setting::deco_type{'map'}    = 'map';
        $setting::deco_dir{'marker'}  = '地図マーカー画像';
        $setting::deco_type{'marker'} = 'marker';
    }
}

1;
