package FileUL;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（ファイル添付）
# FileUL.pm 1.2.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-12-22
######################################################################
#	(0)-8 obj->align;
sub align ($) {
    my $this = shift;
    return 'center';
}

######################################################################
sub __set ($$) {
    my ( $this, $setting ) = @_;
    if (   ref $setting
        && defined $setting->{'mimetype'}
        && $setting->{'mimetype'} ne '' )
    {
        undef %{ $this->{'ext_type'} };
        undef %{ $this->{'type_ext'} };
        undef %{ $this->{'mimename'} };
        delete $this->{'ext_type'};
        delete $this->{'type_ext'};
        delete $this->{'mimename'};
        my @mime
            = split( /\x0A|,[\x09\x0A\x0C\x0D\x20]*/, $$setting{'mimetype'} );
        foreach my $type (@mime) {
            next if !defined $type || $type eq '';
            if ( $type =~ /^([^:]+):([^:]+)[:]{0,1}([^:]*)$/ ) {
                $this->{'ext_type'}{$2} = $1;
                $this->{'type_ext'}{$1} = $2;
                $this->{'mimename'}{$2} = ( $3 ne '' ) ? $3 : $2;
            }
        }
    }
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }

    #複数の要素を登録
    $this->{'elements'} = 1;

    $this->___set($setting);
    return;
}

sub ___set ($$) {
    my ( $this, $setting ) = @_;
    $this->{'maxsize'} = 128000 if !$this->{'maxsize'};

    #ファイルアップロード
    $cgi::enctype_multipart = 1;
}

######################################################################
# obj->image(path[,[0-9]ata,option])
#		ファイルに対応したアイコンを表示するためのタグを返します。アイコン画像の
#		サイズは32×32で、env::iconDirで設定したディレクトリになければなりません。
sub image ($;$$$$) {
    my ( $this, $path, $data, $embed_option, $align, $valign ) = @_;
    return (   ( $this->{'is_temporary_data'} || $path )
            && ref $data
            && $data->{'type'} )
        ? $this->_image( $path, $data, $embed_option, $align, $valign,
        $this->{'id'} . '.' )
        : '';
}

sub _image ($;$$$$$) {
    my ( $this, $path, $data, $embed_option, $align, $valign, $prefix ) = @_;
    my ($out,          $file_name, $icon_dir,  $icon_image,
        $original_dir, $file_ext,  $attribute, $div_flag,
        $size,         $previewpath
    );
    $prefix = '' if !defined $prefix;
    if ( $data->{'previewtype'} ) {
        my $ppath
            = ( $this->{'is_temporary_data'} && $this->{'is_posted_preview'} )
            ? $cgi::input->get_public_clipboard
            : $path;
        $ppath
            = $this->base_change( $ppath, $cgi::output, $cgi::output_option )
            if $cgi::output || $cgi::output_option;
        $previewpath
            = $ppath . '/' . $prefix . 'preview.' . $data->{'previewtype'};
    }
    $path = $cgi::input->get_public_clipboard
        if $this->{'is_temporary_data'} && $this->{'is_posted_file'};
    $original_dir = $path;
    $size
        = -s $path . '/'
        . $prefix
        . (
        (          defined $data->{'name'}
                && $data->{'name'} ne ''
                && $data->{'name'} !~ m!/!
        ) ? $data->{'name'} . '.' : ''
        ) . $data->{'type'};
    if ( !$size ) {
        return '';
    }
    elsif ( $size >= 1000000 ) {
        $size = $this->digit( int( ( $size + 50000 ) / 100000 ) / 10 ) . 'MB';
    }
    elsif ( $size >= 1000 ) {
        $size = int( ( $size + 500 ) / 1000 ) . 'KB';
    }
    else {
        $size .= 'B';
    }
    $path = $this->base_change( $path, $cgi::output, $cgi::output_option )
        if $cgi::output || $cgi::output_option;
    $file_name = $prefix
        . (
        (          defined $data->{'name'}
                && $data->{'name'} ne ''
                && $data->{'name'} !~ m!/!
        ) ? $data->{'name'} . '.' : ''
        ) . $data->{'type'};
    $path = $path . '/' . $file_name;
    $data->{'border'} = 0 if !defined $data->{'border'};
    $file_ext = ( defined $data->{'type'} ) ? lc $data->{'type'} : '';
    $embed_option = '' if !defined $embed_option;
    $align        = '' if !defined $align;
    $valign       = '' if !defined $valign;
    $div_flag
        = ( defined $valign && ( $valign eq 'top' || $valign eq 'bottom' ) )
        ? 1
        : 0;

    #プレビューが存在する場合
    if (   defined $data->{'previewtype'}
        && $data->{'previewtype'}
        && ((   defined $this->{'icon_type'}
                && $this->{'icon_type'} eq 'preview'
            )
            || ((      !defined $this->{'icon_type'}
                    || $this->{'icon_type'} ne 'image'
                    || $file_ext !~ /^(gif|jpg|jpeg|png)$/i
                )
                && ( !defined $this->{'thumbnail'}
                    || $this->{'thumbnail'} ne 'no' )
                && ((   (      !$this->{'html5Enabled'}
                            || !grep( $_ eq $file_ext, @Common::html5_video )
                        )
                        && !grep( $_ eq $file_ext,
                            'flv',
                            (     ( $this->{'useFlash'} )
                                ? ( 'mp3', 'mp4' )
                                : ()
                            ) )
                    )
                    || !$data->{'width'}
                    || (   $this->{'maxwidth'}
                        && $data->{'width'} > $this->{'maxwidth'} )
                    || !$data->{'height'}
                    || (   $this->{'maxheight'}
                        && $data->{'height'} > $this->{'maxheight'} )
                )
            )
        )
        )
    {
        my $title
            = ( defined $data->{'title'} )
            ? $this->escape_except_html_entity( $data->{'title'} )
            . ( ( !$this->{'dlDisabled'} ) ? '(' . $size . ')' : '' )
            : ( !$this->{'dlDisabled'} ) ? $size
            :                              '';
        $title =~ s/[\x0A\x0D]//sg;
        if ( !$this->{'icon_type'} ) {
            $out
                .= '<div class="object"'
                . ( ( $align ne '' )
                ? ' style="text-align: ' . $align . '"'
                : '' )
                . '>'
                if defined $div_flag && $div_flag;
            $out .= '<a href="' . $path . '"';
            if ( $this->{'rel'}
                && grep( $_ eq $file_ext, 'gif', 'jpg', 'png' ) )
            {
                $out
                    .= ' rel="'
                    . $this->escape_except_html_entity( $this->{'rel'} )
                    . '"';
            }
            elsif (
                grep( $_ eq $file_ext,
                    'gif',               'jpg',
                    'png',               @Common::html5_audio,
                    @Common::video_plug, @Common::image_plug,
                    @Common::sound_plug )
                )
            {
                $out .= $this->set_fileviewer_window(
                    $data, 640, 480,
                    (   (   @Common::video_plug
                                && grep( $_ eq $file_ext,
                                @Common::video_plug )
                        ) ? 48 : 0
                    )
                );
            }
            $out .= ' title="' . $title . '">';
        }
        $out .= '<img src="' . $previewpath . '"';
        $out .= ' width="' . $data->{'previewwidth'} . '"'
            if defined $data->{'previewwidth'} && $data->{'previewwidth'};
        $out .= ' height="' . $data->{'previewheight'} . '"'
            if defined $data->{'previewheight'} && $data->{'previewheight'};
        $out .= ' border="' . $data->{'border'} . '"'
            if defined $data->{'border'} && $data->{'border'} ne '';
        $out .= ' alt="' . $title . '" title="' . $title . '"';
        $out .= ' style="float: ' . $align . '"'
            if !$div_flag && $align ne '';
        $out .= ' />';

        if ( !$this->{'icon_type'} ) {
            $out .= '</a>';
            $out .= '</div>' if $div_flag;
        }
        return $out;

        #画像ファイル
    }
    elsif ( $file_ext =~ /^(gif|jpg|jpeg|png)$/i ) {
        my $max_width = (
                   defined $this->{'icon_type'}
                && $this->{'icon_type'} eq 'preview'
                && $this->{'previewwidthlimit'}
                && ( !$this->{'maxwidth'}
                || $this->{'previewwidthlimit'} < $this->{'maxwidth'} )
        ) ? $this->{'previewwidthlimit'} : $this->{'maxwidth'};
        my $max_height = (
                   defined $this->{'icon_type'}
                && $this->{'icon_type'} eq 'preview'
                && $this->{'previewheightlimit'}
                && ( !$this->{'maxheight'}
                || $this->{'previewheightlimit'} < $this->{'maxheight'} )
        ) ? $this->{'previewheightlimit'} : $this->{'maxheight'};
        my $url
            = ( defined $data->{'title'}
                && !$this->is_invalid_url( $data->{'title'} ) )
            ? $this->escape_except_html_entity( $data->{'title'} )
            : '';
        my $rel = (
            $this->{'rel'}
                && ( $data->{'width'} > $max_width
                || $data->{'height'} > $max_height )
        ) ? $this->escape_except_html_entity( $this->{'rel'} ) : '';
        return (
            ( !$this->{'icon_type'} && $div_flag )
            ? '<div class="object"'
                . (
                ( $align ne '' ) ? ' style="text-align: ' . $align . '"' : ''
                )
                . '>'
            : ''
            )
            . (
            ($url)
            ? '<a href="' . $url . '" target="_blank">'
            : ($rel) ? '<a href="'
                . $path
                . '" rel="'
                . $rel
                . '" title="'
                . (
                ( $data->{'title'} )
                ? $this->escape_except_html_entity( $data->{'title'} )
                    . ( ( !$this->{'dlDisabled'} ) ? '(' . $size . ')' : '' )
                : ( !$this->{'dlDisabled'} ) ? $size
                :                              ''
                )
                . '">'
            : ''
            )
            . $cgi::Image->image_tag(
            $path,
            $data->{'width'},
            $data->{'height'},
            $data->{'border'},
            ( ( defined $data->{'title'} && !$url ) ? $data->{'title'} : '' ),
            (   ( !$this->{'icon_type'} && !$div_flag && $align ne '' )
                ? ' style="float: ' . $align . '"'
                : ''
            ),
            $max_width,
            $max_height,
            '', '',
            ( ( $url || $rel || $this->{'imageViewerDisabled'} ) ? 1 : 0 )
            )
            . ( ( $url || $rel ) ? '</a>' : '' )
            . ( ( !$this->{'icon_type'} && $div_flag ) ? '</div>' : '' );

        #アイコン用に書き出す場合→アイコンの取得へ
    }
    elsif ( $this->{'icon_type'} ) {

        #画像サイズが分かっているファイル(QT etc)
    }
    elsif (
        (   ( $this->{'defaultwidth'} && $this->{'defaultheight'} )
            || ($data->{'width'}
                && (!$this->{'maxwidth'}
                    || (   $this->{'maxwidth'}
                        && $data->{'width'} <= $this->{'maxwidth'} )
                )
                && $data->{'height'}
                && (!$this->{'maxheight'}
                    || (   $this->{'maxheight'}
                        && $data->{'height'} <= $this->{'maxheight'} )
                )
            )
        )
        && grep( $_ eq $file_ext,
            (   @Common::video_plug,
                @Common::image_plug,
                (   ( $this->{'useGoogleDocumentViewer'} )
                    ? @Common::gdv_plug
                    : ()
                ),
                'flv'
            ) )
        )
    {
        my $title
            = ( defined $data->{'title'} )
            ? $this->escape_except_html_entity( $data->{'title'} )
            : '';
        my ( $width, $height ) = (
            $data->{'width'}
                && (
                !$this->{'maxwidth'}
                || (   $this->{'maxwidth'}
                    && $data->{'width'} <= $this->{'maxwidth'} )
                )
                && $data->{'height'}
                && (
                !$this->{'maxheight'}
                || (   $this->{'maxheight'}
                    && $data->{'height'} <= $this->{'maxheight'} )
                )
            ) ? ( $data->{'width'}, $data->{'height'} )
            : ( $this->{'defaultwidth'} && $this->{'defaultheight'} )
            ? ( $this->{'defaultwidth'}, $this->{'defaultheight'} )
            : ( $this->{'maxwidth'}, $this->{'maxheight'} );
        $out
            .= '<div class="object"'
            . ( ( $align ne '' )
            ? ' style="'
                . ( ($div_flag) ? 'text-align' : 'float' ) . ': '
                . $align . '"'
            : '' )
            . '>';

        #SVG
        if ( $file_ext eq 'svg' || $file_ext eq 'svgz' ) {
            $out
                .= '<img src="'
                . $path
                . '" width="'
                . $width
                . '" height="'
                . $height . '"';
            $out .= ' alt="' . $title . '" title="' . $title . '"'
                if $title ne '';
            $out .= '>';

            #Flash
        }
        elsif ( $file_ext eq 'swf' ) {
            $out
                .= '<object type="application/x-shockwave-flash" classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000" width="'
                . $width
                . '" height="'
                . $height . '"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '>';
            $out .= '<param name="movie" value="' . $path . '"></param>';
            $out .= '<param name="quality" value="autohigh"></param>';
            $out .= '<param name="salign" value="TL"></param>';
            $out .= '<param name="allowfullscreen" value="true"></param>';
            $out
                .= '<embed type="application/x-shockwave-flash" src="'
                . $path
                . '" width="'
                . $width
                . '" height="'
                . $height
                . '" quality="autohigh" salign="TL" allowfullscreen="true" pluginspage="http://get.adobe.com/flashplayer/"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '></embed></object>';

            #Director
        }
        elsif ( $file_ext eq 'dcr' ) {
            $out
                .= '<object type="application/x-director" classid="clsid:166B1BCA-3F9C-11CF-8075-444553540000" width="'
                . $width
                . '" height="'
                . $height . '"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '>';
            $out .= '<param name="src" value="' . $path . '"></param>';
            $out .= '<param name="swStretchStyle" value="stage"></param>';
            $out
                .= '<param name="swRemote" value="swSaveEnabled=\'true\' swVolume=\'true\' swRestart=\'true\' swPausePlay=\'true\' swFastForward=\'true\' "></param>';
            $out .= '<param name="bgColor" value="#cccccc"></param>';
            $out .= '<param name="swStretchHAlign" value="Left"></param>';
            $out .= '<param name="swStretchVAlign" value="Top"></param>';
            $out
                .= '<embed src="'
                . $path
                . '" bgcolor="#cccccc" swstretchhalign="Left" swstretchvalign="Top" width="'
                . $width
                . '" height="'
                . $height
                . '" swremote="swSaveEnabled=\'true\' swVolume=\'true\' swRestart=\'true\' swPausePlay=\'true\' swFastForward=\'true\' " swstretchstyle="stage" type="application/x-director" pluginspage="http://get.adobe.com/shockwave/"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '></embed></object>';

            #Google Document Viewer
        }
        elsif (
               $this->{'useGoogleDocumentViewer'}
            && grep( $_ eq $file_ext, @Common::gdv_plug )
            && (   !$this->{'html5Enabled'}
                || !grep( $_ eq $file_ext, @Common::video_plug ) )
            )
        {
            $out
                .= '<iframe src="//docs.google.com/gview?url='
                . $this->url_encode(
                $this->full_path( $original_dir . '/' . $file_name ) )
                . '&amp;embedded=true" width="'
                . $width
                . '" height="'
                . $height . '"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out
                .= '><a href="'
                . $path
                . '" title="'
                . $size . '">'
                . ( ($title) ? $title . ' ' : '' )
                . '[Download:'
                . $size
                . ']</a></iframe>';

            #FLV
        }
        elsif ( $file_ext eq 'flv'
            || ( $this->{'useFlash'} && $file_ext eq 'mp4' ) )
        {
            my $videotag_enabled;
            my $player = (
                ( $cgi::output || $cgi::output_option )
                ? $this->base_change( $env::javascript, $cgi::output,
                    $cgi::output_option )
                : $env::javascript
            ) . '/player.swf';
            my $flash_vars = $this->escape(
                $this->url_encoded_query(
                    {   'path' => $this->base_change(
                            $original_dir . '/' . $file_name,
                            $env::javascript . '/player',
                            $cgi::output_option
                        ),
                        'type'       => $file_ext,
                        'frontcolor' => 'FFFFFF',
                        'backcolor'  => '000000',
                        'fullscreen' => 'true',
                        (     ( $data->{'previewtype'} )
                            ? ( 'thumbnail' => $previewpath )
                            : ()
                        )
                    }
                )
            );
            if ( ( $this->{'useFlash'} && $file_ext eq 'mp4' )
                && $this->{'html5Enabled'} )
            {
                $videotag_enabled = 1;
                $out
                    .= $this->media_tag_start( 'video', $path, $data, $title,
                    ( ( $data->{'previewtype'} ) ? $previewpath : '' ),
                    $embed_option );
            }
            $out
                .= '<object type="application/x-shockwave-flash" classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000" width="'
                . $width
                . '" height="'
                . $height . '"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '>';
            $out .= '<param name="movie" value="' . $player . '"></param>';
            $out .= '<param name="quality" value="high"></param>';
            $out
                .= '<param name="allowScriptAccess" value="sameDomain"></param>';
            $out .= '<param name="allowFullScreen" value="true"></param>';
            $out
                .= '<param name="flashVars" value="'
                . $flash_vars
                . '"></param>';
            $out
                .= '<embed type="application/x-shockwave-flash" src="'
                . $player
                . '" width="'
                . $width
                . '" height="'
                . $height . '"';
            $out .= ' quality="high"';
            $out .= ' allowscriptaccess="sameDomain"';
            $out .= ' allowfullscreen="true"';
            $out .= ' flashvars="' . $flash_vars . '"';
            $out .= ' pluginspage="http://get.adobe.com/flashplayer/"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option if $embed_option ne '';
            $out .= '></embed></object>';
            $out .= '</video>' if $videotag_enabled;

            #QT
        }
        else {
            my $controller;
            my $videotag_enabled;
            if (   $width
                && $height
                && @Common::video_plug
                && grep( $_ eq $file_ext, @Common::video_plug ) )
            {
                $height += 16;
                $controller = 'true';
            }
            else {
                $controller = 'false';
            }
            if (   $this->{'html5Enabled'}
                && @Common::html5_video
                && grep( $_ eq $file_ext, @Common::html5_video ) )
            {
                $videotag_enabled = 1;
                $out
                    .= $this->media_tag_start( 'video', $path, $data, $title,
                    ( ( $data->{'previewtype'} ) ? $previewpath : '' ),
                    $embed_option );
            }
            $out .= '<object data="' . $path . '"';
            $out .= ' type="' . $Common::mime_type{$file_ext} . '"'
                if $Common::mime_type{$file_ext};
            $out .= ' width="' . $width . '" height="' . $height . '"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option if $embed_option ne '';
            $out .= '>';
            $out .= '<param name="src" value="' . $path . '"></param>';
            $out .= '<param name="autoplay" value="false"></param>';
            $out .= '<param name="scale" value="aspect"></param>';
            $out
                .= '<param name="controller" value="'
                . $controller
                . '"></param>';
            $out .= '<embed';
            $out .= ' type="' . $Common::mime_type{$file_ext} . '"'
                if $Common::mime_type{$file_ext};
            $out
                .= ' src="'
                . $path
                . '" width="'
                . $width
                . '" height="'
                . $height
                . '" scale="aspect" autoplay="false" controller="'
                . $controller . '"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '></embed></object>';
            $out .= '</video>'                if $videotag_enabled;
        }
        $out
            .= '<div class="file-download"><a href="'
            . $path
            . '" title="'
            . $size . '">'
            . ( ($title) ? $title . ' ' : '' )
            . '[Download:'
            . $size
            . ']</a></div>'
            if !$this->{'dlDisabled'};
        $out .= '</div>';
        return $out;

        #QTsound
    }
    elsif (
        (   !$this->{'maxwidth'}
            || ( $this->{'maxwidth'} && 100 < $this->{'maxwidth'} )
        )
        && ( !$this->{'maxheight'}
            || ( $this->{'maxheight'} && 16 < $this->{'maxheight'} ) )
        && grep( $_ eq $file_ext, @Common::sound_plug )
        )
    {
        my $audiotag_enabled;
        my $title
            = ( defined $data->{'title'} )
            ? $this->escape_except_html_entity( $data->{'title'} )
            : '';
        $out
            .= '<div class="object"'
            . ( ( $align ne '' )
            ? ' style="'
                . ( ($div_flag) ? 'text-align' : 'float' ) . ': '
                . $align . '"'
            : '' )
            . '>';
        if (   $this->{'html5Enabled'}
            && @Common::html5_audio
            && grep( $_ eq $file_ext, @Common::html5_audio ) )
        {
            $audiotag_enabled = 1;
            $out .= $this->media_tag_start( 'audio', $path, $data, $title, '',
                $embed_option );
        }
        if ( $this->{'useFlash'} && $file_ext eq 'mp3' ) {
            my $width = (
                $this->{'maxwidth'}
                    && ( !$data->{'width'}
                    || $data->{'width'} > $this->{'maxwidth'} )
                ) ? $this->{'maxwidth'}
                : (
                $this->{'widthlimit'}
                    && ( !$data->{'width'}
                    || $data->{'width'} > $this->{'widthlimit'} )
                )                      ? $this->{'widthlimit'}
                : ( $data->{'width'} ) ? $data->{'width'}
                :                        240;
            my $player = (
                ( $cgi::output || $cgi::output_option )
                ? $this->base_change( $env::javascript, $cgi::output,
                    $cgi::output_option )
                : $env::javascript
            ) . '/player.swf';
            my $flash_vars = $this->escape(
                $this->url_encoded_query(
                    {
#'path' => $this->base_change($original_dir . '/' . $file_name,$env::javascript . '/player',$cgi::output_option),
                        'path'       => $path,      #MP3だけ？？
                        'type'       => 'mp3',
                        'frontcolor' => 'FFFFFF',
                        'backcolor'  => '000000',
                        (     ( $data->{'previewtype'} )
                            ? ( 'thumbnail' => $previewpath )
                            : ()
                        )
                    }
                )
            );
            $out
                .= '<object type="application/x-shockwave-flash" classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000" width="'
                . $width
                . '" height="'
                . ( ( $data->{'previewtype'} && $data->{'height'} )
                ? $data->{'height'}
                : '27' )
                . '"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '>';
            $out .= '<param name="movie" value="' . $player . '"></param>';
            $out .= '<param name="quality" value="high"></param>';
            $out
                .= '<param name="allowScriptAccess" value="sameDomain"></param>';
            $out .= '<param name="allowFullScreen" value="true"></param>';
            $out
                .= '<param name="flashVars" value="'
                . $flash_vars
                . '"></param>';
            $out
                .= '<embed type="application/x-shockwave-flash" src="'
                . $player
                . '" width="'
                . $width
                . '" height="'
                . ( ( $data->{'previewtype'} && $data->{'height'} )
                ? $data->{'height'}
                : '27' )
                . '"';
            $out .= ' quality="high"';
            $out .= ' allowscriptaccess="sameDomain"';
            $out .= ' allowfullscreen="true"';
            $out .= ' flashvars="' . $flash_vars . '"';
            $out .= ' pluginspage="http://get.adobe.com/flashplayer/"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option if $embed_option ne '';
            $out .= '></embed></object>';

        }
        else {
            my $width
                = ( $data->{'width'} )
                ? ( $data->{'width'} > $this->{'maxwidth'} )
                    ? $this->{'maxwidth'}
                    : $data->{'width'}
                : 100;
            $out .= '<object';
            $out .= ' type="' . $Common::mime_type{$file_ext} . '"'
                if $Common::mime_type{$file_ext};
            $out .= ' width="' . $width . '" height="16"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= '>';
            $out .= '<param name="src" value="' . $path . '"></param>';
            $out .= '<param name="autoplay" value="false"></param>';
            $out .= '<param name="controller" value="true"></param>';
            $out .= '<embed';
            $out .= ' type="' . $Common::mime_type{$file_ext} . '"'
                if $Common::mime_type{$file_ext};
            $out
                .= ' src="'
                . $path
                . '" width="'
                . $width
                . '" height="16" autoplay="false" controller="true"';
            $out .= ' title="' . $title . '"' if $title ne '';
            $out .= ' ' . $embed_option       if $embed_option ne '';
            $out .= '></embed></object>';
        }
        $out .= '</audio>' if $audiotag_enabled;
        $out
            .= '<div class="file-download"><a href="'
            . $path
            . '" title="'
            . $size . '">'
            . ( ($title) ? $title . ' ' : '' )
            . '[Download:'
            . $size
            . ']</a></div>'
            if !$this->{'dlDisabled'};
        $out .= '</div>';
        return $out;

    }

    #添付ファイル
    if ( defined $cgi::output && $cgi::output ) {
        $icon_dir = $this->base_change( $env::icon_dir, $cgi::output );
    }
    else {
        $icon_dir = $env::icon_dir;
    }
    if ( !@env::icon_images && $env::icon_dir && -d $env::icon_dir ) {
        opendir( DIR, $env::icon_dir );
        @env::icon_images = readdir(DIR);
        closedir(DIR);
    }

    #アイコン画像
    ($icon_image) = grep( $_ =~ /^$file_ext\./, @env::icon_images );

    #QTプラグイン表示のできるムービー
    if ( @Common::video_plug && grep( $_ eq $file_ext, @Common::video_plug ) )
    {
        $attribute = $this->set_fileviewer_window( $data, 352, 255, 50 );
        $icon_image = 'mov.png'
            if !$icon_image && grep( $_ eq 'mov.png', @env::icon_images );

        #プラグイン表示のできる画像
    }
    elsif ( @Common::image_plug
        && grep( $_ eq $file_ext, @Common::image_plug ) )
    {
        $attribute = $this->set_fileviewer_window( $data, 600, 400 );
        $icon_image = 'image.png'
            if !$icon_image && grep( $_ eq 'image.png', @env::icon_images );

        #QTsound
    }
    elsif ( @Common::sound_plug
        && grep( $_ eq $file_ext, @Common::sound_plug ) )
    {
        $attribute = $this->set_fileviewer_window( $data, 320, 20 );
        $icon_image = 'sound.png'
            if !$icon_image && grep( $_ eq 'sound.png', @env::icon_images );

    }

    my $title
        = ( defined $data->{'title'} )
        ? $this->escape_except_html_entity( $data->{'title'} )
        . ( ( !$this->{'dlDisabled'} ) ? '(' . $size . ')' : '' )
        : ( !$this->{'dlDisabled'} ) ? $size
        :                              '';
    $icon_image = 'unknown.png' if !$icon_image;
    $attribute .= ' title="' . $title . '"';

    $icon_dir .= '/' if $icon_dir !~ m!/$!;
    $out
        .= '<div class="object"'
        . ( ( $align ne '' ) ? ' style="text-align: ' . $align . '"' : '' )
        . '>'
        if $div_flag;
    if ( $this->{'html5Enabled'} ) {
        if ( @Common::html5_video
            && grep( $_ eq $file_ext, @Common::html5_video ) )
        {
            $out
                .= $this->media_tag_start( 'video', $path, $data, $title,
                ( ( $data->{'previewtype'} ) ? $previewpath : '' ),
                $embed_option );
            $out .= '</video>';
        }
        elsif ( @Common::html5_audio
            && grep( $_ eq $file_ext, @Common::html5_audio ) )
        {
            $out .= $this->media_tag_start( 'audio', $path, $data, $title, '',
                $embed_option );
            $out .= '</audio>';
        }
    }
    $out .= '<a href="' . $path . '"' . $attribute . '>';
    $out
        .= '<img src="'
        . $icon_dir
        . $icon_image
        . '" border="0" width="32" height="32" alt="'
        . $title . '"'
        . ( ( !$div_flag && $align ne '' )
        ? ' style="float: ' . $align . '"'
        : '' )
        . ' />';
    $out .= '</a>';
    $out .= '</div>' if $div_flag;

    return $out;
}

sub media_tag_start ($$$$;$$) {
    my ( $this, $type, $path, $data, $title, $preview, $embed_option ) = @_;
    $title = $this->escape_except_html_entity( $data->{'title'} )
        if !$title && defined $data->{'title'};
    my ( $w, $h ) = ( $data->{'width'}, $data->{'height'} );
    my $style = '';
    if ( $this->{'maxwidth'} ) {
        if ( !$w ) {
            $style .= 'max-width:' . $this->{'maxwidth'} . 'px;';
        }
        elsif ( $w > $this->{'maxwidth'} ) {
            $h = int( $h * $this->{'maxwidth'} / $w ) if $h;
            $w = $this->{'maxwidth'};
        }
    }
    if ( $this->{'maxheight'} ) {
        if ( !$h ) {
            $style .= 'max-height:' . $this->{'maxheight'} . 'px;';
        }
        elsif ( $h > $this->{'maxheight'} ) {
            $w = int( $w * $this->{'maxheight'} / $h ) if $w;
            $h = $this->{'maxheight'};
        }
    }
    return
          '<'
        . $type
        . ' src="'
        . $path . '"'
        . ( ($w) ? ' width="' . $w . '"'  : '' )
        . ( ($h) ? ' height="' . $h . '"' : '' )
        . (
        ( defined $title && $title ne '' ) ? ' title="' . $title . '"' : '' )
        . ( ( $this->{'autoplayEnabled'} ) ? ' autoplay="autoplay"' : '' )
        . (
        ( $this->{'preloadDisabled'} || $preview ) ? ' preload="none"' : '' )
        . ( ( $this->{'loopEnabled'} ) ? ' loop="loop"' : '' )
        . ( ( !$this->{'controlsDisabled'} ) ? ' controls="controls"' : '' )
        . ( ($preview)      ? ' poster="' . $preview . '"' : '' )
        . ( ($embed_option) ? ' ' . $embed_option          : '' )
        . ( ($style)        ? ' style="' . $style . '"'    : '' ) . '>';
}

sub set_fileviewer_window ($$) {
    my ( $this, $data, $w, $h, $c ) = @_;
    my ( $width, $height ) = ( $data->{'width'}, $data->{'height'} );
    $width  = $w || 640 if !$width;
    $height = $h || 480 if !$height;
    my ( $win_width, $win_height ) = ( $width, $height );
    $win_width  = 800 if !$win_width  || $win_width > 800;
    $win_height = 600 if !$win_height || $win_height > 600;
    if ($c) {
        $height     += $c;
        $win_height += $c;
    }
    return
        " target=\"fileViewer\" onclick=\"if(typeof cmonos == 'object'){cmonos.window.open('fileViewer',$width,$height)}else{window.open('','fileViewer','width=$win_width,height=$win_height,scrollbars=yes,resizable=yes')} return true\"";
}

######################################################################
#	(1)-0 obj->match_enabled;
sub match_enabled ($) {
    my $this = shift;
    $this->search_ext;
    return (
        @{ $this->{'filetype'} }
            || ( defined $main::search{ $this->{'id'} }{'str'}
            && $main::search{ $this->{'id'} }{'str'} ne '' )
    ) ? 1 : 0;
}

#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    my ( $matched, $cmt_matched, %data );
    return 1 if !$this->match_enabled;

    #データを分解
    %data = $this->elements($$str);

    #拡張子照合
    $matched = 0;
    if ( @{ $this->{'filetype'} } ) {
        my ( $e, $ext );
        foreach ( @{ $this->{'filetype'} } ) {
            if ( $_ eq '-e' ) {
                if ( defined $data{'type'} && $data{'type'} ne '' ) {
                    $matched = 1;
                    last;
                }
            }
            elsif ( $_ eq '!e' ) {
                if ( !defined $data{'type'} || $data{'type'} eq '' ) {
                    $matched = 1;
                    last;
                }
            }
            else {
                if ( eval { $data{'type'} =~ /^\Q$_\E$/i } ) {
                    $matched = 1;
                    last;
                }
            }
        }
    }
    else {
        $matched = 1;
    }

    #コメント照合
    if ( $matched && $this->{'comment'} ) {
        @{ $this->{'pattern'} } = $this->make_patterns(
            \$main::search{ $this->{'id'} }{'str'},
            (   (          $this->{'ignoreCase'}
                        || $main::search{ $this->{'id'} }{'i'}
                ) ? 1 : 0
            ),
            $main::search{ $this->{'id'} }{'mode'}
        ) if !ref $this->{'pattern'};
        $matched = 0
            if !$this->pattern_matched(
            \$data{'comment'},
            $this->{'pattern'},
            (   (          $this->{'ignoreCase'}
                        || $main::search{ $this->{'id'} }{'i'}
                ) ? 1 : 0
            ),
            $main::search{ $this->{'id'} }{'mode'}
            );
    }

    return $matched;
}

#検索対象ファイルの拡張子指定
sub search_ext ($) {
    my $this = shift;
    return if ref $this->{'filetype'};

    if ( defined $main::search{ $this->{'id'} }{'ext'}
        && $main::search{ $this->{'id'} }{'ext'} ne '' )
    {
        $main::search{ $this->{'id'} }{'ext'}
            = $this->z_to_h( $main::search{ $this->{'id'} }{'ext'} );
        push(
            @{ $this->{'filetype'} },
            split(
                /[\x09\x0A\x0C\x0D\x20]+/,
                $main::search{ $this->{'id'} }{'ext'}
            )
        );
        map( $_ =~ s/^\.//, @{ $this->{'filetype'} } );
    }
    if ( defined $main::search{ $this->{'id'} }{'type'} ) {
        $this->scalar_to_array( \$main::search{ $this->{'id'} }{'type'} );
        push(
            @{ $this->{'filetype'} },
            @{ $main::search{ $this->{'id'} }{'type'} }
        );
    }
    @{ $this->{'filetype'} }
        = grep( defined $_ && $_ ne '', @{ $this->{'filetype'} } );
    if ( grep( $_ ne '-e' && $_ ne '!e', @{ $this->{'filetype'} } ) ) {
        @{ $this->{'filetype'} }
            = grep( $_ ne '-e' && $_ ne '!e', @{ $this->{'filetype'} } );
    }
}

######################################################################
#	(2) obj->form;
sub form {
    my ( $this, $status, $original, $path ) = @_;
    return ''
        if $ENV{'HTTP_USER_AGENT'}
        && $ENV{'HTTP_USER_AGENT'}
        =~ /(?:DoCoMo|KDDI|UP\.Browser)/i;    # docomo au

    my ( $name, $form, @menu, $title, $v );

    #名前にデータIDを埋め込む
    $name = 'data:' . $this->{'id'};

    #デフォルト作成
    $main::data{ $this->{'id'} }{'comment'} = $this->{'default'}
        if ( !defined $status || $status eq '' )
        && ( !defined $main::data{ $this->{'id'} }{'comment'}
        || $main::data{ $this->{'id'} }{'comment'} eq '' );
    $title = $this->escape( $main::data{ $this->{'id'} }{'title'} );

    #テーブル復元
    if (   ( !$this->{'nostyle'} || $this->{'nostyle'} <= 2 )
        && defined $main::data{ $this->{'id'} }{'comment'}
        && $main::data{ $this->{'id'} }{'comment'}
        =~ /<!--_[ct]sv(?![0-9a-zA-Z_])[^>]*-->/ )
    {
        my $tag = ( $this->{'csv'} ) ? 'no' : 'tag';
        $main::data{ $this->{'id'} }{'comment'}
            = $this->{'Table'}
            ->text_to_sv( $main::data{ $this->{'id'} }{'comment'},
            $tag, 'csv' );
    }

    #フィールドネーム+ヘッダ
    $form = $this->field_header('50');

    #アップロードフォーム
    $form .= '<table class="sub-field-table"><tbody>';
    if ( $this->{'importEnabled'} ) {
        $form
            .= '<tr><td class="sub-field-name"><label for="'
            . $name
            . ':url" class="datatitle">URL</label></td><td class="sub-field">';
        $form
            .= '<input type="url" name="'
            . $name
            . ':url" id="'
            . $name
            . ':url" size="36" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'url'} ) . '"'
            . ( ( $this->{'is_invalid_field'}{'url'} )
            ? $this->set_invalid_class
            : '' )
            . ' title="外部コンテンツの URL を指定します。" />';
        if ((      $this->{'photoSharingServiceName'}
                && $this->{'photoSharingServiceAccount'}
            )
            || $this->{'photoSharingServiceRSS'}
            )
        {
            $form
                .= ' <a href="./start.cgi'
                . $this->escape( $this->get_query_for_editor )
                . '" target="photoViewer" class="note" onclick="cmonos.window.open(\'photoViewer\',640,600)">コンテンツ一覧を開く</a>';
            $form .= '<div id="content-sample-' . $this->{'id'} . '"></div>';
        }
        $form .= '</td></tr>';
    }
    $form
        .= '<tr><td class="sub-field-name"><label for="'
        . $name
        . ':file">ファイル</label></td><td class="sub-field">';
    $form
        .= '<span id="'
        . $name
        . ':file-wrapper"><input type="file" name="'
        . $name
        . ':file" id="'
        . $name
        . ':file"'
        . ( ( $this->{'is_invalid_field'}{'file'} )
        ? $this->set_invalid_class
        : '' )
        . ' /></span><input type="button" value="&#139; リセット" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . ':file\').reset()">';
    $form .= '<div class="note">(&lt; ' . $this->{'maxsize'} . ' byte) </div>'
        if defined $this->{'maxsize'} && $this->{'maxsize'};

    #ファイルの存在をチェック
    (   $main::data{ $this->{'id'} }{'width'},
        $main::data{ $this->{'id'} }{'height'},
        $main::data{ $this->{'id'} }{'type'},
        $main::data{ $this->{'id'} }{'name'}
        )
        = ( '', '', '', '' )
        unless $main::data{ $this->{'id'} }{'type'}
        && -f ( ( !$cgi::input->{'clipboard_expired'}
                && $this->{'is_posted_file'} )
        ? $cgi::input->get_public_clipboard
        : ( $path || './' ) )
        . '/'
        . $this->{'id'}
        . (
        (          defined $main::data{ $this->{'id'} }{'name'}
                && $main::data{ $this->{'id'} }{'name'} ne ''
                && $main::data{ $this->{'id'} }{'name'} !~ /[^0-9a-zA-Z_]/
        ) ? '.' . $main::data{ $this->{'id'} }{'name'} : ''
        )
        . '.'
        . $main::data{ $this->{'id'} }{'type'};

    #画像サイズ
    if ( defined $this->{'imagesize'} && $this->{'imagesize'} ) {
        $form
            .= '</td></tr><tr><td class="sub-field-name"><span class="datatitle">サイズ</span></td><td class="sub-field">';
        $form
            .= '<label>幅<input type="number" name="'
            . $name
            . ':width" id="'
            . $name
            . ':width" size="4"';
        $form
            .= ' value="'
            . $this->escape( $main::data{ $this->{'id'} }{'width'} ) . '"'
            if defined $main::data{ $this->{'id'} }{'width'}
            && $main::data{ $this->{'id'} }{'width'};
        $form .= $this->set_invalid_class
            if $this->{'is_invalid_field'}{'width'};
        $form .= ' /></label>';
        $form
            .= '<span class="note"> (&lt;'
            . $this->{'widthlimit'}
            . ') </span>'
            if $this->{'widthlimit'};
        $form
            .= '×<label>高<input type="number" name="'
            . $name
            . ':height" size="4"';
        $form
            .= ' value="'
            . $this->escape( $main::data{ $this->{'id'} }{'height'} ) . '"'
            if defined $main::data{ $this->{'id'} }{'height'}
            && $main::data{ $this->{'id'} }{'height'};
        $form .= $this->set_invalid_class
            if $this->{'is_invalid_field'}{'height'};
        $form .= ' /></label>';
        $form
            .= '<span class="note"> (&lt;'
            . $this->{'heightlimit'}
            . ') </span>'
            if $this->{'heightlimit'};
        $form .= '<span class="unit">ピクセル</span>';
    }
    else {
        $form
            .= '<div class="note">(&lt; '
            . (
            ( $this->{'widthlimit'} ) ? $this->{'widthlimit'} : '&infin;' )
            . ' × '
            . (
            ( $this->{'heightlimit'} ) ? $this->{'heightlimit'} : '&infin;' )
            . ' <span class="unit">ピクセル</span>) </div>'
            if ( defined $this->{'widthlimit'} && $this->{'widthlimit'} )
            || ( defined $this->{'heightlimit'} && $this->{'heightlimit'} );
        $form
            .= '<input type="hidden" name="'
            . $name
            . ':width" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'width'} ) . '" />';
        $form
            .= '<input type="hidden" name="'
            . $name
            . ':height" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'height'} )
            . '" />';
    }
    $form
        .= '<input type="hidden" name="'
        . $name
        . ':latlng" value="'
        . $this->escape( $main::data{ $this->{'id'} }{'latlng'} ) . '" />'
        if $main::data{ $this->{'id'} }{'latlng'};
    $form .= '</td></tr>';
    if (   !$this->{'titlefrom'}
        || !$this->{'usertitledisabled'}
        || !ref $this->{'field_obj'}->{ $this->{'titlefrom'} }
        || $this->{'field_obj'}->{ $this->{'titlefrom'} }->{'class'}
        !~ /^text/
        || (   $this->{'field_obj'}->{ $this->{'titlefrom'} }->{'rows'}
            && $this->{'field_obj'}->{ $this->{'titlefrom'} }->{'rows'} > 1 )
        )
    {
        $form
            .= '<tr><td class="sub-field-name"><label for="'
            . $name
            . ':title">タイトル</label></td><td class="sub-field"><textarea name="'
            . $name
            . ':title" id="'
            . $name
            . ':title" cols="36" rows="1" title="ファイルを表示できなかったとき表示する代替文字列を入力します。">';
        $form .= $title if defined $title && $title;
        $form .= '</textarea>';
        $form
            .= ' <label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\''
            . $name
            . ':title\').setAutoFit(this.checked);" />フィット</label></td></tr>';
    }

    #プレビュー
    if ( defined $this->{'preview'} && $this->{'preview'} ) {
        $form .= '<tr><td class="sub-field-name">';
        $form
            .= '<label for="'
            . $name
            . ':preview">サムネイル</label></td><td class="sub-field">';
        $form
            .= '<span id="'
            . $name
            . ':preview-wrapper"><input type="file" name="'
            . $name
            . ':preview" id="'
            . $name
            . ':preview" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'preview'} ) . '"'
            . ( ( $this->{'is_invalid_field'}{'preview'} )
            ? $this->set_invalid_class
            : '' )
            . ' /></span><input type="button" value="&#139; リセット" onclick="cmonos.form.item(this.form).fieldname(\''
            . $name
            . ':preview\').reset()">';
        $form
            .= '<div class="note">(&lt; '
            . (
            ( $this->{'previewwidthlimit'} ) ? $this->{'previewwidthlimit'}
            : '&infin;' )
            . ' × '
            . (
            ( $this->{'previewheightlimit'} ) ? $this->{'previewheightlimit'}
            : '&infin;' )
            . ' <span class="unit">ピクセル</span>) </div>'
            if $this->{'previewwidthlimit'} || $this->{'previewheightlimit'};
        $form .= '</td></tr>';
    }
    $form .= '</tbody></table>';

    #既存ファイルの表示
    if ((      ( defined $status && $status ne '' )
            || $this->{'is_posted_file'}
            || $this->{'is_posted_preview'}
        )
        && ((   defined $main::data{ $this->{'id'} }{'type'}
                && $main::data{ $this->{'id'} }{'type'} ne ''
            )
            || (   $this->{'preview'}
                && $main::data{ $this->{'id'} }{'previewtype'} )
        )
        )
    {
        local $this->{'is_temporary_data'}
            = ( !$cgi::input->{'clipboard_expired'} ) ? 1 : 0;
        $form .= '<div class="sample-image">';
        my $max_width = $this->{'maxwidth'};
        $this->{'maxwidth'}
            = ( !$max_width || $max_width > 300 ) ? 300 : $max_width;
        if ( defined $main::data{ $this->{'id'} }{'type'}
            && $main::data{ $this->{'id'} }{'type'} ne '' )
        {
            $form .= $this->image( $path, $main::data{ $this->{'id'} },
                $this->{'option'} );
            if ( $this->{'is_temporary_data'} && $this->{'is_posted_file'} ) {
                $form
                    .= '<div class="extra-field"><label><input type="checkbox" name="option:'
                    . $this->{'id'}
                    . ':cancel" value="1" /> この<strong>未確定</strong>ファイルを削除する</label></div>';
            }
            elsif ( !$this->{'required'} ) {
                $form
                    .= '<div class="extra-field"><label><input type="checkbox" name="'
                    . $name
                    . ':delete" value="1"'
                    . (
                    (          $this->{'is_temporary_data'}
                            && $main::data{ $this->{'id'} }{'delete'}
                    ) ? ' checked="checked"' : ''
                    ) . ' /> このファイルを削除する</label></div>';
            }
            $form
                .= '<input type="hidden" name="'
                . $name
                . ':type" value="'
                . $this->escape( $main::data{ $this->{'id'} }{'type'} )
                . '" />';
            $form
                .= '<input type="hidden" name="'
                . $name
                . ':name" value="'
                . $this->escape( $main::data{ $this->{'id'} }{'name'} )
                . '" />';
        }
        if (   $this->{'preview'}
            && $main::data{ $this->{'id'} }{'previewtype'} )
        {
            if ( !defined $main::data{ $this->{'id'} }{'type'}
                || $main::data{ $this->{'id'} }{'type'} eq '' )
            {
                $form .= '<img src="'
                    . (
                    (          $this->{'is_temporary_data'}
                            && $this->{'is_posted_preview'}
                    ) ? $cgi::input->get_public_clipboard : $path
                    )
                    . '/'
                    . $this->{'id'}
                    . '.preview.'
                    . $main::data{ $this->{'id'} }{'previewtype'} . '"';
                $form
                    .= ' width="'
                    . $main::data{ $this->{'id'} }{'previewwidth'} . '"'
                    if defined $main::data{ $this->{'id'} }{'previewwidth'}
                    && $main::data{ $this->{'id'} }{'previewwidth'};
                $form
                    .= ' height="'
                    . $main::data{ $this->{'id'} }{'previewheight'} . '"'
                    if defined $main::data{ $this->{'id'} }{'previewheight'}
                    && $main::data{ $this->{'id'} }{'previewheight'};
                $form .= ' alt="' . $title . '" title="' . $title . '"'
                    if defined $title && $title ne '';
                $form .= ' />';
            }
            if (   $this->{'is_temporary_data'}
                && $this->{'is_posted_preview'} )
            {
                $form
                    .= '<div class="extra-field"><label><input type="checkbox" name="option:'
                    . $this->{'id'}
                    . ':previewcancel" value="1" /> この<strong>未確定</strong>サムネイル画像を削除する</label></div>';
            }
            else {
                $form
                    .= '<div class="extra-field"><label><input type="checkbox" name="'
                    . $name
                    . ':deletepreview" value="1"'
                    . (
                    (          $this->{'is_temporary_data'}
                            && $main::data{ $this->{'id'} }{'deletepreview'}
                    ) ? ' checked="checked"' : ''
                    )
                    . ' /> このサムネイル画像を削除する</label></div>';
            }
            $form
                .= '<input type="hidden" name="'
                . $name
                . ':previewtype" value="'
                . $this->escape( $main::data{ $this->{'id'} }{'previewtype'} )
                . '" />';
            $form
                .= '<input type="hidden" name="'
                . $name
                . ':previewwidth" value="'
                . $this->escape(
                $main::data{ $this->{'id'} }{'previewwidth'} )
                . '" />';
            $form
                .= '<input type="hidden" name="'
                . $name
                . ':previewheight" value="'
                . $this->escape(
                $main::data{ $this->{'id'} }{'previewheight'} )
                . '" />';
        }
        $form .= '</div>';
        $this->{'maxwidth'} = $max_width;
        delete $this->{'is_temporary_data'};
    }

    #コメント
    $form .= $this->comment_form( $name, $original ) if $this->{'comment'};

    $form .= $this->field_footer;
    return $form;
}

sub comment_form ($$$) {
    my ( $this, $name, $original ) = @_;
    $main::data{ $this->{'id'} }{'valign'} = 'top'
        if ( !defined $main::data{ $this->{'id'} }{'type'}
        || $main::data{ $this->{'id'} }{'type'} eq '' )
        && ( !defined $main::data{ $this->{'id'} }{'comment'}
        || $main::data{ $this->{'id'} }{'comment'} eq '' )
        && ( !defined $main::data{ $this->{'id'} }{'align'}
        || $main::data{ $this->{'id'} }{'align'} eq '' )
        && ( !defined $main::data{ $this->{'id'} }{'valign'}
        || $main::data{ $this->{'id'} }{'valign'} eq '' );
    $this->{'cols'} = 40 if !$this->{'cols'};
    my $form
        = '<div class="additional-field"><label for="'
        . $name
        . ':comment">コメント</label>';

    #テキスト入力欄
    #高さが1以下の場合
    if ( !$this->{'rows'} || $this->{'rows'} <= 1 ) {
        $main::data{ $this->{'id'} }{'comment'} =~ s/\x0A//gs;
        $form
            .= '<input type="text" name="'
            . $name
            . ':comment" id="'
            . $name
            . ':comment" size="'
            . $this->{'cols'} . '"';
        $form .= ' maxlength="' . $this->{'maxlength'} . '"'
            if $this->{'maxlength'};
        $form .= ' value="'
            . $this->escape(
            $this->unescape_data_tag(
                $main::data{ $this->{'id'} }{'comment'}
            )
            )
            . '"'
            if defined $main::data{ $this->{'id'} }{'comment'}
            && $main::data{ $this->{'id'} }{'comment'};
        $form .= $this->set_invalid_class
            if $this->{'is_invalid_field'}{'comment'};
        $form .= ' />';

        #高さが2以上の場合
    }
    else {
#$main::data{$this->{'id'}}{'comment'} =~ s/<br(?![0-9a-zA-Z_])[^>]*>/\x0A/sig;
        $form
            .= '<textarea name="'
            . $name
            . ':comment" id="'
            . $name
            . ':comment" cols="'
            . $this->{'cols'}
            . '" rows="'
            . $this->{'rows'} . '"'
            . ( ( $this->{'is_invalid_field'}{'preview'} )
            ? $this->set_invalid_class
            : '' )
            . '>';
        $form .= $this->escape(
            $this->blockquote_to_quotation(
                $this->unescape_data_tag(
                    $main::data{ $this->{'id'} }{'comment'}
                ),
                $this->{'cols'}
            )
            )
            if defined $main::data{ $this->{'id'} }{'comment'}
            && $main::data{ $this->{'id'} }{'comment'};
        $form .= '</textarea>';

        #ツールボックス
        my $original_text
            = (    ref $original eq 'HASH'
                && defined $original->{'comment'}
                && $original->{'comment'} ne '' )
            ? $original->{'comment'}
            : '';
        $form .= $this->textarea_tool( 'data:' . $this->{'id'} . ':comment',
            $original_text );
        $form
            .= '<div class="hints">カンマ区切り文字列は、自動的に表に変換されます。</div>'
            if defined $this->{'csv'} && $this->{'csv'};
    }
    $form .= '</div>';

    #アイコン画像位置
    if ( !$setting::field_for_mail ) {
        $form
            .= '<div class="additional-field"><span class="datatitle">ファイル表示位置</span>';
        $form .= $this->mk_select_menu(
            $name . ':align', '', '', $main::data{ $this->{'id'} }{'align'},
            ''       => '',
            'left'   => '左',
            'center' => '中央',
            'right'  => '右'
        );
        $form .= $this->mk_select_menu(
            $name . ':valign', '', '', $main::data{ $this->{'id'} }{'valign'},
            ''       => '',
            'top'    => '上',
            'middle' => '回り込み',
            'bottom' => '下'
        );
        $form .= '</div>';
    }
    return $form;
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my ( $this, $path ) = @_;
    delete $this->{'is_invalid_field'} if ref $this->{'is_invalid_field'};
    my ($is_valid,       $name,         $width,
        $height,         $ext,          $latlng,
        $file_name,      $preview_name, $preview_width,
        $preview_height, $preview_ext,  $previewwidthlimit,
        $previewheightlimit
    );
    $is_valid = 1;
    $name     = 'data:' . $this->{'id'} . ':file';
    $this->fill(
        \$main::data{ $this->{'id'} }{'width'},
        \$main::data{ $this->{'id'} }{'height'},
        \$main::data{ $this->{'id'} }{'type'},
        \$main::data{ $this->{'id'} }{'name'},
        \$main::data{ $this->{'id'} }{'latlng'},
    );
    $preview_name = 'data:' . $this->{'id'} . ':preview';
    if ( $this->{'preview'} ) {
        $main::data{ $this->{'id'} }{'previewwidth'}
            = $main::data{ $this->{'id'} }{'previewheight'}
            = $main::data{ $this->{'id'} }{'previewtype'} = ''
            if !$default::static && ( !defined $main::n || $main::n eq '' );
        $this->fill(
            \$main::data{ $this->{'id'} }{'previewwidth'},
            \$main::data{ $this->{'id'} }{'previewheight'},
            \$main::data{ $this->{'id'} }{'previewtype'}
        );
        $previewwidthlimit
            = ( $this->{'previewwidthlimit'} ) ? $this->{'previewwidthlimit'}
            : ( $this->{'maxwidth'} ) ? $this->{'maxwidth'}
            :                           '';
        $previewheightlimit
            = ( $this->{'previewheightlimit'} )
            ? $this->{'previewheightlimit'}
            : ( $this->{'maxheight'} ) ? $this->{'maxheight'}
            :                            '';
    }

    #タイトル
    if (   $this->{'titlefrom'}
        && ref $this->{'field_obj'}->{ $this->{'titlefrom'} }
        && $this->{'field_obj'}->{ $this->{'titlefrom'} }->{'class'}
        =~ /^text/
        && (  !$this->{'field_obj'}->{ $this->{'titlefrom'} }->{'rows'}
            || $this->{'field_obj'}->{ $this->{'titlefrom'} }->{'rows'} == 1 )
        && (   !defined $main::data{ $this->{'id'} }{'title'}
            || $main::data{ $this->{'id'} }{'title'} eq ''
            || $this->{'usertitledisabled'} )
        )
    {
        $main::data{ $this->{'id'} }{'title'}
            = ( defined $main::data{ $this->{'titlefrom'} }{'str'} )
            ? $main::data{ $this->{'titlefrom'} }{'str'}
            : '';
        $main::data{ $this->{'id'} }{'title'} =~ s/<>[#0-9a-zA-z]{3,10}\Z//s;
        $this->detag( \$main::data{ $this->{'id'} }{'title'} );
    }
    $main::data{ $this->{'id'} }{'title'} =~ s/[\x0A\x0D\x09\x20]+/ /gs
        if $main::data{ $this->{'id'} }{'title'};

    #既存ファイルの情報
    ( $width, $height ) = (
        $main::data{ $this->{'id'} }{'width'},
        $main::data{ $this->{'id'} }{'height'}
    ) if $this->{'imagesize'};
    if ( !$path || !-d $path ) {
        $main::data{ $this->{'id'} }{'width'}
            = $main::data{ $this->{'id'} }{'height'} = ''
            if !$this->{'imagesize'};
        $main::data{ $this->{'id'} }{'type'}
            = $main::data{ $this->{'id'} }{'name'}
            = $main::data{ $this->{'id'} }{'latlng'} = '';
        $main::data{ $this->{'id'} }{'previewwidth'}
            = $main::data{ $this->{'id'} }{'previewheight'}
            = $main::data{ $this->{'id'} }{'previewtype'} = ''
            if $this->{'preview'};
        undef $path;
    }
    else {
        (   $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'},
            $main::data{ $this->{'id'} }{'name'}
            )
            = $this->get_field_file_info(
            $path,
            $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'},
            $main::data{ $this->{'id'} }{'name'},
            'preview',
            0,
            $main::data{ $this->{'id'} }{'latlng'},
            );
        if ( $this->{'imagesize'} ) {
            $main::data{ $this->{'id'} }{'width'} = $width
                if $width && !$main::data{ $this->{'id'} }{'width'};
            $main::data{ $this->{'id'} }{'height'} = $height
                if $height && !$main::data{ $this->{'id'} }{'height'};
        }
        if ( $this->{'preview'} ) {
            (   $main::data{ $this->{'id'} }{'previewwidth'},
                $main::data{ $this->{'id'} }{'previewheight'},
                $main::data{ $this->{'id'} }{'previewtype'}
                )
                = $this->get_field_file_info(
                $path,
                $main::data{ $this->{'id'} }{'previewwidth'},
                $main::data{ $this->{'id'} }{'previewheight'},
                $main::data{ $this->{'id'} }{'previewtype'},
                'preview',
                '',
                1
                );
        }
    }

    #添付ファイル情報の取得
    if (   $main::clipboard_id
        && !$main::data{ $this->{'id'} }{'file'}
        && !$main::option{ $this->{'id'} }{'cancel'} )
    {
        $cgi::input->inherit_posted_file_properties($name);
    }
    my %properties
        = $cgi::input->get_posted_file_properties_from_param($name);
    if ( $properties{'delete'} ) {
        delete $properties{'delete'};
        $cgi::input->set_posted_file_properties_from_param( $name, 0,
            \%properties );
    }
    my %preview_properties = ();
    if ( defined $this->{'preview'} && $this->{'preview'} ) {
        if (   $main::clipboard_id
            && !$main::data{ $this->{'id'} }{'preview'}
            && !$main::option{ $this->{'id'} }{'previewcancel'} )
        {
            $cgi::input->inherit_posted_file_properties($preview_name);
        }
        %preview_properties
            = $cgi::input->get_posted_file_properties_from_param(
            $preview_name);
    }
    if ( $preview_properties{'delete'} ) {
        delete $preview_properties{'delete'};
        $cgi::input->set_posted_file_properties_from_param( $preview_name, 0,
            \%preview_properties );
    }

    #URLからデータを取得
    if ( !$properties{'filename'} && $main::data{ $this->{'id'} }{'url'} ) {
        if ( $main::data{ $this->{'id'} }{'url'}
            =~ /^https?:\/\/(?:www\.youtube\.com\/watch\?.*?v=|youtu\.be\/)([0-9a-zA-Z_\-]+)/is
            )
        {
            $main::data{ $this->{'id'} }{'url'}
                = 'http://i1.ytimg.com/vi/' . $1 . '/sddefault.jpg';
        }
        if ( $this->is_invalid_url( $main::data{ $this->{'id'} }{'url'} ) ) {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . ' 外部コンテンツの読み込みに失敗しました。URL を確認してください。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'url'} = 1;
            $is_valid = 0;
        }
        else {
            %properties = ();
            my $path = $cgi::input->set_clipboard(
                'data:' . $this->{'id'} . ':file', 0 );
            require CMONOS::CGI_Lib::HTTP_Client;
            my $res = HTTP_Client->new->request(
                'GET' => $this->image_url_encode(
                    $main::data{ $this->{'id'} }{'url'}
                ),
                {},
                '', '', '',
                $path . '/data'
            );
            if ( $res->is_success ) {
                my $url = $res->url || 'image.jpg';
                $url =~ s/\#.*?$//;
                $url =~ s/\?.*?$//;
                $url =~ s/\.jpeg/.jpg/i;
                my $url_ext = ( $url =~ /\.([0-9a-zA-Z_]+)$/ ) ? lc $1 : '';
                $properties{'name'} = $properties{'filename'}
                    = $main::data{ $this->{'id'} }{'file'}
                    = ( $url =~ /([^\/]+)$/ ) ? $1 : '';
                $properties{'type'} = $Common::mime_type{$url_ext}
                    || 'application/octet-stream';
                $cgi::input->set_posted_file_properties( $path,
                    \%properties );
                delete $main::data{ $this->{'id'} }{'url'};
            }
            elsif ( $main::data{ $this->{'id'} }{'url'}
                =~ s!(http://i1\.ytimg\.com/vi/.*?)/sddefault\.jpg!$1/hqdefault.jpg!
                )
            {
                $res = HTTP_Client->new->request(
                    'GET' => $this->image_url_encode(
                        $main::data{ $this->{'id'} }{'url'}
                    ),
                    {},
                    '', '', '',
                    $path . '/data'
                );
                if ( $res->is_success ) {
                    $properties{'name'} = $properties{'filename'}
                        = $main::data{ $this->{'id'} }{'file'}
                        = 'default.jpg';
                    $properties{'type'} = $Common::mime_type{'jpg'};
                    $cgi::input->set_posted_file_properties( $path,
                        \%properties );
                    delete $main::data{ $this->{'id'} }{'url'};
                }
                elsif ( $main::data{ $this->{'id'} }{'url'}
                    =~ s!(http://i1\.ytimg\.com/vi/.*?)/hqdefault\.jpg!$1/mqdefault.jpg!
                    )
                {
                    $res = HTTP_Client->new->request(
                        'GET' => $this->image_url_encode(
                            $main::data{ $this->{'id'} }{'url'}
                        ),
                        {},
                        '', '', '',
                        $path . '/data'
                    );
                    if ( $res->is_success ) {
                        $properties{'name'} = $properties{'filename'}
                            = $main::data{ $this->{'id'} }{'file'}
                            = 'default.jpg';
                        $properties{'type'} = $Common::mime_type{'jpg'};
                        $cgi::input->set_posted_file_properties( $path,
                            \%properties );
                        delete $main::data{ $this->{'id'} }{'url'};
                    }
                }
            }
            else {
                push(
                    @{ $cgi::action->{'dialog'} },
                    ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                        . ' 外部コンテンツの読み込みに失敗しました。'
                ) if ref $cgi::action;
                $this->{'is_invalid_field'}{'url'} = 1;
                $is_valid = 0;
            }
        }
    }
    elsif ( defined $main::data{ $this->{'id'} }{'url'} ) {
        delete $main::data{ $this->{'id'} }{'url'};
    }

    #添付ファイルの処理
    if ( $properties{'filename'} ) {
        my ( $is_image, @stat );

        #添付ファイルが画像ファイルかどうかチェック
        @stat = $cgi::Image->input_info(
            $cgi::input,            $name,
            0,                      $this->{'widthlimit'},
            $this->{'heightlimit'}, 1,
            \%properties,           $this->{'clearAllProfiles'},
            $this->{'maxsize'}
        );
        if ( $stat[2] ) {
            ( $width, $height, $ext ) = @stat;
            if ( ref $stat[4] ) {
                my ( $lat, $lng ) = $cgi::Image->latlng_from_exif( $stat[4] );
                $latlng = $lat . ',' . $lng if $lat && $lng;
            }
            $is_image = 1;
        }
        elsif ( $properties{'filename'} =~ /\.([0-9a-zA-Z_]+)$/ ) {
            $ext = lc $1;
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( $this->{'name'} )
                    ? $this->{'name'}
                    : '添付ファイル'
                    )
                    . 'を確認してください。適切な拡張子をつけてください。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'file'} = 1;
            $is_valid = 0;
        }

        #ファイルサイズチェック
        if (   $this->{'maxsize'}
            && $cgi::input->get_posted_file_size_from_param($name)
            > $this->{'maxsize'} )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( $this->{'name'} )
                    ? $this->{'name'}
                    : '添付ファイル'
                    )
                    . 'のファイルサイズを確認してください。'
                    . $this->{'maxsize'}
                    . 'バイト以内にしてください。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'file'} = 1;
            $is_valid = 0;
        }

        if ( defined $this->{'anyfile'} && $this->{'anyfile'} ) {

            #何もしない
        }
        elsif ( ref $this->{'type_ext'} eq 'HASH'
            && %{ $this->{'type_ext'} } )
        {
            #MIME-TYPEチェック
            if ( !defined $this->{'type_ext'}{ $properties{'type'} } ) {
                my $mimename = '';
                foreach my $e ( sort keys %{ $this->{'mimename'} } ) {
                    $mimename
                        .= ( ($mimename) ? ',' : '' )
                        . $this->{'mimename'}{$e}
                        . ( ( $this->{'mimename'}{$e} ne $e )
                        ? '(' . $e . ')'
                        : '' );
                }
                push(
                    @{ $cgi::action->{'dialog'} },
                    (   ( $this->{'name'} )
                        ? $this->{'name'}
                        : '添付ファイル'
                        )
                        . 'のファイル形式を確認してください。下記形式のみ許可されています。'
                        . $mimename
                ) if ref $cgi::action;
                $this->{'is_invalid_field'}{'file'} = 1;
                $is_valid = 0;
            }
            else {
                $properties{'filename'}
                    =~ s/\.[0-9a-zA-Z_]+$/\.$this->{'type_ext'}{$properties{'type'}}/;
            }
        }
        elsif ( !$is_image ) {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( $this->{'name'} )
                    ? $this->{'name'}
                    : '添付ファイル'
                    )
                    . 'は、Webに適した画像ではありません。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'file'} = 1;
            $is_valid = 0;
        }

        #ファイル名
        $file_name = $this->z_to_h( $properties{'filename'}, 'utf8' );
        $file_name =~ s/\.[0-9a-zA-Z_]+?$//s;
        $file_name =~ s/[\x09\x0A\x0C\x0D\x20\.\-\+]+/_/gs;
        $file_name = ''
            if $file_name =~ /[^0-9a-zA-Z_]/ || $file_name eq 'preview';

        #プレビュー自動作成
        if (   $this->{'preview'}
            && $env::clipboard
            && $cgi::Image->image_magick_enabled
            && $is_image
            && ( $main::data{ $this->{'id'} }{'deletepreview'}
                || !$main::data{ $this->{'id'} }{'previewtype'} )
            && !$cgi::input->get_posted_file_size_from_param($preview_name)
            && (   ( $previewwidthlimit && $width > $previewwidthlimit )
                || ( $previewheightlimit && $height > $previewheightlimit ) )
            )
        {
            %preview_properties
                = $cgi::input->clone_posted_file( $name, $preview_name );
        }

        if ($is_valid) {
            if ( !defined $main::data{ $this->{'id'} }{'title'}
                || $main::data{ $this->{'id'} }{'title'} eq '' )
            {
                my $title = $this->decode_utf8( $properties{'filename'} );
                $title =~ s/\.[a-zA-Z0-9]+$//s;
                $title =~ s/[\x0A\x0D]//gs;
                $main::data{ $this->{'id'} }{'title'} = $title;
            }
        }
        else {
            $cgi::input->remove_posted_file($name);
            undef %properties;
        }

    }
    else {
        ( $width, $height, $ext, $file_name, $latlng ) = (
            $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'},
            $main::data{ $this->{'id'} }{'name'},
            $main::data{ $this->{'id'} }{'latlng'}
        );
        $file_name = ''
            if $file_name =~ /[^0-9a-zA-Z_]/ || $file_name eq 'preview';
        if (   $ext
            && ref $this->{'ext_type'} eq 'HASH'
            && %{ $this->{'ext_type'} }
            && !defined $this->{'ext_type'}{$ext} )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( $this->{'name'} )
                    ? $this->{'name'}
                    : '添付ファイル'
                    )
                    . 'のファイルタイプを確認してください。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'file'} = 1;
            $is_valid = 0;
        }
    }

    #画像サイズチェック
    if ( $this->{'widthlimit'} && $width && $width > $this->{'widthlimit'} ) {
        if ( $properties{'filename'}
            || !grep( $_ eq $ext, 'gif', 'png', 'jpg' ) )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( $this->{'name'} )
                    ? $this->{'name'}
                    : '添付ファイル'
                    )
                    . 'の横幅を確認してください。'
                    . $this->{'widthlimit'}
                    . 'ピクセル以内にしてください。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'file'}
                = $this->{'is_invalid_field'}{'width'} = 1;
            $is_valid = 0;
        }
        elsif ( $env::clipboard && $cgi::Image->image_magick_enabled ) {
            ( $width, $height )
                = $this->field_image_fit( $path, $width, $height, $file_name,
                $ext, $this->{'widthlimit'}, $this->{'heightlimit'} );
        }
    }
    if (   $this->{'heightlimit'}
        && $height
        && $height > $this->{'heightlimit'} )
    {
        if ( $properties{'filename'}
            || !grep( $_ eq $ext, 'gif', 'png', 'jpg' ) )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( $this->{'name'} )
                    ? $this->{'name'}
                    : '添付ファイル'
                    )
                    . 'の高さを確認してください。'
                    . $this->{'heightlimit'}
                    . 'ピクセル以内にしてください。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'file'}
                = $this->{'is_invalid_field'}{'height'} = 1;
            $is_valid = 0;
        }
        elsif ( $env::clipboard && $cgi::Image->image_magick_enabled ) {
            ( $width, $height )
                = $this->field_image_fit( $path, $width, $height, $file_name,
                $ext, $this->{'widthlimit'}, $this->{'heightlimit'} );
        }
    }

    #必須項目
    if ( $this->{'required'} && ( !defined $ext || $ext eq '' ) ) {
        push(
            @{ $cgi::action->{'dialog'} },
            ( ( $this->{'name'} ) ? $this->{'name'} : '添付ファイル' )
                . 'を指定してください。'
        ) if ref $cgi::action;
        $this->{'is_invalid_field'}{'file'} = 1;
        $is_valid = 0;
    }

    #サムネイル画像の処理
    if ( defined $this->{'preview'} && $this->{'preview'} ) {
        if ( $preview_properties{'filename'} ) {
            my $is_valid_preview = 1;

            #添付ファイルが画像ファイルかどうかチェック
            my @stat
                = $cgi::Image->input_info( $cgi::input, $preview_name, 0,
                $previewwidthlimit, $previewheightlimit, 1,
                \%preview_properties, 1 );
            if ( @stat > 2 ) {
                ( $preview_width, $preview_height, $preview_ext ) = @stat;
            }
            else {
                push(
                    @{ $cgi::action->{'dialog'} },
                    (   ( $this->{'name'} )
                        ? $this->{'name'}
                        : '添付ファイル'
                        )
                        . 'のサムネイル画像（プレビュー）を確認してください。Web に適した画像形式ではありません。'
                ) if ref $cgi::action;
                $this->{'is_invalid_field'}{'preview'} = 1;
                $is_valid_preview = $is_valid = 0;
            }

            #ファイルサイズチェック
            if (   $this->{'maxsize'}
                && $cgi::input->get_posted_file_size_from_param($preview_name)
                > $this->{'maxsize'} )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    (   ( $this->{'name'} )
                        ? $this->{'name'}
                        : '添付ファイル'
                        )
                        . 'のサムネイル画像（プレビュー）ファイルサイズを確認してください。'
                        . $this->{'maxsize'}
                        . 'バイト以内にしてください。'
                ) if ref $cgi::action;
                $this->{'is_invalid_field'}{'preview'} = 1;
                $is_valid_preview = $is_valid = 0;
            }

            if ( !$is_valid_preview ) {
                $cgi::input->remove_posted_file($preview_name);
                undef %preview_properties;
            }

        }
        else {
            ( $preview_width, $preview_height, $preview_ext ) = (
                $main::data{ $this->{'id'} }{'previewwidth'},
                $main::data{ $this->{'id'} }{'previewheight'},
                $main::data{ $this->{'id'} }{'previewtype'}
            );
            if ( $preview_ext && $preview_ext !~ /^(gif|jpg|jpeg|png)$/ ) {
                push(
                    @{ $cgi::action->{'dialog'} },
                    (   ( $this->{'name'} )
                        ? $this->{'name'}
                        : '添付ファイル'
                        )
                        . 'のサムネイル画像（プレビュー）を確認してください。Web に適した画像形式ではありません。'
                ) if ref $cgi::action;
                $this->{'is_invalid_field'}{'preview'} = 1;
                $is_valid = 0;
            }
        }

        #画像サイズチェック
        if (   $previewwidthlimit
            && $preview_width
            && $preview_width > $previewwidthlimit )
        {
            if ( $preview_properties{'filename'} ) {
                push(
                    @{ $cgi::action->{'dialog'} },
                    (   ( $this->{'name'} )
                        ? $this->{'name'}
                        : '添付ファイル'
                        )
                        . 'のサムネイル画像（プレビュー）横幅を確認してください。'
                        . $this->{'previewwidthlimit'}
                        . 'ピクセル以内にしてください。'
                ) if ref $cgi::action;
                $this->{'is_invalid_field'}{'preview'} = 1;
                $is_valid = 0;
            }
            elsif ( $env::clipboard && $cgi::Image->image_magick_enabled ) {
                ( $preview_width, $preview_height )
                    = $this->field_image_fit( $path, $preview_width,
                    $preview_height, 'preview', $preview_ext,
                    $previewwidthlimit, $previewheightlimit );
            }
        }
        if (   $previewheightlimit
            && $preview_height
            && $preview_height > $previewheightlimit )
        {
            if ( $preview_properties{'filename'} ) {
                push(
                    @{ $cgi::action->{'dialog'} },
                    (   ( $this->{'name'} )
                        ? $this->{'name'}
                        : '添付ファイル'
                        )
                        . 'のサムネイル画像（プレビュー）高さを確認してください。'
                        . $this->{'previewheightlimit'}
                        . 'ピクセル以内にしてください。'
                ) if ref $cgi::action;
                $this->{'is_invalid_field'}{'preview'} = 1;
                $is_valid = 0;
            }
            elsif ( $env::clipboard && $cgi::Image->image_magick_enabled ) {
                ( $preview_width, $preview_height )
                    = $this->field_image_fit( $path, $preview_width,
                    $preview_height, 'preview', $preview_ext,
                    $previewwidthlimit, $previewheightlimit, 1 );
            }
        }
    }

    #コメントチェック
    if ( $this->{'comment'} ) {
        $is_valid = 0 if !$this->is_not_taboo('comment');
        $is_valid = 0 if !$this->is_not_too_long( 'comment', 'コメント' );
    }

    return 0 if !defined $is_valid || !$is_valid;

    #ファイル削除予約
    my @ignore_keys = ( 'file', 'delete', 'preview', 'deletepreview' );
    if (  !$this->{'required'}
        && $main::data{ $this->{'id'} }{'delete'}
        && !$properties{'filename'} )
    {
        push( @ignore_keys, 'width', 'height', 'type', 'name', 'latlng' );
        $cgi::input->parameters_trash(
            \$main::data{ $this->{'id'} }{'width'},
            \$main::data{ $this->{'id'} }{'height'},
            \$main::data{ $this->{'id'} }{'type'},
            \$main::data{ $this->{'id'} }{'name'},
            \$main::data{ $this->{'id'} }{'latlng'}
        );
    }
    else {
        $file_name = ''
            if $file_name =~ /[^0-9a-zA-Z_]/ || $file_name eq 'preview';
        (   $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'},
            $main::data{ $this->{'id'} }{'name'},
            $main::data{ $this->{'id'} }{'latlng'}
        ) = ( $width, $height, $ext, $file_name, $latlng );
    }
    if (!$this->{'preview'}
        || ( $main::data{ $this->{'id'} }{'deletepreview'}
            && !$preview_properties{'filename'} )
        )
    {
        push( @ignore_keys, 'previewwidth', 'previewheight', 'previewtype' );
        $cgi::input->parameters_trash(
            \$main::data{ $this->{'id'} }{'previewwidth'},
            \$main::data{ $this->{'id'} }{'previewheight'},
            \$main::data{ $this->{'id'} }{'previewtype'}
        );
    }
    else {
        (   $main::data{ $this->{'id'} }{'previewwidth'},
            $main::data{ $this->{'id'} }{'previewheight'},
            $main::data{ $this->{'id'} }{'previewtype'}
        ) = ( $preview_width, $preview_height, $preview_ext );
    }

    #データ整形
    $this->data(@ignore_keys);

    #ファイル保存/削除を予約
    if ( $properties{'filename'} ) {
        $mail::file .= '<' . $name . '>,'
            if !$default::static
            && ( $this->{'mail'} || $setting::field_for_mail )
            ;    #メール添付予約
        $cgi::input->save_file(
            $name,
            '!'
                . $this->{'id'}
                . (
                (          defined $main::data{ $this->{'id'} }{'name'}
                        && $main::data{ $this->{'id'} }{'name'} ne ''
                        && $main::data{ $this->{'id'} }{'name'}
                        !~ /[^0-9a-zA-Z_]/
                ) ? '.' . $main::data{ $this->{'id'} }{'name'} : ''
                )
                . '.'
                . $main::data{ $this->{'id'} }{'type'},
            '', '', '',
            $this->{'id'} . '(\\.(?!preview)[0-9a-zA-Z_]+)?'
        );
        $this->{'is_posted_file'} = 1;
    }
    elsif ( !$this->{'required'} && $main::data{ $this->{'id'} }{'delete'} ) {
        $properties{'delete'}
            = $this->{'id'} . '(\\.(?!preview)[0-9a-zA-Z_]+)?';
        $cgi::input->set_posted_file_properties_from_param( $name, 0,
            \%properties );
    }
    if ( $preview_properties{'filename'} ) {
        $cgi::input->save_file( $preview_name,
            '!' . $this->{'id'} . '.preview.' . $preview_ext,
            '', '', '', $this->{'id'} . '\\.preview' );
        $this->{'is_posted_preview'} = 1;
    }
    elsif ( !$this->{'preview'}
        || $main::data{ $this->{'id'} }{'deletepreview'} )
    {
        $preview_properties{'delete'} = $this->{'id'} . '\\.preview';
        $cgi::input->set_posted_file_properties_from_param( $preview_name, 0,
            \%preview_properties );
    }
    delete $main::data{ $this->{'id'} }{'file'};
    delete $main::data{ $this->{'id'} }{'preview'};

    return 1;
}

sub field_image_fit ($$$$$$$$;$) {
    my ($this,       $path,        $width,
        $height,     $name,        $ext,
        $widthlimit, $heightlimit, $is_thumbnail
    ) = @_;
    my ( $w, $h )
        = $cgi::Image->image_fit( $width, $height, $widthlimit,
        $heightlimit );
    my $file_path
        = $path . '/'
        . $this->{'id'} . '.'
        . ( ( defined $name && $name ne '' ) ? $name . '.' : '' )
        . $ext;
    if ( $cgi::Image->image_file_resize( $file_path, $w, $h, $is_thumbnail ) )
    {
        $width  = $w;
        $height = $h;
    }
    return ( $width, $height );
}

######################################################################
#	(4) obj->usual_format(str);
sub usual_format {
    my ( $this, $str, $path, $c, $n ) = @_;
    return $this->field_tag($str) if !defined $str || $str eq '';

    my ( %data, $align, $div );
    %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    undef $str;
    $this->preview_complement( \%data, $path )
        if $default::static
        && $this->{'preview'}
        && $this->{'previewComplementEnabled'};
    return $this->image( $path, \%data, $this->{'option'} )
        if $this->{'icon_type'};
    return $this->field_tag( $this->data_tag_from_ref( \%data ), 1 )
        if ( !defined $data{'type'} || $data{'type'} eq '' )
        && ( !defined $data{'comment'} || $data{'comment'} eq '' );

    #コメント
    if ( defined $this->{'comment'} && $this->{'comment'} ) {

        #画像タグ
        my $image
            = $this->image( $path, \%data, $this->{'option'}, $data{'align'},
            $data{'valign'} );

        #コメント調整
        $this->text_conv( \$data{'comment'} );
        $this->text_fix( \$data{'comment'}, $this->{'csv'} );
        $str = $this->data_tag( 'comment', $data{'comment'} );

        if (   !defined $data{'valign'}
            || $data{'valign'} eq ''
            || $data{'valign'} eq 'middle'
            || !defined $image
            || $image eq '' )
        {
            $str = $this->style(
                $image . $str,       $this->{'style'},
                $this->{'fontsize'}, $this->{'fontcolor'},
                $this->{'fontface'}
            );
        }
        else {
            $str = $this->style( $str, $this->{'style'}, $this->{'fontsize'},
                $this->{'fontcolor'}, $this->{'fontface'} );
            if ( $data{'valign'} eq 'top' ) {
                $str = $image . $str;
            }
            elsif ( $data{'valign'} eq 'bottom' ) {
                $str .= $image;
            }
        }
    }
    else {
        $str = $this->image( $path, \%data, $this->{'option'} );
    }
    if ( $this->{'is_temporary_data'} ) {
        if (   $main::data{ $this->{'id'} }{'delete'}
            && !$this->{'required'}
            && !$this->{'is_posted_file'} )
        {
            $str
                .= '<div class="note">このファイルを削除する</div>';
        }
        if ( $main::data{ $this->{'id'} }{'deletepreview'}
            && !$this->{'is_posted_preview'} )
        {
            $str
                .= '<div class="note">このサムネイル画像を削除する</div>';
        }
    }
    return $this->field_tag(
        $this->data_tag_from_ref( \%data, 'comment' ) . $str );
}

sub image_file_url ($$$$) {
    my ( $this, $str, $path, $c, $n ) = @_;
    return '' if !$str;
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    my $url = '';
    return '' if !$data{'type'};
    if ( $data{'type'} && grep( $_ eq $data{'type'}, 'gif', 'jpg', 'png' ) ) {
        $url
            = $path . '/'
            . $this->{'id'} . '.'
            . (
            (          defined $data{'name'}
                    && $data{'name'} ne ''
                    && $data{'name'} !~ m!/!
            ) ? $data{'name'} . '.' : ''
            ) . $data{'type'};
    }
    elsif ( $data{'previewtype'} ) {
        $url
            = $path . '/'
            . $this->{'id'}
            . '.preview.'
            . $data{'previewtype'};
    }
    return ($url) ? $this->full_path($url) : '';
}

sub preview_complement ($$$) {
    my ( $this, $data, $path ) = @_;
    return 1
        if !$data->{'type'}
        || $data->{'previewtype'}
        || !$cgi::Image->image_magick_enabled
        || ( $data->{'type'} ne 'jpg'
        && $data->{'type'} ne 'gif'
        && $data->{'type'} ne 'png' );
    my $file_path
        = $path . '/'
        . $this->{'id'} . '.'
        . (
        (          defined $data->{'name'}
                && $data->{'name'} ne ''
                && $data->{'name'} !~ m!/!
        ) ? $data->{'name'} . '.' : ''
        ) . $data->{'type'};
    my $preview_path
        = $path . '/' . $this->{'id'} . '.preview.' . $data->{'type'};
    my $previewwidthlimit
        = ( $this->{'previewwidthlimit'} ) ? $this->{'previewwidthlimit'}
        : ( $this->{'maxwidth'} ) ? $this->{'maxwidth'}
        :                           '';
    my $previewheightlimit
        = ( $this->{'previewheightlimit'} ) ? $this->{'previewheightlimit'}
        : ( $this->{'maxheight'} ) ? $this->{'maxheight'}
        :                            '';
    return 1 if !$previewwidthlimit && !$previewheightlimit;
    my ( $width, $height ) = $cgi::Image->info($file_path);

    if (   ( $width && $previewwidthlimit && $width > $previewwidthlimit )
        || ( $height && $previewheightlimit && $height > $previewheightlimit )
        )
    {
        ( $width, $height )
            = $cgi::Image->image_fit( $width, $height, $previewwidthlimit,
            $previewheightlimit );
        if ( $width && $height ) {
            my $magick = Image::Magick->new;
            $magick->Read($file_path);
            $magick->Resize( 'width' => $width, 'height' => $height );
            $magick->Write($preview_path);
            undef $magick;
            if ( -s $preview_path ) {
                (   $data->{'previewwidth'},
                    $data->{'previewheight'},
                    $data->{'previewtype'}
                ) = ( $width, $height, $data->{'type'} );
                return 1;
            }
            else {
                unlink $preview_path;
                return 0;
            }
        }
    }
    return 0;
}

sub has_content {
    my ( $this, $str ) = @_;
    return 0 if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return (
        %data
            && ( ( defined $data{'type'} && $data{'type'} ne '' )
            || ( defined $data{'comment'} && $data{'comment'} ne '' ) )
    ) ? 1 : 0;
}

#---------------------------------------------------------------------
#	(4)-3 obj->format_for_detail(str);
sub format_for_detail {
    my ( $this, $str, @data ) = @_;
    local $this->{'thumbnail'} = 'no' if $this->{'nopreviewdetail'};
    return $this->usual_format( $str, @data );
}

#---------------------------------------------------------------------
#	(4) obj->format_for_mail(str);
sub format_for_mail {
    my ( $this, $str ) = @_;
    return '' if !defined $str || $str eq '';
    my (%data);
    %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return '' if !$data{'type'} && !$data{'comment'};
    $str
        = '['
        . $this->name . "]\x0A"
        . ( ( $data{'type'} )
        ? '(添付ファイル'
            . ( ( $data{'title'} ) ? ':' . $data{'title'} : '' ) . ')'
        : '' )
        . ( ( defined $data{'comment'} )
        ? "\x0A" . $this->blockquote_to_quotation( $data{'comment'}, 70 )
        : '' );
    $str =~ s/[\x0A]*\Z/\x0A\x0A/s;
    return $str;
}

#---------------------------------------------------------------------
#	(4)-4 obj->format_for_text(str);
sub format_for_text {
    my ( $this, $str ) = @_;
    return '' if !defined $str || $str eq '';
    my (%data);
    %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return '' if !$data{'type'} && !$data{'comment'};
    return $this->blockquote_to_quotation( $data{'comment'},
        ( ( $this->{'cols'} ) ? $this->{'cols'} : 40 ) )
        . ( ( $data{'type'} )
        ? ' (添付ファイル有り'
            . ( ( $data{'title'} ) ? ':' . $data{'title'} : '' ) . ')'
        : '' );
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled, $tips ) = @_;
    my ($name);

    #名前
    $name = 'data:' . $this->{'id'};

    $this->print_field_info_form($name);
    print '<fieldset>';
    print '<legend>入力欄設定</legend>';
    print
        '<div class="hints">画像サイズが制限値を超える場合、画像添付を拒絶します。'
        . ( ( !$html_disabled )
        ? 'また、画像サイズが最大値を超える場合、表示サイズが最大幅に縮小されます。'
        : '' )
        . '</div>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_maxsize">ファイルサイズ制限</label></td>';
    print '<td class="field" colspan="3"><input type="text" name="'
        . $name
        . '_maxsize" id="'
        . $name
        . '_maxsize" value="'
        . $this->escape( $this->{'maxsize'} )
        . '" size="15" /> <span class="unit">byte</span><div class="hints">設定が無い場合、128,000byte（128KB）に設定されます。</div></td>';
    print '</tr>';

    if ( !$html_disabled ) {
        print '<tr>';
        print '<td class="field-name"><label for="' . $name
            . '_maxwidth">最大横幅（画像）</label></td>';
        print '<td class="q-field"><input type="number" name="'
            . $name
            . '_maxwidth" id="'
            . $name
            . '_maxwidth" value="'
            . $this->escape( $this->{'maxwidth'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '<td class="field-name"><label for="' . $name
            . '_maxheight">最大高さ（画像）</label></td>';
        print '<td class="q-field"><input type="number" name="'
            . $name
            . '_maxheight" id="'
            . $name
            . '_maxheight" value="'
            . $this->escape( $this->{'maxheight'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '</tr>';
    }
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_widthlimit">横幅制限（画像）</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_widthlimit" id="'
        . $name
        . '_widthlimit" value="'
        . $this->escape( $this->{'widthlimit'} )
        . '" size="4" /> <span class="unit">ピクセル</span></td>';
    print '<td class="field-name"><label for="' . $name
        . '_heightlimit">高さ制限（画像）</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_heightlimit" id="'
        . $name
        . '_heightlimit" value="'
        . $this->escape( $this->{'heightlimit'} )
        . '" size="4" /> <span class="unit">ピクセル</span></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_defaultwidth">デフォルトの横幅</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_defaultwidth" id="'
        . $name
        . '_defaultwidth" value="'
        . $this->escape( $this->{'defaultwidth'} )
        . '" size="4" /> <span class="unit">ピクセル</span></td>';
    print '<td class="field-name"><label for="' . $name
        . '_defaultheight">デフォルトの高さ</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_defaultheight" id="'
        . $name
        . '_defaultheight" value="'
        . $this->escape( $this->{'defaultheight'} )
        . '" size="4" /> <span class="unit">ピクセル</span></td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print '<td class="field-name">オプション</td>';
    print '<td class="field" colspan="3"><ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_imagesize" value="1"'
        . ( ( defined $this->{'imagesize'} && $this->{'imagesize'} )
        ? ' checked="checked"'
        : '' )
        . ' /> サイズ欄を表示</label> (ブラウザで表示できるファイルにのみ適用されます)</li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_clearAllProfiles" value="1"'
        . (
        (   defined $this->{'clearAllProfiles'} && $this->{'clearAllProfiles'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 撮影情報(Exif)など画像付属情報をすべて削除する</label></li>'
        if $cgi::Image->image_magick_enabled;

    if ( !$default::static && !$setting::field_for_mail ) {
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_mail" value="1"'
            . ( ( defined $this->{'mail'} && $this->{'mail'} )
            ? ' checked="checked"'
            : '' )
            . ' /> 通知メールにファイルを添付する</label></li>';
    }
    if ( !$setting::field_for_mail ) {
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_dlDisabled" value="1"'
            . ( ( defined $this->{'dlDisabled'} && $this->{'dlDisabled'} )
            ? ' checked="checked"'
            : '' )
            . ' /> ダウンロードリンクを表示しない</label></li>';
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_imageViewerDisabled" value="1"'
            . (
            (   defined $this->{'imageViewerDisabled'}
                    && $this->{'imageViewerDisabled'}
            ) ? ' checked="checked"' : ''
            )
            . ' /> 画像サイズが最大値を超えたとき、画像をクリックすることで等倍画像表示ウィンドウを<strong>開かない</strong>。</label></li>';
    }
    print '</ol></td>';
    print '</tr>';
    print '</tbody>';
    if ( !$html_disabled ) {
        print '<tbody>';
        print '<tr class="tbody-end">';
        print '<td class="field-name"><label for="' . $name
            . '_option">タグオプション</label></td>';
        print '<td class="field" colspan="3"><input type="text" name="'
            . $name
            . '_option" id="'
            . $name
            . '_option" value="'
            . $this->escape( $this->{'option'} )
            . '" size="50" title="添付ファイルを埋め込む際のオプションを設定します。" /></td>';
        print '</tr>';
        print '</tbody>';
        print '<tbody>';
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><span class="datatitle">Google ドキュメントビューワ</span></td>';
        print '<td class="field" colspan="3">';
        print
            '<label title="アクセス制限がかかったページや、ローカルネットワーク上のページでは動作しませんのでご注意ください。"><input type="checkbox" name="'
            . $name
            . '_useGoogleDocumentViewer" value="1"'
            . ( ( $this->{'useGoogleDocumentViewer'} )
            ? ' checked="checked"'
            : '' )
            . ' /> <a href="https://docs.google.com/" target="_blank">Google ドキュメントビューワ</a>を使う。</label><div class="note">（サイズ欄が指定されている場合、または、「デフォルトの横幅」と「デフォルトの高さ」が設定されている場合のみ有効）</div></li>';
        print '</td>';
        print '</tr>';
        print '</tbody>';
        print '<tbody>';
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><span class="datatitle">MP3/MP4</span></td>';
        print '<td class="field" colspan="3">';
        print '<label><input type="checkbox" name="'
            . $name
            . '_useFlash" value="1"'
            . ( ( $this->{'useFlash'} ) ? ' checked="checked"' : '' )
            . ' /> <a href="http://sourceforge.net/projects/gsplayer/" target="_blank">GS Player</a> で MP3 と MP4 ファイルを表示する。</label></li>';
        print '</td>';
        print '</tr>';
        print '</tbody>';
        print '<tbody>';
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><label title="HTML5 の &lt;video&gt;/&lt;audio&gt; を使用するかどうか設定します。ブラウザによって再生できるファイル形式が異なりますので注意してください。"><input type="checkbox" name="'
            . $name
            . '_html5Enabled" value="1"'
            . ( ( defined $this->{'html5Enabled'} && $this->{'html5Enabled'} )
            ? ' checked="checked"'
            : '' )
            . ' />&lt;video&gt;/&lt;audio&gt;</label></td>';
        print '<td class="field" colspan="3"><ol class="checkbox">';
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_preloadDisabled" value="1"'
            . ( ( $this->{'preloadDisabled'} ) ? ' checked="checked"' : '' )
            . ' /> 動画・音声をあらかじめ読み込ま<strong>ない</strong>。</label></li>';
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_autoplayEnabled" value="1"'
            . ( ( $this->{'autoplayEnabled'} ) ? ' checked="checked"' : '' )
            . ' /> 動画・音声を自動再生する。</label></li>';
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_loopEnabled" value="1"'
            . ( ( $this->{'loopEnabled'} ) ? ' checked="checked"' : '' )
            . ' /> 動画・音声をループ再生する。</label></li>';
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_controlsDisabled" value="1"'
            . ( ( $this->{'controlsDisabled'} ) ? ' checked="checked"' : '' )
            . ' /> コントローラーを表示し<strong>ない</strong>。</label></li>';
        print '</ol></td>';
        print '</tr>';
        print '</tbody>';
    }
    print '<tbody>';
    print '<tr class="tbody-end">';
    print '<td class="field-name"><label for="' . $name
        . '_mimetype">MIME-TYPE 制限</label></td>';
    print '<td class="field" colspan="3">';
    print '<div class="extra-field"><label><input type="checkbox" name="'
        . $name
        . '_anyfile" value="1"'
        . ( ( defined $this->{'anyfile'} && $this->{'anyfile'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 全てのファイルを許可する</label></div>';
    print '<textarea name="'
        . $name
        . '_mimetype" id="'
        . $name
        . '_mimetype" cols="80" rows="6" wrap="off">'
        . $this->escape( $this->{'mimetype'} )
        . '</textarea>';
    $this->print_mimetype_menu( $name . '_mimetype' );
    print
        '<div class="hints">設定がない場合、Webに適した画像のみ受け付けます。</div>';
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '<tbody>';
    print '<tr class="tbody-end">';
    print '<td class="field-name"><label for="' . $name
        . '_titlefrom">タイトル<span class="note">(代替文字列)</span></label></td>';
    print '<td class="field" colspan="3">タイトルがないとき、';
    print '<select name="'
        . $name
        . '_titlefrom" id="'
        . $name
        . '_titlefrom">';
    print '<option value=""></option>';

    foreach my $id (
        sort {
            $this->{'field_obj'}->{$a}->{'ORDER'}
                <=> $this->{'field_obj'}->{$b}->{'ORDER'}
                || $a <=> $b
        } keys %{ $this->{'field_obj'} }
        )
    {
        next
            unless ref $this->{'field_obj'}->{$id}
            && $this->{'field_obj'}->{$id}->{'class'} =~ /^text/
            && ( !$this->{'field_obj'}->{$id}->{'rows'}
            || $this->{'field_obj'}->{$id}->{'rows'} == 1 );
        print '<option value="'
            . $id . '"'
            . (
            ( defined $this->{'titlefrom'} && $this->{'titlefrom'} eq $id )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $this->jfold( $this->{'field_obj'}->{$id}->name, 32, 'I', 1 )
            . '</option>';
    }
    print '</select>からタイトルを作成する';
    print '<div class="extra-field"><label><input type="checkbox" name="'
        . $name
        . '_usertitledisabled" value="1"'
        . ( ( $this->{'usertitledisabled'} ) ? ' checked="checked"' : '' )
        . ' /> 常にこのフィールドの値をタイトルにする。</label></div>';
    print '</td>';
    print '</tr>';
    print '</tbody>';
    if ( !$html_disabled ) {
        print '<tbody>';
        print '<tr>';
        print '<td class="field-name"><label><input type="checkbox" name="'
            . $name
            . '_preview" id="'
            . $name
            . '_preview" value="1"'
            . ( ( defined $this->{'preview'} && $this->{'preview'} )
            ? ' checked="checked"'
            : '' )
            . ' /> サムネイル画像</label></td>';
        print
            '<td class="field" colspan="3"><label><input type="checkbox" name="'
            . $name
            . '_nopreviewdetail" id="'
            . $name
            . '_nopreviewdetail" value="1"'
            . (
            (   defined $this->{'nopreviewdetail'}
                    && $this->{'nopreviewdetail'}
            ) ? ' checked="checked"' : ''
            )
            . ' /> 一覧でのみサムネイル画像を使用する。</label></td>';
        print '</tr>';
        print '<tr>';
        print '<td class="field-name"><label for="' . $name
            . '_previewwidthlimit">横幅制限<span class="note">(サムネイル画像)</span></label></td>';
        print '<td class="q-field"><input type="number" name="'
            . $name
            . '_previewwidthlimit" id="'
            . $name
            . '_previewwidthlimit" value="'
            . $this->escape( $this->{'previewwidthlimit'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '<td class="field-name"><label for="' . $name
            . '_previewheightlimit">高さ制限<span class="note">(サムネイル画像)</span></label></td>';
        print '<td class="q-field"><input type="number" name="'
            . $name
            . '_previewheightlimit" id="'
            . $name
            . '_previewheightlimit" value="'
            . $this->escape( $this->{'previewheightlimit'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '</tr>';

        if ( $default::static && $cgi::Image->image_magick_enabled ) {
            print '<tr>';
            print
                '<td class="field-name"><span class="datatitle">画像補完</span></td>';
            print
                '<td class="field" colspan="3"><label><input type="checkbox" name="'
                . $name
                . '_previewComplementEnabled" id="'
                . $name
                . '_previewComplementEnabled" value="1"'
                . ( ( $this->{'previewComplementEnabled'} )
                ? ' checked="checked"'
                : '' )
                . ' /> 書き出し時にサムネイル画像がみつからないとき、アップロードした(画像)ファイルから作成する。</label></td>';
            print '</tr>';
        }
        print '</tbody>';
    }
    print '</table>';
    print '</fieldset>';
    if ( !$html_disabled ) {
        print '<fieldset>';
        print '<legend><label for="' . $name
            . '_rel">画像リンクの rel 属性</label></legend>';
        print '<input type="text" name="'
            . $name
            . '_rel" id="'
            . $name
            . '_rel" value="'
            . $this->escape( $this->{'rel'} )
            . '" size="16" />';
        print
            '<div class="note">lightbox 等を使う場合設定します。例）&quot;lightbox[picture]&quot;</div></fieldset>';
    }

    print '<fieldset>';
    print '<legend><label><input type="checkbox" name="'
        . $name
        . '_importEnabled" value="1"'
        . ( ( defined $this->{'importEnabled'} && $this->{'importEnabled'} )
        ? ' checked="checked"'
        : '' )
        . ' onclick="cmonos.element.item(\'import-settings\').disappear(!this.checked);" /> 外部コンテンツ読み込み</label></legend>';
    print '<table class="field-table" id="import-settings">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_photoSharingServiceName">写真共有サービス</label></td>';
    print '<td class="q-field">';
    print $this->mk_select_menu(
        $name . '_photoSharingServiceName', '', '',
        $this->{'photoSharingServiceName'},
        'TwitPic'     => 'TwitPic',
        'Twitgoo'     => 'Twitgoo',
        'Mobypicture' => 'Mobypicture',
        'img.ly'      => 'img.ly',
        'flickr'      => 'flickr',
        'hatena'      => 'はてな・フォトライフ',
    );
    print '</td>';
    print '<td class="field-name"><label for="' . $name
        . '_photoSharingServiceAccount">アカウント</label></td>';
    print '<td class="q-field"><input type="text" name="'
        . $name
        . '_photoSharingServiceAccount" id="'
        . $name
        . '_photoSharingServiceAccount" value="'
        . $this->escape( $this->{'photoSharingServiceAccount'} )
        . '" size="20" title="写真共有サービスのアカウントを設定します。" /></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_photoSharingServiceRSS">その他の RSS URL</label></td>';
    print '<td class="field" colspan="3"><input type="text" name="'
        . $name
        . '_photoSharingServiceRSS" id="'
        . $name
        . '_photoSharingServiceRSS" value="'
        . $this->escape( $this->{'photoSharingServiceRSS'} )
        . '" size="90" title="その他の写真共有サービス RSS。" />';
    print
        '<div class="hints">「写真共有サービス」メニューよりも「その他の RSS URL」が優先されます。</div></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';
    print $this->javascript(
        'cmonos.element.item(\'import-settings\').disappear(true);')
        if !$this->{'importEnabled'};

    print '<fieldset>';
    print '<legend>コメント欄設定</legend>';
    print '<label><input type="checkbox" name="'
        . $name
        . '_comment" value="1"'
        . ( ( defined $this->{'comment'} && $this->{'comment'} )
        ? ' checked="checked"'
        : '' )
        . ' onclick="cmonos.element.tagClass(\'div\',\'cmtsetting\',\'fieldset\',\'cmtsetting\').disappear(!this.checked);" /> コメント欄を追加する</label>';
    print '<div class="cmtsetting">';

    if ( !defined $default::static || !$default::static ) {
        print '<label><input type="checkbox" name="'
            . $name
            . '_noFileSearchForm" value="1"'
            . ( ( $this->{'noFileSearchForm'} ) ? ' checked="checked"' : '' )
            . ' /> 詳細条件でコメント検索欄のみ表示する</label>';
    }

    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_cols">入力欄横幅</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_cols" id="'
        . $name
        . '_cols" value="'
        . $this->escape( $this->{'cols'} )
        . '" size="3" title="入力欄の横幅を設定します。" /> <span class="unit">文字(半角)</span></td>';
    print '<td class="field-name"><label for="' . $name
        . '_rows">入力欄高さ</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_rows" id="'
        . $name
        . '_rows" value="'
        . $this->escape( $this->{'rows'} )
        . '" size="3" title="入力欄の高さを設定します。" /> <span class="unit">文字</span></td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print '<td class="field-name"><label for="' . $name
        . '_maxlength">最大文字数</label></td>';
    print '<td class="field" colspan="3"><input type="number" name="'
        . $name
        . '_maxlength" id="'
        . $name
        . '_maxlength" value="'
        . $this->escape( $this->{'maxlength'} )
        . '" size="3" title="コメントの文字数を制限します。" /> <span class="unit">文字(半角)</span></td>';
    print '</tr>';
    print '</tbody>';
    print '<tbody>';
    print '<tr class="tbody-end">';
    print '<td class="field-name"><label for="' . $name
        . '_default">デフォルト</label></td>';
    print '<td class="field" colspan="3"><textarea name="'
        . $name
        . '_default" cols="'
        . ( ( $this->{'cols'} ) ? $this->{'cols'} : '72' )
        . '" rows="'
        . ( ( $this->{'rows'} ) ? $this->{'rows'} : '4' )
        . '" wrap="soft">'
        . $this->escape( $this->{'default'} )
        . '</textarea>'
        . ( ( defined $tips ) ? $tips : '' ) . '</td>';
    print '</tr>';
    print '</tbody></table>';
    print '</div></fieldset>';
    print '<fieldset class="cmtsetting">';
    print '<legend>入力文字列制御(コメント欄)</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_taboowords">禁止文字列</label></td>';
    print '<td class="field" colspan="3"><textarea name="'
        . $name
        . '_taboowords" id="'
        . $name
        . '_taboowords" cols="72" rows="4" wrap="off">'
        . $this->escape( $this->{'taboowords'} )
        . '</textarea>';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . '_taboowords\').setAutoFit(this.checked);" />フィット</label>';
    print '<div class="extra-field"><label><input type="checkbox" name="'
        . $name
        . '_tabooregexp" value="1"'
        . ( ( defined $this->{'tabooregexp'} && $this->{'tabooregexp'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 正規表現を使用する</label></div>';
    print '<div class="note">禁止文字列ごとに改行</div>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_convtable">文字変換表</label></td>';
    print '<td class="field" colspan="3"><textarea name="'
        . $name
        . '_convtable" id="'
        . $name
        . '_convtable" cols="72" rows="4" wrap="off" title="下記書式に従って文字変換表を設定します。">'
        . $this->escape( $this->{'convtable'} )
        . '</textarea>';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . '_convtable\').setAutoFit(this.checked);" />フィット</label>';
    print '<div class="extra-field"><label><input type="checkbox" name="'
        . $name
        . '_convregexp" value="1"'
        . ( ( defined $this->{'convregexp'} && $this->{'convregexp'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 正規表現を使用する</label></div>';
    print
        '<div class="note">書式：変換対象文字列=&gt;変換後文字列[改行]</div>';
    print
        '<div class="example">例）、=>，（句読点の統一例）/SPAM=>spam（語句の統一例）</div>';
    print '</td>';
    print '</tr>';
    print '</tbody></table>';
    print '</fieldset>';

    if ( defined $html_disabled && $html_disabled ) {
        print '<input type="hidden" name="' . $name . '_csv" value="0" />';

    }
    else {
        print '<div class="cmtsetting">';

        #タグの処理
        $this->print_html_tag_setting_form( $name, '(コメント欄)' );

        #大文字と小文字を区別しない
        $this->print_ignorecase_setting_form( $name, '(コメント欄)' );

        #文字設定
        $this->print_font_setting_form( $name, '(コメント欄)' );

        #テーブル設定
        $this->print_table_setting_form( $name, '(コメント欄)' );

        print '</div>';
    }

    $this->print_setting_save_button;
    print $this->javascript(
        'cmonos.element.tagClass(\'div\',\'cmtsetting\',\'fieldset\',\'cmtsetting\').disappear(true);'
    ) if !$this->{'comment'};
    return;
}

sub print_mimetype_menu ($$) {
    my ( $this, $target ) = @_;
    print
        '<div class="toolbox"><select class="tool" onchange="cmonos.form.item(this.form).fieldname(\''
        . $target
        . '\').quotationByMenu(this)">';
    print '<option value="">▼MIME-TYPE 追加</option>';
    print '<option value="'
        . $this->jsquot(
        'image/jpeg:jpg:JPEG画像\nimage/pjpeg:jpg:JPEG画像\nimage/gif:gif:GIF画像\nimage/png:png:PNG画像\nimage/x-png:png:PNG画像\n'
        ) . '">Web に適した画像</option>';
    print '<option value="'
        . $this->jsquot('application/pdf:pdf:PDF\n')
        . '">PDF</option>';
    print '<option value="'
        . $this->jsquot('application/epub+zip:epub:ePub\n')
        . '">ePub</option>';
    print '<option value="'
        . $this->jsquot('video/mp4:mp4:MPEG4\n')
        . '">MPEG4</option>';
    print '<option value="'
        . $this->jsquot('audio/m4a:m4a:MPEG4音声\n')
        . '">MPEG4音声</option>';
    print '<option value="'
        . $this->jsquot('video/x-flv:flv:FLV\n')
        . '">FLV</option>';
    print '<option value="'
        . $this->jsquot('video/3gpp2:3g2:3G2\n')
        . '">3G2</option>';
    print '<option value="'
        . $this->jsquot('video/mp4:m4v:MPEG4\n')
        . '">MPEG4</option>';
    print '<option value="'
        . $this->jsquot('video/ogg:ogg:Ogg\n')
        . '">Ogg</option>';
    print '<option value="'
        . $this->jsquot('video/ogg:ogv:OggVideo\n')
        . '">OggVideo</option>';
    print '<option value="'
        . $this->jsquot('audio/ogg:oga:OggAudio\n')
        . '">OggAudio</option>';
    print '<option value="'
        . $this->jsquot('video/webm:webm:WebM\n')
        . '">WebM</option>';
    print '<option value="'
        . $this->jsquot('audio/mpeg:mp3:MP3\n')
        . '">MP3</option>';
    print '<option value="'
        . $this->jsquot('audio/aac:aac:AAC\n')
        . '">AAC</option>';
    print '<option value="'
        . $this->jsquot('audio/wav:wav:WAV\n')
        . '">WAV</option>';
    print '<option value="'
        . $this->jsquot('audio/midi:mid:MIDI\n')
        . '">MIDI</option>';
    print '<option value="'
        . $this->jsquot('video/mpeg:mpg:MPEG\n')
        . '">MPEG</option>';
    print '<option value="'
        . $this->jsquot('video/quicktime:mov:QuickTime動画\n')
        . '">QuickTime動画</option>';
    print '<option value="'
        . $this->jsquot('video/x-msvideo:avi:AVI\n')
        . '">AVI</option>';
    print '<option value="'
        . $this->jsquot('application/x-shockwave-flash:swf:Flash\n')
        . '">Flash</option>';
    print '<option value="'
        . $this->jsquot('application/lha:lzh:LZH\n')
        . '">LZH</option>';
    print '<option value="'
        . $this->jsquot(
        'application/x-compressed:zip:ZIP\napplication/x-zip-compressed:zip:ZIP\napplication/zip:zip:ZIP\nmultipart/x-zip:zip:ZIP\n'
        ) . '">ZIP</option>';
    print '<option value="'
        . $this->jsquot('application/x-compressed:tgz:TGZ\n')
        . '">TGZ</option>';
    print '<option value="'
        . $this->jsquot('application/x-stuffit:sit:sit\n')
        . '">sit</option>';
    print '<option value="'
        . $this->jsquot('application/x-macbinary:bin:bin\n')
        . '">bin</option>';
    print '<option value="'
        . $this->jsquot('application/mac-binhex40:hqx:hqx\n')
        . '">hqx</option>';
    print '<option value="'
        . $this->jsquot('audio/x-pn-realaudio:ra:ra\n')
        . '">ra</option>';
    print '<option value="'
        . $this->jsquot('image/pict:pct:PICT\nimage/pict:pict:PICT\n')
        . '">PICT</option>';
    print '<option value="'
        . $this->jsquot('image/tiff:tif:TIFF\n')
        . '">TIFF</option>';
    print '<option value="'
        . $this->jsquot('image/x-icon:ico:アイコンファイル\n')
        . '">アイコンファイル</option>';
    print '<option value="'
        . $this->jsquot('video/x-ms-wmv:wmv:WMV\n')
        . '">WMV</option>';
    print '<option value="'
        . $this->jsquot('video/x-ms-asf:asf:ASF\n')
        . '">ASF</option>';
    print '<option value="'
        . $this->jsquot('application/postscript:eps:EPS\n')
        . '">EPS</option>';
    print '<option value="'
        . $this->jsquot('application/postscript:ai:Illustrator\n')
        . '">Illustrator</option>';
    print '<option value="'
        . $this->jsquot('application/vnd.adobe.photoshop:psd:Photoshop\n')
        . '">Photoshop</option>';
    print '<option value="'
        . $this->jsquot('application/x-director:dcr:Director\n')
        . '">Director</option>';
    print '<option value="'
        . $this->jsquot('text/comma-separated-values:csv:CSV\n')
        . '">CSV</option>';
    print '<option value="'
        . $this->jsquot('text/plain:txt:TEXT\n')
        . '">TEXT</option>';
    print '<option value="'
        . $this->jsquot('text/xml:xml:XML\n')
        . '">XML</option>';
    print '<option value="'
        . $this->jsquot('text/xml:kml:KML\n')
        . '">KML</option>';
    print '<option value="'
        . $this->jsquot('text/html:html:HTML\ntext/html:htm:HTML\n')
        . '">HTML</option>';
    print '<option value="'
        . $this->jsquot('application/x-tex:tex:TeX\n')
        . '">TeX</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.ms-powerpoint:ppt:PowerPoint\napplication/vnd.openxmlformats-officedocument.presentationml.presentation:pptx:PowerPoint\n'
        ) . '">PowerPoint</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.ms-excel:xls:Excel\napplication/vnd.openxmlformats-officedocument.spreadsheetml.sheet:xlsx:Excel\n'
        ) . '">Excel</option>';
    print '<option value="'
        . $this->jsquot(
        'application/msword:doc:Word\napplication/vnd.openxmlformats-officedocument.wordprocessingml.document:docx:Word\n'
        ) . '">Word</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.oasis.opendocument.text:odt:OpenDocumentワープロ\n'
        ) . '">OpenDocumentワープロ</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.oasis.opendocument.spreadsheet:ods:OpenDocument表計算\n'
        ) . '">OpenDocument表計算</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.oasis.opendocument.presentation:odp:OpenDocumentプレゼンテーション\n'
        ) . '">OpenDocumentプレゼンテーション</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.oasis.opendocument.graphics:odg:OpenDocument図形\n'
        ) . '">OpenDocument図形</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.oasis.opendocument.formula:odf:OpenDocument数式\n')
        . '">OpenDocument数</option>';
    print '<option value="'
        . $this->jsquot(
        'application/vnd.oasis.opendocument.database:odb:OpenDocumentデータベース\n'
        ) . '">OpenDocumentデータベース</option>';
    print '</select>';
    print ' MIME-TYPE:拡張子:ファイルの種類[改行]</div>';
    print '<div class="example">例）image/jpeg:jpg:JPEG画像</div>';
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting {
    my $this     = shift;
    my $is_valid = 1;
    $is_valid = 0 if !$this->is_valid_text_setting;
    $is_valid = 0 if !$this->is_valid_file_setting;
    if (   defined $main::data{ $this->{'id'} . '_defaultwidth' }
        && $main::data{ $this->{'id'} . '_defaultwidth' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_defaultwidth' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'デフォルトの横幅(動画)設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_defaultwidth' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_defaultwidth' }
        && $main::data{ $this->{'id'} . '_maxwidth' }
        && $main::data{ $this->{'id'} . '_defaultwidth' }
        > $main::data{ $this->{'id'} . '_maxwidth' } )
    {
        $main::data{ $this->{'id'} . '_defaultwidth' }
            = $main::data{ $this->{'id'} . '_maxwidth' };
    }
    elsif ($main::data{ $this->{'id'} . '_defaultwidth' }
        && $main::data{ $this->{'id'} . '_widthlimit' }
        && $main::data{ $this->{'id'} . '_defaultwidth' }
        > $main::data{ $this->{'id'} . '_widthlimit' } )
    {
        $main::data{ $this->{'id'} . '_defaultwidth' }
            = $main::data{ $this->{'id'} . '_widthlimit' };
    }
    if (   defined $main::data{ $this->{'id'} . '_defaultheight' }
        && $main::data{ $this->{'id'} . '_defaultheight' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_defaultheight' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'デフォルトの高さ(動画)設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_defaultheight' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_defaultheight' }
        && $main::data{ $this->{'id'} . '_maxheight' }
        && $main::data{ $this->{'id'} . '_defaultheight' }
        > $main::data{ $this->{'id'} . '_maxheight' } )
    {
        $main::data{ $this->{'id'} . '_defaultheight' }
            = $main::data{ $this->{'id'} . '_maxheight' };
    }
    elsif ($main::data{ $this->{'id'} . '_defaultheight' }
        && $main::data{ $this->{'id'} . '_heightlimit' }
        && $main::data{ $this->{'id'} . '_defaultheight' }
        > $main::data{ $this->{'id'} . '_heightlimit' } )
    {
        $main::data{ $this->{'id'} . '_defaultheight' }
            = $main::data{ $this->{'id'} . '_heightlimit' };
    }

    #チェックボックス
    $main::data{ $this->{'id'} . '_preview' } = ''
        if !defined $main::data{ $this->{'id'} . '_preview' }
        || $main::data{ $this->{'id'} . '_preview' } eq '';
    $main::data{ $this->{'id'} . '_importEnabled' } = ''
        if !$main::data{ $this->{'id'} . '_importEnabled' };
    $main::data{ $this->{'id'} . '_comment' } = ''
        if !defined $main::data{ $this->{'id'} . '_comment' }
        || $main::data{ $this->{'id'} . '_comment' } eq '';
    $main::data{ $this->{'id'} . '_imagesize' } = ''
        if !defined $main::data{ $this->{'id'} . '_imagesize' }
        || $main::data{ $this->{'id'} . '_imagesize' } eq '';
    $main::data{ $this->{'id'} . '_anyfile' } = ''
        if !defined $main::data{ $this->{'id'} . '_anyfile' }
        || $main::data{ $this->{'id'} . '_anyfile' } eq '';
    $main::data{ $this->{'id'} . '_noFileSearchForm' } = ''
        if !defined $main::data{ $this->{'id'} . '_noFileSearchForm' }
        || $main::data{ $this->{'id'} . '_noFileSearchForm' } eq '';
    $main::data{ $this->{'id'} . '_dlDisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_dlDisabled' }
        || $main::data{ $this->{'id'} . '_dlDisabled' } eq '';
    $main::data{ $this->{'id'} . '_imageViewerDisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_imageViewerDisabled' }
        || $main::data{ $this->{'id'} . '_imageViewerDisabled' } eq '';
    $main::data{ $this->{'id'} . '_convregexp' } = ''
        if !defined $main::data{ $this->{'id'} . '_convregexp' }
        || $main::data{ $this->{'id'} . '_convregexp' } eq '';
    $main::data{ $this->{'id'} . '_tabooregexp' } = ''
        if !defined $main::data{ $this->{'id'} . '_tabooregexp' }
        || $main::data{ $this->{'id'} . '_tabooregexp' } eq '';
    $main::data{ $this->{'id'} . '_nopreviewdetail' } = ''
        if !defined $main::data{ $this->{'id'} . '_nopreviewdetail' }
        || $main::data{ $this->{'id'} . '_nopreviewdetail' } eq '';
    $main::data{ $this->{'id'} . '_usertitledisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_usertitledisabled' }
        || !$main::data{ $this->{'id'} . '_usertitledisabled' };
    $main::data{ $this->{'id'} . '_previewComplementEnabled' } = ''
        if $default::static
        && (
           !defined $main::data{ $this->{'id'} . '_previewComplementEnabled' }
        || !$main::data{ $this->{'id'} . '_previewComplementEnabled' } );
    $main::data{ $this->{'id'} . '_ignoreCase' } = ''
        if !$main::data{ $this->{'id'} . '_ignoreCase' };
    $main::data{ $this->{'id'} . '_useGoogleDocumentViewer' } = ''
        if !$main::data{ $this->{'id'} . '_useGoogleDocumentViewer' };
    $main::data{ $this->{'id'} . '_useFlash' } = ''
        if !$main::data{ $this->{'id'} . '_useFlash' };
    $main::data{ $this->{'id'} . '_html5Enabled' } = ''
        if !$main::data{ $this->{'id'} . '_html5Enabled' };
    $main::data{ $this->{'id'} . '_autoplayEnabled' } = ''
        if !$main::data{ $this->{'id'} . '_autoplayEnabled' };
    $main::data{ $this->{'id'} . '_preloadDisabled' } = ''
        if !$main::data{ $this->{'id'} . '_preloadDisabled' };
    $main::data{ $this->{'id'} . '_loopEnabled' } = ''
        if !$main::data{ $this->{'id'} . '_loopEnabled' };
    $main::data{ $this->{'id'} . '_controlsDisabled' } = ''
        if !$main::data{ $this->{'id'} . '_controlsDisabled' };
    $main::data{ $this->{'id'} . '_clearAllProfiles' } = ''
        if !$main::data{ $this->{'id'} . '_clearAllProfiles' };

    if (   defined $main::data{ $this->{'id'} . '_photoSharingServiceRSS' }
        && $main::data{ $this->{'id'} . '_photoSharingServiceRSS' } ne ''
        && $this->is_invalid_url(
            $main::data{ $this->{'id'} . '_photoSharingServiceRSS' }
        )
        )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'その他の写真共有サービス RSS URL を確認してください。'
        ) if ref $cgi::action;
        $is_valid = 0;
        delete $main::data{ $this->{'id'} . '_photoSharingServiceRSS' };
    }

    #タグ
    $this->detag( \$main::data{ $this->{'id'} . '_mimetype' } )
        if defined $main::data{ $this->{'id'} . '_mimetype' }
        && $main::data{ $this->{'id'} . '_mimetype' };
    if ( $main::data{ $this->{'id'} . '_photoSharingServiceAccount' } ) {
        $this->detag(
            \$main::data{ $this->{'id'} . '_photoSharingServiceAccount' } );
        $main::data{ $this->{'id'} . '_photoSharingServiceAccount' }
            =~ s![\x09\x0A\x0C\x0D\x20/]+!!gs;
    }

    $this->escape( \$main::data{ $this->{'id'} . '_rel' } )
        if defined $main::data{ $this->{'id'} . '_rel' }
        && $main::data{ $this->{'id'} . '_rel' };

    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    return $is_valid;
}

######################################################################
#	(7) obj->search_form;
sub search_form ($) {
    my $this = shift;
    my $form
        = '<tr><td class="field-name"><span class="datatitle">'
        . $this->{'name'}
        . '</span></td><td class="field">';

    if ( !$this->{'comment'} || !$this->{'noFileSearchForm'} ) {
        my ( $size, @mimename, $mime );
        @mimename = sort keys %{ $this->{'mimename'} }
            if ref $this->{'mimename'} && %{ $this->{'mimename'} };
        if (@mimename) {
            $size = @mimename;
            $size += 2;
            $size = 8 if $size > 8;
        }

        #デフォルト
        $this->search_ext;
        $form
            .= '<select name="search:'
            . $this->{'id'}
            . ':type"'
            . ( ($size) ? ' size="' . $size . '" multiple="multiple"' : '' )
            . '>';
        $form .= '<option value=""></option>' if !$size;

        #英語表記
        if ( defined $default::english && $default::english ) {
            $form .= '<option value="!e"'
                . (
                (   @{ $this->{'filetype'} }
                        && grep( $_ eq '!e', @{ $this->{'filetype'} } )
                ) ? ' selected="selected"' : ''
                ) . '>No file</option>';
            $form .= '<option value="-e"'
                . (
                (   @{ $this->{'filetype'} }
                        && grep( $_ eq '-e', @{ $this->{'filetype'} } )
                ) ? ' selected="selected"' : ''
                ) . '>Some file</option>';

            #日本語
        }
        else {
            $form .= '<option value="!e"'
                . (
                (   @{ $this->{'filetype'} }
                        && grep( $_ eq '!e', @{ $this->{'filetype'} } )
                ) ? ' selected="selected"' : ''
                ) . '>ファイル無</option>';
            $form .= '<option value="-e"'
                . (
                (   @{ $this->{'filetype'} }
                        && grep( $_ eq '-e', @{ $this->{'filetype'} } )
                ) ? ' selected="selected"' : ''
                ) . '>ファイル有</option>';
        }

        foreach $mime (@mimename) {
            $form
                .= '<option value="'
                . $mime . '"'
                . (
                (   @{ $this->{'filetype'} }
                        && grep( $_ eq $mime, @{ $this->{'filetype'} } )
                ) ? ' selected="selected"' : ''
                )
                . '>'
                . $this->{'mimename'}{$mime}
                . '</option>';
        }
        $form .= '</select>';

        if ( defined $this->{'anyfile'} && $this->{'anyfile'} ) {
            my $ext
                = ( defined $main::search{ $this->{'id'} }{'ext'} )
                ? $this->escape( $main::search{ $this->{'id'} }{'ext'} )
                : '';

            #英語表記
            if ( defined $default::english && $default::english ) {
                $form
                    .= '<div class="extra-field"><label for="search:'
                    . $this->{'id'}
                    . ':ext">Extension</label> <input type="text" size="20" name="search:'
                    . $this->{'id'}
                    . ':ext" id="search:'
                    . $this->{'id'}
                    . ':ext" value="'
                    . $ext
                    . '" /></div>';
                $form .= '<div class="note">ex) .gif .jpg etc..</div>';

                #日本語
            }
            else {
                $form
                    .= '<div class="extra-field"><label for="search:'
                    . $this->{'id'}
                    . ':ext">拡張子指定</label> <input type="text" size="20" name="search:'
                    . $this->{'id'}
                    . ':ext" id="search:'
                    . $this->{'id'}
                    . ':ext" value="'
                    . $ext
                    . '" /></div>';
                $form
                    .= '<div class="note">（複数の場合はスペースで区切る）</div>';
            }
        }
    }

    #コメントの検索
    if ( defined $this->{'comment'} && $this->{'comment'} ) {
        my ( $key, $checked );
        $key
            = ( defined $main::search{ $this->{'id'} }{'str'}
                && $main::search{ $this->{'id'} }{'str'} )
            ? $this->escape( $main::search{ $this->{'id'} }{'str'} )
            : '';
        $checked
            = ( defined $main::search{ $this->{'id'} }{'i'}
                && $main::search{ $this->{'id'} }{'i'} )
            ? ' checked="checked"'
            : '';
        $form .= '<div class="extra-field">';

        #英語表記
        if ( defined $default::english && $default::english ) {
            $form
                .= '<label for="search:'
                . $this->{'id'}
                . ':str">Comment</label> '
                if !defined $this->{'noFileSearchForm'}
                || !$this->{'noFileSearchForm'};
            $form
                .= '<input type="text" size="20" name="search:'
                . $this->{'id'}
                . ':str" id="search:'
                . $this->{'id'}
                . ':str" value="'
                . $key . '" />';
            $form .= $this->mk_select_menu(
                'search:' . $this->{'id'} . ':mode', '', '',
                $main::search{ $this->{'id'} }{'mode'},
                'and' => 'Matches on all words',
                'or'  => 'Matches on any words',
                'A'   => 'Begins with any words',
                'Z'   => 'Finishes with any words',
                'eq'  => 'Equal to any words',
            );
            $form .= '</div>';
            $form
                .= '<div class="extra-field"><label><input type="checkbox" name="search:'
                . $this->{'id'}
                . ':i" value="1"'
                . $checked
                . ' /> Do case-insensitive pattern matching.</label></div>'
                if !$this->{'ignoreCase'};

            #日本語
        }
        else {
            $form
                .= '<label for="search:'
                . $this->{'id'}
                . ':str">コメント検索</label> '
                if !defined $this->{'noFileSearchForm'}
                || !$this->{'noFileSearchForm'};
            $form
                .= '<input type="text" size="20" name="search:'
                . $this->{'id'}
                . ':str" id="search:'
                . $this->{'id'}
                . ':str" value="'
                . $key . '" />';
            $form .= $this->mk_select_menu(
                'search:' . $this->{'id'} . ':mode', '', '',
                $main::search{ $this->{'id'} }{'mode'},
                'and' => '全ての文字列を含む',
                'or'  => 'いずれかの文字列を含む',
                'A'   => 'いずれかの文字列から始まる',
                'Z'   => 'いずれかの文字列で終わる',
                'eq'  => 'いずれかの文字列に一致する',
            );
            $form .= '</div>';
            $form
                .= '<div class="extra-field"><label><input type="checkbox" name="data:'
                . $this->{'id'}
                . ':i" value="1"'
                . $checked
                . ' /> 大文字と小文字を区別しない</label></div>'
                if !$this->{'ignoreCase'};
        }
    }

    return $form;
}

######################################################################
#	(8) obj->sort_enabled;
sub sort_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(9) obj->cookie_enabled;
sub cookie_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
sub get_enclosure_attributes ($$$$$) {
    my ( $this, $str, $path, $c, $n ) = @_;
    my ( %data, $size );
    %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return () if !$data{'type'};
    $path
        = $path . '/'
        . $this->{'id'}
        . (
        (          defined $data{'name'}
                && $data{'name'} ne ''
                && $data{'name'} !~ /[^0-9a-zA-Z_]/
        ) ? '.' . $data{'name'} : ''
        )
        . '.'
        . $data{'type'};
    $size = -s $path;
    return (
        'url'    => $this->full_path($path),
        'length' => ( ($size) ? $size : '' ),
        'type'   => (
            ( $data{'type'} )
            ? ( ref $this->{'ext_type'}
                    && $this->{'ext_type'}{ $data{'type'} } )
                    ? $this->{'ext_type'}{ $data{'type'} }
                    : ( $Common::mime_type{ $data{'type'} } )
                ? $Common::mime_type{ $data{'type'} }
                : 'application/octet-stream'
            : 'application/octet-stream'
        )
    );
}

######################################################################
sub editor_enabled ($) {
    my $this = shift;
    return (
        $this->{'importEnabled'}
            && ( $this->{'photoSharingServiceName'}
            && $this->{'photoSharingServiceAccount'} )
            || $this->{'photoSharingServiceRSS'}
    ) ? 1 : 0;
}

sub is_secure_user_for_edit ($) {
    my $this = shift;
    if ( $this->{'refererCheck'} ) {
        my $my_url = $this->my_url;
        return (   $ENV{'HTTP_REFERER'}
                && $ENV{'HTTP_REFERER'} =~ /^\Q$my_url\E/ ) ? 1 : 0;
    }
    else {
        return 1;
    }
}

sub set_user_generated_field_editor_page_settings ($) {
    my $this = shift;
    $setting::page{'ugfieldeditor'}{'title'} = '外部コンテンツ一覧';

#	$setting::page{'ugfieldeditor'}{'text'} = '#000000';
#	$setting::page{'ugfieldeditor'}{'link'} = '#0000ff';
#	$setting::page{'ugfieldeditor'}{'alink'} = '#99ffff';
#	$setting::page{'ugfieldeditor'}{'vlink'} = '#000099';
#	$setting::page{'ugfieldeditor'}{'hover'} = '#00ffff';
#	$setting::page{'ugfieldeditor'}{'nodeco'} = 'no';
#	$setting::page{'ugfieldeditor'}{'bgcolor'} = '#ffffff';
#	$setting::page{'ugfieldeditor'}{'bodyId'} = 'image-viewer';
#	$setting::page{'ugfieldeditor'}{'cssFileName'} = '';
# 	$setting::page{'ugfieldeditor'}{'CSS'} = 'h1 { font-size: large; font-weight: bold; text-indent: 1em; font-family: "ヒラギノ角ゴ Pro W3",Osaka,"ＭＳ Ｐゴシック",sans-serif;}' . "\x0A";
# 	$setting::page{'ugfieldeditor'}{'CSS'} .= 'h2 { font-size: medium; font-weight: bold; font-family: "ヒラギノ角ゴ Pro W3",Osaka,"ＭＳ Ｐゴシック",sans-serif;}' . "\x0A";
# 	$setting::page{'ugfieldeditor'}{'CSS'} .= 'hr { border-width: 1 0 0 0; }';
# 	$setting::page{'ugfieldeditor'}{'background'} = '';
# 	$setting::page{'ugfieldeditor'}{'bgproperties'} = '';
# 	$setting::page{'ugfieldeditor'}{'topmargin'} = '';
# 	$setting::page{'ugfieldeditor'}{'leftmargin'} = '';
# 	$setting::page{'ugfieldeditor'}{'javascript'} = '';
# 	$setting::page{'ugfieldeditor'}{'header'} = '';
# 	$setting::page{'ugfieldeditor'}{'headerFileName'} = '';
# 	$setting::page{'ugfieldeditor'}{'titleImageAlign'} = '';
# 	$setting::page{'ugfieldeditor'}{'titleImage'} = '';
# 	$setting::page{'ugfieldeditor'}{'align'} = '';
# 	$setting::page{'ugfieldeditor'}{'width'} = '';
# 	$setting::page{'ugfieldeditor'}{'deco'} = '';
# 	$setting::page{'ugfieldeditor'}{'titleText'} = '';
    return 1;
}

sub print_user_generated_field_editor ($) {
    my $this = shift;
    my $site;
    my $site_url;
    my $hash;
    my $items;
    my $rss
        = ( $this->{'photoSharingServiceRSS'} )
        ? $this->{'photoSharingServiceRSS'}
        : ( $this->{'photoSharingServiceName'} eq 'TwitPic' )
        ? 'http://twitpic.com/photos/'
        . $this->{'photoSharingServiceAccount'}
        . '/feed.rss'
        : ( $this->{'photoSharingServiceName'} eq 'Twitgoo' )
        ? 'http://twitgoo.com/u/'
        . $this->{'photoSharingServiceAccount'}
        . '/?format=rss'
        : ( $this->{'photoSharingServiceName'} eq 'Mobypicture' )
        ? 'http://www.mobypicture.com/rss/'
        . $this->{'photoSharingServiceAccount'}
        . '/user.rss'
        : ( $this->{'photoSharingServiceName'} eq 'img.ly' )
        ? 'http://img.ly/images/'
        . $this->{'photoSharingServiceAccount'} . '.rss'
        : ( $this->{'photoSharingServiceName'} eq 'flickr' )
        ? 'http://api.flickr.com/services/feeds/photos_public.gne?id='
        . $this->{'photoSharingServiceAccount'}
        . '&format=rss_200'
        : ( $this->{'photoSharingServiceName'} eq 'hatena' )
        ? 'http://f.hatena.ne.jp/'
        . $this->{'photoSharingServiceAccount'} . '/rss'
        : '';

    if ($rss) {
        require CMONOS::CGI_Lib::HTTP_Client;
        my $res = HTTP_Client->new->request( 'GET' => $rss );
        $hash = $res->hash('utf8') if $res->is_success;
        if ( ref $hash ) {
            if ( ref $hash->{'rss'} ) {
                $site = $hash->{'rss'}->{'channel'}->{'title'}
                    if defined $hash->{'rss'}->{'channel'}->{'title'};
                $site_url = $hash->{'rss'}->{'channel'}->{'link'}
                    if defined $hash->{'rss'}->{'channel'}->{'link'};
                if ( ref $hash->{'rss'}->{'channel'}->{'item'} ) {
                    $hash->{'rss'}->{'channel'}->{'item'}
                        = [ $hash->{'rss'}->{'channel'}->{'item'} ]
                        if ref $hash->{'rss'}->{'channel'}->{'item'} eq
                        'HASH';
                    $items = $hash->{'rss'}->{'channel'}->{'item'};
                }
            }
            elsif ( ref $hash->{'rdf:RDF'} ) {
                $site = $hash->{'rdf:RDF'}->{'channel'}->{'title'}
                    if defined $hash->{'rdf:RDF'}->{'channel'}->{'title'};
                $site_url = $hash->{'rdf:RDF'}->{'channel'}->{'link'}
                    if defined $hash->{'rdf:RDF'}->{'channel'}->{'link'};
                if ( ref $hash->{'rdf:RDF'}->{'item'} ) {
                    $hash->{'rdf:RDF'}->{'item'}
                        = [ $hash->{'rdf:RDF'}->{'item'} ]
                        if ref $hash->{'rdf:RDF'}->{'item'} eq 'HASH';
                    $items = $hash->{'rdf:RDF'}->{'item'};
                }
            }
            elsif ( ref $hash->{'feed'} ) {
                $site
                    = ( ref $hash->{'feed'}->{'title'} )
                    ? $hash->{'feed'}->{'title'}->{'value'}
                    : $hash->{'feed'}->{'title'}
                    if defined $hash->{'feed'}->{'title'};
                $site_url = $this->get_atom_value( $hash->{'feed'}->{'link'},
                    'html', 'href' )
                    if defined $hash->{'feed'}->{'link'};
                if ( ref $hash->{'feed'}->{'entry'} ) {
                    $hash->{'feed'}->{'entry'}
                        = [ $hash->{'feed'}->{'entry'} ]
                        if ref $hash->{'feed'}->{'entry'} eq 'HASH';
                    $items = $hash->{'feed'}->{'entry'};
                }
            }
        }
    }
    $setting::page{'ugfieldeditor'}{'title'}
        = $this->escape($site) . ' 一覧'
        if $site;
    my $css = <<CSS;
body {
  margin: 0;
  padding: 0;
  background: #FFF none;
}
p#summary {
  background-repeat: repeat-x;
  background-position: top;
  background-image: url('/lib/image/bg_plastic_light.gif');
  background-color: #f3f3f3;
  border: 1px solid #ddd;
  border-bottom-color: #bbb;
  padding: 10px 20px;
  text-align: center;
  color: #000000;
  margin: 0;
}
p#summary a {
  color: #000000;
  text-decoration: underline;
}
ul {
  margin: 0;
  padding: 0;
  list-style-type: none;
}
ul li {
  margin: 0;
  padding: 0;
  display: -moz-inline-box;
  display: inline-block;
  vertical-align: middle;
  /display: inline;
  /zoom: 1;
  _display: inline;
  _zoom: 1;
}
ul a {
  display: block;
  text-decoration: none;
  padding: 4px;
  border-radius: 4px;
  -moz-border-radius: 4px;
  -webkit-border-radius: 4px;
  -o-border-radius: 4px;
  -ms-border-radius: 4px;
}
ul a:hover, ul a.selected {
  background-color: #FF0;
}
ul a:active {
  background-color: #FFD;
}
ul a img {
  display: block;
  border: none;
}
CSS
    $cgi::action->set_css($css);

    if ( ref $items ) {
        my $javascript = <<SCRIPT;
cmonos.form.setImageUrl = function (url,sample) {
	if (opener && opener.document.getElementById('data:$main::id:url')) {
		opener.document.getElementById('data:$main::id:url').value = url;
		cmonos.element.item(opener.document.getElementById('data:$main::id:url')).removeClassName('invalid-field');
		opener.document.getElementById('content-sample-$main::id').innerHTML = sample;
	} else {
		alert("呼び出し元フォームがみつかりません。");
	}
	window.close();
};
SCRIPT
        $cgi::action->set_javascript($javascript);
    }
    $cgi::action->print_page_header('ugfieldeditor');
    print '<p id="summary">';
    print '<a href="'
        . $this->escape( $this->image_url_encode($site_url) )
        . '" target="_blank">'
        if $site_url;
    if ($site) {
        print $this->escape($site);
    }
    else {
        print '外部コンテンツ';
    }
    print '</a>' if $site_url;
    print ' 一覧';
    print '</p>';
    if ( ref $items ) {
        print '<ul>';
        foreach my $item ( @{$items} ) {
            my ( $url, $preview_url, $sample );
            if ( ref $item->{'media:content'} ) {
                $url = $item->{'media:content'}{'url'};
                next
                    if $item->{'media:content'}{'type'}
                    && $item->{'media:content'}{'type'}
                    !~ m!^(?:image|video|audio)/!;
                $preview_url
                    = ( ref $item->{'media:thumbnail'}
                        && $item->{'media:thumbnail'}{'url'} )
                    ? $item->{'media:thumbnail'}{'url'}
                    : $url;
                if ($url) {
                    $url =~ s!/mini_([^/]+)$!/original_$1!;
                    $url =~ s/_m(\.[0-9a-zA-Z_]+)$/$1/;
                }
            }
            elsif ( $item->{'hatena:imageurl'} ) {
                $url = $item->{'hatena:imageurl'};
                $preview_url
                    = ( $item->{'hatena:imageurlmedium'} )
                    ? $item->{'hatena:imageurlmedium'}
                    : ( $item->{'hatena:imageurlsmall'} )
                    ? $item->{'hatena:imageurlsmall'}
                    : $url;
            }
            elsif ($item->{'link'}
                && !ref $item->{'link'}
                && $item->{'link'} =~ m!https?://twitpic.com/([^/]+)!i )
            {
                $url = 'http://twitpic.com/show/full/' . $1 . '.jpg';
                $preview_url
                    = (    $item->{'description'}
                        && !ref $item->{'description'}
                        && $item->{'description'}
                        =~ /<img[\x20\x09]+src="(.*?)"/si ) ? $1 : $url;
            }
            elsif ( ref $item->{'enclosure'} ) {
                $url = $item->{'enclosure'}{'url'};
                next
                    if $item->{'enclosure'}{'type'}
                    && $item->{'enclosure'}{'type'}
                    !~ m!^(?:image|video|audio)/!;
                $preview_url
                    = ( ref $item->{'media:thumbnail'}
                        && $item->{'media:thumbnail'}{'url'} )
                    ? $item->{'media:thumbnail'}{'url'}
                    : $url;
                if ($url) {
                    $url =~ s!/mini_([^/]+)$!/original_$1!;
                    $url =~ s/_m(\.[0-9a-zA-Z_]+)$/$1/;
                }
            }
            next if !$url;
            $url         = $this->image_url_encode($url);
            $preview_url = $this->image_url_encode($preview_url);
            $sample
                = '<a href="'
                . $this->escape($url)
                . '" target="_blank"><img src="'
                . $preview_url
                . '" style="border:none;max-width:240px;max-height:240px" /></a>';
            print '<li><a href="JavaScript:cmonos.form.setImageUrl(\''
                . $this->escape($url) . '\',\''
                . $this->escape($sample)
                . '\')" title="このコンテンツを読み込む"><img src="'
                . $preview_url
                . '" /></a></li>';
        }
        print '</ul>';
    }
    $cgi::action->print_page_footer('ugfieldeditor');
    return 1;
}

sub image_url_encode ($$) {
    my ( $this, $text ) = @_;
    return '' if !defined $text || $text eq '';
    $text = $this->jcode_convert( $text, 'utf8' );
    utf8::encode($text) if utf8::is_utf8($text);
    $text =~ s/([\x80-\xFF])/'%' . sprintf("%.2X", unpack("C", $1))/eg;
    $text =~ s/\x20/+/g;
    return $text;
}

sub get_atom_value ($$$$) {
    my ( $this, $element, $type, $name ) = @_;
    return '' if !defined $element;
    $element = [$element] if !ref $element || ref $element ne 'ARRAY';
    foreach my $this_element (@$element) {
        if (   defined $this_element->{'type'}
            && $this_element->{'type'} =~ /$type/
            && defined $this_element->{$name} )
        {
            return $this_element->{$name};
        }
    }
    return '';
}

sub user_generated_field_edit ($) {
    my $this = shift;
    return 1;
}

sub get_query_for_editor ($;$$$) {
    my ( $this, $c ) = @_;
    return
          '?m=UserGeneratedFieldEditor&k=ugfieldeditor&id='
        . $this->{'id'}
        . (
        ( $default::subdivided && defined $main::c ) ? '&c=' . $main::c : '' )
        . (
        (          $main::m
                && $main::m !~ /[^0-9a-zA-Z_]/
                && $main::m ne 'UserGeneratedFieldEditor'
        ) ? '&mod=' . $main::m
        : (        $main::mod
                && $main::mod !~ /[^0-9a-zA-Z_]/
                && $main::mod ne 'UserGeneratedFieldEditor' )
        ? '&mod=' . $main::mod
        : ''
        )
        . (
        ( $main::p && $main::p !~ /[^0-9a-zA-Z_]/ ) ? '&p=' . $main::p : '' );
}

######################################################################
#sub import {
BEGIN {
    Field::class_register( 'file', 'FileUL', 'ファイル添付' );
}

1;
