package GoogleMapsBase;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（Googleマップ）
# GoogleMaps.pm 1.1.8
# Copyright (C) 2007-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-12-04
######################################################################
sub _set ($$) {
    my ( $this, $setting ) = @_;
    $this->{'refererCheck'} = 1      if !defined $this->{'refererCheck'};
    $this->{'canvasWidth'}  = 400    if !$this->{'canvasWidth'};
    $this->{'canvasHeight'} = 400    if !$this->{'canvasHeight'};
    $this->{'viewerWidth'}  = 400    if !$this->{'viewerWidth'};
    $this->{'viewerHeight'} = 400    if !$this->{'viewerHeight'};
    $this->{'widthlimit'}   = 48     if !$this->{'widthlimit'};
    $this->{'heightlimit'}  = 48     if !$this->{'heightlimit'};
    $this->{'alwaysCopy'}   = 1      if !defined $this->{'alwaysCopy'};
    $this->{'maxsize'}      = 128000 if !$this->{'maxsize'};
    delete $this->{'maxwidth'};

    #複数の要素を登録
    $this->{'elements'} = 1;
    $cgi::enctype_multipart = 1;

    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }
    return;
}

sub elements ($$) {
    my ( $this, $str ) = @_;
    return ( 'str' => '' ) if !defined $str;
    my %data = ();
    if ( $str && $str !~ /^[0-9a-zA-Z_\.\,\-\+\/]+\:/m ) {
        $str =~ s/[\x0A\x0D]//gs;
        require CMONOS::CGI_Lib::HTTP_Client;
        my $res
            = HTTP_Client->new->request(
            'GET' => 'http://maps.google.com/maps/api/geocode/xml?address='
                . $this->url_encode($str)
                . '&sensor=false' );
        my $hash;
        $hash = $res->hash('utf8') if $res->is_success;
        if (   ref $hash
            && $hash->{'GeocodeResponse'}
            && ref $hash->{'GeocodeResponse'}
            && ref $hash->{'GeocodeResponse'}->{'result'} )
        {
            my $result
                = ( ref $hash->{'GeocodeResponse'}->{'result'} eq 'ARRAY' )
                ? $hash->{'GeocodeResponse'}->{'result'}[0]
                : $hash->{'GeocodeResponse'}->{'result'};
            if (   ref $result->{'geometry'}
                && ref $result->{'geometry'}->{'location'}
                && defined $result->{'geometry'}->{'location'}->{'lat'}
                && defined $result->{'geometry'}->{'location'}->{'lng'} )
            {
                $data{'c'}
                    = $result->{'geometry'}->{'location'}->{'lat'} . ','
                    . $result->{'geometry'}->{'location'}->{'lng'};
                $data{'enabled'} = 1;
                $data{'z'}       = $this->{'z'} || 4;
                $data{'t'}       = $this->{'t'} || '';
            }
        }
    }
    else {
        my $check = 0;
        while ( $str =~ s/^([0-9a-zA-Z_\.\,\-\+\/]+)\:([^\x0A\x0D]*)\x0A//s )
        {
            $data{$1} = $2;
            $data{$1} =~ s/&#10;/\x0A/sg;
            $check = 1;
        }
        $str =~ s/^\x0A//;
        $str =~ s/^\x0A// if !$check;
        $data{'comment'} .= $str if defined $str && $str ne '';
    }
    return %data;
}

######################################################################
#	(1)-0 obj->match_enabled;
sub match_enabled ($) {
    my $this = shift;
    return (
        (          !$this->{'required'}
                && defined $main::search{ $this->{'id'} }{'check'}
                && $main::search{ $this->{'id'} }{'check'} ne ''
        )
            || ( $main::search{ $this->{'id'} }{'sw'}
            && $main::search{ $this->{'id'} }{'ne'} )
    ) ? 1 : 0;
}

#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    return 1 if !$this->match_enabled;
    my %data = $this->elements($$str);
    if (   $main::search{ $this->{'id'} }{'sw'}
        && $main::search{ $this->{'id'} }{'ne'} )
    {
        my ( $lat, $lng )
            = ( $data{'kml'} && $data{'p'} ) ? split( /,/, $data{'p'} )
            : ( $data{'c'} ) ? split( /,/, $data{'c'} )
            :                  ();
        return 0 if !defined $lat || !defined $lng;
        my ( $s, $w ) = split( /,/, $main::search{ $this->{'id'} }{'sw'} );
        my ( $n, $e ) = split( /,/, $main::search{ $this->{'id'} }{'ne'} );
        if ( $lat <= $n && $lat >= $s ) {    #緯度チェック
            if ( $w > $e ) {                 #軽度チェック
                return ( $lng >= $w || $lng <= $e ) ? 1 : 0;
            }
            else {
                return ( $lng >= $w && $lng <= $e ) ? 1 : 0;
            }
        }
        else {
            return 0;
        }
    }
    else {
        my $field_enabled
            = ( $data{'enabled'} && ( $data{'kml'} || $data{'c'} ) ) ? 1 : 0;
        if (   ( $main::search{ $this->{'id'} }{'check'} && $field_enabled )
            || ( !$main::search{ $this->{'id'} }{'check'} && !$field_enabled )
            )
        {
            return 1;
        }
        else {
            return 0;
        }
    }
}

######################################################################
#	(2) obj->form;
sub form {
    my ( $this, $status, $original, $path ) = @_;
    return ''
        if $ENV{'HTTP_USER_AGENT'}
        && $ENV{'HTTP_USER_AGENT'}
        =~ /(?:DoCoMo|KDDI|UP\.Browser)/i;    # docomo au
    my ( $name, $form, $kml_enabled );
    local $this->{'is_temporary_data'}
        = ( !$cgi::input->{'clipboard_expired'} ) ? 1 : 0;

    #名前にデータIDを埋め込む
    $name = 'data:' . $this->{'id'} . ':';

    my $kml_path
        = (
        ( !$cgi::input->{'clipboard_expired'} && $this->{'is_posted_kml'} )
        ? $cgi::input->get_public_clipboard
        : $path )
        . '/'
        . $this->{'id'}
        . (
        (          $main::data{ $this->{'id'} }{'kml'}
                && $main::data{ $this->{'id'} }{'kml'} eq 'kmz'
        ) ? '.kmz' : '.kml'
        );
    $kml_enabled = ( -f $kml_path ) ? 1 : 0;

    $main::data{ $this->{'id'} }{'m'} = $this->{'default'}
        if ( !defined $status || $status eq '' )
        && ( !defined $main::data{ $this->{'id'} }{'m'}
        || $main::data{ $this->{'id'} }{'m'} eq '' );

    #フィールドネーム+ヘッダ
    $form = $this->field_header;
    $form
        .= '<label><input type="checkbox" name="'
        . $name
        . 'enabled" value="1"'
        . ( ( $main::data{ $this->{'id'} }{'enabled'} ) ? ' checked="checked"'
        : '' )
        . ' onclick="cmonos.element.item(\'gmap-field-'
        . $this->{'id'}
        . '\').disappear(!this.checked);'
        . ( ( !$main::data{ $this->{'id'} }{'enabled'} )
        ? ' if (this.checked && document.getElementById(\'gmap-canvas-'
            . $this->{'id'}
            . '\').src.search(/&reloaded=1$/) == -1) {document.getElementById(\'gmap-canvas-'
            . $this->{'id'}
            . '\').src = document.getElementById(\'gmap-canvas-'
            . $this->{'id'}
            . '\').src + \'&reloaded=1\'}'
        : '' )
        . '" /> 使用する</label>'
        if !$this->{'required'};
    $form .= '<div id="gmap-field-' . $this->{'id'} . '">';
    $form
        .= '<fieldset><legend title="KML/KMZ/GPX/NMEAファイルを指定しない場合は地図画像をドラッグしたり縮尺を変更して中心座標と縮尺を設定します。">KML/KMZ/GPX/NMEAファイル<span class="note">(オプション)</span></legend>';
    $form .= '<table class="sub-field-table">';
    $form .= '<tbody>';
    $form .= '<tr>';
    $form
        .= '<td class="sub-field-name"><label for="option:'
        . $this->{'id'}
        . ':url">URL</label></td>';
    $form
        .= '<td class="sub-field"><input type="text" name="option:'
        . $this->{'id'}
        . ':url" id="option:'
        . $this->{'id'}
        . ':url" size="36" value="'
        . ( ( $main::option{ $this->{'id'} }{'url'} && !$kml_enabled )
        ? $this->escape( $main::option{ $this->{'id'} }{'url'} )
        : '' )
        . '" />';
    $form
        .= '<br />&raquo;<a href="http://maps.google.co.jp/" target="_blank">Googleマップを開く</a>';
    $form .= '</td>';
    $form .= '</tr>';
    $form .= '<tr>';
    $form
        .= '<td class="sub-field-name"><label for="'
        . $name
        . 'file">ファイル</label></td>';
    $form
        .= '<td class="sub-field"><span id="'
        . $name
        . 'file-wrapper"><input type="file" name="'
        . $name
        . 'file" id="'
        . $name
        . 'file" value="'
        . $this->escape( $main::data{ $this->{'id'} }{'file'} )
        . '" /></span><input type="button" value="&#139; リセット" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . 'file\').reset()">';

    if ($kml_enabled) {
        $main::data{ $this->{'id'} }{'url'} = $this->full_path($kml_path);
        if ( $this->{'is_temporary_data'} && $this->{'is_posted_kml'} ) {
            $form
                .= '<div class="extra-field"><label><input type="checkbox" name="'
                . $name
                . 'delete" value="1" /> <a href="'
                . $kml_path
                . '"><strong>未確定</strong>ファイル</a>を削除する</label></div>';
        }
        else {
            $form
                .= '<div class="extra-field"><label><input type="checkbox" name="'
                . $name
                . 'delete" value="1"'
                . (
                (          $this->{'is_temporary_data'}
                        && $main::data{ $this->{'id'} }{'delete'}
                ) ? ' checked="checked"' : ''
                )
                . ' /> <a href="'
                . $kml_path
                . '">既存ファイル</a>を削除する</label></div>';
        }
        $form .= '<input type="hidden" name="' . $name . 'kml" value="1" />';
    }
    $form .= '</td>';
    $form .= '</tr>';
    $form .= '<tr>';
    $form
        .= '<td class="sub-field-name"><span class="datatitle">線の距離</span></td>';
    $form .= '<td class="sub-field">';
    $form .= '<ol class="checkbox">';
    $form
        .= '<li><label><input type="radio" name="'
        . $name
        . 'd" value="0"'
        . (
        ( !$main::data{ $this->{'id'} }{'d'} ) ? ' checked="checked"' : '' )
        . ' /> 始点から終点までの距離情報を表示しない</label></li>';
    $form
        .= '<li><label><input type="radio" name="'
        . $name
        . 'd" value="1"'
        . (
        (          $main::data{ $this->{'id'} }{'d'}
                && $main::data{ $this->{'id'} }{'d'} == 1
        ) ? ' checked="checked"' : ''
        )
        . ' /> 始点から終点までの総延長を表示する</label></li>';
    $form
        .= '<li><label><input type="radio" name="'
        . $name
        . 'd" value="2"'
        . (
        (          $main::data{ $this->{'id'} }{'d'}
                && $main::data{ $this->{'id'} }{'d'} == 2
        ) ? ' checked="checked"' : ''
        )
        . ' /> 総延長に加えて始点と終点の直線距離を表示する</label></li>';
    $form .= '</ol>';
    $form .= '</td>';
    $form .= '</tr>';
    $form .= '<tr>';
    $form
        .= '<td class="sub-field-name"><span class="datatitle">中心座標</span></td>';
    $form .= '<td class="sub-field">';
    $form
        .= '<label title="地図による設定よりも KML を元に割り出される中心座標と拡大率が優先されます"><input type="checkbox" name="'
        . $name
        . 'fit" id="'
        . $name
        . 'fit" value="1"'
        . (
        ( $main::data{ $this->{'id'} }{'fit'} ) ? ' checked="checked"' : '' )
        . ' /> 自動的に KML に合わせる</label>';
    $form .= '</td>';
    $form .= '</tr>';
    $form .= '</tbody>';
    $form .= '</table></fieldset>';
    $form
        .= '<fieldset><legend><label for="'
        . $name
        . 'm">中心座標コメント</label><span class="note">('
        . $this->center_marker_comment_note
        . ')</span></legend>';
    $form
        .= '<textarea name="'
        . $name
        . 'm" id="'
        . $name
        . 'm" cols="32" rows="4" wrap="soft"'
        . (
        ( $this->{'is_invalid_field'}{'m'} ) ? $this->set_invalid_class : '' )
        . '>'
        . $this->escape( $main::data{ $this->{'id'} }{'m'} )
        . '</textarea>';
    $form .= $this->textarea_tool( $name . 'm' );
    $form
        .= '<div class="hints">カンマ区切り文字列は、自動的に表に変換されます。</div>'
        if defined $this->{'csv'} && $this->{'csv'};
    $form .= '</fieldset>';
    $form .= $this->_form($name);
    $form .= '<fieldset><legend>マーカー画像</legend>';
    $form .= '<table class="sub-field-table">';
    $form .= '<tbody>';

    if ( defined $this->{'dir'} && $this->{'dir'} ne '' ) {
        $form .= '<tr>';
        $form
            .= '<td class="sub-field-name"><label for="'
            . $name
            . 'marker">画像メニュー</label></td>';
        $form .= '<td class="sub-field">';
        $form .= $this->get_image_menu( $name . 'marker',
            $main::data{ $this->{'id'} }{'marker'} );
        $form .= '</td>';
        $form .= '</tr>';
    }
    $form .= '<tr>';
    $form
        .= '<td class="sub-field-name"><label for="'
        . $name
        . 'markerfile">画像ファイル</label></td>';
    $form
        .= '<td class="sub-field"><span id="'
        . $name
        . 'markerfile-wrapper"><input type="file" name="'
        . $name
        . 'markerfile" id="'
        . $name
        . 'markerfile" /></span><input type="button" value="&#139; リセット" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . 'markerfile\').reset()">';
    if ((   ( defined $status && $status ne '' )
            || (  !$cgi::input->{'clipboard_expired'}
                && $this->{'is_posted_marker'} )
        )
        && defined $main::data{ $this->{'id'} }{'type'}
        && $main::data{ $this->{'id'} }{'type'} ne ''
        )
    {
        my $marker_path
            = (
            ( $this->{'is_temporary_data'} && $this->{'is_posted_marker'} )
            ? $cgi::input->get_public_clipboard
            : $path )
            . '/'
            . $this->{'id'}
            . '.marker.'
            . $main::data{ $this->{'id'} }{'type'};
        $form .= '<div class="sample-image">';
        $form .= $cgi::Image->image_tag(
            $marker_path,
            $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            0,
            '現在のマーカー画像',
            '',
            300,
            300,
        );
        if ( $this->{'is_temporary_data'} && $this->{'is_posted_marker'} ) {
            $form
                .= '<div class="extra-field"><label><input type="checkbox" id="option:'
                . $this->{'id'}
                . ':markercancel" name="option:'
                . $this->{'id'}
                . ':markercancel" value="1" /> この<strong>未確定</strong>ファイルを削除する</label></div>';
        }
        else {
            $form
                .= '<div class="extra-field"><label><input type="checkbox" id="'
                . $name
                . 'delmk" name="'
                . $name
                . 'delmk" value="1"'
                . (
                (          $this->{'is_temporary_data'}
                        && $main::data{ $this->{'id'} }{'delmk'}
                ) ? ' checked="checked"' : ''
                ) . ' /> このファイルを削除する</label></div>';
        }
        $form
            .= '<input type="hidden" name="'
            . $name
            . 'type" value="'
            . $main::data{ $this->{'id'} }{'type'}
            . '" /></div>';
        $form
            .= '<input type="hidden" name="'
            . $name
            . 'width" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'width'} ) . '" />';
        $form
            .= '<input type="hidden" name="'
            . $name
            . 'height" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'height'} )
            . '" />';
    }
    $form .= '</td>';
    $form .= '</tr>';
    $form .= '<tr>';
    $form
        .= '<td class="sub-field-name"><span class="datatitle">指し示すポイント</span></td>';
    $form .= '<td class="sub-field">';
    $form .= $this->mk_select_menu(
        $name . 'mka', '', '', $main::data{ $this->{'id'} }{'mka'},
        ''       => '',
        'left'   => '左端',
        'center' => '中央',
        'right'  => '右端'
    );
    $form .= $this->mk_select_menu(
        $name . 'mkv', '', '', $main::data{ $this->{'id'} }{'mkv'},
        ''       => '',
        'top'    => '上端',
        'center' => '中央',
        'bottom' => '下端'
    );
    $form .= '</td>';
    $form .= '</tr>';
    $form .= '</tbody>';
    $form .= '</table></fieldset>';
    $form
        .= '<input type="hidden" name="'
        . $name
        . 'p" id="'
        . $name
        . 'p" value="'
        . $this->escape( $main::data{ $this->{'id'} }{'p'} ) . '" />'
        if $main::data{ $this->{'id'} }{'p'};

    foreach my $k ( 'c', 't', 'z' ) {
        $main::data{ $this->{'id'} }{$k} = ''
            if !defined $main::data{ $this->{'id'} }{$k};
        $form
            .= '<input type="hidden" name="'
            . $name
            . $k
            . '" id="'
            . $name
            . $k
            . '" value="'
            . $this->escape( $main::data{ $this->{'id'} }{$k} ) . '" />';
    }
    my %data = %{ $main::data{ $this->{'id'} } };
    if ( $data{'enabled'} ) {
        if ( $this->{'is_temporary_data'} && $this->{'is_posted_kml'} ) {
            $data{'url'}
                = $this->full_path( $cgi::input->get_public_clipboard . '/'
                    . $this->{'id'}
                    . ( ( $data{'kml'} eq 'kmz' ) ? '.kmz' : '.kml' ) )
                if $data{'kml'};
            $data{'m'} = '中心座標マーカー' if !$data{'m'};
        }
        elsif ( $data{'kml'} ) {
            $data{'url'}
                = $this->full_path( $path . '/'
                    . $this->{'id'}
                    . ( ( $data{'kml'} eq 'kmz' ) ? '.kmz' : '.kml' ) );
        }
        if ( $this->{'is_temporary_data'} && $this->{'is_posted_marker'} ) {
            $data{'mk'} = $this->full_path(
                      $cgi::input->get_public_clipboard . '/'
                    . $this->{'id'}
                    . '.marker.'
                    . (
                      ( $data{'type'} ) ? $data{'type'}
                    : ( $data{'mk'} && $data{'mk'} =~ /\.([0-9a-zA-Z_]+)$/ )
                    ? $1
                    : 'gif'
                    )
            );
        }
        elsif ( $data{'mk'} && $data{'mk'} !~ /^https?:\/\//i ) {
            $data{'mk'} = $this->full_path( $path . '/' . $data{'mk'} );
        }
    }
    $form .= $this->gmap_iframe(
        \%data,
        'gmap-canvas-' . $this->{'id'},
        $this->{'canvasWidth'},
        $this->{'canvasHeight'},
        $this->{'canvasOverView'},
        $this->{'canvasStreetView'},
        1, '',
        {   'z_field'   => $name . 'z',
            't_field'   => $name . 't',
            'c_field'   => $name . 'c',
            'm_field'   => $name . 'm',
            'fit_field' => $name . 'fit',
            (   (          $env::zipcode
                        && $this->{'addrfield'}
                        && ref $this->{'field_obj'}->{ $this->{'addrfield'} }
                        && $this->{'field_obj'}->{ $this->{'addrfield'} }
                        ->{'class'} eq 'addr_j'
                )
                ? ( 'addr_field' => 'data:'
                        . $this->{'addrfield'}
                        . ':pref,data:'
                        . $this->{'addrfield'}
                        . ':city,data:'
                        . $this->{'addrfield'}
                        . ':street,data:'
                        . $this->{'addrfield'}
                        . ':building' )
                : ()
            ),
            ( ( $this->{'dir'} ) ? ( 'mk_field' => $name . 'marker' ) : () ),
            (     ( $main::data{ $this->{'id'} }{'type'} )
                ? ( 'mkc_field' => $name . 'delmk' )
                : ()
            ),
            'mka_field' => $name . 'mka',
            'mkv_field' => $name . 'mkv',
        }
    );
    $form .= '</div>';
    $form
        .= $this->javascript( 'cmonos.element.item(\'gmap-field-'
            . $this->{'id'}
            . '\').disappear(true);' )
        if !$main::data{ $this->{'id'} }{'enabled'} && !$this->{'required'};
    $form .= $this->field_footer;
    delete $this->{'is_temporary_data'};
    return $form;
}

sub _form ($$) {
    my $this = shift;
    return '';
}

sub center_marker_comment_note ($) {
    my $this = shift;
    return
        '無記入の場合中心座標マーカーは表示されません';
}

sub gmap_iframe ($$$;$$$$$$$) {
    my ( $this, $data, $fid, $width, $height, $om, $sv, $s, $tr, $field )
        = @_;
    my $iframe
        = '<iframe class="gmap-frame" id="gmap-canvas-' . $this->{'id'} . '"';
    $iframe
        .= ' src="'
        . $this->escape(
        $this->gmap_viewer_url( $data, $om, $sv, $s, $tr, $field ) )
        . '"';
    $iframe .= ' width="' . $width . '"';
    $iframe .= ' height="' . $height . '"';
    $iframe
        .= ' scrolling="no" marginwidth="0" marginheight="0" frameborder="0">';
    my $link = '';
    $link
        .= '<a href="'
        . $this->escape( $this->gmap_viewer_url( $data, $om, $sv, $s, $tr ) )
        . '" target="_blank">'
        . ( $this->{'mapWindowLabel'}
            || '新しいウィンドウで地図を開く' )
        . '</a>'
        if $this->{'mapWindowEnabled'};
    $link
        .= ( ($link) ? ' ' : '' )
        . '<a href="'
        . $this->escape( $data->{'url'} )
        . '">KML</a>'
        if $this->{'kmlDownloadEnabled'}
        && $data->{'url'}
        && !$data->{'opener_url'};
    $iframe .= $link;
    $iframe .= '</iframe>';

    if ( ref $field ) {
        return $iframe;
    }
    else {
        $iframe .= '<div class="note">' . $link . '</div>' if $link;
        return $this->javascript( "document.write('" . $iframe . "');" );
    }
}

sub gmap_viewer_url ($$;$$$$$) {
    my ( $this, $data, $om, $sv, $s, $tr, $field ) = @_;
    my $m = ( $data->{'m'} ) ? $data->{'m'} : '';
    $this->text_fix( \$m, $this->{'csv'}, 1 )
        if $m && ( !$field || !ref $field || !$field->{'m_field'} );
    return (
        ( $cgi::output || $cgi::output_option )
        ? $this->base_change( $env::javascript, $cgi::output,
            $cgi::output_option )
        : $env::javascript
        )
        . '/gmap.html?'
        . $this->url_encoded_query(
        {   (     ($env::google_api_key)
                ? ( 'key' => $env::google_api_key )
                : ()
            ),
            'z' => (
                  ( $data->{'z'} ) ? $data->{'z'}
                : ( $this->{'z'} ) ? $this->{'z'}
                :                    4
            ),
            't' => (
                  ( $data->{'t'} ) ? $data->{'t'}
                : ( $this->{'t'} ) ? $this->{'t'}
                :                    ''
            ),
            'c' => (
                  ( $data->{'c'} ) ? $data->{'c'}
                : ( $this->{'c'} ) ? $this->{'c'}
                :                    '36.315125,137.504883'
            ),
            ( ( ref $field ) ? %$field : () ),
            ( ($om)            ? ( 'om' => 1 )            : () ),
            ( ($sv)            ? ( 'sv' => 1 )            : () ),
            ( ($s)             ? ( 's'  => 1 )            : () ),
            ( ( $data->{'d'} ) ? ( 'd'  => $data->{'d'} ) : () ),
            (     ( $data->{'tr'} ) ? ( 'tr' => $data->{'tr'} )
                : ($tr)             ? ( 'tr' => $tr )
                :                     ()
            ),
            ( ( $this->{'mw'} )  ? ( 'mw'  => $this->{'mw'} )  : () ),
            ( ( $data->{'fit'} ) ? ( 'fit' => $data->{'fit'} ) : () ),
            ( ( $data->{'url'} ) ? ( 'url' => $data->{'url'} ) : () ),
            ( ( $data->{'bg'} )  ? ( 'bg'  => $data->{'bg'} )  : () ),
            (   ( $data->{'wp'} ) ? ( 'wp' => $this->waypoints_kml_path ) : ()
            ),
            ( ( $data->{'r'} ) ? ( 'r' => 1, 'ns' => 1 ) : () ),
            (     ( $data->{'mk'} && ( $data->{'type'} || !ref $field ) )
                ? ( 'mk' => $data->{'mk'} )
                : ( $this->{'defaultMarker'} )
                ? ( 'mk' => $this->{'defaultMarker'} )
                : ()
            ),
            (   (          $data->{'mk'}
                        && ( $data->{'type'} || !ref $field )
                        && $data->{'mka'}
                )
                ? ( 'mka' => $data->{'mka'} )
                : (        $this->{'defaultMarker'}
                        && $this->{'defaultMarkerAlign'} )
                ? ( 'mka' => $this->{'defaultMarkerAlign'} )
                : ()
            ),
            (   (          $data->{'mk'}
                        && ( $data->{'type'} || !ref $field )
                        && $data->{'mkv'}
                )
                ? ( 'mkv' => $data->{'mkv'} )
                : (        $this->{'defaultMarker'}
                        && $this->{'defaultMarkerValign'} )
                ? ( 'mkv' => $this->{'defaultMarkerValign'} )
                : ()
            ),
            ( ( $m ne '' ) ? ( 'm' => $m ) : () ),
            (   ( $data->{'opener_url'} )
                ? ( 'i'   => 1,
                    'opn' => $data->{'opener_url'},
                    ( ( !$data->{'r'} ) ? ( 'ns' => 1 ) : () )
                    )
                : ()
            ),
        }
        );
}

sub waypoints_kml_path ($) {
    my $this = shift;
    return '';
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my ( $this, $path ) = @_;
    delete $this->{'is_invalid_field'};
    my $is_valid  = 1;
    my $file_name = 'data:' . $this->{'id'} . ':file';
    $main::data{ $this->{'id'} }{'enabled'} = 1 if $this->{'required'};
    if (   $main::clipboard_id
        && !$main::data{ $this->{'id'} }{'file'}
        && !$main::option{ $this->{'id'} }{'cancel'} )
    {
        $cgi::input->inherit_posted_file_properties($file_name);
    }
    my %properties
        = $cgi::input->get_posted_file_properties_from_param($file_name);
    if ( $properties{'delete'} ) {
        delete $properties{'delete'};
        $cgi::input->set_posted_file_properties_from_param( $file_name, 0,
            \%properties );
    }
    if ( $main::data{ $this->{'id'} }{'m'}
        && bytes::length $main::data{ $this->{'id'} }{'m'} > 256 )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                . 'の中心座標コメントが長過ぎます。できるだけ短くしてください。'
        ) if ref $cgi::action;
        $this->{'is_invalid_field'}{'m'} = 1;
        $is_valid = 0;
    }

    #URLからデータを取得
    if ( !$properties{'filename'} && $main::option{ $this->{'id'} }{'url'} ) {
        require CMONOS::CGI_Lib::HTTP_Client;
        my $res = HTTP_Client->new->request(
            'GET' => $main::option{ $this->{'id'} }{'url'} );
        if ( $res->is_success ) {
            my $type        = $res->content_type;
            my $disposition = $res->header('Content-Disposition');
            my $ext
                = (    $disposition
                    && $disposition
                    =~ /;\s*filename=[^;\r\n]+\.([a-zA-Z0-9]+)/ )
                ? lc $1
                : ( $main::option{ $this->{'id'} }{'url'}
                    =~ /\.([a-zA-Z0-9]+)$/i ) ? lc $1
                : 'txt';
            $properties{'filename'} = $main::data{ $this->{'id'} }{'file'}
                = $this->{'id'} . '.' . $ext;
            $properties{'type'} = $type || 'application/octet-stream';
            $properties{'name'} = $file_name;
            $cgi::input->post_clipboard( $file_name, 0, $res->content,
                \%properties );
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . ' KML ファイルをダウンロードできませんでした。'
            ) if ref $cgi::action;
            $is_valid = 0;
        }
    }

    #添付ファイルの処理
    if ( $properties{'filename'} ) {
        my $ext
            = ( $properties{'filename'} =~ /\.([a-zA-Z0-9]+)$/i )
            ? lc $1
            : 'txt';
        my $data;
        my $is_kmz = 0;
        if ( $properties{'type'} =~ /(?:kmz|zip)/i || $ext eq 'kmz' ) {
            if ( eval 'use Archive::Zip;1;' ) {
                my $clipboard_dir
                    = $cgi::input->get_clipboard( $file_name, 0 );
                my $zip     = Archive::Zip->new( $clipboard_dir . '/data' );
                my @members = $zip->memberNames();
                foreach my $m (@members) {
                    $zip->extractMember( $m, $clipboard_dir . '/data.temp' );
                }
                rename(
                    $clipboard_dir . '/data.temp',
                    $clipboard_dir . '/data'
                );
                $properties{'name'}     = $file_name;
                $properties{'filename'} = $main::data{ $this->{'id'} }{'file'}
                    = $this->{'id'} . '.kml';
                $properties{'type'} = 'application/vnd.google-earth.kml+xml';
                $cgi::input->set_posted_file_properties_from_param(
                    $file_name, 0, \%properties );
                $data = $cgi::input->get_posted_file_data($file_name);
            }
            else {
                $is_kmz = 1;
            }
        }
        elsif ( $properties{'type'} =~ /gpx/i || $ext eq 'gpx' ) {
            $data = $this->gpx_to_kml(
                $cgi::input->get_posted_file_data($file_name) );
            $properties{'name'}     = $file_name;
            $properties{'filename'} = $main::data{ $this->{'id'} }{'file'}
                = $this->{'id'} . '.kml';
            $properties{'type'} = 'application/vnd.google-earth.kml+xml';
            $cgi::input->post_clipboard( $file_name, 0, $data, \%properties );
        }
        elsif ($properties{'type'} =~ /nmea/i
            || $ext eq 'nmea'
            || $ext eq 'log' )
        {
            $data = $this->nmea_to_kml(
                $cgi::input->get_posted_file_data($file_name) );
            $properties{'name'}     = $file_name;
            $properties{'filename'} = $main::data{ $this->{'id'} }{'file'}
                = $this->{'id'} . '.kml';
            $properties{'type'} = 'application/vnd.google-earth.kml+xml';
            $cgi::input->post_clipboard( $file_name, 0, $data, \%properties );
        }
        elsif ( $properties{'type'} =~ /xml/i || $ext eq 'kml' ) {
            $data = $cgi::input->get_posted_file_data($file_name);
        }
        if ($is_kmz) {
            $main::data{ $this->{'id'} }{'kml'}  = 'kmz';
            $main::data{ $this->{'id'} }{'file'} = $this->{'id'} . '.kmz';
            $cgi::input->save_file( $file_name,
                '!' . $this->{'id'} . '.kmz' );
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . ' KMZ ファイルでは距離を表示できません。'
            ) if ref $cgi::action && $main::data{ $this->{'id'} }{'d'};
            $main::data{ $this->{'id'} }{'p'}
                = $main::data{ $this->{'id'} }{'d'} = '';
            $this->{'is_posted_kml'} = 1;
        }
        elsif ($data
            && $data
            =~ /^[\x09\x0A\x0C\x0D\x20]*<\?xml(?!\w)[^<>]*\?>[\x09\x0A\x0C\x0D\x20]*<kml(?!\w)[^<>]*>/s
            )
        {
            $main::data{ $this->{'id'} }{'kml'} = 'kml';
            $properties{'filename'} = $main::data{ $this->{'id'} }{'file'}
                = $this->{'id'} . '.kml';
            if ( $data =~ /<NetworkLink>(.*?)<\/NetworkLink>/is ) {
                my $link    = $1;
                my $updated = 0;
                if ( $link =~ /<href>(.*?)<\/href>/s ) {
                    my $href = $this->unescape($1);
                    if ($href) {
                        require CMONOS::CGI_Lib::HTTP_Client;
                        my $res = HTTP_Client->new->request( 'GET' => $href );
                        if ( $res->is_success ) {
                            $main::data{ $this->{'id'} }{'file'}
                                = $this->{'id'} . '.kml';
                            $data    = $res->content;
                            $updated = 1;
                        }
                    }
                }
                if ($updated) {
                    $properties{'name'} = $file_name;
                    $properties{'type'}
                        = 'application/vnd.google-earth.kml+xml';
                    $cgi::input->post_clipboard( $file_name, 0, $data,
                        \%properties );
                }
            }
            $cgi::input->save_file( $file_name,
                '!' . $this->{'id'} . '.kml' );
            $this->{'is_posted_kml'} = 1;
            if ( $data
                =~ m!<Point>[\x09\x0A\x0C\x0D\x20]*<coordinates>[\x09\x0A\x0C\x0D\x20]*(-?[0-9\.]+),(-?[0-9\.]+).*?</coordinates>[\x09\x0A\x0C\x0D\x20]*</Point>!si
                )
            {
                $main::data{ $this->{'id'} }{'p'} = $2 . ',' . $1;
            }
            elsif ( $data
                =~ m!<coordinates>[\x09\x0A\x0C\x0D\x20]*(-?[0-9\.]+),(-?[0-9\.]+).*?</coordinates>!si
                )
            {
                $main::data{ $this->{'id'} }{'p'} = $2 . ',' . $1;
            }
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . ' KML ファイルではありません。'
            ) if ref $cgi::action;
            $is_valid = 0;
            $cgi::input->remove_posted_file($file_name);
            undef %properties;
        }
    }
    elsif ($path
        && -d $path
        && -f $path . '/' . $this->{'id'} . '.kml'
        && !$main::data{ $this->{'id'} }{'delete'} )
    {
        my $kml_data
            = $this->read_file( $path . '/' . $this->{'id'} . '.kml', 1 );
        $main::data{ $this->{'id'} }{'kml'} = 'kml';
        if ( $kml_data
            =~ m!<Point>[\x09\x0A\x0C\x0D\x20]*<coordinates>[\x09\x0A\x0C\x0D\x20]*(-?[0-9\.]+),(-?[0-9\.]+),.*?</coordinates>[\x09\x0A\x0C\x0D\x20]*</Point>!s
            )
        {
            $main::data{ $this->{'id'} }{'p'} = $2 . ',' . $1;
        }
        elsif ( $kml_data
            =~ m!<coordinates>[\x09\x0A\x0C\x0D\x20]*(-?[0-9\.]+),(-?[0-9\.]+),.*?</coordinates>!s
            )
        {
            $main::data{ $this->{'id'} }{'p'} = $2 . ',' . $1;
        }
    }
    elsif ($path
        && -d $path
        && -f $path . '/' . $this->{'id'} . '.kmz'
        && !$main::data{ $this->{'id'} }{'delete'} )
    {
        $main::data{ $this->{'id'} }{'kml'} = 'kmz';
        push(
            @{ $cgi::action->{'dialog'} },
            ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                . ' KMZ ファイルでは距離を表示できません。'
        ) if ref $cgi::action && $main::data{ $this->{'id'} }{'d'};
        $main::data{ $this->{'id'} }{'p'} = $main::data{ $this->{'id'} }{'d'}
            = '';
    }
    else {
        if ( $main::data{ $this->{'id'} }{'delete'} ) {
            $properties{'delete'} = $this->{'id'};
            $cgi::input->set_posted_file_properties_from_param( $file_name,
                0, \%properties );
        }
        $main::data{ $this->{'id'} }{'kml'} = '';
        $main::data{ $this->{'id'} }{'p'} = $main::data{ $this->{'id'} }{'d'}
            = '';
    }

    #マーカーの処理
    #シンボリックネーム
    my ( $markerfile, $width, $height, $type, $save_marker_file );
    $markerfile = 'data:' . $this->{'id'} . ':markerfile';

    #既存ファイルの情報
    if ( $path && -d $path ) {
        (   $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'}
            )
            = $this->get_field_file_info(
            $path,
            $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'},
            'marker',
            '',
            1
            );
    }
    else {
        delete $main::data{ $this->{'id'} }{'width'};
        delete $main::data{ $this->{'id'} }{'height'};
        delete $main::data{ $this->{'id'} }{'type'};
    }

    #Exifから中央座標取得
    my $c_updated = ( $main::data{ $this->{'id'} }{'p'} ) ? 1 : 0;
    if (  !$c_updated
        && $this->{'field_for'}->{'media'}
        && $cgi::input->get_posted_file_size_from_param(
            'data:' . $this->{'field_for'}->{'media'} . ':file'
        ) > 0
        )
    {
        my @info = $cgi::Image->input_info( $cgi::input,
            'data:' . $this->{'field_for'}->{'media'} . ':file', 0 );
        my ( $lat, $lng ) = $cgi::Image->latlng_from_exif( $info[4] )
            if ref $info[4];
        if ( $lat && $lng ) {
            $main::data{ $this->{'id'} }{'c'} = $lat . ',' . $lng;
            $main::data{ $this->{'id'} }{'z'} = '16'
                if !$main::data{ $this->{'id'} }{'z'};
            $c_updated = 1;
        }
    }
    if (  !$c_updated
        && $this->{'field_for'}->{'media2'}
        && $cgi::input->get_posted_file_size_from_param(
            'data:' . $this->{'field_for'}->{'media2'} . ':file'
        ) > 0
        )
    {
        my @info = $cgi::Image->input_info( $cgi::input,
            'data:' . $this->{'field_for'}->{'media2'} . ':file', 0 );
        my ( $lat, $lng ) = $cgi::Image->latlng_from_exif( $info[4] )
            if ref $info[4];
        if ( $lat && $lng ) {
            $main::data{ $this->{'id'} }{'c'} = $lat . ',' . $lng;
            $main::data{ $this->{'id'} }{'z'} = '16'
                if !$main::data{ $this->{'id'} }{'z'};
            $c_updated = 1;
        }
    }
    if (  !$c_updated
        && $this->{'field_for'}->{'media3'}
        && $cgi::input->get_posted_file_size_from_param(
            'data:' . $this->{'field_for'}->{'media3'} . ':file'
        ) > 0
        )
    {
        my @info = $cgi::Image->input_info( $cgi::input,
            'data:' . $this->{'field_for'}->{'media3'} . ':file', 0 );
        my ( $lat, $lng ) = $cgi::Image->latlng_from_exif( $info[4] )
            if ref $info[4];
        if ( $lat && $lng ) {
            $main::data{ $this->{'id'} }{'c'} = $lat . ',' . $lng;
            $main::data{ $this->{'id'} }{'z'} = '16'
                if !$main::data{ $this->{'id'} }{'z'};
            $c_updated = 1;
        }
    }
    if (  !$c_updated
        && $cgi::input->get_posted_file_size_from_param($markerfile) > 0 )
    {
        my @info = $cgi::Image->input_info( $cgi::input, $markerfile, 0 );
        my ( $lat, $lng ) = $cgi::Image->latlng_from_exif( $info[4] )
            if ref $info[4];
        if ( $lat && $lng ) {
            $main::data{ $this->{'id'} }{'c'} = $lat . ',' . $lng;
            $main::data{ $this->{'id'} }{'z'} = '16'
                if !$main::data{ $this->{'id'} }{'z'};
            $c_updated = 1;
        }
    }
    if (  !$c_updated
        && $this->{'copyfrom'}
        && (  !$main::data{ $this->{'id'} }{'type'}
            || $main::data{ $this->{'id'} }{'delmk'}
            || $this->{'alwaysCopy'} )
        && $cgi::input->get_posted_file_size_from_param(
            'data:' . $this->{'copyfrom'} . ':file'
        ) > 0
        )
    {
        my @info = $cgi::Image->input_info( $cgi::input,
            'data:' . $this->{'copyfrom'} . ':file', 0 );
        my ( $lat, $lng ) = $cgi::Image->latlng_from_exif( $info[4] )
            if ref $info[4];
        if ( $lat && $lng ) {
            $main::data{ $this->{'id'} }{'c'} = $lat . ',' . $lng;
            $main::data{ $this->{'id'} }{'z'} = '16'
                if !$main::data{ $this->{'id'} }{'z'};
            $c_updated = 1;
        }
    }
    if (   $main::data{ $this->{'id'} }{'enabled'}
        && !$main::data{ $this->{'id'} }{'c'}
        && (   !$main::data{ $this->{'id'} }{'kml'}
            || !$main::data{ $this->{'id'} }{'p'} )
        )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                . 'の地図画像をドラッグして地図の中心座標を指定してください。'
        ) if ref $cgi::action;
        $is_valid = 0;
    }

    #ファイルアップロードフィールドからコピー
    if (   $main::clipboard_id
        && !$main::data{ $this->{'id'} }{'markerfile'}
        && !$main::option{ $this->{'id'} }{'markercancel'} )
    {
        $cgi::input->inherit_posted_file_properties($markerfile);
    }
    my %marker_properties
        = $cgi::input->get_posted_file_properties_from_param($markerfile);
    if ( $marker_properties{'delete'} ) {
        delete $marker_properties{'delete'};
        $cgi::input->set_posted_file_properties_from_param( $markerfile, 0,
            \%marker_properties );
    }
    if (   $this->{'copyfrom'}
        && $cgi::Image->image_magick_enabled
        && !$marker_properties{'filename'}
        && (  !$main::data{ $this->{'id'} }{'type'}
            || $main::data{ $this->{'id'} }{'delmk'}
            || $this->{'alwaysCopy'} )
        && ref $this->{'field_obj'}{ $this->{'copyfrom'} }
        && ( $this->{'field_obj'}{ $this->{'copyfrom'} }->{'class'} eq 'file'
            || $this->{'field_obj'}{ $this->{'copyfrom'} }->{'class'} eq
            'file_shipainter' )
        )
    {
        if ($cgi::input->get_posted_file_size_from_param(
                'data:' . $this->{'copyfrom'} . ':file'
            ) > 0
            )
        {
            my @info = $cgi::Image->input_info( $cgi::input,
                'data:' . $this->{'copyfrom'} . ':file', 0 );
            %marker_properties
                = $cgi::input->clone_posted_file(
                'data:' . $this->{'copyfrom'} . ':file', $markerfile )
                if $info[2];
        }
        elsif ( $this->{'field_obj'}{ $this->{'copyfrom'} }->{'class'} eq
               'file_shipainter'
            && $main::data{ $this->{'copyfrom'} }{'key'}
            && $main::data{ $this->{'copyfrom'} }{'key'}
            !~ /[^0-9a-zA-Z_\@\-]/s )
        {
            use Fcntl qw(:flock);
            my $is_valid_header = 1;
            local (*SPD);
            if (open( SPD,
                          $env::clipboard . '/'
                        . $main::data{ $this->{'copyfrom'} }{'key'}
                )
                )
            {
                binmode SPD;
                flock( SPD, LOCK_EX );
                my $pos = 9;
                my $buf = int( $this->{'field_obj'}{ $this->{'copyfrom'} }
                        ->read_image( \*SPD, 1, 8 ) );
                if ($buf) {
                    my %header
                        = $this->url_decode(
                        $this->{'field_obj'}{ $this->{'copyfrom'} }
                            ->read_image( \*SPD, 9, $buf ) );
                    $pos += $buf;
                    if ($header{'pch'}
                        && (  !$path
                            || $header{'pch'} ne $path . '/'
                            . $this->{'copyfrom'}
                            . '.spch' )
                        )
                    {
                        $is_valid_header = 0;
                    }
                }
                if ($is_valid_header) {
                    $buf = int( $this->{'field_obj'}{ $this->{'copyfrom'} }
                            ->read_image( \*SPD, $pos, 8 ) );
                    $pos += 10;    #8バイト+CRLF
                    if ($buf) {
                        my $image
                            = $this->{'field_obj'}{ $this->{'copyfrom'} }
                            ->read_image( \*SPD, $pos, $buf );
                        my ( $width, $height, $ext, $type )
                            = $cgi::Image->info( \$image );
                        $marker_properties{'filename'}
                            = $main::data{ $this->{'id'} }{'markerfile'}
                            = 'image.' . $ext;
                        $marker_properties{'name'} = $markerfile;
                        $marker_properties{'type'} = $type;
                        $cgi::input->post_clipboard( $markerfile, 0, $image,
                            \%marker_properties );
                    }
                }
                close(SPD);
            }
        }
    }

    if ( $marker_properties{'filename'} ) {
        my $is_valid_marker = 1;

        #添付ファイルが画像ファイルかどうかチェック
        my @stat
            = $cgi::Image->input_info( $cgi::input, $markerfile, 0,
            $this->{'widthlimit'}, $this->{'heightlimit'},
            1, \%marker_properties, 1 );
        ( $width, $height, $type ) = @stat if @stat > 2;

        #MIME-TYPEチェック
        if ( !defined $type || $type eq '' ) {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . 'マーカー画像のファイルタイプを確認してください。'
            ) if ref $cgi::action;
            $is_valid_marker = $is_valid = 0;
        }

        #ファイルサイズチェック
        if (   defined $this->{'maxsize'}
            && $this->{'maxsize'} > 0
            && $cgi::input->get_posted_file_size_from_param($markerfile)
            > $this->{'maxsize'} )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . 'マーカー画像のファイルサイズを確認してください。'
                    . $this->{'maxsize'}
                    . 'バイト以内にしてください。'
            ) if ref $cgi::action;
            $is_valid_marker = $is_valid = 0;
        }

        #マーカー画像サイズチェック
        if (   $this->{'widthlimit'}
            && $width
            && $width > $this->{'widthlimit'} )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . 'マーカー画像の横幅を確認してください。'
                    . $this->{'widthlimit'}
                    . 'ピクセル以内にしてください。'
            ) if ref $cgi::action;
            $is_valid = 0;
        }
        if (   $this->{'heightlimit'}
            && $height
            && $height > $this->{'heightlimit'} )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '' )
                    . 'マーカー画像の高さを確認してください。'
                    . $this->{'heightlimit'}
                    . 'ピクセル以内にしてください。'
            ) if ref $cgi::action;
            $is_valid = 0;
        }

        if ($is_valid_marker) {
            $save_marker_file = 1;
        }
        else {
            $cgi::input->remove_posted_file($markerfile);
            undef %properties;
        }

        #ファイル削除予約
    }
    elsif ( $main::data{ $this->{'id'} }{'delmk'} ) {
        $marker_properties{'delete'} = $this->{'id'} . '\\.marker';
        $cgi::input->set_posted_file_properties_from_param( $markerfile, 0,
            \%marker_properties );

    }
    else {
        ( $width, $height, $type ) = (
            $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'}
        );
    }

    $is_valid = 0 if !$this->_is_valid_data($path);

    #データ整形
    my @ignore_keys = ( 'file', 'markerfile', 'delete', 'delmk' );
    if ( $width && $height && $type ) {
        if ($save_marker_file) {
            $cgi::input->save_file( $markerfile,
                '!' . $this->{'id'} . '.marker.' . $type );
            $this->{'is_posted_marker'} = 1;
        }
        $main::data{ $this->{'id'} }{'mk'}
            = ( ($path) ? $this->full_path($path) . '/' : '' )
            . $this->{'id'}
            . '.marker.'
            . $type;
        (   $main::data{ $this->{'id'} }{'width'},
            $main::data{ $this->{'id'} }{'height'},
            $main::data{ $this->{'id'} }{'type'}
        ) = ( $width, $height, $type );
    }
    else {
        my $marker_enabled = 0;
        if (   $main::data{ $this->{'id'} }{'marker'}
            && $main::data{ $this->{'id'} }{'marker'} =~ /([^\/]+)$/ )
        {
            my $marker_path
                = $env::image_dir . '/imagemenu/' . $this->{'dir'} . '/' . $1;
            if ( -f $marker_path
                && $main::data{ $this->{'id'} }{'marker'} eq
                $this->full_path($marker_path) )
            {
                my @info = $cgi::Image->info($marker_path);
                if ( $info[2] ) {
                    (   $main::data{ $this->{'id'} }{'width'},
                        $main::data{ $this->{'id'} }{'height'}
                    ) = @info;
                    $main::data{ $this->{'id'} }{'mk'}
                        = $main::data{ $this->{'id'} }{'marker'};
                    $marker_enabled = 1;
                }
            }
            else {
                delete $main::data{ $this->{'id'} }{'marker'};
            }
        }
        if ( !$marker_enabled ) {
            push( @ignore_keys, 'width', 'height', 'type' );
            $cgi::input->parameters_trash(
                \$main::data{ $this->{'id'} }{'width'},
                \$main::data{ $this->{'id'} }{'height'},
                \$main::data{ $this->{'id'} }{'type'}
            );
        }
    }
    if (   $main::data{ $this->{'id'} }{'width'}
        && $main::data{ $this->{'id'} }{'height'} )
    {
        $main::data{ $this->{'id'} }{'mka'} = 'center'
            if !$main::data{ $this->{'id'} }{'mka'};
        $main::data{ $this->{'id'} }{'mkv'} = 'bottom'
            if !$main::data{ $this->{'id'} }{'mkv'};
        $main::data{ $this->{'id'} }{'anchorx'}
            = ( $main::data{ $this->{'id'} }{'mka'} eq 'left' ) ? 0
            : ( $main::data{ $this->{'id'} }{'mka'} eq 'right' )
            ? $main::data{ $this->{'id'} }{'width'}
            : int( $main::data{ $this->{'id'} }{'width'} / 2 );
        $main::data{ $this->{'id'} }{'anchory'}
            = ( $main::data{ $this->{'id'} }{'mkv'} eq 'center' )
            ? int( $main::data{ $this->{'id'} }{'height'} / 2 )
            : ( $main::data{ $this->{'id'} }{'mkv'} eq 'top' ) ? 0
            :   $main::data{ $this->{'id'} }{'height'};
    }
    if ($is_valid) {
        delete $main::data{ $this->{'id'} }{'file'};
        delete $main::data{ $this->{'id'} }{'markerfile'};
        $this->data(@ignore_keys);
        return 1;
    }
    else {
        return 0;
    }
}

sub _is_valid_data ($;$) {
    my ( $this, $path ) = @_;
    return 1;
}

sub gpx_to_kml ($$) {
    my ( $this, $xml ) = @_;
    my $kml = '';
    if (   $xml
        && $xml
        =~ /^[\x09\x0A\x0C\x0D\x20]*<\?xml(?!\w)[^<>]*\?>[\x09\x0A\x0C\x0D\x20]*<gpx(?!\w)[^<>]*>/s
        )
    {
        my $title = '';
        if ( $xml =~ /<metadata>(.*?)<\/metadata>/s ) {
            my $metadata = $1;
            $title = $1 if $metadata =~ /<name>(.*?)<\/name>/s;
        }
        while ( $xml =~ /<(trk|rte)(?!\w)[^<>]*>(.*?)<\/\1>/sg ) {
            my $type   = $1;
            my $tracks = $2;
            my $name
                = ( $tracks =~ /<name(?!\w)([^<>]*)>(.*?)<\/name>/s )
                ? $1
                : '';
            my $folder = '';
            if ( $type eq 'trk' ) {
                while ( $tracks =~ /<trkseg(?!\w)[^<>]*>(.*?)<\/trkseg>/sg ) {
                    $folder .= $this->gpx_tracks_to_placemark(
                        $this->gpx_coordinates( $1, 'trkpt' ), $name );
                }
            }
            else {
                $folder .= $this->gpx_tracks_to_placemark(
                    $this->gpx_coordinates( $tracks, 'rtept' ), $name );
            }
            if ($folder) {
                $kml .= '    <Folder>' . "\x0A";
                $kml .= '      <name>' . $name . '</name>' . "\x0A" if $name;
                $kml .= $folder;
                $kml .= '      <open>1</open>' . "\x0A";
                $kml .= '      <visibility>1</visibility>' . "\x0A";
                $kml .= '    </Folder>' . "\x0A";
            }
        }
        if ($kml) {
            $kml
                = '<?xml version="1.0" encoding="UTF-8"?>' . "\x0A"
                . '<kml xmlns="http://earth.google.com/kml/2.0">' . "\x0A"
                . '  <Document>' . "\x0A"
                . (
                ($title) ? '    <name>' . $title . '</name>' . "\x0A" : '' )
                . $kml
                . '    <open>1</open>' . "\x0A"
                . '    <visibility>1</visibility>' . "\x0A"
                . '  </Document>' . "\x0A"
                . '</kml>' . "\x0A";
        }
    }
    return $kml;
}

sub gpx_coordinates ($$$) {
    my ( $this, $tracks, $tag ) = @_;
    my $coordinates = '';
    while ( $tracks =~ /<$tag(?!\w)([^<>]*)>(.*?)<\/$tag>/sg ) {
        my $attributes = $1;
        my $data       = $2;
        my $lat        = ( $attributes =~ /lat=["'](.*?)["']/s ) ? $1 : '';
        my $lon        = ( $attributes =~ /lon=["'](.*?)["']/s ) ? $1 : '';
        my $ele = ( $data =~ /<ele(?!\w)[^<>]*>(.*?)<\/ele>/s ) ? $1 : '';
        if ( $lat && $lon ) {
            $coordinates .= ( ($coordinates) ? ' ' : '' )
                . join( ',', $lon, $lat, $ele );
        }
    }
    return $coordinates;
}

sub gpx_tracks_to_placemark ($$$) {
    my ( $this, $coordinates, $name ) = @_;
    return '' if !$coordinates;
    my @colors
        = ( '730066FF', '73FF0000', '73009900', '730000FF', '73990099' );
    my $kml = '      <Placemark>' . "\x0A";
    $kml .= '        <name>' . $name . '</name>' . "\x0A" if $name;
    $kml .= '        <MultiGeometry>' . "\x0A";
    $kml .= '          <LineString>' . "\x0A";
    $kml .= '            <altitudeMode>clampedToGround</altitudeMode>'
        . "\x0A";
    $kml
        .= '            <coordinates>'
        . $coordinates
        . '</coordinates>' . "\x0A";
    $kml .= '            <tessellate>1</tessellate>' . "\x0A";
    $kml .= '          </LineString>' . "\x0A";
    $kml .= '        </MultiGeometry>' . "\x0A";
    $kml .= '        <Style>' . "\x0A";
    $kml .= '          <LineStyle>' . "\x0A";
    $kml
        .= '            <color>'
        . $colors[ int( rand(@colors) ) ]
        . '</color>' . "\x0A";
    $kml .= '            <width>5</width>' . "\x0A";
    $kml .= '          </LineStyle>' . "\x0A";
    $kml .= '        </Style>' . "\x0A";
    $kml .= '      </Placemark>' . "\x0A";
    return $kml;
}

sub nmea_to_kml ($$) {
    my ( $this, $nmea ) = @_;
    if ( $nmea && $nmea =~ /\$GPGGA,/ ) {
        my ( $prev_lat, $prev_lon, $coordinates );
        while ( $nmea =~ /^(.*)$/mg ) {
            my @data = split( /,/, $1 );
            if ( $data[0] eq '$GPGGA' ) {
                my $lat = $this->coordinate_scale_fix( $data[2], $data[3] );
                my $lon = $this->coordinate_scale_fix( $data[4], $data[5] );
                my $alt = $data[9];
                if (   $lat
                    && $lon
                    && (   !$prev_lat
                        || !$prev_lon
                        || $lat != $prev_lat
                        || $lon != $prev_lon )
                    )
                {
                    $coordinates
                        .= ( ($coordinates) ? ' ' : '' )
                        . ( ( defined $alt && $alt ne '' )
                        ? join( ',', $lon, $lat, $alt )
                        : join( ',', $lon, $lat ) );
                }
                ( $prev_lat, $prev_lon ) = ( $lat, $lon );
            }
        }
        if ($coordinates) {
            return
                  '<?xml version="1.0" encoding="UTF-8"?>' . "\x0A"
                . '<kml xmlns="http://earth.google.com/kml/2.0">' . "\x0A"
                . '  <Document>' . "\x0A"
                . '    <Folder>' . "\x0A"
                . $this->gpx_tracks_to_placemark($coordinates)
                . '      <open>1</open>' . "\x0A"
                . '      <visibility>1</visibility>' . "\x0A"
                . '    </Folder>' . "\x0A"
                . '    <open>1</open>' . "\x0A"
                . '    <visibility>1</visibility>' . "\x0A"
                . '  </Document>' . "\x0A"
                . '</kml>' . "\x0A";
        }
    }
    return '';
}

sub coordinate_scale_fix ($$$) {
    my ( $this, $n, $d ) = @_;
    return
        int( ( int( $n / 100 ) + ( ( $n - int( $n / 100 ) * 100 ) / 60 ) )
        * 1000000 + 0.5 )
        / 1000000 * ( ( $d eq 'S' || $d eq 'W' ) ? -1 : 1 );
}

######################################################################
#	(4) obj->usual_format(str);
sub usual_format {
    my ( $this, $str, $path, $c, $n ) = @_;
    return $this->field_tag('') if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    if (%data) {
        my $static_map = $this->get_static_map( \%data );
        if ( $data{'enabled'} ) {
            if ( $this->{'is_temporary_data'} && $this->{'is_posted_kml'} ) {
                $data{'url'}
                    = $this->full_path(
                          $cgi::input->get_public_clipboard . '/'
                        . $this->{'id'}
                        . ( ( $data{'kml'} eq 'kmz' ) ? '.kmz' : '.kml' ) )
                    if $data{'kml'};
                $data{'m'} = '中心座標マーカー' if !$data{'m'};
            }
            elsif ( $data{'kml'} ) {
                $data{'url'}
                    = $this->full_path( $path . '/'
                        . $this->{'id'}
                        . ( ( $data{'kml'} eq 'kmz' ) ? '.kmz' : '.kml' ) );
            }
            if ( $this->{'is_temporary_data'} && $this->{'is_posted_marker'} )
            {
                $data{'mk'} = $this->full_path(
                          $cgi::input->get_public_clipboard . '/'
                        . $this->{'id'}
                        . '.marker.'
                        . (
                        ( $data{'type'} ) ? $data{'type'}
                        : (        $data{'mk'}
                                && $data{'mk'} =~ /\.([0-9a-zA-Z_]+)$/ ) ? $1
                        : 'gif'
                        )
                );
            }
            elsif ( $data{'mk'} && $data{'mk'} !~ /^https?:\/\//i ) {
                $data{'mk'} = $this->full_path( $path . '/' . $data{'mk'} );
            }
        }
        return $this->field_tag(
            $this->data_tag_from_ref( \%data )
                . (
                  ( $data{'enabled'} && ( $data{'url'} || $data{'c'} ) )
                ? ( $this->{'is_temporary_data'} )
                        ? $this->gmap_iframe(
                            \%data,
                            $this->mk_user_generated_field_id( $c, $n ),
                            (   (          $this->{'maxwidth'}
                                        && $this->{'maxwidth'}
                                        < $this->{'canvasWidth'}
                                )
                                ? $this->{'maxwidth'}
                                : $this->{'canvasWidth'}
                            ),
                            $this->{'canvasHeight'},
                            $this->{'canvasOverView'},
                            $this->{'canvasStreetView'},
                            $this->{'viewerGSearch'},
                            $this->{'viewerLinkTarget'}
                        )
                        . ( (   $main::data{ $this->{'id'} }{'delete'}
                                    && !$this->{'is_posted_file'}
                            )
                            ? '<div class="note">このKMLファイルを削除する</div>'
                            : ''
                        )
                        . ( (   $main::data{ $this->{'id'} }{'delmk'}
                                    && !$this->{'is_posted_marker'}
                            )
                            ? '<div class="note">このマーカー画像を削除する</div>'
                            : ''
                        )
                        : $this->gmap_iframe(
                            \%data,
                            $this->mk_user_generated_field_id( $c, $n ),
                            (   (          $this->{'maxwidth'}
                                        && $this->{'maxwidth'}
                                        < $this->{'viewerWidth'}
                                )
                                ? $this->{'maxwidth'}
                                : $this->{'viewerWidth'}
                            ),
                            $this->{'viewerHeight'},
                            $this->{'viewerOverView'},
                            $this->{'viewerStreetView'},
                            $this->{'viewerGSearch'},
                            $this->{'viewerLinkTarget'}
                        )
                        . $static_map
                : ''
                )
        );
    }
    else {
        return $this->field_tag('');
    }
}

sub get_latlng {
    my ( $this, $str, $path, $c, $n ) = @_;
    return ( '', '' ) if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    if (   %data
        && $data{'enabled'}
        && $data{'c'}
        && $data{'c'} =~ /^(\-?[0-9\.]+),(\-?[0-9\.]+)$/ )
    {
        return ( $1, $2 );
    }
    return ( '', '' );
}

sub get_static_map ($$) {
    my ( $this, $data ) = @_;
    if ( ref $data && $data->{'enabled'} && $data->{'c'} ) {
        my $m = ( $data->{'m'} ) ? $data->{'m'} : '';
        $this->text_fix( \$m, $this->{'csv'}, 1 ) if $m;
        my $w
            = (    $this->{'maxwidth'}
                && $this->{'maxwidth'} < $this->{'viewerWidth'} )
            ? $this->{'maxwidth'}
            : $this->{'viewerWidth'};
        return
              '<noscript><div><img'
            . ' src="http://maps.google.com/maps/api/staticmap?center='
            . $data->{'c'}
            . '&amp;zoom='
            . ( ( $data->{'z'} ) ? $data->{'z'} : 4 )
            . '&amp;size='
            . $w . 'x'
            . $this->{'viewerHeight'}
            . '&amp;markers='
            . $data->{'c'}
            . '&amp;sensor=false' . '"'
            . ' width="'
            . $w . '"'
            . ' height="'
            . $this->{'viewerHeight'} . '"'
            . ' title="'
            . $this->detag( $data->{'m'} ) . '"'
            . ' alt="地図画像" border="0" />'
            . ( ($m) ? '<div class="gmap-caption">' . $m . '</div>' : '' )
            . '</div></noscript>';
    }
    return '';
}

sub has_content {
    my ( $this, $str ) = @_;
    return 0 if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return ( %data && $data{'enabled'} && ( $data{'url'} || $data{'c'} ) )
        ? 1
        : 0;
}

sub format_for_mail {
    my ( $this, $str ) = @_;
    $str = $this->format_for_text($str);
    return ($str) ? '[' . $this->name . "]\x0A" . $str . "\x0A\x0A" : '';
}

sub format_for_text {
    my ( $this, $str, $path, $c, $n ) = @_;
    return '' if !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return ( %data && $data{'enabled'} && ( $data{'c'} || $data{'kml'} ) )
        ? ( ($default::english) ? 'yes' : 'あり' )
        : ( ($default::english) ? 'no' : 'なし' );
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled ) = @_;

    #名前
    my $name = 'data:' . $this->{'id'};

    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend>Googleマップ表示領域設定</legend>';
    print '<fieldset>';
    print '<legend>編集エリア</legend>';
    print '<label>幅 <input type="text" size="6" name="'
        . $name
        . '_canvasWidth" id="'
        . $name
        . '_canvasWidth" value="'
        . $this->escape( $this->{'canvasWidth'} )
        . '" /> px</label> &times; ';
    print '<label>高さ <input type="text" size="6" name="'
        . $name
        . '_canvasHeight" id="'
        . $name
        . '_canvasHeight" value="'
        . $this->escape( $this->{'canvasHeight'} )
        . '" /> px</label>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_canvasOverView" id="'
        . $name
        . '_canvasOverView" value="1"'
        . ( ( $this->{'canvasOverView'} ) ? ' checked="checked"' : '' )
        . ' /> 概観地図を表示する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_canvasStreetView" id="'
        . $name
        . '_canvasStreetView" value="1"'
        . ( ( $this->{'canvasStreetView'} ) ? ' checked="checked"' : '' )
        . ' /> ストリートビューツールを表示する</label></li>';
    print '</ol>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>表示エリア</legend>';
    print '<label>幅 <input type="text" size="6" name="'
        . $name
        . '_viewerWidth" id="'
        . $name
        . '_viewerWidth" value="'
        . $this->escape( $this->{'viewerWidth'} )
        . '" /> px</label> &times; ';
    print '<label>高さ <input type="text" size="6" name="'
        . $name
        . '_viewerHeight" id="'
        . $name
        . '_viewerHeight" value="'
        . $this->escape( $this->{'viewerHeight'} )
        . '" /> px</label>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_viewerOverView" id="'
        . $name
        . '_viewerOverView" value="1"'
        . ( ( $this->{'viewerOverView'} ) ? ' checked="checked"' : '' )
        . ' /> 概観地図を表示する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_viewerStreetView" id="'
        . $name
        . '_viewerStreetView" value="1"'
        . ( ( $this->{'viewerStreetView'} ) ? ' checked="checked"' : '' )
        . ' /> ストリートビューツールを表示する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_viewerGSearch" id="'
        . $name
        . '_viewerGSearch" value="1"'
        . ( ( $this->{'viewerGSearch'} ) ? ' checked="checked"' : '' )
        . ' /> 住所検索ツールを表示する</label></li>';
    print '</ol>';
    print '<label>情報ウィンドウ内リンクを';
    print $this->mk_select_menu(
        $name . '_viewerLinkTarget', '', '', $this->{'viewerLinkTarget'},
        ''  => '新しいウィンドウで開く',
        'p' => '同じウィンドウで開く'
    );
    print '</label>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>デフォルトの地図</legend>';
    print $this->gmap_iframe(
        {   'z' => $this->{'z'},
            't' => $this->{'t'},
            'c' => $this->{'c'},
        },
        'gmap-canvas-' . $this->{'id'},
        $this->{'canvasWidth'},
        $this->{'canvasHeight'},
        $this->{'canvasOverView'},
        $this->{'canvasStreetView'},
        1, '',
        {   'z_field' => $name . '_z',
            't_field' => $name . '_t',
            'c_field' => $name . '_c',
        }
    );

    foreach my $k ( 'c', 't', 'z' ) {
        print '<input type="hidden" name="'
            . $name . '_'
            . $k
            . '" id="'
            . $name . '_'
            . $k
            . '" value="'
            . $this->escape( $this->{$k} ) . '" />';
    }
    print '</fieldset>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mapWindowEnabled" id="'
        . $name
        . '_mapWindowEnabled" value="1"'
        . ( ( $this->{'mapWindowEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> 新しいウィンドウで地図を開くリンクを付記する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_kmlDownloadEnabled" id="'
        . $name
        . '_kmlDownloadEnabled" value="1"'
        . ( ( $this->{'kmlDownloadEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> KML ファイルがあるときダウンロードリンクを付記する</label></li>';
    print '</ol>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_mapWindowLabel">新しいウィンドウで地図を開くリンクのラベル</label></legend>';
    print '<input type="text" size="40" name="'
        . $name
        . '_mapWindowLabel" id="'
        . $name
        . '_mapWindowLabel" value="'
        . $this->escape( $this->{'mapWindowLabel'} ) . '" />';
    print '</fieldset>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>中心座標コメント</legend>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_default">デフォルトのコメント</label></legend>';
    print '<textarea name="'
        . $name
        . '_default" id="'
        . $name
        . '_default" cols="32" rows="4" wrap="soft">'
        . $this->escape( $this->{'default'} )
        . '</textarea>';
    print '</fieldset>';
    $this->print_additional_comment_setting_form($name);
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_mw">情報ウィンドウ最大幅</label></legend>';
    print '<input type="number" size="4" name="'
        . $name
        . '_mw" id="'
        . $name
        . '_mw" value="'
        . $this->escape( $this->{'mw'} )
        . '" /> px';
    print '</fieldset>';
    $this->print_html_tag_setting_form($name);
    $this->print_table_setting_form($name);
    print '</fieldset>';

    if ($env::zipcode) {
        my ($menu);
        foreach my $id (
            sort {
                $this->{'field_obj'}->{$a}->{'ORDER'}
                    <=> $this->{'field_obj'}->{$b}->{'ORDER'}
                    || $a <=> $b
            } keys %{ $this->{'field_obj'} }
            )
        {
            next
                unless ref $this->{'field_obj'}->{$id}
                && $this->{'field_obj'}->{$id}->{'class'} eq 'addr_j';
            $menu
                .= '<option value="'
                . $id . '"'
                . (
                (          defined $id
                        && defined $this->{'addrfield'}
                        && $id eq $this->{'addrfield'}
                ) ? ' selected="selected"' : ''
                )
                . '>'
                . $this->{'field_obj'}->{$id}->name
                . '</option>';
        }
        if ( defined $menu && $menu ne '' ) {
            print '<fieldset>';
            print '<legend><label for="' . $name
                . '_addrfield">住所検索キーワードの取り込み</label></legend>';
            print '<select name="'
                . $name
                . '_addrfield" id="'
                . $name
                . '_addrfield" title="住所検索キーワードとして利用する住所フィールドを選択します。">';
            print '<option value=""></option>';
            print $menu;
            print '</select>';
            print
                '<div class="note">住所検索キーワードとして取り込む住所フィールドを選択してください。</div>';
            print '</fieldset>';
        }
    }

    print '<fieldset>';
    print '<legend>中心座標マーカー</legend>';
    local ( *DIR, *NAME );
    if ( opendir( DIR, $env::image_dir . '/imagemenu' ) ) {
        my @dir
            = grep( $_ !~ /^\./ && -d $env::image_dir . '/imagemenu/' . $_,
            readdir(DIR) );
        closedir(DIR);
        if (@dir) {
            print '<fieldset>';
            print '<legend>マーカー画像メニュー</legend>';
            print '<table class="field-table">';
            print '<tbody>';
            print '<tr>';
            print '<td class="field-name"><label for="' . $name
                . '_dir">画像メニュー選択</label></td><td class="field">';
            print '<select name="' . $name . '_dir" id="' . $name . '_dir">';
            print '<option value=""></option>';

            foreach my $dir_name (@dir) {
                my ($name);
                if (open( NAME,
                              $env::image_dir
                            . '/imagemenu/'
                            . $dir_name . '/'
                            . 'name.txt'
                    )
                    )
                {
                    binmode NAME, ":utf8";
                    flock( NAME, LOCK_SH );
                    $name = <NAME>;
                    close(NAME);
                    chomp $name;
                }
                $name = $dir_name
                    if !defined $name || $name eq '' || $name =~ /:/;
                print '<option value="'
                    . $dir_name . '"'
                    . (
                    ( defined $this->{'dir'} && $dir_name eq $this->{'dir'} )
                    ? ' selected="selected"'
                    : '' )
                    . '>'
                    . $name
                    . '</option>';
            }
            print '</select>';
            print '</td>';
            print '</tr>';
            if ( $this->{'dir'} ) {
                print '<tr>';
                print '<td class="field-name"><label for="' . $name
                    . '_defaultMarker">デフォルトのマーカー画像</label></td><td class="field">';
                print $this->get_image_menu(
                    $name . '_defaultMarker',
                    $this->{'defaultMarker'},
                    'default-marker-sample'
                );
                print '</td>';
                print '</tr>';
                print '<tr>';
                print
                    '<td class="sub-field-name"><span class="datatitle">指し示すポイント</span></td>';
                print '<td class="sub-field">';
                print $this->mk_select_menu(
                    $name . '_defaultMarkerAlign', '', '',
                    $this->{'defaultMarkerAlign'},
                    ''       => '',
                    'left'   => '左端',
                    'center' => '中央',
                    'right'  => '右端'
                );
                print $this->mk_select_menu(
                    $name . '_defaultMarkerValign', '', '',
                    $this->{'defaultMarkerValign'},
                    ''       => '',
                    'top'    => '上端',
                    'center' => '中央',
                    'bottom' => '下端'
                );
                print '</td>';
                print '</tr>';
            }
            print '</tbody></table>';
            print '</fieldset>';
        }
        print '<fieldset>';
        print '<legend>マーカー画像アップロード</legend>';
        print '<table class="field-table">';
        print '<tbody>';
        print '<tr>';
        print '<td class="field-name"><label for="' . $name
            . '_maxsize">ファイルサイズ制限</label></td>';
        print '<td class="field" colspan="3"><input type="text" name="'
            . $name
            . '_maxsize" id="'
            . $name
            . '_maxsize" value="'
            . $this->escape( $this->{'maxsize'} )
            . '" size="16" /> <span class="unit">byte</span>';
        print
            '<div class="hints">設定が無い場合、128,000byte（128KB）に設定されます。</div>';
        print '</td>';
        print '</tr>';
        print '<tr class="tbody-end">';
        print '<td class="field-name"><label for="' . $name
            . '_widthlimit">画像横幅制限</label></td>';
        print '<td class="q-field"><input type="number" name="'
            . $name
            . '_widthlimit" id="'
            . $name
            . '_widthlimit" value="'
            . $this->escape( $this->{'widthlimit'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '<td class="field-name"><label for="' . $name
            . '_heightlimit">画像高さ制限</label></td>';
        print '<td class="q-field"><input type="number" name="'
            . $name
            . '_heightlimit" id="'
            . $name
            . '_heightlimit" value="'
            . $this->escape( $this->{'heightlimit'} )
            . '" size="4" /> <span class="unit">ピクセル</span>';
        print '</td>';
        print '</tr>';
        print '</tbody>';
        print '</table>';

        if ( $cgi::Image->image_magick_enabled ) {
            my $copyfield_menu = '';
            foreach my $id (
                sort {
                    $this->{'field_obj'}->{$a}->{'ORDER'}
                        <=> $this->{'field_obj'}->{$b}->{'ORDER'}
                        || $a <=> $b
                } keys %{ $this->{'field_obj'} }
                )
            {
                if (ref $this->{'field_obj'}{$id}
                    && (   $this->{'field_obj'}{$id}->{'class'} eq 'file'
                        || $this->{'field_obj'}{$id}->{'class'} eq
                        'file_shipainter' )
                    )
                {
                    $copyfield_menu
                        .= '<option value="'
                        . $id . '"'
                        . (
                        (   defined $this->{'copyfrom'}
                                && $this->{'copyfrom'} eq $id
                        ) ? ' selected="selected"' : ''
                        )
                        . '>'
                        . $this->jfold( $this->{'field_obj'}{$id}->name,
                        32, 'I', 1 )
                        . '</option>';
                }
            }
            if ($copyfield_menu) {
                print '<fieldset>';
                print '<legend><label for="' . $name
                    . '_copyfrom">マーカー画像が指定されていないとき、このフィールドからサムネイル画像を作成する</label></legend>';
                print '<select name="'
                    . $name
                    . '_copyfrom" id="'
                    . $name
                    . '_copyfrom">';
                print '<option value=""></option>';
                print $copyfield_menu;
                print '</select>';
                print
                    '<div class="extra-field"><label><input type="checkbox" name="'
                    . $name
                    . '_alwaysCopy" value="1"'
                    . (
                    ( $this->{'alwaysCopy'} ) ? ' checked="checked"' : '' )
                    . ' /> 可能ならば常にこのフィールドの画像で現在のマーカー画像を上書きする。</label></div>';
                print '</fieldset>';
            }
        }
        print '</fieldset>';
    }

    print '</fieldset>';

    $this->print_additional_gmap_option($name);
    $this->print_setting_save_button;
    return;
}

sub print_additional_comment_setting_form ($) {
    my $this = shift;
}

sub print_additional_gmap_option ($) {
    my $this = shift;
}

sub get_image_menu ($$;$$) {
    my ( $this, $name, $selected, $sample_id ) = @_;
    return '' if !defined $this->{'dir'} || $this->{'dir'} eq '';
    my ( $form, $dir, $file, %image );
    local ( *DIR, *NAME );
    $dir = $env::image_dir . '/imagemenu/' . $this->{'dir'};
    return '' unless opendir( DIR, $dir );
    $dir .= '/';
    while ( $file = readdir(DIR) ) {

        if (   $file !~ /^\./
            && $file =~ /\.(gif|jpg|png)$/i
            && -f $dir . $file )
        {
            $image{$file} = 1;
        }
    }
    closedir(DIR);
    $form
        .= '<div class="field-image-sample"><img id="'
        . $sample_id
        . '" name="'
        . $sample_id
        . '" src="'
        . ( ($selected) ? $selected : $env::spacer )
        . '" alt="" /></div>'
        if $sample_id;
    $form
        .= '<select name="'
        . $name
        . '" id="'
        . $name . '"'
        . ( ($sample_id)
        ? ' onchange="cmonos.element.item(\''
            . $sample_id
            . '\').setSrc(this.options[this.selectedIndex].value);"'
        : '' )
        . '>';
    $form
        .= '<option value="'
        . ( ($sample_id) ? $env::spacer : '' )
        . '"></option>';
    if ( open( NAME, $dir . 'name.txt' ) ) {
        binmode NAME, ":utf8";
        flock( NAME, LOCK_SH );
        while ( my $line = <NAME> ) {
            chomp $line;
            next if $line !~ /:/;
            my ( $img, $name, $reserved ) = split( /:/, $line, 3 );
            next if !defined $image{$img} || !$image{$img};
            delete $image{$img};
            my $path = $this->full_path( $dir . $img );
            if ($reserved) {
                if ( $this->is_one_of_owners ) {
                    $name = '*' . $name;
                }
                else {
                    next;
                }
            }
            $form
                .= '<option value="'
                . $path . '"'
                . ( ( $selected && $selected eq $path )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $name
                . '</option>';
        }
        close(NAME);
    }
    foreach my $img ( sort keys %image ) {
        my $path = $this->full_path( $dir . $img );
        $form
            .= '<option value="'
            . $path . '"'
            . ( ( $selected && $selected eq $path )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $img
            . '</option>';
    }
    $form .= '</select>';
    return $form;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this     = shift;
    my $is_valid = 1;
    $is_valid = 0
        if !$this->is_valid_gmapsize( $this->{'id'} . '_canvasWidth',
        '編集エリア幅' );
    $is_valid = 0
        if !$this->is_valid_gmapsize( $this->{'id'} . '_canvasHeight',
        '編集エリア高さ' );
    $is_valid = 0
        if !$this->is_valid_gmapsize( $this->{'id'} . '_viewerWidth',
        '表示エリア幅' );
    $is_valid = 0
        if !$this->is_valid_gmapsize( $this->{'id'} . '_viewerHeight',
        '表示エリア高さ' );
    if (   $main::data{ $this->{'id'} . '_mw' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_mw' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '情報ウィンドウ最大幅を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_mw' };
        $is_valid = 0;
    }
    $is_valid = 0 if !$this->is_valid_file_setting('マーカー画像');
    $main::data{ $this->{'id'} . '_mapWindowEnabled' } = ''
        if !$main::data{ $this->{'id'} . '_mapWindowEnabled' };
    $main::data{ $this->{'id'} . '_kmlDownloadEnabled' } = ''
        if !$main::data{ $this->{'id'} . '_kmlDownloadEnabled' };
    $main::data{ $this->{'id'} . '_canvasOverView' } = '0'
        if !$main::data{ $this->{'id'} . '_canvasOverView' };
    $main::data{ $this->{'id'} . '_canvasStreetView' } = '0'
        if !$main::data{ $this->{'id'} . '_canvasStreetView' };
    $main::data{ $this->{'id'} . '_viewerOverView' } = '0'
        if !$main::data{ $this->{'id'} . '_viewerOverView' };
    $main::data{ $this->{'id'} . '_viewerStreetView' } = '0'
        if !$main::data{ $this->{'id'} . '_viewerStreetView' };
    $main::data{ $this->{'id'} . '_viewerGSearch' } = '0'
        if !$main::data{ $this->{'id'} . '_viewerGSearch' };
    $main::data{ $this->{'id'} . '_viewerLinkTarget' } = ''
        if !$main::data{ $this->{'id'} . '_viewerLinkTarget' }
        || $main::data{ $this->{'id'} . '_viewerLinkTarget' } ne 'p';
    $main::data{ $this->{'id'} . '_alwaysCopy' } = '0'
        if !$main::data{ $this->{'id'} . '_alwaysCopy' };
    $main::data{ $this->{'id'} . '_dir' } = ''
        if !$main::data{ $this->{'id'} . '_dir' }
        || $main::data{ $this->{'id'} . '_dir' } =~ /^\./
        || !-d $env::image_dir
        . '/imagemenu/'
        . $main::data{ $this->{'id'} . '_dir' };
    $main::data{ $this->{'id'} . '_defaultMarker' } = ''
        if !$main::data{ $this->{'id'} . '_defaultMarker' }
        || $main::data{ $this->{'id'} . '_defaultMarker' } eq $env::spacer;

    if ( $main::data{ $this->{'id'} . '_defaultMarker' } ) {
        my $dir
            = $this->full_path( $env::image_dir
                . '/imagemenu/'
                . $main::data{ $this->{'id'} . '_dir' } );
        $main::data{ $this->{'id'} . '_defaultMarker' } = ''
            if $main::data{ $this->{'id'} . '_defaultMarker' }
            !~ /^\Q$dir\E\//;
        if (   $main::data{ $this->{'id'} . '_defaultMarker' }
            && $main::data{ $this->{'id'} . '_defaultMarker' }
            =~ /([^\/]+)$/ )
        {
            my $marker_path
                = $env::image_dir
                . '/imagemenu/'
                . $main::data{ $this->{'id'} . '_dir' } . '/'
                . $1;
            if ( -f $marker_path
                && $main::data{ $this->{'id'} . '_defaultMarker' } eq
                $this->full_path($marker_path) )
            {
                my @info = $cgi::Image->info($marker_path);
                (   $main::data{ $this->{'id'} . '_defaultMarkerWidth' },
                    $main::data{ $this->{'id'} . '_defaultMarkerHeight' }
                    )
                    = @info
                    if $info[2];
            }
            else {
                delete $main::data{ $this->{'id'} . '_defaultMarker' };
            }
        }
    }
    if (   $main::data{ $this->{'id'} . '_defaultMarkerWidth' }
        && $main::data{ $this->{'id'} . '_defaultMarkerHeight' } )
    {
        $main::data{ $this->{'id'} . '_defaultMarkerAlign' } = 'center'
            if !$main::data{ $this->{'id'} . '_defaultMarkerAlign' };
        $main::data{ $this->{'id'} . '_defaultMarkerValign' } = 'bottom'
            if !$main::data{ $this->{'id'} . '_defaultMarkerValign' };
        $main::data{ $this->{'id'} . '_defaultMarkerAnchorX' }
            = (
            $main::data{ $this->{'id'} . '_defaultMarkerAlign' } eq 'left' )
            ? 0
            : (
            $main::data{ $this->{'id'} . '_defaultMarkerAlign' } eq 'right' )
            ? $main::data{ $this->{'id'} . '_defaultMarkerWidth' }
            : int( $main::data{ $this->{'id'} . '_defaultMarkerWidth' } / 2 );
        $main::data{ $this->{'id'} . '_defaultMarkerAnchorY' }
            = ( $main::data{ $this->{'id'} . '_defaultMarkerValign' } eq
                'center' )
            ? int( $main::data{ $this->{'id'} . '_defaultMarkerHeight' } / 2 )
            : (
            $main::data{ $this->{'id'} . '_defaultMarkerValign' } eq 'top' )
            ? 0
            : $main::data{ $this->{'id'} . '_defaultMarkerHeight' };
    }
    $this->tag_fix( \$main::data{ $this->{'id'} . '_mapWindowLabel' } )
        if defined $main::data{ $this->{'id'} . '_mapWindowLabel' }
        && $main::data{ $this->{'id'} . '_mapWindowLabel' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_default' } )
        if defined $main::data{ $this->{'id'} . '_default' }
        && $main::data{ $this->{'id'} . '_default' };
    $this->html_tag_setting_fix;
    $is_valid = 0 if !$this->_is_valid_setting;
    return $is_valid;
}

sub is_valid_gmapsize ($$$) {
    my ( $this, $key, $name ) = @_;
    if ( defined $main::data{$key} && $main::data{$key} ne '' ) {
        $main::data{$key} =~ s/px$//si;
        my $is_percent = ( $main::data{$key} =~ s/[\%％]$//si ) ? 1 : 0;
        if ( $this->is_nan( $main::data{$key} ) ) {
            push(
                @{ $cgi::action->{'dialog'} },
                $name
                    . 'を更新できませんでした。半角数字(+%)にしてください。'
            ) if ref $cgi::action;
            delete $main::data{$key};
            return 0;
        }
        $main::data{$key} .= '%' if $is_percent;
    }
    return 1;
}

sub _is_valid_setting ($) {
    my $this = shift;
    return 1;
}

######################################################################
#	(8) obj->sort_enabled;
sub sort_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(9) obj->cookie_enabled;
sub cookie_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	obj->search_form;
sub search_form_enabled ($) {
    my $this = shift;
    return ( $this->{'required'} ) ? 0 : 1;
}

sub search_form ($) {
    my $this = shift;
    return ( $this->{'required'} )
        ? ''
        : '<tr><td class="field-name"><label for="search:'
        . $this->{'id'}
        . ':check">'
        . $this->name
        . '</label></td><td class="field">'
        . $this->mk_select_menu(
        'search:' . $this->{'id'} . ':check', '', '',
        $main::search{ $this->{'id'} }{'check'},
        ''  => '',
        '1' => ( ($default::english) ? 'ON' : '有' ),
        '0' => ( ($default::english) ? 'OFF' : '無' )
        ) . '</td></tr>';
}

######################################################################

1;
