package GraderBase;
require CMONOS::CGI_Lib::Field::UserGeneratedBarGraph;
@ISA = qw(UserGeneratedBarGraph);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（採点）
# Grader.pm 1.1.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-05-02
######################################################################
sub __set ($$) {
    my $this = shift;
    $this->{'default'} = "+5\x0A+4\x0A+3\x0A+2\x0A+1"
        if !defined $this->{'default'} || !$this->{'default'};
    @{ $this->{'grade'} } = split( /\x0A/, $this->{'default'} );
    return;
}

# --------------------------------------------------------------------
#	(0)-5 データ要素の取得
sub elements ($;$) {
    my ( $this, $str ) = @_;
    my (%data);
    (   $data{'average'},   $data{'total'}, $data{'access'},
        $data{'latest_ip'}, $data{'atime'}, $data{'status'}
        )
        = split( /\x0A/, $str )
        if defined $str;
    $this->dfill( \$data{'average'}, \$data{'total'}, \$data{'access'} );
    $this->fill( \$data{'latest_ip'}, \$data{'atime'}, \$data{'status'} );
    return %data;
}

######################################################################
#	(4) obj->usual_format(str,path,category,no);
sub get_graph_javascript ($$) {
    my ( $this, $id ) = @_;
    return $this->javascript(
        'cmonos.form.clickableBarGraph(\'' . $id . '\');' );
}

sub get_graph_info ($$) {
    my ( $this, $data ) = @_;
    return '' if !$data->{'access'};
    my $round
        = ( !$this->{'roundOffTo'} ) ? 1000
        : ( $this->{'roundOffTo'} eq '1' ) ? 1
        : ( $this->{'roundOffTo'} eq '2' ) ? 10
        : ( $this->{'roundOffTo'} eq '3' ) ? 100
        : ( $this->{'roundOffTo'} eq '4' ) ? 1000
        :                                    1000;
    my $out
        = ( $data->{'average'} )
        ? int( $data->{'average'} * $round + 0.5 ) / $round
        : 0;
    $out .= ($default::english) ? ' points' : '点';
    $out
        .= ' ('
        . ( ( $data->{'total'} ) ? $data->{'total'} : 0 )
        . ( ($default::english)  ? ' points'        : '点' ) . '/'
        . $data->{'access'}
        . ( ($default::english) ? ' voters' : '人' ) . ')';
    return $out;
}

sub get_bar_value ($$) {
    my ( $this, $data ) = @_;
    return ( $data->{'average'} ) ? $data->{'average'} : 0;
}

sub get_bar_max_value ($) {
    my $this = shift;
    return scalar( @{ $this->{'grade'} } );
}

sub get_additional_vote_form ($$$) {
    my ( $this, $data, $id ) = @_;
    return $this->get_bargraph_form(
        'point',
        (   (         !$default::static
                    && $this->{'update_is_failed'}
                    && defined $main::point
            )                        ? $main::point
            : ( $data->{'average'} ) ? int( $data->{'average'} + 0.5 )
            : 0
        ),
        0, $id
    );
}

sub get_bargraph_form ($$$;$$) {
    no warnings qw(numeric);
    my ( $this, $name, $default, $search_flag, $id ) = @_;
    my ( $gcount, $out );
    $default += 0;
    $gcount = @{ $this->{'grade'} };
    $out
        = '<select name="'
        . $name . '"'
        . ( ($id) ? ' id="' . $id . '-selector"' : '' ) . '>';
    foreach my $g ( @{ $this->{'grade'} } ) {
        $out
            .= '<option value="'
            . $gcount . '"'
            . ( ( $gcount == $default ) ? ' selected="selected"' : '' ) . '>'
            . $this->escape($g)
            . '</option>';
        $gcount--;
    }
    if ( defined $search_flag && $search_flag ) {
        $out
            .= '<option value=""'
            . ( ( $default == 0 ) ? ' selected="selected"' : '' )
            . '></option></select>';
    }
    else {
        $out
            .= '<option value="0"'
            . ( ( $default == 0 ) ? ' selected="selected"' : '' )
            . '></option></select>';
    }
    return $out;
}

######################################################################
sub print_sub_setting_form($$) {
    my ( $this, $name ) = @_;
    print '<fieldset>';
    print '<legend>採点機能</legend>';
    print '<label><input type="checkbox" name="'
        . $name
        . '_formdisabled" value="1"'
        . ( ( $this->{'formdisabled'} ) ? ' checked="checked"' : '' )
        . ' /> 停止 (「コメント」フィールドタイプからの入力のみ受け付ける場合)</label>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_default">グレード設定</label></legend>';
    print '<textarea name="'
        . $name
        . '_default" id="'
        . $name
        . '_default" cols="56" rows="6" wrap="off">'
        . $this->escape( $this->{'default'} )
        . '</textarea>';
    print
        '<div class="note">（行数 = 最高点／一行目が最高点／最終行は1点）</div>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_roundOffTo">表示</label></legend>';
    print $this->mk_select_menu(
        $name . '_roundOffTo', '', '', ( $this->{'roundOffTo'} || 4 ),
        '1' => '小数点以下第1位を四捨五入(整数)',
        '2' => '小数点以下第2位を四捨五入',
        '3' => '小数点以下第3位を四捨五入',
        '4' => '小数点以下第4位を四捨五入',
    );
    print '</fieldset>';
}

sub is_valid_bargraph_setting ($) {
    my $this = shift;
    $main::data{ $this->{'id'} . '_formdisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_formdisabled' }
        || !$main::data{ $this->{'id'} . '_formdisabled' };
    return 1;
}

######################################################################
#	(11) obj->update;
sub update_bargraph ($$;$) {
    my ( $this, $data, $synced ) = @_;
    my $access = 1;
    if ($synced) {
        $access = $main::access if defined $main::access;
    }
    else {
        $main::point = 1 if $main::point < 1;
        $main::point = @{ $this->{'grade'} }
            if $main::point > @{ $this->{'grade'} };
        undef $main::access;
    }
    $data->{'total'} = 0 if !$data->{'total'};
    $data->{'total'} += ($main::point) ? $main::point : 0;
    $data->{'access'} += $access;
    $data->{'total'}  = 0 if $data->{'total'} < 0;
    $data->{'access'} = 0 if $data->{'access'} < 0;
    $data->{'average'}
        = ( $data->{'total'} && $data->{'access'} )
        ? $data->{'total'} / $data->{'access'}
        : 0;
    if ( defined $cgi::client_host && $cgi::client_host ) {
        $data->{'latest_ip'} = $cgi::client_host;
    }
    else {
        $data->{'latest_ip'} = $this->to_ip( $ENV{'REMOTE_ADDR'} );
        $data->{'latest_ip'} = $this->to_host( $data->{'latest_ip'} );
    }
    $data->{'atime'} = time;
    $data->{'status'} = 'enabled' if !$data->{'status'};
    return $this->get_bargraph_data($data);
}

sub get_bargraph_data ($$) {
    my ( $this, $data ) = @_;
    return
          ( ( defined $data->{'average'} ) ? $data->{'average'} : 0 ) . "\x0A"
        . ( ( defined $data->{'total'} )   ? $data->{'total'}   : 0 ) . "\x0A"
        . ( ( defined $data->{'access'} )  ? $data->{'access'}  : 0 ) . "\x0A"
        . ( ( defined $data->{'latest_ip'} ) ? $data->{'latest_ip'} : '' )
        . "\x0A"
        . ( ( defined $data->{'atime'} )  ? $data->{'atime'}  : '' ) . "\x0A"
        . ( ( defined $data->{'status'} ) ? $data->{'status'} : 'enabled' );
}

1;
