package SCSales;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（ショッピングカート/売れ行き）
# SCSales.pm 1.0.3
# Copyright (C) 1999-2019 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2019-09-17
######################################################################
#	(0)-3 フィールドDB
sub db ($) {
    my $this = shift;
    return 'sales';
}

# --------------------------------------------------------------------
sub _set ($$) {
    my $this = shift;
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }
    unless ( ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    $this->{'tagpriority'} = \@SCSales::tag if !ref $this->{'tagpriority'};
    $this->{'tag'} = {
        'recommend' => { 'name' => 'おすすめ' },
        'new'       => { 'name' => '新商品', 'lifetime' => 2678400 },
        'topseller' => { 'name' => '売れ筋商品' },
        'popular'   => { 'name' => '人気商品' },
        'standard'  => { 'name' => '定番商品' }
      }
      if !ref $this->{'tag'};
    return;
}

# --------------------------------------------------------------------
#	(0)-5 データ要素の取得
sub elements ($$) {
    my ( $this, $str ) = @_;
    my (%data);
    $str = '' if !defined $str;
    ( $data{'point'}, $data{'daily'}, $data{'total'}, $data{'ctime'} ) =
      split( /\t/, $str, 4 );
    $data{'point'} = 0 if !$data{'point'};
    $data{'daily'} = 0 if !$data{'daily'};
    $data{'daily'} = $data{'total'} = 0 if !$data{'total'};
    $data{'ctime'} = time if !$data{'ctime'};
    $data{'recommend'} = ( $data{'point'} == 999999 ) ? 1 : 0;
    $data{'daily'} = $this->daily_sales( \%data );
    return %data;
}

sub tag_id ($@) {
    my ( $this, %data ) = @_;
    my $id;
    foreach $id ( @{ $this->{'tagpriority'} } ) {
        if ( $id eq 'recommend' && $data{'recommend'} ) {
            return $id;
        }
        elsif ( $id eq 'new' ) {
            return $id
              if $this->{'tag'}{$id}{'lifetime'}
              && $data{'ctime'}
              && time - $data{'ctime'} < $this->{'tag'}{$id}{'lifetime'};
        }
        elsif ($this->{'tag'}{$id}{'threshold'}
            && $this->{'tag'}{$id}{'target'}
            && $data{ $this->{'tag'}{$id}{'target'} }
            && $data{ $this->{'tag'}{$id}{'target'} } >
            $this->{'tag'}{$id}{'threshold'} )
        {
            return $id;
        }
    }
    return '';
}

######################################################################
#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    $this->scalar_to_array( \$main::search{ $this->{'id'} }{'str'} );
    my $tag_id = $this->tag_id( $this->elements($$str) );
    return
      grep( $_ eq $tag_id, @{ $main::search{ $this->{'id'} }{'str'} } )
      ? 1
      : 0;
}

######################################################################
#	(2) obj->form;
sub form {
    my ( $this, $status, $original, $path ) = @_;
    my ($form);
    $form = $this->field_header;
    if ( $this->{'editable'} ) {
        $form .=
            '<label for="data:'
          . $this->{'id'}
          . ':total" class="datatitle">販売総数</label><input type="text" name="data:'
          . $this->{'id'}
          . ':total" id="data:'
          . $this->{'id'}
          . ':total" size="8" value="'
          . $this->escape( $main::data{ $this->{'id'} }{'total'} )
          . '" title="変更しない場合は空欄にします。" /> '
          . '<label><input type="checkbox" name="data:'
          . $this->{'id'}
          . ':reset_ctime" value="1" /> '
          . (
            ($default::english)
            ? 'Reset the sale start date.'
            : '販売開始日をリセット'
          ) . '</label> ';
    }
    $form .=
        '<label><input type="checkbox" name="data:'
      . $this->{'id'}
      . ':recommend" value="1"'
      . (
        ( $main::data{ $this->{'id'} }{'recommend'} ) ? ' checked="checked"'
        : ''
      )
      . ' /> '
      . (
        ( $this->{'tag'}{'recommend'}{'name'} )
        ? $this->{'tag'}{'recommend'}{'name'}
        : ($default::english) ? 'Recommend'
        :                       'おすすめ'
      ) . '</label>';
    $form .= $this->field_footer;
    return $form;
}

######################################################################
#	(3) obj->is_valid_data;
sub is_valid_data ($;$) {
    my $this = shift;
    my ( $data, %data, %db );
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie(
        %db,
        $env::dbm_file,
        $default::db
          . (
            ( !$default::subdivided && defined $main::c && $main::c ne '' )
            ? '/' . $main::c
            : ''
          )
          . '/sales',
        O_RDONLY,
        $env::dbm_mode
    );
    if ( ref tied %db && defined $db{ $this->encode_utf8($main::n) } ) {
        $data = $this->decode_utf8( $db{ $this->encode_utf8($main::n) } );
    }
    untie %db;
    %data = $this->elements($data);
    $data{'recommend'} = $main::data{ $this->{'id'} }{'recommend'};
    if ( $this->{'editable'} ) {
        $data{'total'} = $main::data{ $this->{'id'} }{'total'}
          if !$this->is_nan( $main::data{ $this->{'id'} }{'total'} );
        $data{'ctime'} = time
          if !$this->is_nan( $main::data{ $this->{'id'} }{'reset_ctime'} );
    }
    $data{'total'} = $main::data{ $this->{'id'} }{'total'}
      if !$this->is_nan( $main::data{ $this->{'id'} }{'total'} );
    $main::data{ $this->{'id'} }{'str'} = $this->sales_data( \%data );
    $cookie::data{ 'data:' . $this->{'id'} . ':recommend' } =
      ( $main::data{ $this->{'id'} }{'recommend'} ) ? 1 : 0
      if $this->{'cookie'} && $main::set_cookie;
    return 1;
}

sub sales_data ($$) {
    my ( $this, $data ) = @_;
    $data->{'total'} = 0 if !$data->{'ctime'} || !$data->{'total'};
    $data->{'ctime'} = time if !$data->{'ctime'};
    $data->{'daily'} = $this->daily_sales($data);
    return
        ( ( $data->{'recommend'} ) ? 999999 : $data->{'daily'} ) . "\x09"
      . $data->{'daily'} . "\x09"
      . $data->{'total'} . "\x09"
      . $data->{'ctime'};
}

sub daily_sales ($$) {
    my $this = shift;
    my $data = shift;
    return ( ref $data eq 'HASH' && $data->{'total'} )
      ? int(
        (
            (
                $data->{'total'} /
                  int( ( ( time - $data->{'ctime'} ) / 86400 ) + 1 )
            ) * 1000000
        ) + 0.5
      ) / 1000000
      : 0;
}

######################################################################
#	(4) obj->usual_format(str,path,category,no);
#	str=表示するデータ/path=ファイル保存ディレクトリ
sub usual_format {
    my ( $this, $str, $path, $c, $n ) = @_;
    my ( %data, $tag_id, $out );
    %data =
        ( ref $str eq 'HASH' )
      ? ( defined $$str{'str'} )
          ? $this->elements( $$str{'str'} )
          : %$str
      : $this->elements($str);
    $tag_id = $this->tag_id(%data);
    $out    = '';
    if ($tag_id) {
        if ( $this->{'tag'}{$tag_id}{'image'} ) {
            $out .= $cgi::Image->image_tag(
                $env::image_dir
                  . '/sales/'
                  . $setting::deco{'sales'}
                  { $this->{'tag'}{$tag_id}{'image'} }{'name'},
                $setting::deco{'sales'}{ $this->{'tag'}{$tag_id}{'image'} }
                  {'width'},
                $setting::deco{'sales'}{ $this->{'tag'}{$tag_id}{'image'} }
                  {'height'},
                0,
                $this->escape( $this->tag_name($tag_id) ),
                '',
                '',
                '',
                $cgi::output,
                $cgi::output_option
            );
        }
        else {
            $out .= $this->escape( $this->tag_name($tag_id) );
        }
    }
    $out .=
        ' <span class="sc-sales-info">販売総数:'
      . $data{'total'} . ' ('
      . ( int( $data{'daily'} * 100 + 0.5 ) / 100 )
      . '/1日)</span>'
      if !$cgi::output && $this->is_secure('tools');
    return $this->field_tag(
        $this->style(
            $out,                 $this->{'style'}, $this->{'fontsize'},
            $this->{'fontcolor'}, $this->{'fontface'}
        )
    );
}

sub tag_name ($$) {
    my ( $this, $tag_id ) = @_;
    return
      !$tag_id ? ''
      : ( defined $this->{'tag'}{$tag_id}{'name'}
          && $this->{'tag'}{$tag_id}{'name'} ne '' )
      ? $this->{'tag'}{$tag_id}{'name'}
      : ( $this->{'tag'}{$tag_id}{'image'}
          && defined $setting::deco{'sales'}
          { $this->{'tag'}{$tag_id}{'image'} }{'alt'}
          && $setting::deco{'sales'}{ $this->{'tag'}{$tag_id}{'image'} }{'alt'}
          ne '' )
      ? $setting::deco{'sales'}{ $this->{'tag'}{$tag_id}{'image'} }{'alt'}
      : '';
}

sub has_content {
    my ( $this, $str, ) = @_;
    my ( %data, $tag_id );
    %data =
        ( ref $str eq 'HASH' )
      ? ( defined $$str{'str'} )
          ? $this->elements( $$str{'str'} )
          : %$str
      : $this->elements($str);
    $tag_id = $this->tag_id(%data);
    return (
        $tag_id && ( $this->{'tag'}{$tag_id}{'image'}
            || $this->{'tag'}{$tag_id}{'name'} )
    ) ? 1 : 0;
}

#---------------------------------------------------------------------
#	(4)-2 obj->format_for_mail(str);
sub format_for_mail {
    my ( $this, $str ) = @_;
    my $name = $this->format_for_text($str);
    return ( $name ne '' )
      ? '[' . $this->name . "]\x0A" . $name . "\x0A\x0A"
      : '';
}

#---------------------------------------------------------------------
#	(4)-4 obj->format_for_text(str);
sub format_for_text {
    my ( $this, $str ) = @_;
    return $this->tag_name(
        $this->tag_id(
              ( ref $str eq 'HASH' )
            ? ( defined $$str{'str'} )
                  ? $this->elements( $$str{'str'} )
                  : %$str
            : $this->elements($str)
        )
    );
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my $this = shift;
    my ( $name, $tag_id, $n );

    #名前
    $name = 'data:' . $this->{'id'};

    #デフォルト
    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend>売れ行き表示設定</legend>';

    require CMONOS::CGI_Lib::Table;
    my $table = Table->new;
    $table->set(
        'tableclass' => 'tooltable',
        'thenabled'  => 'true',
        'trclass'    => 'odd,even',
        'class0'     => 'odd,even',
        'style0'     => 'text-align: center'
    );
    print $table->table;
    print $table->tr;
    print $table->td('評価順');
    print $table->td('表示名');
    print $table->td('条件');
    print $table->td('タグ画像');
    print $table->_tr;
    $n = 1;

    foreach $tag_id ( @{ $this->{'tagpriority'} } ) {
        my $name_tag = $name . '_tag_' . $tag_id;
        print $table->tr;
        print $table->td( '<input type="hidden" name="oldorder:'
              . $tag_id
              . '" value="'
              . $n
              . '" /><input type="number" name="order:'
              . $tag_id
              . '" size="2" value="'
              . $n++
              . '" />' );
        print $table->td( '<input type="text" name="'
              . $name_tag
              . '_name" size="16" value="'
              . $this->escape( $this->{'tag'}{$tag_id}{'name'} )
              . '" />' );
        if ( $tag_id eq 'recommend' ) {
            print $table->td('');
        }
        elsif ( $tag_id eq 'new' ) {
            print $table->td(
                $this->mk_select_menu(
                    $name_tag . '_lifetime', '', '',
                    $this->{'tag'}{$tag_id}{'lifetime'},
                    86400    => '1日以内に登録された商品',
                    604800   => '1週間以内に登録された商品',
                    1209600  => '2週間以内に登録された商品',
                    2678400  => '1ヶ月以内に登録された商品',
                    5270400  => '2ヶ月以内に登録された商品',
                    8208000  => '3ヶ月以内に登録された商品',
                    10540800 => '4ヶ月以内に登録された商品',
                    13219200 => '5ヶ月以内に登録された商品',
                    15811200 => '6ヶ月以内に登録された商品',
                    18489600 => '7ヶ月以内に登録された商品',
                    21081600 => '8ヶ月以内に登録された商品',
                    23760000 => '9ヶ月以内に登録された商品',
                    26352000 => '10ヶ月以内に登録された商品',
                    29030400 => '11ヶ月以内に登録された商品',
                    31536000 => '1年以内に登録された商品',
                    47347200 => '1年6ヶ月以内に登録された商品',
                    63072000 => '2年以内に登録された商品',
                    94608000 => '3年以内に登録された商品'
                )
            );
        }
        else {
            print $table->td(
                $this->mk_select_menu(
                    $name_tag . '_target', '', '',
                    $this->{'tag'}{$tag_id}{'target'},
                    'daily' => '一日あたり販売数が',
                    'total' => '販売総数が'
                  )
                  . '<input type="number" name="'
                  . $name_tag
                  . '_threshold" size="4" value="'
                  . $this->escape( $this->{'tag'}{$tag_id}{'threshold'} )
                  . '" /> 以上'
            );
        }
        print $table->td(
            $cgi::Deco->image_menu(
                $name_tag . '_image', 'sales',
                100,                  $this->{'tag'}{$tag_id}{'image'}
            )
        );
        print $table->_tr;
    }
    print $table->_table;

    print '</fieldset>';
    print '<fieldset>';
    print '<legend>販売総数の編集</legend>';
    print '<label><input type="checkbox" name="'
      . $name
      . '_editable" value="1"'
      . ( ( $this->{'editable'} ) ? ' checked="checked"' : '' )
      . ' /> 販売総数の編集を許可する</label>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>ソート</legend>';
    print '<label><input type="checkbox" name="'
      . $name
      . '_sortTotal" value="1"'
      . ( ( $this->{'sortTotal'} ) ? ' checked="checked"' : '' )
      . ' /> 販売総数でソートする</label>';
    print '</fieldset>';

    $this->print_font_setting_form($name);
    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this = shift;
    my ($tag_id);
    my $is_valid = $this->is_valid_font_setting;
    $this->{'tagpriority'} = $main::data{ $this->{'id'} . '_tagpriority' } = [
        (
            sort {
                     $main::order{$a} <=> $main::order{$b}
                  || $main::oldorder{$b} <=> $main::oldorder{$a}
                  || $main::order{$a} cmp $main::order{$b}
                  || $a <=> $b
            } @SCSales::tag
        )
    ];
    foreach $tag_id (@SCSales::tag) {
        my $name = $this->{'id'} . '_tag_' . $tag_id;
        if ( $tag_id eq 'new' ) {
            if (   defined $main::data{ $name . '_lifetime' }
                && $main::data{ $name . '_lifetime' } ne ''
                && $this->is_nan( $main::data{ $name . '_lifetime' } ) )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    $this->escape( $main::data{ $name . '_name' } )
                      . '条件が数ではありません。メニューから選択してください。'
                ) if ref $cgi::action;
                delete $main::data{ $this->{'id'} . '_lifetime' };
                $is_valid = 0;
            }
        }
        elsif ( $tag_id eq 'recommend' ) {
            if ( !defined $main::data{ $name . '_name' }
                || $main::data{ $name . '_name' } eq '' )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    $this->escape( $this->{'tag'}{'recommend'}{'name'} )
                      . '表示名は必ず設定してください。'
                ) if ref $cgi::action;
                delete $main::data{ $this->{'id'} . '_name' };
                $is_valid = 0;
            }
        }
        else {
            if (
                !$main::data{ $name . '_target' }
                || (   $main::data{ $name . '_target' } ne 'daily'
                    && $main::data{ $name . '_target' } ne 'total' )
              )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    $this->escape( $main::data{ $name . '_name' } )
                      . '販売数タイプはメニューから選択してください。'
                ) if ref $cgi::action;
                $main::data{ $this->{'id'} . '_target' } = 'daily';
                $is_valid = 0;
            }
            if (   defined $main::data{ $name . '_threshold' }
                && $main::data{ $name . '_threshold' } ne ''
                && $this->is_nan( $main::data{ $name . '_threshold' } ) )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    $this->escape( $main::data{ $name . '_name' } )
                      . '条件が数ではありません。半角数字にしてください。'
                ) if ref $cgi::action;
                delete $main::data{ $this->{'id'} . '_threshold' };
                $is_valid = 0;
            }
        }
    }
    $main::data{ $this->{'id'} . '_editable' } = ''
      if !defined $main::data{ $this->{'id'} . '_editable' }
      || !$main::data{ $this->{'id'} . '_editable' };
    $main::data{ $this->{'id'} . '_sortTotal' } = ''
      if !defined $main::data{ $this->{'id'} . '_sortTotal' }
      || !$main::data{ $this->{'id'} . '_sortTotal' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
      if defined $main::data{ $this->{'id'} . '_description' }
      && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
      if defined $main::data{ $this->{'id'} . '_note' }
      && $main::data{ $this->{'id'} . '_note' };
    return $is_valid;
}

######################################################################
sub sample ($$) {
    my $this = shift;
    my $str  = shift;
    my %data = $this->elements($str);
    return ( $this->{'sortTotal'} )
      ? $data{'total'}
      : ( ( $data{'point'} == 999999 ) ? 999999 : 0 ) + $data{'daily'};
}

sub search_form ($) {
    my $this = shift;
    $this->scalar_to_array( \$main::search{ $this->{'id'} }{'str'} );
    my $form =
        '<tr><td class="field-name"><label for="search:'
      . $this->{'id'}
      . ':str">'
      . $this->{'name'}
      . '</label></td><td class="field">';
    $form .= $this->mk_select_menu(
        'search:' . $this->{'id'} . ':str',
        '',
        ' multiple="multiple" size="5"',
        $main::search{ $this->{'id'} }{'str'},
        map( ( $_ => $this->tag_name($_) ), @{ $this->{'tagpriority'} } )
    );
    $form .= '</td></tr>';
    return $form;
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);
    if (   $env::checkout
        && -e $env::checkout
        && !$default::static
        && !$setting::field_for_mail
        && ( !defined $default::type || $default::type eq '' ) )
    {
        Field::class_register( 'ysc_sales', 'SCSales', '売れ行き' );
        push( @field::redundancy_forbidden, 'ysc_sales' );
        $setting::deco_dir{'sales'}  = '売れ行きタグ用素材';
        $setting::deco_type{'sales'} = 'image';
        @SCSales::tag =
          ( 'recommend', 'new', 'topseller', 'popular', 'standard' );
    }
}

1;
