package TrackbackPingBase;
require CMONOS::CGI_Lib::Field::UserGeneratedField;
@ISA = qw(UserGeneratedField);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# フィールド定義モジュール（トラックバック）
# TrackbackPingBase.pm 1.0.13
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-04-28
######################################################################
#	(0)-3 対応するDB
sub db ($) {
    my $this = shift;
    return 'trackback';
}

# --------------------------------------------------------------------
sub __set ($$) {
    my ( $this, $setting ) = @_;
    if ( !ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    $this->{'pingUrlName'}
        = ( ($default::english)
        ? 'Trackback URL'
        : 'この記事のトラックバックURL' )
        if !defined $this->{'pingUrlName'} || $this->{'pingUrlName'} eq '';
    $this->{'elements'} = 1;
    $this->{'datetype'}
        = ( $setting::general{'datetype'} )
        ? $setting::general{'datetype'}
        : ''
        if !$this->{'datetype'};
    $this->{'size'} = 30 if !defined $this->{'size'} || $this->{'size'} eq '';
    $this->{'maxlength'}  = 128     if !$this->{'maxlength'};
    $this->{'repetition'} = 'shift' if !$this->{'repetition'};
    $this->{'pingEnabled'} = ($default::static) ? 1 : 0
        if !$this->{'pingEnabled'};

    if ( !defined $this->{'pingServerList'} ) {
        $this->{'pingServerList'} = <<SERVER;
http://ping.bloggers.jp/rpc/>PING.BLOGGERS.JP>http://ping.bloggers.jp/
http://www.blogpeople.net/servlet/weblogUpdates>BlogPeople>http://www.blogpeople.net/
http://blog.goo.ne.jp/XMLRPC>gooブログ>http://blog.goo.ne.jp/portal/recent_entry/
http://rpc.technorati.jp/rpc/ping>Technorati JAPAN>http://www.technorati.jp/
http://blogsearch.google.co.jp/ping/RPC2>Googleブログ検索>http://blogsearch.google.co.jp/
http://api.my.yahoo.co.jp/RPC2>Yahoo!ブログ検索>http://blog-search.yahoo.co.jp/
http://ping.rss.drecom.jp/>ドリコムRSS>http://rss.drecom.jp/
http://rpc.reader.livedoor.com/ping>LivedoorReader>http://reader.livedoor.com/
SERVER
    }

    if ( $this->{'pingServerList'} ) {
        my $id = 0;
        while ( $this->{'pingServerList'} =~ /^(.*?)$/mg ) {
            my ( $ping, $name, $url ) = split( />/, $1, 3 );
            $this->{'pingServer'}{ ++$id } = {
                'ping' => $ping,
                'name' => ( ($name) ? $name : '' ),
                'url'  => ( ($url) ? $url : '' ),
            };
        }
    }
    $this->set_ownerpower;
    $this->set_external_file_status;
    delete $this->{'deletable'};
    return;
}

sub set_external_file_status ($) {
    my $this = shift;
    if ($default::static) {
        $this->{'externalFileDisabled'} = 0;
    }
    elsif (!defined $this->{'externalFileDisabled'}
        || $this->{'externalFileDisabled'} eq ''
        || $this->{'Hidden'}
        || ( defined $this->{'Open'} && $this->{'Open'} == 0 ) )
    {
        $this->{'externalFileDisabled'} = 1;
    }
    return;
}

sub get_rss_name ($) {
    my $this = shift;
    return 'トラックバック';
}

sub get_trackback_template ($;$) {
    my ( $this, $frame_flag ) = @_;
    if ( defined $this->{'template'} && $this->{'template'} ) {
        my $template = $this->{'template'};
        if ( $template =~ /<!--_tate(?![0-9a-zA-Z_])[^>]*-->/ ) {
            require CMONOS::CGI_Lib::Taketori;
            $template = Taketori->new( { 'css' => 1 } )
                ->tategaki_to_text($template);
        }
        if (   $frame_flag
            && $template =~ /<!--frame(?![0-9a-zA-Z_])[^>]*-->/
            && ref $frame::obj{'frame'} )
        {
            require CMONOS::CGI_Lib::Frame;
            $template = Frame->new->text_to_frame( $template, 'no',
                $frame::obj{'frame'} );
        }
        return $template;
    }
    else {
        return <<'TEMPLATE';
<div class="trackback">
<!--title--><!--head--><h4 class="trackback-head"><!--/head--><!--foot--></h4><!--/foot--><!--/title-->
<!--excerpt--><!--head--><p class="trackback-body"><!--/head--><!--foot--></p><!--/foot--><!--/excerpt-->
<div class="trackback-info">
<!--blog_name--><!--head--> <em><!--/head--><!--foot--></em><!--/foot--><!--/blog_name-->
<!--ctime--><!--head--> (<!--/head--><!--foot-->)<!--/foot--><!--/ctime-->
</div>
</div>
TEMPLATE
    }
}

######################################################################
#	(2) obj->form;
sub pre_form_option ($$) {
    my ( $this, $path ) = @_;
}

sub field_memory_required ($) {
    my $this = shift;
    return 0;
}

sub additional_form ($$) {
    my ( $this, $path ) = @_;
    my (%links);
    my $trackback_dat        = $path . '/' . $this->{'id'} . '.dat';
    my $trackback_pinged_dat = $path . '/' . $this->{'id'} . '.pinged.dat';
    my $form                 = '';
    my $ping_enabled         = 0;
    local (*DAT);
    if ( open( DAT, $trackback_pinged_dat ) ) {
        binmode DAT, ":utf8";
        flock( DAT, LOCK_SH );
        my $list_flag = 0;
        my $table     = Table->new;
        $table->set(
            'tableclass' => 'datatable',
            'thenabled'  => 'true',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
        );
        $form
            .= '<tr><td class="sub-field-name"><span class="datatitle">送信ログ</span><td class="sub-field">';
        $form .= $table->table;
        $form .= $table->tr;
        $form .= $table->th('送信') if !$this->{'trackbackDisabled'};
        $form .= $table->th('対象ページ');
        $form .= $table->th('状況');
        $form .= $table->_tr;

        while ( my $line = <DAT> ) {
            chomp $line;
            my ( $id, $ctime, $pinged, $pingurl, $url, $title )
                = $this->svsplit( "\x09", $line, -1 );
            if ($pingurl) {
                my $fid = 'trackback-' . $this->{'id'} . '-' . $id;
                $form .= $table->tr;
                $form .= $table->td(
                          '<input type="checkbox" name="option:'
                        . $this->{'id'}
                        . ':trackback" id="'
                        . $fid
                        . '" value="'
                        . $id
                        . '" title="'
                        . $this->escape($pingurl) . '"'
                        . (
                        ($pinged)
                        ? ' onclick="cmonos.form.field(this).checkboxConfirm(\'既に送信されています。本当にもう一度送信しますか？\')"'
                        : ''
                        )
                        . ' />'
                ) if !$this->{'trackbackDisabled'};
                $form .= $table->td(
                          '<label for="'
                        . $fid . '"'
                        . (
                        ($url)
                        ? '><a href="'
                            . $this->escape($url)
                            . '" target="_blank" title="'
                            . $this->escape( ($title) ? $title : $url ) . '">'
                            . $this->escape(
                            $this->cut_string(
                                ( ($title) ? $title : $url ), 32
                            )
                            )
                            . '</a>'
                        : ' title="'
                            . $this->escape($pingurl) . '">'
                            . $this->escape(
                            $this->cut_string( $pingurl, 32 )
                            )
                        )
                        . '</label>'
                );
                $form .= $table->td(
                    ( $ctime && $pinged ) ? '<span title="'
                        . $this->date(
                        $ctime, $this->{'datelimit'},
                        'gmt' . $this->{'datetype'}
                        )
                        . '">送信済</span>'
                    : ( !$ctime && !$pinged ) ? '未送信'
                    :                           '送信失敗'
                );
                $form .= $table->_tr;
                $list_flag = 1;
            }
        }
        close(DAT);
        if ($list_flag) {
            $form .= $table->_table . '</td></tr>';
        }
        else {
            $form = '';
        }
        $ping_enabled = 1;
    }
    if ( !$this->{'trackbackDisabled'} ) {
        if ( $this->{'pendingEnabled'} ) {
            $form
                .= '<tr><td class="sub-field-name"><span class="datatitle">表示のタイミング</span></td>';
            $form
                .= '<td class="sub-field"><label><input type="checkbox" name="data:'
                . $this->{'id'}
                . ':approvalRequired" value="1"'
                . ( ( $main::data{ $this->{'id'} }{'approvalRequired'} )
                ? ' checked="checked"'
                : '' )
                . ' /> 承認されるまで表示しない。</label></td></tr>';
        }
        $form .= '<tr>';
        $form
            .= '<td class="sub-field-name"><label for="data:'
            . $this->{'id'}
            . ':title">タイトル</label></td>';
        $form
            .= '<td class="sub-field"><input type="text" name="data:'
            . $this->{'id'}
            . ':title" id="data:'
            . $this->{'id'}
            . ':title" size="32" value="'
            . $this->escape_except_html_entity(
            $main::data{ $this->{'id'} }{'title'} )
            . '" /></td>';
        $form .= '</tr>';
        $form .= '<tr>';
        $form
            .= '<td class="sub-field-name"><label for="data:'
            . $this->{'id'}
            . ':excerpt">要約</label></td>';
        $form
            .= '<td class="sub-field"><textarea name="data:'
            . $this->{'id'}
            . ':excerpt" id="data:'
            . $this->{'id'}
            . ':excerpt" cols="32" rows="4" wrap="soft">'
            . $this->escape_except_html_entity(
            $main::data{ $this->{'id'} }{'excerpt'} )
            . '</textarea></td>';
        $form .= '</tr>';
    }
    if ($ping_enabled) {
        if ( !$this->{'trackbackDisabled'} ) {
            $form .= '<tr>';
            $form
                .= '<td class="sub-field-name"><label for="data:'
                . $this->{'id'}
                . ':tblist">トラックバックURL</label></td>';
            $form
                .= '<td class="sub-field"><textarea name="option:'
                . $this->{'id'}
                . ':tblist" id="option:'
                . $this->{'id'}
                . ':tblist" cols="32" rows="3" wrap="off">'
                . $this->escape( $main::option{ $this->{'id'} }{'tblist'} )
                . '</textarea>';
            $form
                .= '<div class="note">（トラックバックURLごとに改行）</div>';
            $form
                .= '<label class="extra-field"><input type="checkbox" name="option:'
                . $this->{'id'}
                . ':search" value="1" /> トラックバック URL リストを自動取得する(送信はしません)</label>';
            $form .= '</td>';
            $form .= '</tr>';
        }
        if ( ref $this->{'pingServer'} && %{ $this->{'pingServer'} } ) {
            $form .= '<tr>';
            $form
                .= '<td class="sub-field-name"><span class="datatitle">更新通知 Ping</span></td>';
            $form .= '<td class="sub-field">';
            $form .= '<ol class="checkbox">';
            $form
                .= '<li><label><input type="checkbox" onclick="cmonos.element.form(this.form).fieldname(\'option:'
                . $this->{'id'}
                . ':ping\').select(this.checked)" /> <em>すべて</em></label></li>';
            foreach
                my $pid ( sort { $a <=> $b } keys %{ $this->{'pingServer'} } )
            {
                $form
                    .= '<li><label><input type="checkbox" name="option:'
                    . $this->{'id'}
                    . ':ping" value="'
                    . $pid . '" /> '
                    . (
                    ( $this->{'pingServer'}{$pid}{'url'} )
                    ? '<a href="'
                        . $this->escape( $this->{'pingServer'}{$pid}{'url'} )
                        . '" target="_blank">'
                        . $this->escape(
                        ( $this->{'pingServer'}{$pid}{'name'} )
                        ? $this->{'pingServer'}{$pid}{'name'}
                        : $this->{'pingServer'}{$pid}{'url'}
                        )
                        . '</a>'
                    : $this->escape(
                        ( $this->{'pingServer'}{$pid}{'name'} )
                        ? $this->{'pingServer'}{$pid}{'name'}
                        : $this->{'pingServer'}{$pid}{'ping'}
                    )
                    ) . '</label></li>';
            }
            $form .= '</ol>';
            $form .= '</td></tr>';
        }
    }
    return $form;
}

######################################################################
#	(3) obj->is_valid_data(path);
sub search_links ($$$) {
    my ( $this, $data, $link_array ) = @_;
    return if !ref $data || !$$data;
    while ( $$data
        =~ m!( href="(https?://[^"]+)"|s?https?://[0-9a-zA-Z_~\-]+\.[0-9a-zA-Z_~\-/\.\?\&=\%#]+)!gis
        )
    {
        my $url = ($2) ? $2 : $1;
        if ( $url =~ /\.([0-9a-zA-Z_]+)(\?|$)/ ) {
            my $ext = lc $1;
            next
                if !grep( $_ eq $ext,
                'html',  'htm',  'shtml', 'shtm', 'stm', 'php',
                'phtml', 'phtm', 'ptm',   'cgi',  'pl',  'plx',
                'asp',   'aspx', 'cfm',   'py',   'rb' );
        }
        push( @$link_array, $url ) if !grep( $_ eq $url, @$link_array );
    }
}

sub search_links_from_files ($$) {
    my ( $this, $link_array ) = @_;
}

sub _is_valid_data ($) {
    my $this = shift;
    local (*DAT);
    my $path                 = $this->get_file_directory;
    my $trackback_pinged_dat = $path . '/' . $this->{'id'} . '.pinged.dat';
    my @links                = ();
    my @new_trackback        = ();
    my %trackback            = ();
    my %trackback_result     = ();
    my $ping_disabled        = 0;
    my $last_id              = 0;
    my $viewer_url           = $this->get_viewer_url( $main::c, $main::n ),
        my @tb_list
        = ( $main::option{ $this->{'id'} }{'tblist'} )
        ? grep( $_ ne '' && $_ =~ m!^https?://!i,
        split( /\x0A+/, $main::option{ $this->{'id'} }{'tblist'} ) )
        : ();
    $this->scalar_to_array( \$main::option{ $this->{'id'} }{'trackback'} );
    $this->scalar_to_array( \$main::option{ $this->{'id'} }{'ping'} );

    #リンク先を自動取得
    if (!$this->{'trackbackDisabled'}
        && ( !-f $trackback_pinged_dat
            || $main::option{ $this->{'id'} }{'search'} )
        )
    {
        while ( my ( $fid, $hash ) = each %main::data ) {
            while ( my ( $key, $value ) = each %$hash ) {
                $this->search_links( \$value, \@links );
            }
        }
        $this->search_links_from_files( \@links );
    }

    if ( -f $trackback_pinged_dat ) {
        if ( !$this->{'trackbackDisabled'}
            && open( DAT, $trackback_pinged_dat ) )
        {
            binmode DAT, ":utf8";
            flock( DAT, LOCK_SH );
            while ( my $line = <DAT> ) {
                chomp $line;
                my ( $id, $ctime, $pinged, $pingurl, $url, $title )
                    = $this->svsplit( "\x09", $line, -1 );
                @links = grep( $_ ne $url, @links );
                $trackback{$id} = $pingurl;
                $last_id = $id;
                if (@tb_list) {
                    if ( $pingurl && grep( $_ eq $pingurl, @tb_list ) ) {
                        push(
                            @{  $main::option{ $this->{'id'} }{'trackback'}
                            },
                            $id
                            )
                            if !grep( $_ eq $id,
                            @{  $main::option{ $this->{'id'} }{'trackback'}
                            } );
                        @tb_list = grep( $_ ne $pingurl, @tb_list );
                    }
                    elsif ( $url && grep( $_ eq $url, @tb_list ) ) {
                        push(
                            @{  $main::option{ $this->{'id'} }{'trackback'}
                            },
                            $id
                            )
                            if !grep( $_ eq $id,
                            @{  $main::option{ $this->{'id'} }{'trackback'}
                            } );
                        @tb_list = grep( $_ ne $url, @tb_list );
                    }
                }
            }
            close(DAT);
        }
        else {
            delete $main::option{ $this->{'id'} }{'trackback'};
            undef @tb_list;
        }
    }
    else {
        delete $main::option{ $this->{'id'} }{'trackback'};
        delete $main::option{ $this->{'id'} }{'ping'};
        undef @tb_list;
        $ping_disabled = 1;
    }

    #リンク先のトラックバックURLを自動取得
    if (@links) {
        require CMONOS::CGI_Lib::Ping::Trackback;
        foreach my $url (@links) {
            push( @new_trackback, Trackback->new( 'url' => $url )->get );
        }
        undef @links;
    }

    #記事タイトル取得
    if ( !$main::data{ $this->{'id'} }{'title'} ) {
        my $fid
            = ($default::static) ? $this->{'field_for'}->{'title'}
            : ( defined $this->{'field_for'}->{'title'}
                && $this->{'field_for'}->{'title'} ne '' )
            ? $this->{'field_for'}->{'title'}
            : $this->{'field_for'}->{'key'};
        $main::data{ $this->{'id'} }{'title'}
            = (    defined $fid
                && $fid ne ''
                && defined $main::data{$fid}{'str'}
                && $main::data{$fid}{'str'} ne '' )
            ? $this->unescape( $main::data{$fid}{'str'} )
            : ( $default::date && $main::n )
            ? $this->date( substr( $main::n, 0, 8 ),
            0, $setting::general{'datetype'} )
            : ($main::n) ? $main::n
            :              '';
    }
    if ( $main::data{ $this->{'id'} }{'title'} ) {
        $main::data{ $this->{'id'} }{'title'} =~ s/\x0A//g;
        $this->text_color( \$main::data{ $this->{'id'} }{'title'} );
        $main::data{ $this->{'id'} }{'title'}
            = $this->cut_string( $main::data{ $this->{'id'} }{'title'},
            $this->{'maxlength'}, 'no' );
    }

    #記事要約を取得
    if ( !$main::data{ $this->{'id'} }{'excerpt'} ) {
        my $fid
            = ($default::static)
            ? $this->{'field_for'}->{'description'}
            : $this->{'field_for'}->{'content'};
        $main::data{ $this->{'id'} }{'excerpt'}
            = $this->unescape( $main::data{$fid}{'str'} )
            if defined $fid && $fid ne '';
    }
    if ( $main::data{ $this->{'id'} }{'excerpt'} ) {
        $main::data{ $this->{'id'} }{'excerpt'} =~ s/\x0A//g;
        $this->text_color( \$main::data{ $this->{'id'} }{'excerpt'} );
        $main::data{ $this->{'id'} }{'excerpt'}
            = $this->cut_string( $main::data{ $this->{'id'} }{'excerpt'},
            $this->{'maxlength'}, 'no' );
    }

    #トラックバック送信
    if ( !$ping_disabled ) {
        my $blog_name
            = ( defined $env::title && $env::title ne '' ) ? $env::title : '';
        if ( $this->{'weblogDirectory'} ) {
            if (   $default::master_preference_dir
                && $default::master_preference_dir ne
                $default::preference_dir )
            {
                $this->import_db(
                    $default::master_preference_dir . '/general',
                    \%master::general, 1 )
                    if !%master::general;
                $blog_name
                    .= ( ( $blog_name ne '' ) ? ':' : '' )
                    . $master::general{'cginame'}
                    if defined $master::general{'cginame'}
                    && $master::general{'cginame'} ne '';
                $blog_name
                    .= ( ( $blog_name ne '' ) ? ':' : '' )
                    . $setting::general{'cginame'}
                    if $this->{'weblogDirectory'} eq 'category'
                    && defined $setting::general{'cginame'}
                    && $setting::general{'cginame'} ne '';
            }
            elsif ( defined $setting::general{'cginame'}
                && $setting::general{'cginame'} ne '' )
            {
                $blog_name .= ( ( $blog_name ne '' ) ? ':' : '' )
                    . $setting::general{'cginame'};
            }
        }
        if (%trackback) {
            if ( @{ $main::option{ $this->{'id'} }{'trackback'} } ) {
                require CMONOS::CGI_Lib::Ping::Trackback;
                foreach my $tid (
                    @{ $main::option{ $this->{'id'} }{'trackback'} } )
                {
                    if ( $trackback{$tid} && !$trackback_result{$tid} ) {
                        @tb_list = grep( $_ ne $trackback{$tid}, @tb_list );
                        $trackback_result{$tid} = Trackback->new(
                            'ping_url' => $trackback{$tid},
                            'url'      => $viewer_url,
                            'title' => $main::data{ $this->{'id'} }{'title'},
                            'excerpt' =>
                                $main::data{ $this->{'id'} }{'excerpt'},
                            'blog_name' => $blog_name,
                        )->ping->hash_ref;
                    }
                }
            }
            undef %trackback;
        }
        if (@tb_list) {
            require CMONOS::CGI_Lib::Ping::Trackback;
            foreach my $ping (@tb_list) {
                push(
                    @new_trackback,
                    Trackback->new(
                        'ping_url' => $ping,
                        'url'      => $viewer_url,
                        'title'    => $main::data{ $this->{'id'} }{'title'},
                        'excerpt'  => $main::data{ $this->{'id'} }{'excerpt'},
                        'blog_name' => $blog_name,
                    )->ping->receiver->hash_ref(1)
                );
            }
            undef @tb_list;
        }

        #Ping送信
        if ( @{ $main::option{ $this->{'id'} }{'ping'} } ) {
            my %ping_info = ();
            my ( $rss_dir, $link, $master_rss_dir, $master_link )
                = $this->get_rss_path;
            if (   !$default::static
                || !$this->{'weblogDirectory'}
                || !$rss_dir )
            {
                %ping_info = (
                    'weblogname' => (
                        ($env::title) ? $env::title : $this->home_location
                    ),
                    'weblogurl' => $this->full_path('../../'),
                    'rssurl'    => (
                        ( -f '../../index.xml' )
                        ? $this->home_location . '/index.xml'
                        : ''
                    ),
                );
            }
            else {
                if ( $this->{'weblogDirectory'} eq 'category'
                    || !$master_rss_dir )
                {
                    %ping_info = (
                        'weblogname' => (
                            ( $setting::general{'cginame'} )
                            ? $setting::general{'cginame'}
                            : ($env::title) ? $env::title
                            :                 $this->home_location
                        ),
                        'weblogurl' =>
                            ( ($link) ? $link : $this->full_path('../../') ),
                        'rssurl' => (
                            ( $rss_dir && -f $rss_dir . '/index.xml' )
                            ? $this->full_path( $rss_dir . '/index.xml' )
                            : ''
                        ),
                    );
                }
                else {
                    $this->import_db(
                        $default::master_preference_dir . '/general',
                        \%master::general, 1 )
                        if !%master::general;
                    %ping_info = (
                        'weblogname' => (
                            ( $master::general{'cginame'} )
                            ? $master::general{'cginame'}
                            : ($env::title) ? $env::title
                            :                 $this->home_location
                        ),
                        'weblogurl' => (
                            ($master_link) ? $master_link
                            : $this->full_path('../../')
                        ),
                        'rssurl' => (
                            (   $master_rss_dir
                                    && -f $master_rss_dir . '/index.xml'
                            ) ? $this->full_path(
                                $master_rss_dir . '/index.xml'
                            ) : ''
                        ),
                    );
                }
            }
            require CMONOS::CGI_Lib::Ping::WeblogUpdatesPing;
            foreach my $pid ( @{ $main::option{ $this->{'id'} }{'ping'} } ) {
                next if !$pid;
                my $ping = WeblogUpdatesPing->new();
                $ping = WeblogUpdatesPing->new(
                    'ping_url' => $this->{'pingServer'}{$pid}{'ping'},
                    %ping_info
                )->ping;
                push(
                    @{ $cgi::action->{'dialog'} },
                    $this->escape(
                        ( $this->{'pingServer'}{$pid}{'name'} )
                        ? $this->{'pingServer'}{$pid}{'name'}
                        : $this->{'pingServer'}{$pid}{'ping'}
                        )
                        . (
                        ( $ping->flerror )
                        ? ' への Ping 送信に失敗しました。'
                        : ' に Ping を送信しました。'
                        )
                        . (
                        ( $ping->message )
                        ? '(' . $this->escape( $ping->message ) . ')'
                        : ''
                        )
                ) if ref $cgi::action;
            }
        }
    }

    if ( @new_trackback || %trackback_result ) {
        delete $main::option{ $this->{'id'} }{'tblist'};
        my $new = '';
        if (@new_trackback) {
            foreach my $tb (@new_trackback) {
                next if !ref $tb || ( !$tb->{'url'} && !$tb->{'trackback'} );
                push(
                    @{ $cgi::action->{'dialog'} },
                    $this->escape(
                        ( $tb->{'title'} ) ? $tb->{'title'}
                        : $tb->{'ping_url'}
                        )
                        . (
                        ( $tb->{'error'} )
                        ? ' へのトラックバック送信に失敗しました。'
                        : ' にトラックバックを送信しました。'
                        )
                        . (
                        ( $tb->{'message'} ) ? '(' . $tb->{'message'} . ')'
                        : ''
                        )
                ) if ref $cgi::action && $tb->{'ping_url'};
                $new .= $this->svjoin(
                    "\x09",
                    ++$last_id,
                    ( ( $tb->{'ptime'} ) ? $tb->{'ptime'} : '' ),
                    ( ( $tb->{'ptime'} && !$tb->{'error'} ) ? 1 : 0 ),
                    ( ( $tb->{'trackback'} ) ? $tb->{'trackback'} : '' ),
                    ( ( $tb->{'url'} )       ? $tb->{'url'}       : '' ),
                    ( ( $tb->{'title'} )     ? $tb->{'title'}     : '' )
                    )
                    . "\x0A"
                    if $tb->{'ptime'}
                    || ( !$tb->{'ptime'} && !$tb->{'error'} );
            }
        }
        if ( -f $trackback_pinged_dat ) {
            local ( *DAT, *TEMP );
            my $done = 0;
            my $trackback_pinged_temp
                = $path . '/' . $this->{'id'} . '.pinged.temp.dat';
            if ( open( TEMP, ">>$trackback_pinged_temp" ) ) {
                binmode TEMP, ":utf8";
                flock( TEMP, LOCK_EX );
                if ( open( DAT, $trackback_pinged_dat ) ) {
                    binmode DAT, ":utf8";
                    flock( DAT, LOCK_SH );
                    while ( my $line = <DAT> ) {
                        next if !$line;
                        chomp $line;
                        my ( $id, $ctime, $pinged, $pingurl, $url, $title )
                            = $this->svsplit( "\x09", $line, -1 );
                        if ( $trackback_result{$id} ) {
                            push(
                                @{ $cgi::action->{'dialog'} },
                                $this->escape(
                                    ($title) ? $title
                                    : $trackback_result{$id}->{'ping_url'}
                                    )
                                    . (
                                    ( $trackback_result{$id}->{'error'} )
                                    ? ' へのトラックバック送信に失敗しました。'
                                    : ' にトラックバックを送信しました。'
                                    )
                                    . (
                                    ( $trackback_result{$id}->{'message'} )
                                    ? '('
                                        . $trackback_result{$id}->{'message'}
                                        . ')'
                                    : ''
                                    )
                            ) if ref $cgi::action;
                            print TEMP $this->svjoin(
                                "\x09", $id,
                                (   ( $trackback_result{$id}->{'ptime'} )
                                    ? $trackback_result{$id}->{'ptime'}
                                    : $ctime
                                ),
                                (   (   $trackback_result{$id}->{'ptime'}
                                            && !$trackback_result{$id}
                                            ->{'error'}
                                    ) ? 1 : $pinged
                                ),
                                (   ( $trackback_result{$id}->{'ping_url'} )
                                    ? $trackback_result{$id}->{'ping_url'}
                                    : $pingurl
                                ),
                                $url, $title,
                                ),
                                "\x0A";
                        }
                        else {
                            print TEMP $line, "\x0A";
                        }
                    }
                    close(DAT);
                    $done = 1;
                }
                print TEMP $new;
                close(TEMP);
            }
            if ($done) {
                rename( $trackback_pinged_temp, $trackback_pinged_dat );
                chmod(
                    ( ($env::ownerpower) ? 0600 : $env::file_mode ),
                    $trackback_pinged_dat
                );
            }
        }
        else {
            my $name = 'data:' . $this->{'id'} . ':pinged';
            $cgi::input->post_clipboard(
                $name, 0, $new,
                {   'filename' => 'data.dat',
                    'name'     => $name,
                    'type'     => 'application/octet-stream',
                }
            );
            $cgi::input->save_file( $name,
                '!' . $this->{'id'} . '.pinged.dat' );
        }
    }
    elsif ( !-f $trackback_pinged_dat ) {
        my $name = 'data:' . $this->{'id'} . ':pinged';
        $cgi::input->post_clipboard(
            $name, 0, '',
            {   'filename' => 'data.dat',
                'name'     => $name,
                'type'     => 'application/octet-stream',
            }
        );
        $cgi::input->save_file( $name, '!' . $this->{'id'} . '.pinged.dat' );
    }
    return 1;
}

######################################################################
sub get_user_generated_field_data ($$$$;$$$) {
    my ( $this, $memory, $data_ref, $path, $c, $n, $content_only ) = @_;
    return ''
        if ( defined $this->{'masterMemory'}
        && $this->{'masterMemory'} eq '0' )
        || ( defined $memory && $memory eq '0' )
        || ( !$default::static
        && $ENV{'HTTP_USER_AGENT'}
        && $ENV{'HTTP_USER_AGENT'}
        =~ /(?:Googlebot|Slurp|msnbot|https?:\/\/)/i );
    my $approval_required = ( $data_ref->{'approvalRequired'} ) ? 1 : 0;
    my $id                = $this->mk_user_generated_field_id( $c, $n );
    my $template          = $this->get_trackback_template(1);
    my $trackback_count   = 0;
    my $memory_count      = 0;
    my $trackback_content = '';
    $memory = $this->{'masterMemory'}
        if $this->{'masterMemory'}
        && ( !defined $memory
        || $memory eq ''
        || $memory > $this->{'masterMemory'} );
    $this->use_deco( $this->get_output_path );
    local (*DAT);

    if ( open( DAT, $path . '/' . $this->{'id'} . '.dat' ) ) {
        binmode DAT, ":utf8";
        flock( DAT, LOCK_SH );
        my ( $head, $foot ) = ( '', '' );
        ( $head, $template, $foot )
            = ( $1, $this->join_header_and_footer( $2, $3 ), $4 )
            if $template
            =~ /^(.*)<!--article(?![0-9a-zA-Z_])([^>]*)-->(.*)<!--\/article-->(.*)$/s;
        while ( my $line = <DAT> ) {
            chomp $line;
            $memory_count++;
            $trackback_content
                .= $this->replace_trackback( $template, $approval_required,
                [ $this->svsplit( "\x09", $line ) ],
                0, \$trackback_count );
            last if $memory && $trackback_count >= $memory;
        }
        close(DAT);
        $trackback_content = $head . $trackback_content . $foot
            if $trackback_content;
    }
    if ($content_only) {
        $cgi::Deco->restore_output;
        return $trackback_content;
    }
    my $trackback_url = $this->get_trackback_url( $c, $n );
    my $trackback
        = '<div id="'
        . $id
        . '" class="trackback-list">'
        . $trackback_content
        . '</div>';
    if ( $this->{'size'} ) {
        $trackback
            .= '<form id="'
            . $id
            . '-form" class="trackback-url-box" action="#" onsubmit="document.getElementById(\''
            . $id
            . '-ping-url\').focus();document.getElementById(\''
            . $id
            . '-ping-url\').select();return false;">';
        $trackback
            .= '<label for="'
            . $id
            . '-ping-url">'
            . $this->{'pingUrlName'}
            . '</label>';
        $trackback
            .= ' <input type="text" id="'
            . $id
            . '-ping-url" name="ping_url" size="'
            . $this->{'size'}
            . '" value="'
            . $this->escape($trackback_url)
            . '" onfocus="this.select();" />';
        $trackback .= '</form>';
    }
    $trackback .= '<p id="' . $id . '-info" class="trackback-list-info">';
    $trackback
        .= '<a href="javascript:void(0)" onclick="cmonos.element.item(\''
        . $id . '\''
        . ( ( $this->{'size'} ) ? ',\'' . $id . '-form\'' : '' )
        . ').toggle()">'
        . $this->name(1)
        . '<span id="'
        . $id
        . '-count">('
        . $trackback_count
        . ')</span>' . '</a>';
    $trackback .= $this->field_editor_link(
        ( ($default::english) ? '[EDIT]' : '[編集]' ),
        $id . '-editor-link',
        'display: none',
        $c, $n
    );
    $trackback .= '</p>';

    #RDF
    require CMONOS::CGI_Lib::Ping::Trackback;
    $trackback .= "<!--\x0A";
    $trackback .= Trackback->new(
        'trackback' => $trackback_url,
        'url'       => $this->get_viewer_url( $c, $n ),
        'excerpt'   => $data_ref->{'excerpt'},
        'title'     => $data_ref->{'title'},
    )->rdf;
    $trackback .= "-->\x0A";

    $trackback
        .= $this->javascript( 'cmonos.element.item(\''
            . $id . '\''
            . ( ( $this->{'size'} ) ? ',\'' . $id . '-form\'' : '' )
            . ').toggle();' )
        if $this->{'hiddenTrackback'};
    $cgi::Deco->restore_output;
    return (wantarray) ? ( $trackback, $trackback_content ) : $trackback;
}

sub replace_trackback ($$$$;$$) {
    my ( $this, $template, $approval_required, $data, $file_flag, $count )
        = @_;
    my ( $id, $ctime, $approved, $url, $title, $excerpt, $blog_name, $ip )
        = @$data;
    if ( !$file_flag ) {
        return ''
            if !$url
            || ( $this->{'pendingEnabled'}
            && $approval_required
            && !$approved );
        $title
            = ( $title ne '' )
            ? '<a href="'
            . $this->escape($url) . '"'
            . ( ( $this->{'nofollow'} ) ? ' rel="nofollow"' : '' ) . '>'
            . $this->escape($title) . '</a>'
            : '';
        $blog_name
            = ( $blog_name ne '' )
            ? '<a href="'
            . $this->escape($url) . '"'
            . ( ( $this->{'nofollow'} ) ? ' rel="nofollow"' : '' ) . '>'
            . $this->escape($blog_name) . '</a>'
            : '';
    }
    $$count++ if ref $count;
    $url
        = '<a href="'
        . $this->escape($url) . '"'
        . ( ( $this->{'nofollow'} ) ? ' rel="nofollow"' : '' ) . '>'
        . $this->escape($url) . '</a>';
    $excerpt
        = ( $this->{'maxlength'} )
        ? $this->cut_string( $excerpt, $this->{'maxlength'}, 'no' )
        : $this->detag($excerpt);
    $template
        =~ s/<\!--blog_name(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/blog_name(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('blog_name',$1,$2,$blog_name,$file_flag)/ges;
    $template
        =~ s/<\!--title(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/title(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('title',$1,$2,$title,$file_flag)/ges;
    $template
        =~ s/<\!--excerpt(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/excerpt(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('excerpt',$1,$2,$excerpt,$file_flag)/ges;
    $template
        =~ s/<\!--url(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/url(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('url',$1,$2,$url,$file_flag)/ges;
    $template
        =~ s/<\!--ip(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/ip(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('ip',$1,$2,$ip,$file_flag)/ges;
    $template
        =~ s/<\!--ctime(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/ctime(?![0-9a-zA-Z_])[^>]*-->/$this->datetime_tag_replace('ctime',$1,$2,$ctime,(($this->{'externalFileDisabled'}) ? $this->{'datelimit'} : 0),'gmt' . $this->{'datetype'},'',$file_flag)/ges;

    if ( $template =~ /<!--tate(?![0-9a-zA-Z_])[^>]*-->/ ) {
        require CMONOS::CGI_Lib::Taketori;
        $template = Taketori->new( { 'css' => 1 } )
            ->text_to_tategaki( $template, $file_flag );
    }
    return $template;
}

sub get_user_generated_field_data_for_text ($$$$) {
    my ( $this, $memory, $data_ref, $path ) = @_;
    my $approval_required = ( $data_ref->{'approvalRequired'} ) ? 1 : 0;
    my $out               = '';
    my $template          = $this->get_trackback_template(1);
    my $title_enabled
        = (
        $template =~ /<!--title(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/title-->/ )
        ? 1
        : 0;
    my $email_enabled
        = (
        $template =~ /<!--email(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/email-->/ )
        ? 1
        : 0;
    my $url_enabled
        = ( $template =~ /<!--url(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/url-->/ )
        ? 1
        : 0;
    my $icon_enabled
        = (    $this->{'dir'}
            && $template
            =~ /<!--icon(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/icon-->/ ) ? 1 : 0;
    $memory = $this->{'masterMemory'}
        if $this->{'masterMemory'}
        && ( !defined $memory
        || $memory eq ''
        || $memory > $this->{'masterMemory'} );
    local (*DAT);

    if ( open( DAT, $path . '/' . $this->{'id'} . '.dat' ) ) {
        binmode DAT, ":utf8";
        flock( DAT, LOCK_SH );
        my $count;
        while ( my $line = <DAT> ) {
            chomp $line;
            my ( $id, $ctime, $approved, $url, $title, $excerpt, $blog_name,
                $ip )
                = $this->svsplit( "\x09", $line, -1 );
            if (   !$this->{'pendingEnabled'}
                || !$approval_required
                || $approved )
            {
                $out
                    .= ( ($default::english) ? 'Blog Name' : 'サイト名' )
                    . ' > '
                    . $blog_name . "\x0A"
                    if defined $blog_name && $blog_name ne '';
                $out
                    .= (
                    ($default::english) ? 'Title' : '記事タイトル' )
                    . ' > '
                    . $title . "\x0A"
                    if defined $title && $title ne '';
                $out .= 'URL > ' . $url . "\x0A";
                $out .= ( ($default::english) ? 'Excerpt' : '要約' ) . ' >'
                    . "\x0A";
                $out
                    .= $this->detag($excerpt) . "\x0A("
                    . $this->date( $ctime, 0, 'gmt' . $this->{'datetype'} )
                    . ")\x0A\x0A";
                $count++;
            }
            last if $memory && $count >= $memory;
        }
        close(DAT);
    }
    return $out;
}

sub _confirmation ($$) {
    my ( $this, $data ) = @_;
    return ( ( defined $data->{'title'} && $data->{'title'} ne '' )
        ? '<dt>タイトル</dt><dd>' . $data->{'title'} . '</dd>'
        : '' )
        . ( ( defined $data->{'excerpt'} && $data->{'excerpt'} ne '' )
        ? '<dt>要約</dt><dd>' . $data->{'excerpt'} . '</dd>'
        : '' );
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my $this = shift;

    #名前
    my $name = 'data:' . $this->{'id'};

    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_template">テンプレート</label></legend>';
    print '<textarea name="'
        . $name
        . '_template" id="'
        . $name
        . '_template" cols="72" rows="10" wrap="off" class="content-editor-area">'
        . $this->escape( $this->{'template'} )
        . '</textarea>';
    print '<div class="toolbox">';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'',
        $name,
        '_template\').setAutoFit(this.checked);" />フィット</label> ';
    print
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\''
        . $name
        . '_template\').quotationByMenu(this)" title="これらのタグは、書き出し時に対応するトラックバックデータに変換されます。">';
    print
        '<option value="">▼トラックバックデータタグを挿入する</option>';
    print
        '<option value="&lt;!--article--&gt;&lt;!--/article--&gt;">繰り返しの範囲 &lt;!--article--&gt;&lt;!--/article--&gt;</option>';
    print
        '<option value="&lt;!--blog_name--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/blog_name--&gt;">サイト名 &lt;!--blog_name--&gt;&lt;!--/blog_name--&gt;</option>';
    print
        '<option value="&lt;!--title--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/title--&gt;">タイトル &lt;!--title--&gt;&lt;!--/title--&gt;</option>';
    print
        '<option value="&lt;!--excerpt--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/excerpt--&gt;">要約 &lt;!--excerpt--&gt;&lt;!--/excerpt--&gt;</option>';
    print
        '<option value="&lt;!--url--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/url--&gt;">URL &lt;!--url--&gt;&lt;!--/url--&gt;</option>';
    print
        '<option value="&lt;!--ip--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/ip--&gt;">トラックバック元 IP &lt;!--ip--&gt;&lt;!--/ip--&gt;</option>';
    print
        '<option value="&lt;!--ctime--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/ctime--&gt;">トラックバック日時 &lt;!--ctime--&gt;&lt;!--/ctime--&gt;</option>';
    print '<option value=""></option>';
    print '<option value="' . $this->jsquot(
              '<div class="trackback">\n'
            . '<!--title--><!--head--><h4 class="trackback-head"><!--/head--><!--foot--></h4><!--/foot--><!--/title-->\n'
            . '<!--excerpt--><!--head--><p class="trackback-body"><!--/head--><!--foot--></p><!--/foot--><!--/excerpt-->\n'
            . '<div class="trackback-info">\n'
            . '<!--blog_name--><!--head--> <em><!--/head--><!--foot--></em><!--/foot--><!--/blog_name-->\n'
            . '<!--ctime--><!--head--> (<!--/head--><!--foot-->)<!--/foot--><!--/ctime-->\n'
            . '</div>\n'
            . '</div>\n'

    ) . '">テンプレート例</option>';
    print '<option value="'
        . $this->jsquot( '<dl class="log">\n'
            . '<!--article--><dt>\n'
            . '<!--title--><!--head--><span class="name"><!--/head--><!--foot--></span><!--/foot--><!--/title--> \n'
            . '<!--blog_name--><!--head--><span class="name"><!--/head--><!--foot--></span><!--/foot--><!--/blog_name--> \n'
            . '<span class="date"><!--ctime--><!--/ctime--></span></dt>\n'
            . '<!--excerpt--><!--head--><dd><!--/head--><!--foot--></dd><!--/foot--><!--/excerpt-->\n'
            . '<!--/article--></dl>\n' )
        . '">Vicuna CMS 互換テンプレート例</option>';
    print '</select>';
    require CMONOS::CGI_Lib::HTMLHelper;
    print HTMLHelper->new->html_helper_menu( $name . '_template' );

    if ( ref $frame::obj{'frame'} ) {
        my $sample = '';
        print
            ' <select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'',
            $name,
            '_template\').quotationByMenu(this)" title="これらのタグは、書き出し時に対応するフレームに変換されます。">';
        print
            '<option value="">▼フレームタグを挿入する</option>';
        foreach my $frid ( sort { $a <=> $b } keys %{ $frame::obj{'frame'} } )
        {
            $sample = $frid;
            print '<option value="&lt;!--frame id=&quot;', $frid,
                '&quot;--&gt;&lt;!--titlebar--&gt;&lt;!--/titlebar--&gt;&lt;!--contents--&gt;&lt;!--/contents--&gt;&lt;!--/frame--&gt;">ID : ',
                $frid, '</option>';
        }
        print '</select>';
        print
            '<div id="frame-tips"><label title="チェックするとフレームサンプルを表示します"><input type="checkbox" onclick="cmonos.element.item(\'framesample\').disappear(!this.checked);" /> フレームサンプルを表示する</label> <div id="framesample" style="display: none">';
        print $cgi::Deco->frame_menu( '', 'frame', '', '', 'ID' );
        print '</div>';
        print
            '<label title="チェックするとフレームタグの書き方を表示します"><input type="checkbox" onclick="cmonos.element.item(\'frametagsample\').disappear(!this.checked);" /> フレームタグの書き方を表示する</label> <table id="frametagsample" style="display: none"><tbody><tr><td id="sample-frame">';
        my ( $contents, $title ) = (
            '<strong>コンテンツ</strong>',
            '<strong>タイトル</strong>'
        );
        print $cgi::Deco->frame( 'frame', $sample, \$contents, \$title, 'no',
            '', 0, 'type', 'frame', 'titlenowrap', 1, 'contentsnowrap', 1 );
        print '</td><td id="sample-tag">';
        print
            '&lt;!--frame id=&quot;<strong>ID</strong>&quot;--&gt;<br />&lt;!--titlebar--&gt;<strong>タイトル</strong>&lt;!--/titlebar--&gt;<br />&lt;!--contents--&gt;<strong>コンテンツ</strong>&lt;!--/contents--&gt;<br />&lt;!--/frame--&gt;';
        print '</td></tr></tbody></table>';
        print '</div>';
    }

    print '</div></fieldset>';

    print '<fieldset>';
    print '<legend>トラックバック表示設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    if ( !$default::static ) {
        print '<tr>';
        print
            '<td class="field-name"><span class="datatitle">受信日時の表示</span></td>';
        print '<td class="field"><label for="'
            . $name
            . '_datelimit"><input type="number" size="3" maxlength="3" name="'
            . $name
            . '_datelimit" id="'
            . $name
            . '_datelimit" value="'
            . $this->escape( $this->{'datelimit'} )
            . '" /> 日以内なら相対日時で表現する（きょう/きのう/○日前）</label></td>';
        print '</tr>';
    }
    print '<tr class="tbody-end">';
    print '<td class="field-name"><label for="' . $name
        . '_datetype">受信日時表示形式</label></td>';
    print '<td class="field">';
    $this->print_date_type_menu( $name . '_', $this->{'datetype'} );
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '<tbody>';
    print '</tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_pingUrlName">URL ボックス名</size></td>';
    print '<td class="field"><input type="text" name="'
        . $name
        . '_pingUrlName" id="'
        . $name
        . '_pingUrlName" size="40" value="'
        . $this->escape( $this->{'pingUrlName'} )
        . '" /></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_size">URL ボックス幅</size></td>';
    print '<td class="field"><input type="number" name="'
        . $name
        . '_size" id="'
        . $name
        . '_size" size="3" value="'
        . $this->escape( $this->{'size'} )
        . '" /> 文字(半角)[0=非表示]</td>';
    print '</tr>';
    print '<tbody>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">表示のタイミング</span></td>';
    print '<td class="field"><label><input type="checkbox" name="'
        . $name
        . '_pendingEnabled" id="'
        . $name
        . '_pendingEnabled" value="1"'
        . ( ( defined $this->{'pendingEnabled'} && $this->{'pendingEnabled'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 承認されるまで表示しないことを可能にする</label></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><span class="datatitle">隠す</span></td>';
    print '<td class="field"><label><input type="checkbox" name="'
        . $name
        . '_hiddenTrackback" id="'
        . $name
        . '_hiddenTrackback" value="1"'
        . (
        ( defined $this->{'hiddenTrackback'} && $this->{'hiddenTrackback'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 初期状態でトラックバック数のみ表示する(クリックでトラックバックを表示)</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">リンクオプション</span></td>';
    print '<td class="field"><label><input type="checkbox" name="'
        . $name
        . '_nofollow" value="1"'
        . ( ( defined $this->{'nofollow'} && $this->{'nofollow'} )
        ? ' checked="checked"'
        : '' )
        . ' /> リンク先を検索エンジンに無視させる</label></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>トラックバック送信設定</legend>';
    print '<label><input type="checkbox" name="'
        . $name
        . '_trackbackDisabled" id="'
        . $name
        . '_trackbackDisabled" value="1"'
        . ( ( $this->{'trackbackDisabled'} ) ? ' checked="checked"' : '' )
        . ' /> トラックバック送信機能を<strong>使用しない</strong></label>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>トラックバック受信設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_masterMemory">トラックバック最大保存数</label></td>';
    print '<td class="field"><input type="number" name="'
        . $name
        . '_masterMemory" id="'
        . $name
        . '_masterMemory" value="'
        . $this->escape( $this->{'masterMemory'} )
        . '" size="3" title="各記事の設定に優先する最大保存数を設定します。" /> <span class="unit">件</span></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_maxlength">要約(excerpt)最大文字数</label></td>';
    print '<td class="field"><input type="number" name="'
        . $name
        . '_maxlength" id="'
        . $name
        . '_maxlength" value="'
        . $this->escape( $this->{'maxlength'} )
        . '" size="3" title="入力欄の文字数を制限します。" /> <span class="unit">文字（半角）</span></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">要約(excerpt)チェック</span></td>';
    print '<td class="field"><label><input type="checkbox" name="'
        . $name
        . '_type" id="'
        . $name
        . '_type" value="j"'
        . ( ( $this->{'type'} && $this->{'type'} eq 'j' )
        ? ' checked="checked"'
        : '' )
        . ' /> 日本語文字列必須</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">相互リンクチェック</span></td>';
    print '<td class="field"><label><input type="checkbox" name="'
        . $name
        . '_linkCheck" id="'
        . $name
        . '_linkCheck" value="1"'
        . ( ( $this->{'linkCheck'} ) ? ' checked="checked"' : '' )
        . ' /> トラックバック元に記事へのリンクがない場合、そのトラックバックを拒否する</label></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_repetition">重複トラックバックの扱い</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        $name . '_repetition', '重複トラックバックの処理', '',
        $this->{'repetition'},
        'allow' => '重複を許可する',
        'refuse' =>
            '新しいトラックバックを拒絶して古いトラックバックを残す',
        'shift' =>
            '古いトラックバックを削除して新しいトラックバックを追加する'
    );
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">後処理</span></td>';
    print '<td class="field"><ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_externalFileDisabled" id="'
        . $name
        . '_externalFileDisabled" value="1"'
        . (
        (   defined $this->{'externalFileDisabled'}
                && $this->{'externalFileDisabled'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> JavaScript による埋め込みではなく、その都度描画する (外部 JavaScript ファイルを作成しない)</label></li>'
        if !$default::static;
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mTime" id="'
        . $name
        . '_mTime" value="1"'
        . ( ( defined $this->{'mTime'} && $this->{'mTime'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 投稿時に更新時データベースを更新する</label></li>'
        if !$default::static;
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_rssEnabled" id="'
        . $name
        . '_rssEnabled" value="1"'
        . ( ( defined $this->{'rssEnabled'} && $this->{'rssEnabled'} )
        ? ' checked="checked"'
        : '' )
        . ' /> トラックバック RSS を更新する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_naviEnabled" id="'
        . $name
        . '_naviEnabled" value="1"'
        . ( ( defined $this->{'naviEnabled'} && $this->{'naviEnabled'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 最新トラックバック一覧を更新する</label></li>';
    print '</ol>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_mailtoMaster">受信通知送信先</label></td>';
    print '<td class="field">';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mailtoMaster" id="'
        . $name
        . '_mailtoMaster" value="1"'
        . ( ( defined $this->{'mailtoMaster'} && $this->{'mailtoMaster'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 管理人アドレス</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mailtoOwners" value="1"'
        . ( ( defined $this->{'mailtoOwners'} && $this->{'mailtoOwners'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 共同オーナーアドレス</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mailtoStaff" value="1"'
        . ( ( defined $this->{'mailtoStaff'} && $this->{'mailtoStaff'} )
        ? ' checked="checked"'
        : '' )
        . ' /> スタッフアドレス</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mailtoAllEMailAddresses" value="1"'
        . (
        (   defined $this->{'mailtoAllEMailAddresses'}
                && $this->{'mailtoAllEMailAddresses'}
        ) ? ' checked="checked"' : ''
        )
        . ' />すべてのメールアドレスフィールド</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mailToUser" value="1"'
        . ( ( defined $this->{'mailToUser'} && $this->{'mailToUser'} )
        ? ' checked="checked"'
        : '' )
        . ' /> ユーザー指定アドレス</label></li>';
    print '</ol>';
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>更新通知 Ping 送信設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr class="tbody-end">';
    print '<td class="field-name"><span class="datatitle">状態</span></td>';
    print '<td class="field"><label><input type="checkbox" name="'
        . $name
        . '_pingEnabled" id="'
        . $name
        . '_pingEnabled" value="1"'
        . ( ( $this->{'pingEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> 更新通知 Ping 送信機能を使用する</label></td>';
    print '</tr>';

    if ($default::static) {
        my ( $rss_dir, $link, $master_rss_dir, $master_link )
            = $this->get_rss_path;
        if ($rss_dir) {
            print '<tr>';
            print
                '<td class="field-name"><span class="datatitle">通知対象ディレクトリ</span></td>';
            print '<td class="field">';
            print '<ol class="checkbox">';
            print '<li><label><input type="radio" name="'
                . $name
                . '_weblogDirectory" value=""'
                . (
                (   !defined $this->{'weblogDirectory'}
                        || $this->{'weblogDirectory'} eq ''
                ) ? ' checked="checked"' : ''
                )
                . ' />'
                . ( ($env::title)
                ? $this->escape($env::title)
                : 'このサイト' )
                . '</label></li>';
            if ($master_rss_dir) {
                $this->import_db(
                    $default::master_preference_dir . '/general',
                    \%master::general, 1 )
                    if !%master::general;
                print '<li><label><input type="radio" name="'
                    . $name
                    . '_weblogDirectory" value="directory"'
                    . (
                    (   defined $this->{'weblogDirectory'}
                            && $this->{'weblogDirectory'} eq 'directory'
                    ) ? ' checked="checked"' : ''
                    )
                    . ' />'
                    . ( ( $master::general{'cginame'} )
                    ? $this->escape( $master::general{'cginame'} )
                    : 'このディレクトリ' )
                    . '</label></li>';
                print '<li><label><input type="radio" name="'
                    . $name
                    . '_weblogDirectory" value="category"'
                    . (
                    (   defined $this->{'weblogDirectory'}
                            && $this->{'weblogDirectory'} eq 'category'
                    ) ? ' checked="checked"' : ''
                    )
                    . ' />'
                    . ( ( $setting::general{'cginame'} )
                    ? $this->escape( $setting::general{'cginame'} )
                    : 'このカテゴリ' )
                    . '</label></li>';
            }
            else {
                print '<li><label><input type="radio" name="'
                    . $name
                    . '_weblogDirectory" value="directory"'
                    . (
                    (   defined $this->{'weblogDirectory'}
                            && $this->{'weblogDirectory'} eq 'directory'
                    ) ? ' checked="checked"' : ''
                    )
                    . ' />'
                    . ( ( $setting::general{'cginame'} )
                    ? $this->escape( $setting::general{'cginame'} )
                    : 'このカテゴリ' )
                    . '</label></li>';
            }
        }
    }
    print '</ol>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_pingServerList">更新通知 Ping 送信先</label></td>';
    print '<td class="field">';
    print '<textarea name="'
        . $name
        . '_pingServerList" id="'
        . $name
        . '_pingServerList" cols="72" rows="10" wrap="off">'
        . $this->escape( $this->{'pingServerList'} )
        . '</textarea>';
    print
        '<div class="note">書式="Ping URL&gt;Pingサーバ名&gt;PingサービスURL[改行]"</div>';
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';

    $this->print_setting_save_button;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this     = shift;
    my $is_valid = 1;
    $this->tag_fix( \$main::data{ $this->{'id'} . '_pingUrlName' } )
        if defined $main::data{ $this->{'id'} . '_pingUrlName' }
        && $main::data{ $this->{'id'} . '_pingUrlName' };
    if (   defined $main::data{ $this->{'id'} . '_size' }
        && $main::data{ $this->{'id'} . '_size' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_size' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'トラックバック URL 表示用テキストボックスサイズを更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_size' };
        $is_valid = 0;
    }
    $is_valid = 0 if !$this->is_valid_sub_setting;
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    $main::data{ $this->{'id'} . '_hiddenTrackback' } = ''
        if !defined $main::data{ $this->{'id'} . '_hiddenTrackback' }
        || !$main::data{ $this->{'id'} . '_hiddenTrackback' };
    $main::data{ $this->{'id'} . '_rssEnabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_rssEnabled' }
        || $main::data{ $this->{'id'} . '_rssEnabled' } eq '';
    $main::data{ $this->{'id'} . '_naviEnabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_naviEnabled' }
        || $main::data{ $this->{'id'} . '_naviEnabled' } eq '';
    $main::data{ $this->{'id'} . '_pingEnabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_pingEnabled' }
        || $main::data{ $this->{'id'} . '_pingEnabled' } eq '';
    $main::data{ $this->{'id'} . '_trackbackDisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_trackbackDisabled' }
        || $main::data{ $this->{'id'} . '_trackbackDisabled' } eq '';
    $main::data{ $this->{'id'} . '_linkCheck' } = ''
        if !defined $main::data{ $this->{'id'} . '_linkCheck' }
        || $main::data{ $this->{'id'} . '_linkCheck' } eq '';

    if (   $main::data{ $this->{'id'} . '_trackbackDisabled' }
        && !$main::data{ $this->{'id'} . '_pingEnabled' }
        && defined $main::data{ $this->{'id'} . '_masterMemory' }
        && $main::data{ $this->{'id'} . '_masterMemory' } eq '0' )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'トラックバックの送受信も Ping も使用しない設定になっています。このフィールドを使用するには、いずれかの機能を使用する必要があります。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_trackbackDisabled' };
        delete $main::data{ $this->{'id'} . '_pingEnabled' };
        delete $main::data{ $this->{'id'} . '_masterMemory' };
        $is_valid = 0;
    }
    $main::data{ $this->{'id'} . '_mailToUser' } = ''
        if defined $main::data{ $this->{'id'} . '_masterMemory' }
        && $main::data{ $this->{'id'} . '_masterMemory' } eq '0';
    $main::data{ $this->{'id'} . '_nofollow' } = ''
        if !defined $main::data{ $this->{'id'} . '_nofollow' }
        || $main::data{ $this->{'id'} . '_nofollow' } eq '';
    return $is_valid;
}

######################################################################
#	(11) obj->update;
sub update ($;$) {
    my $this = shift;

    #トラックバック閉鎖
    if ( defined $this->{'masterMemory'} && $this->{'masterMemory'} eq '0' ) {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'Your trackback is denied.'
                : 'トラックバックを受け付けていません。(Your trackback is denied.)'
            )
        ) if ref $cgi::action;
        return 0;
    }

    #入力のチェック
    my $viewer_url = $this->get_viewer_url( $main::c, $main::n );
    my $input_is_valid = 1;
    if ( !$main::url ) {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'The URL is undefined.'
                : 'URL がみつかりません。(The URL is undefined.)'
            )
        ) if ref $cgi::action;
        $input_is_valid = 0;
    }
    elsif ( $this->is_invalid_url($main::url) ) {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'The URL is invalid.'
                : 'URL が正しくありません。(The URL is invalid.)'
            )
        ) if ref $cgi::action;
        $input_is_valid = 0;
    }
    elsif ( $this->{'linkCheck'} ) {
        require CMONOS::CGI_Lib::Ping::Trackback;
        if (!Trackback->new( 'url' => $main::url )->linked( $viewer_url, 1 ) )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ($default::english)
                    ? 'There is no link to this article.'
                    : 'トラックバック元の記事にトラックバック先の記事へのリンクがありません。(There is no link to this article.)'
                )
            ) if ref $cgi::action;
            $input_is_valid = 0;
        }
    }
    if (   $this->{'type'}
        && $this->{'type'} eq 'j'
        && $this->is_not_japanese($main::excerpt) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'The excerpt is not japanese.'
                : '記事の要約が日本語ではありません。(The excerpt is not japanese.)'
            )
        ) if ref $cgi::action;
        $input_is_valid = 0;
    }
    return 0 if !$input_is_valid;

    #トラックバック設定読み込み
    my $setting = $this->get_field_data;

    #トラックバック閉鎖
    if (   !ref $setting
        || ( defined $setting->{'memory'}  && $setting->{'memory'} eq '0' )
        || ( !defined $setting->{'memory'} && $this->{'protected'} ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'Your trackback is denied.'
                : 'トラックバックを受け付けていません。(Your trackback is denied.)'
            )
        ) if ref $cgi::action;
        return 0;
    }

    my $file_dir = $this->get_file_directory;
    my $dat_path = $file_dir . '/' . $this->{'id'} . '.dat';

    #トラックバック数のカウント
    my $memory_count = 0;
    my $last_id      = 0;
    my $is_repeater  = 0;
    my $approval_required
        = ( $this->{'pendingEnabled'} && $setting->{'approvalRequired'} )
        ? 1
        : 0;
    local (*DAT);
    if ( open( DAT, $dat_path ) ) {
        binmode DAT, ":utf8";
        flock( DAT, LOCK_SH );
        my $repetition_check = ( $this->{'repetition'} eq 'refuse'
                || $this->{'repetition'} eq 'shift' ) ? 1 : 0;
        my $line = '';
        while ( $line = <DAT> ) {
            if ($line) {
                $memory_count++;
                $last_id = $1 if $line =~ /^([0-9]+)/;
                if ($repetition_check) {
                    my ( $id, $ctime, $approved, $url, $title, $excerpt,
                        $blog_name, $ip )
                        = $this->svsplit( "\x09", $line );
                    $is_repeater = 1 if $url eq $main::url;
                }
            }
        }
        close(DAT);
    }
    if ( $this->{'repetition'} eq 'refuse' && $is_repeater ) {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'Your entry have been already received.'
                : 'このトラックバックは既に受け付けられています。(Your entry have been already received.)'
            )
        ) if ref $cgi::action;
        return 0;
    }

    my $memory = (
        $this->{'masterMemory'}
            && ( !defined $setting->{'memory'}
            || $setting->{'memory'} eq ''
            || $setting->{'memory'} > $this->{'masterMemory'} )
    ) ? $this->{'masterMemory'} : $setting->{'memory'};
    $memory_count++ if !$is_repeater;

    #トラックバック閉鎖
    if ( defined $memory && $memory ne '' && $memory_count > $memory ) {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'Your trackback is denied.'
                : 'トラックバック数が最大保存数に達したので、これ以上受け付けることが出来ません。(Your trackback is denied.)'
            )
        ) if ref $cgi::action;
        return 0;
    }

    my $editor_url    = $this->get_user_generated_field_editor_url(1);
    my @new_trackback = (
        ( $last_id + 1 ),
        time,
        ( ($approval_required) ? 0 : 1 ),
        $main::url,
        ( ( defined $main::title )     ? $main::title     : '' ),
        ( ( defined $main::excerpt )   ? $main::excerpt   : '' ),
        ( ( defined $main::blog_name ) ? $main::blog_name : '' ),
        ( $ENV{'REMOTE_HOST'} || $this->to_host( $ENV{'REMOTE_ADDR'} ) ),
    );
    my $content = '';
    if ($is_repeater) {
        my $temp_path = $file_dir . '/' . $this->{'id'} . '.temp.dat';
        my $done      = 0;
        local (*TEMP);
        if ( open( TEMP, ">>$temp_path" ) ) {
            binmode TEMP, ":utf8";
            flock( TEMP, LOCK_EX );
            truncate( TEMP, 0 );
            seek( TEMP, 0, 0 );
            if ( open( DAT, $dat_path ) ) {
                binmode DAT, ":utf8";
                flock( DAT, LOCK_SH );
                while ( my $line = <DAT> ) {
                    my ( $id, $ctime, $approved, $url, $title, $excerpt,
                        $blog_name, $ip )
                        = $this->svsplit( "\x09", $line );
                    print TEMP $line if $line && $url ne $main::url;
                }
                close(DAT);
                $done = 1;
                print TEMP $this->svjoin( "\x09", @new_trackback ), "\x0A";
            }
            close(TEMP);
        }
        if ($done) {
            rename( $temp_path, $dat_path );
            chmod(
                ( ($env::ownerpower) ? 0600 : $env::file_mode ),
                $dat_path
            );
        }
    }
    elsif ( open( DAT, ">>$dat_path" ) ) {
        binmode DAT, ":utf8";
        flock( DAT, LOCK_EX );
        print DAT $this->svjoin( "\x09", @new_trackback ), "\x0A";
        close(DAT);
        chmod( ( ($env::ownerpower) ? 0600 : $env::file_mode ), $dat_path );
    }
    else {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'An unexpected error occured.'
                : '新しいトラックバックの保存に失敗しました。(An unexpected error occured.)'
            )
        ) if ref $cgi::action;
        return 0;
    }

    #通知メール
    if (   $this->{'mailtoMaster'}
        || $this->{'mailtoOwners'}
        || $this->{'mailtoStaff'}
        || $this->{'mailtoAllEMailAddresses'}
        || (   $this->{'mailToUser'}
            && $setting->{'mailto'}
            && $setting->{'mailto'} ne '' )
        )
    {
        my @bcc = ();
        if ( $this->{'mailtoAllEMailAddresses'} ) {
            push( @bcc, $this->get_all_mail_field_data_list );
        }
        elsif ($this->{'mailToUser'}
            && $setting->{'mailto'}
            && $setting->{'mailto'} ne '' )
        {
            push( @bcc, $this->get_mail_field_data( $setting->{'mailto'} ) );
        }
        push( @bcc, $setting::general{'master'} )
            if defined $this->{'mailtoMaster'}
            && $this->{'mailtoMaster'}
            && $setting::general{'master'};
        push( @bcc, $setting::general{'owners'} )
            if defined $this->{'mailtoOwners'}
            && $this->{'mailtoOwners'}
            && $setting::general{'owners'};
        push( @bcc, $setting::general{'staff'} )
            if defined $this->{'mailtoStaff'}
            && $this->{'mailtoStaff'}
            && $setting::general{'staff'};
        if (@bcc) {
            unless ( ref $cgi::SendMail ) {
                require CMONOS::CGI_Lib::SendMail;
                $cgi::SendMail = SendMail->new(
                    ( ( defined $env::sendmail ) ? $env::sendmail : '' ),
                    (   ( defined $env::master_email )
                        ? $env::master_email
                        : ''
                    ),
                    '',
                    ( ( ref $cgi::input ) ? $cgi::input : '' ),
                    1
                );
            }
            my $msg
                = $this->name
                . 'に新しいトラックバックがあります。'
                . "\x0A";
            $msg .= '新しいトラックバックがあるページ : '
                . $viewer_url . "\x0A";
            $msg
                .= 'トラックバック編集ツール : '
                . $editor_url . "\x0A"
                if $approval_required;
            $msg .= "\x0A\x0A";
            $msg
                .= '['
                . ( ($default::english) ? 'Blog Name' : 'サイト名' )
                . "]\x0A"
                . $main::blog_name
                . "\x0A\x0A"
                if defined $main::blog_name && $main::blog_name ne '';
            $msg
                .= '['
                . ( ($default::english) ? 'Title' : '記事タイトル' )
                . "]\x0A"
                . $main::title
                . "\x0A\x0A"
                if defined $main::title && $main::title ne '';
            $msg .= "[URL]\x0A" . $main::url . "\x0A\x0A";
            $msg
                .= '['
                . ( ($default::english) ? 'Excerpt' : '要約' ) . "]\x0A"
                . $main::excerpt
                . "\x0A\x0A"
                if defined $main::excerpt && $main::excerpt ne '';
            $cgi::SendMail->send_mail(
                'Bcc'     => join( ',', $this->get_address(@bcc) ),
                'From'    => $env::master_email,
                'Subject' => $this->{'name'}
                    . 'に新しいトラックバックがあります！',
                'message' => $msg,
                'info'    => 1
            );
            push( @{ $this->{'dialog'} }, $cgi::SendMail->dialog );
        }
    }
    if ($approval_required) {
        $this->save_pending_db( $editor_url, $this->get_title_field_data );
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'Thank you for your trackback.'
                : 'トラックバックありがとうございます。このトラックバックは内容が確認された後、表示されます。(Thank you for your trackback.)'
            )
        ) if ref $cgi::action;
    }
    else {
        $content
            = $this->set_embed_files( $memory, $setting, $file_dir, $main::c,
            $main::n, 1 );
        $this->feed_field_rss(
            'trackback',
            {   'title' => (
                    ( $main::blog_name ne '' ) ? $main::blog_name . ':' : ''
                    )
                    . ( ( defined $main::title ) ? $main::title : '' ),
                'link' => $main::url,
                'guid' => {
                    'isPermaLink' => 'true',
                    'value'       => $viewer_url,
                },
                'content:encoded' => '(<a href="'
                    . $viewer_url . '">'
                    . (
                    ( $setting->{'title'} )
                    ? $setting->{'title'}
                    : $viewer_url
                    )
                    . '</a> へのトラックバック) '
                    . $this->detag($main::excerpt),
                'category' => (
                    (          $default::subdivided
                            && defined $main::c
                            && $main::c ne ''
                            && $setting::general{'cginame'}
                    ) ? $setting::general{'cginame'} : ''
                ),
            }
        ) if $this->{'rssEnabled'};
        $this->navi_update( \@new_trackback, $viewer_url )
            if $this->{'naviEnabled'};
        $this->post_update;
        push(
            @{ $cgi::action->{'dialog'} },
            (   ($default::english)
                ? 'Thank you for your trackback.'
                : 'トラックバックありがとうございます。(Thank you for your trackback.)'
            )
        ) if ref $cgi::action;
    }
    return 1;
}

sub get_navi_info ($) {
    my $this = shift;
    return ( 'recenttrackbacks', 'recenttrackbacks.ssi',
        'recentTrackbacksTemplate' );
}

sub mk_recent_items_article ($$$;$) {
    my ( $this, $item_array, $template, $url ) = @_;
    if ($url) {
        $url = $this->escape($url);
        my @item_data = @$item_array;
        my $title     = $this->get_title_field_data;
        $title
            = $this->escape( $setting::general{'cginame'}
                . ( ( $setting::general{'cginame'} && $title ) ? ' > ' : '' )
                . $title );
        $title = ' title="' . $title . '"' if $title;
        if ( $template
            =~ /<!--title(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/title-->/s )
        {
            if ( defined $item_data[4] && $item_data[4] ne '' ) {
                $item_data[4]
                    = '<a href="'
                    . $url . '"'
                    . $title . '>'
                    . $this->detag_text( $item_data[4] ) . '</a>';
            }
            elsif ( $template
                =~ /<!--excerpt(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/excerpt-->/ )
            {
                $item_data[5]
                    = '<a href="'
                    . $url . '"'
                    . $title . '>'
                    . $this->detag_text( $item_data[5] ) . '</a>';
            }
            else {
                $item_data[4]
                    = '<a href="'
                    . $url . '"'
                    . $title . '>'
                    . $this->detag_text( $item_data[5] ) . '</a>';
            }
        }
        else {
            $item_data[5]
                = '<a href="'
                . $url . '">'
                . $this->detag_text( $item_data[5] ) . '</a>';
        }
        return $this->replace_trackback( $template, 0, \@item_data, 1 );
    }
    else {
        return $this->replace_trackback( $template, 0, $item_array, 1 );
    }
}

sub get_array_from_template ($$) {
    my ( $this, $template ) = @_;
    my $trackback = $this->get_hash_from_template($template);
    return [
        '',
        (   ( defined $trackback->{'ctime'} && !ref $trackback->{'ctime'} )
            ? $trackback->{'ctime'}
            : ''
        ),
        1,
        (   ( defined $trackback->{'url'} && !ref $trackback->{'url'} )
            ? $trackback->{'url'}
            : ''
        ),
        (   ( defined $trackback->{'title'} && !ref $trackback->{'title'} )
            ? $trackback->{'title'}
            : ''
        ),
        (   (   defined $trackback->{'excerpt'}
                    && !ref $trackback->{'excerpt'}
            ) ? $trackback->{'excerpt'} : ''
        ),
        (   (   defined $trackback->{'blog_name'}
                    && !ref $trackback->{'blog_name'}
            ) ? $trackback->{'blog_name'} : ''
        ),
        (   ( defined $trackback->{'ip'} && !ref $trackback->{'ip'} )
            ? $trackback->{'ip'}
            : ''
        )
    ];
}

######################################################################
sub set_user_generated_field_editor_page_settings ($) {
    my $this = shift;
    $setting::page{'ugfieldeditor'}{'title'}
        = $this->detag( $this->get_title_field_data );

    #	$setting::page{'ugfieldeditor'}{'text'} = '#000000';
    #	$setting::page{'ugfieldeditor'}{'link'} = '#0000ff';
    #	$setting::page{'ugfieldeditor'}{'alink'} = '#99ffff';
    #	$setting::page{'ugfieldeditor'}{'vlink'} = '#000099';
    #	$setting::page{'ugfieldeditor'}{'hover'} = '#00ffff';
    #	$setting::page{'ugfieldeditor'}{'nodeco'} = 'no';
    #	$setting::page{'ugfieldeditor'}{'bgcolor'} = '#ffffff';
    $setting::page{'ugfieldeditor'}{'bodyId'}      = 'trackback-editor';
    $setting::page{'ugfieldeditor'}{'cssFileName'} = 'trackback-editor.css';

# 	$setting::page{'ugfieldeditor'}{'CSS'} = 'h1 { font-size: large; font-weight: bold; text-indent: 1em; font-family: "ヒラギノ角ゴ Pro W3",Osaka,"ＭＳ Ｐゴシック",sans-serif;}' . "\x0A";
# 	$setting::page{'ugfieldeditor'}{'CSS'} .= 'h2 { font-size: medium; font-weight: bold; font-family: "ヒラギノ角ゴ Pro W3",Osaka,"ＭＳ Ｐゴシック",sans-serif;}' . "\x0A";
# 	$setting::page{'ugfieldeditor'}{'CSS'} .= 'hr { border-width: 1 0 0 0; }';
# 	$setting::page{'ugfieldeditor'}{'background'} = '';
# 	$setting::page{'ugfieldeditor'}{'bgproperties'} = '';
# 	$setting::page{'ugfieldeditor'}{'topmargin'} = '';
# 	$setting::page{'ugfieldeditor'}{'leftmargin'} = '';
# 	$setting::page{'ugfieldeditor'}{'javascript'} = '';
# 	$setting::page{'ugfieldeditor'}{'header'} = '';
# 	$setting::page{'ugfieldeditor'}{'headerFileName'} = '';
# 	$setting::page{'ugfieldeditor'}{'titleImageAlign'} = '';
# 	$setting::page{'ugfieldeditor'}{'titleImage'} = '';
# 	$setting::page{'ugfieldeditor'}{'align'} = '';
# 	$setting::page{'ugfieldeditor'}{'width'} = '';
# 	$setting::page{'ugfieldeditor'}{'deco'} = '';
# 	$setting::page{'ugfieldeditor'}{'titleText'} = '';
    return 1;
}

sub user_generated_field_edit ($) {
    my $this = shift;
    return 0 if !$this->is_editable;
    local ( *DAT, *TEMP );
    my $done            = 0;
    my $last_deleted_id = '';
    my $last_id         = '';
    my @last_approved_trackback;
    my $delete_list = '';
    my $temp_path
        = $this->{'file_directory'} . '/' . $this->{'id'} . '.temp.dat';

    if ( open( TEMP, ">>$temp_path" ) ) {
        binmode TEMP, ":utf8";
        flock( TEMP, LOCK_EX );
        truncate( TEMP, 0 );
        seek( TEMP, 0, 0 );
        if ( open( DAT, $this->{'dat_path'} ) ) {
            binmode DAT, ":utf8";
            flock( DAT, LOCK_SH );
            my $line;
            while ( $line = <DAT> ) {
                chomp $line;
                my @data = $this->svsplit( "\x09", $line );
                $last_id = $data[0]
                    if !$this->{'pendingEnabled'}
                    || !$this->{'field_data'}->{'approvalRequired'}
                    || $data[2];
                if ( $main::data{ $data[0] } ) {
                    if ( $main::data{ $data[0] } eq 'delete' ) {
                        $last_deleted_id = $data[0];
                        next;
                    }
                    elsif ($this->{'pendingEnabled'}
                        && $this->{'field_data'}->{'approvalRequired'} )
                    {
                        if ( $main::data{ $data[0] } eq 'approve' ) {
                            @last_approved_trackback = @data if !$data[2];
                            $data[2] = 1;
                        }
                        else {
                            $last_deleted_id = $data[0] if $data[2];
                            $data[2] = 0;
                        }
                    }
                    else {
                        $data[2] = 1;
                    }
                    print TEMP $this->svjoin( "\x09", @data ), "\x0A";
                }
                else {
                    print TEMP $line, "\x0A";
                }
            }
            close(DAT);
            $done = 1;
        }
        close(TEMP);
    }
    if ($done) {
        rename( $temp_path, $this->{'dat_path'} );
        chmod(
            ( ($env::ownerpower) ? 0600 : $env::file_mode ),
            $this->{'dat_path'}
        );
    }
    if ( @last_approved_trackback || $last_deleted_id ) {
        my $viewer_url = $this->get_viewer_url( $main::c, $main::n );
        $this->set_embed_files(
            $this->{'field_data'}->{'memory'},
            $this->{'field_data'}, $this->{'file_directory'},
            $main::c, $main::n
        );
        if (@last_approved_trackback) {
            my ( $id, $ctime, $approved, $url, $title, $excerpt, $blog_name,
                $ip )
                = @last_approved_trackback;
            $this->feed_field_rss(
                'trackback',
                {   'title' =>
                        ( ( $blog_name ne '' ) ? $blog_name . ':' : '' )
                        . ( ( $title ne '' ) ? $title : '' ),
                    'link' => $url,
                    'guid' => {
                        'isPermaLink' => 'true',
                        'value'       => $viewer_url,
                    },
                    'content:encoded' => '(<a href="'
                        . $viewer_url . '">'
                        . (
                        ( $this->{'field_data'}->{'title'} )
                        ? $this->{'field_data'}->{'title'}
                        : $viewer_url
                        )
                        . '</a> へのトラックバック) '
                        . $this->detag($excerpt),
                    'category' => (
                        (          $default::subdivided
                                && defined $main::c
                                && $main::c ne ''
                                && $setting::general{'cginame'}
                        ) ? $setting::general{'cginame'} : ''
                    ),
                }
            ) if $this->{'rssEnabled'};
            $this->navi_update( \@last_approved_trackback, $viewer_url )
                if $this->{'naviEnabled'};
            $this->post_update;
        }
        elsif ( $last_id eq $last_deleted_id ) {
            $this->navi_update( '', '', 1 ) if $this->{'naviEnabled'};
            $this->delete_field_rss_item( 'trackback', $viewer_url );
            $this->post_update(1);
        }
        else {
            $this->post_update(1);
        }
    }
    return 1;
}

sub print_user_generated_field_editor ($) {
    my $this = shift;
    $this->mk_settings_for_editor;
    $this->print_error_dialog(
        $this->name(1) . ' は現在利用できません。' )
        if !ref $this->{'field_data'};
    my $approval_required = ( $this->{'pendingEnabled'}
            && $this->{'field_data'}->{'approvalRequired'} ) ? 1 : 0;
    my $javascript = <<'SCRIPT';
function classChange (target_id,class_name,flag) {
	if (flag) document.getElementById(target_id).className = class_name;
}
SCRIPT
    $cgi::action->set_javascript($javascript);
    undef $javascript;
    $cgi::action->print_page_header( 'ugfieldeditor', '', '', 0, '',
        $this->name . '編集' );
    print $this->form_start(
        'key'      => 'ugfieldeditor',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()'
    );
    print $this->get_query_for_user_generated_field_editor(1);
    print '<input type="hidden" name="action" value="edit" />';
    local (*DAT);

    if ( open( DAT, $this->{'dat_path'} ) ) {
        binmode DAT, ":utf8";
        flock( DAT, LOCK_SH );
        while ( my $line = <DAT> ) {
            chomp $line;
            my ( $id, $ctime, $approved, $url, $title, $excerpt, $blog_name,
                $ip )
                = $this->svsplit( "\x09", $line, -1 );
            $this->tag_fix( \$excerpt );
            print '<div class="'
                . ( ( $approval_required && !$approved )
                ? 'pending-trackback'
                : 'approved-trackback' )
                . '" id="trackback-'
                . $id . '">';
            print '<div class="controller">';
            print '<label><input type="radio" name="data:'
                . $id
                . '" value="'
                . ( ($approved) ? '' : 'approve' ) . '"'
                . ( ( !$approval_required || $approved )
                ? ' checked="checked"'
                : '' )
                . ' onclick="cmonos.element.item(\'trackback-'
                . $id
                . '\').changeClassName(\'approved-trackback\',this.checked);" />承認</label>';
            print ' <label><input type="radio" name="data:'
                . $id
                . '" value="'
                . ( ( !$approved ) ? '' : 'pending' ) . '"'
                . ( ( !$approved ) ? ' checked="checked"' : '' )
                . ' onclick="cmonos.element.item(\'trackback-'
                . $id
                . '\').changeClassName(\'pending-trackback\',this.checked);" />保留</label>'
                if $approval_required;
            print ' <label><input type="radio" name="data:'
                . $id
                . '" value="delete" onclick="cmonos.element.item(\'trackback-'
                . $id
                . '\').changeClassName(\'delete-trackback\',this.checked);" />削除</label>';
            print '</div>';
            print '<div class="trackback">';
            print '<h1>' . $title . '(' . $id . ')</h1><p>';
            print $excerpt;
            print '</p><div class="trackback-info">'
                . $this->date( $ctime, $this->{'datelimit'},
                'gmt' . $this->{'datetype'} )
                . '/'
                . $this->escape($ip)
                . ( ($blog_name) ? '/<em>' . $blog_name . '</em>' : '' )
                . '</div>';
            print '</div>';
            print '</div>';
        }
        close(DAT);
    }
    print '<input type="submit" value="更新する" class="submit" />';
    print '</form>';
    $cgi::action->print_page_footer('ugfieldeditor');
    $this->rm_settings_for_editor;
    return 1;
}

######################################################################
#sub import {
BEGIN {
    push( @field::redundancy_forbidden, 'ug_trackback' );
}

1;
