package NavUpdate;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
eval 'use ' . $env::dbm_file . ';';
######################################################################
# ナビゲーション更新モジュール
# NavUpdate.pm ver1.0.5
# Copyright (C) 1999-2021 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2021-12-20
######################################################################
sub new ($$;$$$$) {
    my ( $pkg, $navi_db, $cgi_id, $sitemap_path, $sitemap_ssi, $navi_path,
        $top_path, $frame_path )
        = @_;
    my $this = {};
    if ($navi_db) {
        $this->{'navi_db'} = $navi_db;
        ( $this->{'page_pref'} = $navi_db ) =~ s/\/navi_db$/\/page/;
    }
    else {
        my $navi_pref
            = $default::master_preference_dir || $default::preference_dir;
        $navi_pref =~ s/([0-9a-zA-Z_]+)$/navimaker/;
        $this->{'navi_db'}   = $navi_pref . '/navi_db';
        $this->{'page_pref'} = $navi_pref . '/page';
    }
    local (*FH);
    if ((      !$cgi_id
            || !$sitemap_path
            || !$sitemap_ssi
            || !$navi_path
            || !$top_path
            || !$frame_path
        )
        && open( FH, '../navimaker/default.pl' )
        )
    {
        binmode FH;
        flock( FH, LOCK_SH );
        my $navi_setting = join( '', <FH> );
        close(FH);
        if (  !$cgi_id
            && $navi_setting
            =~ /\$id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*['"](.*?)['"];/s
            )
        {
            $cgi_id = $1;
        }
        if (  !$sitemap_path
            && $navi_setting
            =~ /\$sitemap_path[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
            )
        {
            $sitemap_path = eval $1;
        }
        if (  !$sitemap_ssi
            && $navi_setting
            =~ /\$sitemap_ssi[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
            )
        {
            $sitemap_ssi = eval $1;
        }
        if (  !$navi_path
            && $navi_setting
            =~ /\$navi_path[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
            )
        {
            $navi_path = eval $1;
        }
        if (  !$top_path
            && $navi_setting
            =~ /\$top_path[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
            )
        {
            $top_path = eval $1;
        }
        if (  !$frame_path
            && $navi_setting
            =~ /\$frame_path[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
            )
        {
            $frame_path = eval $1;
        }
    }

    $this->{'cgi_id'} = $cgi_id || '_navimaker';
    $this->{'sitemap_path'}
        = $sitemap_path
        || '../../sitemap.'
        . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' );
    $this->{'sitemap_ssi'}
        = $sitemap_ssi || $env::externalfiles . '/site-map.ssi';
    $this->{'navi_path'}  = $navi_path  || '';
    $this->{'top_path'}   = $top_path   || '';
    $this->{'frame_path'} = $frame_path || '';
    $this->{'ignorable_cgi'} = [
        'GateKeeper',         'SetupAssistant',
        'MaintenanceStation', 'ZipCodeSearch'
    ];
    bless $this;
    $this;
}

######################################################################
# サイトスキャン
sub scan ($;$$) {
    my ( $this, $old_order, $new_order, $delete_array, $data ) = @_;
    my ( @folder, %del, $save );
    $new_order = {} if !defined $new_order || !ref $new_order;
    $old_order = { $this->import_order( $this->{'navi_db'}, $new_order ) }
        if !defined $old_order || !ref $old_order;
    if ( !ref $data ) {
        $data = {};
        $save = 1;
    }
    local (*DIR);
    %del = %$old_order;
    opendir( DIR, '../' );
    @folder = grep( $_ !~ /^\./ && -d '../' . $_, readdir(DIR) );
    closedir(DIR);
    foreach my $dir (@folder) {
        no warnings qw(untie);
        my ($setting,     $name,       $lib,        $preference_dir,
            $mode,        $subdivided, $categories, $title,
            $description, $uri,        %general,    $id
        );
        local (*SETTING);
        next
            if !-f '../' . $dir . '/start.cgi'
            || !open( SETTING, '../' . $dir . '/default.pl' );
        binmode SETTING, ":utf8";
        flock( SETTING, LOCK_SH );
        $setting = join( '', <SETTING> );
        close(SETTING);

        if ( $setting
            =~ /^[\x20\x09]*\$lib[\x20\x09]*=[\x20\x09]*["']?([^"']+?)["']?;/m
            )
        {
            $lib = $1;
        }
        else {
            next;
        }
        next
            if ref $this->{'ignorable_cgi'}
            && grep( $_ eq $lib, @{ $this->{'ignorable_cgi'} } );
        if ( $setting
            =~ /^[\x20\x09]*\$id[\x20\x09]*=[\x20\x09]*["']([0-9a-zA-Z_]+)["'];/m
            )
        {
            $id = $1;
        }
        else {
            $id = $dir;
        }

#if ($setting =~ /^[\x20\x09]*\$preference_dir[\x20\x09]*=[\x20\x09]*["']?([^"']+?)["']?;/m) {
#	$preference_dir = $1;
        if ( $setting =~ /^[\x20\x09]*\$preference_dir[\x20\x09]*=(.*?);/m ) {
            $preference_dir = eval($1);
        }
        else {
            next;
        }
        if ( $lib eq 'NaviMaker' ) {
            $data->{'navimakerSiteMap_title'}       = 'サイトマップ';
            $data->{'navimakerSiteMap_id'}          = $id . '_sitemap';
            $data->{'navimakerSiteMap_uri'}         = $this->{'sitemap_path'};
            $data->{'navimakerSiteMap_lib'}         = $lib;
            $data->{'navimakerSiteMap_mode'}        = '';
            $data->{'navimakerSiteMap_description'} = '';
            $data->{'navimakerSiteMap_subdivided'}  = '';
            $data->{'navimakerSiteMap_categories'}  = '';
            $data->{'navimakerSiteMap_menu'}        = '';
            delete $del{'navimakerSiteMap'};
            my %page;
            tie( %page, $env::dbm_file, $preference_dir . '/page',
                O_RDONLY, $env::dbm_mode );

            if ( ref tied %page ) {
                $data->{'navimakerSiteMap_title'} = $page{'sitemap_title'}
                    if $page{'sitemap_title'};
            }
            untie %page;
            $old_order->{'navimakerSiteMap'} = 9999
                if !defined $old_order->{'navimakerSiteMap'};
            $new_order->{'navimakerSiteMap'} = 9999
                if !defined $new_order->{'navimakerSiteMap'};
        }
        else {
            if ( $setting
                =~ /^[\x20\x09]*\$name[\x20\x09]*=[\x20\x09]*["']?([^"']+?)["']?;/m
                )
            {
                $name = $1;
            }
            if ( $lib eq 'HomePageMaker' ) {
                $mode = 'date'
                    if $setting
                    =~ /^[\x20\x09]*\$date[\x20\x09]*=[\x20\x09]*["']?([^"']+?)["']?;/m
                    && $1;
                $uri = eval($1)
                    if $setting =~ /^[\x20\x09]*\$home[\x20\x09]*=(.*?);/m;

#$uri = $1 if $setting =~ /^[\x20\x09]*\$home[\x20\x09]*=[\x20\x09]*["']?([^"']+?)["']?;/m;
            }
            elsif ( $lib eq 'HomePageMakerLite' ) {
                $uri = eval($1)
                    if $setting =~ /^[\x20\x09]*\$home[\x20\x09]*=(.*?);/m;

#$uri = $1 if $setting =~ /^[\x20\x09]*\$home[\x20\x09]*=[\x20\x09]*["']?([^"']+?)["']?;/m;
            }
            else {
                $uri = '../' . $dir . '/start.cgi';
            }
            if ( $setting
                =~ /^[\x20\x09]*\$subdivided[\x20\x09]*=[\x20\x09]*["']?([^"']+?)["']?;/m
                )
            {
                $subdivided = $1;
            }
            tie( %general, $env::dbm_file, $preference_dir . '/general',
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %general ) {
                my ($html_flag);
                $title       = $this->decode_utf8( $general{'cginame'} );
                $description = $this->decode_utf8( $general{'description'} )
                    if $general{'description'};
                $html_flag = $this->decode_utf8( $general{'html'} )
                    if !$subdivided && $lib eq 'Baobab';
                untie %general;
                if ( !$subdivided && $lib eq 'Baobab' ) {
                    my (%security);
                    tie( %security, $env::dbm_file,
                        $preference_dir . '/security',
                        O_RDONLY, $env::dbm_mode );
                    if ( ref tied %security ) {
                        $html_flag = 0
                            if $this->decode_utf8(
                            $security{'browse_passwordCheck'} ) ne 'nocheck'
                            || $this->decode_utf8(
                            $security{'secret_passwordCheck'} ) ne 'nocheck'
                            || $this->decode_utf8(
                            $security{'closed_passwordCheck'} ) ne 'nocheck';
                    }
                    untie %security;
                    if ( defined $html_flag && $html_flag ) {
                        $mode = 'html';
                        $uri  = '../' . $dir . '/files/index.html';
                    }
                }
            }
            else {
                untie %general;
            }
            if ( $lib eq 'TableMaker' && $dir ne 'order' ) {
                $categories = $this->get_tm_categories($preference_dir);
            }
            elsif (
                $subdivided
                && (   $lib eq 'Baobab'
                    || $lib eq 'Calendar'
                    || $lib eq 'HomePageMaker' )
                && opendir( DIR, $preference_dir )
                )
            {
                $categories
                    = $this->get_categories( $lib, $preference_dir, $id,
                    $uri );
            }
            $title = $this->get_default_cgi_name( $id, $name )
                if !defined $title || $title eq '';
            $data->{ $dir . '_title' }       = $title;
            $data->{ $dir . '_id' }          = $id;
            $data->{ $dir . '_uri' }         = $uri;
            $data->{ $dir . '_lib' }         = $lib;
            $data->{ $dir . '_mode' }        = $mode;
            $data->{ $dir . '_description' } = $description || '';
            $data->{ $dir . '_subdivided' }  = $subdivided || '';
            $data->{ $dir . '_categories' }  = $categories || '';
            $data->{ $dir . '_menu' }        = '' if !$categories;
            delete $del{$dir};
            $old_order->{$dir} = 9999 if !defined $old_order->{$dir};
            $new_order->{$dir} = 9999 if !defined $new_order->{$dir};

            if ( $dir eq 'order' ) {
                $data->{'order_pagetitle'} = '注文履歴';
                $data->{'order_uri'} = '../order/start.cgi?m=DetailGate';

                if ( $setting
                    =~ /^[\x20\x09]*\$file[\x20\x09]*=[\x20\x09]*["']([^"']+?)["'];/m
                    )
                {
                    $data->{'orderPaymentInfo_uri'}
                        = $this->un_base_change( $1, '../order/start.cgi',
                        'rel', 1 )
                        . '/payment_info.'
                        . ( ( $env::ssi && $env::ssi_ext )
                        ? $env::ssi_ext
                        : 'html' );
                    $data->{'orderPaymentInfo_title'}
                        = 'お支払い方法および送料について';

                    $data->{'orderCartViewer_uri'}
                        = '../order/start.cgi?m=CartViewer&amp;iframe_disabled=1';
                    $data->{'orderCartViewer_title'} = 'カートを見る';

                    my %page;
                    tie( %page, $env::dbm_file, $preference_dir . '/page',
                        O_RDONLY, $env::dbm_mode );
                    if ( ref tied %page ) {
                        $data->{'order_pagetitle'} = $page{'detailgate_title'}
                            if $page{'detailgate_title'};
                        $data->{'orderPaymentInfo_title'}
                            = $page{'info_title'}
                            if $page{'info_title'};
                        $data->{'orderCartViewer_title'}
                            = $page{'cartviewer_title'}
                            if $page{'cartviewer_title'};
                    }
                    untie %page;

                    $data->{'orderPaymentInfo_id'}         = $id . '_terms';
                    $data->{'orderPaymentInfo_lib'}        = '';
                    $data->{'orderPaymentInfo_mode'}       = '';
                    $data->{'orderPaymentInfo_subdivided'} = '';
                    delete $del{'orderPaymentInfo'};
                    $old_order->{'orderPaymentInfo'} = 9999
                        if !defined $old_order->{'orderPaymentInfo'};
                    $new_order->{'orderPaymentInfo'} = 9999
                        if !defined $new_order->{'orderPaymentInfo'};

                    $data->{'orderCartViewer_id'}         = $id . '_cart';
                    $data->{'orderCartViewer_lib'}        = '';
                    $data->{'orderCartViewer_mode'}       = '';
                    $data->{'orderCartViewer_subdivided'} = '';
                    delete $del{'orderCartViewer'};
                    $old_order->{'orderCartViewer'} = 9999
                        if !defined $old_order->{'orderCartViewer'};
                    $new_order->{'orderCartViewer'} = 9999
                        if !defined $new_order->{'orderCartViewer'};
                }
            }
        }
    }

    #データ削除
    if (%del) {
        push( @$delete_array, grep( $_ !~ /^URI[0-9]+$/, keys %del ) );
        undef %del;
    }

    if ($save) {

        #削除項目の順序判定への影響を回避
        foreach (@$delete_array) {
            delete $old_order->{$_};
            delete $new_order->{$_};
        }

        #順序判定
        my $no = 0;
        no warnings qw(numeric);
        foreach my $num (
            sort {
                       $new_order->{$a} <=> $new_order->{$b}
                    || $old_order->{$b} <=> $old_order->{$a}
                    || $new_order->{$a} cmp $new_order->{$b}
                    || $a <=> $b
                    || $a cmp $b
            } keys %$old_order
            )
        {
            $no++;
            $data->{ 'ORDER_' . $no } = $num;
            $data->{ $num . '_ORDER' } = $no;
        }
        $this->write_db( $this->{'navi_db'}, $data, 'ORDER', $delete_array );
    }
    return $this;
}

######################################################################
# ナビゲーション更新
sub update ($;$$$$) {
    my ( $this, $navi, $page, $icon, $icon_setting ) = @_;
    local (
        $cgi::icon,     $default::id,     $default::lib,
        $default::name, $default::static, %setting::page,
        %setting::icon, $setting::key,    $setting::security_key,
        *OUT
    );
    $default::static = 1;
    $default::id     = $this->{'cgi_id'};
    $default::lib    = 'NaviMaker';
    $default::name   = 'ナビゲーション作成';
    $setting::key    = 'navi';
    %setting::page   = ();

    if ( ref $navi ) {
        $this->{'navi'} = $navi;
    }
    else {
        $this->{'navi'} = {};
        $this->import_db( $this->{'navi_db'}, $this->{'navi'}, 1 );
    }
    if ( ref $page ) {
        %setting::page = %$page;
        $this->{'page'} = $page;
    }
    else {
        $this->import_db( $this->{'page_pref'}, \%setting::page, 1, 'navi',
            'sitemap' );
        $this->{'page'} = \%setting::page;
    }
    if ( ref $icon ) {
        $this->{'icon'} = $cgi::icon = $icon;
    }
    else {
        $this->use_icon( 'navi', 'navi_icon' );
        $this->{'icon'} = $cgi::icon;
    }
    if ( ref $icon_setting ) {
        $this->{'icon_setting'} = $icon_setting;
    }
    else {
        $this->use_icon( 'navi', 'navi_icon' );
        $this->{'icon'}         = $cgi::icon;
        $this->{'icon_setting'} = \%setting::icon;
    }

    #チェックアウトのタイトル調整
    $this->{'navi'}{'order'}{'title'} = $this->{'navi'}{'order'}{'pagetitle'}
        if defined $this->{'navi'}{'order'}{'pagetitle'}
        && $this->{'navi'}{'order'}{'pagetitle'} ne '';

    #埋め込み方式
    if ( defined $env::inline_navi && $env::inline_navi ) {
        my $navi_path = $env::externalfiles . '/site-navi.ssi';
        if ( open( OUT, ">>$navi_path" ) ) {
            binmode OUT, ":utf8";
            flock( OUT, LOCK_EX );
            truncate( OUT, 0 );
            seek( OUT, 0, 0 );
            print OUT $this->gsearch_tag
                if $this->{'page'}{'navi'}{'gsearchenabled'};
            print OUT $this->{'page'}{'navi'}{'naviheader'}
                if defined $this->{'page'}{'navi'}{'naviheader'}
                && $this->{'page'}{'navi'}{'naviheader'};

            #テンプレート
            if ( $this->{'page'}{'navi'}{'type'} eq 'template' ) {
                $this->print_navi_from_template( \*OUT, $navi_path, 'abs' );
                if ( $this->{'page'}{'navi'}{'template'}
                    =~ /<\!--sitenavi(?![0-9a-zA-Z_])[^>]*type="icon"[^>]*-->/s
                    )
                {
                    $this->file_copy_and_add(
                        $env::css . '/navi-image.css',
                        $env::css . '/site-navi.css',
                        $this->gsearch_css
                            . (
                            ( $this->{'page'}{'navi'}{'navicss'} )
                            ? $this->{'page'}{'navi'}{'navicss'}
                            : ''
                            )
                    );
                }
                else {
                    $this->write_file(
                        $env::css . '/site-navi.css',
                        $this->gsearch_css
                            . (
                            ( $this->{'page'}{'navi'}{'navicss'} )
                            ? $this->{'page'}{'navi'}{'navicss'}
                            : ''
                            ),
                        1
                    );
                }

                #VICUNA
            }
            elsif ( $this->{'page'}{'navi'}{'type'} eq 'vicuna' ) {
                $this->print_vicuna_header( \*OUT, $navi_path, 'abs' );

                #アイコン
            }
            elsif ( $this->{'page'}{'navi'}{'type'} eq 'icon' ) {
                my $background    = '';
                my $site_navi_css = '';
                if ( $this->{'page'}{'navi'}{'background'} ) {
                    if ( $this->{'page'}{'navi'}{'background'}
                        =~ /^[0-9a-zA-Z_]+$/i )
                    {
                        $background
                            = $this->image_file_whole_path( $env::image_dir
                                . '/background/'
                                . $this->{'page'}{'navi'}{'background'} );
                    }
                    elsif ( $this->{'page'}{'navi'}{'background'}
                        =~ /\.(gif|jpg|jpeg|png)$/i )
                    {
                        $background = $this->{'page'}{'navi'}{'background'};
                    }
                }

                #CSS
                if ( !$this->{'page'}{'navi'}{'icontable'} ) {
                    my $navi_width = 0;
                    my $navi_image_dir
                        = $this->base_change( $env::image_dir,
                        $env::css . '/site-navi.css' )
                        . '/navi/';
                    $site_navi_css .= 'div#site-navi {' . "\x0A";
                    if ( defined $this->{'page'}{'navi'}{'iconbgalign'}
                        && $this->{'page'}{'navi'}{'iconbgalign'} )
                    {
                        if ( $this->{'page'}{'navi'}{'iconbgalign'} eq
                            'center' )
                        {
                            $site_navi_css
                                .= '  margin: 0 auto 0 auto;' . "\x0A";
                        }
                        elsif ( $this->{'page'}{'navi'}{'iconbgalign'} eq
                            'right' )
                        {
                            $site_navi_css
                                .= '  margin: 0 0 0 auto;' . "\x0A";
                        }
                        else {
                            $site_navi_css .= '  margin: 0;' . "\x0A";
                        }
                    }
                    else {
                        $site_navi_css .= '  margin: 0;' . "\x0A";
                    }
                    $site_navi_css .= '  padding: 0;' . "\x0A";
                    $site_navi_css
                        .= '  width: '
                        . ( ( $this->{'page'}{'navi'}{'iconbgwidth'} )
                        ? $this->{'page'}{'navi'}{'iconbgwidth'} . 'px'
                        : '100%' )
                        . ';' . "\x0A";
                    $site_navi_css
                        .= '  height: '
                        . $this->{'page'}{'navi'}{'size'} . 'px;' . "\x0A"
                        if defined $this->{'page'}{'navi'}{'size'}
                        && $this->{'page'}{'navi'}{'size'};
                    $site_navi_css
                        .= '  text-align: '
                        . $this->{'page'}{'navi'}{'align'} . ';' . "\x0A"
                        if defined $this->{'page'}{'navi'}{'align'}
                        && $this->{'page'}{'navi'}{'align'}
                        && $this->{'page'}{'navi'}{'align'}
                        =~ /^(left|right|center)$/;
                    $site_navi_css
                        .= '  background-color: '
                        . $this->{'page'}{'navi'}{'bgcolor'} . ';' . "\x0A"
                        if defined $this->{'page'}{'navi'}{'bgcolor'}
                        && $this->{'page'}{'navi'}{'bgcolor'};
                    $site_navi_css .= '  background-image: url("'
                        . $this->base_change( $background,
                        $env::css . '/site-navi.css' )
                        . '");' . "\x0A"
                        if defined $background && $background;
                    $site_navi_css .= '}' . "\x0A";
                    $site_navi_css .= 'div#site-navi ul {' . "\x0A";
                    $site_navi_css .= '  list-style-type: none;' . "\x0A";

                    if ( defined $this->{'page'}{'navi'}{'align'}
                        && $this->{'page'}{'navi'}{'align'} )
                    {
                        if ( $this->{'page'}{'navi'}{'align'} eq 'left' ) {
                            $site_navi_css .= '  margin: 0;' . "\x0A";
                        }
                        elsif ( $this->{'page'}{'navi'}{'align'} eq 'center' )
                        {
                            $site_navi_css
                                .= '  margin: 0 auto 0 auto;' . "\x0A";
                        }
                        elsif ( $this->{'page'}{'navi'}{'align'} eq 'right' )
                        {
                            $site_navi_css
                                .= '  margin: 0 0 0 auto;' . "\x0A";
                        }
                    }
                    $site_navi_css .= '  padding: 0;' . "\x0A";
                    $site_navi_css .= '  width: _W_px;' . "\x0A";
                    $site_navi_css
                        .= '  height: '
                        . $this->{'page'}{'navi'}{'size'} . 'px;' . "\x0A"
                        if defined $this->{'page'}{'navi'}{'size'}
                        && $this->{'page'}{'navi'}{'size'};
                    $site_navi_css .= '}' . "\x0A";
                    $site_navi_css .= 'div#site-navi ul li {' . "\x0A";
                    $site_navi_css .= '  display: block;' . "\x0A";
                    $site_navi_css .= '  float: left;' . "\x0A";
                    $site_navi_css .= '  margin: 0;' . "\x0A";

                    if ( defined $this->{'page'}{'navi'}{'space'}
                        && $this->{'page'}{'navi'}{'space'} )
                    {
                        my ( $padding_left, $padding_right );
                        $padding_left
                            = int( $this->{'page'}{'navi'}{'space'} / 2 );
                        $padding_right = $padding_left
                            + ( $this->{'page'}{'navi'}{'space'} % 2 );
                        $site_navi_css .= '  padding-left: '
                            . $padding_left . 'px;' . "\x0A";
                        $site_navi_css .= '  padding-right: '
                            . $padding_right . 'px;' . "\x0A";
                    }
                    else {
                        $site_navi_css .= '  padding: 0;' . "\x0A";
                    }
                    $site_navi_css .= '}' . "\x0A";

                   #$site_navi_css .= 'div#site-navi ul li a span {' . "\x0A";
                   #$site_navi_css .= '  display: none;' . "\x0A";
                   #$site_navi_css .= '}' . "\x0A";
                    $site_navi_css .= 'div#site-navi ul li a {' . "\x0A";
                    $site_navi_css .= '  display: block;' . "\x0A";
                    $site_navi_css .= '  float: left;' . "\x0A";
                    $site_navi_css .= '  margin: 0;' . "\x0A";
                    $site_navi_css .= '  padding: 0;' . "\x0A";
                    $site_navi_css
                        .= '  background-repeat: no-repeat;' . "\x0A";
                    $site_navi_css
                        .= '  background-position: left top;' . "\x0A";
                    $site_navi_css .= '  font-size: 1px;' . "\x0A";
                    $site_navi_css .= '  line-height: 1px;' . "\x0A";
                    $site_navi_css .= '  text-indent: -4000px;' . "\x0A";
                    $site_navi_css .= '  text-decoration: none;' . "\x0A";
                    $site_navi_css .= '  overflow: hidden;' . "\x0A";
                    $site_navi_css .= '  outline: none;' . "\x0A";
                    $site_navi_css .= '}' . "\x0A";

                    print OUT '<div id="site-navi"><ul>';
                    foreach ( sort { $a <=> $b }
                        keys %{ $this->{'navi'}{'ORDER'} } )
                    {
                        my ( $id, $icon, $uri, $categories );
                        $id = $this->{'navi'}{'ORDER'}{$_};
                        next
                            if !defined $this->{'navi'}{$id}{'use'}
                            || !$this->{'navi'}{$id}{'use'};
                        $this->{'navi'}{$id}{'title'} = ''
                            if !defined $this->{'navi'}{$id}{'title'};
                        ( $uri, $categories, $icon )
                            = $this->get_default_uri( $id,
                            $this->{'navi'}{$id} );
                        print OUT '<li id="tab-'
                            . $id
                            . '"><a href="'
                            . $this->escape_except_html_entity(
                            $this->path_change(
                                $uri, $ENV{'SCRIPT_NAME'}, 'abs'
                            )
                            )
                            . '" title="'
                            . $this->escape_except_html_entity(
                            $this->{'navi'}{$id}{'title'} )
                            . (
                            ( $this->{'navi'}{$id}{'description'} ) ? ':'
                                . $this->detag_text(
                                $this->{'navi'}{$id}{'description'}
                                ) : ''
                            )
                            . '"'
                            . ( ( $this->{'navi'}{$id}{'openWindow'} )
                            ? ' target="_blank"'
                            : '' )
                            . '><span>'
                            . $this->{'navi'}{$id}{'title'}
                            . '</span></a>'
                            . $categories . '</li>';
                        if (   defined $this->{'navi'}{$id}{'icon'}
                            && $this->{'navi'}{$id}{'icon'} ne ''
                            && ref $this->{'icon_setting'}{$icon}
                            { $this->{'navi'}{$id}{'icon'} }
                            && $this->{'icon_setting'}{$icon}
                            { $this->{'navi'}{$id}{'icon'} }{'link'} )
                        {
                            my ($padding_top, $padding_bottom,
                                $this_icon,   $this_image_dir
                            );
                            $this_icon = $this->{'icon_setting'}{$icon}
                                { $this->{'navi'}{$id}{'icon'} };
                            $this_image_dir
                                = $navi_image_dir
                                . $icon . '/'
                                . $this->{'navi'}{$id}{'icon'} . '/';
                            if (   $this->{'page'}{'navi'}{'cellvalign'}
                                && $this->{'page'}{'navi'}{'cellvalign'} ne
                                'top'
                                && $this->{'page'}{'navi'}{'size'}
                                && $this_icon->{'linkheight'}
                                && $this->{'page'}{'navi'}{'size'}
                                > $this_icon->{'linkheight'} )
                            {
                                my $remainder
                                    = $this->{'page'}{'navi'}{'size'}
                                    - $this_icon->{'linkheight'};
                                if ( $this->{'page'}{'navi'}{'cellvalign'} eq
                                    'bottom' )
                                {
                                    $padding_top = $remainder;
                                }
                                else {
                                    $padding_top = int( $remainder / 2 );
                                    $padding_bottom
                                        = $padding_top + ( $remainder % 2 );
                                }
                                $site_navi_css .= 'div#site-navi ul li#tab-'
                                    . $id . ' {' . "\x0A";
                                $site_navi_css
                                    .= '  padding-top: '
                                    . $padding_top . 'px;' . "\x0A"
                                    if defined $padding_top;
                                $site_navi_css
                                    .= '  padding-bottom: '
                                    . $padding_bottom . 'px;' . "\x0A"
                                    if defined $padding_bottom;
                                $site_navi_css .= '}' . "\x0A";
                            }
                            if ( !$this_icon->{'hover'} ) {
                                if ( $this_icon->{'active'} ) {
                                    (   $this_icon->{'hover'},
                                        $this_icon->{'hoverwidth'},
                                        $this_icon->{'hoverheight'}
                                        )
                                        = (
                                        $this_icon->{'active'},
                                        $this_icon->{'activewidth'},
                                        $this_icon->{'activeheight'}
                                        );
                                }
                                elsif ( $this_icon->{'visited'} ) {
                                    (   $this_icon->{'hover'},
                                        $this_icon->{'hoverwidth'},
                                        $this_icon->{'hoverheight'}
                                        )
                                        = (
                                        $this_icon->{'visited'},
                                        $this_icon->{'visitedwidth'},
                                        $this_icon->{'visitedheight'}
                                        );
                                }
                            }
                            $site_navi_css .= 'div#site-navi ul li#tab-'
                                . $id . ' a {' . "\x0A";
                            $site_navi_css .= '  width: '
                                . $this_icon->{'linkwidth'} . 'px;' . "\x0A";
                            $site_navi_css .= '  height: '
                                . $this_icon->{'linkheight'} . 'px;' . "\x0A";
                            $site_navi_css
                                .= '  background-image: url("'
                                . $this_image_dir
                                . $this_icon->{'link'} . '");' . "\x0A";
                            $site_navi_css .= '}' . "\x0A";
                            if ( $this_icon->{'visited'} ) {
                                $site_navi_css
                                    .= 'body#document_'
                                    . $id
                                    . ' div#site-navi ul li#tab-'
                                    . $id
                                    . ' a, div#site-navi ul li#tab-'
                                    . $id
                                    . ' a:visited {' . "\x0A";
                                $site_navi_css
                                    .= '  width: '
                                    . $this_icon->{'visitedwidth'} . 'px;'
                                    . "\x0A";
                                $site_navi_css
                                    .= '  height: '
                                    . $this_icon->{'visitedheight'} . 'px;'
                                    . "\x0A";
                                $site_navi_css
                                    .= '  background-image: url("'
                                    . $this_image_dir
                                    . $this_icon->{'visited'} . '");'
                                    . "\x0A";
                                $site_navi_css .= '}' . "\x0A";
                            }
                            if ( $this_icon->{'hover'} ) {
                                $site_navi_css
                                    .= 'body#document_'
                                    . $id
                                    . ' div#site-navi ul li#tab-'
                                    . $id
                                    . ' a, div#site-navi ul li#tab-'
                                    . $id
                                    . ' a:hover {' . "\x0A";
                                $site_navi_css
                                    .= '  width: '
                                    . $this_icon->{'hoverwidth'} . 'px;'
                                    . "\x0A";
                                $site_navi_css
                                    .= '  height: '
                                    . $this_icon->{'hoverheight'} . 'px;'
                                    . "\x0A";
                                $site_navi_css
                                    .= '  background-image: url("'
                                    . $this_image_dir
                                    . $this_icon->{'hover'} . '");' . "\x0A";
                                $site_navi_css .= '}' . "\x0A";
                            }
                            if ( $this_icon->{'active'} ) {
                                $site_navi_css
                                    .= 'div#site-navi ul li#tab-'
                                    . $id
                                    . ' a:active {' . "\x0A";
                                $site_navi_css
                                    .= '  width: '
                                    . $this_icon->{'activewidth'} . 'px;'
                                    . "\x0A";
                                $site_navi_css
                                    .= '  height: '
                                    . $this_icon->{'activeheight'} . 'px;'
                                    . "\x0A";
                                $site_navi_css
                                    .= '  background-image: url("'
                                    . $this_image_dir
                                    . $this_icon->{'active'} . '");' . "\x0A";
                                $site_navi_css .= '}' . "\x0A";
                            }
                            $navi_width += $this_icon->{'linkwidth'};
                        }
                        else {
                            $site_navi_css .= 'div#site-navi ul li#tab-'
                                . $id . ' a {' . "\x0A";
                            $site_navi_css .= '  width: 100px;' . "\x0A";
                            $site_navi_css .= '  max-width: 100px;' . "\x0A";
                            $site_navi_css
                                .= '  height: '
                                . $this->{'page'}{'navi'}{'size'} . 'px;'
                                . "\x0A";
                            $site_navi_css
                                .= '  max-height: '
                                . $this->{'page'}{'navi'}{'size'} . 'px;'
                                . "\x0A";
                            $site_navi_css .= '  overflow: hidden;' . "\x0A";
                            $site_navi_css .= '  font-size: 10px;' . "\x0A";
                            $site_navi_css .= '  line-height: 10px;' . "\x0A";
                            $site_navi_css .= '  text-indent: 0;' . "\x0A";
                            $site_navi_css .= '}' . "\x0A";
                            $site_navi_css
                                .= 'div#site-navi ul li#tab-'
                                . $id
                                . ' a span {' . "\x0A";
                            $site_navi_css .= '  display: inline;' . "\x0A";
                            $site_navi_css .= '}' . "\x0A";
                            $navi_width += 100;
                        }
                        $navi_width += $this->{'page'}{'navi'}{'space'}
                            if defined $this->{'page'}{'navi'}{'space'}
                            && $this->{'page'}{'navi'}{'space'};
                    }
                    print OUT '</ul></div>', "\x0A";
                    $this->print_accestat_image( \*OUT );
                    $site_navi_css =~ s/_W_/$navi_width/;
                    $site_navi_css .= $this->category_menu_css(
                        'site-navi',
                        $this->{'page'}{'navi'}{'menuhovercolor'},
                        $this->{'page'}{'navi'}{'menuactivecolor'}
                        )

                        #テーブルレイアウト
                }
                else {
                    $background = $this->path_change( $background,
                        $ENV{'SCRIPT_NAME'}, 'abs' );
                    print OUT
                        '<table id="site-navi-table-wrapper" cellpadding="0" cellspacing="0" border="0"';
                    print OUT ( ( $this->{'page'}{'navi'}{'iconbgwidth'} )
                        ? ' width="'
                            . $this->{'page'}{'navi'}{'iconbgwidth'} . '"'
                        : ' width="100%"' );
                    print OUT ' height="'
                        . $this->{'page'}{'navi'}{'size'} . '"'
                        if defined $this->{'page'}{'navi'}{'size'}
                        && $this->{'page'}{'navi'}{'size'};
                    print OUT ' bgcolor="'
                        . $this->{'page'}{'navi'}{'bgcolor'} . '"'
                        if defined $this->{'page'}{'navi'}{'bgcolor'}
                        && $this->{'page'}{'navi'}{'bgcolor'};
                    print OUT ' background="' . $background . '"'
                        if defined $background && $background;
                    print OUT ' align="'
                        . $this->{'page'}{'navi'}{'iconbgalign'} . '"'
                        if defined $this->{'page'}{'navi'}{'iconbgalign'}
                        && $this->{'page'}{'navi'}{'iconbgalign'}
                        && $this->{'page'}{'navi'}{'iconbgalign'}
                        =~ /^(left|center|right)$/;
                    print OUT '><tbody><tr><td';
                    print OUT ' align="'
                        . $this->{'page'}{'navi'}{'align'} . '"'
                        if defined $this->{'page'}{'navi'}{'align'}
                        && $this->{'page'}{'navi'}{'align'}
                        && $this->{'page'}{'navi'}{'align'}
                        =~ /^(left|center|right)$/;
                    print OUT '>';
                    $this->print_navi_table( \*OUT, $navi_path, 'abs' );
                    print OUT '</td></tr></tbody></table>';
                    $site_navi_css
                        .= $this->read_file( $env::css . '/navi-image.css',
                        1 );
                }
                $site_navi_css .= $this->{'page'}{'navi'}{'iconcss'}
                    if defined $this->{'page'}{'navi'}{'iconcss'}
                    && $this->{'page'}{'navi'}{'iconcss'};
                $this->write_file( $env::css . '/site-navi.css',
                    $this->gsearch_css . $site_navi_css, 1 );

                #CSSタブ
            }
            else {
                if ( $this->{'page'}{'navi'}{'tabtype'} eq 'image' ) {
                    $this->print_navi_image_tabs( \*OUT, $navi_path, 'abs' );
                }
                else {
                    $this->print_navi_tabs( \*OUT, $navi_path, 'abs' );
                }
            }
            print OUT $this->{'page'}{'navi'}{'navifooter'}
                if defined $this->{'page'}{'navi'}{'navifooter'}
                && $this->{'page'}{'navi'}{'navifooter'};
            close(OUT);
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                'ナビゲーションの書き出しに失敗しました。ファイルが開けませんでした。'
            );
        }
        $this->mk_external_javascript($navi_path);
        $this->mk_navi_by_other_jcode($navi_path);

        #フレーム方式
    }
    else {

        #フレーム書き出し
        if ( open( OUT, ">>$this->{'frame_path'}" ) ) {
            binmode OUT, ":utf8";
            flock( OUT, LOCK_EX );
            truncate( OUT, 0 );
            seek( OUT, 0, 0 );
            my ($favicon_path, $navi_path,   $top_path, $frame_ratio,
                $frame_scroll, $marginwidth, $marginheight
            );
            if ( -f '../../favicon.ico' ) {
                $favicon_path = '../../favicon.ico';
            }
            else {
                my $count = 0;
                while ( $ENV{'SCRIPT_NAME'} =~ m!\G/[^/]+!g ) {
                    $count++;
                    next if $count < 3;
                    my $path = ( '../' x $count ) . 'favicon.ico';
                    if ( -f $path ) {
                        $favicon_path = $path;
                        last;
                    }
                }
            }
            $navi_path = $this->base_change( $this->{'navi_path'},
                $this->{'frame_path'} );
            $top_path
                = ( defined $this->{'navi'}{'TOP'}
                    && ref $this->{'navi'}{ $this->{'navi'}{'TOP'} } )
                ? $this->get_default_uri( $this->{'navi'}{'TOP'},
                $this->{'navi'}{ $this->{'navi'}{'TOP'} } )
                : $this->{'top_path'};
            $top_path
                = $this->base_change( $top_path, $this->{'frame_path'} );
            $this->{'page'}{'navi'}{'size'} = 40
                if !defined $this->{'page'}{'navi'}{'size'}
                || !$this->{'page'}{'navi'}{'size'};
            $frame_ratio
                = (
                ( $this->{'page'}{'navi'}{'frametype'} =~ /^v/ ) ? ' cols="'
                : ' rows="' )
                . ( ( $this->{'page'}{'navi'}{'frametype'} =~ /[br]$/ )
                ? '*, ' . $this->{'page'}{'navi'}{'size'}
                : $this->{'page'}{'navi'}{'size'} . ', *' )
                . '"';
            $frame_scroll
                = ( $this->{'page'}{'navi'}{'scroll'} )
                ? $this->{'page'}{'navi'}{'scroll'}
                : 'auto';
            $marginwidth
                = ( $this->{'page'}{'navi'}{'marginwidth'} )
                ? $this->{'page'}{'navi'}{'marginwidth'}
                : '0';
            $marginheight
                = ( $this->{'page'}{'navi'}{'marginheight'} )
                ? $this->{'page'}{'navi'}{'marginheight'}
                : '0';
            print OUT
                '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN" "http://www.w3.org/TR/html4/frameset.dtd">',
                "\x0A";
            print OUT '<html>', "\x0A";
            print OUT '<head>', "\x0A";
            print OUT
                '<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />',
                "\x0A";
            print OUT '<title>'
                . $this->escape_except_html_entity(
                $this->{'page'}{'navi'}{'title'} )
                . '</title>', "\x0A";

            if ( defined $favicon_path && $favicon_path ne '' ) {
                print OUT '<link rel="shortcut icon" href="'
                    . $this->base_change( $favicon_path,
                    $this->{'frame_path'} )
                    . '" type="image/x-icon" />', "\x0A";
            }
            print OUT '</head>', "\x0A";
            print OUT '<frameset frameborder="no" border="0" framespacing="0"'
                . $frame_ratio
                . '>', "\x0A";
            if ( $this->{'page'}{'navi'}{'frametype'} =~ /[br]$/ ) {
                print OUT '<frame src="'
                    . $top_path
                    . '" name="'
                    . $env::home_dir_id
                    . 'main" />', "\x0A";
                print OUT '<frame src="'
                    . $navi_path
                    . '" name="'
                    . $env::home_dir_id
                    . 'navi" marginwidth="'
                    . $marginwidth
                    . '" marginheight="'
                    . $marginheight
                    . '" scrolling="'
                    . $frame_scroll
                    . '" frameborder="0" />', "\x0A";
            }
            else {
                print OUT '<frame src="'
                    . $navi_path
                    . '" name="'
                    . $env::home_dir_id
                    . 'navi" marginwidth="'
                    . $marginwidth
                    . '" marginheight="'
                    . $marginheight
                    . '" scrolling="'
                    . $frame_scroll
                    . '" frameborder="0" />', "\x0A";
                print OUT '<frame src="'
                    . $top_path
                    . '" name="'
                    . $env::home_dir_id
                    . 'main" />', "\x0A";
            }
            print OUT '</frameset>', "\x0A";
            print OUT '<noframes>';
            print OUT '<body bgcolor="#ffffff">';
            print OUT
                '<p>このページを見るにはフレームの表示ができるブラウザが必要です。</p>';
            print OUT '<p><a href="./navi.html">ナビ</a></p>';
            print OUT '</body>';
            print OUT '</noframes>', "\x0A";
            print OUT '</html>';
            close(OUT);
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                'フレームページの書き出しに失敗しました。ファイルが開けませんでした。'
            );
        }

        #ナビフレーム書き出し
        if ( open( OUT, ">>$this->{'navi_path'}" ) ) {
            binmode OUT, ":utf8";
            flock( OUT, LOCK_EX );
            truncate( OUT, 0 );
            seek( OUT, 0, 0 );
            if ( defined $this->{'page'}{'navi'}{'type'}
                && $this->{'page'}{'navi'}{'type'} eq 'template' )
            {
                $this->set_style_sheet_link(
                    $this->base_change(
                        $env::css . '/site-navi.css',
                        $this->{'navi_path'}
                    )
                );
                $this->print_page_header( 'navi', \*OUT,
                    $this->{'navi_path'}, 0, '<>' );
                $this->print_navi_from_template( \*OUT,
                    $this->{'navi_path'} );
            }
            else {
                if ( defined $this->{'page'}{'navi'}{'align'}
                    && $this->{'page'}{'navi'}{'align'} ne '' )
                {
                    $this->{'page'}{'navi'}{'textAlign'}
                        = $this->{'page'}{'navi'}{'align'};
                    $this->{'CSS'} .= 'table#site-navi-table {' . "\x0A";
                    $this->{'CSS'} .= '  margin-left: auto;' . "\x0A"
                        if $this->{'page'}{'navi'}{'align'} eq 'center'
                        || $this->{'page'}{'navi'}{'align'} eq 'right';
                    $this->{'CSS'} .= '  margin-right: auto;' . "\x0A"
                        if $this->{'page'}{'navi'}{'align'} eq 'center'
                        || $this->{'page'}{'navi'}{'align'} eq 'left';
                    $this->{'CSS'} .= '}' . "\x0A";
                }
                $this->print_page_header( 'navi', \*OUT,
                    $this->{'navi_path'}, 0, '<>' );
                $this->print_navi_table( \*OUT, $this->{'navi_path'} );
            }
            $this->print_page_footer( 'navi', \*OUT, $this->{'navi_path'} );
            close(OUT);
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                'ナビゲーションフレームの書き出しに失敗しました。ファイルが開けませんでした。'
            );
        }
        if ( $this->{'page'}{'navi'}{'type'} eq 'template' ) {
            if ( $this->{'page'}{'navi'}{'template'}
                =~ /<\!--sitenavi(?![0-9a-zA-Z_])[^>]*type="icon"[^>]*-->/s )
            {
                $this->file_copy_and_add(
                    $env::css . '/navi-image.css',
                    $env::css . '/site-navi.css',
                    $this->{'page'}{'navi'}{'navicss'}
                );
            }
            else {
                $this->write_file( $env::css . '/site-navi.css',
                    $this->{'page'}{'navi'}{'navicss'}, 1 );
            }
        }
        else {
            $this->file_copy_and_add(
                $env::css . '/navi-image.css',
                $env::css . '/site-navi.css'
            );
        }
        $this->write_file( $env::externalfiles . '/site-navi.ssi', '', 1 );
        $this->write_file( $env::externalfiles . '/site-navi.js',  '', 1 );
        $this->write_file( $env::externalfiles . '/site-navi-sjis.ssi',
            '', 1 );
        $this->write_file( $env::externalfiles . '/site-navi-sjis.js', '',
            1 );
        $this->write_file( $env::externalfiles . '/site-navi-utf8.ssi',
            '', 1 );
        $this->write_file( $env::externalfiles . '/site-navi-utf8.js', '',
            1 );
    }

    my $sitemap = $this->get_sitemap(
        $setting::page{'sitemap'}{'descriptionEnabled'} );
    if ( open( OUT, ">>$this->{'sitemap_path'}" ) ) {
        binmode OUT, ":utf8";
        flock( OUT, LOCK_EX );
        truncate( OUT, 0 );
        seek( OUT, 0, 0 );
        my $template = $setting::page{'sitemap'}{'template'}
            || '<!--sitemap--><!--/sitemap-->';
        $template
            =~ s/<\!--sitemap(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/sitemap(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('sitemap',$1,$2,$sitemap)/iges;
        $template = $this->page_item_replace(
            $template, 1,
            $this->{'sitemap_path'},
            $setting::page{'sitemap'}{'title'}
        );
        my $noheader
            = ( $template =~ /<(?:html|body)(?!\w)[^>]*>/is ) ? 1 : 0;
        $setting::page{'sitemap'}{'bodyId'}
            = 'document' . $this->{'cgi_id'} . '_sitemap';

        if ( !$noheader ) {
            $this->set_last_modified(time);
            $this->print_page_header( 'sitemap', \*OUT,
                $this->{'sitemap_path'},
                0, '<>' );
        }
        print OUT $template;
        $this->print_page_footer( 'sitemap', \*OUT, $this->{'sitemap_path'} )
            if !$noheader;
        close(OUT);
    }

    $sitemap = $this->get_sitemap
        if !$setting::page{'sitemap'}{'descriptionEnabled'};
    $this->write_file( $this->{'sitemap_ssi'}, $sitemap, 1 );
    $this->mk_external_javascript( $this->{'sitemap_ssi'}, '', $sitemap );

    delete $this->{'navi'};
    delete $this->{'page'};
    delete $this->{'icon'};
    delete $this->{'icon_setting'};
    return $this;
}

sub get_sitemap ($$) {
    my ( $this, $description_enabled ) = @_;
    my $sitemap = '';
    foreach ( sort { $a <=> $b } keys %{ $this->{'navi'}{'ORDER'} } ) {
        my $id = $this->{'navi'}{'ORDER'}{$_};
        if ( $this->{'navi'}{$id}{'sitemap'} ) {
            my ( $uri, $categories )
                = $this->get_default_uri( $id, $this->{'navi'}{$id},
                0, 1, $description_enabled );
            $sitemap .= '<li>'
                . $this->get_nav_link(
                $uri,
                $this->{'navi'}{$id}{'title'},
                $this->{'navi'}{$id}{'description'},
                $this->{'navi'}{$id}{'openWindow'},
                $description_enabled
                ) . $categories . '</li>';
        }
    }
    $sitemap = '<ul class="sitemap">' . $sitemap . '</ul>' if $sitemap;
    return $sitemap;
}

sub get_nav_link ($$$$;$$) {
    my ( $this, $url, $title, $description, $openwindow,
        $description_enabled, $is_tab )
        = @_;
    return '<a href="'
        . $this->escape_except_html_entity(
        $this->path_change( $url, $ENV{'SCRIPT_NAME'}, 'abs' ) )
        . '"'
        . ( ($openwindow) ? ' target="_blank"' : '' )
        . ( ( !$description_enabled && $description )
        ? ' title="' . $this->detag_text($description) . '"'
        : '' )
        . '>'
        . ( ($is_tab) ? '<span>' : '' )
        . $this->escape_except_html_entity($title)
        . ( ($is_tab) ? '</span>' : '' ) . '</a>'
        . ( ( $description_enabled && $description )
        ? '<div class="description">'
            . $this->deblock($description)
            . '</div>'
        : '' );
}

sub file_copy_and_add ($$$;$) {
    my ( $this, $file, $copy, $data ) = @_;
    my $check = 0;
    return 0 if $file eq '' || $copy eq '';
    return 1 if $file eq $copy;
    local ( *FILE, *COPY );
    if ( open( FILE, $file ) ) {
        binmode FILE, ":utf8";
        flock( FILE, LOCK_SH );
        if ( open( COPY, ">>$copy" ) ) {
            binmode COPY, ":utf8";
            flock( COPY, LOCK_EX );
            truncate( COPY, 0 );
            seek( COPY, 0, 0 );
            while (<FILE>) {
                print COPY;
            }
            print COPY $data if defined $data;
            close(COPY);
            $check = 1;
        }
        close(FILE);
        chmod( $env::file_mode, $copy );
    }
    return $check;
}

sub get_default_cgi_name ($;$$) {
    my ( $this, $id, $name ) = @_;
    if ($name) {
        return $name;
    }
    elsif ( $id =~ /([a-zA-Z0-9]+)$/ ) {
        return $1;
    }
    else {
        return $id;
    }
}

sub get_default_uri ($$$;$$$) {
    my ( $this, $id, $setting, $ignore_categories, $nobubble,
        $description_enabled )
        = @_;
    my ( $uri, $categories, $icon );
    $setting->{'uri'} = '../' . $id . '/start.cgi'
        if !defined $setting->{'uri'} || $setting->{'uri'} eq '';
    $uri = $setting->{'uri'};
    if ( defined $setting->{'lib'} && $setting->{'lib'} eq 'TableMaker' ) {
        $icon = 'db';
        if ( $id eq 'order' ) {
            $uri .= '?m=CartViewer&amp;iframe_disabled=1' if $uri !~ /\?/;

            #	} elsif ($id eq 'account') {
        }
        elsif ( $setting->{'default'} eq 'table' ) {
            $uri .= '?m=TableViewer';
        }
        elsif ( $setting->{'default'} eq 'tile' ) {
            $uri .= '?m=TileViewer';
        }
        elsif ( $setting->{'default'} eq 'gmap' ) {
            $uri .= '?m=GMapViewer';
        }
        elsif ( $setting->{'default'} eq 'dgate' ) {
            $uri .= '?m=DetailGate';
        }
        elsif ( $setting->{'default'} eq 'gate' ) {
            $uri .= '?m=Gate';
        }
        elsif ( $setting->{'default'} eq 'input' ) {
            $uri .= '?m=Input';
        }
        elsif ( $setting->{'default'} eq 'index' ) {
            $uri .= '?m=Index';
        }
    }
    elsif ( defined $setting->{'lib'}
        && $setting->{'lib'} eq 'HomePageMaker' )
    {
        $icon = 'page';
        $uri .= '/' if $uri !~ m!/$!;
        if ( !$setting->{'subdivided'} && $setting->{'default'} eq 'latest' )
        {
            $uri .= 'new/';
        }
        elsif ( !$setting->{'subdivided'}
            && $setting->{'default'} eq 'first' )
        {
            $uri .= 'first/';
        }
    }
    elsif ( defined $setting->{'lib'} && $setting->{'lib'} eq 'Baobab' ) {
        $uri .= '?sort=' . $setting->{'default'}
            if $setting->{'subdivided'} && $setting->{'default'} ne '';
        $icon = 'bbs';
    }
    elsif ( defined $setting->{'lib'}
        && $setting->{'lib'} eq 'HomePageMakerLite' )
    {
        $icon = 'page';
    }
    elsif ( $id =~ /^URI[0-9]+$/ ) {
        $icon = $setting->{'mode'};
    }
    else {
        $icon = 'equipment';
    }
    $categories = '';
    if (   $env::inline_navi
        && wantarray
        && !$ignore_categories
        && $setting->{'menu'}
        && $setting->{'categories'} )
    {
        if ( defined $setting->{'lib'} && $setting->{'lib'} eq 'TableMaker' )
        {
            if ( $setting->{'categories'} ) {
                if ($nobubble) {
                    $categories = $setting->{'categories'};
                }
                else {
                    $categories
                        = '<div class="menu"><div class="bubble">'
                        . $setting->{'categories'}
                        . '</div></div>';
                }
                my $url = $this->escape_except_html_entity(
                    $this->path_change( $uri, $ENV{'SCRIPT_NAME'}, 'abs' ) );
                if ( $url !~ /\?/ ) {
                    $categories =~ s/<a href="&amp;/<a href="$url?/g;
                }
                else {
                    $categories =~ s/<a href="&/<a href="$url&/g;
                }
            }
        }
        else {
            while (
                $setting->{'categories'} =~ /^(.*?)\x09(.*?)\x09(.*?)\x0A/mg )
            {
                my ( $u, $n, $d ) = ( $1, $2, $3 );
                if (  !$setting->{'lib'}
                    || $setting->{'lib'} ne 'HomePageMaker' )
                {
                    my $url = $uri;
                    $url .= '?c=' . $u unless $url =~ s/\?/\?c=$u&/;
                    $u = $url;
                }
                $categories .= '<li>'
                    . $this->get_nav_link( $u, $n, $d,
                    $setting->{'openWindow'},
                    $description_enabled )
                    . '</li>';
            }
            if ($categories) {
                if ($nobubble) {
                    $categories = '<ul>' . $categories . '</ul>';
                }
                else {
                    $categories
                        = '<div class="menu"><div class="bubble"><ul>'
                        . $categories
                        . '</ul></div></div>';
                }
            }
        }
    }
    return wantarray ? ( $uri, $categories, $icon ) : $uri;
}

sub deblock ($$) {
    my ( $this, $html ) = @_;
    $html
        =~ s/<(embed|object|param|video|audio|script)(?!\w)[^*>]*>.*?<\1(?!\w)[^*>]*>//igs;
    $html =~ s/<\/?(?!a(?!\w))\w+[^*>]*>//igs;
    return $html;
}

sub despace ($$) {
    my ( $this, $html ) = @_;
    $html =~ s/[\x0A\x0D\x09]+//gs;
    return $html;
}

sub import_order ($$$) {
    no warnings qw(untie);
    my ( $this, $db, $new_order ) = @_;
    my (%db);
    tie( %db, $env::dbm_file, $db, O_RDONLY, $env::dbm_mode );
    if ( ref tied %db ) {
        my %order;
        while ( my ( $key, $value ) = each %db ) {
            utf8::decode($key)   if !utf8::is_utf8($key);
            utf8::decode($value) if !utf8::is_utf8($value);
            if ( $key =~ /^ORDER_(.*)$/ ) {
                $order{$value} = $1;
                $new_order->{$value} = $1
                    if ref $new_order
                    && ( !defined $new_order->{$value}
                    || $new_order->{$value} eq '' );
            }
        }
        untie %db;
        return %order;
    }
    else {
        untie %db;
        my ($dbname);
        $dbname = '\n' . $1 if $db ne $env::register_db && $db =~ m!([^/]+)$!;
        push(
            @{ $cgi::action->{'dialog'} },
            'データベース読み込みに失敗しました。'
                . $dbname
        );
        return ();
    }
}

sub get_tm_categories ($$) {
    my ( $this, $preference_dir ) = @_;
    my %category;
    $this->import_db( $preference_dir . '/category', \%category, 1 );
    my @list = sort {
               $category{$a}{'ORDER'} <=> $category{$b}{'ORDER'}
            || $a <=> $b
    } grep( defined $_ && $_ !~ /[^0-9]/, keys %category );
    return $this->tm_category_list( \%category, \@list, '' );
}

sub tm_category_list ($$$$) {
    my ( $this, $category, $category_list, $key ) = @_;
    $key = '' if !defined $key;
    my ( $sub_key, @sub_list, $list, $c );
    $sub_key = $list = '';
    foreach $c (@$category_list) {
        next if !defined $c;
        $sub_key = $1
            if $sub_key eq ''
            && $category->{$c}->{'name'}
            && $category->{$c}->{'name'} =~ m!^($key[^/]+/?)!;
        if (   $sub_key ne ''
            && $sub_key ne $key
            && $category->{$c}->{'name'} =~ /^$sub_key/
            && ( $sub_key =~ m!/$! || $category->{$c}->{'name'} eq $sub_key )
            )
        {
            push( @sub_list, $c );
        }
        else {
            if ( $sub_key ne '' && $sub_key ne $key ) {
                $list .= '<li>'
                    . $this->_tm_category_list( $category, \@sub_list,
                    $sub_key );
                $list
                    .= $this->tm_category_list( $category, \@sub_list,
                    $sub_key )
                    if @sub_list && $key ne $sub_key;
                $list .= '</li>';
            }
            $sub_key = $1
                if $category->{$c}->{'name'}
                && $category->{$c}->{'name'} =~ m!^($key[^/]+/?)!;
            undef @sub_list;
            push( @sub_list, $c );
        }
    }
    if ( $sub_key ne '' && $sub_key ne $key ) {
        $list .= '<li>'
            . $this->_tm_category_list( $category, \@sub_list, $sub_key );
        $list .= $this->tm_category_list( $category, \@sub_list, $sub_key )
            if @sub_list && $key ne $sub_key;
        $list .= '</li>';
    }
    return ($list) ? '<ul>' . $list . '</ul>' : '';
}

sub _tm_category_list ($$$$) {
    my ( $this, $category, $sub_list, $sub_key ) = @_;
    no warnings qw(numeric);
    my $c = join( ',', sort { $a <=> $b || $a cmp $b } @$sub_list );
    return
          '<a href="&amp;c='
        . $c . '">'
        . ( ( $sub_key =~ m!([^/]+)/?$! ) ? $1 : '' ) . '</a>';
}

sub get_categories ($$$$$) {
    my ( $this, $lib, $preference_dir, $id, $uri ) = @_;
    no warnings qw(numeric);
    my @folder = grep( $_ =~ /^[0-9a-zA-Z_]+$/ && -d "$preference_dir/$_",
        readdir(DIR) );
    closedir(DIR);
    my %category;
    my $categories = '';
    foreach my $c (@folder) {
        my (%setting);
        tie( %setting, $env::dbm_file,
            $preference_dir . '/' . $c . '/general',
            O_RDONLY, $env::dbm_mode );
        $category{$c} = (
            (          ref tied %setting
                    && defined $setting{'cginame'}
                    && $setting{'cginame'} ne ''
            )
            ? $this->despace( $this->decode_utf8( $setting{'cginame'} ) )
            : $c
            )
            . "\x09"
            . (
            (          ref tied %setting
                    && defined $setting{'description'}
                    && $setting{'description'} ne ''
            ) ? $this->despace(
                $this->decode_utf8( $setting{'description'} )
            ) : ''
            );
        untie %setting;
    }
    if ( $lib eq 'HomePageMaker'
        && -f $uri
        . '/index.'
        . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' ) )
    {
        my $index_html = $this->read_file(
            $uri
                . '/index.'
                . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' ),
            1
        );
        my $my_index_id = ( defined $id && $id ne '' ) ? $id : 'homepage';
        my %order;
        if ( $index_html
            =~ /<!--index(?![0-9a-zA-Z_])[^>]*? id="$my_index_id"[^>]*-->(.*?)<!--\/index-->/s
            )
        {
            my $my_index_html = $1;
            my $no            = 0;
            while ( $my_index_html
                =~ /<!--subindex(?![0-9a-zA-Z_])([^>]*)-->.*?<!--\/subindex-->/gs
                )
            {
                my $subindex_setting = $1;
                $order{$1} = ++$no
                    if $subindex_setting
                    =~ /[\x09\x0A\x0C\x0D\x20]*id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s;
            }
        }
        @folder = sort {
            (          defined $order{$a}
                    && defined $order{$b}
                    && $order{$a} <=> $order{$b} )
                || $category{$a} <=> $category{$b}
                || $category{$a} cmp $category{$b}
                || $a <=> $b
                || $a cmp $b
        } @folder;
    }
    else {
        @folder = sort {
                   $category{$a} <=> $category{$b}
                || $category{$a} cmp $category{$b}
                || $a <=> $b
                || $a cmp $b
        } @folder;
    }
    foreach my $c (@folder) {
        $categories
            .= ( ( $lib eq 'HomePageMaker' )
            ? $uri . '/'
                . $c
                . '/index.'
                . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' )
            : $c )
            . "\x09"
            . $category{$c} . "\x0A";
    }
    return $categories;
}

sub gsearch_tag ($) {
    my $this = shift;
    my $tag  = '';
    my $site = $this->full_path('../../');
    $site =~ s!^https?://!!i;
    $tag
        .= '<script language="JavaScript" type="text/javascript" src="'
        . $this->path_change( $env::javascript, $ENV{'SCRIPT_NAME'}, 'abs' )
        . '/gsearch.js" charset="UTF-8"></script>' . "\x0A";
    $tag .= '<script language="JavaScript" type="text/javascript">' . "\x0A";
    $tag .= '//<![CDATA[' . "\x0A";
    $tag .= 'cmonos.tool.gsearch.set({' . "\x0A";
    $tag .= '	\'cx\' : \'' . $env::google_custom_search_cx . '\',' . "\x0A"
        if $env::google_custom_search_cx;
    $tag
        .= '	\'title\' : \''
        . $this->escape_except_html_entity(
        $this->{'page'}{'navi'}{'gsearchtitle'} )
        . '\',' . "\x0A"
        if $this->{'page'}{'navi'}{'gsearchtitle'};
    $tag
        .= '	\'label\' : \''
        . $this->escape_except_html_entity(
        $this->{'page'}{'navi'}{'gsearchlabel'} )
        . '\',' . "\x0A"
        if $this->{'page'}{'navi'}{'gsearchlabel'};
    $tag .= '	\'site\' : \'' . $site . '\'' . "\x0A";
    $tag .= '});' . "\x0A";
    $tag .= '//]]>' . "\x0A";
    $tag .= '</script>' . "\x0A";
    return $tag;
}

sub gsearch_css ($) {
    my $this = shift;
    return '' if !$this->{'page'}{'navi'}{'gsearchenabled'};
    my $css = '';
    $css .= 'div#gsearch-box div#gsearch-form form#gsearch-legacy {' . "\x0A";
    $css .= '  margin: 0;' . "\x0A";
    $css .= '  font-size: 13px;' . "\x0A";
    $css .= '  text-align: center;' . "\x0A";
    $css .= '}' . "\x0A";
    $css .= 'div#gsearch-box div#gsearch-form form#gsearch-legacy img {'
        . "\x0A";
    $css .= '  border: none;' . "\x0A";
    $css .= '  vertical-align: middle;' . "\x0A";
    $css .= '}' . "\x0A";
    $css
        .= 'div#gsearch-box div#gsearch-form form#gsearch-legacy input.gsc-input {'
        . "\x0A";
    $css .= '  width: 120px;' . "\x0A";
    $css .= '}' . "\x0A";
    $css .= 'div#gsearch-box {' . "\x0A";
    $css
        .= '  position: '
        . (
        ( $this->{'page'}{'navi'}{'gsearchfixed'} ) ? 'fixed' : 'absolute' )
        . ';' . "\x0A";
    $css .= '  _position: absolute !important;' . "\x0A"
        if $this->{'page'}{'navi'}{'gsearchfixed'};
    $css .= '  z-index: 888888;' . "\x0A";
    $css .= '  top: 0;' . "\x0A";
    $css .= '  '
        . (
        (          $this->{'page'}{'navi'}{'gsearchalign'}
                && $this->{'page'}{'navi'}{'gsearchalign'} eq 'right'
        ) ? 'right' : 'left'
        ) . ': 0;' . "\x0A";
    $css .= '}' . "\x0A";
    $css .= '.gssb_c {' . "\x0A";
    $css .= '  z-index: 888889 !important;' . "\x0A";
    $css .= '  position: fixed !important;' . "\x0A"
        if $this->{'page'}{'navi'}{'gsearchfixed'};
    $css .= '}' . "\x0A";

    if ( $this->{'page'}{'navi'}{'gsearchbutton'} ) {
        my %image = $this->get_gsearch_button_settings(
            $this->{'page'}{'navi'}{'gsearchbutton'} );
        my $image_dir
            = $this->base_change( $env::image_dir,
            $env::css . '/site-navi.css' )
            . '/navi/search/'
            . $this->{'page'}{'navi'}{'gsearchbutton'};
        $css .= 'div#gsearch-box span#gsearch-title {' . "\x0A";
        $css .= '  display: none;' . "\x0A";
        $css .= '}' . "\x0A";
        $css .= 'div#gsearch-box {' . "\x0A";
        $css .= '  cursor: pointer;' . "\x0A";
        $css .= '  width: ' . $image{'linkwidth'} . 'px;' . "\x0A";
        $css .= '  height: ' . $image{'linkheight'} . 'px;' . "\x0A";
        $css
            .= '  background: transparent url("'
            . $image_dir . '/'
            . $image{'link'}
            . '") no-repeat top left;' . "\x0A";
        $css .= '}' . "\x0A";

        if ( $image{'hover'} ) {
            $css .= 'div#gsearch-box:hover {' . "\x0A";
            $css .= '  width: ' . $image{'hoverwidth'} . 'px;' . "\x0A";
            $css .= '  height: ' . $image{'hoverheight'} . 'px;' . "\x0A";
            $css
                .= '  background: transparent url("'
                . $image_dir . '/'
                . $image{'hover'}
                . '") no-repeat top left;' . "\x0A";
            $css .= '}' . "\x0A";
        }
    }
    else {
        $css .= 'div#gsearch-box {' . "\x0A";
        $css .= '  display: block;' . "\x0A";
        $css .= '  margin: 0;' . "\x0A";
        $css .= '  width: 80px;' . "\x0A";
        $css .= '  height: 13px;' . "\x0A";
        $css .= '  text-align: center;' . "\x0A";
        $css .= '  color: #FFF;' . "\x0A";
        $css .= '  font-size: 10px;' . "\x0A";
        $css .= '  border: 1px #000 solid;' . "\x0A";
        $css .= '  background-color: #444;' . "\x0A";
        $css .= '}' . "\x0A";
    }
    $css .= 'div#gsearch-box table, div#gsearch-box td {' . "\x0A";
    $css .= '  margin: 0;' . "\x0A";
    $css .= '  background: transparent none;' . "\x0A";
    $css .= '  border: none;' . "\x0A";
    $css .= '}' . "\x0A";
    $css .= 'div#gsearch-box div#gsearch-form {' . "\x0A";
    $css
        .= '  position: '
        . (
        ( $this->{'page'}{'navi'}{'gsearchfixed'} ) ? 'fixed' : 'absolute' )
        . ';' . "\x0A";
    $css .= '  _position: absolute !important;' . "\x0A"
        if $this->{'page'}{'navi'}{'gsearchfixed'};
    $css .= '  margin: 0;' . "\x0A";
    $css .= '  padding: 5px;' . "\x0A";
    $css .= '  width: 480px;' . "\x0A";
    $css .= '  top: 0;' . "\x0A";
    $css .= '  text-align: left;' . "\x0A";
    $css .= '  background: #fff none;' . "\x0A";
    $css .= '  border-bottom: 1px #ddd solid;' . "\x0A";

    if (   $this->{'page'}{'navi'}{'gsearchalign'}
        && $this->{'page'}{'navi'}{'gsearchalign'} eq 'right' )
    {
        $css .= '  right: -500px;' . "\x0A";
        $css .= '  _right: 0;' . "\x0A";
        $css .= '  _top: -300px;' . "\x0A";
        $css .= '  border-left: 1px #ddd solid;' . "\x0A";
        $css .= '  -moz-border-bottom-left-radius: 5px;' . "\x0A";
        $css .= '  -webkit-border-bottom-left-radius: 5px;' . "\x0A";
        $css .= '  -o-border-bottom-left-radius: 5px;' . "\x0A";
        $css .= '  -ms-border-bottom-left-radius: 5px;' . "\x0A";
        $css .= '  border-bottom-left-radius: 5px;' . "\x0A";
    }
    else {
        $css .= '  left: -500px;' . "\x0A";
        $css .= '  border-right: 1px #ddd solid;' . "\x0A";
        $css .= '  -moz-border-bottom-right-radius: 5px;' . "\x0A";
        $css .= '  -webkit-border-bottom-right-radius: 5px;' . "\x0A";
        $css .= '  -o-border-bottom-right-radius: 5px;' . "\x0A";
        $css .= '  -ms-border-bottom-right-radius: 5px;' . "\x0A";
        $css .= '  border-bottom-right-radius: 5px;' . "\x0A";
    }
    $css .= '  -moz-box-shadow: 0 8px 8px rgba(0,0,0,0.5);' . "\x0A";
    $css .= '  -webkit-box-shadow: 0 8px 8px rgba(0,0,0,0.5);' . "\x0A";
    $css .= '  -o-box-shadow: 0 8px 8px rgba(0,0,0,0.5);' . "\x0A";
    $css .= '  -ms-box-shadow: 0 8px 8px rgba(0,0,0,0.5);' . "\x0A";
    $css .= '  box-shadow: 0 8px 8px rgba(0,0,0,0.5);' . "\x0A";
    $css .= '  -moz-transition-property:  left, right;' . "\x0A";
    $css .= '  -moz-transition-duration: 0.5s;' . "\x0A";
    $css .= '  -webkit-transition-property: left, right;' . "\x0A";
    $css .= '  -webkit-transition-duration: 0.5s;' . "\x0A";
    $css .= '  -o-transition-property: left, right;' . "\x0A";
    $css .= '  -o-transition-duration: 0.5s;' . "\x0A";
    $css .= '  transition-property: left, right;' . "\x0A";
    $css .= '  transition-duration: 0.5s;' . "\x0A";
    $css .= '}' . "\x0A";
    $css .= 'div.hover div#gsearch-form,' . "\x0A";
    $css .= 'div.focused div#gsearch-form,' . "\x0A";
    $css .= 'div#gsearch-box:hover div#gsearch-form {' . "\x0A";

    if (   $this->{'page'}{'navi'}{'gsearchalign'}
        && $this->{'page'}{'navi'}{'gsearchalign'} eq 'right' )
    {
        $css .= '  right: 0 !important;' . "\x0A";
        $css .= '  _top: 0 !important;' . "\x0A";
    }
    else {
        $css .= '  left: 0 !important;' . "\x0A";
    }
    $css .= '}' . "\x0A";
    return $css;
}

sub get_gsearch_button_settings ($$) {
    my ( $this, $id ) = @_;
    my %data;
    my $dir = $env::image_dir . '/navi/search/' . $id;
    if ( -f "$dir/link.gif" ) {
        $data{'link'} = 'link.gif';
    }
    elsif ( -f "$dir/link.jpg" ) {
        $data{'link'} = 'link.jpg';
    }
    elsif ( -f "$dir/link.png" ) {
        $data{'link'} = 'link.png';
    }
    else {
        return;
    }
    ( $data{'linkwidth'}, $data{'linkheight'} )
        = $cgi::Image->info("$dir/$data{'link'}")
        if $data{'link'};
    if ( -f "$dir/visited.gif" ) {
        $data{'visited'} = 'visited.gif';
    }
    elsif ( -f "$dir/visited.jpg" ) {
        $data{'visited'} = 'visited.jpg';
    }
    elsif ( -f "$dir/visited.png" ) {
        $data{'visited'} = 'visited.png';
    }
    ( $data{'visitedwidth'}, $data{'visitedheight'} )
        = $cgi::Image->info("$dir/$data{'visited'}")
        if $data{'visited'};
    if ( -f "$dir/hover.gif" ) {
        $data{'hover'} = 'hover.gif';
    }
    elsif ( -f "$dir/hover.jpg" ) {
        $data{'hover'} = 'hover.jpg';
    }
    elsif ( -f "$dir/hover.png" ) {
        $data{'hover'} = 'hover.png';
    }
    ( $data{'hoverwidth'}, $data{'hoverheight'} )
        = $cgi::Image->info("$dir/$data{'hover'}")
        if $data{'hover'};
    if ( -f "$dir/active.gif" ) {
        $data{'active'} = 'active.gif';
    }
    elsif ( -f "$dir/active.jpg" ) {
        $data{'active'} = 'active.jpg';
    }
    elsif ( -f "$dir/active.png" ) {
        $data{'active'} = 'active.png';
    }
    ( $data{'activewidth'}, $data{'activeheight'} )
        = $cgi::Image->info("$dir/$data{'active'}")
        if $data{'active'};
    return %data;
}

sub mk_navi_by_other_jcode ($$) {
    my ( $this, $navi_path ) = @_;
    my $navi_name = $navi_path;
    $navi_name =~ s/\.ssi$//;
    if ( !ref $cgi::Jcode ) {
        use Jcode;
        $Jcode::FALLBACK = &Jcode::FB_XMLCREF if defined $Jcode::FALLBACK;
        $cgi::Jcode = Jcode->new;
    }
    my $data = $this->read_file( $navi_path, 1 );
    utf8::encode($data) if utf8::is_utf8($data);
    $cgi::Jcode->set( $data, 'utf8' );
    $this->write_file( $navi_name . '-sjis.ssi', $cgi::Jcode->sjis, 1 );
    $this->write_file( $navi_name . '-euc.ssi',  $cgi::Jcode->euc,  1 );
    $data = $this->read_file( $navi_name . '.js', 1 );
    $cgi::Jcode->set( $data, 'utf8' );
    $this->write_file( $navi_name . '-sjis.js', $cgi::Jcode->sjis, 1 );
    $this->write_file( $navi_name . '-euc.js',  $cgi::Jcode->euc,  1 );
}

sub print_navi_from_template ($$$;$) {
    my ( $this, $handle, $output, $output_option ) = @_;
    $handle = \*STDOUT if !$handle;
    require CMONOS::CGI_Lib::Template;
    my $this_template = Template->new;
    $this->{'icon'}->set_output($output);
    my $template = $this_template->deco( $this->{'page'}{'navi'}{'template'},
        $output, $output_option );
    $template
        =~ s/<\!--sitenavi(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/sitenavi(?![0-9a-zA-Z_])[^>]*-->/$this->get_navi_object($1,$2,$output,$output_option)/ges;
    print $handle $template;
    $this->print_accestat_image( $handle,
        ( ( $output_option && $output_option eq 'abs' ) ? 0 : 1 ) )
        if defined $this->{'page'}{'navi'}{'accestat'}
        && $this->{'page'}{'navi'}{'accestat'};
    $this->{'icon'}->restore_output;
}

sub print_vicuna_header ($$$;$) {
    my ( $this, $handle, $output, $output_option ) = @_;
    my $vicuna_header_css   = '';
    my %current_css         = ();
    my $current_selector_li = '';
    my $current_selector_a  = '';
    if ( !$this->{'page'}{'navi'}{'currentdisabled'}
        && -d $env::css . '/vicuna' )
    {
        my $vcss;
        if ( -f $env::css . '/vicuna/module/mod_gNavi/mod_gNavi.css' ) {
            $vcss = $this->read_file(
                $env::css . '/vicuna/module/mod_gNavi/mod_gNavi.css', 1 );
        }
        elsif ( -f $env::css . '/vicuna/core.css' ) {
            $vcss = $this->read_file( $env::css . '/vicuna/core.css', 1 );
        }
        if ($vcss) {
            while ( $vcss
                =~ /([^{}]*li\.current(?![0-9a-zA-Z_])[^{}]*)(\{.*?\})/sg )
            {
                my $selector = $1;
                my $setting  = $2;
                $selector =~ s!/\*.*?\*/!!sg;
                $selector =~ s/^[\x09\x0A\x0C\x0D\x20]+//;
                $selector =~ s/[\x09\x0A\x0C\x0D\x20]+$//;
                $selector = join(
                    ",\x0A",
                    map( 'body#{BODY_ID} ' . $_,
                        grep( $_ =~ s/li\.current(?![0-9a-zA-Z_])/li#\{ID\}/,
                            split(
                                /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,
                                $selector
                            ) ) )
                );
                $current_css{$selector} = $setting;
            }
        }
    }
    $handle = \*STDOUT if !$handle;
    print $handle '<div id="header">';
    $this->print_accestat_image( $handle,
        ( ( $output_option && $output_option eq 'abs' ) ? 0 : 1 ) )
        if defined $this->{'page'}{'navi'}{'accestat'}
        && $this->{'page'}{'navi'}{'accestat'};
    print $handle "\x0A";
    print $handle '	<p class="siteName" id="siteName"><a href="'
        . $this->site_title_link
        . '" id="homeUrl">'
        . $env::title
        . '</a></p>', "\x0A"
        if defined $env::title && $env::title ne '';
    print $handle '	<p class="description">' . $env::description . '</p>',
        "\x0A"
        if defined $env::description && $env::description ne '';
    print $handle '	<ul id="globalNavi">', "\x0A";

    foreach ( sort { $a <=> $b } keys %{ $this->{'navi'}{'ORDER'} } ) {
        my $id = $this->{'navi'}{'ORDER'}{$_};
        if ( $this->{'navi'}{$id}{'use'} ) {
            my ( $url, $categories )
                = $this->get_default_uri( $id, $this->{'navi'}{$id} );
            print $handle '		<li id="header-'
                . $id . '">'
                . $this->get_nav_link(
                $url,
                $this->{'navi'}{$id}{'title'},
                $this->{'navi'}{$id}{'description'},
                $this->{'navi'}{$id}{'openWindow'}
                ) . $categories . '</li>';
            if (%current_css) {
                foreach my $selector ( keys %current_css ) {
                    my $s = $selector;
                    $selector =~ s/\{ID\}/header-$id/sg;
                    $selector
                        =~ s/\{BODY_ID\}/document$this->{'navi'}{$id}{'id'}/sg;
                    $current_css{$s}
                        = $selector
                        . ( ( $current_css{$s} =~ /^\{/ ) ? ' ' : ",\x0A" )
                        . $current_css{$s};
                }
            }
            $current_selector_li
                .= ( ($current_selector_li) ? ",\x0A" : '' )
                . 'body#document'
                . $this->{'navi'}{$id}{'id'}
                . ' div#header ul#globalNavi li#header-'
                . $id;
            $current_selector_a
                .= ( ($current_selector_a) ? ",\x0A" : '' )
                . 'body#document'
                . $this->{'navi'}{$id}{'id'}
                . ' div#header ul#globalNavi li#header-'
                . $id . ' a';
        }
    }
    print $handle '	</ul>', "\x0A";
    print $handle '</div>', "\x0A";
    if (%current_css) {
        foreach my $css ( values %current_css ) {
            $vicuna_header_css .= $css . "\x0A";
        }
    }
    my $vicuna_header_user_css
        = ( ( $this->{'page'}{'navi'}{'vicunacss'} )
        ? $this->{'page'}{'navi'}{'vicunacss'}
        : '' );
    $vicuna_header_user_css =~ s/\/\*CURRENT_LI\*\//$current_selector_li/gs;
    $vicuna_header_user_css =~ s/\/\*CURRENT_A\*\//$current_selector_a/gs;
    $this->write_file(
        $env::css . '/site-navi.css',
        '@charset "UTF-8";' . "\x0A"
            . $this->gsearch_css
            . $vicuna_header_css
            . $this->category_menu_css(
            'header',
            $this->{'page'}{'navi'}{'menuhovercolor'},
            $this->{'page'}{'navi'}{'menuactivecolor'}
            )
            . $vicuna_header_user_css,
        1
    );
}

sub get_navi_object ($$$) {
    my ( $this, $setting, $data, $output, $output_option ) = @_;
    my ( $navi_tab, $header, $footer, $type, $class );
    ( $header, $footer ) = $this->get_header_and_footer( $setting, $data );
    $type = ( $setting =~ / type="([0-9a-zA-Z_]+)"/ ) ? $1 : '';
    $class = ( $setting =~ / class="([^"]+)"/ ) ? $1 : '';
    foreach ( sort { $a <=> $b } keys %{ $this->{'navi'}{'ORDER'} } ) {
        my ( $id, $icon, $categories, $uri );
        $id = $this->{'navi'}{'ORDER'}{$_};
        next
            if !defined $this->{'navi'}{$id}{'use'}
            || !$this->{'navi'}{$id}{'use'};
        $navi_tab .= $header;
        ( $uri, $categories, $icon )
            = $this->get_default_uri( $id, $this->{'navi'}{$id}, 1 );
        if ( $type eq 'icon' ) {
            $navi_tab .= $this->{'icon'}->button(
                $icon,
                $this->{'navi'}{$id}{'icon'},
                $uri,
                $this->escape_except_html_entity(
                    $this->{'navi'}{$id}{'title'}
                    )
                    . (
                    ( $this->{'navi'}{$id}{'description'} ) ? ':'
                        . $this->detag_text(
                        $this->{'navi'}{$id}{'description'}
                        ) : ''
                    ),
                (     ( $this->{'navi'}{$id}{'openWindow'} ) ? '_blank'
                    : ($env::inline_navi)                    ? ''
                    :   $env::home_dir_id . 'main'
                ),
                '',
                $output_option
            ) . $categories;
        }
        elsif ( $type eq 'list' ) {
            $navi_tab
                .= '<li id="header-'
                . $id
                . '"><a href="'
                . $this->base_change( $uri, $output, $output_option ) . '"'
                . ( ( $class )
                ? ' class="'
                    . $this->detag_text( $class )
                    . '"'
                : '' )
                . (
                  ( $this->{'navi'}{$id}{'openWindow'} ) ? ' target="_blank"'
                : ($env::inline_navi)                    ? ''
                :   ' target="' . $env::home_dir_id . 'main"' )
                . ( ( $this->{'navi'}{$id}{'description'} )
                ? ' title="'
                    . $this->detag_text( $this->{'navi'}{$id}{'description'} )
                    . '"'
                : '' )
                . '>'
                . $this->{'navi'}{$id}{'title'} . '</a>'
                . $categories . '</li>';
        }
        else {
            $navi_tab
                .= '<a href="'
                . $this->base_change( $uri, $output, $output_option )
                . '" id="header-'
                . $id . '"'
                . ( ( $class )
                ? ' class="'
                    . $this->detag_text( $class )
                    . '"'
                : '' )
                . (
                  ( $this->{'navi'}{$id}{'openWindow'} ) ? ' target="_blank"'
                : ($env::inline_navi)                    ? ''
                :   ' target="' . $env::home_dir_id . 'main"' )
                . ( ( $this->{'navi'}{$id}{'description'} )
                ? ' title="'
                    . $this->detag_text( $this->{'navi'}{$id}{'description'} )
                    . '"'
                : '' )
                . '>'
                . $this->{'navi'}{$id}{'title'} . '</a>'
                . $categories;
        }
        $navi_tab .= $footer;
    }
    return $navi_tab;
}

sub print_navi_table ($$$;$) {
    my ( $this, $handle, $output, $output_option ) = @_;
    $handle = \*STDOUT if !$handle;
    my ( $v_flag, $valign, $cell_count, $spacer );
    $v_flag = ( !$env::inline_navi
            && $this->{'page'}{'navi'}{'frametype'} =~ /^v/ ) ? 1 : 0;
    $this->{'icon'}->set_output($output);
    $valign = ' valign="' . $this->{'page'}{'navi'}{'cellvalign'} . '"'
        if defined $this->{'page'}{'navi'}{'cellvalign'}
        && $this->{'page'}{'navi'}{'cellvalign'};
    if (   !defined $this->{'page'}{'navi'}{'space'}
        || !$this->{'page'}{'navi'}{'space'} )
    {
        $spacer = '';
    }
    elsif ($v_flag) {
        $spacer = '<tr>' if $v_flag;
        $spacer
            .= '<td><img src="'
            . $this->base_change( $env::spacer, $output, $output_option )
            . '" width="1" height="'
            . $this->{'page'}{'navi'}{'space'}
            . '" style="display: block" alt="" /></td>';
        $spacer .= '</tr>' if $v_flag;
    }
    else {
        $spacer = '<tr>' if $v_flag;
        $spacer
            .= '<td><img src="'
            . $this->base_change( $env::spacer, $output, $output_option )
            . '" width="'
            . $this->{'page'}{'navi'}{'space'}
            . '" height="1" style="display: block" alt="" /></td>';
        $spacer .= '</tr>' if $v_flag;
    }
    print $handle
        '<table border="0" cellpadding="0" cellspacing="0" id="site-navi-table"><tbody>';
    print $handle '<tr>' if !$v_flag;
    foreach ( sort { $a <=> $b } keys %{ $this->{'navi'}{'ORDER'} } ) {
        my ( $id, $icon, $categories, $uri );
        $id = $this->{'navi'}{'ORDER'}{$_};
        next
            if !defined $this->{'navi'}{$id}{'use'}
            || !$this->{'navi'}{$id}{'use'};
        print $handle $spacer if defined $cell_count && $cell_count;
        print $handle '<tr>' if $v_flag;
        print $handle '<td' . $valign . '>';
        ( $uri, $categories, $icon )
            = $this->get_default_uri( $id, $this->{'navi'}{$id}, 1 );
        print $handle $this->{'icon'}->button(
            $icon,
            $this->{'navi'}{$id}{'icon'},
            $uri,
            $this->{'navi'}{$id}{'title'},
            (     ( $this->{'navi'}{$id}{'openWindow'} ) ? '_blank'
                : ($env::inline_navi)                    ? ''
                :   $env::home_dir_id . 'main'
            ),
            '',
            $output_option
        );
        print $handle '</td>';
        print $handle '</tr>' if $v_flag;
        $cell_count++;
    }
    print $handle '</tr>' if !$v_flag;
    print $handle '</tbody></table>';
    if ( defined $this->{'page'}{'navi'}{'accestat'}
        && $this->{'page'}{'navi'}{'accestat'} )
    {
        $this->print_accestat_image( $handle,
            ( ( defined $output_option && $output_option eq 'abs' ) ? 0 : 1 )
        );
    }
    $this->{'icon'}->restore_output;
}

sub print_accestat_image ($$;$) {
    my ( $this, $handle, $for_frame ) = @_;
    $handle = \*STDOUT if !$handle;
    print $handle '<script type="text/javascript" language="JavaScript">',
        "\x0A";
    print $handle '<!-- <![CDATA[', "\x0A";
    if ($for_frame) {
        my $cgi_path = $this->base_change( '../', '../../' );
        print $handle 'cmonos.statimage = {',                         "\x0A";
        print $handle '	call : function () {',                        "\x0A";
        print $handle '		if (top.innerWidth) {',                      "\x0A";
        print $handle '			this.write();',                             "\x0A";
        print $handle '		} else {',                                   "\x0A";
        print $handle '			var wait_flag = false;',                    "\x0A";
        print $handle '			for (var i=0; i<top.frames.length; i++) {', "\x0A";
        print $handle
            '				if (top.frames[i].document && !top.frames[i].document.body) {',
            "\x0A";
        print $handle '					wait_flag = true;',    "\x0A";
        print $handle '				}',                     "\x0A";
        print $handle '			}',                      "\x0A";
        print $handle '			if (wait_flag) {',       "\x0A";
        print $handle '				var this_stat = this;', "\x0A";
        print $handle '				setTimeout(function () { this.call() },500);',
            "\x0A";
        print $handle '			} else {',               "\x0A";
        print $handle '				this.write();',         "\x0A";
        print $handle '			}',                      "\x0A";
        print $handle '		}',                       "\x0A";
        print $handle '	},',                       "\x0A";
        print $handle '	write : function () {',    "\x0A";
        print $handle '		var window_size = \'\';', "\x0A";
        print $handle '		if (top.innerWidth) {',   "\x0A";
        print $handle
            '			window_size += \'&innerWidth=\'+top.innerWidth+\'&innerHeight=\'+top.innerHeight;',
            "\x0A";
        print $handle '		} else {',                                   "\x0A";
        print $handle '			var inner_width = 0;',                      "\x0A";
        print $handle '			var inner_height = 0;',                     "\x0A";
        print $handle '			for (var i=0; i<top.frames.length; i++) {', "\x0A";
        print $handle
            '				if (top.frames[i].document && top.frames[i].document.body ) {',
            "\x0A";
        print $handle
            '					var client_width = (document.compatMode != null && document.compatMode != "BackCompat") ? top.frames[i].document.documentElement.clientWidth : top.frames[i].document.body.clientWidth;',
            "\x0A";
        print $handle
            '					if ((inner_width == 0) || (30 < Math.abs(inner_width - client_width))) {',
            "\x0A";
        print $handle '						inner_width += client_width;', "\x0A";
        print $handle '					}',                             "\x0A";
        print $handle
            '					var client_height = (document.compatMode != null && document.compatMode != "BackCompat") ? top.frames[i].document.documentElement.clientHeight : top.frames[i].document.body.clientHeight;',
            "\x0A";
        print $handle
            '					if ((inner_height == 0) || (30 < Math.abs(inner_height - client_height))) {',
            "\x0A";
        print $handle '						inner_height += client_height;', "\x0A";
        print $handle '					}',                               "\x0A";
        print $handle '				}',                                "\x0A";
        print $handle '			}',                                 "\x0A";
        print $handle
            '			window_size += \'&innerWidth=\'+inner_width+\'&innerHeight=\'+inner_height;',
            "\x0A";
        print $handle '		}',                                "\x0A";
        print $handle '	}',                                 "\x0A";
        print $handle '	var accestat_image = new Image();', "\x0A";
        print $handle '	accestat_image.src = \''
            . $cgi_path
            . '/accestat/stat.cgi?\'+\'referer=\'+escape(top.document.referrer)+window_size+\'&screenWidth=\'+screen.width+\'&screenHeight=\'+screen.height+\'&id=\'+(new Date()).getTime;',
            "\x0A";
        print $handle '}',                        "\x0A";
        print $handle 'cmonos.statimage.call();', "\x0A";
        print $handle '// ]]> -->',               "\x0A";
        print $handle '</script>';
        print $handle '<noscript>';
        print $handle '<img src="'
            . $cgi_path
            . '/accestat/stat.cgi?id=index" height="1" width="1" alt="" />';
        print $handle '</noscript>';
    }
    else {
        my $accestat_path = $this->path_change( '../accestat/stat.cgi',
            $ENV{'SCRIPT_NAME'}, 'abs' );
        print $handle '(new Image()).src = \''
            . $accestat_path
            . '?referer=\'+escape(top.document.referrer)';
        print $handle
            '+\'&innerWidth=\'+((top.innerWidth) ? top.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? document.documentElement.clientWidth : document.body.clientWidth))';
        print $handle
            '+\'&innerHeight=\'+((top.innerHeight) ? top.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? document.documentElement.clientHeight : document.body.clientHeight))';
        print $handle '+\'&screenWidth=\'+screen.width';
        print $handle '+\'&screenHeight=\'+screen.height';
        print $handle '+\'&id=\'+(new Date()).getTime;', "\x0A";
        print $handle '// ]]> -->',                      "\x0A";
        print $handle '</script>';
        print $handle '<noscript>';
        print $handle '<img src="'
            . $accestat_path
            . '?id=index" height="1" width="1" alt="" />';
        print $handle '</noscript>';
    }
}

sub site_title_link ($) {
    my $this = shift;
    return ( $this->{'page'}{'navi'}{'sitetitleurl'} )
        ? $this->escape_except_html_entity(
        $this->{'page'}{'navi'}{'sitetitleurl'} )
        : $this->home_directory(1) . '/';
}

sub print_navi_tabs ($) {
    my ( $this, $handle, $output, $output_option ) = @_;
    my ($sitetitlecolor,     $sitesubtitlecolor, $sitetitle_height,
        $fontsize,           $sitetitlesize,     $beltwidth,
        $tab_padding_right,  $tab_margin_left,   $tab_padding_top,
        $tab_padding_bottom, $tab_padding_left
    );
    local (*CSS);
    $handle = \*STDOUT if !$handle;
    unless ( open( CSS, ">>$env::css/site-navi.css" ) ) {
        push(
            @{ $cgi::action->{'dialog'} },
            'CSS の書き出しに失敗しました。ファイルが開けませんでした。'
        );
        return;
    }
    binmode CSS, ":utf8";
    flock( CSS, LOCK_EX );
    truncate( CSS, 0 );
    seek( CSS, 0, 0 );

    #ナビ共通（前）
    print $handle '<div id="site-navi"><ul>';
    if ((      $this->{'page'}{'navi'}{'sitetitle'}
            || $this->{'page'}{'navi'}{'logo'}
        )
        && defined $env::title
        && $env::title ne ''
        )
    {
        print $handle '<li id="site-info">';
        print $handle '<p id="site-title"><a href="'
            . $this->site_title_link . '">'
            . $env::title
            . '</a></p>';
        print $handle '<p id="site-subtitle">' . $env::description . '</p>'
            if $env::description;
        print $handle '</li>';
    }

    #CSS共通（前）
    $sitetitlecolor
        = ( $this->{'page'}{'navi'}{'sitetitlecolor'} )
        ? $this->{'page'}{'navi'}{'sitetitlecolor'}
        : '#000000';
    $sitesubtitlecolor
        = ( $this->{'page'}{'navi'}{'sitesubtitlecolor'} )
        ? $this->{'page'}{'navi'}{'sitesubtitlecolor'}
        : $sitetitlecolor;
    $fontsize
        = ( $this->{'page'}{'navi'}{'fontsize'} )
        ? $this->{'page'}{'navi'}{'fontsize'}
        : 100;
    $sitetitlesize
        = ( $this->{'page'}{'navi'}{'sitetitlesize'} )
        ? $this->{'page'}{'navi'}{'sitetitlesize'}
        : $fontsize;

    #ロゴ
    if ( $this->{'page'}{'navi'}{'logo'}
        && ref $this->{'icon_setting'}{'logo'}
        { $this->{'page'}{'navi'}{'logo'} }
        && $this->{'icon_setting'}{'logo'}{ $this->{'page'}{'navi'}{'logo'} }
        {'height'} )
    {
        $this->{'page'}{'navi'}{'tabmargintop'}
            = $this->{'icon_setting'}{'logo'}
            { $this->{'page'}{'navi'}{'logo'} }{'height'}
            if !$this->{'page'}{'navi'}{'tabmargintop'}
            || $this->{'page'}{'navi'}{'tabmargintop'}
            < $this->{'icon_setting'}{'logo'}
            { $this->{'page'}{'navi'}{'logo'} }{'height'};
    }
    else {
        delete $this->{'page'}{'navi'}{'logo'};
    }

    #サイトタイトルの大きさ
    if (   $this->{'page'}{'navi'}{'sitetitlesize'}
        || $this->{'page'}{'navi'}{'logo'}
        || (   $this->{'page'}{'navi'}{'sitetitle'}
            && $this->{'page'}{'navi'}{'sitetitle'} == 2 )
        )
    {
        $sitetitle_height = int(
            14 * 0.9 * (
                ( $this->{'page'}{'navi'}{'sitetitlesize'} )
                ? $this->{'page'}{'navi'}{'sitetitlesize'} / 100
                : 1
            ) + 0.9
            ) + (
            (          $this->{'page'}{'navi'}{'sitetitle'}
                    && $this->{'page'}{'navi'}{'sitetitle'} == 2
            ) ? 20 : 0
            );
        $this->{'page'}{'navi'}{'tabmargintop'} = $sitetitle_height + 20
            if !$this->{'page'}{'navi'}{'tabmargintop'}
            || $this->{'page'}{'navi'}{'tabmargintop'} < $sitetitle_height;
    }

    $this->{'page'}{'navi'}{'tabmargintop'} = 10
        if !$this->{'page'}{'navi'}{'tabmargintop'};
    $this->{'page'}{'navi'}{'titlewidth'} = 960
        if !$this->{'page'}{'navi'}{'titlewidth'};
    $tab_padding_top
        = ( defined $this->{'page'}{'navi'}{'tabpaddingtop'}
            && $this->{'page'}{'navi'}{'tabpaddingtop'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingtop'}
        : ( defined $this->{'page'}{'navi'}{'tabpaddingbottom'}
            && $this->{'page'}{'navi'}{'tabpaddingbottom'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingbottom'}
        : 3;
    $tab_padding_bottom
        = ( defined $this->{'page'}{'navi'}{'tabpaddingbottom'}
            && $this->{'page'}{'navi'}{'tabpaddingbottom'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingbottom'}
        : $tab_padding_top;
    $tab_padding_bottom
        = ( $tab_padding_bottom > 4 )
        ? ( $tab_padding_bottom - 4 ) . 'px'
        : 0;
    $tab_padding_left
        = ( defined $this->{'page'}{'navi'}{'tabpaddingleft'}
            && $this->{'page'}{'navi'}{'tabpaddingleft'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingleft'}
        : ( defined $this->{'page'}{'navi'}{'tabpaddingright'}
            && $this->{'page'}{'navi'}{'tabpaddingright'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingright'}
        : 10;
    $tab_padding_right
        = ( defined $this->{'page'}{'navi'}{'tabpaddingright'}
            && $this->{'page'}{'navi'}{'tabpaddingright'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingright'}
        : $tab_padding_left;
    $tab_margin_left
        = ( defined $this->{'page'}{'navi'}{'tabmarginleft'}
            && $this->{'page'}{'navi'}{'tabmarginleft'} ne '' )
        ? $this->{'page'}{'navi'}{'tabmarginleft'}
        . ( ( $this->{'page'}{'navi'}{'tabmarginleft'} ) ? 'px' : '' )
        : '0';

    $beltwidth
        = ( $this->{'page'}{'navi'}{'beltwidth'} )
        ? $this->{'page'}{'navi'}{'beltwidth'}
        : 10;
    print CSS '@charset "UTF-8";' . "\x0A";
    print CSS 'div#site-navi {' . "\x0A";
    print CSS '  width: '
        . (
        ( $this->{'page'}{'navi'}{'tabbgwidth'} )
        ? ( ( $this->{'page'}{'navi'}{'tabmarginleft'} )
            ?         ( $this->{'page'}{'navi'}{'tabbgwidth'}
                    - ( $this->{'page'}{'navi'}{'tabmarginleft'} * 2 ) )
            : $this->{'page'}{'navi'}{'tabbgwidth'}
            )
            . 'px;'
        : '100%'
        ) . ';' . "\x0A";
    print CSS '  background-color: '
        . ( ( $this->{'page'}{'navi'}{'tabbgcolor'} )
        ? $this->{'page'}{'navi'}{'tabbgcolor'}
        : 'transparent' )
        . ';' . "\x0A";

    if (   $this->{'page'}{'navi'}{'tabbgalign'}
        && $this->{'page'}{'navi'}{'tabbgalign'} eq 'center' )
    {
        print CSS '  margin: 0 auto 0 auto;' . "\x0A";
    }
    elsif ($this->{'page'}{'navi'}{'tabbgalign'}
        && $this->{'page'}{'navi'}{'tabbgalign'} eq 'right' )
    {
        print CSS '  margin: 0 0 0 auto;' . "\x0A";
    }
    else {
        print CSS '  margin: 0;' . "\x0A";
    }
    print CSS '  padding: 0;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul {' . "\x0A";
    print CSS '  position: relative;' . "\x0A";
    print CSS '  border-bottom: '
        . $beltwidth
        . 'px solid '
        . (
        (   defined $this->{'navi'}{'ORDER'}{1}
                && $this->{'navi'}{ $this->{'navi'}{'ORDER'}{1} }{'tabcolor'}
        )
        ? $this->{'navi'}{ $this->{'navi'}{'ORDER'}{1} }{'tabcolor'}
        : '#666666'
        ) . ';' . "\x0A";

    #print CSS '  overflow: hidden;' . "\x0A";
    print CSS '  margin: 0;' . "\x0A";
    print CSS '  padding: '
        . $this->{'page'}{'navi'}{'tabmargintop'} . 'px '
        . $tab_margin_left . ' 0 '
        . $tab_margin_left . ';' . "\x0A";
    print CSS
        '  font-family: Avenir, "Lucida Grande", "Helvetica Neue", Helvetica, Arial, Verdana, Roboto, "ヒラギノ角ゴ Pro W3", "Hiragino Kaku Gothic Pro", "游ゴシック", "Yu Gothic", "游ゴシック体", "YuGothic", "Meiryo UI", "メイリオ", Meiryo, "ＭＳ Ｐゴシック", "MS PGothic", sans-serif;'
        . "\x0A";
    print CSS '  font-size: ' . $fontsize . '%;' . "\x0A";
    print CSS '  font-weight: '
        . ( ( $this->{'page'}{'navi'}{'fontweight'} )
        ? $this->{'page'}{'navi'}{'fontweight'}
        : 'normal' )
        . ';' . "\x0A";
    print CSS '  text-align: '
        . ( ( $this->{'page'}{'navi'}{'colortabalign'} )
        ? $this->{'page'}{'navi'}{'colortabalign'}
        : 'center' )
        . ';' . "\x0A";
    print CSS '  white-space: nowrap;' . "\x0A";
    print CSS '  list-style-type: none;' . "\x0A";

    if ( defined $this->{'page'}{'navi'}{'colortabbg'}
        && $this->{'page'}{'navi'}{'colortabbg'} )
    {
        my ($background);
        if ( $this->{'page'}{'navi'}{'colortabbg'} ) {
            if ( $this->{'page'}{'navi'}{'colortabbg'} =~ /^[0-9a-zA-Z_]+$/i )
            {
                $background
                    = $this->image_file_whole_path( $env::image_dir
                        . '/background/'
                        . $this->{'page'}{'navi'}{'colortabbg'} );
            }
            elsif ( $this->{'page'}{'navi'}{'colortabbg'}
                =~ /\.(gif|jpg|jpeg|png)$/i )
            {
                $background = $this->{'page'}{'navi'}{'colortabbg'};
            }
        }
        print CSS '  background-image: url("'
            . $this->base_change( $background, $env::css . '/site-navi.css' )
            . '");' . "\x0A";
        print CSS
            "  background-position: bottom;\x0A  background-repeat: repeat-x;\x0A"
            if $this->{'page'}{'navi'}{'colortabbgrepeatx'};
    }
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul li, div#site-navi ul li a {' . "\x0A";
    print CSS '  display: -moz-inline-box;' . "\x0A";
    print CSS '  display: inline-block;' . "\x0A";
    print CSS '  vertical-align: bottom;' . "\x0A";
    print CSS '  text-align: center;' . "\x0A";
    print CSS '  /display: inline;' . "\x0A";
    print CSS '  /zoom: 1;' . "\x0A";
    print CSS '  _display: inline;' . "\x0A";
    print CSS '  _zoom: 1;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul li {' . "\x0A";
    print CSS '  line-height: 1;' . "\x0A";

    if ( defined $this->{'page'}{'navi'}{'tabspace'}
        && $this->{'page'}{'navi'}{'tabspace'} )
    {
        my $margin_left = int( $this->{'page'}{'navi'}{'tabspace'} / 2 );
        print CSS '  margin: 0 '
            . ( $margin_left + ( $this->{'page'}{'navi'}{'tabspace'} % 2 ) )
            . 'px 0 '
            . $margin_left . 'px;' . "\x0A";
    }
    else {
        print CSS '  margin: 0;' . "\x0A";
    }
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul li a {' . "\x0A";
    print CSS '  text-decoration: none;' . "\x0A";
    print CSS '  padding: '
        . $tab_padding_top . 'px '
        . $tab_padding_right . 'px '
        . $tab_padding_bottom . ' '
        . $tab_padding_left . 'px;' . "\x0A";
    print CSS '  -moz-border-radius-topright: 5px;' . "\x0A";
    print CSS '  -webkit-border-top-right-radius: 5px;' . "\x0A";
    print CSS '  -o-border-top-right-radius: 5px;' . "\x0A";
    print CSS '  -ms-border-top-right-radius: 5px;' . "\x0A";
    print CSS '  border-top-right-radius: 5px;' . "\x0A";
    print CSS '  -moz-border-radius-topleft: 5px;' . "\x0A";
    print CSS '  -webkit-border-top-left-radius:5px;' . "\x0A";
    print CSS '  -o-border-top-left-radius:5px;' . "\x0A";
    print CSS '  -ms-border-top-left-radius:5px;' . "\x0A";
    print CSS '  border-top-left-radius:5px;' . "\x0A";
    print CSS '  -moz-box-shadow: 0px -2px 8px rgba(0,0,0,0.3);' . "\x0A";
    print CSS '  -webkit-box-shadow: 0px -2px 4px rgba(0,0,0,0.3);' . "\x0A";
    print CSS '  -o-box-shadow: 0px -2px 4px rgba(0,0,0,0.3);' . "\x0A";
    print CSS '  -ms-box-shadow: 0px -2px 4px rgba(0,0,0,0.3);' . "\x0A";
    print CSS '  box-shadow: 0px -2px 4px rgba(0,0,0,0.3);' . "\x0A";
    print CSS '  outline: none;' . "\x0A";
    print CSS '}' . "\x0A";

    if ($sitetitle_height) {
        my $titlemargin = int(
            (         $this->{'page'}{'navi'}{'tabmargintop'}
                    + $tab_padding_top
                    - $sitetitle_height
            ) / 2
        );
        my $padding_left = 0;
        print CSS 'div#site-navi ul li#site-info {' . "\x0A";
        print CSS '  display: block;' . "\x0A";
        print CSS '  position: absolute;' . "\x0A";
        print CSS '  top: 0;' . "\x0A";
        print CSS '  left: 0;' . "\x0A";
        print CSS '  margin: 0 auto;' . "\x0A";
        print CSS '  padding: 0;' . "\x0A";
        print CSS '  width: 100%;' . "\x0A";
        print CSS '  height: '
            . $this->{'page'}{'navi'}{'tabmargintop'} . 'px;' . "\x0A";
        print CSS '  text-align: '
            . ( $this->{'page'}{'navi'}{'colortabalign'} || 'center' ) . ';'
            . "\x0A";
        print CSS '  text-shadow: rgba(0,0,0,0.3) 0 2px 2px;' . "\x0A"
            if $this->{'page'}{'navi'}{'sitetitleshadow'};
        print CSS '  white-space: normal;' . "\x0A";
        print CSS '}' . "\x0A";
        print CSS 'div#site-navi ul li#site-info p {' . "\x0A";
        print CSS '  width: '
            . $this->{'page'}{'navi'}{'titlewidth'} . 'px;' . "\x0A";

        if (   $this->{'page'}{'navi'}{'colortabalign'}
            && $this->{'page'}{'navi'}{'colortabalign'} eq 'left' )
        {
            print CSS '  margin: 0;' . "\x0A";
        }
        elsif ($this->{'page'}{'navi'}{'colortabalign'}
            && $this->{'page'}{'navi'}{'colortabalign'} eq 'right' )
        {
            print CSS '  margin: 0 0 0 auto;' . "\x0A";
        }
        else {
            print CSS '  margin: 0 auto 0 auto;' . "\x0A";
        }
        print CSS '  padding: 0;' . "\x0A";
        print CSS '  line-height: 1;' . "\x0A";
        print CSS '  text-align: left;' . "\x0A";
        print CSS '}' . "\x0A";
        print CSS 'div#site-navi ul li#site-info p#site-title {' . "\x0A";
        print CSS '  position: relative;' . "\x0A";
        print CSS '}' . "\x0A";
        print CSS 'div#site-navi ul li#site-info p#site-title a {' . "\x0A";
        print CSS '  display: block;' . "\x0A";
        print CSS '  position: absolute;' . "\x0A";
        print CSS '  top: 0;' . "\x0A";
        print CSS '  left: 0;' . "\x0A";
        print CSS '  margin: 0;' . "\x0A";
        print CSS '  text-align: left;' . "\x0A";

        if ( $this->{'page'}{'navi'}{'sitetitle'} ) {
            $padding_left
                = $tab_padding_left
                + ( ( $this->{'page'}{'navi'}{'logo'} )
                ? $tab_padding_left
                    + $this->{'icon_setting'}{'logo'}
                    { $this->{'page'}{'navi'}{'logo'} }{'width'}
                : 0 );
            print CSS '  width: '
                . ( $this->{'page'}{'navi'}{'titlewidth'} - $padding_left )
                . 'px;' . "\x0A";
            print CSS '  height: '
                . ( $this->{'page'}{'navi'}{'tabmargintop'} - $titlemargin )
                . 'px;' . "\x0A";
            print CSS '  padding: '
                . $titlemargin
                . 'px 0 0 '
                . $padding_left . 'px;' . "\x0A";
            print CSS '  font-size: ' . $sitetitlesize . '%;' . "\x0A";
            print CSS '  color: ' . $sitetitlecolor . ';' . "\x0A";
            print CSS '  _width /**/: '
                . $this->{'page'}{'navi'}{'titlewidth'} . 'px;' . "\x0A";
            print CSS '  _height /**/: '
                . $this->{'page'}{'navi'}{'tabmargintop'} . 'px;' . "\x0A";
        }
        else {
            print CSS '  padding: 0;' . "\x0A";
            print CSS '  font-size: 1px;' . "\x0A";
            print CSS '  text-indent: -999px;' . "\x0A";
            print CSS '  overflow: hidden;' . "\x0A";
            print CSS '  width: '
                . $this->{'page'}{'navi'}{'titlewidth'} . 'px;' . "\x0A";
            print CSS '  height: '
                . $this->{'page'}{'navi'}{'tabmargintop'} . 'px;' . "\x0A";
        }
        if ( $this->{'page'}{'navi'}{'logo'} ) {
            my $top = int(
                (         $this->{'page'}{'navi'}{'tabmargintop'}
                        + $tab_padding_top
                        - $this->{'icon_setting'}{'logo'}
                        { $this->{'page'}{'navi'}{'logo'} }{'height'}
                ) / 2
            );
            $top
                = $this->{'page'}{'navi'}{'tabmargintop'}
                - $this->{'icon_setting'}{'logo'}
                { $this->{'page'}{'navi'}{'logo'} }{'height'}
                if $this->{'page'}{'navi'}{'tabmargintop'}
                < $this->{'icon_setting'}{'logo'}
                { $this->{'page'}{'navi'}{'logo'} }{'height'}
                + $top;
            print CSS '  background: transparent url("'
                . $this->base_change( $env::image_dir,
                $env::css . '/site-navi.css' )
                . '/navi/logo/'
                . $this->{'icon_setting'}{'logo'}
                { $this->{'page'}{'navi'}{'logo'} }{'name'}
                . '") no-repeat '
                . (
                ( $this->{'page'}{'navi'}{'sitetitle'} && $tab_padding_left )
                ? $tab_padding_left . 'px'
                : '0' )
                . ' '
                . ( ($top) ? $top . 'px' : '0' ) . ';' . "\x0A";
        }
        print CSS '  -moz-border-radius-topright: none;' . "\x0A";
        print CSS '  -webkit-border-top-right-radius: 0;' . "\x0A";
        print CSS '  -o-border-top-right-radius: 0;' . "\x0A";
        print CSS '  -ms-border-top-right-radius: 0;' . "\x0A";
        print CSS '  border-top-right-radius: 0;' . "\x0A";
        print CSS '  -moz-border-radius-topleft: none;' . "\x0A";
        print CSS '  -webkit-border-top-left-radius: 0;' . "\x0A";
        print CSS '  -o-border-top-left-radius: 0;' . "\x0A";
        print CSS '  -ms-border-top-left-radius: 0;' . "\x0A";
        print CSS '  border-top-left-radius: 0;' . "\x0A";
        print CSS '  -moz-box-shadow: none;' . "\x0A";
        print CSS '  -webkit-box-shadow: none;' . "\x0A";
        print CSS '  -o-box-shadow: none;' . "\x0A";
        print CSS '  -ms-box-shadow: none;' . "\x0A";
        print CSS '  box-shadow: none;' . "\x0A";
        print CSS '}' . "\x0A";

        if (  !$this->{'page'}{'navi'}{'sitetitle'}
            || $this->{'page'}{'navi'}{'sitetitle'} == 1 )
        {
            print CSS 'div#site-navi ul li#site-info p#site-subtitle {'
                . "\x0A";
            print CSS '  display: none;' . "\x0A";
            print CSS '}' . "\x0A";
        }
        else {
            print CSS 'div#site-navi ul li#site-info p#site-subtitle {'
                . "\x0A";
            print CSS '  margin-top:'
                . ( $titlemargin + $sitetitle_height - 13 ) . 'px;' . "\x0A";
            print CSS '  font-weight: normal;' . "\x0A";
            print CSS '  color: ' . $sitesubtitlecolor . ';' . "\x0A";
            print CSS '  padding: 0 0 0 ' . $padding_left . 'px;' . "\x0A";
            print CSS '  width: '
                . ( $this->{'page'}{'navi'}{'titlewidth'} - $padding_left )
                . 'px;' . "\x0A";
            print CSS '  _width /**/: '
                . $this->{'page'}{'navi'}{'titlewidth'} . 'px;' . "\x0A";
            print CSS '}' . "\x0A";
        }
    }

    foreach ( sort { $a <=> $b } keys %{ $this->{'navi'}{'ORDER'} } ) {
        my ( $id, $uri, $categories, $tabcolor, $shadow, $darkshadow );
        $id = $this->{'navi'}{'ORDER'}{$_};
        next
            if !defined $this->{'navi'}{$id}{'use'}
            || !$this->{'navi'}{$id}{'use'};
        ( $uri, $categories )
            = $this->get_default_uri( $id, $this->{'navi'}{$id} );
        $tabcolor
            = ( $this->{'navi'}{$id}{'tabcolor'} )
            ? $this->{'navi'}{$id}{'tabcolor'}
            : '#666666';
        ( $shadow, $darkshadow ) = $this->get_shadow_color($tabcolor);

        #ナビ
        print $handle '<li id="tab-'
            . $id . '">'
            . $this->get_nav_link(
            $uri,
            $this->{'navi'}{$id}{'title'},
            $this->{'navi'}{$id}{'description'},
            $this->{'navi'}{$id}{'openWindow'}
            ) . $categories . '</li>';

        #ナビCSS
        print CSS 'div#site-navi ul li#tab-' . $id . ' a {' . "\x0A";
        print CSS '  border-bottom: 4px solid ' . $darkshadow . ';' . "\x0A";
        print CSS '  background: ' . $shadow . ';' . "\x0A";
        print CSS '  color: '
            . ( ( $this->{'navi'}{$id}{'tabfontcolor'} )
            ? $this->{'navi'}{$id}{'tabfontcolor'}
            : '#ffffff' )
            . ';' . "\x0A";
        print CSS '  background: -moz-linear-gradient(top,'
            . $shadow . ','
            . $shadow . ' 80%,'
            . $darkshadow . ');' . "\x0A";
        print CSS
            '  background: -webkit-gradient(linear,left top,left bottom,from('
            . $shadow
            . '),color-stop(0.7, '
            . $shadow . '),to('
            . $darkshadow . '));' . "\x0A";
        print CSS '  _background: ' . $shadow . ';' . "\x0A";
        print CSS '}' . "\x0A";
        print CSS 'div#site-navi ul li#tab-'
            . $id
            . ':hover a, body#document'
            . $this->{'navi'}{$id}{'id'}
            . ' div#site-navi ul li#tab-'
            . $id . ' a {' . "\x0A";
        print CSS '  border-bottom: 4px solid ' . $tabcolor . ';' . "\x0A";
        print CSS '  background: ' . $tabcolor . ';' . "\x0A";
        print CSS '}' . "\x0A";
        print CSS 'body#document'
            . $this->{'navi'}{$id}{'id'}
            . ' div#site-navi ul {' . "\x0A";
        print CSS '  border-bottom: '
            . $beltwidth
            . 'px solid '
            . $tabcolor . ';' . "\x0A";
        print CSS '}' . "\x0A";
    }
    print CSS $this->gsearch_css;
    print CSS $this->category_menu_css(
        'site-navi',
        $this->{'page'}{'navi'}{'menuhovercolor'},
        $this->{'page'}{'navi'}{'menuactivecolor'}
    );
    print CSS $this->{'page'}{'navi'}{'colortabcss'}
        if defined $this->{'page'}{'navi'}{'colortabcss'}
        && $this->{'page'}{'navi'}{'colortabcss'};
    close(CSS);

    print $handle '</ul></div>', "\x0A";
    if ( defined $this->{'page'}{'navi'}{'accestat'}
        && $this->{'page'}{'navi'}{'accestat'} )
    {
        $this->print_accestat_image( $handle,
            ( ( $output_option eq 'abs' ) ? 0 : 1 ) );
    }
}

sub print_navi_image_tabs ($) {
    my ( $this, $handle, $output, $output_option ) = @_;
    my ($navi_image_dir,             $left_on_selector,
        $sitetitle_height,           $tabwidth,
        $right_on_selector,          $current_selector,
        $menu_selector,              $tab_padding_top,
        $tab_padding_left,           $tab_padding_right,
        $current_tab_padding_bottom, $tab_padding_bottom
    );
    $navi_image_dir
        = $this->base_change( $env::image_dir, $env::css . '/site-navi.css' )
        . '/navi';
    local (*CSS);
    $handle = \*STDOUT if !$handle;
    unless ( open( CSS, ">>$env::css/site-navi.css" ) ) {
        push(
            @{ $cgi::action->{'dialog'} },
            'CSS の書き出しに失敗しました。ファイルが開けませんでした。'
        );
        return;
    }
    binmode CSS, ":utf8";
    flock( CSS, LOCK_EX );
    truncate( CSS, 0 );
    seek( CSS, 0, 0 );

    #ロゴ
    if ( $this->{'page'}{'navi'}{'logo'}
        && ref $this->{'icon_setting'}{'logo'}
        { $this->{'page'}{'navi'}{'logo'} }
        && $this->{'icon_setting'}{'logo'}{ $this->{'page'}{'navi'}{'logo'} }
        {'height'} )
    {
        $this->{'page'}{'navi'}{'tabmargintop'}
            = $this->{'icon_setting'}{'logo'}
            { $this->{'page'}{'navi'}{'logo'} }{'height'}
            if !$this->{'page'}{'navi'}{'tabmargintop'}
            || $this->{'page'}{'navi'}{'tabmargintop'}
            < $this->{'icon_setting'}{'logo'}
            { $this->{'page'}{'navi'}{'logo'} }{'height'};
    }
    else {
        delete $this->{'page'}{'navi'}{'logo'};
    }

    #サイトタイトルの大きさ
    if (   $this->{'page'}{'navi'}{'sitetitlesize'}
        || $this->{'page'}{'navi'}{'logo'}
        || (   $this->{'page'}{'navi'}{'sitetitle'}
            && $this->{'page'}{'navi'}{'sitetitle'} == 2 )
        )
    {
        $sitetitle_height = int(
            14 * 0.9 * (
                ( $this->{'page'}{'navi'}{'sitetitlesize'} )
                ? $this->{'page'}{'navi'}{'sitetitlesize'} / 100
                : 1
            ) + 0.9
            ) + (
            (          $this->{'page'}{'navi'}{'sitetitle'}
                    && $this->{'page'}{'navi'}{'sitetitle'} == 2
            ) ? 20 : 0
            );
        $this->{'page'}{'navi'}{'tabmargintop'} = $sitetitle_height + 20
            if !$this->{'page'}{'navi'}{'tabmargintop'}
            || $this->{'page'}{'navi'}{'tabmargintop'} < $sitetitle_height;
    }

    #余白
    $tab_padding_top
        = ( defined $this->{'page'}{'navi'}{'tabpaddingtop'}
            && $this->{'page'}{'navi'}{'tabpaddingtop'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingtop'}
        : 4;
    $tab_padding_left
        = ( defined $this->{'page'}{'navi'}{'tabpaddingleft'}
            && $this->{'page'}{'navi'}{'tabpaddingleft'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingleft'}
        : ( defined $this->{'page'}{'navi'}{'tabpaddingright'}
            && $this->{'page'}{'navi'}{'tabpaddingright'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingright'}
        : 15;
    $tab_padding_right
        = ( defined $this->{'page'}{'navi'}{'tabpaddingright'}
            && $this->{'page'}{'navi'}{'tabpaddingright'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingright'}
        : $tab_padding_left;
    $tab_padding_bottom
        = ( defined $this->{'page'}{'navi'}{'tabpaddingbottom'}
            && $this->{'page'}{'navi'}{'tabpaddingbottom'} ne '' )
        ? $this->{'page'}{'navi'}{'tabpaddingbottom'}
        : $tab_padding_top;
    $current_tab_padding_bottom
        = ( defined $this->{'page'}{'navi'}{'tabbg'}
            && $this->{'icon_setting'}{'tabbg'}
            { $this->{'page'}{'navi'}{'tabbg'} }{'border'} )
        ? $tab_padding_bottom
        + $this->{'icon_setting'}{'tabbg'}
        { $this->{'page'}{'navi'}{'tabbg'} }{'border'}
        : $tab_padding_bottom;
    if ( $tab_padding_bottom < 0 ) {
        $tab_padding_top -= $tab_padding_bottom;
        $tab_padding_bottom = 0;
    }

    #ナビ共通（前）
    print $handle '<div id="site-navi"><div id="site-navi-bg"><ul>';
    if ( defined $env::title && $env::title ne '' ) {
        print $handle '<li id="site-info">';
        print $handle '<p id="site-title"><a href="'
            . $this->site_title_link . '">'
            . $env::title
            . '</a></p>';
        print $handle '<p id="site-subtitle">' . $env::description . '</p>'
            if $env::description;
        print $handle '</li>';
    }

    #CSS共通（前）
    print CSS '@charset "UTF-8";' . "\x0A";
    print CSS
        '/* BASED ON http://www.alistapart.com/articles/slidingdoors/ */'
        . "\x0A";
    print CSS 'div#site-navi {' . "\x0A";
    print CSS
        '  font-family: Avenir, "Lucida Grande", "Helvetica Neue", Helvetica, Arial, Verdana, Roboto, "ヒラギノ角ゴ Pro W3", "Hiragino Kaku Gothic Pro", "游ゴシック", "Yu Gothic", "游ゴシック体", "YuGothic", "Meiryo UI", "メイリオ", Meiryo, "ＭＳ Ｐゴシック", "MS PGothic", sans-serif;'
        . "\x0A";
    print CSS '  font-size: '
        . $this->{'page'}{'navi'}{'fontsize'} . '%;' . "\x0A"
        if $this->{'page'}{'navi'}{'fontsize'};
    print CSS '  font-weight: '
        . $this->{'page'}{'navi'}{'fontweight'} . ';' . "\x0A"
        if $this->{'page'}{'navi'}{'fontweight'};
    if ( defined $this->{'page'}{'navi'}{'tabbgalign'}
        && $this->{'page'}{'navi'}{'tabbgalign'} )
    {

        if ( $this->{'page'}{'navi'}{'tabbgalign'} eq 'center' ) {
            print CSS '  margin: 0 auto 0 auto;' . "\x0A";
        }
        elsif ( $this->{'page'}{'navi'}{'tabbgalign'} eq 'right' ) {
            print CSS '  margin: 0 0 0 auto;' . "\x0A";
        }
        else {
            print CSS '  margin: 0;' . "\x0A";
        }
    }
    else {
        print CSS '  margin: 0;' . "\x0A";
    }
    print CSS '  width: '
        . ( ( $this->{'page'}{'navi'}{'tabbgwidth'} )
        ? $this->{'page'}{'navi'}{'tabbgwidth'} . 'px'
        : '100%' )
        . ';' . "\x0A";
    print CSS '  background-color: '
        . $this->{'page'}{'navi'}{'tabbgcolor'} . ';' . "\x0A"
        if $this->{'page'}{'navi'}{'tabbgcolor'};
    print CSS '  line-height: 1.1;' . "\x0A";
    print CSS '  text-align: '
        . $this->{'page'}{'navi'}{'tabalign'} . ';' . "\x0A"
        if $this->{'page'}{'navi'}{'tabalign'}
        && $this->{'page'}{'navi'}{'tabwidth'}
        && $this->{'page'}{'navi'}{'tabalign'} =~ /^(left|right|center)$/;
    print CSS '  text-shadow: rgba(0,0,0,0.3) 0 2px 2px;' . "\x0A"
        if $this->{'page'}{'navi'}{'textshadow'};
    print CSS '  /min-height: 0;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi:after {' . "\x0A";
    print CSS '  content: ".";' . "\x0A";
    print CSS '  display: block;' . "\x0A";
    print CSS '  clear: both;' . "\x0A";
    print CSS '  height: 0;' . "\x0A";
    print CSS '  visibility: hidden;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS '* html div#site-navi {' . "\x0A";
    print CSS '  height: 0;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi div#site-navi-bg {' . "\x0A";
    print CSS '  float:left;' . "\x0A";
    print CSS '  margin: 0;' . "\x0A";
    print CSS '  width: 100%;' . "\x0A";

    if (   defined $this->{'page'}{'navi'}{'tabbg'}
        && $this->{'page'}{'navi'}{'tabbg'}
        && defined $this->{'icon_setting'}{'tabbg'}
        { $this->{'page'}{'navi'}{'tabbg'} }{'name'}
        && $this->{'icon_setting'}{'tabbg'}
        { $this->{'page'}{'navi'}{'tabbg'} }{'name'} )
    {
        print CSS '  background-image: url("'
            . $navi_image_dir
            . '/tabbg/'
            . $this->{'icon_setting'}{'tabbg'}
            { $this->{'page'}{'navi'}{'tabbg'} }{'name'} . '");' . "\x0A";
        print CSS '  background-repeat: repeat-x;' . "\x0A";
        print CSS '  background-position: bottom;' . "\x0A";
    }
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul {' . "\x0A";
    if ( $this->{'page'}{'navi'}{'tabwidth'} ) {
        $tabwidth = (
            ( $this->{'page'}{'navi'}{'tabmarginleft'} )
            ?         ( $this->{'page'}{'navi'}{'tabwidth'}
                    - ( $this->{'page'}{'navi'}{'tabmarginleft'} * 2 ) )
            : $this->{'page'}{'navi'}{'tabwidth'}
        );
        print CSS '  width: ' . $tabwidth . 'px;' . "\x0A";
        if ( defined $this->{'page'}{'navi'}{'tabalign'}
            && $this->{'page'}{'navi'}{'tabalign'} eq 'center' )
        {
            print CSS '  margin: 0 auto 0 auto;' . "\x0A";
        }
        elsif ( defined $this->{'page'}{'navi'}{'tabalign'}
            && $this->{'page'}{'navi'}{'tabalign'} eq 'right' )
        {
            print CSS '  margin: 0 0 0 auto;' . "\x0A";
        }
        else {
            print CSS '  margin: 0;' . "\x0A";
        }
    }
    else {
        print CSS '  margin: 0;' . "\x0A";
    }
    print CSS '  padding: '
        . (
        (   defined $this->{'page'}{'navi'}{'tabmargintop'}
                && $this->{'page'}{'navi'}{'tabmargintop'} ne ''
        )
        ? $this->{'page'}{'navi'}{'tabmargintop'}
            . ( ( $this->{'page'}{'navi'}{'tabmargintop'} ) ? 'px' : '' )
        : '10px'
        )
        . (
        (   defined $this->{'page'}{'navi'}{'tabmarginleft'}
                && $this->{'page'}{'navi'}{'tabmarginleft'} ne ''
        )
        ? ' '
            . $this->{'page'}{'navi'}{'tabmarginleft'}
            . ( ( $this->{'page'}{'navi'}{'tabmarginleft'} ) ? 'px' : '' )
            . ' 0 '
            . $this->{'page'}{'navi'}{'tabmarginleft'}
            . ( ( $this->{'page'}{'navi'}{'tabmarginleft'} ) ? 'px' : '' )
        : ' 0 0 0'
        ) . ";\x0A";
    print CSS '  list-style-type: none;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul li {' . "\x0A";
    print CSS '  float: left;' . "\x0A";
    if ( defined $this->{'page'}{'navi'}{'tabspace'}
        && $this->{'page'}{'navi'}{'tabspace'} )
    {
        my ( $margin_left, $margin_right );
        $margin_left = int( $this->{'page'}{'navi'}{'tabspace'} / 2 );
        $margin_right
            = $margin_left + ( $this->{'page'}{'navi'}{'tabspace'} % 2 );
        print CSS '  margin: 0 '
            . $margin_right . 'px 0 '
            . $margin_left . 'px;' . "\x0A";
    }
    else {
        print CSS '  margin: 0;' . "\x0A";
    }
    print CSS '  padding: 0;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul li a {' . "\x0A";
    print CSS '  float: left;' . "\x0A";
    print CSS '  margin: 0;' . "\x0A";
    print CSS '  margin-bottom: '
        . $this->{'icon_setting'}{'tabbg'}
        { $this->{'page'}{'navi'}{'tabbg'} }{'border'} . 'px;' . "\x0A"
        if defined $this->{'page'}{'navi'}{'tabbg'}
        && $this->{'icon_setting'}{'tabbg'}
        { $this->{'page'}{'navi'}{'tabbg'} }{'border'};
    if (   defined $this->{'page'}{'navi'}{'offtab'}
        && $this->{'page'}{'navi'}{'offtab'}
        && defined $this->{'icon_setting'}{'tab'}
        { $this->{'page'}{'navi'}{'offtab'} }{'padding'}
        && $this->{'icon_setting'}{'tab'}
        { $this->{'page'}{'navi'}{'offtab'} }{'padding'} )
    {
        print CSS '  padding: 0 0 0 '
            . $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'padding'} . 'px;' . "\x0A";
        print CSS '  background-image: url("'
            . $navi_image_dir . '/tab/'
            . $this->{'page'}{'navi'}{'offtab'} . '/'
            . $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'left'} . '");' . "\x0A";
        print CSS '  background-repeat: no-repeat;' . "\x0A";
        print CSS '  background-position: left top;' . "\x0A";
    }
    print CSS '  text-decoration: none;' . "\x0A";
    print CSS '  color: '
        . $this->{'icon_setting'}{'tab'}{ $this->{'page'}{'navi'}{'offtab'} }
        {'color'} . ';' . "\x0A"
        if $this->{'icon_setting'}{'tab'}
        { $this->{'page'}{'navi'}{'offtab'} }{'color'};
    print CSS '  outline: none;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul li a span {' . "\x0A";
    print CSS '  float: left;' . "\x0A";
    print CSS '  display: block;' . "\x0A";

    if (   defined $this->{'page'}{'navi'}{'offtab'}
        && $this->{'page'}{'navi'}{'offtab'}
        && defined $this->{'icon_setting'}{'tab'}
        { $this->{'page'}{'navi'}{'offtab'} }{'padding'}
        && $this->{'icon_setting'}{'tab'}
        { $this->{'page'}{'navi'}{'offtab'} }{'padding'} )
    {
        my $padding_left
            = $tab_padding_left
            - $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'padding'};
        my $padding_right = $tab_padding_right;
        if ( $padding_left < 0 ) {
            $padding_left     = 0;
            $tab_padding_left = $this->{'icon_setting'}{'tab'}
                { $this->{'page'}{'navi'}{'offtab'} }{'padding'};
        }
        if ( $padding_right < $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'padding'} )
        {
            $padding_right = $tab_padding_right
                = $this->{'icon_setting'}{'tab'}
                { $this->{'page'}{'navi'}{'offtab'} }{'padding'};
        }
        print CSS '  background-image: url("'
            . $navi_image_dir . '/tab/'
            . $this->{'page'}{'navi'}{'offtab'} . '/'
            . $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'right'} . '");' . "\x0A";
        print CSS '  background-repeat: no-repeat;' . "\x0A";
        print CSS '  background-position: right top;' . "\x0A";
        print CSS '  padding: '
            . ( ($tab_padding_top) ? $tab_padding_top . 'px' : '0' ) . ' '
            . $padding_right . 'px '
            . ( ($tab_padding_bottom) ? $tab_padding_bottom . 'px' : '0' )
            . ' '
            . $padding_left . 'px;' . "\x0A";
    }
    print CSS '}' . "\x0A";
    print CSS 'div#site-navi ul li#site-info p {' . "\x0A";
    print CSS '  margin: 0;' . "\x0A";
    print CSS '  padding: 0;' . "\x0A";
    print CSS '  line-height: 1;' . "\x0A";
    print CSS '  text-shadow: rgba(0,0,0,0.3) 0 2px 2px;' . "\x0A"
        if $this->{'page'}{'navi'}{'sitetitleshadow'};
    print CSS '}' . "\x0A";

    if ($sitetitle_height) {
        my $titlewidth = ($tabwidth) ? $tabwidth : 640;
        my $titlemargin = int(
            (         $this->{'page'}{'navi'}{'tabmargintop'}
                    + $tab_padding_top
                    - $sitetitle_height
            ) / 2
        );
        my $padding_left = 0;
        print CSS 'div#site-navi ul li#site-info {' . "\x0A";
        print CSS '  background-image: none;' . "\x0A";
        print CSS '  position: relative;' . "\x0A";
        print CSS '  width: ' . $titlewidth . 'px;' . "\x0A";
        print CSS '  height: '
            . $this->{'page'}{'navi'}{'tabmargintop'} . 'px;' . "\x0A";
        print CSS '  margin: -'
            . $this->{'page'}{'navi'}{'tabmargintop'} . 'px -'
            . $titlewidth
            . 'px 0 0;' . "\x0A";
        print CSS '  padding: 0;' . "\x0A";
        print CSS '  text-align: left;' . "\x0A";
        print CSS '}' . "\x0A";
        print CSS 'div#site-navi ul li#site-info p#site-title a {' . "\x0A";
        print CSS '  float: none;' . "\x0A";
        print CSS '  display: block;' . "\x0A";
        print CSS '  background: transparent none;' . "\x0A";
        print CSS '  text-align: left;' . "\x0A";
        print CSS '  text-decoration: none;' . "\x0A";

        if ( $this->{'page'}{'navi'}{'sitetitle'} ) {
            $padding_left
                = $tab_padding_left
                + ( ( $this->{'page'}{'navi'}{'logo'} )
                ? $tab_padding_left
                    + $this->{'icon_setting'}{'logo'}
                    { $this->{'page'}{'navi'}{'logo'} }{'width'}
                : 0 );
            print CSS '  width: '
                . ( $titlewidth - $padding_left ) . 'px;' . "\x0A";
            print CSS '  height: '
                . ( $this->{'page'}{'navi'}{'tabmargintop'} - $titlemargin )
                . 'px;' . "\x0A";
            print CSS '  padding: '
                . $titlemargin
                . 'px 0 0 '
                . $padding_left . 'px;' . "\x0A";
            print CSS '  font-size: '
                . $this->{'page'}{'navi'}{'sitetitlesize'} . '%;' . "\x0A";
            print CSS '  color: '
                . $this->{'page'}{'navi'}{'sitetitlecolor'} . ';' . "\x0A"
                if $this->{'page'}{'navi'}{'sitetitlecolor'};
            print CSS '  _width /**/: ' . $titlewidth . 'px;' . "\x0A";
            print CSS '  _height /**/: '
                . $this->{'page'}{'navi'}{'tabmargintop'} . 'px;' . "\x0A";
        }
        else {
            print CSS '  padding: 0;' . "\x0A";
            print CSS '  font-size: 1px;' . "\x0A";
            print CSS '  text-indent: -999px;' . "\x0A";
            print CSS '  overflow: hidden;' . "\x0A";
            print CSS '  width: ' . $titlewidth . 'px;' . "\x0A";
            print CSS '  height: '
                . $this->{'page'}{'navi'}{'tabmargintop'} . 'px;' . "\x0A";
        }
        if ( $this->{'page'}{'navi'}{'logo'} ) {
            my $top = int(
                (         $this->{'page'}{'navi'}{'tabmargintop'}
                        + $tab_padding_top
                        - $this->{'icon_setting'}{'logo'}
                        { $this->{'page'}{'navi'}{'logo'} }{'height'}
                ) / 2
            );
            $top
                = $this->{'page'}{'navi'}{'tabmargintop'}
                - $this->{'icon_setting'}{'logo'}
                { $this->{'page'}{'navi'}{'logo'} }{'height'}
                if $this->{'page'}{'navi'}{'tabmargintop'}
                < $this->{'icon_setting'}{'logo'}
                { $this->{'page'}{'navi'}{'logo'} }{'height'}
                + $top;
            print CSS '  background: transparent url("'
                . $this->base_change( $env::image_dir,
                $env::css . '/site-navi.css' )
                . '/navi/logo/'
                . $this->{'icon_setting'}{'logo'}
                { $this->{'page'}{'navi'}{'logo'} }{'name'}
                . '") no-repeat '
                . (
                ( $this->{'page'}{'navi'}{'sitetitle'} && $tab_padding_left )
                ? $tab_padding_left . 'px'
                : '0' )
                . ' '
                . ( ($top) ? $top . 'px' : '0' ) . ';' . "\x0A";
        }
        print CSS '}' . "\x0A";
        if (  !$this->{'page'}{'navi'}{'sitetitle'}
            || $this->{'page'}{'navi'}{'sitetitle'} == 1 )
        {
            print CSS 'div#site-navi ul li#site-info p#site-subtitle {'
                . "\x0A";
            print CSS '  display: none;' . "\x0A";
            print CSS '}' . "\x0A";
        }
        else {
            print CSS 'div#site-navi ul li#site-info p#site-subtitle {'
                . "\x0A";
            print CSS '  position: absolute;' . "\x0A";
            print CSS '  top: '
                . ( $titlemargin + $sitetitle_height - 13 ) . 'px;' . "\x0A";
            print CSS '  left: ' . $padding_left . 'px;' . "\x0A";
            print CSS '  font-weight: normal;' . "\x0A";
            if ( $this->{'page'}{'navi'}{'sitesubtitlecolor'} ) {
                print CSS '  color: '
                    . $this->{'page'}{'navi'}{'sitesubtitlecolor'} . ';'
                    . "\x0A";
            }
            elsif ( $this->{'page'}{'navi'}{'sitetitlecolor'} ) {
                print CSS '  color: '
                    . $this->{'page'}{'navi'}{'sitetitlecolor'} . ';'
                    . "\x0A";
            }
            print CSS '}' . "\x0A";
        }
    }
    else {
        print CSS 'div#site-navi ul li#site-info p#site-title a {' . "\x0A";
        print CSS '  display: block;' . "\x0A";
        print CSS '  background-image: none;' . "\x0A";
        print CSS '  text-decoration: none;' . "\x0A";
        print CSS '  padding: '
            . (
            ( $tab_padding_top > 2 ) ? ( $tab_padding_top - 2 ) . 'px' : '0' )
            . ' '
            . ( ($tab_padding_left) ? $tab_padding_left . 'px' : '0' )
            . ' 0 0;' . "\x0A";
        print CSS '  color: '
            . $this->{'page'}{'navi'}{'sitetitlecolor'} . ';' . "\x0A"
            if $this->{'page'}{'navi'}{'sitetitlecolor'};
        print CSS '}' . "\x0A";
    }

    foreach ( sort { $a <=> $b } keys %{ $this->{'navi'}{'ORDER'} } ) {
        my ( $id, $uri, $categories );
        $id = $this->{'navi'}{'ORDER'}{$_};
        next
            if !defined $this->{'navi'}{$id}{'use'}
            || !$this->{'navi'}{$id}{'use'};
        ( $uri, $categories )
            = $this->get_default_uri( $id, $this->{'navi'}{$id} );

        #ナビ
        print $handle '<li id="tab-'
            . $id . '">'
            . $this->get_nav_link(
            $uri,
            $this->{'navi'}{$id}{'title'},
            $this->{'navi'}{$id}{'description'},
            $this->{'navi'}{$id}{'openWindow'},
            0, 1
            ) . $categories . '</li>';

        $left_on_selector
            .= (
            ( defined $left_on_selector && $left_on_selector ) ? ', ' : '' )
            . 'body#document'
            . $this->{'navi'}{$id}{'id'}
            . ' div#site-navi ul li#tab-'
            . $id
            . ' a, div#site-navi ul li#tab-'
            . $id
            . ':hover a';
        $right_on_selector
            .= (
            ( defined $right_on_selector && $right_on_selector ) ? ', ' : '' )
            . 'body#document'
            . $this->{'navi'}{$id}{'id'}
            . ' div#site-navi ul li#tab-'
            . $id
            . ' a span, div#site-navi ul li#tab-'
            . $id
            . ':hover a span';
        $current_selector
            .= (
            ( defined $current_selector && $current_selector ) ? ', ' : '' )
            . 'body#document'
            . $this->{'navi'}{$id}{'id'}
            . ' div#site-navi ul li#tab-'
            . $id
            . ' a span';
        $menu_selector
            .= ( ( defined $menu_selector && $menu_selector ) ? ', ' : '' )
            . 'body#document'
            . $this->{'navi'}{$id}{'id'}
            . ' div#site-navi ul li#tab-'
            . $id
            . ':hover div.menu div.bubble';
    }

    #ナビCSS
    print CSS $left_on_selector . ' {' . "\x0A";
    if (   defined $this->{'page'}{'navi'}{'ontab'}
        && $this->{'page'}{'navi'}{'ontab'}
        && defined $this->{'icon_setting'}{'tab'}
        { $this->{'page'}{'navi'}{'ontab'} }{'padding'}
        && $this->{'icon_setting'}{'tab'}{ $this->{'page'}{'navi'}{'ontab'} }
        {'padding'} )
    {
        if (defined $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'padding'}
            && $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'ontab'} }{'padding'}
            != $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'padding'} )
        {
            print CSS '  padding: 0 0 0 '
                . $this->{'icon_setting'}{'tab'}
                { $this->{'page'}{'navi'}{'ontab'} }{'padding'} . 'px;'
                . "\x0A";
        }
        print CSS '  background-image: url("'
            . $navi_image_dir . '/tab/'
            . $this->{'page'}{'navi'}{'ontab'} . '/'
            . $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'ontab'} }{'left'} . '");' . "\x0A";
    }
    print CSS '  color: '
        . $this->{'icon_setting'}{'tab'}{ $this->{'page'}{'navi'}{'ontab'} }
        {'color'} . ';' . "\x0A"
        if $this->{'icon_setting'}{'tab'}{ $this->{'page'}{'navi'}{'ontab'} }
        {'color'};
    print CSS '  margin-bottom: 0;' . "\x0A";
    print CSS '}' . "\x0A";
    print CSS $right_on_selector . ' {' . "\x0A";
    if (   defined $this->{'page'}{'navi'}{'ontab'}
        && $this->{'page'}{'navi'}{'ontab'}
        && defined $this->{'icon_setting'}{'tab'}
        { $this->{'page'}{'navi'}{'ontab'} }{'padding'}
        && $this->{'icon_setting'}{'tab'}{ $this->{'page'}{'navi'}{'ontab'} }
        {'padding'} )
    {
        print CSS '  background-image: url("'
            . $navi_image_dir . '/tab/'
            . $this->{'page'}{'navi'}{'ontab'} . '/'
            . $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'ontab'} }{'right'} . '");' . "\x0A";
        if (defined $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'padding'}
            && $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'ontab'} }{'padding'}
            != $this->{'icon_setting'}{'tab'}
            { $this->{'page'}{'navi'}{'offtab'} }{'padding'} )
        {
            my $padding_left
                = $tab_padding_left
                - $this->{'icon_setting'}{'tab'}
                { $this->{'page'}{'navi'}{'ontab'} }{'padding'};
            my $padding_right = $tab_padding_right;
            if ( $padding_left < 0 ) {
                $padding_left  = 0;
                $padding_right = $this->{'icon_setting'}{'tab'}
                    { $this->{'page'}{'navi'}{'ontab'} }{'padding'};
            }
            print CSS '  padding: '
                . ( ($tab_padding_top) ? $tab_padding_top . 'px' : '0' ) . ' '
                . $padding_right . 'px '
                . ( ($tab_padding_bottom) ? $tab_padding_bottom . 'px' : '0' )
                . ' '
                . $padding_left . 'px;' . "\x0A";
        }
    }
    print CSS '}' . "\x0A";
    if ( defined $this->{'page'}{'navi'}{'tabbg'}
        && $this->{'icon_setting'}{'tabbg'}
        { $this->{'page'}{'navi'}{'tabbg'} }{'border'} )
    {
        print CSS $current_selector . '{' . "\x0A";
        print CSS '  padding-bottom: '
            . $current_tab_padding_bottom . 'px;' . "\x0A";
        print CSS '}' . "\x0A";
    }
    print CSS $this->category_menu_css(
        'site-navi',
        $this->{'page'}{'navi'}{'menuhovercolor'},
        $this->{'page'}{'navi'}{'menuactivecolor'}
    );
    print CSS $menu_selector . '{' . "\x0A";
    print CSS '  margin-top: '
        . ( 4 + $tab_padding_bottom - $current_tab_padding_bottom ) . 'px;'
        . "\x0A";
    print CSS '}' . "\x0A";
    print CSS $this->gsearch_css;
    print CSS $this->{'page'}{'navi'}{'tabcss'}
        if defined $this->{'page'}{'navi'}{'tabcss'}
        && $this->{'page'}{'navi'}{'tabcss'};
    close(CSS);

    print $handle '</ul></div></div>';
    if ( defined $this->{'page'}{'navi'}{'accestat'}
        && $this->{'page'}{'navi'}{'accestat'} )
    {
        $this->print_accestat_image( $handle,
            ( ( $output_option eq 'abs' ) ? 0 : 1 ) );
    }
}

sub category_menu_css ($;$$$) {
    my ( $this, $header_name, $hovercolor, $activecolor ) = @_;
    $header_name = 'site-navi' if !$header_name;
    $hovercolor  = '#0088cc'   if !$hovercolor;
    $activecolor = '#F7931E'   if !$activecolor;

    return <<CSS;
div#$header_name ul li ul,
div#$header_name ul li ul li,
div#$header_name ul li ul li a,
div#$header_name ul li ul li a:hover {
  float: none !important;
  display: block !important;
  width: auto !important;
  height: auto !important;
  max-width: none !important;
  min-width: 0 !important;
  margin: 0;
  padding: 0;
  background-color: transparent !important;
  background-image: none !important;
  text-decoration: none !important;
  font-size: 12px !important;
  font-weight: normal !important;
  font-family: Avenir, "Lucida Grande", "Helvetica Neue", Helvetica, Arial, Verdana, Roboto, "ヒラギノ角ゴ Pro W3", "Hiragino Kaku Gothic Pro", "游ゴシック", "Yu Gothic", "游ゴシック体", "YuGothic", "Meiryo UI", "メイリオ", Meiryo, "ＭＳ Ｐゴシック", "MS PGothic", sans-serif;
  text-shadow: none !important;
  text-indent: 0 !important;
  line-height: 1.2 !important;
  color: #999999;
  border: none !important;
  -moz-border-radius: 0;
  -webkit-border-radius: 0;
  -o-border-radius: 0;
  -ms-border-radius: 0;
  border-radius: 0;
  white-space: normal !important;
}
div#$header_name ul li ul li a {
  box-shadow: none !important;
  -moz-opacity: 1 !important;
  -webkit-opacity: 1 !important;
  -o-opacity: 1 !important;
  opacity: 1 !important;
  -ms-filter: none !important;
  filter: none !important;
}

div#$header_name ul li div.menu {
  clear: both;
  width: 2px;
  margin: 0 auto;
  visibility: hidden;
  /display: none;
}
div#$header_name ul li:hover div.menu {
  visibility: visible;
}
div#$header_name ul li div.menu div.bubble,
div#$header_name ul li div.menu div.bubble ul li ul {
  position: absolute;
  padding: 0;
  width: 158px !important;
  height: auto;
  border: 1px #ddd solid !important;
  background: #fff none !important;
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  -o-border-radius: 5px;
  -ms-border-radius: 5px;
  border-radius: 5px;
  -moz-box-shadow: 0 8px 8px rgba(0,0,0,0.5);
  -webkit-box-shadow: 0 8px 8px rgba(0,0,0,0.5);
  -o-box-shadow: 0 8px 8px rgba(0,0,0,0.5);
  -ms-box-shadow: 0 8px 8px rgba(0,0,0,0.5);
  box-shadow: 0 8px 8px rgba(0,0,0,0.5);
  -moz-opacity: 0;
  -webkit-opacity: 0;
  -o-opacity: 0;
  opacity: 0;
  -moz-transition-property: margin-top, opacity;
  -moz-transition-duration: 0.5s;
  -webkit-transition-property: margin-top, opacity;
  -webkit-transition-duration: 0.5s;
  -o-transition-property: margin-top, opacity;
  -o-transition-duration: 0.5s;
  transition-property: margin-top, opacity;
  transition-duration: 0.5s;
}
div#$header_name ul li div.menu div.bubble {
  z-index: 999;
  margin: 100px -79px 0 -79px;
}
div#$header_name ul li div.menu div.bubble > ul {
  display: block;
  list-style: none;
  margin: -8px 0 0 0;
  padding: 8px 0 0 0;
  background: transparent url("../image/tri_white.gif") no-repeat center top !important;
  text-align: left;
}
div#$header_name ul li div.menu div.bubble ul li {
  position: relative;
}
div#$header_name ul li div.menu div.bubble ul li ul {
  z-index: 2;
  visibility: hidden;
  margin: 0;
  top: 0;
  left: 156px;
}
div#$header_name ul li:hover div.menu div.bubble,
div#$header_name ul li div.menu div.bubble ul li:hover > ul {
  -moz-opacity: 1;
  -webkit-opacity: 1;
  -o-opacity: 1;
  opacity: 1;
}
div#$header_name ul li:hover div.menu div.bubble {
  margin-top: 4px;
}
div#$header_name ul li div.menu div.bubble ul li:hover > ul {
  visibility: visible;
}
div#$header_name ul li div.menu div.bubble ul li ul:hover {
  top: -1px;
}

div#$header_name ul li div.menu div.bubble ul li a {
  width: 130px !important;
  margin: 0 !important;
  padding: 8px 14px !important;
  color: #999 !important;
  text-decoration: none !important;
  border-bottom: #eee 1px solid !important;
}
div#$header_name ul li div.menu div.bubble ul li:first-child > a {
  -webkit-border-radius: 5px 5px 0 0;
     -moz-border-radius: 5px 5px 0 0;
          border-radius: 5px 5px 0 0;
}
div#$header_name ul li div.menu div.bubble ul li:last-child > a {
  -webkit-border-radius: 0 0 5px 5px;
     -moz-border-radius: 0 0 5px 5px;
          border-radius: 0 0 5px 5px;
  border-bottom: none !important;
  margin: 0 !important;
}
div#$header_name ul li div.menu div.bubble ul li:first-child:last-child > a {
  -webkit-border-radius: 5px 5px 5px 5px;
     -moz-border-radius: 5px 5px 5px 5px;
          border-radius: 5px 5px 5px 5px;
}

div#$header_name ul li div.menu div.bubble ul li:hover > a {
  color: #666 !important;
  background: #eee none !important;
}
div#$header_name ul li div.menu div.bubble ul li a:hover {
  margin: -1px -1px 0 -1px !important;
  padding: 9px 15px !important;
  border-bottom: none !important;
  color: #ffffff !important;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.2) !important;
  background: $hovercolor none !important;
}
div#$header_name ul li div.menu div.bubble ul li a:active {
  background: $activecolor none !important;
}
div#$header_name ul li div.menu div.bubble ul li:last-child > a:hover,
div#$header_name ul li div.menu div.bubble ul li:last-child > a:active {
  margin: -1px !important;
}
CSS
}

sub get_shadow_color ($$) {
    my ( $this, $color ) = @_;
    my ( $R, $G, $B );
    if ( $color =~ /^[#]*([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})/ ) {
        ( $R, $G, $B ) = (
            unpack( "C", pack( "H2", $1 ) ),
            unpack( "C", pack( "H2", $2 ) ),
            unpack( "C", pack( "H2", $3 ) )
        );
    }
    return (
        '#'
            . unpack( "H2", pack( "C", $this->color_adjuster( $R - 51 ) ) )
            . unpack( "H2", pack( "C", $this->color_adjuster( $G - 51 ) ) )
            . unpack( "H2", pack( "C", $this->color_adjuster( $B - 51 ) ) ),
        '#'
            . unpack( "H2", pack( "C", $this->color_adjuster( $R - 68 ) ) )
            . unpack( "H2", pack( "C", $this->color_adjuster( $G - 68 ) ) )
            . unpack( "H2", pack( "C", $this->color_adjuster( $B - 68 ) ) )
    );
}

sub color_adjuster ($$) {
    my ( $this, $num ) = @_;
    $num = 255 if $num > 255;
    $num = 0   if $num < 0;
    return $num;
}

######################################################################

1;
