package ImageEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 画像ライブラリ編集モジュール
# ImageEditor.pm ver1.1.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新：2008-05-21
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ($image,   $image_dir, $image_lib,  $deco_pref,
        $setting, $deco_obj,  $image_name, $image_css
    );

    #不正なID
    $main::id  = '' if !defined $main::id  || $main::id =~ /[^0-9a-zA-Z]/;
    $main::dir = '' if !defined $main::dir || $main::dir =~ /[^0-9a-zA-Z]/;

    #画像基本情報取得
    ( $image_dir, $image_lib, $setting, $deco_pref, $image_css )
        = $this->get_image_info;

    require CMONOS::CGI_Lib::Deco;
    $deco_obj = Deco->new( $image_dir, $image_lib, '', $setting, $deco_pref );

    #画像編集オブジェクト
    $image = ImageLib->new( $setting::deco_type{$main::dir},
        $deco_obj, $image_css );
    $image_name
        = (    defined $setting::deco_dir{$main::dir}
            && defined $setting::deco_dir{$main::dir} ne '' )
        ? $setting::deco_dir{$main::dir}
        : '素材';
    $image_name =~ s/^([0-9])[0-9]*://;

    #素材ライブラリ用イメージCSSの有無をチェック
    if ( $image->image_css_is_ready( $image_lib, $main::dir, $setting ) ) {
        $this->set_style_sheet_link(
            $image_lib . '/' . $main::dir . '/image.css' );
    }

    #書き込み
    if ( defined $main::a && $main::a eq 'w' ) {
        my ($delete);

        #削除
        if ( defined $main::id && $main::id ne '' && $main::delete ) {
            $delete = $main::id;
            $image->erase;
            $this->delete_data( $deco_pref,
                $main::dir . '_' . $main::id . '_' );
            $main::id = '';

        }
        else {
            my ( $check, $new );

            #IDなしのときは新規作成
            if ( !defined $main::id || $main::id eq '' ) {
                $main::id = $image->get_id($main::dir);
                $new      = 1;
            }

            #新規にアップロード
            if ( grep( $main::image{$_} ne '', keys %main::image ) ) {
                $check = $image->save_images;
                $cgi::input->clear_clipboard;

                #ライブラリから加工
            }
            elsif ( defined $main::img && $main::img ne '' ) {
                $check = $image->import_images;

                #画像から加工
            }
            elsif ( !$new && defined $main::color && $main::color ne '' ) {
                $check = $image->change_image_color;

                #設定のみ
            }
            elsif (%main::setting) {
                if ( defined $new && $new ) {
                    push(
                        @{ $this->{'dialog'} },
                        '設定を保存できませんでした。'
                            . $image_name
                            . 'が登録されていません。'
                    );
                    $main::id = '';
                    %{ $setting::deco{$main::dir}{$main::id} }
                        = %main::setting;
                    $check = 0;
                }
                else {
                    $check = 1;
                }
            }

            #詳細設定
            if ($check) {
                $image->set_image;
                push(
                    @{ $this->{'dialog'} },
                    $image_name
                        . 'を更新しました。表示に変更が反映されない場合は、再読込をするかキャッシュをクリアしてください。'
                );
                $this->_success_dialog($new);
            }
            else {
                $main::id = '' if defined $new && $new;
                push(
                    @{ $this->{'dialog'} },
                    $image_name . '更新に失敗しました。'
                );
            }
        }
    }
    $this->set_javascript(
        <<'JAVASCRIPT'
cmonos.form.showColorTableSample = function () {
	if (this.currentField && this.currentField.value == '') {
		document.getElementById('colortable-sample').innerHTML = '';
		this.currentField.style.backgroundColor = '';
	} else {
		var sample = '';
		var fixed_color = '';
		var color = this.currentField.value.split(/[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/);
		for (var i=0; i<color.length; i++) {
			if (color[i] != null && color[i].search(/^#[0-9a-fA-F]{6}$/) != -1) {
				sample += '<div style="float: left; width: 15px; height: 15px; background-color: ' + color[i] + ';" title="' + color[i] + '"></div>';
				fixed_color += ((fixed_color != '') ? ',' : '') + color[i];
			} else if (color[i] != null && color[i].search(/^random$/i) != -1) {
				fixed_color += ((fixed_color != '') ? ',' : '') + color[i];
			}
		}
		if (sample != '') sample += '<div style="clear:left"></div>';
		document.getElementById('colortable-sample').innerHTML = sample;
		this.currentField.value = fixed_color;
	}
}
JAVASCRIPT
    );
    $this->set_onload('cmonos.init.clearButtonSync();');
    $this->jump_menu('tools');

    #ライブラリメニュー
    $setting::page{'tools'}{'subMenu'} = $this->form_start(
        'key'      => $setting::security_key,
        'name'     => 'imageselector',
        'required' => \@setting::form
    );
    $setting::page{'tools'}{'subMenu'}
        .= '<select name="dir" onchange="submit()">';
    my ( $image_type, $image_no, $lib_menu );
    foreach (
        sort {
            $setting::deco_type{$a} cmp $setting::deco_type{$b}
                || $setting::deco_dir{$a} cmp $setting::deco_dir{$b}
        } keys %setting::deco_dir
        )
    {
        $setting::page{'tools'}{'subMenu'}
            .= '<option value="' . $main::dir . '"></option>'
            if ( defined $image_type
            && $image_type ne ''
            && $image_type ne $setting::deco_type{$_} )
            || ( defined $image_no
            && $image_no ne ''
            && $setting::deco_dir{$_} !~ /^$image_no/ );
        $image_type = $setting::deco_type{$_};
        my $name = $setting::deco_dir{$_};
        $image_no = ( $name =~ s/^([0-9])[0-9]*:// ) ? $1 : '';
        $setting::page{'tools'}{'subMenu'}
            .= '<option value="'
            . $_ . '"'
            . ( ( $_ eq $main::dir ) ? ' selected="selected"' : '' ) . '>'
            . $name
            . '</option>';
    }
    $setting::page{'tools'}{'subMenu'} .= '</select>';
    $setting::page{'tools'}{'subMenu'}
        .= '<input name="subJumpMenu" type="submit" value="選択" /></form>';
    if ( defined $setting::deco_dir{$main::dir}
        && $setting::deco_dir{$main::dir} ne '' )
    {
        $setting::page{'tools'}{'topicpath'} .= (
            (   defined $setting::page{'tools'}{'topicpath'}
                    && $setting::page{'tools'}{'topicpath'} ne ''
            ) ? ' &gt; ' : ''
        ) . $image_name;
        $this->push_window_title($image_name);
    }

    #ページヘッダ
    $this->print_page_header('tools');

    #画像選択
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'image',
        'required' => \@setting::form
    );
    print '<fieldset id="toolnavi">';
    print '<legend>' . $image_name . 'ライブラリ</legend>';
    print '<input type="hidden" name="dir" value="'
        . $this->escape($main::dir) . '" />';
    print $image->menu( 'id', 0, $main::id );
    print
        '<div class="choice"><input type="submit" class="choose" value="選択" /></div>';
    print '</fieldset></form>';

    #IDなしのときは新規作成
    my ( $targetmark, $save_button );
    if ( !defined $main::id || $main::id eq '' ) {
        $save_button
            = 'この ' . $image_name . ' をライブラリに追加する';
        print '<h1>' . $image_name . '追加</h1>';
    }
    else {
        $save_button = 'この ' . $image_name . ' に差し替える';
        print '<h1>' . $image_name . '差し替え</h1>';
        print
            '<p class="note"><span class="markersample">▲</span>マークのついた'
            . $image_name
            . 'を更新します。</p>';
    }

    #フォームヘッダ
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'multi'    => 1,
        'required' => \@setting::form
    );
    print '<input type="hidden" name="dir" value="'
        . $this->escape($main::dir) . '" />';
    print '<input type="hidden" name="id" value="'
        . $this->escape($main::id) . '" />';
    print '<input type="hidden" name="a" value="w" />';

    #詳細設定
    $image->print_setting_form;

    #画像入れ換え
    print '<fieldset><legend>ライブラリ画像の登録</legend>';

    print '<fieldset class="material">';
    print '<legend>素材選択</legend>';
    print $image->menu( 'img', 1, $main::img );
    print '</fieldset>';

    print '<fieldset class="coloroption">';
    print
        '<legend title="GIF 画像の色を変更します。">GIF 画像配色表変更</legend>';

    print '<table class="field-table"><tbody>';

    print '<tr><td class="field-name">';
    print '<label><input type="radio" name="color" value=""'
        . ( ( !defined $main::color || $main::color eq '' )
        ? ' checked="checked"'
        : '' )
        . ' />同一配色表</label>';
    print
        '</td><td class="field">ライブラリの配色に準じます。</td></tr>';

    print '<tr><td class="field-name">';
    print '<label><input type="radio" name="color" value="slide"'
        . ( ( defined $main::color && $main::color eq 'slide' )
        ? ' checked="checked"'
        : '' )
        . ' />配色表調整</label></td>';
    print
        '<td class="field"><input type="text" name="slide:color" id="slide:color" size="8" maxlength="8" onfocus="cmonos.form.item(this.form).fieldname(\'color\',1).select()" onclick="cmonos.tool.colorPalette.open(\'slide:color\',\'dom\')" onchange="cmonos.validate.field(this).asColor(this.value,\'rgb\')" value="'
        . $this->escape( $main::slide{'color'} )
        . '" title="random = ランダムに色を指定します。"'
        . ( ( $main::slide{'color'} )
        ? ' style="background-color: ' . $main::slide{'color'} . '"'
        : '' )
        . ' />';
    print
        ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'slide:color\')" ondblclick="cmonos.tool.colorPalette.open(\'slide:color\',\'window\')" onclick="cmonos.form.item(document.input).fieldname(\'color\',1).select()"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print
        ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'slide:color\')" ondblclick="cmonos.tool.colorSlider.open(\'slide:color\',\'window\')" onclick="cmonos.form.item(document.input).fieldname(\'color\',1).select()"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '<label for="slide:color" class="note">（基準色）</label>';

    $main::slide{'rev'} = '-#666666'
        if !defined $main::slide{'rev'} || $main::slide{'rev'} eq '';
    my $rev = $main::slide{'rev'};
    $rev =~ s/^[+\-]//;
    print
        ' <input type="text" name="slide:rev" id="slide:rev" size="8" maxlength="8" onfocus="cmonos.form.item(this.form).fieldname(\'color\',1).select()" onclick="cmonos.tool.colorPalette.open(\'slide:rev\',\'dom\')" onchange="cmonos.validate.field(this).asColor(this.value,\'rgb\')" value="'
        . $this->escape( $main::slide{'rev'} )
        . '" style="background-color: '
        . $rev . '" />';
    print
        ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'slide:rev\')" ondblclick="cmonos.tool.colorPalette.open(\'slide:rev\',\'window\')" onclick="cmonos.form.item(document.input).fieldname(\'color\',1).select()"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print
        ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'slide:rev\')" ondblclick="cmonos.tool.colorSlider.open(\'slide:rev\',\'window\')" onclick="cmonos.form.item(document.input).fieldname(\'color\',1).select()"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '<label for="slide:rev" class="note">（補正色）</label>';

    print
        ' <label class="note"><input type="checkbox" name="slide:abs" id="slide:abs" value="1" onclick="cmonos.form.item(this.form).fieldname(\'color\',1).select()"'
        . ( ( $main::slide{'abs'} ) ? ' checked="checked"' : '' ) . ' /> ';
    print '値をループする</label>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label><input type="radio" name="color" value="change"'
        . ( ( defined $main::color && $main::color eq 'change' )
        ? ' checked="checked"'
        : '' )
        . ' />配色表置換</label></td>';
    print
        '<td class="field"><input type="text" name="change:color" id="change:color"  size="46" onclick="cmonos.tool.colorPalette.open(\'change:color+\',\'dom\')" onfocus="cmonos.form.item(this.form).fieldname(\'color\',2).select();cmonos.form.field(this).showColorTableSample();" onchange="cmonos.form.field(this).showColorTableSample();" value="'
        . $this->escape( $main::change{'color'} )
        . '" title="random,random,random = ランダムに色を指定します。" />';
    print
        ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'change:color+\')" ondblclick="cmonos.tool.colorPalette.open(\'change:color+\',\'window\')" onclick="cmonos.form.item(document.input).fieldname(\'color\',2).select()"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print '<label for="change:color" class="note">（配色表）</label>';
    print '<div id="colortable-sample"></div>';
    print '</td></tr>';

    print '</tbody></table>';
    print '</fieldset>';

    print '<div class="submit"><input type="submit" value="'
        . $save_button
        . '" /></div>';
    print '</fieldset>';

    #画像新規登録
    $image->print_upload_form($save_button);

    if ( defined $main::id && $main::id ne '' ) {
        print
            '<fieldset class="delete"><input type="submit" name="delete=1" value="この '
            . $image_name
            . ' を削除する" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\');" /></fieldset>';
    }
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# 画像基本情報
sub get_image_info ($) {
    my $this = shift;
    $this->call_field(2)
        if defined $default::field_pref_dir && $default::field_pref_dir;
    if (   !defined $main::dir
        || $main::dir eq ''
        || !defined $setting::deco_type{$main::dir} )
    {
        if ( defined $setting::deco_type{'frame'}
            && $setting::deco_type{'frame'} )
        {
            $main::dir = 'frame';
        }
        else {
            $main::dir = ( sort keys %setting::deco_type )[0];
        }
    }
    return (
        $env::image_dir, $env::image_lib, \%setting::deco,
        $env::preference_dir . '/deco',
        $env::css . '/image.css'
    );
}

sub _success_dialog ($$) {
    my $this = shift;
    return;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( defined $default::subdivided
        && $default::subdivided
        && defined $main::c
        && $main::c ne '' )
    {

        #クラス登録
        Common::class_register( '0100', 'ImageEditor' );

        #メニュー登録
        $setting::jump_menu{'0100'} = '画像ライブラリ編集(共通)';
        $setting::master_check_option{'0100'} = 1;
    }
}

1;
