package ImageMenuEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 画像メニュー編集モジュール
# ImageMenuEditor.pm ver1.1.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新：2009-03-17
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my (%image);
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }
    $main::a      = '' if !defined $main::a;
    $main::target = '' if !defined $main::target;
    my $image_dir = '';
    if ($main::target =~ /^[0-9a-zA-Z_]{1,16}$/
        && ((   $main::a eq 'new' && $this->set_dir(
                    $env::image_dir . '/imagemenu/' . $main::target
                )
            )
            || ( $main::a ne 'new'
                && -d $env::image_dir . '/imagemenu/' . $main::target )
        )
        )
    {
        $image_dir = $env::image_dir . '/imagemenu/' . $main::target;
        if ( $main::a eq 'e' && !$main::remove ) {
            push( @main::delete, $main::delete )
                if !@main::delete && $main::delete;
            foreach (@main::delete) {
                unlink $image_dir . '/' . $_
                    if $_ =~ /^[0-9a-zA-Z_]+\.(gif|png|jpg)$/i;
            }
            if ( defined $main::new{'file'} && $main::new{'file'} ne '' ) {
                my $new_file
                    = ( $main::new{'file'} =~ /([^\/\\:]*)(\.[^\.]+)$/ )
                    ? $1 . ( lc $2 )
                    : $main::new{'file'};
                if ($cgi::Image->is_not_a_web_image(
                        $cgi::input, 'new:file'
                    )
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        '画像を追加できませんでした。画像ファイルをブラウザで表示できる形式(GIF,JPEG,PNG)にしてください。'
                    );
                }
                elsif ( $new_file !~ /^[0-9a-zA-Z_]{1,16}\.(gif|png|jpg)$/ ) {
                    push(
                        @{ $this->{'dialog'} },
                        '画像を追加できませんでした。画像ファイル名を英数字列にしてください。'
                    );
                }
                elsif ( -f $image_dir . '/' . $new_file ) {
                    push(
                        @{ $this->{'dialog'} },
                        '画像を追加できませんでした。同じ名前の画像ファイルが既に存在します。'
                    );
                }
                else {
                    $cgi::input->save_file(
                        'new:file',
                        '<client>',
                        [   'image/jpeg:jpg', 'image/pjpeg:jpg',
                            'image/gif:gif',  'image/png:png',
                            'image/x-png:png'
                        ],
                        '',
                        $image_dir
                    );
                    if ( -f $image_dir . '/' . $new_file ) {
                        my $key = $new_file;
                        $key =~ s/\.(gif|jpg|png)$/_$1/i;
                        $main::image{$key}{'file'} = $new_file;
                        $main::image{$key}{'no'}
                            = ( defined $main::new{'no'}
                                && $main::new{'no'} ne '' )
                            ? $main::new{'no'}
                            : 999;
                        $main::image{$key}{'name'}
                            = ( $main::new{'name'} )
                            ? $main::new{'name'}
                            : '';
                        $main::image{$key}{'reserved'}
                            = ( $main::new{'reserved'} )
                            ? $main::new{'reserved'}
                            : '';
                        undef %main::new;
                    }
                }
                $cgi::input->clear_clipboard;
            }
            $this->write_image_menu( $image_dir, \%image, \%main::image,
                \$main::title );
        }
        elsif ($main::remove) {
            use File::Path;
            rmtree $image_dir;
            push(
                @{ $this->{'dialog'} },
                '画像メニューフォルダを削除しました。'
            );
            $image_dir    = '';
            $main::target = '';
        }
        else {
            $main::title = $this->read_image_menu( $image_dir, \%image );
        }
    }
    elsif ( $main::a eq 'new' ) {
        push(
            @{ $this->{'dialog'} },
            '画像メニューフォルダを作成できませんでした。16字以内の英数字列にしてください。'
        );
    }

    #ページヘッダ
    $this->jump_menu('tools');

    #画像メニューフォルダセレクタ
    local (*DIR);
    if ( opendir( DIR, $env::image_dir . '/imagemenu' ) ) {
        my @menu
            = grep( $_ !~ /^\./ && -d $env::image_dir . '/imagemenu/' . $_,
            readdir(DIR) );
        closedir(DIR);
        $setting::page{'tools'}{'subMenu'} = $this->form_start(
            'key'      => $setting::security_key,
            'name'     => 'folderselector',
            'required' => \@setting::form
        );
        $setting::page{'tools'}{'subMenu'}
            .= '<select name="target" onchange="submit()"><option value="">新規作成</option>';
        $setting::page{'tools'}{'subMenu'} .= '<option value=""></option>'
            if @menu;
        foreach ( sort @menu ) {
            my $title = $this->get_image_menu_name($_);
            $setting::page{'tools'}{'subMenu'}
                .= '<option value="'
                . $_ . '"'
                . ( ( defined $main::target && $main::target eq $_ )
                ? ' selected="selected"'
                : '' )
                . '>'
                . ( ( defined $title && $title ne '' ) ? $title : $_ )
                . '</option>';
        }
        $setting::page{'tools'}{'subMenu'}
            .= '</select><input name="subJumpMenu" type="submit" value="選択" /></form>';
    }

    #ページヘッダ
    $this->print_page_header('tools');
    print '<h1>画像メニュー編集</h1>';

    #新規作成
    if ( $image_dir eq '' ) {
        print $this->form_start(
            'key'    => 'tools',
            'name'   => 'input',
            'method' => 'post',
            'onsubmit' =>
                'return cmonos.validate.field(document.input.target).asWord() &amp;&amp; cmonos.form.item(this).isNotSubmitted()',
            'required' => \@setting::form
        );
        print '<input type="hidden" name="a" value="new" />';
        print '<fieldset class="submit">';
        print
            '<label for="target" class="datatitle">フォルダ名</label><input type="text" name="target" value="'
            . $this->escape($main::target)
            . '" size="16" maxlength="16" />';
        print
            '<input type="submit" value="新しいフォルダを作成する" /></fieldset></form>';
        $this->print_page_footer('tools');
        exit;
    }

    #フォームヘッダ
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'multi'    => 1,
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );

    print '<input type="hidden" name="target" value="'
        . $main::target . '" />';
    print '<input type="hidden" name="a" value="e" />';
    print
        '<fieldset><legend><label for="title">画像メニュー名</label></legend>';
    print '<input type="text" name="title" value="'
        . $this->escape($main::title)
        . '" size="30" />';
    print '</fieldset>';
    print '<fieldset><legend>画像設定と削除</legend>';

    #テーブル設定
    require CMONOS::CGI_Lib::Table;
    my $table = Table->new;
    $table->set(
        'tableclass' => 'tooltable',
        'trclass'    => 'odd,even',
        'class0'     => 'odd,even',
        'style0'     => 'white-space: nowrap;text-align: center;',
        'thenabled'  => 'true',
        'classF'     => 'caution'
    );

    print $table->table;

    #ヘッダ
    print $table->tr;
    print $table->td('削除');
    print $table->td('表示順');
    print $table->td('ファイル名');
    print $table->td('名前');
    print $table->td('画像');
    print $table->td('オーナー専用');
    print $table->_tr;

    no warnings qw(numeric);
    my ( $key, $n );
    $n = 0;
    foreach $key ( sort { $image{$a}{'no'} <=> $image{$b}{'no'} || $a cmp $b }
        keys %image )
    {
        print $table->tr;
        print $table->td( '<input type="checkbox" name="delete" value="'
                . $image{$key}{'file'}
                . '" />' );
        print $table->td( '<input type="text" name="image:'
                . $key
                . ':no" value="'
                . ( ++$n )
                . '" size="3" />' );
        print $table->td( $image{$key}{'file'} );
        print $table->td( '<input type="text" name="image:'
                . $key
                . ':name" value="'
                . $this->escape( $image{$key}{'name'} )
                . '" size="20" />' );
        print $table->td(
            $cgi::Image->tag(
                $image_dir . '/' . $image{$key}{'file'},
                0, $image{$key}{'name'},
                '', 200, 150
            )
        );
        print $table->td( '<input type="checkbox" name="image:'
                . $key
                . ':reserved" value="1"'
                . ( ( $image{$key}{'reserved'} ) ? ' checked="checked"' : '' )
                . ' title="管理権限のない一般ユーザーの使用を禁止するオプションです。" />'
        );
        print $table->_tr;
    }
    print $table->tr;
    print $table->td('新規');
    print $table->td( '<input type="text" name="new:no" value="'
            . ( ++$n )
            . '" size="3" />' );
    print $table->td('');
    print $table->td( '<input type="text" name="new:name" value="'
            . $this->escape( $main::new{'name'} )
            . '" size="20" />' );
    print $table->td( '<input type="file" name="new:file" value="'
            . $this->escape( $main::new{'file'} )
            . '" />' );
    print $table->td( '<input type="checkbox" name="new::reserved" value="1"'
            . ( ( $main::new{'reserved'} ) ? ' checked="checked"' : '' )
            . ' title="管理権限のない一般ユーザーの使用を禁止するオプションです。" />'
    );
    print $table->_tr;

    print $table->_table;
    print '</fieldset>';
    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    print
        '<fieldset class="delete"><input type="submit" name="remove" value="この画像メニューフォルダを削除する" onclick="return confirm(\'この操作は取り消せません。本当にこの画像メニューフォルダを削除しますか？\')" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# 画像メニュー書き込み
sub write_image_menu ($$$) {
    no warnings qw(numeric);
    my ( $this, $image_dir, $image, $new_image, $title ) = @_;
    my ($key);
    local (*NAME);
    my $old_title = $this->read_image_menu( $image_dir, $image );
    if ( defined $$title ) {
        $$title =~ s/[\x0A\x0D]/ /sg;
        $$title =~ s/:/&#58;/g;
    }
    else {
        $$title = $old_title;
    }
    my $n         = 0;
    my $is_opened = 0;
    if ( open( NAME, ">>$image_dir/name.txt" ) ) {
        binmode NAME, ":utf8";
        flock( NAME, LOCK_EX );
        truncate( NAME, 0 );
        seek( NAME, 0, 0 );
        print NAME $$title, "\x0A" if $$title ne '';
        $is_opened = 1;
    }
    foreach $key (
        sort {
            $new_image->{$a}->{'no'} <=> $new_image->{$b}->{'no'}
                || $image->{$a}->{'no'} <=> $image->{$b}->{'no'}
                || $a cmp $b
        } keys %$image
        )
    {
        $new_image->{$key}->{'name'} =~ s/[\x0A\x0D]/ /sg;
        $new_image->{$key}->{'name'} =~ s/:/&#58;/g;
        $new_image->{$key}->{'reserved'} = ''
            if !$new_image->{$key}->{'reserved'};
        $image->{$key}->{'no'} = $new_image->{$key}->{'no'} = ++$n;
        $image->{$key}->{'name'}     = $new_image->{$key}->{'name'};
        $image->{$key}->{'reserved'} = $new_image->{$key}->{'reserved'};
        print NAME $image->{$key}->{'file'} . ':'
            . $image->{$key}->{'name'} . ':'
            . $image->{$key}->{'reserved'} . "\x0A"
            if $is_opened;
    }
    close(NAME) if $is_opened;
}

######################################################################
# 画像メニュー読み込み
sub read_image_menu ($$$) {
    my ( $this, $image_dir, $image ) = @_;
    my ($file);
    my $n     = 0;
    my $title = '';
    %$image = ();
    local (*DIR);
    local (*NAME);
    opendir( DIR, $image_dir );
    $image_dir .= '/';

    while ( $file = readdir(DIR) ) {
        if (   $file =~ /\.(gif|jpg|png)$/i
            && $file !~ /^\./
            && -f $image_dir . $file )
        {
            my $key = $file;
            $key =~ s/\.(gif|jpg|png)$/_$1/i;
            $image->{$key}{'file'}     = $file;
            $image->{$key}{'name'}     = '';
            $image->{$key}{'no'}       = 999;
            $image->{$key}{'reserved'} = '';
        }
    }
    closedir(DIR);
    if ( open( NAME, $image_dir . 'name.txt' ) ) {
        binmode NAME, ":utf8";
        flock( NAME, LOCK_SH );
        while ( my $line = <NAME> ) {
            chomp $line;
            if ( !$n && $line !~ /:/ ) {
                $title = $line;
                $n++;
                next;
            }
            my ( $file, $name, $reserved ) = split( /:/, $line, 3 );
            my $key = $file;
            $key =~ s/\.(gif|jpg|png)$/_$1/i;
            next if !ref $image->{$key};
            $n++;
            $image->{$key}{'name'}     = $name;
            $image->{$key}{'no'}       = $n;
            $image->{$key}{'reserved'} = $reserved;
        }
        close(NAME);
    }
    return $title;
}

sub get_image_menu_name ($$) {
    my ( $this, $image ) = @_;
    my $title = '';
    local (*NAME);
    if (open( NAME, $env::image_dir . '/imagemenu/' . $image . '/name.txt' ) )
    {
        binmode NAME, ":utf8";
        flock( NAME, LOCK_SH );
        $title = <NAME>;
        close(NAME);
        chomp $title if $title;
    }
    return ( $title && $title !~ /:/ ) ? $title : '';
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( defined $default::subdivided
        && $default::subdivided
        && defined $main::c
        && $main::c ne '' )
    {

        #クラス登録
        Common::class_register( '0101', 'ImageMenuEditor' );

        #メニュー登録
        $setting::jump_menu{'0101'} = '画像メニュー編集(共通)';
        $setting::master_check_option{'0101'} = 1;
    }
}

1;
