package Preference;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 初期設定モジュール
# Preference.pm ver1.1.5
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-12-20
######################################################################
# 実行
sub run ($) {
    no warnings qw(once);
    my $this = shift;
    my ( $k, $v );

    $this->set_preferences;

    #書き込み
    if ( defined $main::a && $main::a eq 'w' ) {
        my ($id);

        $this->get_setting_items;
        push( @main::id, $main::id )       if defined $main::id;
        push( @main::id, $preference::id ) if !@main::id;
        foreach $id (@main::id) {
            if ( !defined $id ) {
                $id = '';
            }
            elsif ( $id ne '' ) {
                $id .= '_';
            }

            #必須項目チェック
            while ( my ( $k, $v ) = each %check::required ) {
                if ( !defined $main::data{ $id . $k }
                    || $main::data{ $id . $k } eq '' )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v . 'が入力されていません。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #チェックボックス調整
            foreach my $k (@check::checkbox) {
                $main::data{ $id . $k } = ''
                    if !defined $main::data{ $id . $k }
                    || $main::data{ $id . $k } eq '';
            }

            #タグ除去項目
            foreach my $k (@check::notag) {
                $this->detag( \$main::data{ $id . $k } )
                    if defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne '';
            }

            #タグ修正項目
            foreach my $k (@check::tagfix) {
                $this->tag_fix( \$main::data{ $id . $k } )
                    if defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne '';
            }

            #カラー書式チェック
            while ( my ( $k, $v ) = each %check::color ) {
                if (   defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne ''
                    && $this->is_invalid_color( $main::data{ $id . $k } ) )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。色指定を確認してください。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #複数カラー書式チェック
            while ( my ( $k, $v ) = each %check::colors ) {
                if (   defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne ''
                    && $this->is_invalid_cscolor( $main::data{ $id . $k } ) )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。色指定を確認してください。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #メール書式チェック
            while ( ( $k, $v ) = each %check::email ) {
                if (   defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne ''
                    && $this->is_invalid_email_address(
                        $main::data{ $id . $k }
                    )
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。メールアドレスを確認してください。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #複数メール書式チェック
            while ( my ( $k, $v ) = each %check::emails ) {
                if (   defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne ''
                    && $this->is_invalid_email_address_header(
                        $main::data{ $id . $k }
                    )
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。メールアドレスを確認してください。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #URL書式チェック
            while ( my ( $k, $v ) = each %check::url ) {
                if (   defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne ''
                    && $this->is_invalid_url( $main::data{ $id . $k } ) )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。URLを確認してください。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #数字チェック
            while ( my ( $k, $v ) = each %check::num ) {
                if (   defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne ''
                    && $this->is_nan( $main::data{ $id . $k } ) )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。半角数字にしてください。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #半角英数字チェック
            while ( my ( $k, $v ) = each %check::word ) {
                $main::data{ $id . $k }
                    = $this->z_to_h( $main::data{ $id . $k } );
                if (   defined $main::data{ $id . $k }
                    && $main::data{ $id . $k } ne ''
                    && $main::data{ $id . $k } =~ /[^0-9a-zA-Z_\-\x20]/ )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。半角英数字にしてください。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #サイズチェック
            while ( my ( $k, $v ) = each %check::maxlength ) {
                if ( defined $main::data{ $id . $k }
                    && bytes::length( $id . $k . $main::data{ $id . $k } )
                    > $env::db_maxlength
                    && $env::db_maxlength > 0 )
                {
                    push(
                        @{ $this->{'dialog'} },
                        $v
                            . 'を更新できませんでした。容量オーバーです。'
                    );
                    delete $main::data{ $id . $k };
                }
            }

            #画像チェック
            while ( my ( $k, $v ) = each %check::imagefile ) {
                my $n = $id . $k;
                if (   defined $main::data{$n}
                    && defined $preference::filedir{$k}
                    && $main::data{$n} ne ''
                    && $preference::filedir{$k} ne '' )
                {
                    if ( $this->is_invalid_image( $n, $k, $v ) ) {
                        push(
                            @{ $this->{'dialog'} },
                            $v . 'を更新できませんでした。'
                        );
                        $this->pref_unlink_file( $preference::filedir{$k},
                            $n )
                            if defined $main::delete{$n} && $main::delete{$n};
                        delete $main::data{$n};
                    }
                }
                elsif ( defined $main::delete{$n} && $main::delete{$n} ) {
                    $this->pref_unlink_file( $preference::filedir{$k}, $n );
                }
                else {
                    delete $main::data{$n};
                }
            }
        }

        #モジュールオリジナルチェック
        $this->is_valid_setting;

        #書き込み
        $this->write_settings;

        #後処理
        $this->update_settings;
        $cgi::input->clear_clipboard;

    }
    else {
        #読み込み
        $this->read_settings;
    }

    #ページヘッダ
    $this->set_page_header;
    $this->jump_menu( 'tools', $preference::submenukey );
    $this->print_page_header('tools');

    $this->print_progress_bar;

    #タイトル
    print '<h1'
        . (
        ($preference::title) ? ' title="' . $preference::title . '"' : '' )
        . '>'
        . $preference::name . '</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted(true)',
        'multi'    => ( ($cgi::enctype_multipart) ? 1 : 0 ),
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="w" />';

    $this->print_preference_panel;

    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');

    $this->site_update;
}

######################################################################
#設定書き込み読み込み
sub write_settings ($) {
    my $this = shift;
    $this->write_db( $default::preference_dir . '/' . $preference::db,
        \%main::data, '', '', \%main::data, 0, @preference::import_key );
}

sub read_settings ($) {
    my $this = shift;
    $this->import_db( $default::preference_dir . '/' . $preference::db,
        \%main::data, 0, @preference::import_key );
}

######################################################################
# 画像設定チェック
sub is_invalid_image ($$$$) {
    my ( $this, $inputname, $key, $name ) = @_;
    my ( $is_valid, $width, $height, $ext, $formname );
    $is_valid = 1;
    $formname = 'data:' . $inputname;

    #添付ファイルが画像ファイルかどうかチェック
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }
    ( $width, $height, $ext ) = $cgi::Image->input_info(
        $cgi::input, $formname, 0,
        $check::widthlimit{$key},
        $check::heightlimit{$key}, 1
    );

    #MIME-TYPEチェック
    if ( !defined $ext || $ext eq '' || !$width || !$height ) {
        push(
            @{ $this->{'dialog'} },
            $name . 'のファイルタイプを確認してください。'
        );
        $is_valid = 0;
    }

    if ($is_valid) {
        if ( $check::ogimage{$key} ) {

            #画像サイズチェック
            if ( $width < 50 && $width > 640 ) {
                push(
                    @{ $this->{'dialog'} },
                    $name
                        . 'の横幅を確認してください。50ピクセル以上640ピクセル以内にしてください。'
                );
                $is_valid = 0;
            }
            if ( $height < 50 && $height > 640 ) {
                push(
                    @{ $this->{'dialog'} },
                    $name
                        . 'の高さを確認してください。50ピクセル以上640ピクセル以内にしてください。'
                );
                $is_valid = 0;
            }
        }
        else {

            #ファイルサイズチェック
            if (   defined $check::maxsize{$key}
                && $check::maxsize{$key} > 0
                && $cgi::input->get_posted_file_size_from_param($formname)
                > $check::maxsize{$key} )
            {
                push(
                    @{ $this->{'dialog'} },
                    $name
                        . 'のファイルサイズを確認してください。'
                        . $check::maxsize{$key}
                        . 'バイト以内にしてください。'
                );
                $is_valid = 0;
            }

            #画像サイズチェック
            if (   $width
                && $check::widthlimit{$key}
                && $width > $check::widthlimit{$key} )
            {
                push(
                    @{ $this->{'dialog'} },
                    $name
                        . 'の横幅を確認してください。'
                        . $check::widthlimit{$key}
                        . 'ピクセル以内にしてください。'
                );
                $is_valid = 0;
            }
            if (   $height
                && $check::heightlimit{$key}
                && $height > $check::heightlimit{$key} )
            {
                push(
                    @{ $this->{'dialog'} },
                    $name
                        . 'の高さを確認してください。'
                        . $check::heightlimit{$key}
                        . 'ピクセル以内にしてください。'
                );
                $is_valid = 0;
            }
        }
    }
    if ($is_valid) {
        $main::data{$inputname}              = $inputname . '.' . $ext;
        $main::data{ $inputname . 'width' }  = $width;
        $main::data{ $inputname . 'height' } = $height;
        $cgi::input->save_file(
            $formname, '!' . $inputname,
            '', '', $preference::filedir{$key},
        );
        return 0;

    }
    else {
        delete $main::data{$inputname};
        delete $main::data{ $inputname . 'width' };
        delete $main::data{ $inputname . 'height' };
        return 1;
    }
}

sub pref_unlink_file ($$$) {
    my ( $this, $dir, $name ) = @_;
    unlink( $this->fileglob( $dir, '^' . $name . '\.', 'f' ) )
        if defined $dir && $dir;
    $main::data{$name}              = '';
    $main::data{ $name . 'width' }  = '';
    $main::data{ $name . 'height' } = '';
    delete $main::delete{$name};
}

######################################################################
# 画像設定フォーム
sub print_image_upload_form ($$$$) {
    my ( $this, $inputname, $key, $data ) = @_;

    #ファイルの存在をチェック
    ( $$data{$key}, $$data{ $key . 'width' }, $$data{ $key . 'height' } ) = ()
        unless $$data{$key}
        && -f $preference::filedir{$key} . '/' . $$data{$key};

    #アップロードフォーム
    print '<input type="file" name="data:'
        . $inputname
        . '" id="data:'
        . $inputname . '" />';
    print '<div class="note">(&lt; ' . $check::maxsize{$key} . ' byte)</div>'
        if $check::maxsize{$key};
    print '<div class="note">(&lt; '
        . (
        ( $check::widthlimit{$key} ) ? $check::widthlimit{$key} : '&infin;' )
        . ' × '
        . ( ( $check::heightlimit{$key} )
        ? $check::heightlimit{$key}
        : '&infin;' )
        . ' ピクセル)</div>'
        if ( defined $check::widthlimit{$key} && $check::widthlimit{$key} )
        || ( defined $check::heightlimit{$key} && $check::heightlimit{$key} );

    #既存ファイルの表示
    if ( defined $$data{$key} && $$data{$key} ne '' ) {
        print '<div class="sample-image">';
        unless ( ref $cgi::Image ) {
            require CMONOS::CGI_Lib::Image;
            $cgi::Image = Image->new;
        }
        print $cgi::Image->image_tag(
            $preference::filedir{$key} . '/' . $$data{$key},
            $$data{ $key . 'width' },
            $$data{ $key . 'height' }
        );
        print
            '<div class="extra-field"><label><input type="checkbox" name="delete:'
            . $inputname
            . '" value="1" /> このファイルを削除する</label></div>';
        print '</div>';
    }
    return;
}

######################################################################
# 設定環境
sub set_preferences ($) {
    my $this = shift;
    push(
        @{ $this->{'dialog'} },
        'スクリプトエラー：Preference モジュールに異常があります。'
    );
    $this->print_error_dialog('tools');
}

sub set_page_header ($) {
    my $this = shift;
    return;
}

######################################################################
# チェック項目
sub get_setting_items ($) {
    my $this = shift;
    return;
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;
    push(
        @{ $this->{'dialog'} },
        'スクリプトエラー：Preference モジュールに異常があります。'
    );
    $this->print_error_dialog('tools');
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    return;
}

######################################################################
# 後処理
sub update_settings ($) {
    my $this = shift;
    return;
}

sub print_progress_bar ($) {
    my $this = shift;
    return;
}

sub site_update ($) {
    my $this = shift;
    return;
}

1;
