package UserAgent;
use strict;
use utf8;
######################################################################
# ユーザー環境を解析するモジュール
# UserAgent.pm ver1.1.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-12-07
######################################################################
# 使い方
# use UserAgent;
# obj = UserAgent->new
# (platform,platform_ver,browser,browser_ver,limitsize,device,language)
#	= obj->get_user_agent_info($ENV{'HTTP_USER_AGENT'});
#
#	platform			プラットフォーム
#	platform_ver		OSバージョン
#	browser				ブラウザ
#	browser_ver			ブラウザバージョン
#	limitsize			容量制限情報
#	device				携帯機種名
#	language			言語
######################################################################
# オブジェクトを作る。
sub new ($) {
    my $pkg  = shift;
    my $this = {};
    bless $this;
    $this;
}

# ユーザー環境解析
sub get_user_agent_info ($;$) {
    my ( $this, $user_agent ) = @_;
    my ( $ua, $platform, $platform_ver, $browser, $browser_ver, $limitsize,
        $device, $lang );
    $user_agent = $ENV{'HTTP_USER_AGENT'}
        if !defined $user_agent || $user_agent eq '';
    $ua = $user_agent;

    # 言語
    $lang = $1 if $user_agent =~ s/\[(.*?)\]/ /s;

    # 携帯？
    if ( $user_agent !~ /\// ) {
        $platform = '携帯電話';
        $platform_ver = $device = $browser = $user_agent;

        # Google Web Preview
    }
    elsif ( $user_agent =~ /Google Wireless Transcoder/ ) {
        $browser_ver  = $1 if $user_agent =~ /Version\/([0-9\.abr]+)/;
        $platform     = '携帯電話';
        $platform_ver = $device = $browser = 'Google モバイル検索';

        # Google Web Preview
    }
    elsif ( $user_agent =~ /Google Web Preview/ ) {
        $platform     = '検索ロボット';
        $platform_ver = 'Google Web Preview';

        # 検索ロボット
    }
    elsif ( $user_agent
        =~ s![\x09\x0A\x0C\x0D\x20]*\(([^\(\)]*?https?://[^\(\)]*?)\)!!i )
    {
        my ( @info, $url );

        # プラットフォーム情報
        @info = split( /;[\x09\x0A\x0C\x0D\x20]*\+?/, $1 );
        foreach my $data (@info) {
            if ( $data =~ m!https?://! ) {
                $url = $data;
            }
            elsif ( !$platform_ver ) {
                if ( $data =~ m!^([^\/]+)/[0-9\.abr]+! ) {
                    $platform_ver = $1;
                }
                elsif ( $data =~ m!/! ) {
                    $platform_ver = $data;
                }
            }
        }
        $platform     = '検索ロボット';
        $platform_ver = $1
            if !$platform_ver
            && $user_agent
            =~ s/^([^\/]+)\/([0-9\.abr]+[N]?)[\x09\x0A\x0C\x0D\x20]*//;

        # PC
    }
    elsif ( $user_agent =~ s/[\x09\x0A\x0C\x0D\x20]*\((.*?)\)// ) {
        my ( @info, $engine, $engine_ver, $option );

        # プラットフォーム情報
        @info = split( /;[\x09\x0A\x0C\x0D\x20]*/, $1 );

        # Mozilla
        ( $engine, $engine_ver )
            = ( $user_agent
                =~ s/^([^\/]+)\/([0-9\.abr]+[N]?)[\x09\x0A\x0C\x0D\x20]*// )
            ? ( $1, $2 )
            : ( '', '' );

        # Mozilla互換
        if ( $engine eq 'Mozilla' ) {

            # MSIE形式
            if ( defined $info[0] && $info[0] eq 'compatible' ) {
                ( $browser, $browser_ver )
                    = ( defined $info[1]
                        && $info[1]
                        =~ s/^([^\/\x09\x0A\x0C\x0D\x20]+)([\x09\x0A\x0C\x0D\x20\/]|[\x09\x0A\x0C\x0D\x20]*Ver[\x09\x0A\x0C\x0D\x20]*)([0-9\.abr]+)[\x09\x0A\x0C\x0D\x20]*//i
                    ) ? ( $1, $3 ) : ( '', '' );
                $browser = 'InternetExplorer' if $browser eq 'MSIE';
                $platform_ver = ( defined $info[2] ) ? $info[2] : '';
                $platform_ver = 'Windows Me'
                    if defined $info[3]
                    && $platform_ver eq 'Windows 98'
                    && $info[3] =~ /Win 9x/;

                # Opera形式
                if ( $user_agent
                    =~ s/[\x09\x0A\x0C\x0D\x20]*Opera[\x09\x0A\x0C\x0D\x20]*([0-9\.abr]+)//
                    )
                {
                    $browser_ver = $1;
                    $browser_ver = $1
                        if $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*Version\/([0-9\.abr]+)//;
                    $browser = 'Opera';
                    ( $platform, $platform_ver, $device )
                        = ( $1, $2, $2 . '/' . $3 )
                        if defined $info[2]
                        && $info[2] =~ m!^([^/]+)/([^/]+)/([0-9\.abr]+)!;
                }

                # Mac
                if (grep( $_
                            =~ /Mac[\x09\x0A\x0C\x0D\x20]*OS[\x09\x0A\x0C\x0D\x20]*X/i,
                        @info )
                    )
                {
                    $platform     = 'Macintosh';
                    $platform_ver = 'MacOS X';
                }

                # Mozilla系
            }
            elsif ( grep( $_ =~ /^[NIU]$/, @info )
                || $user_agent
                =~ /[\x09\x0A\x0C\x0D\x20]*Gecko\/?[0-9\.abr]*/ )
            {
                if ( $user_agent
                    =~ s/[\x09\x0A\x0C\x0D\x20]*AppleWebKit\/([0-9\.abr]+)// )
                {
                    $browser     = 'AppleWebKit';
                    $browser_ver = $1;
                    $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*Mobile\/[^\x09\x0A\x0C\x0D\x20]+//;
                    if ( $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*OPR\/([0-9\.abr]+)//i )
                    {
                        $browser     = 'Opera';
                        $browser_ver = $1;
                    }
                    elsif ( $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*(?:Chrome|CriOS)\/([0-9\.abr]+)//i
                        )
                    {
                        $browser     = 'Chrome';
                        $browser_ver = $1;
                    }
                    elsif ( $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*Shiira\/([0-9\.abr]+)//i )
                    {
                        $browser     = 'Shiira';
                        $browser_ver = $1;
                    }
                    elsif ( $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*([a-zA-Z]+)[0-9\.abr]*\/([0-9\.abr]+)//
                        )
                    {
                        $browser     = $1;
                        $browser_ver = $2;
                        my $version_is_found
                            = ( $browser eq 'Version' ) ? 1 : 0;
                        if (   $version_is_found
                            && $user_agent
                            =~ s/[\x09\x0A\x0C\x0D\x20]*([a-zA-Z]+)[0-9\.abr]*\/[0-9\.abr]+//
                            )
                        {
                            $browser = $1;
                        }
                        if ((      $browser eq 'AppleWebKit'
                                || $browser eq 'Safari'
                            )
                            && $user_agent
                            =~ s/[\x09\x0A\x0C\x0D\x20]*([a-zA-Z]+)[0-9\.abr]*\/([0-9\.abr]+)//
                            )
                        {
                            $browser = $1;
                            $browser_ver = $2 if !$version_is_found;
                            if ((      $browser eq 'AppleWebKit'
                                    || $browser eq 'Safari'
                                )
                                && $user_agent
                                =~ s/[\x09\x0A\x0C\x0D\x20]*([a-zA-Z]+)[0-9\.abr]*\/([0-9\.abr]+)//
                                )
                            {
                                $browser = $1;
                                $browser_ver = $2 if !$version_is_found;
                            }
                        }
                    }
                    if ( $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*Version\/([0-9\.abr]+)//i
                        )
                    {
                        $browser_ver = $1;
                    }
                }
                elsif ( grep( $_ =~ /^Trident\/[0-9\.abr]+/i, @info ) ) {
                    my ($ver) = grep( $_ =~ /^rv:[0-9\.abr]+$/, @info );
                    if ( $ver && $ver =~ /^rv:([0-9\.abr]+)$/ ) {
                        ( $browser, $browser_ver )
                            = ( 'InternetExplorer', $1 );
                        $platform_ver = ( defined $info[0] ) ? $info[0] : '';
                    }
                }
                elsif ( $user_agent
                    =~ s/[\x09\x0A\x0C\x0D\x20]*Gecko\/?[0-9\.abr]*// )
                {
                    $browser = 'Firefox';
                    if ( $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*Firefox\/([0-9\.abr]+)// )
                    {
                        $browser_ver = $1;
                        if ( $user_agent
                            =~ s/[\x09\x0A\x0C\x0D\x20]*([a-zA-Z]+)[0-9\.abr]*\/([0-9\.abr]+)//
                            )
                        {
                            $browser     = $1;
                            $browser_ver = $2;
                        }
                    }
                    elsif ( $user_agent
                        =~ s/[\x09\x0A\x0C\x0D\x20]*([a-zA-Z]+)[0-9\.abr]*\/([0-9\.abr]+)//
                        )
                    {
                        $browser     = $1;
                        $browser_ver = $2;
                    }
                    else {
                        $browser = 'Gecko';
                        $browser_ver
                            = ( defined $info[4]
                                && $info[4] =~ /^rv:([0-9\.]+)/ )
                            ? $1
                            : ( defined $info[1]
                                && $info[1] =~ /^rv:([0-9\.]+)/ ) ? $1
                            : '';
                    }
                }
                $browser_ver = $engine_ver
                    if !defined $browser_ver || !$browser_ver;

                $platform = ( defined $info[0] ) ? $info[0] : '';
                $platform_ver
                    = ( defined $info[1] && $info[1] =~ /^[NIU]$/ )
                    ? ( defined $info[2] )
                        ? $info[2]
                        : ''
                    : ( defined $info[1] ) ? $info[1]
                    : ''
                    if !$platform_ver;

                # DreamPassport
            }
            elsif ( defined $info[0]
                && $info[0] =~ /^DreamPassport\/([0-9\.abr]+)/ )
            {
                $browser      = 'DreamPassport';
                $browser_ver  = $1;
                $platform     = 'その他';
                $platform_ver = 'Dreamcast';

                # PSP
            }
            elsif ( defined $info[0] && $info[0] =~ /^PSP/ ) {
                $browser      = 'PSP';
                $browser_ver  = ( $user_agent =~ /([0-9\.]+)/ ) ? $1 : '';
                $platform     = 'その他';
                $platform_ver = 'PSP';

                # PlayStation
            }
            elsif ( defined $info[0]
                && $info[0]
                =~ /^(?:PLAYSTATION[\x09\x0A\x0C\x0D\x20]*|PS)([0-9]*)/i )
            {
                $browser      = 'PlayStation ' . $1;
                $browser_ver  = $info[1];
                $platform     = 'その他';
                $platform_ver = $browser;

                # jig browser
            }
            elsif ( defined $info[0]
                && $info[0]
                =~ /^jig browser[\x09\x0A\x0C\x0D\x20\/]([0-9\.abr]+)/i )
            {
                $browser = $platform_ver = 'jig browser';
                $browser_ver = $1;
                $platform    = '携帯電話';
                $device      = $info[1];

            }
            elsif ( defined $info[0] && $info[0] =~ /^jig browser/i ) {
                $browser = $platform_ver = $info[0];
                $browser_ver = $info[1];
                $platform    = '携帯電話';
                $device      = $info[2];

                # NetFront
            }
            elsif ( $user_agent
                =~ s/^[\x09\x0A\x0C\x0D\x20]*NetFront\/([0-9\.abr]+)[\x09\x0A\x0C\x0D\x20]*//
                )
            {
                $browser     = 'NetFront';
                $browser_ver = $1;
                if ( $info[1] eq 'SoftBank' ) {
                    $platform     = '携帯電話';
                    $platform_ver = $info[1];
                    $device       = $info[0];
                }
                else {
                    $platform = ( defined $info[0] ) ? $info[0] : '';
                    my @platform_info
                        = ( $info[1] )
                        ? split( "/", $info[1] )
                        : ( '', '', '', '' );
                    $platform_ver = $platform_info[0];
                    $device       = 'CLIE'
                        if $platform_info[0] eq 'PalmOS'
                        && $platform_info[1] eq 'sony';
                    $device .= $platform_info[2] . '/' . $platform_info[3]
                        if $platform_info[2]
                        =~ s/^model[\x09\x0A\x0C\x0D\x20]+//;
                }

                # Opera形式
            }
            elsif ( $user_agent
                =~ s/[\x09\x0A\x0C\x0D\x20]*Opera[\x09\x0A\x0C\x0D\x20]*([0-9\.abr]+)//
                )
            {
                $browser_ver = $1;
                $browser_ver = $1
                    if $user_agent
                    =~ s/[\x09\x0A\x0C\x0D\x20]*Version\/([0-9\.abr]+)//;
                $browser = 'Opera';
                ( $platform, $platform_ver, $device )
                    = ( defined $info[1]
                        && $info[1] =~ m!^([^/]+)/([^/]+)/([0-9\.abr]+)! )
                    ? ( $1, $2, $2 . '/' . $3 )
                    : ( '', '', '' );

                # AppleWebKit形式
            }
            elsif ( $user_agent
                =~ s/[\x09\x0A\x0C\x0D\x20]*AppleWebKit\/([0-9\.abr]+)// )
            {
                $browser_ver  = $1;
                $platform     = 'Macintosh';
                $platform_ver = 'MacOS X';
                if ( $user_agent
                    =~ s/([^\/\x09\x0A\x0C\x0D\x20]+)\/([0-9\.abr]+)// )
                {
                    $browser     = $1;
                    $browser_ver = $2;
                }
                else {
                    $browser = 'AppleWebKit';
                }

                # Handycam
            }
            elsif ( $user_agent
                =~ s/[\x09\x0A\x0C\x0D\x20]*NF([0-9\.abr]+) \(NQ\)// )
            {
                $browser_ver  = $1;
                $browser      = 'NetFront';
                $platform     = 'その他';
                $platform_ver = $device = 'Handycam';
            }

            # Mozilla非互換
        }
        else {
            if ( grep( $_ =~ /^[NIU]$/, @info ) ) {
                $platform = ( defined $info[0] ) ? $info[0] : '';
                $platform_ver
                    = ( defined $info[1] && $info[1] =~ /^[NIU]$/ )
                    ? ( defined $info[0] )
                        ? $info[0]
                        : ''
                    : ( defined $info[1] ) ? $info[1]
                    :                        '';
                $lang
                    = (    defined $info[1]
                        && $info[1] =~ /^[NIU]$/
                        && defined $info[2] ) ? $info[2]
                    : ( defined $info[3] ) ? $info[3]
                    :                        '';
            }
            elsif ( defined $info[0]
                && $info[0]
                =~ s/^(Windows NT[\x09\x0A\x0C\x0D\x20][0-9\.abr]+)[\x09\x0A\x0C\x0D\x20]*//i
                )
            {
                ( $platform, $platform_ver ) = ( 'Windows', $1 );
            }
            elsif ( defined $info[0]
                && $info[0]
                =~ s/^([^\/\x09\x0A\x0C\x0D\x20]+)([\x09\x0A\x0C\x0D\x20\/]|[\x09\x0A\x0C\x0D\x20]*Ver[\x09\x0A\x0C\x0D\x20]*)([0-9\.abr]+)[\x09\x0A\x0C\x0D\x20]*//i
                )
            {
                ( $platform, $platform_ver ) = ( $1, $3 );
            }
            if ( $engine eq 'sharp pda browser' ) {

                # DoCoMo BrowserBoard
                if ( $platform eq 'BrowserBoard' ) {
                    $platform_ver = 'DoCoMo BrowserBoard';

                    # ZAURUS
                }
                else {
                    $platform_ver = 'ZAURUS/' . $platform_ver;
                }
                $platform = 'その他';

            }
            elsif ( $engine =~ /DoCoMo/i ) {
                $platform     = '携帯電話';
                $platform_ver = 'DoCoMo/' . $engine_ver;
                $device
                    = ( $user_agent =~ /([0-9a-zA-Z_\-\.\,]+)/ )
                    ? $1
                    : $platform_ver;

            }
            elsif (
                $engine =~ /(KDDI-?([^\x09\x0A\x0C\x0D\x20]*)|UP\.Browser)/i )
            {
                $platform     = '携帯電話';
                $platform_ver = 'au';
                $device       = $2 || 'au';

            }
            else {
                ( $browser, $browser_ver ) = ( $engine, $engine_ver );
                $browser_ver = $1
                    if $user_agent
                    =~ s/[\x09\x0A\x0C\x0D\x20]*Version\/([0-9\.abr]+)//;
                $platform = $info[0] if !defined $platform;
                $platform_ver = $info[1] || $info[0]
                    if !defined $platform_ver;
                if ( $browser eq 'Xiino' || $browser eq 'Palmscape' ) {
                    $platform     = 'その他';
                    $platform_ver = 'PalmOS';
                }
                elsif ( $platform_ver !~ /$platform/ ) {
                    $platform_ver = $platform . ' ' . $platform_ver;
                }
            }
        }

        # プラットフォーム補完
        $platform     = '' if !defined $platform;
        $platform_ver = '' if !defined $platform_ver;
        $browser      = '' if !defined $browser;

        if ( $platform =~ /^Win/i || !$platform_ver ) {
            $platform_ver = $platform
                if !defined $platform_ver
                || $platform_ver eq ''
                || $platform =~ /[0-9]/;
            if ( $platform_ver =~ /Win 9x/i ) {
                $platform_ver = 'Windows Me';
            }
            else {
                $platform_ver
                    =~ s/^Win(?!d)([a-zA-Z]*)[\x09\x0A\x0C\x0D\x20]*([0-9\.abr]*)$/'Windows' . (($1) ? ' ' . $1 : '') . (($2) ? ' ' . $2 : '')/ie;
            }
        }
        if ( $platform_ver =~ /^Windows Phone/i ) {
            $platform = '携帯電話';
            my $os = ( $platform_ver =~ /([0-9]+\.?[0-9]*)$/ ) ? $1 : '';
            $device = $info[$#info] . ( ($os) ? ' (' . $os . ')' : '' );
            $device =~ s/^.*?-// if $os =~ /^[0-6](?:\.|$)/;
            $platform_ver = 'Windows Phone';
        }
        elsif ( grep( $_ =~ /Windows Phone/i, @info ) ) {
            $platform     = '携帯電話';
            $platform_ver = 'Windows Phone';
            my $os = ( grep( $_ =~ /Windows Phone/i, @info ) )[0];

            #$os = ($os =~ /([0-9ab\.]+)$/) ? $1 : '';
            $os = ( $os =~ /([0-9]+\.?[0-9]*)$/ ) ? $1 : '';
            $device
                = $info[ ( $#info - 1 ) ] . ( ($os) ? ' (' . $os . ')' : '' );
            $device =~ s/^.*?-// if $os =~ /^[0-6](?:\.|$)/;
        }
        elsif ( grep( $_ =~ /Xbox One/i, @info ) ) {
            $platform = 'Xbox One';
        }
        elsif ( grep( $_ =~ /Xbox/i, @info ) ) {
            $platform = 'Xbox 360';
        }
        elsif ( $platform_ver =~ /^Win/i ) {
            $platform = 'Windows';
        }
        elsif ( $platform_ver =~ /^X[0-9]+$/i || $platform_ver eq 'FreeBSD' )
        {
            $platform = 'UNIX';
        }
        elsif ( $platform_ver
            =~ /^(?:PLAYSTATION[\x09\x0A\x0C\x0D\x20]*|PS)([0-9]*)\s*([0-9\.abr]*)/i
            )
        {
            $browser     = 'PlayStation ' . $1;
            $browser_ver = $2;
            $platform    = 'その他';
            $device      = $platform_ver = $browser;
        }
        elsif ( $browser =~ /DreamPassport/i ) {
            $platform = 'Dreamcast';
        }
        elsif ( $platform_ver =~ /^Win/i || $platform =~ /^Win/ ) {
            $platform = 'Windows';
        }
        else {
            $platform = ( defined $platform
                    && $platform =~ /^([^\x09\x0A\x0C\x0D\x20]+)/ ) ? $1 : '';
        }
        if ( $platform =~ /^Mac/i || $platform_ver =~ /^Mac/i ) {
            $platform = 'Macintosh';
            if ( $platform_ver =~ /X/i || $browser =~ /OmniWeb/i ) {
                $platform_ver = 'MacOS X';
            }
            elsif ($platform_ver =~ /(PPC|PowerPC)/i
                || $platform_ver =~ /^Macintosh$/ )
            {
                $platform_ver = 'Macintosh(PPC)';
            }
            elsif ( $platform_ver =~ /(68K|68000)/i ) {
                $platform_ver = 'Macintosh(68K)';
            }
        }
        elsif ( $platform_ver eq 'Windows NT 5.0'
            || ( $platform =~ /^Win/i && $platform_ver =~ /2000/ ) )
        {
            $platform_ver = 'Windows 2000';
        }
        elsif ( $platform_ver eq 'Windows NT 5.1' ) {
            $platform_ver = 'Windows XP';
        }
        elsif ( $platform_ver eq 'Windows NT 5.2' ) {
            $platform_ver = 'Windows Server 2003';
        }
        elsif ( $platform_ver eq 'Windows NT 6.0' ) {
            $platform_ver = 'Windows Vista';
        }
        elsif ( $platform_ver eq 'Windows NT 6.1' ) {
            $platform_ver = 'Windows 7';
        }
        elsif ( $platform_ver eq 'Windows NT 6.2' ) {
            $platform_ver = 'Windows 8';
        }
        elsif ( $platform_ver eq 'Windows NT 6.3' ) {
            $platform_ver = 'Windows 8.1';
        }
        elsif ( $platform_ver eq 'Windows NT 10.0' ) {
            $platform_ver = 'Windows 10';
        }
        elsif ( $platform_ver eq 'Windows ME' ) {
            $platform_ver = 'Windows Me';
        }
        elsif ( $platform =~ /^X[0-9]+$/i ) {
            $platform = 'UNIX';
            $platform_ver =~ s/^X[0-9]+[\x09\x0A\x0C\x0D\x20]+//i;
            $platform_ver = ( defined $platform_ver
                    && $platform_ver =~ /^([0-9a-zA-Z_\-]+)/ ) ? $1 : '';
        }

        # 特殊なプラットフォーム
        if (  !$platform
            && $user_agent
            =~ s/^[\x09\x0A\x0C\x0D\x20]*([^\/]+)\/([0-9\.abr]+)[\x09\x0A\x0C\x0D\x20]*//
            )
        {
            ( $platform, $platform_ver ) = ( $1, $1 . ' ' . $2 );

            #AVE-Front
            if ( $platform eq 'AVE-Front' ) {
                ( $browser, $browser_ver ) = ( $platform, $platform_ver );
                $platform = 'その他';

                if ( defined $info[0]
                    && $info[0] =~ s/^BrowserInfo[\x09\x0A\x0C\x0D\x20]+//i )
                {
                    foreach (@info) {
                        if ( $_ =~ /^Product=(.*)$/i ) {
                            $platform_ver = $1;
                            last;
                        }
                    }
                }
                else {
                    $platform_ver = 'ワープロ';
                }
            }
        }
        if (!$platform
            || (   $platform ne 'Windows'
                && $platform ne 'Macintosh'
                && $platform ne 'UNIX'
                && $platform ne '携帯電話'
                && $platform ne 'その他' )
            )
        {
            if ( $platform_ver =~ /like Mac OS X/i ) {

                #my $os_ver = ($platform_ver =~ / OS ([0-9_]+)/i) ? $1 : 1;
                my $os_ver
                    = ( $platform_ver =~ / OS ([0-9]+_?[0-9]*)/i ) ? $1 : 1;
                $os_ver =~ s/_/./g;
                $device = $platform_ver = $platform;
                $device .= ' (' . $os_ver . ')';
            }
            elsif ( $platform_ver =~ /Android/i ) {

#my $os_ver = ($platform_ver =~ /Android\s+([0-9\.]+)/i) ? ' (' . $1 . ')' : '';
                my $os_ver
                    = ( $platform_ver =~ /Android\s+([0-9]+\.?[0-9]*)/i )
                    ? ' (' . $1 . ')'
                    : '';
                $platform_ver
                    = 'Android'
                    . ( ( $user_agent && $user_agent =~ / Mobile/ )
                    ? ' 携帯'
                    : ' タブレット' );
                $device
                    = ( $info[$#info]
                        =~ /^[\x09\x0A\x0C\x0D\x20]*(.*?)[\x09\x0A\x0C\x0D\x20]+Build\/\S+$/
                    ) ? $1 . $os_ver : ''
                    if !$device;
            }
            elsif ( $platform && $platform ne $platform_ver ) {
                $platform_ver = $platform . '(' . $platform_ver . ')';
            }
            $platform = 'その他';
        }
        $platform_ver = 'デジタルテレビ' if $platform_ver eq 'DTV';

        # 携帯
    }
    else {

        # 携帯
        $platform = '携帯電話';

        # プラットフォーム情報
        my @info = split( /\//, $user_agent );

        # プラットフォーム情報
        # Wget
        if ( defined $info[0] && $info[0] =~ /wget/i ) {
            $platform = 'UNIX';
            $browser  = 'Wget';
            $browser_ver
                = ( $info[1] && $info[1] =~ s/^([0-9\.]+)\s*// ) ? $1 : '';
            $platform_ver
                = ( $info[1] && $info[1] =~ /Red Hat/i )
                ? 'Red Hat Linux'
                : $info[1];

            # KDDI
        }
        elsif ( defined $info[0] && $info[0] eq 'UP.Browser' ) {
            $platform_ver = 'au';

            # その他
        }
        else {
            $info[0] = 'SoftBank'
                if $info[0] eq 'J-PHONE' || $info[0] eq 'Vodafone';
            $platform_ver = $info[0] . '/' . $info[1];
            $device       = $info[2] if $info[2];
            $limitsize    = $1 if $info[3] && $info[3] =~ /^c([0-9]+)$/;
        }
    }

    $platform     = 'その他' if !$platform;
    $platform_ver = $ua         if !$platform_ver;
    $browser      = ''          if !defined $browser;
    $browser_ver  = ''          if !defined $browser_ver;
    $browser_ver  = $1          if $browser_ver =~ /^([0-9]+\.[0-9]+)/;
    $limitsize    = ''          if !defined $limitsize;
    $device       = ''          if !defined $device;
    $lang         = ''          if !defined $lang;
    return ( $platform, $platform_ver, $browser, $browser_ver, $limitsize,
        $device, $lang );
}

1;
