package CalDayView;
require CMONOS::Calendar::CalCommon;
@ISA = qw(CalCommon);
use strict;
use utf8;
######################################################################
# カレンダー週日表示共通モジュール
# CalDayView.pm ver1.0.9
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-08-07
######################################################################
sub print_events ($) {
    my $this   = shift;
    my $today  = $this->time_to_dtstamp( time, 1 );
    my $dcount = 0;
    my @wclass = (
        'calsunday',   'calmonday', 'caltuesday', 'calwednesday',
        'calthursday', 'calfriday', 'calsaturday'
    );
    my @wname
        = (    $setting::page{$setting::key}{'wdayFormat'}
            && $setting::page{$setting::key}{'wdayFormat'} eq 'e' )
        ? ( 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' )
        : (    $setting::page{$setting::key}{'wdayFormat'}
            && $setting::page{$setting::key}{'wdayFormat'} eq 'E' )
        ? (
        'Sunday',   'Monday', 'Tuesday', 'Wednesday',
        'Thursday', 'Friday', 'Saturday'
        )
        : (    $setting::page{$setting::key}{'wdayFormat'}
            && $setting::page{$setting::key}{'wdayFormat'} eq 'J' )
        ? (
        '日曜日', '月曜日', '火曜日', '水曜日',
        '木曜日', '金曜日', '土曜日'
        )
        : ( '日', '月', '火', '水', '木', '金', '土' );
    my @reserved  = ();
    my $day_limit = $this->day_limit();
    my ( $target_y, $target_m )
        = ( $main::date =~ /^([0-9]{4})([0-9]{2})/ )
        ? ( $1 + 0, $2 + 0 )
        : ( 0, 0 );
    my $query = $this->amp_escape(
        $this->set_query( 'url', [ 'and', 'search' ], 1 ) );
    my $calendar_type = $this->calendar_type;
    my $wname_omitted = ( $setting::page{$setting::key}{'weekRow'}
            && $calendar_type eq 'monthly' ) ? 1 : 0;
    print '<!--m:config skippable+="daytable"-->' if !$this->need_day_events;
    print '<div id="calendar-body" class="' . $calendar_type . '-calendar">';
    print '<table id="daytable">';
    $this->print_table_header( \@wclass, \@wname );
    print '<tbody>';
    print '<tr>';

    foreach my $date ( sort { $a <=> $b } keys %{ $this->{'date'} } ) {
        if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
            no warnings qw(numeric);
            my $isfirst = 0;
            if ( $dcount && $dcount % $day_limit == 0 ) {
                $this->print_event_separator();
                $isfirst = 1;
            }
            my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
            my $w = $this->{'date'}{$date}{'wday'};
            print '<td class="'
                . $wclass[$w]
                . (
                ( $this->{'date'}{$date}{'holiday'} ) ? ' calholiday' : '' )
                . ( ( $date == $today ) ? ' caltoday' : '' )
                . (
                ( $target_y != $y || $target_m != $m ) ? ' calextra' : '' )
                . '"';
            print ' ondblclick="cmonos.window.open(\'detailViewer\','
                . ( ( $setting::page{'detailWindowWidth'} )
                ? $setting::page{'detailWindowWidth'}
                : '640' )
                . ','
                . ( ( $setting::page{'detailWindowHeight'} )
                ? $setting::page{'detailWindowHeight'}
                : '480' )
                . ',\''
                . $cgi::path
                . '?m=Editor'
                . (
                ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c
                : '' )
                . '&amp;v='
                . $main::v
                . '&amp;dtstart='
                . $date . '\');"'
                if $this->is_secure('input');
            print ' title="' . $this->{'date'}{$date}{'holiday'} . '"'
                if $this->{'date'}{$date}{'holiday'};
            print '>';

            print '<div class="cal-date">';
            print '<a href="'
                . $cgi::path
                . '?m=Viewer&amp;v=d'
                . ( ( $default::subdivided && $main::c )
                ? '&amp;c=' . $main::c
                : '' )
                . '&amp;date='
                . $date
                . $query . '">';
            print $m . '/' if !$setting::page{$setting::key}{'monOmitted'};
            print $d;
            print '(' . $wname[$w] . ')' if !$wname_omitted;
            print '</a>';
            $this->print_edit_button($date);
            print '</div>';
            print '<div class="day-events">' if $this->need_day_events;

            my @new_events = sort {
                $this->{'date'}{$date}{'date'}{$b}{'prev'}
                    <=> $this->{'date'}{$date}{'date'}{$a}{'prev'}
                    || $this->{'date'}{$date}{'date'}{$b}{'next'}
                    <=> $this->{'date'}{$date}{'date'}{$a}{'next'}
                    || $a <=> $b
            } keys %{ $this->{'date'}{$date}{'date'} };
            if (@new_events) {
                my @events = ();
                if ( $day_limit > 1 && @reserved ) {
                    my @temp;
                    foreach my $n (@new_events) {
                        push( @temp, $n )
                            if !grep( defined $_ && $_ eq $n, @reserved );
                    }
                    @new_events = @temp;
                    for ( my $i = 0; $i < @reserved; $i++ ) {
                        ;
                        if ( $reserved[$i] ) {
                            push( @events, $reserved[$i] );
                        }
                        else {
                            my $e = shift(@new_events);
                            push( @events, ( ($e) ? $e : '' ) );
                        }
                    }
                }
                push( @events, @new_events );
                my @new_reserved = ();
                my $i            = 0;
                foreach my $n (@events) {
                    my $this_event = $this->{'date'}{$date}{'date'}{$n};
                    my $microdata_disabled = (
                               $n
                            && $day_limit > 1
                            && ( grep( defined $_ && $n eq $_, @reserved )
                            || $this_event->{'hidden'} )
                    ) ? 1 : 0;
                    print '<div class="day-event-box'
                        . ( ( $n && $this->{'event'}{$n}{'sellout'} )
                        ? ' event-sellout'
                        : ( $n && $this->{'event'}{$n}{'remaining'} )
                        ? ' event-remaining'
                        : '' )
                        . '"'
                        . ( ( !$microdata_disabled )
                        ? ' itemscope="itemscope" itemtype="http://data-vocabulary.org/Event"'
                        : '' )
                        . '>';
                    if ($n) {
                        my $title
                            = ( defined $this->{'event'}{$n}{'title'}
                                && $this->{'event'}{$n}{'title'} ne '' )
                            ? $this->{'event'}{$n}{'title'}
                            : ($default::english) ? 'EVENT'
                            :                       '予定';
                        print '<a';
                        print ' href="'
                            . $cgi::path
                            . '?m=DetailViewer'
                            . ( ( $default::subdivided && $main::c )
                            ? '&amp;c=' . $main::c
                            : '' )
                            . '&amp;n='
                            . $n
                            . '&amp;v='
                            . $main::v
                            . $query . '"';
                        if ( !$setting::page{'detailInSameWindow'} ) {
                            print ' target="detailViewer"';
                            print ' onclick="'
                                . $this->detail_viewer_opener . '"';
                        }
                        print ' class="';
                        if ( $day_limit > 1 ) {
                            my $day = 1;
                            my $span
                                = ( defined $this_event->{'span'} )
                                ? $this_event->{'span'}
                                : $this_event->{'next'};
                            if ($span) {
                                $new_reserved[$i] = $n;
                                $day += $span;
                            }
                            if ((   !$isfirst
                                    && grep( defined $_ && $n eq $_,
                                        @reserved )
                                )
                                || $this_event->{'hidden'}
                                )
                            {
                                print 'below-event';
                            }
                            else {
                                my $max
                                    = $day_limit - ( $dcount % $day_limit );
                                my $eday = ( $max < $day ) ? $max : $day;
                                print 'day-event-bar event-'
                                    . ( ( $eday == 1 )
                                    ? $eday . 'day'
                                    : $eday . 'days' );
                                print ' event-start'
                                    if !$this_event->{'prev'};
                                print ' event-end'
                                    if $day <= $max
                                    && ( !$this_event->{'next'}
                                    || $span == $this_event->{'next'} );
                                print ' event-category-'
                                    . ( ( $this->{'event'}{$n}{'category'} )
                                    ? $this->{'event'}{$n}{'category'}
                                    : '1' );
                            }
                        }
                        else {
                            print 'day-event-bar event-1day';
                            print ' event-start' if !$this_event->{'prev'};
                            print ' event-end'   if !$this_event->{'next'};
                            print ' event-category-'
                                . ( ( $this->{'event'}{$n}{'category'} )
                                ? $this->{'event'}{$n}{'category'}
                                : '1' );
                        }
                        print '" title="';
                        print $this->get_span($n);
                        print ' ' . $title if $title;
                        print '"';
                        print ' itemprop="url"' if !$microdata_disabled;
                        print '>';
                        print '<del>' if $this->{'event'}{$n}{'sellout'};
                        print '<span itemprop="summary">'
                            if !$microdata_disabled;
                        print $title;
                        print '</span>' if !$microdata_disabled;
                        print '</del>'  if $this->{'event'}{$n}{'sellout'};
                        print '</a>';

                        if ( !$microdata_disabled ) {
                            print '<meta itemprop="startDate" content="'
                                . $1 . '-'
                                . $2 . '-'
                                . $3 . '" />'
                                if $this->{'event'}{$n}{'start_date'}
                                && $this->{'event'}{$n}{'start_date'}
                                =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/;
                            print '<meta itemprop="endDate" content="'
                                . $1 . '-'
                                . $2 . '-'
                                . $3 . '" />'
                                if $this->{'event'}{$n}{'end_date'}
                                && $this->get_prev_date(
                                $this->{'event'}{$n}{'end_date'} )
                                =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/;
                        }
                    }
                    print '</div>';
                    $i++;
                }
                @reserved = @new_reserved;
            }
            $this->print_hourly_events_list( $date, $query );
            print '</div>' if $this->need_day_events;
            print '</td>';
            $dcount++;
        }
    }
    print '</tr>';
    print '</tbody>';
    print '</table>';
    $this->print_hourly_events( $today, $query );
    print '</div>';
}

sub print_edit_button ($$;$) {
    my ( $this, $date, $h ) = @_;
    if ( $this->is_secure('input') ) {
        $this->use_icon('calendar') if !$cgi::icon;
        print $cgi::icon->button(
            'eventadd',
            $setting::page{$setting::key}{'eventadd'},
            $cgi::path
                . '?m=Editor&amp;k=editor'
                . (
                ( $default::subdivided && $main::c )
                ? '&amp;c=' . $main::c
                : ''
                )
                . '&amp;v='
                . $main::v
                . '&amp;dtstart='
                . $date
                . (
                ( defined $h ) ? 'T' . sprintf( "%.2d", $h ) . '0000' : ''
                ),
            ' +追加 ',
            'detailViewer',
            { 'onclick' => $this->detail_viewer_opener }
        );
    }
}

sub print_table_header ($) {
    my $this = shift;
}

sub print_hourly_events_list ($$;$) {
    my $this = shift;
}

sub print_hourly_events ($$) {
    my ( $this, $today, $query ) = @_;
    my @dates = sort { $a <=> $b } keys %{ $this->{'date'} };
    my @wclass = (
        'calsunday',   'calmonday', 'caltuesday', 'calwednesday',
        'calthursday', 'calfriday', 'calsaturday'
    );
    my @wname = ( '日', '月', '火', '水', '木', '金', '土' );
    my @time_scale = (
        '0000', '0030', '0100', '0130', '0200', '0230', '0300', '0330',
        '0400', '0430', '0500', '0530', '0600', '0630', '0700', '0730',
        '0800', '0830', '0900', '0930', '1000', '1030', '1100', '1130',
        '1200', '1230', '1300', '1330', '1400', '1430', '1500', '1530',
        '1600', '1630', '1700', '1730', '1800', '1830', '1900', '1930',
        '2000', '2030', '2100', '2130', '2200', '2230', '2300', '2330'
    );
    my $now_h = ( localtime(time) )[2];
    print '<div id="timetable-box">';
    print '<table id="timetable-grid">';
    print '<thead>';
    print '<tr><th>時</th>';
    my $dcount = 0;

    foreach my $date (@dates) {
        if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
            my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
            my $w = $this->{'date'}{$date}{'wday'};
            print '<th class="'
                . $wclass[$w]
                . (
                ( $this->{'date'}{$date}{'holiday'} ) ? ' calholiday' : '' )
                . ( ( $date == $today ) ? ' caltoday' : '' ) . '"'
                . ( ( $this->{'date'}{$date}{'holiday'} )
                ? ' title="' . $this->{'date'}{$date}{'holiday'} . '"'
                : '' )
                . '>';
            print '<a href="'
                . $cgi::path
                . '?m=Viewer&amp;v=d'
                . ( ( $default::subdivided && $main::c )
                ? '&amp;c=' . $main::c
                : '' )
                . '&amp;date='
                . $date
                . $query . '">';
            print $m . '/' . $d . '(' . $wname[$w] . ')';
            print '</a>';
            $this->print_edit_button( $date, $now_h );
            print '</th>';
        }
        $dcount++;
    }
    print '</tr>';
    print '</thead>';
    print '<tbody>';
    print '<tr>';
    print '<td class="time-scale"';
    print ' title="ダブルクリックで入力欄が開きます→"'
        if $this->is_secure('input');
    print '>';

    foreach my $h (@time_scale) {
        my ( $hr, $min )
            = ( $h =~ /^([0-9]{2})([0-9]{2})$/ )
            ? ( $1 + 0, $2 )
            : ( '00', '00' );
        print '<div class="hourly-list '
            . ( ( $min eq '30' ) ? 'after-half' : 'before-half' ) . ' hm'
            . $h . '">';
        print $hr . ':' . $min;
        print '</div>';
    }
    print '</td>';
    my $tz = ( $ENV{'TZ'} && $ENV{'TZ'} =~ /([+\-][0-9]+)$/ ) ? $1 * -1 : 0;
    $tz = ($tz) ? sprintf( "%+.2d", $tz ) . ':00' : 'Z';
    foreach my $date (@dates) {
        no warnings qw(numeric);
        print '<td class="'
            . $wclass[ $this->{'date'}{$date}{'wday'} ]
            . ( ( $this->{'date'}{$date}{'holiday'} ) ? ' calholiday' : '' )
            . ( ( $date == $today ) ? ' caltoday' : '' ) . '">';
        print '<div class="daybox">';
        my $ymd
            = ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ )
            ? $1 . '-' . $2 . '-' . $3
            : '';

        foreach my $h (@time_scale) {
            my ( $hr, $min )
                = ( $h =~ /^([0-9]{2})([0-9]{2})$/ )
                ? ( $1 + 0, $2 )
                : ( '00', '00' );
            print '<div class="hourly-list '
                . ( ( $min eq '30' ) ? 'after-half' : 'before-half' ) . ' hm'
                . $h . '"';
            print ' ondblclick="cmonos.window.open(\'detailViewer\','
                . ( ( $setting::page{'detailWindowWidth'} )
                ? $setting::page{'detailWindowWidth'}
                : '640' )
                . ','
                . ( ( $setting::page{'detailWindowHeight'} )
                ? $setting::page{'detailWindowHeight'}
                : '480' )
                . ',\''
                . $cgi::path
                . '?m=Editor'
                . (
                ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c
                : '' )
                . '&amp;v='
                . $main::v
                . '&amp;dtstart='
                . $date . 'T'
                . $h
                . '00\');"'
                if $this->is_secure('input');
            print '>';
            if (   ref $this->{'date'}
                && ref $this->{'date'}{$date}
                && ref $this->{'date'}{$date}{'time'}
                && ref $this->{'date'}{$date}{'time'}{$h} )
            {
                foreach my $n (
                    sort {
                        $this->{'event'}{$a}{'start_date'}
                            <=> $this->{'event'}{$b}{'start_date'}
                            || $this->{'event'}{$a}{'start_time'}
                            <=> $this->{'event'}{$b}{'start_time'}
                            || $a <=> $b
                    } keys %{ $this->{'date'}{$date}{'time'}{$h} }
                    )
                {
                    my $this_event = $this->{'date'}{$date}{'time'}{$h}{$n};
                    next if !defined $this_event->{'start_min'};
                    next
                        if $this_event->{'prev'}
                        && !$this_event->{'next'}
                        && !$this_event->{'end_time'}
                        && !$this->{'event'}{$n}{'end_time'};
                    my %order   = ();
                    my %overlap = ();
                    my $overlap = 0;
                    foreach my $hr ( sort { $a <=> $b }
                        keys %{ $this->{'date'}{$date}{'time'} } )
                    {

                        foreach my $num (
                            sort {
                                $this->{'date'}{$date}{'time'}{$hr}{$a}
                                    {'start_min'}
                                    <=> $this->{'date'}{$date}{'time'}{$hr}
                                    {$b}{'start_min'}
                            } keys %{ $this->{'date'}{$date}{'time'}{$hr} }
                            )
                        {
                            next if $n eq $num;
                            my $other_event
                                = $this->{'date'}{$date}{'time'}{$hr}{$num};
                            if ((   $other_event->{'start_min'}
                                    <= $this_event->{'start_min'}
                                    && $other_event->{'end_min'}
                                    > $this_event->{'start_min'}
                                )
                                || ( $other_event->{'start_min'}
                                    > $this_event->{'start_min'}
                                    && $other_event->{'start_min'}
                                    < $this_event->{'end_min'} )
                                )
                            {
                                $order{ $other_event->{'order'} } = 1
                                    if $other_event->{'order'};
                                foreach my $o (
                                    sort { $overlap{$a} <=> $overlap{$b} }
                                    keys %overlap )
                                {
                                    if ( $overlap{$o}
                                        <= $other_event->{'start_min'} )
                                    {
                                        delete $overlap{$o};
                                    }
                                    else {
                                        last;
                                    }
                                }
                                $overlap{$num} = $other_event->{'end_min'};
                                my $c = scalar( keys %overlap );
                                $overlap = $c if $overlap < $c;
                            }
                        }
                    }
                    foreach my $o ( 1 .. 8 ) {
                        if ( !$order{$o} ) {
                            $this_event->{'order'} = $o;
                            last;
                        }
                    }
                    $this_event->{'order'} = 8 if !$this_event->{'order'};
                    $overlap++;
                    $overlap = 8 if $overlap > 8;
                    my $height = $this_event->{'end_min'}
                        - $this_event->{'start_min'} - 2;
                    $height = 12 if $height < 12;
                    print '<div class="'
                        . (
                        ( $this->{'event'}{$n}{'sellout'} ) ? 'event-sellout'
                        : 'event-remaining' )
                        . '" onclick="cmonos.tool.calendar.moveToFront(this)" style="position: absolute; '
                        . ( ( $this_event->{'start_min'} )
                        ? 'top: ' . $this_event->{'start_min'} . 'px;'
                        : '' )
                        . ' height: '
                        . ( $height + 2 ) . 'px;">'
                        if $this->{'event'}{$n}{'sellout'}
                        || $this->{'event'}{$n}{'remaining'};
                    print '<div class="event-bar bar-order-'
                        . $this_event->{'order'}
                        . ' bar-overlap-'
                        . $overlap;
                    print ' event-start'
                        if !$this_event->{'prev'}
                        && !$this_event->{'prev_time'};
                    print ' event-end'
                        if !$this_event->{'next_time'}
                        && (
                        !$this_event->{'next'}
                        || ( $this->{'event'}{$n}{'end_time'} == 0
                            && !$this->{'date'}
                            { $this->get_next_date($date) }{'time'}{'0000'}
                            {$n}->{'end_min'} )
                        );
                    print ' event-category-'
                        . ( ( $this->{'event'}{$n}{'category'} )
                        ? $this->{'event'}{$n}{'category'}
                        : '1' );
                    print ' event-id-' . $n;
                    print '" style="'
                        . (
                        (          !$this->{'event'}{$n}{'sellout'}
                                && !$this->{'event'}{$n}{'remaining'}
                                && $this_event->{'start_min'}
                        ) ? 'top: ' . $this_event->{'start_min'} . 'px;' : ''
                        )
                        . ' height: '
                        . $height
                        . 'px; _height /**/: '
                        . ( $height + 2 ) . 'px;"';
                    print ' onclick="cmonos.tool.calendar.moveToFront(this)"'
                        if !$this->{'event'}{$n}{'sellout'}
                        && !$this->{'event'}{$n}{'remaining'};
                    print ' title="'
                        . $this->get_span($n)
                        . (
                        (   defined $this->{'event'}{$n}{'title'}
                                && $this->{'event'}{$n}{'title'} ne ''
                        ) ? ' ' . $this->{'event'}{$n}{'title'} : ''
                        ) . '"';
                    print
                        ' itemscope="itemscope" itemtype="http://data-vocabulary.org/Event">';

                    if ( $height >= 24 ) {
                        print '<div class="time-info">';
                        print '<time itemprop="startDate" datetime="'
                            . $ymd . 'T'
                            . (
                            (          $this_event->{'start_time'}
                                    && $this_event->{'start_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/
                            )                           ? $1 . ':' . $2
                            : ( $this_event->{'prev'} ) ? '0:00'
                            : ( $this->{'event'}{$n}{'start_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/ )
                            ? $1 . ':' . $2
                            : ''
                            )
                            . $tz . '">'
                            . (
                            (          $this_event->{'start_time'}
                                    && $this_event->{'start_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/
                            ) ? ( $1 + 0 ) . ':' . $2
                            : ( $this_event->{'prev'} ) ? '0:00'
                            : ( $this->{'event'}{$n}{'start_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/ )
                            ? ( $1 + 0 ) . ':' . $2
                            : ''
                            ) . '</time>';
                        print '-'
                            . '<time itemprop="endDate" datetime="'
                            . $ymd . 'T'
                            . (
                            (          $this_event->{'end_time'}
                                    && $this_event->{'end_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/
                            )                           ? $1 . ':' . $2
                            : ( $this_event->{'next'} ) ? '24:00'
                            : ( $this->{'event'}{$n}{'end_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/ )
                            ? $1 . ':' . $2
                            : ''
                            )
                            . $tz . '">'
                            . (
                            (          $this_event->{'end_time'}
                                    && $this_event->{'end_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/
                            ) ? ( $1 + 0 ) . ':' . $2
                            : ( $this_event->{'next'} ) ? '24:00'
                            : ( $this->{'event'}{$n}{'end_time'}
                                    =~ /^([0-9]{2})([0-9]{2})/ )
                            ? ( $1 + 0 ) . ':' . $2
                            : ''
                            ) . '</time>';
                        print '</div>';
                    }
                    print '<div class="event-title">';
                    print '<a href="'
                        . $cgi::path
                        . '?m=DetailViewer'
                        . ( ( $default::subdivided && $main::c )
                        ? '&amp;c=' . $main::c
                        : '' )
                        . '&amp;n='
                        . ( ( $n =~ /^([0-9]+)\.[0-9]+$/ ) ? $1 : $n )
                        . '&amp;v='
                        . $main::v
                        . $query . '"';
                    if ( !$setting::page{'detailInSameWindow'} ) {
                        print ' target="detailViewer"';
                        print ' onclick="'
                            . $this->detail_viewer_opener . '"';
                    }
                    print ' itemprop="url">';
                    print '<del>' if $this->{'event'}{$n}{'sellout'};
                    print '<span itemprop="summary">'
                        . (
                        (   defined $this->{'event'}{$n}{'title'}
                                && $this->{'event'}{$n}{'title'} ne ''
                        )                     ? $this->{'event'}{$n}{'title'}
                        : ($default::english) ? 'EVENT'
                        :                       '予定'
                        ) . '</span>';
                    print '</del>' if $this->{'event'}{$n}{'sellout'};
                    print '</a>';
                    print $this->get_tool_buttons($n);
                    print '</div>';
                    print '</div>';
                    print '</div>'
                        if $this->{'event'}{$n}{'sellout'}
                        || $this->{'event'}{$n}{'remaining'};
                }
            }
            print '</div>';
        }

        print '</div>';
        print '</td>';
    }
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</div>';
}

sub get_span ($$) {
    my ( $this, $n ) = @_;
    return $this->get_event_date_info(
        $this->{'event'}{$n}{'start_date'}
            . ( $this->{'event'}{$n}{'start_time'} || '' ),
        $this->{'event'}{$n}{'end_date'}
            . ( $this->{'event'}{$n}{'end_time'} || '' ),
        $this->{'event'}{$n}{'rrule'},
        1
    );
}

sub print_event_separator ($) {
    my $this = shift;
}

sub have_timetable ($) {
    my $this = shift;
    return 0;
}

sub need_day_events ($) {
    my $this = shift;
    return ( !$setting::general{'dateDisabled'} ) ? 1 : 0;
}

1;
