package DetailViewer;
require CMONOS::Calendar::CalCommon;
@ISA = qw(CalCommon);
use strict;
use utf8;
######################################################################
# 詳細ビューワモジュール
# DetailViewer.pm ver1.2.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-06-27
######################################################################
# メイン
sub run ($) {
    my $this = shift;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    $this->error404 if $this->is_nan($main::n) || !-d $default::db;

    #---------------------------------------------------------------------
    #保留中
    if (   $setting::general{'pendingTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        my ( %ctime, $is_pending );
        tie( %ctime, $env::dbm_file, "$default::db/ctime", O_RDONLY,
            $env::dbm_mode );
        if ( ref tied %ctime ) {
            $is_pending = 1
                if $ctime{ $this->encode_utf8($main::n) }
                && $ctime{ $this->encode_utf8($main::n) }
                > time - $setting::general{'pendingTime'};
        }
        untie %ctime;
        $this->error404 if $is_pending;
    }

    #---------------------------------------------------------------------
    #要求チェック
    my ( %mtime, $last_modified );
    tie( %mtime, $env::dbm_file, "$default::db/mtime", O_RDONLY,
        $env::dbm_mode );
    if ( ref tied %mtime ) {
        $last_modified
            = $this->decode_utf8( $mtime{ $this->encode_utf8($main::n) } )
            if $mtime{ $this->encode_utf8($main::n) };
    }
    untie %mtime;
    $this->error404 if !$last_modified;

    #---------------------------------------------------------------------
    #イベント基本情報
    my ( $event_dt, $dt_start );
    ( $event_dt, $this->{'target_date'}, $dt_start )
        = $this->get_event_date($main::n);

    #---------------------------------------------------------------------
    #表示可能
    if (   $setting::general{'previewTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        use Time::Local;
        my ( $y, $m, $d, $h, $min ) = $this->timestamp_to_array($dt_start);
        $this->error404
            if time + $setting::general{'previewTime'}
            < timelocal( 0, $min, $h, $d, ( $m - 1 ), $y );
    }

    #---------------------------------------------------------------------
    #アクセス権チェック
    if (!$this->is_secure( 'detail', 1 )
        || (   defined $setting::security{'detail'}{'passwordCheck'}
            && $setting::security{'detail'}{'passwordCheck'} eq 'member'
            && !$this->is_staff
            && !$this->is_guest )
        )
    {
        $this->print_password_check_form($setting::key);
        return;
    }

    #カテゴリーチェック
    my $category = $this->get_db_data( 'catetory', $main::n );
    if ((     !$setting::security{'closed'}{'passwordCheck'}
            || $setting::security{'closed'}{'passwordCheck'} ne 'nocheck'
        )
        && %setting::event_category
        && ref $setting::event_category{'ORDER'}
        && %{ $setting::event_category{'ORDER'} }
        && defined $category
        && ref $setting::event_category{$category}
        && $setting::event_category{$category}{'closed'}
        && !$this->is_secure('closed')
        && !$this->is_guest(1)
        )
    {
        $this->print_password_check_form($setting::key);
        return;
    }

    #---------------------------------------------------------------------
    #特殊フィールド処理
    if (   defined $main::id
        && $main::id ne ''
        && ref $field::obj{$main::id}
        && $field::obj{$main::id}->update_enabled
        && $field::obj{$main::id}->is_secure_user_for_update
        && $field::obj{$main::id}->update )
    {
        $this->set_onload('cmonos.window.refreshOpener(\'\',false)');
    }

    #---------------------------------------------------------------------
    #書き出し
    my ($have_template);

    #ファイル格納ディレクトリ
    my $path = "$default::file/$main::n";

    #テンプレート
    if ( defined $setting::page{$setting::key}{'template'}
        && $setting::page{$setting::key}{'template'} ne '' )
    {
        $have_template = 1;
        $have_template++
            if $setting::page{$setting::key}{'template'}
            =~ /<body(?![0-9a-zA-Z_])[^>]*>/i;
    }

    #タイトル抽出
    my $title_id = $this->get_title_id;
    my $title    = '';
    if ( ref $field::obj{$title_id} ) {
        $title = $field::obj{$title_id}->usual_format(
            $this->get_db_data( $field::obj{$title_id}->db, $main::n ) );
        $this->push_window_title($title);
    }
    if ( $field::for{'content'} && ref $field::obj{ $field::for{'content'} } )
    {
        $this->set_description(
            $field::obj{ $field::for{'content'} }->usual_format(
                $this->get_db_data(
                    $field::obj{ $field::for{'content'} }->db, $main::n
                )
            )
        );
    }
    my $image_url;
    if ( $field::for{'media'} && ref $field::obj{ $field::for{'media'} } ) {
        $image_url = $field::obj{ $field::for{'media'} }->image_file_url(
            $this->get_db_data(
                $field::obj{ $field::for{'media'} }->db, $main::n
            ),
            $path, $main::c, $main::n
        );
    }
    if (  !$image_url
        && $field::for{'media2'}
        && ref $field::obj{ $field::for{'media2'} } )
    {
        $image_url = $field::obj{ $field::for{'media2'} }->image_file_url(
            $this->get_db_data(
                $field::obj{ $field::for{'media2'} }->db, $main::n
            ),
            $path, $main::c, $main::n
        );
    }
    if (  !$image_url
        && $field::for{'media3'}
        && ref $field::obj{ $field::for{'media3'} } )
    {
        $image_url = $field::obj{ $field::for{'media3'} }->image_file_url(
            $this->get_db_data(
                $field::obj{ $field::for{'media3'} }->db, $main::n
            ),
            $path, $main::c, $main::n
        );
    }
    $this->set_og_image($image_url) if $image_url;
    if ( $field::for{'coordinates'}
        && ref $field::obj{ $field::for{'coordinates'} } )
    {
        $this->set_og_latlng(
            $field::obj{ $field::for{'coordinates'} }->get_latlng(
                $this->get_db_data(
                    $field::obj{ $field::for{'coordinates'} }->db, $main::n
                ),
                $path, $main::c, $main::n
            )
        );
    }

    #ページヘッダ
    $this->set_last_modified($last_modified)
        if $setting::general{'lastModifiedHeaderEnabled'};
    if ( !$have_template || $have_template < 2 ) {
        $this->set_canonical(
                  $this->my_url
                . '?m=DetailViewer'
                . (
                (          $default::subdivided
                        && defined $main::c
                        && $main::c ne ''
                ) ? '&c=' . $main::c : ''
                )
                . (
                ( defined $main::n && $main::n ne '' )
                ? '&n=' . $main::n
                : ''
                )
        );
        $this->print_page_header($setting::key);
        print $this->get_tool_buttons( $main::n, 'event-editor', '',
            ( ( $setting::page{'detailInSameWindow'} ) ? 0 : 1 ) );
    }
    else {
        my $buttons = $this->get_tool_buttons( $main::n, 'event-editor', '',
            ( ( $setting::page{'detailInSameWindow'} ) ? 0 : 1 ) );
        $setting::page{$setting::key}{'template'}
            =~ s/(<body(?![0-9a-zA-Z_])[^>]*>)/$1$buttons/si;
        $this->print_http_header;
    }

    #テンプレートにしたがって出力
    if ($have_template) {
        my $template = $setting::page{$setting::key}{'template'};
        $this->_page_item_replace( \$template );
        my ($monthly_event_hash, $daily_event_hash,
            $has_main,           $target_prefix
        );
        my $title_id = $this->get_title_id;
        require CMONOS::CGI_Lib::Template;
        my $template_obj = Template->new;
        $template_obj->set(
            'field_style_disabled' => $setting::field_style_disabled,
            'field_tag_enabled'    => $setting::field_tag_enabled
        );
        $template = $template_obj->deco($template);

        while ( $template
            =~ /\G(.*?)(<!--(monthlylist|dailylist|list)(?!\w)([^<>]*?)-->(.*?)<!--\/\3(?!\w)[^<>]*?-->|\Z)/isg
            )
        {
            my ( $html, $tag, $attributes, $content ) = ( $1, $3, $4, $5 );
            if ( $html =~ /<!--field(?!\w)[^<>]*?-->/ ) {
                $has_main = 1;
                $template_obj->template_parser( \$html );
                print $this->get_event_data( $template_obj, $html, $main::n,
                    $title_id, '', $event_dt, $category );
            }
            else {
                print $this->event_info_replace(
                    $this->page_item_replace(
                        $html, 0, '', $setting::page{$setting::key}{'title'}
                    ),
                    $main::n,
                    $event_dt,
                    $title
                    ),
                    ;
            }
            if ($tag) {
                my ( $event_hash, $sort, $header, $footer, $prefix );
                ( $header, $footer, $attributes, $content )
                    = $this->get_header_and_footer( $attributes, $content );
                $sort
                    = (
                    $attributes =~ /[\x09\x0A\x0C\x0D\x20]+sort="([^"]+)"/i )
                    ? $this->unescape($1)
                    : 'ascend';
                if ( $tag eq 'monthlylist' ) {
                    $monthly_event_hash = $this->get_target_events_hash(
                        [   $this->get_target_month_dates(
                                $this->{'target_date'}
                            )
                        ]
                    ) if !ref $monthly_event_hash;
                    $event_hash    = $monthly_event_hash;
                    $prefix        = 'monthly-';
                    $target_prefix = $prefix if !$target_prefix;
                }
                else {
                    $daily_event_hash
                        = $this->get_target_events_hash(
                        [ $this->{'target_date'} ] )
                        if !ref $daily_event_hash;
                    $event_hash = $daily_event_hash;
                    $target_prefix = $prefix = 'daily-';
                }
                my @events = $this->sort_target_events( $event_hash, $sort );
                if (@events) {
                    print $header;
                    $template_obj->template_parser( \$content );
                    foreach my $n (@events) {
                        print $this->get_event_data( $template_obj, $content,
                            $n, $title_id, $prefix )
                            if $n !~ /\D/;
                    }
                    print $footer;
                }
            }
        }
        print $this->javascript( 'if (!location.hash) location.hash = "'
                . $target_prefix
                . 'event-detail-'
                . $main::n
                . '";' )
            if !$has_main && $target_prefix;
        $this->untie_all_dbs;

    }
    else {
        my ( $table_field, $top_field ) = ( '', '' );
        print '<div class="detail-event-category-'
            . ( ($category) ? $category : '1' )
            . '" itemscope="itemscope" itemtype="http://data-vocabulary.org/Event">';

        #装飾調整

        #項目フレーム調整
        if ( $setting::page{$setting::key}{'fieldFrame'}
            && ref $frame::obj{'frame'}
            { $setting::page{$setting::key}{'fieldFrame'} } )
        {
            my @frame_setting = (
                'framewidth' => $setting::page{$setting::key}{'fieldWidth'},
                'fixed'      => 'no',
                'frameclass' => 'item-body'
            );
            push( @frame_setting,
                'contentsalign', $setting::page{$setting::key}{'boxAlign'} )
                if defined $setting::page{$setting::key}{'boxAlign'}
                && $setting::page{$setting::key}{'boxAlign'};
            push( @frame_setting,
                'contentsvalign', $setting::page{$setting::key}{'boxValign'} )
                if defined $setting::page{$setting::key}{'boxValign'}
                && $setting::page{$setting::key}{'boxValign'};
            $frame::obj{'frame'}
                { $setting::page{$setting::key}{'fieldFrame'} }
                ->set_direct(@frame_setting);
        }

        #項目名フレーム調整
        if ( defined $setting::page{$setting::key}{'fieldTitleOption'}
            && $setting::page{$setting::key}{'fieldTitleOption'} )
        {
            if ( $setting::page{$setting::key}{'fieldFrame'}
                && ref $frame::obj{'frame'}
                { $setting::page{$setting::key}{'fieldFrame'} } )
            {
                $frame::obj{'frame'}
                    { $setting::page{$setting::key}{'fieldFrame'} }->{'type'}
                    = 'frame';
            }
        }
        elsif ( $setting::page{$setting::key}{'fieldTitleFrame'}
            && ref $frame::obj{'frame'}
            { $setting::page{$setting::key}{'fieldTitleFrame'} } )
        {
            $frame::obj{'frame'}
                { $setting::page{$setting::key}{'fieldTitleFrame'} }
                ->set_direct( 'framewidth',
                $setting::page{$setting::key}{'fieldTitleWidth'},
                'frameclass', 'item-titlebar' );
        }

        #トップ項目
        if (   defined $field::for{'top'}
            && $field::for{'top'} ne ''
            && ref $field::obj{ $field::for{'top'} }
            && $field::obj{ $field::for{'top'} }->format_for_detail_enabled
            && !$field::obj{ $field::for{'top'} }->{'Hidden'}
            && (   $field::obj{ $field::for{'top'} }->{'Open'}
                || $this->is_secure('secret') )
            )
        {
            my $db = $field::obj{ $field::for{'top'} }->db;
            if ( defined $db && $db ) {
                my ( $contents, %db );
                tie( %db, $env::dbm_file, "$default::db/$db", O_RDONLY,
                    $env::dbm_mode );
                $contents
                    = $field::obj{ $field::for{'top'} }->format_for_detail(
                    $this->decode_utf8( $db{ $this->encode_utf8($main::n) } ),
                    $path, $main::c, $main::n );
                untie(%db);
                $top_field
                    .= ( $field::for{'top'} eq $title_id )
                    ? '<h1 id="top-item" itemprop="summary">'
                    . $contents . '</h1>'
                    : '<div id="top-item" class="section"><div class="body">'
                    . $contents
                    . '</div></div>'
                    if defined $contents && $contents ne '';
            }
        }

        # 一覧表示項目
        my ( $table, $table_check, $name );
        if (   defined $setting::page{$setting::key}{'table'}
            && $setting::page{$setting::key}{'table'} ne ''
            && ref $table::obj{ $setting::page{$setting::key}{'table'} } )
        {
            $table = $table::obj{ $setting::page{$setting::key}{'table'} };
            $table_field = $table->table;
        }
        else {
            $table_field = '<table class="field-table"><tbody>';
        }
        if ( ref $table ) {
            $table_field .= $table->tr;
            $table_field .= $table->td('日時');
            $table_field .= $table->td($event_dt);
            $table_field .= $table->_tr;
        }
        else {
            $table_field
                .= '<tr class="event-date"><td class="field-name"><span class="datatitle">日時</span></td><td class="field">'
                . $event_dt
                . '</td></tr>';
        }
        foreach ( sort { $a <=> $b } keys %field::ORDER ) {
            my ( $id, %db, $db, $title, $contents );
            $id = $field::ORDER{$_};

            #トップ項目をスキップ
            next if $id eq $field::for{'top'};

       #テーブル項目以外をスキップ/不正な項目をスキップ
            next
                unless ref $field::obj{$id}
                && $field::obj{$id}->{'detailTable'}
                && !$field::obj{$id}->{'Hidden'}
                && ( $field::obj{$id}->{'Open'}
                || $this->is_secure('secret') )
                && $field::obj{$id}->format_for_detail_enabled;

            $db = $field::obj{$id}->db;
            next if !defined $db || !$db;
            tie( %db, $env::dbm_file, "$default::db/$db", O_RDONLY,
                $env::dbm_mode );
            $contents
                = $field::obj{$id}->format_for_detail(
                $this->decode_utf8( $db{ $this->encode_utf8($main::n) } ),
                $path, $main::c, $main::n );
            untie %db;
            next if !defined $contents || $contents eq '';

            #タイトルバー
            if (   $setting::page{$setting::key}{'fieldTitleOption'}
                && ( !defined $name || $name eq '' )
                && (   $id eq $field::for{'title'}
                    || $id eq $field::for{'key'}
                    || $id eq $field::for{'name'} )
                && $contents =~ /[^\x09\x0A\x0C\x0D\x20]/
                && $contents ne '&nbsp;'
                )
            {
                $name
                    = '<h2 class="item-title"'
                    . ( ( $id eq $title_id ) ? ' itemprop="summary"' : '' )
                    . '>'
                    . $contents . '</h2>';
                next;
            }

            #一覧表示項目
            $title = $field::obj{$id}->name;
            if ( ref $table ) {
                $table_field .= $table->tr;
                $table_field .= $table->td($title);
                $table_field .= $table->td( $contents,
                    ( ( $id eq $title_id ) ? ' itemprop="summary"' : '' ) );
                $table_field .= $table->_tr;
            }
            else {
                $table_field
                    .= '<tr><td class="field-name"><span class="datatitle">'
                    . $title
                    . '</span></td><td class="field"'
                    . ( ( $id eq $title_id ) ? ' itemprop="summary"' : '' )
                    . '>'
                    . $contents
                    . '</td></tr>';
            }
            $table_check = 1;
        }
        if ($table_check) {
            $table_field
                .= ( ref $table ) ? $table->_table : '</tbody></table>';
        }
        else {
            $table_field = '';
        }
        $table_field
            = ( $setting::page{$setting::key}{'fieldFrame'} )
            ? $this->get_frame( $setting::page{$setting::key}{'fieldFrame'},
            $table_field, $name, 'no', 'section', 'item-table' )
            : '<div id="item-table" class="section">'
            . ( ( defined $name ) ? $name : '' )
            . '<div class="body">'
            . $table_field
            . '</div></div>'
            if defined $table_field && $table_field ne '';
        $table_field = $top_field . $table_field;

        #一覧表示
        print $table_field
            if !defined $setting::page{$setting::key}{'tableValign'}
            || $setting::page{$setting::key}{'tableValign'} ne 'bottom';

        #書き出し
        foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
            my ( $id, %db, $db, $data );
            $id = $field::ORDER{$i};
            if ( ref $field::obj{$id} ) {
                if (   $field::obj{$id}->format_for_detail_enabled
                    && !$field::obj{$id}->{'Hidden'}
                    && (   $field::obj{$id}->{'Open'}
                        || $this->is_secure('secret') )
                    )
                {
                    if ( $id ne $field::for{'top'}
                        && !$field::obj{$id}->{'detailTable'} )
                    {
                        $db = $field::obj{$id}->db;
                        if ( defined $db && $db ) {
                            tie( %db, $env::dbm_file, "$default::db/$db",
                                O_RDONLY, $env::dbm_mode );
                            $data = $this->decode_utf8(
                                $db{ $this->encode_utf8($main::n) } );
                            untie %db;
                        }

                        my ( $name, $title, $contents );

                        #空コンテンツをスキップ
                        $contents
                            = $field::obj{$id}
                            ->format_for_detail( $data, $path, $main::c,
                            $main::n );
                        next
                            if $contents !~ /[^\x09\x0A\x0C\x0D\x20]/
                            || $contents eq '&nbsp;';
                        print '<div class="section" id="item-' . $id . '">';

                        #項目名
                        $name = $field::obj{$id}->name;
                        if (   defined $name
                            && $name ne ''
                            && $name ne '&nbsp;'
                            && $name =~ /[^\x09\x0A\x0C\x0D\x20]/ )
                        {
                            $name
                                = '<h2 class="item-title">' . $name . '</h2>';
                            if (defined $setting::page{$setting::key}
                                {'fieldTitleOption'}
                                && $setting::page{$setting::key}
                                {'fieldTitleOption'} )
                            {
                                $title = $name;
                            }
                            else {
                                print(
                                    (   $setting::page{$setting::key}
                                            {'fieldTitleFrame'}
                                    )
                                    ? $this->get_frame(
                                        $setting::page{$setting::key}
                                            {'fieldTitleFrame'},
                                        $name,
                                        '',
                                        'no',
                                        'item-titlebar'
                                        )
                                    : $name
                                );
                            }
                        }

                        #項目欄
                        print(
                            ( $setting::page{$setting::key}{'fieldFrame'} )
                            ? $this->get_frame(
                                $setting::page{$setting::key}{'fieldFrame'},
                                $contents, $title, 'no', 'item-body' )
                            : ( ( defined $title ) ? $title : '' )
                                . '<div class="body">'
                                . $contents
                                . '</div>'
                        );
                        print '</div>';
                    }
                }
                else {
                    $field::obj{$id}->disabled;
                }
            }
        }

        #一覧表示
        if ( defined $setting::page{$setting::key}{'tableValign'}
            && $setting::page{$setting::key}{'tableValign'} eq 'bottom' )
        {
            print $table_field;
        }
        print '</div>';
    }

    #ページフッタ
    if ( !$have_template || $have_template < 2 ) {
        print '<div id="close-button" class="panel">'
            . $this->close_button(
            (   ( $this->{'target_date'} && !$ENV{'HTTP_REFERER'} )
                ? $cgi::path . '?'
                    . ( ( defined $main::c ) ? 'c=' . $main::c . '&' : '' )
                    . 'date='
                    . $this->{'target_date'}
                : ''
            ),
            $setting::page{'closeButton'},
            $setting::page{'backButton'}
            ) . '</div>';
        $this->print_page_footer($setting::key);
    }

}

sub is_guest ($;$) {
    my ( $this, $strictly ) = @_;
    return $this->{'is_guest'} if defined $this->{'is_guest'} && $strictly;
    my ( $id, $field, @mail, $is_guest, $has_inviter );
    $is_guest = $has_inviter = 0;
FIELD:
    while ( ( $id, $field ) = each %field::obj ) {
        if ( $field->{'e-mail'} || $field->{'class'} eq 'xcal_inviter' ) {
            my (%db);
            tie( %db, $env::dbm_file, $default::db . '/' . $field->db,
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %db ) {
                if ( $field->{'class'} eq 'xcal_inviter' ) {
                    $has_inviter = 1;
                    my $data = $this->decode_utf8(
                        $db{ $this->encode_utf8($main::n) } );
                GUEST:
                    while ( $data =~ /^(.*?)$/mg ) {
                        my $line = $1;
                        next GUEST if $line =~ /^organizer=/;
                        chomp $line;
                        my ( $email, $key ) = split( /\x09/, $line );
                        next GUEST if !$email;
                        push( @mail, $email );
                        if (   $main::guest{'email'}
                            && $email eq $main::guest{'email'}
                            && $main::guest{'key'}
                            && $key eq $main::guest{'key'} )
                        {
                            $is_guest = 1;
                            last GUEST;
                        }
                    }
                }
                else {
                    push(
                        @mail,
                        $this->decode_utf8(
                            $db{ $this->encode_utf8($main::n) }
                        )
                    );
                }
            }
            untie(%db);
            last FIELD if $is_guest;
        }
    }
    if ( !$is_guest && @mail ) {
        @mail = $this->get_address(@mail);
        $is_guest = 1 if $this->is_authorized(@mail);
    }
    $this->{'is_guest'} = $is_guest;
    return ( $is_guest || ( !$strictly && !$has_inviter ) ) ? 1 : 0;
}

sub get_db_data ($$) {
    my ( $this, $db, $key ) = @_;
    my ( %db, $data );
    tie( %db, $env::dbm_file, $default::db . '/' . $db,
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %db ) {
        $data = $this->decode_utf8( $db{ $this->encode_utf8($key) } );
    }
    untie(%db);
    return $data;
}

######################################################################
# ヘッダ・フッタ
sub _page_item_replace ($$;$$$$) {
    my ( $this, $str, $file_flag, $output, $title, $info ) = @_;
    $$str
        =~ s/<\!--date(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/date(?![0-9a-zA-Z_])[^>]*-->/$this->target_date($1,$2)/iges;
    $$str
        =~ s/<\!--return(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/return(?![0-9a-zA-Z_])[^>]*-->/$this->return_link($1,$2)/iges;
}

sub target_date ($$$) {
    my ( $this, $attributes, $content ) = @_;
    return $this->item_tag_replace(
        'date',
        $attributes,
        $content,
        $this->date(
            $this->{'target_date'},
            (   ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+limit="([^"]+)"/i )
                ? $1
                : ''
            ),
            (   ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+type="([^"]+)"/i )
                ? $1
                : 'c'
            ),
            (   ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+format="([^"]+)"/i )
                ? $1
                : ''
            )
        )
    );
}

sub return_link ($$$) {
    my ( $this, $attributes, $content ) = @_;
    $content
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+text="([^"]+)"/i )
        ? $1
        : ''
        if !$content;
    my $class
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+class="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    my $id
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+id="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    return
          '<a href="'
        . $cgi::path
        . '?m=Viewer'
        . ( ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
        . '&amp;date='
        . $this->{'target_date'}
        . $this->amp_escape(
        $this->set_query( 'url', [ 'and', 'search' ], 1 ) )
        . '"'
        . ( ($id)    ? ' id="' . $this->escape($id) . '"'       : '' )
        . ( ($class) ? ' class="' . $this->escape($class) . '"' : '' ) . '>'
        . ( $content || '戻る' ) . '</a>';
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::deco     = 1;    #テキスト装飾設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み
    $setting::table    = 1;    #テーブル設定読み込み
    $setting::page     = 1;    #ページ設定読み込み

    $setting::field_db_name = 'inputfield';    #フィールド設定DB
    $setting::field         = 2
        ; #フィールド設定読み込み（2=オリジナルも読み込む）
    $setting::field_style_disabled = 0
        ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視
    $setting::field_tag_enabled
        = 'no'; #tag=タグのみ/all=全ての設定/no=タグをつけない

    #設定種キー設定
    $setting::security_key
        = 'browse';    #デフォルトセキュリティキー
    $setting::key = 'detail';    #ページ設定キー

    #クッキー
    $cookie::id_enabled = 1;     #ID情報クッキー読み込み

    #デフォルトキー
    $main::k = 'detail' if !defined $main::k || !$main::k;

    #必須フォーム
    @setting::form = ( 'm', 'k', 'n' );
    push( @setting::form, 'c' ) if $default::subdivided;

    #イメージCSS
    $setting::image_css = 'calendar-image.css';

    #クラス登録
    Common::class_register( 'detail', 'DetailViewer' );

}

1;
