package Editor;
require CMONOS::Calendar::CalCommon;
@ISA = qw(CalCommon);
use strict;
use utf8;
######################################################################
# カレンダーイベント編集モジュール
# Editor.pm ver1.1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-04-12
######################################################################
# DT
# DTSTART-DTEND <TAB> RRULE <TAB> カンマ区切りDATEリスト
######################################################################
# DATE
# カンマ区切りEVENT IDリスト
# 書式）PREV<EVENT ID>NEXT = PREV日前からNEXT日後まで続く EVENT ID
######################################################################
# 実行
sub run ($;$) {
    my ( $this, $output ) = @_;
    my ( $input_is_valid, @available_wday, $is_repeatable, $se_gap,
        $event_dt, $original_n );
    my $repeat_count = 0;

    use Fcntl qw(:DEFAULT :flock);
    use Time::Local;

    #データ読み込み
    $main::a = '' if $main::delete || !%main::data || !defined $main::a;
    undef $main::n if $main::n && $main::n =~ /[^0-9]/;
    if ( $main::a && $main::n && $main::copy ) {
        $original_n = $main::n;
        undef $main::n;
    }
    $this->dbm if !$main::a && $main::n;
    if (   $field::NAMECHECK
        && $field::for{'name'}
        && !defined $main::data{ $field::for{'name'} }{'str'}
        && $this->use_register )
    {
        $main::data{ $field::for{'name'} }{'str'}
            = $cgi::Register->email_to_name( $main::pswd, $main::email,
            $security::registered );
    }

    #更新権限チェック
    my $input_only_user = $this->is_input_only_user;

    $main::dtstart = $this->dt_fix($main::dtstart) if $main::dtstart;

    #データ削除
    if ($main::delete) {
        if (   defined $main::n
            && $main::n ne ''
            && !$input_only_user
            && $this->is_secure( 'delete', 0, 'revise' ) )
        {
            my ( %date, %dt, @date );

            #日時データベース更新
            tie( %dt, $env::dbm_file, $default::db . '/dt',
                O_RDWR, $env::dbm_mode );
            if ( ref tied %dt && $dt{ $this->encode_utf8($main::n) } ) {
                @date = split( /,/, $1 )
                    if $this->decode_utf8(
                    $dt{ $this->encode_utf8($main::n) } )
                    =~ /^[^\x09\x0A\x0C\x0D\x20]+\x09[^\x09\x0A\x0C\x0D\x20]*\x09(.*)$/;
                delete $dt{ $this->encode_utf8($main::n) };
            }
            untie %dt;
            tie( %date, $env::dbm_file, $default::db . '/date',
                O_RDWR, $env::dbm_mode );
            if ( ref tied %date ) {
                my $n = $this->encode_utf8($main::n);
                foreach my $d (@date) {
                    $d =~ s/^[\-]?[0-9]+<//;
                    $d =~ s/>[\-]?[0-9]+$//;
                    utf8::encode($d) if utf8::is_utf8($d);
                    if ( $date{$d} ) {
                        $date{$d} =~ s/,?([\-]?[0-9]+<|)$n(>[\-]?[0-9]+|)//g;
                        delete $date{$d} if !$date{$d};
                    }
                }
            }
            untie %date;

            #データ削除
            $this->delete_from_dbs($main::n);

            #添付ファイル削除
            use File::Path;
            rmtree "$default::file/$main::n";

            #RSS/インデックス掃除
            $this->index_sweeper($main::n);

            #オープナーURL指定
            $this->set_opener_location;

            #ページヘッダ
            $this->print_page_header('editor');

            print
                '<p>指定されたイベントを削除しました。</p>';
            print '<div id="close-button">'
                . $this->close_button(
                $this->{'return_url'},
                $setting::page{'closeButton'},
                $setting::page{'backButton'}
                ) . '</div>';

            #ページフッタ
            $this->print_page_footer('editor');
            exit;

        }
        elsif ( $input_only_user
            || !$this->is_secure( 'delete', 0, 'revise' ) )
        {
            push(
                @{ $this->{'dialog'} },
                '削除に失敗しました。削除権限がありません。'
            );
        }
        elsif ( !$input_only_user ) {
            push(
                @{ $this->{'dialog'} },
                '削除に失敗しました。対象イベントが指定されていません。'
            );
        }

        #入力のチェック
    }
    elsif ( $main::a eq 'w' ) {
        $input_is_valid = 1;

        #使用停止項目
        if ( $setting::general{'minScale'} ) {
            $main::dt{'smin'}
                = $setting::general{'minScale'} *
                int( $main::dt{'smin'} / $setting::general{'minScale'} )
                if $main::dt{'smin'};
            $main::dt{'emin'}
                = $setting::general{'minScale'} *
                int( $main::dt{'emin'} / $setting::general{'minScale'} )
                if $main::dt{'emin'};
        }
        if ( $setting::general{'dateDisabled'} ) {
            delete $main::dt{'date'};
            if ( $setting::general{'endDateDisabled'} ) {
                $main::dt{'ey'} = $main::dt{'sy'};
                $main::dt{'em'} = $main::dt{'sm'};
                $main::dt{'ed'} = $main::dt{'sd'};
            }
        }
        if ( $setting::general{'freqDisabled'} ) {
            $main::dt{'freq'}       = '';
            $main::dt{'interval'}   = 1;
            $main::dt{'by'}         = 'bymday';
            $main::dt{'indefinite'} = 0;
            $main::dt{'uy'}         = $main::dt{'ey'};
            $main::dt{'um'}         = $main::dt{'em'};
            $main::dt{'ud'}         = $main::dt{'ed'};
        }

        #終日補正
        ( $main::dt{'ey'}, $main::dt{'em'}, $main::dt{'ed'} )
            = $this->get_next_date_array( $main::dt{'ey'}, $main::dt{'em'},
            $main::dt{'ed'} )
            if $main::dt{'date'};

        #イベントカテゴリー設定読み込み
        my ( @available_time, @samezone_categories, $event_margin,
            $category_name, $category_id, @admins );
        if ( $field::for{'category'} ) {
            $this->import_db( $default::preference_dir . '/event_category',
                \%setting::event_category, 1 )
                unless %setting::event_category
                && ref $setting::event_category{'ORDER'}
                && %{ $setting::event_category{'ORDER'} };
            if (   defined $main::data{ $field::for{'category'} }{'str'}
                && %setting::event_category
                && ref $setting::event_category{'ORDER'}
                && %{ $setting::event_category{'ORDER'} }
                && ref
                $setting::event_category{ $main::data{ $field::for{'category'}
                        }{'str'} } )
            {
                $category_id = $main::data{ $field::for{'category'} }{'str'};
                if (defined $setting::event_category{$category_id}
                    {'availableTime'}
                    && $setting::event_category{$category_id}{'availableTime'}
                    ne '' )
                {
                    @available_time = split(
                        /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,
                        $setting::event_category{$category_id}
                            {'availableTime'}
                    );
                }
                if (defined $setting::event_category{$category_id}
                    {'availableWday'} )
                {
                    if (ref $setting::event_category{$category_id}
                        {'availableWday'} )
                    {
                        @available_wday
                            = @{ $setting::event_category{$category_id}
                                {'availableWday'} };
                    }
                    elsif ( $setting::event_category{$category_id}
                        {'availableWday'} ne '' )
                    {
                        @available_wday
                            = ( $setting::event_category{$category_id}
                                {'availableWday'} );
                    }
                }
                $event_margin
                    = $setting::event_category{$category_id}{'eventMargin'}
                    || 0;
                $category_name
                    = $setting::event_category{$category_id}{'name'};
                @admins = $this->get_address(
                    $setting::event_category{$category_id}{'admin'} );
                if ( $setting::event_category{$category_id}{'overlapDisabled'}
                    )
                {
                    @samezone_categories = ($category_id);
                    if ( $setting::event_category{$category_id}{'zone'} ) {
                        foreach my $n (
                            keys %{ $setting::event_category{'ORDER'} } )
                        {
                            my $id = $setting::event_category{'ORDER'}{$n};
                            if (   $category_id ne $id
                                && $setting::event_category{$id}
                                {'overlapDisabled'}
                                && $setting::event_category{$id}{'zone'} eq
                                $setting::event_category{$category_id}{'zone'}
                                )
                            {
                                push( @samezone_categories, $id );
                                $event_margin
                                    = $setting::event_category{$id}
                                    {'eventMargin'}
                                    if $setting::event_category{$id}
                                    {'eventMargin'}
                                    && $event_margin
                                    < $setting::event_category{$id}
                                    {'eventMargin'};
                            }
                        }
                    }
                }
            }
        }

        #イベント日時のチェック
        $main::dt{'interval'} = 1 if !$main::dt{'interval'};
        my $dt_is_valid = 1;
        foreach my $k (
            'sy', 'sm', 'sd', 'sh',   'smin', 'ey',
            'em', 'ed', 'eh', 'emin', 'uy',   'um',
            'ud', 'interval'
            )
        {
            $dt_is_valid = 0
                if !defined $main::dt{$k}
                || $main::dt{$k} eq ''
                || $this->is_nan( $main::dt{$k} )
                || ( $k =~ /h$/
                && ( $main::dt{$k} < 0 || $main::dt{$k} > 23 ) )
                || ( $k =~ /min$/
                && ( $main::dt{$k} < 0 || $main::dt{$k} > 59 ) );
        }
        if ($dt_is_valid) {
            if ( $main::dt{'sy'} < 1970 || 2036 < $main::dt{'sy'} ) {
                push(
                    @{ $this->{'dialog'} },
                    '開始年を確認してください。1970年以前または2036年以降の日付は設定できません。'
                );
                $main::dt{'sy'} = ( $main::dt{'sy'} < 1970 ) ? 1970 : 2036;
                $dt_is_valid = 0;
            }
            if ( $main::dt{'ey'} < 1970 || 2036 < $main::dt{'ey'} ) {
                push(
                    @{ $this->{'dialog'} },
                    '終了年を確認してください。1970年以前または2036年以降の日付は設定できません。'
                );
                $main::dt{'ey'} = ( $main::dt{'ey'} < 1970 ) ? 1970 : 2036;
                $dt_is_valid = 0;
            }
            if ($this->is_impossible_mday(
                    $main::dt{'sy'}, $main::dt{'sm'}, $main::dt{'sd'}
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    '開始日を確認してください。存在しない日付です。'
                );
                $main::dt{'sd'}
                    = ( $main::dt{'sm'} == 2 )
                    ? ( $this->is_leap_year( $main::dt{'sy'} ) )
                        ? 29
                        : 28
                    : 30;
                $dt_is_valid = 0;
            }
            if ($this->is_impossible_mday(
                    $main::dt{'ey'}, $main::dt{'em'}, $main::dt{'ed'}
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    '終了日を確認してください。存在しない日付です。'
                );
                $main::dt{'ed'}
                    = ( $main::dt{'em'} == 2 )
                    ? ( $this->is_leap_year( $main::dt{'ey'} ) )
                        ? 29
                        : 28
                    : 30;
                $dt_is_valid = 0;
            }
            $main::dt{'indefinite'} = '' if !$main::dt{'freq'};
            if ( !$main::dt{'indefinite'} ) {
                if ( !$setting::general{'freqDisabled'} ) {
                    if ( $main::dt{'uy'} < 1970 || 2036 < $main::dt{'uy'} ) {
                        push(
                            @{ $this->{'dialog'} },
                            '期限を確認してください。1970年以前または2036年以降の日付は設定できません。'
                        );
                        $main::dt{'uy'}
                            = ( $main::dt{'uy'} < 1970 ) ? 1970 : 2036;
                        $dt_is_valid = 0;
                    }
                    if ($this->is_impossible_mday(
                            $main::dt{'uy'}, $main::dt{'um'},
                            $main::dt{'ud'}
                        )
                        )
                    {
                        push(
                            @{ $this->{'dialog'} },
                            '期限を確認してください。存在しない日付です。'
                        );
                        $main::dt{'ud'}
                            = ( $main::dt{'um'} == 2 )
                            ? ( $this->is_leap_year( $main::dt{'uy'} ) )
                                ? 29
                                : 28
                            : 30;
                        $dt_is_valid = 0;
                    }
                }
            }
            elsif (@samezone_categories) {
                push(
                    @{ $this->{'dialog'} },
                    '重複が禁止されている「'
                        . $category_name
                        . '」では繰り返しの「期限」を「無期限」に設定することができません。'
                );
                $main::dt{'indefinite'} = 0;
                $dt_is_valid = 0;
            }
            elsif ($main::dt{'freq'}
                && $main::decompose
                && $main::dt{'indefinite'} )
            {
                push(
                    @{ $this->{'dialog'} },
                    '繰り返しの「期限」が「無期限」に設定されている場合、予定を個別に保存することができません。'
                );
                $dt_is_valid = $main::decompose = 0;

#} elsif (@available_wday && $main::dt{'freq'}) {
#	push(@{$this->{'dialog'}},'設定可能曜日が制限されている「' . $category_name . '」では繰り返しの「期限」を「無期限」に設定することができません。');
#	$main::dt{'indefinite'} = 0;
#	$dt_is_valid = 0;
            }
            $main::repeat{'h'} = ''
                if !defined $main::repeat{'h'}
                || $main::repeat{'h'} eq ''
                || $this->is_nan( $main::repeat{'h'} )
                || $main::repeat{'h'} < 0
                || $main::repeat{'h'} > 12;
            $main::repeat{'min'} = ''
                if !defined $main::repeat{'min'}
                || $main::repeat{'min'} eq ''
                || $this->is_nan( $main::repeat{'min'} )
                || $main::repeat{'min'} < 0
                || $main::repeat{'min'} > 59;
            $main::repeat{'uh'} = ''
                if !defined $main::repeat{'uh'}
                || $main::repeat{'uh'} eq ''
                || $this->is_nan( $main::repeat{'uh'} )
                || $main::repeat{'uh'} < 0
                || $main::repeat{'uh'} > 23;
            $main::repeat{'umin'} = ''
                if !defined $main::repeat{'umin'}
                || $main::repeat{'umin'} eq ''
                || $this->is_nan( $main::repeat{'umin'} )
                || $main::repeat{'umin'} < 0
                || $main::repeat{'umin'} > 59;
            if (   ( $main::repeat{'h'} || $main::repeat{'min'} )
                && ( $main::repeat{'uh'} || $main::repeat{'umin'} )
                && !$main::dt{'date'} )
            {
                $main::repeat{'h'} = 0
                    if !$main::repeat{'h'} && $main::repeat{'min'};
                $main::repeat{'min'} = 0
                    if !$main::repeat{'min'} && $main::repeat{'h'};
                $main::repeat{'uh'} = 0
                    if !$main::repeat{'uh'} && $main::repeat{'umin'};
                $main::repeat{'umin'} = 0
                    if !$main::repeat{'umin'} && $main::repeat{'uh'};
                $is_repeatable = 1;
            }
            else {
                $main::repeat{'h'} = $main::repeat{'min'}
                    = $main::repeat{'uh'} = $main::repeat{'umin'} = '';
            }

           #開始＜終了＜期限　になっているか、重複の禁止
            if ($dt_is_valid) {
                my $dt_start_time = timelocal(
                    0,
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'smin'} ),
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'sh'} ),
                    $main::dt{'sd'},
                    ( $main::dt{'sm'} - 1 ),
                    $main::dt{'sy'}
                );
                my $dt_end_time = timelocal(
                    0,
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'emin'} ),
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'eh'} ),
                    $main::dt{'ed'},
                    ( $main::dt{'em'} - 1 ),
                    $main::dt{'ey'}
                );
                $se_gap = $dt_end_time - $dt_start_time;
                if ( $se_gap <= 0 ) {
                    push(
                        @{ $this->{'dialog'} },
                        '日付を確認してください。終了日時が開始日時よりも前になっています。'
                    );
                    $dt_is_valid = 0;
                }
                elsif ( $main::dt{'freq'} && $se_gap > 86400 ) {
                    my $se_gap_day = int( $se_gap / 86400 )
                        + ( ( $se_gap % 86400 ) ? 1 : 0 );
                    if (   $main::dt{'freq'} eq 'daily'
                        || $main::dt{'freq'} eq 'weekly' )
                    {
                        push(
                            @{ $this->{'dialog'} },
                            '繰り返し設定を確認してください。1日以上にわたる予定は日もしくは週単位の繰り返しを設定できません。'
                        );
                        $dt_is_valid = 0;
                    }
                    elsif ($se_gap_day > 364
                        && $main::dt{'freq'} eq 'yearly' )
                    {
                        push(
                            @{ $this->{'dialog'} },
                            '繰り返し設定を確認してください。364日以上にわたる予定は繰り返しを設定できません。'
                        );
                        $dt_is_valid = 0;
                    }
                    elsif ($se_gap_day > 28
                        && $main::dt{'freq'} eq 'monthly' )
                    {
                        push(
                            @{ $this->{'dialog'} },
                            '繰り返し設定を確認してください。28日以上にわたる予定は年単位の繰り返しのみ設定できます。'
                        );
                        $dt_is_valid = 0;
                    }
                }
                if ($setting::general{'minLength'}
                    && $this->timeunit_to_sec(
                        $setting::general{'minLength'},
                        $setting::general{'minLengthUnit'}
                    ) > $se_gap
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        '開始から終了までを'
                            . $setting::general{'minLength'}
                            . $this->timeunit_to_japanese(
                            $setting::general{'minLengthUnit'}
                            )
                            . '以上にしてください。'
                    );
                    $dt_is_valid = 0;
                }
                elsif (
                    $setting::general{'maxLength'}
                    && $this->timeunit_to_sec(
                        $setting::general{'maxLength'},
                        $setting::general{'maxLengthUnit'}
                    ) < $se_gap
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        '開始から終了までを'
                            . $setting::general{'maxLength'}
                            . $this->timeunit_to_japanese(
                            $setting::general{'maxLengthUnit'}
                            )
                            . '以下にしてください。'
                    );
                    $dt_is_valid = 0;
                }
                if ($this->is_out_of_bounds(
                        $dt_start_time,
                        $dt_end_time,
                        $setting::general{'boundsStart'},
                        $setting::general{'boundsStartUnit'},
                        $setting::general{'boundsStartDirection'},
                        1
                    )
                    || $this->is_out_of_bounds(
                        $dt_start_time,
                        $dt_end_time,
                        $setting::general{'boundsEnd'},
                        $setting::general{'boundsEndUnit'},
                        $setting::general{'boundsEndDirection'}
                    )
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        '日時を確認してください。現在設定可能な日時は'
                            . $this->bounds_to_datetime(
                            $setting::general{'boundsStart'},
                            $setting::general{'boundsStartUnit'},
                            $setting::general{'boundsStartDirection'},
                            1
                            )
                            . $this->bounds_to_datetime(
                            $setting::general{'boundsEnd'},
                            $setting::general{'boundsEndUnit'},
                            $setting::general{'boundsEndDirection'}
                            )
                            . 'です。'
                    );
                    $dt_is_valid = 0;
                }
                my ( $dt_last_end_time, $repeat_interval );
                if ($is_repeatable) {
                    my $dt_last_start_time = $dt_start_time;
                    my ( $y, $m, $d )
                        = (    defined $main::dt{'sh'}
                            && $main::dt{'sh'} =~ /^[0-9]+$/
                            && defined $main::dt{'smin'}
                            && $main::dt{'smin'} =~ /^[0-9]+$/
                            && $main::repeat{'uh'} * 100
                            + $main::repeat{'umin'} <= $main::dt{'sh'} * 100
                            + $main::dt{'smin'} )
                        ? $this->get_next_date_array( $main::dt{'sy'},
                        $main::dt{'sm'}, $main::dt{'sd'} )
                        : ( $main::dt{'sy'}, $main::dt{'sm'},
                        $main::dt{'sd'} );
                    my $last = timelocal( 0, $main::repeat{'umin'},
                        $main::repeat{'uh'}, $d, ( $m - 1 ), $y );
                    $repeat_interval = $main::repeat{'h'} * 3600
                        + $main::repeat{'min'} * 60;
                    while ( $dt_last_start_time + $repeat_interval <= $last )
                    {
                        $dt_last_start_time += $repeat_interval;
                        $repeat_count++;
                    }
                    $dt_last_end_time = $dt_last_start_time + $se_gap;
                    if ($this->is_out_of_bounds(
                            $dt_last_start_time,
                            $dt_last_end_time,
                            $setting::general{'boundsStart'},
                            $setting::general{'boundsStartUnit'},
                            $setting::general{'boundsStartDirection'},
                            1
                        )
                        || $this->is_out_of_bounds(
                            $dt_last_start_time,
                            $dt_last_end_time,
                            $setting::general{'boundsEnd'},
                            $setting::general{'boundsEndUnit'},
                            $setting::general{'boundsEndDirection'}
                        )
                        )
                    {
                        push(
                            @{ $this->{'dialog'} },
                            '複製の最終時を確認してください。現在設定可能な日時は'
                                . $this->bounds_to_datetime(
                                $setting::general{'boundsStart'},
                                $setting::general{'boundsStartUnit'},
                                $setting::general{'boundsStartDirection'},
                                1
                                )
                                . $this->bounds_to_datetime(
                                $setting::general{'boundsEnd'},
                                $setting::general{'boundsEndUnit'},
                                $setting::general{'boundsEndDirection'}
                                )
                                . 'です。'
                        );
                        $dt_is_valid = 0;
                    }
                }
                if ( $main::dt{'freq'} && $main::dt{'freq'} eq 'weekly' ) {
                    $this->scalar_to_array( \$main::dt{'byday'} );
                    @{ $main::dt{'byday'} }
                        = grep( $_ && $_ =~ /^(SU|MO|TU|WE|TH|FR|SA)$/,
                        @{ $main::dt{'byday'} } );
                    if (@available_wday) {
                        my @temp;
                        foreach my $day ( @{ $main::dt{'byday'} } ) {
                            push( @temp, $day )
                                if grep( $_ != 7
                                    && $day eq $this->number_to_wday($_),
                                @available_wday );
                        }
                        @{ $main::dt{'byday'} } = @temp;
                    }
                    if ( !@{ $main::dt{'byday'} } ) {
                        $main::dt{'byday'} = [
                            $this->number_to_wday(
                                $this->get_wday(
                                    $main::dt{'sy'}, $main::dt{'sm'},
                                    $main::dt{'sd'}
                                )
                            )
                        ];
                    }
                    elsif ( @{ $main::dt{'byday'} } >= 7 ) {
                        $main::dt{'freq'} = 'daily';
                    }
                }
                elsif (@available_wday
                    && $main::dt{'freq'}
                    && $main::dt{'freq'} eq 'monthly'
                    && $main::dt{'by'}
                    && $main::dt{'by'} eq 'bywday' )
                {
                    my $wday
                        = $this->get_wday( $main::dt{'sy'}, $main::dt{'sm'},
                        $main::dt{'sd'} );
                    my $wday_end;
                    if (   $main::dt{'sh'}
                        && $main::dt{'smin'}
                        && $is_repeatable
                        && $main::repeat{'uh'} * 100 + $main::repeat{'umin'}
                        <= $main::dt{'sh'} * 100 + $main::dt{'smin'} )
                    {
                        $wday_end = ( localtime($dt_last_end_time) )[6];
                    }
                    if (!grep( $_ != 7 && $wday == $_, @available_wday )
                        || (defined $wday_end
                            && !grep( $_ != 7 && $wday_end == $_,
                                @available_wday )
                        )
                        )
                    {
                        push(
                            @{ $this->{'dialog'} },
                            '「'
                                . $category_name
                                . '」は設定可能な'
                                . (
                                (@available_time) ? '時間帯が' : '日が'
                                )
                                . (
                                (@available_wday)
                                ? '「'
                                    . $this->wdaynum_to_name(@available_wday)
                                    . '」'
                                : ''
                                )
                                . (
                                  (@available_time)
                                ? ( (@available_wday) ? 'の' : '' ) . '「'
                                    . join( ',', @available_time ) . '」'
                                : ''
                                )
                                . 'に制限されています。'
                        );
                        $dt_is_valid = 0;
                    }
                }
                if ( !$main::dt{'indefinite'} ) {
                    if ( $setting::general{'freqDisabled'} ) {
                        ( $main::dt{'uy'}, $main::dt{'um'}, $main::dt{'ud'} )
                            = (
                            $main::dt{'ey'}, $main::dt{'em'}, $main::dt{'ed'}
                            ) if $dt_is_valid;
                    }
                    else {
                        my $dt_until_time
                            = timelocal( 0, 0, 0, $main::dt{'ud'},
                            ( $main::dt{'um'} - 1 ),
                            $main::dt{'uy'} );
                        if ( $dt_end_time > $dt_until_time ) {
                            if ( $main::dt{'freq'} ) {
                                push(
                                    @{ $this->{'dialog'} },
                                    '期限を確認してください。期限が開始日時よりも前になっています。'
                                );
                                $dt_is_valid = 0;
                            }
                            elsif ($dt_is_valid) {
                                (   $main::dt{'uy'}, $main::dt{'um'},
                                    $main::dt{'ud'}
                                    )
                                    = (
                                    $main::dt{'ey'}, $main::dt{'em'},
                                    $main::dt{'ed'}
                                    );
                            }
                        }
                        elsif (
                            $main::dt{'freq'}
                            && ($this->is_out_of_bounds(
                                    $dt_start_time,
                                    $dt_until_time,
                                    $setting::general{'boundsStart'},
                                    $setting::general{'boundsStartUnit'},
                                    $setting::general{'boundsStartDirection'},
                                    1
                                )
                                || $this->is_out_of_bounds(
                                    $dt_start_time,
                                    $dt_until_time,
                                    $setting::general{'boundsEnd'},
                                    $setting::general{'boundsEndUnit'},
                                    $setting::general{'boundsEndDirection'}
                                )
                            )
                            )
                        {
                            push(
                                @{ $this->{'dialog'} },
                                '期限を確認してください。現在設定可能な期限は'
                                    . $this->bounds_to_datetime(
                                    $setting::general{'boundsStart'},
                                    $setting::general{'boundsStartUnit'},
                                    $setting::general{'boundsStartDirection'},
                                    1
                                    )
                                    . $this->bounds_to_datetime(
                                    $setting::general{'boundsEnd'},
                                    $setting::general{'boundsEndUnit'},
                                    $setting::general{'boundsEndDirection'}
                                    )
                                    . 'です。'
                            );
                            $dt_is_valid = 0;
                        }
                    }
                }

                #設定可能範囲が含まれるか/重複禁止チェック
                if (   $is_repeatable
                    && !$main::dt{'date'}
                    && @samezone_categories
                    && $se_gap + ( $event_margin * 60 ) > $repeat_interval )
                {
                    my $min = int(
                        ( $se_gap + ( $event_margin * 60 ) ) / 60 + 0.99 );
                    my $h = int( $min / 60 );
                    $min = $min % 60;
                    push(
                        @{ $this->{'dialog'} },
                        (   ( $h > 24 )
                            ? '重複が禁止されているため、この予定を複製できませんでした。'
                            : '複製の間隔が短すぎます。'
                                . $h
                                . ' 時間 '
                                . $min
                                . ' 分以上にしてください。'
                        )
                    );
                    $dt_is_valid = 0;
                }
                if (   $dt_is_valid
                    && ( !$main::dt{'freq'} || !$main::dt{'indefinite'} )
                    && (   @available_time
                        || @available_wday
                        || @samezone_categories )
                    )
                {
                    my %dt = %main::dt;
                    for ( my $i = 0; $i <= $repeat_count; $i++ ) {
                        if ($i) {
                            $dt{'smin'} += $main::repeat{'min'};
                            if ( $dt{'smin'} > 59 ) {
                                $dt{'smin'} -= 60;
                                $dt{'sh'}++;
                            }
                            $dt{'sh'} += $main::repeat{'h'};
                            if ( $dt{'sh'} > 23 ) {
                                $dt{'sh'} -= 24;
                                ( $dt{'sy'}, $dt{'sm'}, $dt{'sd'} )
                                    = $this->get_next_date_array( $dt{'sy'},
                                    $dt{'sm'}, $dt{'sd'} );
                            }
                            $dt{'emin'} += $main::repeat{'min'};
                            if ( $dt{'emin'} > 59 ) {
                                $dt{'emin'} -= 60;
                                $dt{'eh'}++;
                            }
                            $dt{'eh'} += $main::repeat{'h'};
                            if ( $dt{'eh'} > 23 ) {
                                $dt{'eh'} -= 24;
                                ( $dt{'ey'}, $dt{'em'}, $dt{'ed'} )
                                    = $this->get_next_date_array( $dt{'ey'},
                                    $dt{'em'}, $dt{'ed'} );
                            }
                        }
                        if (!$this->is_available_time(
                                $main::n,
                                \%dt,
                                \@available_time,
                                \@available_wday,
                                \@samezone_categories,
                                $field::for{'category'},
                                $category_id,
                                $category_name
                            )
                            )
                        {
                            $dt_is_valid = 0;
                            last;
                        }
                        elsif ( !$dt{'date'}
                            && @samezone_categories
                            && $event_margin )
                        {
                            my $dst = timelocal(
                                0,
                                ( ( $dt{'date'} ) ? 0 : $dt{'smin'} ),
                                ( ( $dt{'date'} ) ? 0 : $dt{'sh'} ),
                                $dt{'sd'},
                                ( $dt{'sm'} - 1 ),
                                $dt{'sy'}
                            );
                            my $det = timelocal(
                                0,
                                ( ( $dt{'date'} ) ? 0 : $dt{'emin'} ),
                                ( ( $dt{'date'} ) ? 0 : $dt{'eh'} ),
                                $dt{'ed'},
                                ( $dt{'em'} - 1 ),
                                $dt{'ey'}
                            );
                            (   $dt{'smin'}, $dt{'sh'}, $dt{'sd'},
                                $dt{'sm'},   $dt{'sy'}
                                )
                                = ( localtime( $dst - $event_margin * 60 ) )
                                [ 1 .. 5 ];
                            (   $dt{'emin'}, $dt{'eh'}, $dt{'ed'},
                                $dt{'em'},   $dt{'ey'}
                                )
                                = ( localtime( $det + $event_margin * 60 ) )
                                [ 1 .. 5 ];
                            $dt{'sm'}++;
                            $dt{'sy'} += 1900;
                            $dt{'em'}++;
                            $dt{'ey'} += 1900;

                            if (!$this->is_available_time(
                                    $main::n,
                                    \%dt,
                                    \@available_time,
                                    \@available_wday,
                                    \@samezone_categories,
                                    $field::for{'category'},
                                    $category_id,
                                    $category_name,
                                    1
                                )
                                )
                            {
                                push(
                                    @{ $this->{'dialog'} },
                                    '予定の前後は '
                                        . $event_margin
                                        . ' 分以上空けてください。'
                                );
                                $dt_is_valid = 0;
                                last;
                            }
                        }
                    }
                }

                #容量チェック
                my $repeat_total = $repeat_count || 1;
                if (   $dt_is_valid
                    && $main::dt{'freq'}
                    && !$main::dt{'indefinite'} )
                {
                    my @list = $this->get_date_list( \%main::dt );
                    if ($main::decompose) {
                        $repeat_total
                            = ( $repeat_count || 1 ) * scalar(@list);
                    }
                    else {
                        my $list = join( ',', @list );
                        my $available_list = bytes::substr( $list, 0,
                            $env::db_maxlength - 128, '' );
                        if ($list) {
                            push(
                                @{ $this->{'dialog'} },
                                '期限を確認してください。期限までが長過ぎるため予定を保存できませんでした。'
                            );
                            if ( $available_list
                                =~ /[^0-9]([0-9]{4})([0-9]{2})([0-9]{2})(?:>[0-9]*)?,?[^,]*?$/s
                                )
                            {
                                (   $main::dt{'uy'}, $main::dt{'um'},
                                    $main::dt{'ud'}
                                ) = ( $1, $2 + 0, $3 + 0 );
                            }
                            $dt_is_valid = 0;
                        }
                    }
                }
                if ( $repeat_total > 1000 ) {
                    push(
                        @{ $this->{'dialog'} },
                        '一度に1000件以上の予定を「個別に保存」もしくは「複製」することはできません。「期限」または「複製」を確認してください。'
                    );
                    $dt_is_valid = 0;
                }
            }
        }
        if ( !$dt_is_valid ) {
            push(
                @{ $this->{'dialog'} },
                '日付と時間の設定を確認してください。'
            ) if !ref $this->{'dialog'} || !@{ $this->{'dialog'} };
            $input_is_valid = 0;
        }

        #入力チェック
        foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
            my $id = $field::ORDER{$o};
            if (ref $field::obj{$id}
                && (!$field::obj{$id}->{'protected'}
                    || (   defined $main::n
                        && $main::n ne ''
                        && $this->is_secure(
                            'reviseProtectedFields', 0, 'revise'
                        )
                    )
                    || (( !defined $main::n || $main::n eq '' )
                        && $this->is_secure(
                            'inputProtectedFields', 0, 'input'
                        )
                    )
                )
                )
            {
                $input_is_valid = 0
                    if !$field::obj{$id}->is_valid_data(
                    ( defined $original_n && $original_n ne '' )
                    ? $default::file . '/' . $original_n
                    : ( defined $main::n && $main::n ne '' )
                    ? $default::file . '/' . $main::n
                    : '' );
            }
            else {
                delete $main::data{$id};
                delete $main::option{$id};
            }
        }

        #EMAILチェック
        if (   $field::for{'mail'}
            && ref $field::obj{ $field::for{'mail'} }
            && $field::obj{ $field::for{'mail'} }->{'e-mail'}
            && @admins
            && (!$main::data{ $field::for{'mail'} }{'str'}
                || !grep( $_ eq $main::data{ $field::for{'mail'} }{'str'},
                    @admins )
            )
            )
        {
            push(
                @{ $this->{'dialog'} },
                $field::obj{ $field::for{'mail'} }->name
                    . 'を確認してください。'
                    . $category_name
                    . '担当者メールアドレスに変更しました。'
            );
            $main::data{ $field::for{'mail'} }{'str'} = $admins[0];
            $input_is_valid = 0;
        }
        if (( !defined $main::n || $main::n eq '' )
            && ( !defined $setting::security{'revise'}{'passwordCheck'}
                || $setting::security{'revise'}{'passwordCheck'} ne
                'nocheck' )
            && $this->is_secure('revise')
            && $this->is_registered_email
            && !$this->is_secure_email
            && $this->email_field_enabled
            )
        {
            push(
                @{ $this->{'dialog'} },
                '登録メールアドレスがみつかりません。'
            );
            $main::data{ $field::for{'mail'} }{'str'} = $main::email
                if $field::for{'mail'}
                && $main::data{ $field::for{'mail'} }{'str'} ne '';
            $input_is_valid = 0;
        }

        #名前チェック
        if (   $field::NAMECHECK
            && $input_is_valid
            && !$this->is_secure_name(1) )
        {
            $input_is_valid = 0;
        }

        #入力エラー
        if ( !$input_is_valid || $main::cancel ) {

            #確認画面
        }
        else {

            #イベントの日時
            $event_dt = $this->get_event_dt(
                $main::dt{'sy'},         $main::dt{'sm'},
                $main::dt{'sd'},         $main::dt{'sh'},
                $main::dt{'smin'},       $main::dt{'ey'},
                $main::dt{'em'},         $main::dt{'ed'},
                $main::dt{'eh'},         $main::dt{'emin'},
                $main::dt{'uy'},         $main::dt{'um'},
                $main::dt{'ud'},         $main::dt{'date'},
                $main::dt{'indefinite'}, $main::dt{'freq'},
                $main::dt{'by'},         $main::dt{'interval'},
                $main::dt{'byday'},      $se_gap
            );
            $event_dt
                .= '<div class="note">'
                . (
                ( $main::repeat{'h'} ) ? $main::repeat{'h'} . '時間' : '' )
                . (
                ( $main::repeat{'min'} ) ? $main::repeat{'min'} . '分' : '' )
                . 'ごとに'
                . (
                ( $main::repeat{'uh'} ) ? $main::repeat{'uh'} . '時' : '' )
                . ( ( $main::repeat{'umin'} )
                ? $main::repeat{'umin'} . '分'
                : '' )
                . 'まで繰り返す。</div>'
                if !$main::dt{'date'} && $is_repeatable;
            $event_dt .= '<div class="note">個別に保存する。</div>'
                if $main::dt{'freq'} && $main::decompose;

            if ( !defined $main::n || $main::n eq '' ) {
                if (  !$setting::general{'skipConfirmation'}
                    && $input_only_user
                    && !$cgi::confirmation_disabled
                    && !$main::ok )
                {

                    #ページヘッダ
                    $this->set_opener_location(1);
                    $this->print_page_header('editor');

                    #終日補正復元
                    ( $main::dt{'ey'}, $main::dt{'em'}, $main::dt{'ed'} )
                        = $this->get_prev_date_array( $main::dt{'ey'},
                        $main::dt{'em'}, $main::dt{'ed'} )
                        if $main::dt{'date'};

                    #フォーム
                    print $this->form_start(
                        'key'    => $setting::security_key,
                        'name'   => 'input',
                        'method' => 'post',
                        'onsubmit' =>
                            'return cmonos.form.item(this).isNotSubmitted()',
                        'required' =>
                            [ @setting::form, 'dt', 'repeat', 'decompose' ]
                    );
                    print '<input type="hidden" name="a" value="w" />';
                    print '<input type="hidden" name="v" value="'
                        . $this->escape($main::v) . '" />'
                        if defined $main::v && $main::v;

                    print '<p id="form-confirmation">'
                        . ( ($default::english)
                        ? 'Confirmation.'
                        : '下記内容に問題がなければ「保存」ボタンを押してください。'
                        ) . '</p>';
                    print '<table class="field-table"><tbody>';

                    #予定の日時
                    print
                        '<tr><td class="field-name"><span class="datatitle">日時</span></td><td class="field">';
                    print $event_dt;
                    print '</td></tr>';
                    print '</tbody><tbody>';

                    my $path = $default::file . '/'
                        . ( ( defined $main::n ) ? $main::n : '' );
                    my $cookie_enabled = 0;
                    foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
                        my $id = $field::ORDER{$o};
                        if (ref $field::obj{$id}
                            && (!$field::obj{$id}->{'protected'}
                                || $this->is_secure(
                                    'inputProtectedFields', 0, 'input'
                                )
                            )
                            && ref $main::data{$id}
                            && ( ( defined $main::n && $main::n ne '' )
                                || $field::obj{$id}->{'class'} ne
                                'user_xtime' )
                            )
                        {
                            $cookie_enabled = 1
                                if $field::obj{$id}->cookie_enabled
                                && $field::obj{$id}->{'cookie'};
                            $field::obj{$id}
                                ->print_confirmation_form_table_row( $path,
                                $main::c, $main::n );
                        }
                    }
                    print '</tbody><tbody>';
                    print
                        '<tr class="tbody-start"><td class="submit-box" colspan="2">';
                    print
                        '<input type="hidden" name="set_cookie" value="1" />'
                        if $cookie_enabled && $main::set_cookie;
                    print '<input type="submit" name="cancel=1" value="'
                        . (
                        ($default::english) ? 'Cancel' : 'キャンセル' )
                        . '" class="cancel" />';
                    print '<input type="submit" name="ok=1" value="'
                        . ( ($default::english) ? 'Save' : '保存' )
                        . '" class="save" />';
                    print '</td></tr>';
                    print '</tbody></table></form>';

                    print '<div id="close-button">'
                        . $this->close_button(
                        $this->{'return_url'},
                        $setting::page{'closeButton'},
                        $setting::page{'backButton'}
                        ) . '</div>';

                    #ページフッタ
                    $this->print_page_footer('editor');
                    return;

                    #新規登録
                }
                else {
                    $main::n = $this->new_record_id;
                    $this->{'new_data'} = 1;
                }
            }
        }
    }

    #書き込み/新規入力の告知
    if ( $input_is_valid && ( !$input_only_user || $this->{'new_data'} ) ) {
        $this->directory_copy(
            $default::file . '/' . $original_n,
            $default::file . '/' . $main::n,
            0, [ '^[0-9]+\.dat$', '\.html$', '\.js$' ]
            )
            if $main::copy
            && $original_n
            && -d $default::file . '/' . $original_n;
        my $first_n;
        my %dt = %main::dt;
    REPEAT:
        for ( my $i = 0; $i <= $repeat_count; $i++ ) {
            if ($i) {
                %main::dt = %dt;
                $main::dt{'smin'} += $main::repeat{'min'};
                if ( $main::dt{'smin'} > 59 ) {
                    $main::dt{'smin'} -= 60;
                    $main::dt{'sh'}++;
                }
                $main::dt{'sh'} += $main::repeat{'h'};
                if ( $main::dt{'sh'} > 23 ) {
                    $main::dt{'sh'} -= 24;
                    ( $main::dt{'sy'}, $main::dt{'sm'}, $main::dt{'sd'} )
                        = $this->get_next_date_array( $main::dt{'sy'},
                        $main::dt{'sm'}, $main::dt{'sd'} );
                    ( $main::dt{'uy'}, $main::dt{'um'}, $main::dt{'ud'} )
                        = $this->get_next_date_array( $main::dt{'uy'},
                        $main::dt{'um'}, $main::dt{'ud'} );
                    @{ $main::dt{'byday'} } = map {
                        $this->number_to_wday(
                            ( $this->wday_to_number($_) + 1 ) % 7 )
                    } @{ $main::dt{'byday'} } if ref $main::dt{'byday'};
                }
                $main::dt{'emin'} += $main::repeat{'min'};
                if ( $main::dt{'emin'} > 59 ) {
                    $main::dt{'emin'} -= 60;
                    $main::dt{'eh'}++;
                }
                $main::dt{'eh'} += $main::repeat{'h'};
                if ( $main::dt{'eh'} > 23 ) {
                    $main::dt{'eh'} -= 24;
                    ( $main::dt{'ey'}, $main::dt{'em'}, $main::dt{'ed'} )
                        = $this->get_next_date_array( $main::dt{'ey'},
                        $main::dt{'em'}, $main::dt{'ed'} );
                }
                %dt = %main::dt;
            }
            if ( $main::dt{'freq'} && $main::decompose ) {
                my @date = $this->get_date_list( \%main::dt );
                $main::dt{'freq'} = '';
            DATE:
                foreach my $d (@date) {
                    if ( $d =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/ ) {
                        ( $main::dt{'sy'}, $main::dt{'sm'}, $main::dt{'sd'} )
                            = ( $1 + 0, $2 + 0, $3 + 0 );
                        next DATE
                            if @available_wday
                            && $dt{'freq'} eq 'weekly'
                            && !$this->is_available_wday(
                            \@available_wday, $main::dt{'sy'},
                            $main::dt{'sm'},  $main::dt{'sd'}
                            );
                        ( $main::dt{'ed'}, $main::dt{'em'}, $main::dt{'ey'} )
                            = (
                            localtime(
                                timelocal(
                                    0,
                                    (   ( $main::dt{'date'} ) ? 0
                                        : $main::dt{'smin'}
                                    ),
                                    (   ( $main::dt{'date'} ) ? 0
                                        : $main::dt{'sh'}
                                    ),
                                    $main::dt{'sd'},
                                    ( $main::dt{'sm'} - 1 ),
                                    $main::dt{'sy'}
                                ) + $se_gap
                            )
                            )[ 3 .. 5 ];
                        $main::dt{'em'}++;
                        $main::dt{'ey'} += 1900;
                        if ((   defined $setting::general{'boundsStart'}
                                && $setting::general{'boundsStart'} ne ''
                            )
                            || ( defined $setting::general{'boundsEnd'}
                                && $setting::general{'boundsEnd'} ne '' )
                            )
                        {
                            my $dt_start_time = timelocal(
                                0,
                                (   ( $main::dt{'date'} ) ? 0
                                    : $main::dt{'smin'}
                                ),
                                (   ( $main::dt{'date'} ) ? 0
                                    : $main::dt{'sh'}
                                ),
                                $main::dt{'sd'},
                                ( $main::dt{'sm'} - 1 ),
                                $main::dt{'sy'}
                            );
                            my $dt_end_time = timelocal(
                                0,
                                (   ( $main::dt{'date'} ) ? 0
                                    : $main::dt{'emin'}
                                ),
                                (   ( $main::dt{'date'} ) ? 0
                                    : $main::dt{'eh'}
                                ),
                                $main::dt{'ed'},
                                ( $main::dt{'em'} - 1 ),
                                $main::dt{'ey'}
                            );
                            if ($this->is_out_of_bounds(
                                    $dt_start_time,
                                    $dt_end_time,
                                    $setting::general{'boundsStart'},
                                    $setting::general{'boundsStartUnit'},
                                    $setting::general{'boundsStartDirection'},
                                    1
                                )
                                || $this->is_out_of_bounds(
                                    $dt_start_time,
                                    $dt_end_time,
                                    $setting::general{'boundsEnd'},
                                    $setting::general{'boundsEndUnit'},
                                    $setting::general{'boundsEndDirection'}
                                )
                                )
                            {
                                next DATE;
                            }
                        }
                        if ($first_n) {
                            $main::n = $this->new_record_id;
                            $this->{'new_data'} = 1;
                            $this->directory_copy(
                                $default::file . '/' . $first_n,
                                $default::file . '/' . $main::n,
                                0,
                                [ '\.dat$', '\.html$', '\.js$' ]
                            ) if -d $default::file . '/' . $first_n;
                        }
                        $this->dbm(
                            $this->text_dt(
                                $this->get_event_dt(
                                    $main::dt{'sy'},
                                    $main::dt{'sm'},
                                    $main::dt{'sd'},
                                    $main::dt{'sh'},
                                    $main::dt{'smin'},
                                    $main::dt{'ey'},
                                    $main::dt{'em'},
                                    $main::dt{'ed'},
                                    $main::dt{'eh'},
                                    $main::dt{'emin'},
                                    $main::dt{'uy'},
                                    $main::dt{'um'},
                                    $main::dt{'ud'},
                                    $main::dt{'date'},
                                    $main::dt{'indefinite'},
                                    $main::dt{'freq'},
                                    $main::dt{'by'},
                                    $main::dt{'interval'},
                                    $main::dt{'byday'},
                                    $se_gap
                                )
                            ),
                            $first_n
                        );
                        $first_n = $main::n if !$first_n;
                    }
                }
            }
            else {
                if ($first_n) {
                    $main::n = $this->new_record_id;
                    $this->{'new_data'} = 1;
                    $this->directory_copy(
                        $default::file . '/' . $first_n,
                        $default::file . '/' . $main::n,
                        0, [ '\.dat$', '\.html$', '\.js$' ]
                    ) if -d $default::file . '/' . $first_n;
                }
                $this->dbm(
                    $this->text_dt(
                        $this->get_event_dt(
                            $main::dt{'sy'},         $main::dt{'sm'},
                            $main::dt{'sd'},         $main::dt{'sh'},
                            $main::dt{'smin'},       $main::dt{'ey'},
                            $main::dt{'em'},         $main::dt{'ed'},
                            $main::dt{'eh'},         $main::dt{'emin'},
                            $main::dt{'uy'},         $main::dt{'um'},
                            $main::dt{'ud'},         $main::dt{'date'},
                            $main::dt{'indefinite'}, $main::dt{'freq'},
                            $main::dt{'by'},         $main::dt{'interval'},
                            $main::dt{'byday'},      $se_gap
                        )
                    ),
                    $first_n
                );
                $first_n = $main::n if !$first_n;
            }
        }
        undef %main::repeat;
        undef $main::copy;
        undef $main::decompose;

        if ( $setting::general{'notice'} && $this->{'new_record'} ) {

            #メール設定読み込み
            $this->import_db( $default::preference_dir . '/mail',
                \%setting::mail, 1 );

            #投稿通知
            $this->send_notice_mail( $this->text_dt($event_dt) );
            undef %setting::mail;
        }
    }

    #データ修正パスワードチェック
    if ( defined $main::n && $main::n ne '' && $input_only_user ) {

        #新規入力
        if ( defined $this->{'new_data'} && $this->{'new_data'} ) {
            my ( $title, $name, %thanks );
            if ( defined $field::for{'name'} && $field::for{'name'} ) {
                $name = $this->detag_text(
                    $field::obj{ $field::for{'name'} }->usual_format(
                        $main::data{ $field::for{'name'} }{'str'}
                    )
                );
            }
            $name = ($default::english) ? 'Anonymous' : '匿名'
                if !defined $name || $name eq '';
            if ( defined $field::for{'title'} && $field::for{'title'} ) {
                $title = $this->detag_text(
                    $field::obj{ $field::for{'title'} }->usual_format(
                        $main::data{ $field::for{'title'} }{'str'}
                    )
                );
            }
            $title = ($default::english) ? 'This event' : 'これ'
                if !defined $title || $title eq '';

            #入力お礼設定読み込み
            $this->import_db( $default::preference_dir . '/thanks',
                \%thanks );

            #書き込みお礼
            if ( defined $thanks{'thanks'} && $thanks{'thanks'} ne '' ) {
                $thanks{'thanks'} =~ s/<name>/$name/igs;
                $thanks{'thanks'} =~ s/<title>/$title/igs;
                $thanks{'thanks'} =~ s/<dt>/$event_dt/igs;
            }
            if ( defined $thanks{'thankstitle'}
                && $thanks{'thankstitle'} ne '' )
            {
                $thanks{'thankstitle'} =~ s/<name>/$name/igs;
                $thanks{'thankstitle'} =~ s/<title>/$title/igs;
                $thanks{'thankstitle'} =~ s/<dt>/$event_dt/igs;
                $this->push_window_title( $thanks{'thankstitle'} );
                $setting::page{'editor'}{'titleText'}
                    =~ s/((<[^>]+>)*)[^<]+/$1$thanks{'thankstitle'}/s
                    if defined $setting::page{'editor'}{'titleText'}
                    && $setting::page{'editor'}{'titleText'} ne '';
            }
            $this->set_opener_location;
            $this->print_page_header('editor');
            if ( defined $thanks{'thanks'} && $thanks{'thanks'} ne '' ) {
                $this->tag_fix( \$thanks{'thanks'}, 2, 0, 0, 1 );
                print $thanks{'thanks'};
            }
            else {
                print '<p>登録されました。</p>';
            }
            print '<div id="close-button">'
                . $this->close_button(
                $this->{'return_url'},
                $setting::page{'closeButton'},
                $setting::page{'backButton'}
                ) . '</div>';
            $this->print_page_footer('editor');
            return;

#特殊なケース
#} elsif (!$this->email_field_enabled) {
#	$this->print_dialog($setting::key,'メールアドレスフィールドが存在しません。');

            #パスワードチェック
        }
        elsif ( defined $main::a && $main::a eq 'w' ) {
            $this->print_password_check_form(
                'editor',
                (   ( $this->{'user_address'} ) ? $this->{'user_address'}
                    : 'none'
                ),
                (   ( $this->{'need_new_pswd'} ) ? $this->{'need_new_pswd'}
                    : 0
                ),
                ['dt']
            );
        }
        else {
            $this->print_password_check_form( 'editor', '', 0, ['dt'] );
        }
    }

    if ( $main::a && $original_n && $main::copy ) {
        $main::n = $original_n;
    }

    #ページヘッダ
    $this->set_onload('cmonos.validate.calendarDatetime.load();');
    $this->set_javascript_link( $env::javascript . '/dt.js' );
    $this->set_opener_location(
        ( $input_is_valid && !$main::cancel ) ? 0 : 1 );
    $this->print_page_header('editor');

    #終日補正復元
    ( $main::dt{'ey'}, $main::dt{'em'}, $main::dt{'ed'} )
        = $this->get_prev_date_array( $main::dt{'ey'}, $main::dt{'em'},
        $main::dt{'ed'} )
        if $main::dt{'date'};

    #フォーム
    print $this->form_start(
        'key'      => $setting::security_key,
        'name'     => 'input',
        'multi'    => $cgi::enctype_multipart,
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="w" />';
    print '<input type="hidden" name="opener" value="'
        . $this->escape($main::opener) . '" />'
        if defined $main::opener && $main::opener;

    #ファイル格納ディレクトリ
    my $path
        = $default::file . '/' . ( ( defined $main::n ) ? $main::n : '' );

    print '<table class="field-table"><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle"><strong>日時</strong></span></td>';
    print '<td class="field">';
    print '<div id="dt-start">';
    $this->print_date_form('s');
    print '<span id="dt-start-time">';
    $this->print_time_form('s');
    print '</span>から</div><!--m:content <hr> -->';
    print '<div id="dt-end">';
    $this->print_date_form('e')
        if !$setting::general{'dateDisabled'}
        || !$setting::general{'endDateDisabled'};
    print '<span id="dt-end-time">';
    $this->print_time_form('e');
    print '</span>まで</div>';
    print
        '<div class="extra-field"><label><input type="checkbox" name="dt:date" id="dt:date" value="1"'
        . (
        (   $main::dt{'date'}
                || ( $main::dtstart && $main::dtstart =~ /^[0-9]{8}$/ )
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="cmonos.validate.calendarDatetime.fix(\'s\')" /> 終日</label></div>'
        if !$setting::general{'dateDisabled'};
    print '<div id="dt-span"></div>' if $setting::general{'freqDisabled'};
    print '</td>';
    print '</tr>';

    if ( !$setting::general{'freqDisabled'} ) {
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><label for="dt:freq">繰り返し</label></td>';
        print '<td class="field">';
        print '<fieldset id="dt-freq-settings"><legend>';
        print
            '<select name="dt:freq" id="dt:freq" onchange="cmonos.validate.calendarDatetime.setRRuleField()">';
        print '<option value=""'
            . ( ( !$main::dt{'freq'} || $main::dt{'freq'} eq '' )
            ? ' selected="selected"'
            : '' )
            . '>繰り返しなし</option>';
        print '<option value="daily"'
            . ( ( $main::dt{'freq'} && $main::dt{'freq'} eq 'daily' )
            ? ' selected="selected"'
            : '' )
            . '>毎日</option>';
        print '<option value="weekly"'
            . ( ( $main::dt{'freq'} && $main::dt{'freq'} eq 'weekly' )
            ? ' selected="selected"'
            : '' )
            . '>毎週</option>';
        print '<option value="monthly"'
            . ( ( $main::dt{'freq'} && $main::dt{'freq'} eq 'monthly' )
            ? ' selected="selected"'
            : '' )
            . '>毎月</option>';
        print '<option value="yearly"'
            . ( ( $main::dt{'freq'} && $main::dt{'freq'} eq 'yearly' )
            ? ' selected="selected"'
            : '' )
            . '>毎年</option>';
        print '</select><span id="dt-rrule-title"></span></legend>';
        print '<table class="sub-field-table" id="dt-rrule-settings">';
        print '<tbody>';
        print '<tr id="dt-interval-field">';
        print
            '<td class="sub-field-name"><label for="dt:interval">間隔</label></td>';
        print '<td class="sub-field">';
        print
            '<select name="dt:interval" id="dt:interval" onchange="cmonos.validate.calendarDatetime.setRRuleTitle()">';

        for ( my $i = 1; $i <= 14; $i++ ) {
            print '<option value="'
                . $i . '"'
                . ( ( $main::dt{'interval'} && $i == $main::dt{'interval'} )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $i
                . '</option>';
        }
        print '</select>';
        print '<span id="dt-interval-unit">日|週|ヶ月|年 ごと</span>';
        print
            ' <label id="dt-decompose"><input type="checkbox" name="decompose" id="decompose" value="1"'
            . ( ($main::decompose) ? ' checked="checked"' : '' )
            . ' onclick="cmonos.validate.calendarDatetime.setHourly()" />個別に保存</label>';
        print '</td>';
        print '</tr>';
        print
            '<tr id="dt-byday-field" title="毎週繰り返す場合のみ有効">';
        print
            '<td class="sub-field-name"><span class="datatitle">曜日</span></td>';
        print '<td class="sub-field">';
        print
            ' <label><input type="checkbox" name="dt:byday" id="dt-byday-su" value="SU" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . (
            (   ref $main::dt{'byday'}
                    && grep( $_ eq 'SU', @{ $main::dt{'byday'} } )
            ) ? ' checked="checked"' : ''
            ) . ' />日</label>';
        print
            ' <label><input type="checkbox" name="dt:byday" id="dt-byday-mo" value="MO" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . (
            (   ref $main::dt{'byday'}
                    && grep( $_ eq 'MO', @{ $main::dt{'byday'} } )
            ) ? ' checked="checked"' : ''
            ) . ' />月</label>';
        print
            ' <label><input type="checkbox" name="dt:byday" id="dt-byday-tu" value="TU" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . (
            (   ref $main::dt{'byday'}
                    && grep( $_ eq 'TU', @{ $main::dt{'byday'} } )
            ) ? ' checked="checked"' : ''
            ) . ' />火</label>';
        print
            ' <label><input type="checkbox" name="dt:byday" id="dt-byday-we" value="WE" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . (
            (   ref $main::dt{'byday'}
                    && grep( $_ eq 'WE', @{ $main::dt{'byday'} } )
            ) ? ' checked="checked"' : ''
            ) . ' />水</label>';
        print
            ' <label><input type="checkbox" name="dt:byday" id="dt-byday-th" value="TH" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . (
            (   ref $main::dt{'byday'}
                    && grep( $_ eq 'TH', @{ $main::dt{'byday'} } )
            ) ? ' checked="checked"' : ''
            ) . ' />木</label>';
        print
            ' <label><input type="checkbox" name="dt:byday" id="dt-byday-fr" value="FR" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . (
            (   ref $main::dt{'byday'}
                    && grep( $_ eq 'FR', @{ $main::dt{'byday'} } )
            ) ? ' checked="checked"' : ''
            ) . ' />金</label>';
        print
            ' <label><input type="checkbox" name="dt:byday" id="dt-byday-sa" value="SA" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . (
            (   ref $main::dt{'byday'}
                    && grep( $_ eq 'SA', @{ $main::dt{'byday'} } )
            ) ? ' checked="checked"' : ''
            ) . ' />土</label>';
        print '</td>';
        print '</tr>';
        print
            '<tr id="dt-by-field" title="毎月繰り返す場合のみ有効">';
        print
            '<td class="sub-field-name"><span class="datatitle">繰り返しの基準</span></td>';
        print '<td class="sub-field">';
        print
            ' <label><input type="radio" name="dt:by" id="dt-bymday" value="bymday" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . ( ( $main::dt{'by'} && $main::dt{'by'} eq 'bymday' )
            ? ' checked="checked"'
            : '' )
            . ' />日付</label>';
        print
            ' <label><input type="radio" name="dt:by" id="dt-bywday" value="bywday" onclick="cmonos.validate.calendarDatetime.setRRuleTitle()"'
            . ( ( $main::dt{'by'} && $main::dt{'by'} eq 'bywday' )
            ? ' checked="checked"'
            : '' )
            . ' />曜日</label>';
        print '</td>';
        print '</tr>';
        print '<tr id="dt-until">';
        print
            '<td class="sub-field-name"><span class="datatitle">期限</span></td>';
        print '<td class="sub-field">';
        print '<span id="dt-until-date">';
        $this->print_date_form('u');
        print '</span>';
        print
            ' <label><input type="checkbox" name="dt:indefinite" id="dt:indefinite" value="1" onclick="cmonos.element.item(\'dt-until-date\',\'dt-decompose\').disappear(this.checked);if(this.checked)cmonos.form.checkboxSyncById(\'decompose\',false);cmonos.validate.calendarDatetime.setHourly();"'
            . ( ( $main::dt{'indefinite'} ) ? ' checked="checked"' : '' )
            . ' />無期限</label>';
        print $this->javascript(
            'cmonos.element.item(\'dt-until-date\',\'dt-decompose\').disappear(true);'
        ) if $main::dt{'indefinite'};
        print '</td>';
        print '</tr>';
        print
            '<tr id="dt-repeat" title="この予定を指定時間ごとに複製する">';
        print
            '<td class="sub-field-name"><span class="datatitle">複製</span></td>';
        print '<td class="sub-field">';
        print '<label><select name="repeat:h"><option value=""></option>';

        for ( my $i = 0; $i <= 12; $i++ ) {
            print '<option value="'
                . $i . '"'
                . ( ( defined $main::repeat{'h'} && $i eq $main::repeat{'h'} )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $i
                . '</option>';
        }
        print '</select>';
        print '時間';
        print '<label><select name="repeat:min"><option value=""></option>';
        for ( my $i = 0; $i <= 59; $i++ ) {
            print '<option value="'
                . $i . '"'
                . (
                (   defined $main::repeat{'min'} && $i eq $main::repeat{'min'}
                ) ? ' selected="selected"' : ''
                )
                . '>'
                . $i
                . '</option>';
        }
        print '</select>';
        print '分ごと</label>に<br />';
        print
            '<label><select name="repeat:uh" id="repeat:uh"><option value=""></option>';
        for ( my $i = 0; $i <= 23; $i++ ) {
            print '<option value="'
                . $i . '"'
                . (
                ( defined $main::repeat{'uh'} && $i eq $main::repeat{'uh'} )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $i
                . '</option>';
        }
        print '</select>';
        print '時　';
        print
            '<label><select name="repeat:umin" id="repeat:umin"><option value=""></option>';
        for ( my $i = 0; $i <= 59; $i++ ) {
            print '<option value="'
                . $i . '"'
                . (
                (   defined $main::repeat{'umin'}
                        && $i eq $main::repeat{'umin'}
                ) ? ' selected="selected"' : ''
                )
                . '>'
                . $i
                . '</option>';
        }
        print '</select>';
        print '分まで(開始時)</label>';
        print '</td>';
        print '</tr>';
        print '</tbody>';
        print '</table>';
        print '</fieldset>';
        print '</td>';
        print '</tr>';
    }

    print '</tbody><tbody>';

    my $cookie_enabled = 0;
    foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
        my $id = $field::ORDER{$o};
        if (ref $field::obj{$id}
            && (!$field::obj{$id}->{'protected'}
                || (   defined $main::n
                    && $main::n ne ''
                    && $this->is_secure( 'reviseProtectedFields', 0,
                        'revise' ) )
                || ( ( !defined $main::n || $main::n eq '' )
                    && $this->is_secure( 'inputProtectedFields', 0, 'input' )
                )
            )
            && ( ( defined $main::n && $main::n ne '' )
                || $field::obj{$id}->{'class'} ne 'user_xtime' )
            )
        {
            $cookie_enabled = 1
                if $field::obj{$id}->cookie_enabled
                && $field::obj{$id}->{'cookie'};
            print $field::obj{$id}->form( $main::n, '', $path );
        }
    }

    print '</tbody><tbody>';
    print '<tr class="tbody-start">';

    print '<td class="submit-box" colspan="2">';
    if ( defined $main::n && $main::n ne '' ) {
        print '<div id="submit-button">';
        print
            '<label title="上記予定を別の予定として保存します。"><input type="checkbox" name="copy" value="1"'
            . ( ($main::copy) ? ' checked="checked"' : '' )
            . ' />別の予定として保存する</label> ';
    }
    if ( defined $cookie_enabled && $cookie_enabled ) {
        print
            '<label title="同じ内容を入力することが多いいくつかの項目を記憶します。"><input type="checkbox" name="set_cookie" value="1"'
            . ( ( $cookie::read{$cookie::name} || $main::set_cookie )
            ? ' checked="checked"'
            : '' )
            . ' />情報を記憶する</label> ';
    }
    print
        '<input type="submit" name="submit_button" value="保存" class="save" />';
    print '</div>' if defined $main::n && $main::n ne '';

    if (   defined $main::n
        && $main::n ne ''
        && $this->is_secure( 'delete', 0, 'revise' ) )
    {
        print '<div id="delete-button">';
        print
            '<input type="submit" name="delete=1" value="削除" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\')" />';
        print '</div>';
    }

    print '</td></tr></tbody></table></form>';
    print '<div id="close-button">'
        . $this->close_button(
        $this->{'return_url'},
        $setting::page{'closeButton'},
        $setting::page{'backButton'}
        ) . '</div>';

    #ページフッタ
    $this->print_page_footer('editor');
}

sub text_dt ($$) {
    my ( $this, $event_dt ) = @_;
    $event_dt =~ s/<.*?>/\x0A/sg;
    $event_dt =~ s/\x0A+/\x0A/sg;
    $event_dt =~ s/\s+$//s;
    $event_dt =~ s/^\s+//s;
    return $event_dt;
}

sub is_out_of_bounds ($$$$$$;$) {
    my ( $this, $dt_start, $dt_end, $bounds, $bounds_unit, $bounds_direction,
        $from )
        = @_;
    if ( defined $bounds && $bounds ne '' ) {
        $bounds_unit = 'day' if !$bounds_unit;
        my $time = time;
        if ( $bounds_unit eq 'min' ) {
            $bounds *= 60;
        }
        elsif ( $bounds_unit eq 'hour' ) {
            $bounds *= 3600;
        }
        elsif ( $bounds_unit eq 'month' ) {
            $bounds *= 2592000;
        }
        elsif ( $bounds_unit eq 'year' ) {
            $bounds *= 31536000;
        }
        else {
            if ( $bounds_unit eq 'week' ) {
                $bounds *= 604800;
            }
            else {
                $bounds *= 86400;
            }
            my ( $sec, $min, $hour ) = localtime();
            $time -= $hour * 3600 + $min * 60 + $sec;
        }
        if ( $bounds_direction && $bounds_direction eq 'before' ) {
            $bounds = $time - $bounds;
        }
        else {
            $bounds = $time + $bounds;
        }
        if (   ( $from && ( $bounds > $dt_start || $bounds > $dt_end ) )
            || ( !$from && ( $bounds < $dt_start || $bounds < $dt_end ) ) )
        {
            return 1;
        }
    }
    return 0;
}

sub timeunit_to_sec ($$$) {
    my ( $this, $t, $unit ) = @_;
    if ( defined $t && $t ne '' ) {
        $unit = 'min' if !$unit;
        if ( $unit eq 'min' ) {
            $t *= 60;
        }
        elsif ( $unit eq 'hour' ) {
            $t *= 3600;
        }
        elsif ( $unit eq 'month' ) {
            $t *= 2592000;
        }
        elsif ( $unit eq 'year' ) {
            $t *= 31536000;
        }
        elsif ( $unit eq 'week' ) {
            $t *= 604800;
        }
        else {
            $t *= 86400;
        }
        return $t;
    }
    return 0;
}

sub bounds_to_datetime ($$$$$) {
    my ( $this, $bounds, $bounds_unit, $bounds_direction, $from ) = @_;
    if ( defined $bounds && $bounds ne '' ) {
        $bounds_unit = 'day' if !$bounds_unit;
        return (
            ( !$bounds ) ? ' 現在'
            : ' '
                . $bounds
                . $this->timeunit_to_japanese($bounds_unit)
                . (
                ( $bounds_direction && $bounds_direction eq 'before' )
                ? '前'
                : '後'
                )
        ) . ( ($from) ? '以降 ' : 'まで ' );
    }
    return '';
}

sub timeunit_to_japanese ($$) {
    my ( $this, $unit ) = @_;
    if ( $unit eq 'min' ) {
        $unit = '分';
    }
    elsif ( $unit eq 'hour' ) {
        $unit = '時間';
    }
    elsif ( $unit eq 'month' ) {
        $unit = 'ヶ月';
    }
    elsif ( $unit eq 'year' ) {
        $unit = '年';
    }
    elsif ( $unit eq 'week' ) {
        $unit = '週';
    }
    else {
        $unit = '日';
    }
    return $unit;
}

sub wdaynum_to_name ($@) {
    my ( $this, @wday ) = @_;
    my @name = (
        '日', '月', '火', '水',
        '木', '金', '土', '(休日は例外)'
    );
    return join( '・', map( $name[$_], @wday ) );
}

sub print_date_form ($$) {
    my ( $this, $t ) = @_;
    my $is_not_date = ( $main::dtstart && $main::dtstart =~ /T/ ) ? 1 : 0;
    my @localtime
        = (    !$main::dt{ $t . 'y' }
            || !$main::dt{ $t . 'm' }
            || !$main::dt{ $t . 'd' } )
        ? localtime(
        ( ($main::dtstart) ? $this->dt_to_time($main::dtstart) : time ) + (
              ( $t eq 's' || ( $t eq 'e' && $is_not_date ) ) ? 1799
            : ( $t eq 'e' || $is_not_date ) ? 0
            :                                 174599
        )
        )
        : ();
    $main::dt{ $t . 'y' } = $localtime[5] + 1900 if !$main::dt{ $t . 'y' };
    print '<span class="date-selector"><label><select name="dt:'
        . $t
        . 'y" id="dt:'
        . $t
        . 'y" onchange="cmonos.validate.calendarDatetime.fix(\''
        . $t . '\')">';
    for ( my $i = -10; $i <= 10; $i++ ) {
        my $year = $i + $main::dt{ $t . 'y' };
        next if $year < 1970;
        last if $year > 2036;
        print '<option value="'
            . $year . '"'
            . ( ( $i == 0 ) ? ' selected="selected"' : '' ) . '>'
            . $year
            . '</option>';
    }
    print '</select>';
    print '年';
    print '</label>';
    $main::dt{ $t . 'm' } = $localtime[4] + 1 if !$main::dt{ $t . 'm' };
    print '<label><select name="dt:'
        . $t
        . 'm" id="dt:'
        . $t
        . 'm" onchange="cmonos.validate.calendarDatetime.fix(\''
        . $t . '\')">';
    for ( my $i = 1; $i <= 12; $i++ ) {
        print '<option value="'
            . $i . '"'
            . (
            ( $i == $main::dt{ $t . 'm' } ) ? ' selected="selected"' : '' )
            . '>'
            . $i
            . '</option>';
    }
    print '</select>';
    print '月';
    print '</label>';
    $main::dt{ $t . 'd' } = $localtime[3] if !$main::dt{ $t . 'd' };
    print '<label><select name="dt:'
        . $t
        . 'd" id="dt:'
        . $t
        . 'd" onchange="cmonos.validate.calendarDatetime.fix(\''
        . $t . '\')">';
    for ( my $i = 1; $i <= 31; $i++ ) {
        print '<option value="'
            . $i . '"'
            . (
            ( $i == $main::dt{ $t . 'd' } ) ? ' selected="selected"' : '' )
            . '>'
            . $i
            . '</option>';
    }
    print '</select>';
    print '日';
    print '</label>';
    print
        '<a href="javascript:void(0)" onclick="cmonos.tool.calendarPalette.open(\'dt:'
        . $t
        . 'y\',\'dt:'
        . $t
        . 'm\',\'dt:'
        . $t
        . 'd\',function () {cmonos.validate.calendarDatetime.fix(\''
        . $t
        . '\')})" title="クリックするとカレンダーから日付を設定できます。"><img src="'
        . $env::image_dir
        . '/tool/calendar.gif" width="15" height="15" border="0" alt="カレンダーパネル" /></a></span>';
}

sub print_time_form ($$) {
    my ( $this, $t ) = @_;
    my @localtime
        = (    !defined $main::dt{ $t . 'h' }
            || !defined $main::dt{ $t . 'min' } )
        ? localtime(
        (   ( $main::dtstart && $main::dtstart =~ /T/ )
            ? $this->dt_to_time($main::dtstart)
            : time
        ) + ( ( $t eq 's' ) ? 1799 : 5399 )
        )
        : ();
    $main::dt{ $t . 'h' } = $localtime[2] if !defined $main::dt{ $t . 'h' };
    print '<span class="time-selector"><label><select name="dt:'
        . $t
        . 'h" id="dt:'
        . $t
        . 'h" onchange="cmonos.validate.calendarDatetime.fix(\''
        . $t . '\')">';
    for ( my $i = 0; $i <= 23; $i++ ) {
        print '<option value="'
            . $i . '"'
            . (
            ( $i == $main::dt{ $t . 'h' } ) ? ' selected="selected"' : '' )
            . '>'
            . $i
            . '</option>';
    }
    print '</select>';
    print '時';
    print '</label>';
    $main::dt{ $t . 'min' } = ( $localtime[1] < 30 ) ? 0 : 30
        if !defined $main::dt{ $t . 'min' };
    print '<label><select name="dt:'
        . $t
        . 'min" id="dt:'
        . $t
        . 'min" onchange="cmonos.validate.calendarDatetime.fix(\''
        . $t . '\')">';
    for ( my $i = 0; $i <= 59; $i++ ) {
        if (  !$setting::general{'minScale'}
            || $i % $setting::general{'minScale'} == 0 )
        {
            print '<option value="'
                . $i . '"'
                . ( ( $i == $main::dt{ $t . 'min' } )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $i
                . '</option>';
        }
    }
    print '</select>';
    print '分';
    print '</label></span>';
}

########################################################################
#イベント作成者かどうか
sub is_secure_name ($;$) {
    my ( $this, $input_is_valid ) = @_;
    return $this->{'is_secure_name'}
        if !$input_is_valid && defined $this->{'is_secure_name'};
    if (   $field::NAMECHECK
        && $field::for{'name'}
        && $main::data{ $field::for{'name'} }{'str'} ne ''
        && $this->use_register )
    {
        if (   $main::shift_flag
            && $this->{'old_pswd'}
            && $this->{'old_email'} )
        {
            if ( defined $input_is_valid && $input_is_valid ) {
                if ($cgi::Register->remove(
                        $this->{'old_pswd'}, $this->{'old_email'}
                    )
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        'これまで登録されていたメールアドレス（'
                            . $this->{'old_email'}
                            . '）を抹消しました。'
                    );
                    $cgi::Register->success_dialog;
                }
            }
            else {
                push( @setting::form, 'shift_flag', 'old_pswd', 'old_email' );
            }
        }
        my ( $name, $fontcolor );
        $name
            = ( ref $main::data{ $field::for{'name'} }
                && defined $main::data{ $field::for{'name'} }{'str'} )
            ? $main::data{ $field::for{'name'} }{'str'}
            : '';
        $fontcolor = $1 if $name =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
        if (   defined $name
            && $name ne ''
            && !$this->is_same_name_and_email( $main::n, $name ) )
        {
            if ((      !defined $main::n
                    || $main::n eq ''
                    || $this->is_secure_email
                )
                && $cgi::Register->is_available_name(
                    $main::pswd, $main::email,
                    $name,       $security::registered
                )
                )
            {
                push( @{ $this->{'dialog'} },
                    $cgi::Register->success_dialog );
            }
            else {
                my @msg = $cgi::Register->error_dialog;
                push(
                    @{ $this->{'dialog'} },
                    (   (@msg)
                        ? @msg
                        : '認証に必要なメールアドレスが変更されました。'
                    )
                );
                $main::data{ $field::for{'name'} }{'str'}
                    = $name
                    . ( ( defined $fontcolor && $fontcolor ne '' )
                    ? '<>' . $fontcolor
                    : '' );
                $this->{'is_secure_name'} = 0;
                if ( $input_is_valid && $this->{'old_email'} ) {
                    return 1;
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'この予定は確定していません。'
                    );
                    return 0;
                }
            }
        }
    }
    $this->{'is_secure_name'} = 1;
    return 1;
}

sub is_same_name_and_email ($$$) {
    my ( $this, $n, $name ) = @_;
    return 0 if !defined $n || $n eq '' || !$this->{'is_admin'};
    my ( @check_fields, %db, $is_not_same );
    @check_fields = @{ $field::for{'mail_address'} }
        if ref $field::for{'mail_address'};
    return 1 if !@check_fields;
    push( @check_fields, $field::for{'name'} );
    foreach my $id (@check_fields) {
        if ( ref $field::obj{$id} ) {
            tie( %db, $env::dbm_file,
                $default::db . '/' . $field::obj{$id}->db,
                O_RDONLY, $env::dbm_mode );
            $is_not_same = 1
                if !ref tied %db
                || ( defined $db{ $this->encode_utf8($n) }
                && $this->decode_utf8( $db{ $this->encode_utf8($n) } ) ne
                $main::data{$id}{'str'} );
            untie(%db);
            return 0 if $is_not_same;
        }
    }
    $this->{'keep_original_user_info'} = 1;
    return 1;
}

######################################################################
# オープナーのURL
sub set_opener_location ($;$) {
    my ( $this, $url_only ) = @_;
    $this->{'return_url'}
        = (    ( !$main::dt{'sy'} || !$main::dt{'sm'} || !$main::dt{'sd'} )
            && $ENV{'HTTP_REFERER'}
            && $ENV{'HTTP_REFERER'} !~ /=DetailViewer/ )
        ? $ENV{'HTTP_REFERER'}
        : $cgi::path
        . '?m=Viewer'
        . ( ( $default::subdivided && $main::c ) ? '&c=' . $main::c : '' )
        . (
        ( $main::dt{'sy'} ) ? '&date='
            . $this->dt_stamp( $main::dt{'sy'}, $main::dt{'sm'},
            $main::dt{'sd'} ) : ''
        ) . ( ( $main::v && $main::v =~ /^[mwd]$/ ) ? '&v=' . $main::v : '' );
    $this->set_onload( 'cmonos.window.refreshOpener(\''
            . $this->escape( $this->{'return_url'} )
            . '\',true)' )
        if !$url_only;
}

######################################################################
# 新規入力のみのユーザーかどうか
sub is_input_only_user ($;$) {
    my ( $this, $check ) = @_;
    if ( !defined $main::n || $main::n eq '' ) {
        return (
            (   !defined $setting::security{'revise'}{'passwordCheck'}
                    || $setting::security{'revise'}{'passwordCheck'} ne
                    'nocheck'
            )
                && (
                !$this->is_secure('revise')
                || (   !$this->is_one_of_owners
                    && !$this->email_field_enabled($check) )
                )
        ) ? 1 : 0;
    }
    else {
        if ( defined $setting::security{'revise'}{'passwordCheck'}
            && $setting::security{'revise'}{'passwordCheck'} eq 'nocheck' )
        {
            $this->{'is_admin'} = 1
                if $this->is_secure('revise') && $this->is_one_of_owners;
            $this->is_authorized_email($check);
        }
        else {
            $this->{'is_admin'} = 1
                if !$this->is_authorized_email($check)
                && $this->is_secure('revise')
                && $this->is_one_of_owners;
        }
        return (
            (   !defined $setting::security{'revise'}{'passwordCheck'}
                    || $setting::security{'revise'}{'passwordCheck'} ne
                    'nocheck'
            )
                && ( !$this->is_secure_email($check)
                || !$this->is_secure('revise') )
        ) ? 1 : 0;
    }
}

sub is_secure_email ($;$) {
    my ( $this, $check ) = @_;
    if ( !$check && defined $this->{'is_secure_email'} ) {
        return $this->{'is_secure_email'};
    }
    elsif ($this->is_master
        || $this->is_owner
        || $this->is_one_of_owners
        || $this->is_staff )
    {
        $main::set_cookie_id = 1
            if !$env::session_enabled
            && $cookie::read{ $this->login_cookie_name };
        $this->{'is_secure_email'} = 1;
    }
    elsif ( $this->is_authorized_email($check) ) {
        $this->{'is_secure_email'} = 1;
    }
    else {
        $this->{'is_secure_email'} = 0;
    }
    return $this->{'is_secure_email'};
}

sub email_field_enabled ($;$) {
    my ( $this, $check ) = @_;
    $this->is_authorized_email($check)
        if $check || !defined $this->{'email_field_enabled'};
    return $this->{'email_field_enabled'};
}

sub is_authorized_email ($;$) {
    my ( $this, $check ) = @_;
    if ( !$check && defined $this->{'is_authorized_email'} ) {
        return $this->{'is_authorized_email'};
    }
    else {
        $this->{'is_authorized_email'} = 0;
        $this->{'email_field_enabled'} = 0;
        $this->{'user_address'}        = '';
        if ( ref $field::for{'mail_address'}
            && @{ $field::for{'mail_address'} } )
        {
            my ( @user_address, @new_user_address );
            $this->{'email_field_enabled'} = 1;
            foreach my $id ( @{ $field::for{'mail_address'} } ) {
                push( @new_user_address, $main::data{$id}{'str'} )
                    if $main::data{$id}{'str'};
                if ( defined $main::n && $main::n ne '' ) {
                    if ( ref $field::obj{$id} ) {
                        my (%db);
                        tie( %db, $env::dbm_file,
                            $default::db . '/' . $field::obj{$id}->db,
                            O_RDONLY, $env::dbm_mode );
                        if ( ref tied %db ) {
                            push(
                                @user_address,
                                $this->decode_utf8(
                                    $db{ $this->encode_utf8($main::n) }
                                )
                            ) if $db{ $this->encode_utf8($main::n) };
                        }
                        untie(%db);
                    }
                }
                elsif ( $main::data{$id}{'str'} ) {
                    push( @user_address, $main::data{$id}{'str'} );
                }
            }
            @user_address     = $this->get_address(@user_address);
            @new_user_address = $this->get_address(@new_user_address);
            if ( @user_address && @new_user_address ) {
                my $user_address_count
                    = grep( defined $_
                        && defined $main::email
                        && $_ eq $main::email,
                    @user_address );
                my $new_user_address_count
                    = grep( defined $_
                        && defined $main::email
                        && $_ eq $main::email,
                    @new_user_address );

#データベースにも新しいデータにも登録されたE-Mailがある
                if (   $main::email
                    && $user_address_count
                    && $new_user_address_count
                    && ($security::registered
                        || ($this->use_register
                            && $cgi::Register->is_registered(
                                $main::pswd, $main::email
                            )
                        )
                    )
                    )
                {
                    $security::registered = 1;
                    $main::set_cookie_id  = 1
                        if !$env::session_enabled
                        && $cookie::read{ $this->login_cookie_name };
                    $this->{'is_authorized_email'} = 1;
                    $this->{'user_address'}        = $main::email;

#データベースには登録されたE-Mailがあるが新しいデータにない
                }
                elsif (
                       $user_address_count
                    && !$new_user_address_count
                    && $this->use_register
                    && $cgi::Register->is_registered(
                        $main::pswd, $main::email
                    )
                    )
                {
                    $this->{'need_new_pswd'} = 1;
                    if (   $main::new_email
                        && $main::new_pswd
                        && grep( defined $_ && $_ eq $main::new_email,
                            @new_user_address )
                        && $cgi::Register->is_registered(
                            $main::new_pswd, $main::new_email
                        )
                        )
                    {
                        $this->{'is_authorized_email'} = 1;
                        $this->{'old_email'}           = $main::email;
                        $this->{'old_pswd'}            = $main::pswd;
                        $this->{'user_address'}        = $main::email
                            = $main::new_email;
                        $main::pswd = $main::new_pswd;
                        undef $main::new_email;
                        undef $main::new_pswd;
                        if ($env::session_enabled) {

                            if ( $this->use_session ) {
                                $session::id = $cgi::Session->set_user_info(
                                    $main::email, $main::pswd, $session::id );
                                $session::is_changed = 1;
                            }
                        }
                        elsif ( $cookie::read{ $this->login_cookie_name } ) {
                            $main::set_cookie_id = 1;
                        }
                    }
                    else {
                        if ($main::new_email
                            && !grep( defined $_ && $_ eq $main::new_email,
                                @new_user_address )
                            )
                        {
                            push(
                                @{ $this->{'dialog'} },
                                '入力された項目に新しいメールアドレスがみつかりません。'
                            );
                        }
                        elsif ($main::new_pswd) {
                            push(
                                @{ $this->{'dialog'} },
                                '新しいパスワードを確認してください。'
                            );
                        }
                        elsif (
                            $setting::security{'revise'}{'passwordCheck'} eq
                            'nocheck' )
                        {
                            $this->{'is_authorized_email'} = 1;
                        }
                        $this->{'user_address'}
                            = (@new_user_address)
                            ? $new_user_address[0]
                            : $user_address[0];
                    }

                    #その他
                }
                else {
                    if ( $setting::security{'revise'}{'passwordCheck'} eq
                        'nocheck' )
                    {
                        $this->{'is_authorized_email'} = 1;
                    }
                    $this->{'user_address'}
                        = (@new_user_address)
                        ? $new_user_address[0]
                        : $user_address[0];
                }
            }
            elsif (
                $setting::security{'revise'}{'passwordCheck'} eq 'nocheck' )
            {
                $this->{'is_authorized_email'} = 1;
            }
        }
        elsif ( $setting::security{'revise'}{'passwordCheck'} eq 'nocheck' ) {
            $this->{'is_authorized_email'} = 1;
        }
    }
    return $this->{'is_authorized_email'};
}

######################################################################
# データ削除
sub delete_from_dbs ($@) {
    my ( $this, @keys ) = @_;
    my ( $db, @files, %db );
    local (*DIR);
    opendir( DIR, $default::db );
    @files = readdir(DIR);
    closedir(DIR);
    foreach $db (@files) {
        next
            if $db =~ /^\./
            || $db =~ /\.htm[l]?$/i
            || !-f $default::db . '/' . $db;
        $db =~ s/\.[^\/\.]+$//;
        next if $db eq 'date' || $db eq 'dt';
        $db{$db} = 1;
    }
    while ( ($db) = each %db ) {
        my (%database);
        tie( %database, $env::dbm_file, $default::db . '/' . $db,
            O_RDWR, $env::dbm_mode );
        if ( ref tied %database ) {
            foreach my $k (@keys) {
                utf8::encode($k) if utf8::is_utf8($k);
                delete $database{$k};
            }
        }
        untie %database;
    }
}

######################################################################
# カウント
sub new_record_id ($;$) {
    my $this = shift;
    my ( %mtime, $n );
    $n = time;
    tie( %mtime, $env::dbm_file, $default::db . '/mtime',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %mtime ) {
        $n++ while $mtime{$n};
    }
    untie(%mtime);
    return $n;
}

######################################################################
#データベース読み書き
sub get_all_date ($$$;$$) {
    my ( $this, $dt, $until, $se_dgap, $interval ) = @_;
    return () if !$until;
    $interval = 1 if !$interval;
    my @dates = ();
    while ( $dt <= $until ) {
        push( @dates, $this->get_continuous_dates( $dt, $se_dgap ) );
        $dt = $this->get_next_date( $dt, $interval );
    }
    return @dates;
}

sub get_continuous_dates ($$$) {
    my ( $this, $dt, $se_dgap ) = @_;
    my @dates = ();
    if ($se_dgap) {
        for ( my $i = 0; $i <= $se_dgap; $i++ ) {
            $dt = $this->get_next_date( $dt, 1 ) if $i;
            push( @dates,
                      ( ($i) ? $i . '<' : '' )
                    . $dt
                    . ( ( $i < $se_dgap ) ? '>' . ( $se_dgap - $i ) : '' ) );
        }
    }
    else {
        push( @dates, $dt );
    }
    return @dates;
}

sub get_all_weekly_date ($$$$;$$) {
    my ( $this, $dt, $until, $se_dgap, $day, $interval ) = @_;
    return () if !$until;
    my ( $swn, @day, @wgap, @dgap, @dates );
    $interval = 1 if !$interval;
    $interval--;
    $swn = $this->dt_to_wday($dt);
    @day = sort { $a <=> $b } map( $this->wday_to_number($_), @$day );
    if ( !grep( $swn == $_, @day ) ) {

        if ( grep( $swn < $_, @day ) ) {
            my $next = ( grep( $swn < $_, @day ) )[0];
            $dt = $this->get_next_date( $dt, ( $next - $swn ) );
            $swn = $next;
        }
        else {
            $dt = $this->get_next_date( $dt, ( $day[0] + 7 - $swn ) );
            $swn = $day[0];
        }
    }
    foreach my $w (@day) {
        my $wn = $w - $swn;
        $wn += 7 if $wn < 0;
        push( @wgap, $wn );
    }
    @wgap = sort { $a <=> $b } @wgap;
    for ( my $i = 0; $i < @wgap; $i++ ) {
        push(
            @dgap,
            (   (   ( defined $wgap[ $i + 1 ] )
                    ? $wgap[ $i + 1 ]
                    : 7 + $wgap[0]
                ) - $wgap[$i]
            )
        );
    }
    my $n = 0;
    my $c = @dgap;
    while ( $dt <= $until ) {
        push( @dates, $this->get_continuous_dates( $dt, $se_dgap ) );
        my $w   = $n % $c;
        my $gap = $dgap[$w];
        $gap += 7 * $interval if $n && $w == $c - 1 && $interval;
        $dt = $this->get_next_date( $dt, $gap );
        $n++;
    }
    return @dates;
}

sub get_all_wday ($$$;$) {
    my ( $this, $dt, $se_dgap ) = @_;
    if ( $se_dgap && $dt =~ /^([0-9]+)([0-9a-zA-Z_]+)$/ ) {
        my $wn        = $1;
        my $w         = $this->wday_to_number($2);
        my $c         = 0;
        my $n         = 0;
        my $num       = 1;
        my @dates     = ();
        my @dates2    = ();
        my @pre_dates = ();
        my @week      = ( 'SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA' );
        for ( my $i = 0; $i <= $se_dgap; $i++ ) {
            $n = ( $w + $i ) % 7;
            $c++ if $i && !$n;
            $num = $wn + $c;
            push( @dates, $num . $week[$n] . '>' . $se_dgap ) if $num <= 5;
            if ( $c == 1 && $n < $w ) {
                push( @pre_dates, $wn . $week[$n] . '>' . $se_dgap );
            }
            if ( !@dates2 && $num == 5 ) {
                for ( my $t = 0; $t < $n; $t++ ) {
                    push( @dates2,
                        '30<1' . $week[$t] . '>' . ( $se_dgap - 30 ) );
                }
            }
            push( @dates2,
                '30<' . ( $num - 4 ) . $week[$n] . '>' . ( $se_dgap - 30 ) )
                if $num > 4;
        }
        unshift( @dates, @pre_dates ) if @pre_dates;
        if ( $num < 5 && !$c ) {
            $num++;
            for ( my $t = $w + 1; $t <= $n; $t++ ) {
                push( @dates, $num . $week[$t] . '>' . $se_dgap );
            }
        }
        elsif ( $num <= 5 && $w < $n && $n < 6 ) {
            for ( my $t = $n; $t < 7; $t++ ) {
                push( @dates, $num . $week[$t] . '>' . $se_dgap );
            }
        }
        push( @dates, @dates2 ) if @dates2;
        return @dates;
    }
    else {
        return ($dt);
    }
}

sub get_all_monthly_wday ($$$$$$;$) {
    my ( $this, $y, $m, $d, $until, $se_dgap, $interval ) = @_;
    return () if !$until;
    $interval = 1 if !$interval;
    my @dates = ();
    my $num   = int( $d / 7 ) + ( ( $d % 7 ) ? 1 : 0 );
    my $wday  = $this->get_wday( $y, $m, $d );
    my $mday  = $num * 7 + $wday - 6;
    my $dt    = $this->dt_stamp( $y, $m, $d );
    while ( $dt <= $until ) {
        push( @dates, $this->get_continuous_dates( $dt, $se_dgap ) )
            if !$this->is_impossible_mday( $y, $m, $d );
        $m += $interval;
        while ( $m > 12 ) {
            $m -= 12;
            $y++;
        }
        my $fw = $this->get_wday( $y, $m, 1 );
        $d = $mday - $fw + ( ( $wday < $fw ) ? 7 : 0 );
        $dt = $this->dt_stamp( $y, $m, $d );
    }
    return @dates;
}

sub get_all_mday ($$$;$) {
    my ( $this, $d, $se_dgap ) = @_;
    if ($se_dgap) {
        my @dates  = ();
        my @dates2 = ();
        for ( my $i = 0; $i <= $se_dgap; $i++ ) {
            my $mday = $d + $i;
            if ( $mday < 28 ) {
                push( @dates,
                          ( ($i) ? $i . '<' : '' )
                        . $mday
                        . ( ( $i < $se_dgap ) ? '>' . ( $se_dgap - $i ) : '' )
                );
            }
            else {
                push( @dates,
                          ( ($i) ? $i . '<' : '' )
                        . $mday
                        . ( ( $i < $se_dgap ) ? '>' . ( $se_dgap - $i ) : '' )
                ) if $mday <= 31;
                push(
                    @dates2,
                    ( $i + 3 ) . '<'
                        . ( $mday - 28 )
                        . (
                        ( $se_dgap - $i != 3 )
                        ? '>' . ( $se_dgap - $i - 3 )
                        : ''
                        )
                );
            }
        }
        push( @dates, @dates2 ) if @dates2;
        return @dates;
    }
    else {
        return ($d);
    }
}

sub get_all_monthlyday ($$$$$$;$) {
    my ( $this, $y, $m, $d, $until, $se_dgap, $interval ) = @_;
    return () if !$until;
    $interval = 1 if !$interval;
    my @dates = ();
    my $dt = $this->dt_stamp( $y, $m, $d );
    while ( $dt <= $until ) {
        push( @dates, $this->get_continuous_dates( $dt, $se_dgap ) )
            if !$this->is_impossible_mday( $y, $m, $d );
        $m += $interval;
        while ( $m > 12 ) {
            $m -= 12;
            $y++;
        }
        $dt = $this->dt_stamp( $y, $m, $d );
    }
    return @dates;
}

sub get_all_yday ($$$;$) {
    my ( $this, $m, $d, $se_dgap ) = @_;
    if ($se_dgap) {
        my @dates = (
            sprintf( "%.2d", $m ) . sprintf( "%.2d", $d ) . '>' . $se_dgap );
        if ( $m == 2 && $d == 29 ) {
            $m = 3;
            $d = 1;
            $se_dgap--;
            if ( !$se_dgap ) {
                push( @dates, sprintf( "%.2d", $m ) . sprintf( "%.2d", $d ) );
                return @dates;
            }
            else {
                push( @dates,
                          sprintf( "%.2d", $m )
                        . sprintf( "%.2d", $d ) . '>'
                        . $se_dgap );
            }
        }
        for ( my $i = 1; $i <= $se_dgap; $i++ ) {
            $d++;
            if ( ( ( $m == 4 || $m == 6 || $m == 9 || $m == 11 ) && $d > 30 )
                || ( $m == 2 && $d > 28 )
                || $d > 31 )
            {
                if ( $m == 2 && $d > 28 ) {
                    push(
                        @dates,
                        $i . '<0229'
                            . (
                            ( $i < $se_dgap ) ? '>' . ( $se_dgap - $i ) : ''
                            )
                    );
                }
                $d = 1;
                $m++;
            }
            $m = 1 if $m > 12;
            push( @dates,
                      $i . '<'
                    . ( sprintf( "%.2d", $m ) . sprintf( "%.2d", $d ) )
                    . ( ( $i < $se_dgap ) ? '>' . ( $se_dgap - $i ) : '' ) );
        }
        return @dates;
    }
    else {
        return ( sprintf( "%.2d", $m ) . sprintf( "%.2d", $d ) );
    }
}

sub get_all_yearlyday ($$$$$$;$) {
    my ( $this, $y, $m, $d, $until, $se_dgap, $interval ) = @_;
    return () if !$until;
    $interval = 1 if !$interval;
    my @dates = ();
    my $dt = $this->dt_stamp( $y, $m, $d );
    while ( $dt <= $until ) {
        push( @dates, $this->get_continuous_dates( $dt, $se_dgap ) )
            if !$this->is_impossible_mday( $y, $m, $d );
        $y += $interval;
        $dt = $this->dt_stamp( $y, $m, $d );
    }
    return @dates;
}

sub dbm ($;$$) {
    my ( $this, $event_dt, $is_not_first ) = @_;
    my ( $flags, $filedir );

    if ( !$this->set_dir($default::db) ) {
        push(
            @{ $this->{'dialog'} },
            'データベース書き込みに失敗しました。保存用ディレクトリが存在しません。'
        );
        return;
    }
    if ( !%field::obj ) {
        push(
            @{ $this->{'dialog'} },
            'データベース書き込みに失敗しました。入力フィールドが存在しません。'
        );
        return;
    }

    if ( defined $main::a && $main::a eq 'w' && $main::n ) {
        $this->record_info_modify;

        #名前登録
        if (   $field::NAMEREGISTER
            && $field::for{'name'}
            && ref $main::data{ $field::for{'name'} }
            && defined $main::data{ $field::for{'name'} }{'str'}
            && $main::data{ $field::for{'name'} }{'str'} ne ''
            && $this->use_register )
        {
            my $user_name = $main::data{ $field::for{'name'} }{'str'};
            $user_name =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
            $cgi::Register->user_name_register(
                $main::pswd, $main::email,
                $user_name,  $security::registered
            ) if $user_name ne '';
        }

        #日時書き込み
        my ( %dt, %date, @date, @old_date, $rrule, $dt_span );
        $rrule   = '';
        @date    = ();
        $dt_span = $this->dt_stamp(
            $main::dt{'sy'}, $main::dt{'sm'},   $main::dt{'sd'},
            $main::dt{'sh'}, $main::dt{'smin'}, $main::dt{'date'}
            )
            . '-'
            . $this->dt_stamp(
            $main::dt{'ey'}, $main::dt{'em'},   $main::dt{'ed'},
            $main::dt{'eh'}, $main::dt{'emin'}, $main::dt{'date'}
            );
        if ( $main::dt{'freq'} ) {
            my $until
                = $this->dt_stamp( $main::dt{'uy'}, $main::dt{'um'},
                $main::dt{'ud'} )
                if !$main::dt{'indefinite'};
            if (   $main::dt{'freq'} eq 'daily'
                || $main::dt{'freq'} eq 'weekly' )
            {
                my $dt_start_date
                    = $this->dt_stamp( $main::dt{'sy'}, $main::dt{'sm'},
                    $main::dt{'sd'} );
                my $dt_end_date
                    = $this->dt_stamp( $main::dt{'ey'}, $main::dt{'em'},
                    $main::dt{'ed'} );
                my $se_dgap
                    = ( !$main::dt{'date'} && $dt_start_date < $dt_end_date )
                    ? 1
                    : 0;
                if ( $main::dt{'freq'} eq 'daily' ) {
                    $rrule = 'FREQ=DAILY;';
                    $rrule .= 'INTERVAL=' . $main::dt{'interval'} . ';';
                    if ( $main::dt{'indefinite'} ) {
                        if ($se_dgap) {
                            @date = ( 'DAILY>1', '1<DAILY' );
                        }
                        else {
                            @date = ('DAILY');
                        }
                    }
                    else {
                        @date = $this->get_all_date( $dt_start_date, $until,
                            $se_dgap, $main::dt{'interval'} );
                    }
                }
                elsif ( $main::dt{'freq'} eq 'weekly' ) {
                    $rrule = 'FREQ=WEEKLY;';
                    $rrule .= 'INTERVAL=' . $main::dt{'interval'} . ';';
                    $rrule .= 'BYDAY='
                        . join( ',', @{ $main::dt{'byday'} } ) . ';';
                    if ( $main::dt{'indefinite'} ) {
                        if ($se_dgap) {
                            foreach my $day ( @{ $main::dt{'byday'} } ) {
                                push(
                                    @date,
                                    $day . '>1',
                                    '1<'
                                        . $this->number_to_wday(
                                        ( $this->wday_to_number($day) + 1 )
                                        % 7
                                        )
                                );
                            }
                        }
                        else {
                            @date = @{ $main::dt{'byday'} };
                        }
                    }
                    else {
                        @date
                            = $this->get_all_weekly_date( $dt_start_date,
                            $until, $se_dgap, $main::dt{'byday'},
                            $main::dt{'interval'} );
                    }
                }
            }
            else {
                my $dt_start_time = timelocal(
                    0,
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'smin'} ),
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'sh'} ),
                    $main::dt{'sd'},
                    ( $main::dt{'sm'} - 1 ),
                    $main::dt{'sy'}
                );
                my $dt_end_time = timelocal(
                    0,
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'emin'} ),
                    ( ( $main::dt{'date'} ) ? 0 : $main::dt{'eh'} ),
                    $main::dt{'ed'},
                    ( $main::dt{'em'} - 1 ),
                    $main::dt{'ey'}
                );
                my $se_gap  = $dt_end_time - $dt_start_time;
                my $se_dgap = int(
                    (   ( $main::dt{'date'} )
                        ? $se_gap - 1
                        : $se_gap + $main::dt{'sh'} * 3600
                            + $main::dt{'smin'} * 60
                    ) / 86400
                );
                if ( $main::dt{'freq'} eq 'monthly' ) {
                    $rrule = 'FREQ=MONTHLY;';
                    $rrule .= 'INTERVAL=' . $main::dt{'interval'} . ';';
                    if ( $main::dt{'by'} && $main::dt{'by'} eq 'bywday' ) {
                        my $byday
                            = (
                            int( $main::dt{'sd'} / 7 )
                                + ( ( $main::dt{'sd'} % 7 ) ? 1 : 0 ) )
                            . $this->number_to_wday(
                            $this->get_wday(
                                $main::dt{'sy'}, $main::dt{'sm'},
                                $main::dt{'sd'}
                            )
                            );
                        $rrule .= 'BYDAY=' . $byday . ';';
                        if ( $main::dt{'indefinite'} ) {
                            @date = $this->get_all_wday( $byday, $se_dgap );
                        }
                        else {
                            @date = $this->get_all_monthly_wday(
                                $main::dt{'sy'}, $main::dt{'sm'},
                                $main::dt{'sd'}, $until,
                                $se_dgap,        $main::dt{'interval'}
                            );
                        }
                    }
                    else {
                        $rrule .= 'BYMONTHDAY=' . $main::dt{'sd'} . ';';
                        if ( $main::dt{'indefinite'} ) {
                            @date = $this->get_all_mday( $main::dt{'sd'},
                                $se_dgap );
                        }
                        else {
                            @date = $this->get_all_monthlyday(
                                $main::dt{'sy'}, $main::dt{'sm'},
                                $main::dt{'sd'}, $until,
                                $se_dgap,        $main::dt{'interval'}
                            );
                        }
                    }
                }
                elsif ( $main::dt{'freq'} eq 'yearly' ) {
                    $rrule = 'FREQ=YEARLY;';
                    $rrule .= 'INTERVAL=' . $main::dt{'interval'} . ';';
                    $rrule .= 'BYMONTH=' . $main::dt{'sm'} . ';';
                    $rrule .= 'BYMONTHDAY=' . $main::dt{'sd'} . ';';
                    if ( $main::dt{'indefinite'} ) {
                        @date = $this->get_all_yday( $main::dt{'sm'},
                            $main::dt{'sd'}, $se_dgap );
                    }
                    else {
                        @date
                            = $this->get_all_yearlyday( $main::dt{'sy'},
                            $main::dt{'sm'}, $main::dt{'sd'}, $until,
                            $se_dgap, $main::dt{'interval'} );
                    }
                }
            }
            if ( !$main::dt{'indefinite'} ) {
                my $uh = 0;
                my $nd = 0;
                if ( $ENV{'TZ'} && $ENV{'TZ'} =~ /(\-?)([0-9]+)$/ ) {
                    if ( $1 eq '-' ) {
                        $uh = 23 - $2;
                    }
                    elsif ($2) {
                        $uh = $2;
                        $nd = 1;
                    }
                    else {
                        $uh = 23;
                    }
                }
                if ($nd) {
                    $rrule .= 'UNTIL='
                        . $this->get_next_date(
                        $this->dt_stamp(
                            $main::dt{'uy'}, $main::dt{'um'},
                            $main::dt{'ud'}
                        )
                        )
                        . 'T'
                        . sprintf( "%.2d", $uh )
                        . '5959Z;';
                }
                else {
                    $rrule .= 'UNTIL='
                        . $this->dt_stamp( $main::dt{'uy'}, $main::dt{'um'},
                        $main::dt{'ud'} )
                        . 'T'
                        . sprintf( "%.2d", $uh )
                        . '5959Z;';
                }

#$rrule .= 'UNTIL=' . $this->dt_stamp($main::dt{'uy'},$main::dt{'um'},$main::dt{'ud'}) . ';';
            }
        }
        else {
            my $dt_start_time = timelocal(
                0,
                ( ( $main::dt{'date'} ) ? 0 : $main::dt{'smin'} ),
                ( ( $main::dt{'date'} ) ? 0 : $main::dt{'sh'} ),
                $main::dt{'sd'},
                ( $main::dt{'sm'} - 1 ),
                $main::dt{'sy'}
            );
            my $dt_end_time = timelocal(
                0,
                ( ( $main::dt{'date'} ) ? 0 : $main::dt{'emin'} ),
                ( ( $main::dt{'date'} ) ? 0 : $main::dt{'eh'} ),
                $main::dt{'ed'},
                ( $main::dt{'em'} - 1 ),
                $main::dt{'ey'}
            );
            my $se_gap  = $dt_end_time - $dt_start_time;
            my $se_dgap = int(
                (   ( $main::dt{'date'} )
                    ? $se_gap - 1
                    : $se_gap + $main::dt{'sh'} * 3600
                        + $main::dt{'smin'} * 60
                ) / 86400
            );
            if ($se_dgap) {
                @date = $this->get_continuous_dates(
                    $this->dt_stamp(
                        $main::dt{'sy'}, $main::dt{'sm'}, $main::dt{'sd'}
                    ),
                    $se_dgap
                );
            }
            else {
                @date = (
                    $this->dt_stamp(
                        $main::dt{'sy'}, $main::dt{'sm'}, $main::dt{'sd'}
                    )
                );
            }
        }
        tie(%dt, $env::dbm_file,
            $default::db . '/dt',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %dt ) {
            $rrule =~ s/;$//s;
            if ( $dt{ $this->encode_utf8($main::n) } ) {
                my @info = split( /\x09/,
                    $this->decode_utf8( $dt{ $this->encode_utf8($main::n) } )
                );
                @old_date = split( /,/, $info[2] );
            }
            $dt{ $this->encode_utf8($main::n) } = $this->encode_utf8(
                $dt_span . "\x09" . $rrule . "\x09" . join( ',', @date ) );
        }
        untie %dt;
        if (@old_date) {
            my @temp = ();
            foreach my $d (@old_date) {
                push( @temp, $d ) if !grep( $_ eq $d, @date );
            }
            @old_date = @temp;
        }
        tie(%date, $env::dbm_file,
            $default::db . '/date',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %date ) {
            foreach my $d (@old_date) {
                my $n = $main::n;
                $n = $1 . $n if $d =~ s/^([\-]?[0-9]+<)//;
                $n .= $1 if $d =~ s/(>[\-]?[0-9]+)$//;
                utf8::encode($d) if utf8::is_utf8($d);
                if ( $date{$d} ) {
                    my @d = grep( $_ ne $n,
                        split( /,/, $this->decode_utf8( $date{$d} ) ) );
                    $date{$d} = $this->encode_utf8( join( ',', @d ) );
                    delete $date{$d} if !$date{$d};
                }
            }
            foreach my $d (@date) {
                my $n = $main::n;
                $n = $1 . $n if $d =~ s/^([\-]?[0-9]+<)//;
                $n .= $1 if $d =~ s/(>[\-]?[0-9]+)$//;
                utf8::encode($d) if utf8::is_utf8($d);
                if ( $date{$d} ) {
                    my @d = split( /,/, $this->decode_utf8( $date{$d} ) );
                    if ( !grep( $_ eq $n, @d ) ) {
                        push( @d, $n );
                        $date{$d} = $this->encode_utf8( join( ',', @d ) );
                    }
                }
                else {
                    $date{$d} = $this->encode_utf8($n);
                }
            }
        }
        untie %date;

        #ファイル保存
        $filedir = $default::file . '/' . $main::n;
        if ( !$is_not_first ) {
            $cgi::input->fix_all_files($filedir);
            if ( ref $cgi::input->{'dialog'} eq 'ARRAY'
                && @{ $cgi::input->{'dialog'} } > 0 )
            {
                push( @{ $this->{'dialog'} }, @{ $cgi::input->{'dialog'} } );
            }
        }

        $flags = O_RDWR | O_CREAT;
    }
    else {
        $flags = O_RDONLY;

        #日時読み込み
        my ( %dt, $dt_start, $dt_end, $rrule, %rrule );
        if ( defined $main::n && $main::n ne '' ) {
            tie( %dt, $env::dbm_file, $default::db . '/dt',
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %dt && $dt{ $this->encode_utf8($main::n) } ) {
                ( $dt_start, $rrule ) = split( /\x09/,
                    $this->decode_utf8( $dt{ $this->encode_utf8($main::n) } )
                );
            }
            untie %dt;
        }
        ( $dt_start, $dt_end )
            = ($dt_start) ? split( /\-/, $dt_start ) : ( '', '' );
        $main::dt{'date'} = 1 if $dt_start =~ /^[0-9]{8}$/;
        (   $main::dt{'sy'}, $main::dt{'sm'}, $main::dt{'sd'},
            $main::dt{'sh'}, $main::dt{'smin'}
        ) = $this->timestamp_to_array($dt_start);
        (   $main::dt{'ey'}, $main::dt{'em'}, $main::dt{'ed'},
            $main::dt{'eh'}, $main::dt{'emin'}
        ) = $this->timestamp_to_array($dt_end);
        if ($rrule) {
            my %rrule = $this->get_hash_from_rrule($rrule);
            if ( $rrule{'FREQ'} ) {
                $main::dt{'freq'} = lc $rrule{'FREQ'};
                if ( $rrule{'UNTIL'} ) {
                    if ( $rrule{'UNTIL'} =~ /^([0-9]{8})T[0-9]{6}Z$/ ) {
                        my $udt = $1;
                        if (   $ENV{'TZ'}
                            && $ENV{'TZ'} =~ /(\-?)([0-9]+)$/
                            && !$1
                            && $2 )
                        {
                            $udt = $this->get_prev_date($udt);
                        }
                        ( $main::dt{'uy'}, $main::dt{'um'}, $main::dt{'ud'} )
                            = $this->timestamp_to_array($udt);
                    }
                    else {
                        ( $main::dt{'uy'}, $main::dt{'um'}, $main::dt{'ud'} )
                            = $this->timestamp_to_array( $rrule{'UNTIL'} );
                    }
                }
                else {
                    $main::dt{'indefinite'} = 1;
                }
                $main::dt{'interval'}
                    = ( $rrule{'INTERVAL'} ) ? $rrule{'INTERVAL'} : 1;
                if ( $rrule{'BYDAY'} ) {
                    $this->scalar_to_array( \$rrule{'BYDAY'} );
                    @{ $main::dt{'byday'} } = @{ $rrule{'BYDAY'} };
                    if ( $main::dt{'freq'} eq 'monthly' ) {
                        $main::dt{'by'} = 'bywday';
                        @{ $main::dt{'byday'} } = grep( $_ =~ s/^[0-9]+//g,
                            @{ $rrule{'BYDAY'} } );
                    }
                }
                $main::dt{'by'} = 'bymday' if $rrule{'BYMONTHDAY'};
            }
        }
    }

    if ( defined $main::n && $main::n ne '' ) {
        while ( my ( $id, $field ) = each %field::obj ) {
            my ( %database, $db );
            $db = $field->db;
            next if !$db;
            tie( %database, $env::dbm_file, $default::db . '/' . $db,
                $flags, $env::dbm_mode );
            if ( ref tied %database ) {

                #書き込み
                if ( $main::a eq 'w' ) {
                    if ( $field->input_enabled ) {
                        if (!$field->{'protected'}
                            || (!$this->{'new_data'}
                                && $this->is_secure(
                                    'reviseProtectedFields', 0, 'revise'
                                )
                            )
                            || ($this->{'new_data'}
                                && $this->is_secure(
                                    'inputProtectedFields', 0, 'input'
                                )
                            )
                            )
                        {
                            eval
                                '$database{$this->encode_utf8($main::n)} = $this->encode_utf8($main::data{$id}{\'str\'})';
                            if ( defined $@ && $@ ) {
                                push(
                                    @{ $this->{'dialog'} },
                                    '書き込みに失敗しました。('
                                        . $field->{'name'} . ')'
                                );
                            }
                            else {
                                $field->settle_up($filedir);
                            }
                        }
                    }

                    #読み込み
                }
                else {
                    %{ $main::data{$id} } = $field->elements(
                        $this->decode_utf8(
                            $database{ $this->encode_utf8($main::n) }
                        )
                    );
                }

            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    'データベースに接続できませんでした。('
                        . $field->{'name'} . ')'
                );
            }
            untie %database;
        }

        #RSS更新
        if (   defined $main::a
            && $main::a eq 'w'
            && (   $setting::general{'RSSEnabled'}
                || $setting::general{'masterRSSEnabled'}
                || $setting::general{'commentRSSEnabled'}
                || $setting::general{'siteRSSEnabled'} )
            )
        {
            my $my_url          = $this->my_url;
            my $my_category_url = $my_url
                . (
                (          $default::subdivided
                        && defined $main::c
                        && $main::c ne ''
                ) ? '?c=' . $main::c : ''
                );
            my $guid
                = $my_url
                . '?m=DetailViewer'
                . (
                (          $default::subdivided
                        && defined $main::c
                        && $main::c ne ''
                ) ? '&c=' . $main::c : ''
                )
                . '&n='
                . $main::n;
            if (   $setting::security{'browse'}{'passwordCheck'} eq 'nocheck'
                && $setting::security{'closed'}{'passwordCheck'} eq
                'nocheck' )
            {
                $this->import_db(
                    $default::preference_dir . '/event_category',
                    \%setting::event_category, 1 )
                    unless %setting::event_category
                    && ref $setting::event_category{'ORDER'}
                    && %{ $setting::event_category{'ORDER'} };
                $this->feed_rss(
                    {   'title' => (
                            (          $field::for{'title'}
                                    && ref $field::obj{ $field::for{'title'} }
                                    && $field::obj{ $field::for{'title'} }
                                    ->{'Open'}
                                    && !$field::obj{ $field::for{'title'} }
                                    ->{'Hidden'}
                                    && $main::data{ $field::for{'title'} }
                                    {'str'}
                            )
                            ? $field::obj{ $field::for{'title'} }
                                ->usual_format(
                                $main::data{ $field::for{'title'} }{'str'},
                                $default::file . '/' . $main::n,
                                $main::c,
                                $main::n
                                )
                            : '新しいイベントが投稿されました。'
                            )
                            . ( ($event_dt) ? '(' . $event_dt . ')' : '' ),
                        'link'            => $guid,
                        'content:encoded' => (
                            (   $field::for{'content'}
                                    && ref
                                    $field::obj{ $field::for{'content'} }
                                    && $field::obj{ $field::for{'content'} }
                                    ->{'Open'}
                                    && !$field::obj{ $field::for{'content'} }
                                    ->{'Hidden'}
                                    && $main::data{ $field::for{'content'} }
                                    {'str'}
                            )
                            ? $field::obj{ $field::for{'content'} }
                                ->format_for_detail(
                                $main::data{ $field::for{'content'} }{'str'},
                                $default::file . '/' . $main::n,
                                $main::c,
                                $main::n
                                )
                            : '新しいイベントが投稿されました。'
                        ),
                        'dc:creator' => (
                            (          $field::for{'name'}
                                    && ref $field::obj{ $field::for{'name'} }
                                    && $main::data{ $field::for{'name'} }
                                    {'str'}
                            )
                            ? $field::obj{ $field::for{'name'} }
                                ->format_for_detail(
                                $main::data{ $field::for{'name'} }{'str'},
                                $default::file . '/' . $main::n,
                                $main::c,
                                $main::n
                                )
                            : ( !$field::for{'name'}
                                    && defined $setting::general{'mastername'}
                                ) ? $setting::general{'mastername'}
                            : ''
                        ),
                        'category' => (
                            (          $default::subdivided
                                    && defined $main::c
                                    && $main::c ne ''
                                    && $setting::general{'cginame'}
                            ) ? $setting::general{'cginame'} . '/' : ''
                            )
                            . (
                            (          defined $field::for{'category'}
                                    && $field::for{'category'} ne ''
                                    && defined
                                    $main::data{ $field::for{'category'} }
                                    {'str'}
                                    && %setting::event_category
                                    && ref $setting::event_category{
                                    $main::data{ $field::for{'category'} }
                                        {'str'}
                                    }
                                    && $setting::event_category{
                                    $main::data{ $field::for{'category'} }
                                        {'str'}
                                    }{'name'}
                            ) ? $setting::event_category{
                                $main::data{ $field::for{'category'} }{'str'}
                            }{'name'} : ''
                            ),
                        'enclosure' => $this->article_enclosure,
                        'guid'      => $guid,
                        (   (          $env::checkout
                                    && $field::for{'showcase'}
                                    && ref
                                    $field::obj{ $field::for{'showcase'} }
                                    && $field::obj{ $field::for{'showcase'} }
                                    ->{'Open'}
                                    && !$field::obj{ $field::for{'showcase'} }
                                    ->{'Hidden'}
                            )
                            ? $field::obj{ $field::for{'showcase'} }
                                ->get_gdata( $guid,
                                $main::data{ $field::for{'showcase'} } )
                            : ()
                        )
                    },
                    $default::file,
                    $my_category_url,
                    (   ($default::subdivided) ? $default::master_file_dir
                        : ''
                    ),
                    ( ($default::subdivided) ? $my_url : '' )
                );
            }
            else {
                $this->feed_rss(
                    {   'title' =>
                            '新しい予定が書き込まれました。',
                        'link' => $guid,
                        'content:encoded' =>
                            '新しい予定が書き込まれました。',
                        'guid' => $guid
                    },
                    $default::file,
                    $my_category_url,
                    (   ($default::subdivided)
                        ? $default::master_file_dir
                        : ''
                    ),
                    ( ($default::subdivided) ? $my_url : '' )
                );
            }
        }
    }
}

sub article_enclosure ($) {
    my $this = shift;
    my $hash
        = (    $field::for{'media'}
            && ref $field::obj{ $field::for{'media'} }
            && $field::obj{ $field::for{'media'} }->{'Open'}
            && !$field::obj{ $field::for{'media'} }->{'Hidden'}
            && $main::data{ $field::for{'media'} }{'str'} )
        ? {
        $field::obj{ $field::for{'media'} }->get_enclosure_attributes(
            $main::data{ $field::for{'media'} }{'str'},
            $default::file . '/' . $main::n,
            $main::c, $main::n
        )
        }
        : '';
    $hash
        = (    $field::for{'media2'}
            && ref $field::obj{ $field::for{'media2'} }
            && $field::obj{ $field::for{'media2'} }->{'Open'}
            && !$field::obj{ $field::for{'media2'} }->{'Hidden'}
            && $main::data{ $field::for{'media2'} }{'str'} )
        ? {
        $field::obj{ $field::for{'media2'} }->get_enclosure_attributes(
            $main::data{ $field::for{'media2'} }{'str'},
            $default::file . '/' . $main::n,
            $main::c, $main::n
        )
        }
        : ''
        if !ref $hash || !%$hash;
    $hash
        = (    $field::for{'media3'}
            && ref $field::obj{ $field::for{'media3'} }
            && $field::obj{ $field::for{'media3'} }->{'Open'}
            && !$field::obj{ $field::for{'media3'} }->{'Hidden'}
            && $main::data{ $field::for{'media3'} }{'str'} )
        ? {
        $field::obj{ $field::for{'media3'} }->get_enclosure_attributes(
            $main::data{ $field::for{'media3'} }{'str'},
            $default::file . '/' . $main::n,
            $main::c, $main::n
        )
        }
        : ''
        if !ref $hash || !%$hash;
    return $hash;
}

######################################################################
sub record_info_modify ($) {
    my $this = shift;
    my ( %mtime, %ip, %agent, %general, $time, $last );
    $time = time;
    $last = 0;

    #更新時刻の書き込み
    tie(%mtime, $env::dbm_file,
        $default::db . '/mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %mtime ) {
        $last
            = ( $mtime{ $this->encode_utf8($main::n) } )
            ? $this->decode_utf8( $mtime{ $this->encode_utf8($main::n) } )
            : 0;
        $mtime{ $this->encode_utf8($main::n) } = $time;
    }
    else {
        push(
            @{ $this->{'dialog'} },
            '書き込みに失敗しました。(mtime)'
        );
    }
    untie %mtime;

    $this->{'new_record'} = 1 if $this->{'new_data'} || $time - $last > 10800;

    #作成時刻の書き込み
    if ( defined $this->{'new_data'} && $this->{'new_data'} ) {
        my (%ctime);
        tie(%ctime, $env::dbm_file,
            $default::db . '/ctime',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %ctime ) {
            $ctime{ $this->encode_utf8($main::n) } = $time
                if !$ctime{ $this->encode_utf8($main::n) };
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(ctime)'
            );
        }
        untie %ctime;
    }

    if ( !$this->{'keep_original_user_info'} ) {

        #投稿者IPの書き込み
        tie(%ip, $env::dbm_file,
            $default::db . '/ip',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %ip ) {
            $ip{ $this->encode_utf8($main::n) }
                = $this->encode_utf8( $ENV{'REMOTE_HOST'}
                    || $this->to_host( $ENV{'REMOTE_ADDR'} ) );
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(ip)'
            );
        }
        untie %ip;

        #投稿者環境の書き込み
        tie(%agent, $env::dbm_file,
            $default::db . '/agent',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %agent ) {
            $agent{ $this->encode_utf8($main::n) }
                = $this->encode_utf8( $ENV{'HTTP_USER_AGENT'} );
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(agent)'
            );
        }
        untie %agent;
    }

    #カテゴリー更新時更新
    tie(%general, $env::dbm_file,
        $default::preference_dir . '/general',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %general ) {
        $general{'mtime'} = $time;
    }
    else {
        push(
            @{ $this->{'dialog'} },
            '書き込みに失敗しました。(general)'
        );
    }
    untie %general;
    $setting::general{'mtime'} = $time;
    if ( defined $default::subdivided && $default::subdivided ) {
        my (%category_mtime);
        tie(%category_mtime, $env::dbm_file,
            $default::master_db_dir . '/category_mtime',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %category_mtime ) {
            $category_mtime{ $this->encode_utf8($main::c) } = $time;
            $category_mtime{'Last-Modified'} = $time;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(category_mtime)'
            );
        }
        untie %category_mtime;
        tie(%general, $env::dbm_file,
            $default::master_db_dir . '/general',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %general ) {
            $general{'mtime'} = $time;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(/general)'
            );
        }
        untie %general;
    }
}

######################################################################
sub send_notice_mail ($$$) {
    my ( $this, $dt ) = @_;
    return 0 if !$setting::general{'notice'};
    my @addr = $this->get_address(
        $setting::general{'master'},
        $setting::general{'owners'},
        $setting::general{'staff'}
    );
    return 0 if !@addr;
    my $subject
        = ( $setting::mail{'subject'} )
        ? $setting::mail{'subject'}
        : '【お知らせ】新しい予定が書き込まれました。';
    my $msg
        = ( $setting::mail{'msg'} )
        ? $setting::mail{'msg'}
        : '新しい予定が書き込まれました。' . "\x0A"
        . '詳細は下記URLをご覧ください。'
        . "\x0A\x0A" . '<url>'
        . "\x0A\x0A";
    my ( $name, $title, $url );
    if ( defined $field::for{'name'} && $field::for{'name'} ) {
        $name = $main::data{ $field::for{'name'} }{'str'};
        $name =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
    }
    $name = ($default::english) ? 'Anonymous' : '匿名'
        if !defined $name || $name eq '';
    if ( defined $field::for{'title'} && $field::for{'title'} ) {
        $title = $main::data{ $field::for{'title'} }{'str'};
        $title =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
    }
    $title = ($default::english) ? 'This event' : 'これ'
        if !defined $title || $title eq '';
    $subject =~ s/<name>/$name/igs;
    $msg =~ s/<name>/$name/igs;
    $subject =~ s/<title>/$title/igs;
    $msg =~ s/<title>/$title/igs;
    $url
        = $this->my_url
        . '?m=DetailViewer'
        . ( ( $default::subdivided && defined $main::c && $main::c ne '' )
        ? '&c=' . $main::c
        : '' )
        . '&n='
        . $main::n;
    $msg =~ s/<url>/$url/igs;

    if ( $subject =~ /<dt>/ || $msg =~ /<dt>/ ) {
        $subject =~ s/<dt>/$dt/igs;
        $msg =~ s/<dt>/$dt/igs;
    }
    if ( $setting::general{'notice'} eq 'data' ) {
        my $list = $this->data_for_notice_mail( \%field::obj, \%main::data );
        $msg .= "\x0A\x0A" . $list . "\x0A" unless $msg =~ s/<data>/$list/igs;
    }
    else {
        $msg =~ s/<data>//igs;
    }
    if ( !ref $cgi::SendMail ) {
        require CMONOS::CGI_Lib::SendMail;
        $cgi::SendMail
            = SendMail->new( $env::sendmail, $env::master_email, '',
            $cgi::input, 1 );
    }
    $cgi::SendMail->send_mail(
        'From' => (
            (          defined $field::for{'mail'}
                    && ref $field::obj{ $field::for{'mail'} }
                    && !$field::obj{ $field::for{'mail'} }->{'Hidden'}
                    && $field::obj{ $field::for{'mail'} }->{'Open'}
                    && $main::data{ $field::for{'mail'} }{'str'}
            )
            ? ( ( $main::data{ $field::for{'name'} }{'str'} )
                ? $main::data{ $field::for{'name'} }{'str'} . ' <'
                    . $main::data{ $field::for{'mail'} }{'str'} . '>'
                : $main::data{ $field::for{'mail'} }{'str'} )
            : $setting::general{'master'}
        ),
        'Bcc'     => join( ', ', @addr ),
        'Subject' => $subject,
        'message' => $msg . "\x0A\x0A",
        'file' =>
            ( ( %main::data && defined $mail::file ) ? $mail::file : '' )
    );
    return 1;
}

######################################################################
#設定可能時間制限チェック/重複チェック
sub is_available_time ($$$$$$$$$) {
    my ($this,                 $this_n,
        $this_dt,              $available_time_array,
        $available_wday_array, $samezone_categories_array,
        $category_field,       $category_id,
        $category_name,        $is_extended
    ) = @_;
    my ( @date, $is_not_available_time, @overlapping_date, $holiday,
        $time_check, $wday_check );
    if ( grep( $_ == 7, @$available_wday_array ) ) {
        require CMONOS::CGI_Lib::Holiday;
        $holiday = Holiday->new;
    }
    @date = ();
    if ( !$is_extended ) {
        $time_check
            = ( ref $available_time_array && @$available_time_array ) ? 1 : 0;
        $wday_check
            = ( ref $available_wday_array && @$available_wday_array ) ? 1 : 0;
    }
    if ( $wday_check || $time_check ) {
        my $dt_start_time = timelocal(
            0,
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'smin'} ),
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'sh'} ),
            $this_dt->{'sd'},
            ( $this_dt->{'sm'} - 1 ),
            $this_dt->{'sy'}
        );
        my $dt_end_time = timelocal(
            0,
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'emin'} ),
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'eh'} ),
            $this_dt->{'ed'},
            ( $this_dt->{'em'} - 1 ),
            $this_dt->{'ey'}
        );
        my $se_gap  = $dt_end_time - $dt_start_time;
        my $se_dgap = int(
            (   ( $this_dt->{'date'} )
                ? $se_gap - 1
                : $se_gap + $this_dt->{'sh'} * 3600 + $this_dt->{'smin'} * 60
            ) / 86400
        );
        my $s
            = ( $this_dt->{'date'} )
            ? 0
            : $this_dt->{'sh'} * 100 + $this_dt->{'smin'};
        my $e
            = ( $this_dt->{'date'} )
            ? 0
            : $this_dt->{'eh'} * 100 + $this_dt->{'emin'};
        if (   $time_check
            && !$this_dt->{'date'}
            && $available_time_array->[ $#{$available_time_array} ]
            && $available_time_array->[ $#{$available_time_array} ]
            =~ /^([0-9]{1,2}):([0-9]{2})-([0-9]{1,2}):([0-9]{2})/ )
        {
            my ( $f, $t ) = ( ( $1 * 100 + $2 ), ( $3 * 100 + $4 ) );
            if ( $s >= $t ) {
                if ($se_dgap) {
                    ( $this_dt->{'sy'}, $this_dt->{'sm'}, $this_dt->{'sd'} )
                        = $this->get_next_date_array( $this_dt->{'sy'},
                        $this_dt->{'sm'}, $this_dt->{'sd'} );
                    $s = $this_dt->{'sh'} = $this_dt->{'smin'} = 0;
                    $se_dgap--;
                }
                else {
                    $is_not_available_time = 1;
                }
            }
        }
        if ( $wday_check && $se_dgap && !$this_dt->{'freq'} ) {
            while (
                !$this->is_available_wday(
                    $available_wday_array, $this_dt->{'sy'},
                    $this_dt->{'sm'},      $this_dt->{'sd'},
                    $holiday
                )
                )
            {
                if ( $se_dgap <= 0 ) {
                    $is_not_available_time = 1;
                    last;
                }
                ( $this_dt->{'sy'}, $this_dt->{'sm'}, $this_dt->{'sd'} )
                    = $this->get_next_date_array( $this_dt->{'sy'},
                    $this_dt->{'sm'}, $this_dt->{'sd'} );
                $s = $this_dt->{'sh'} = $this_dt->{'smin'} = 0
                    if !$this_dt->{'date'};
                $se_dgap--;
            }
        }
        if (   $time_check
            && !$this_dt->{'date'}
            && $available_time_array->[0]
            && $available_time_array->[0]
            =~ /^([0-9]{1,2}):([0-9]{2})-([0-9]{1,2}):([0-9]{2})/ )
        {
            my ( $f, $t ) = ( ( $1 * 100 + $2 ), ( $3 * 100 + $4 ) );
            if ( $e <= $f ) {
                if ($se_dgap) {
                    $e = $this_dt->{'eh'} = $this_dt->{'emin'} = 0;
                    $se_dgap--;
                }
                else {
                    $is_not_available_time = 1;
                }
            }
        }
        if ( $wday_check && $se_dgap && !$this_dt->{'freq'} ) {
            $e = $this_dt->{'eh'} = $this_dt->{'emin'} = 0
                if !$this_dt->{'date'}
                && !$this->is_available_wday( $available_wday_array,
                $this_dt->{'ey'}, $this_dt->{'em'}, $this_dt->{'ed'},
                $holiday );
            while (
                !$this->is_available_wday(
                    $available_wday_array,
                    $this->get_prev_date_array(
                        $this_dt->{'ey'}, $this_dt->{'em'},
                        $this_dt->{'ed'}
                    ),
                    $holiday
                )
                )
            {
                if ( $se_dgap <= 0 ) {
                    $is_not_available_time = 1;
                    last;
                }
                ( $this_dt->{'ey'}, $this_dt->{'em'}, $this_dt->{'ed'} )
                    = $this->get_prev_date_array( $this_dt->{'ey'},
                    $this_dt->{'em'}, $this_dt->{'ed'} );
                $se_dgap--;
            }
        }
        if ( $time_check && !$this_dt->{'date'} && !$is_not_available_time ) {
            foreach my $span (@$available_time_array) {
                if ( $span
                    =~ /^([0-9]{1,2}):([0-9]{2})-([0-9]{1,2}):([0-9]{2})/ )
                {
                    my ( $f, $t ) = ( ( $1 * 100 + $2 ), ( $3 * 100 + $4 ) );
                    if ( $s >= $f && $s < $t ) {
                        last;
                    }
                    elsif ( $s < $f ) {
                        $this_dt->{'sh'}   = ($f) ? int( $f / 100 ) : 0;
                        $this_dt->{'smin'} = ($f) ? ( $f % 100 )    : 0;
                        last;
                    }
                }
            }
            foreach my $span ( reverse @$available_time_array ) {
                if ( $span
                    =~ /^([0-9]{1,2}):([0-9]{2})-([0-9]{1,2}):([0-9]{2})/ )
                {
                    my ( $f, $t ) = ( ( $1 * 100 + $2 ), ( $3 * 100 + $4 ) );
                    if (   ( $e <= $t && $e > $f )
                        || ( $t == 2400 && $e == 0 && $se_dgap ) )
                    {
                        last;
                    }
                    elsif ( $t < 2400 && $e == 0 && $se_dgap ) {
                        (   $this_dt->{'ey'}, $this_dt->{'em'},
                            $this_dt->{'ed'}
                            )
                            = $this->timestamp_to_array(
                            $this->get_prev_date(
                                $this->dt_stamp(
                                    $this_dt->{'ey'}, $this_dt->{'em'},
                                    $this_dt->{'ed'}
                                )
                            )
                            );
                        $this_dt->{'eh'}   = ($t) ? int( $t / 100 ) : 0;
                        $this_dt->{'emin'} = ($t) ? ( $t % 100 )    : 0;
                        last;
                    }
                    elsif ( $e > $t ) {
                        $this_dt->{'eh'}   = ($t) ? int( $t / 100 ) : 0;
                        $this_dt->{'emin'} = ($t) ? ( $t % 100 )    : 0;
                        last;
                    }
                }
            }
        }
    }
    if (!$is_not_available_time
        && ( $wday_check
            || ( ref $samezone_categories_array
                && @$samezone_categories_array ) )
        )
    {
        $this_dt->{'indefinite'} = 0
            if $this_dt->{'freq'}
            ;    #無期限に重複をチェックできない
        @date = $this->get_date_list($this_dt);

        #設定可能曜日チェック
        if ($wday_check) {
            $is_not_available_time = 1;
            foreach my $d (@date) {
                if ( $d
                    =~ /^(-?[0-9]+<|)([0-9]{4})([0-9]{2})([0-9]{2})(>-?[0-9]+|)$/
                    )
                {
                    my ( $p, $y, $m, $d, $n )
                        = ( $1, $2, $3 + 0, $4 + 0, $5 );
                    next
                        if $p
                        && $p =~ /[1-9]/
                        && ( !$n || $n !~ /[1-9]/ )
                        && $this_dt->{'eh'} == 0
                        && $this_dt->{'emin'} == 0;
                    if ($this->is_available_wday(
                            $available_wday_array,
                            $y, $m, $d, $holiday
                        )
                        )
                    {
                        $is_not_available_time = 0;
                        last;
                    }
                }
            }
        }

        #重複チェック
        if (   !$is_not_available_time
            && ref $samezone_categories_array
            && @$samezone_categories_array
            && $category_field
            && $category_id )
        {
            my ( %dt, %date, %category );
            tie( %date, $env::dbm_file, $default::db . '/date',
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %date ) {
                tie( %dt, $env::dbm_file, $default::db . '/dt',
                    O_RDONLY, $env::dbm_mode );
                tie( %category, $env::dbm_file, "$default::db/category",
                    O_RDONLY, $env::dbm_mode )
                    if $category_field;
                if ( ref tied %dt ) {
                    foreach my $ymd (@date) {
                        my ( $y, $m, $d, $md );
                        my $this_prev
                            = ( $ymd =~ s/^([\-]?[0-9]+)<// ) ? eval($1) : 0;
                        my $this_next
                            = ( $ymd =~ s/>([\-]?[0-9]+)$// ) ? eval($1) : 0;
                        my $this_start_min
                            = ( !$this_dt->{'date'} && !$this_prev )
                            ? $this_dt->{'sh'} * 60 + $this_dt->{'smin'}
                            : 0;
                        my $this_end_min
                            = ( !$this_dt->{'date'} && !$this_next )
                            ? $this_dt->{'eh'} * 60 + $this_dt->{'emin'}
                            : 1440;

                        #関連する日付
                        my @dt = ( 'DAILY', $ymd );
                        if ( $ymd =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
                            $md = $2 . $3;
                            ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
                            my $w = $this->number_to_wday(
                                $this->get_wday( $y, $m, $d ) );
                            my $wn
                                = int( $d / 7 ) + ( ( $d % 7 ) ? 1 : 0 ) . $w;
                            push( @dt, $w, $wn, $d, $md );
                        }

                        #関連する日付のイベント
                    DATEID:
                        foreach my $dt (@dt) {
                            if ( $date{ $this->encode_utf8($dt) } ) {
                                my @n = split(
                                    /,/,
                                    $this->decode_utf8(
                                        $date{ $this->encode_utf8($dt) }
                                    )
                                );
                            EVENT:
                                foreach my $n (@n) {
                                    my $prev
                                        = ( $n =~ s/^([\-]?[0-9]+)<// )
                                        ? eval($1)
                                        : 0;
                                    my $next
                                        = ( $n =~ s/>([\-]?[0-9]+)$// )
                                        ? eval($1)
                                        : 0;

                                    #同一番号/カテゴリーチェック
                                    next
                                        if ( defined $this_n
                                        && $this_n ne ''
                                        && $this_n eq $n )
                                        || !$category{ $this->encode_utf8($n)
                                        }
                                        || !grep( $this->decode_utf8(
                                            $category{ $this->encode_utf8($n)
                                            }
                                            ) eq $_,
                                        @$samezone_categories_array );

                                    if ($this->decode_utf8(
                                            $dt{ $this->encode_utf8($n) }
                                        )
                                        =~ /^([0-9]{8})T?([0-9]{6}|)-([0-9]{8})T?([0-9]{6}|)\x09([^\x09\x0A\x0C\x0D\x20]*)/
                                        )
                                    {
                                        my ($start_date, $start_time,
                                            $end_date,   $end_time,
                                            $rrule
                                        ) = ( $1, $2, $3, $4, $5 );
                                        my $rrule_hash;
                                        if ($rrule) {
                                            while ( $rrule
                                                =~ /\G([0-9a-zA-Z_]+)=([0-9a-zA-Z_,]+);?/g
                                                )
                                            {
                                                my ( $k, $v ) = ( $1, $2 );
                                                if ( $v =~ /,/ ) {
                                                    $rrule_hash->{$k} = [
                                                        split( /,/, $v ) ];
                                                }
                                                else {
                                                    $rrule_hash->{$k} = $v;
                                                }
                                            }
                                            $this->scalar_to_array(
                                                \$rrule_hash->{'BYDAY'} )
                                                if $rrule_hash->{'BYDAY'};
                                            next EVENT
                                                if !$this->is_required_date(
                                                $dt,         $rrule_hash,
                                                $start_date, $end_date,
                                                $ymd,        $y,
                                                $m,          $d,
                                                $md,         \$prev,
                                                \$next
                                                );
                                        }
                                        if (  !$this_dt->{'date'}
                                            && $start_time ne '' )
                                        {
                                            my $start_min
                                                = (   !$prev
                                                    && $start_time
                                                    =~ /^([0-9]{2})([0-9]{2})/
                                                ) ? $1 * 60 + $2 : 0;
                                            my $end_min
                                                = (   !$next
                                                    && $end_time
                                                    =~ /^([0-9]{2})([0-9]{2})/
                                                ) ? $1 * 60 + $2 : 1440;
                                            if ((   $start_min
                                                    < $this_start_min
                                                    && $end_min
                                                    > $this_start_min
                                                )
                                                || ( $start_min
                                                    > $this_start_min
                                                    && $start_min
                                                    < $this_end_min )
                                                || ($end_min > $this_start_min
                                                    && $end_min
                                                    < $this_end_min )
                                                || ($start_min < $this_end_min
                                                    && $end_min
                                                    > $this_end_min )
                                                )
                                            {
                                                push( @overlapping_date,
                                                    $ymd )
                                                    if !grep( $_ eq $ymd,
                                                    @overlapping_date );
                                            }
                                        }
                                        else {
                                            push( @overlapping_date, $ymd )
                                                if !grep( $_ eq $ymd,
                                                @overlapping_date );
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                untie %dt;
            }
            untie %date;
        }
    }
    if ($is_not_available_time) {
        push(
            @{ $this->{'dialog'} },
            '「'
                . $category_name
                . '」は設定可能な'
                . ( ($time_check) ? '時間帯が' : '日が' )
                . (
                ($wday_check)
                ? '「'
                    . $this->wdaynum_to_name(@$available_wday_array) . '」'
                : ''
                )
                . (
                  ($time_check)
                ? ( ($wday_check) ? 'の' : '' ) . '「'
                    . join( ',', @$available_time_array ) . '」'
                : ''
                )
                . 'に制限されています。'
        );
        return 0;
    }
    if (@overlapping_date) {
        my $categories = '「' . $category_name . '」';
        foreach my $id (@$samezone_categories_array) {
            $categories
                .= '「' . $setting::event_category{$id}{'name'} . '」'
                if $id ne $category_id;
        }
        push(
            @{ $this->{'dialog'} },
            $categories
                . 'は予定の重複が禁止されています。次の日付の他の予定と重複しています。'
                . join( '、',
                map( $this->ymd_to_date($_), @overlapping_date ) )
                . '。'
        );
        return 0;
    }
    return 1;
}

sub get_date_list ($$) {
    my ( $this, $this_dt ) = @_;
    my @date = ();
    if ( $this_dt->{'freq'} ) {
        my $until = $this->dt_stamp( $this_dt->{'uy'}, $this_dt->{'um'},
            $this_dt->{'ud'} );
        if ( $this_dt->{'freq'} eq 'daily' || $this_dt->{'freq'} eq 'weekly' )
        {
            my $dt_start_date
                = $this->dt_stamp( $this_dt->{'sy'}, $this_dt->{'sm'},
                $this_dt->{'sd'} );
            my $dt_end_date
                = $this->dt_stamp( $this_dt->{'ey'}, $this_dt->{'em'},
                $this_dt->{'ed'} );
            my $se_dgap
                = ( !$this_dt->{'date'} && $dt_start_date < $dt_end_date )
                ? 1
                : 0;
            if ( $this_dt->{'freq'} eq 'daily' ) {
                @date = $this->get_all_date( $dt_start_date, $until, $se_dgap,
                    $this_dt->{'interval'} );
            }
            elsif ( $this_dt->{'freq'} eq 'weekly' ) {
                @date = $this->get_all_weekly_date( $dt_start_date, $until,
                    $se_dgap, $this_dt->{'byday'}, $this_dt->{'interval'} );
            }
        }
        else {
            my $dt_start_time = timelocal(
                0,
                ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'smin'} ),
                ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'sh'} ),
                $this_dt->{'sd'},
                ( $this_dt->{'sm'} - 1 ),
                $this_dt->{'sy'}
            );
            my $dt_end_time = timelocal(
                0,
                ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'emin'} ),
                ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'eh'} ),
                $this_dt->{'ed'},
                ( $this_dt->{'em'} - 1 ),
                $this_dt->{'ey'}
            );
            my $se_gap  = $dt_end_time - $dt_start_time;
            my $se_dgap = int(
                (   ( $this_dt->{'date'} )
                    ? $se_gap - 1
                    : $se_gap + $this_dt->{'sh'} * 3600
                        + $this_dt->{'smin'} * 60
                ) / 86400
            );
            if ( $this_dt->{'freq'} eq 'monthly' ) {
                if ( $this_dt->{'by'} && $this_dt->{'by'} eq 'bywday' ) {
                    @date
                        = $this->get_all_monthly_wday( $this_dt->{'sy'},
                        $this_dt->{'sm'}, $this_dt->{'sd'}, $until, $se_dgap,
                        $this_dt->{'interval'} );
                }
                else {
                    @date
                        = $this->get_all_monthlyday( $this_dt->{'sy'},
                        $this_dt->{'sm'}, $this_dt->{'sd'}, $until, $se_dgap,
                        $this_dt->{'interval'} );
                }
            }
            elsif ( $this_dt->{'freq'} eq 'yearly' ) {
                @date
                    = $this->get_all_yearlyday( $this_dt->{'sy'},
                    $this_dt->{'sm'}, $this_dt->{'sd'}, $until, $se_dgap,
                    $this_dt->{'interval'} );
            }
        }
    }
    else {
        my $dt_start_time = timelocal(
            0,
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'smin'} ),
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'sh'} ),
            $this_dt->{'sd'},
            ( $this_dt->{'sm'} - 1 ),
            $this_dt->{'sy'}
        );
        my $dt_end_time = timelocal(
            0,
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'emin'} ),
            ( ( $this_dt->{'date'} ) ? 0 : $this_dt->{'eh'} ),
            $this_dt->{'ed'},
            ( $this_dt->{'em'} - 1 ),
            $this_dt->{'ey'}
        );
        my $se_gap  = $dt_end_time - $dt_start_time;
        my $se_dgap = int(
            (   ( $this_dt->{'date'} )
                ? $se_gap - 1
                : $se_gap + $this_dt->{'sh'} * 3600 + $this_dt->{'smin'} * 60
            ) / 86400
        );
        if ($se_dgap) {
            @date = $this->get_continuous_dates(
                $this->dt_stamp(
                    $this_dt->{'sy'}, $this_dt->{'sm'}, $this_dt->{'sd'}
                ),
                $se_dgap
            );
        }
        else {
            @date = (
                $this->dt_stamp(
                    $this_dt->{'sy'}, $this_dt->{'sm'}, $this_dt->{'sd'}
                )
            );
        }
    }
    return @date;
}

sub ymd_to_date ($$) {
    my ( $this, $ymd ) = @_;
    if ( $ymd =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        return
              $y . '年'
            . $m . '月'
            . $d . '日('
            . $this->get_wday_name( $y, $m, $d ) . ')';
    }
    else {
        return '';
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #設定DB読み込み設定
        $setting::general  = 1;    #一般設定読み込み
        $setting::deco     = 1;    #テキスト装飾設定読み込み
        $setting::security = 1;    #アクセス権設定読み込み
        $setting::table    = 1;    #テーブル設定読み込み
        $setting::page     = 1;    #ページ設定読み込み

        #クッキー
        $cookie::id_enabled = 1;    #ID情報クッキー読み込み
        $cookie::default_enabled
            = 1;    #ユーザー設定情報クッキー読み込み
        $cookie::name
            = ( $default::subdivided && defined $main::c && $main::c ne '' )
            ? 'INPUT_' . ( uc $main::c )
            : 'INPUT';
        $cookie::path = $ENV{'SCRIPT_NAME'};

        #フィールド読み込み設定
        $setting::field_db_name = 'inputfield';    #フィールド設定DB
        $setting::field         = 2
            ; #フィールド設定読み込み（2=オリジナルも読み込む）
        $setting::field_style_disabled = 0
            ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視
        $setting::field_tag_enabled = 'no'
            ;   #tag=タグのみ/all=全ての設定/no=タグをつけない

        #必須項目
        @setting::form = ( 'k', 'm', 'v', 'n' );
        if ( $ENV{'QUERY_STRING'} ) {
            undef $main::a;
            undef $main::delete
                if $main::delete && !$cgi::input->referer_is_me(1);
        }
        elsif ( $ENV{'REQUEST_METHOD'} eq 'POST'
            && !$cgi::input->referer_is_me )
        {
            undef $main::a;
            undef $main::delete;
        }

        #デフォルト画面の調整
        if ( $default::subdivided && defined $main::c && $main::c ne '' ) {
            unshift( @setting::form, 'c' );
        }

        #イメージCSS
        $setting::image_css = 'calendar-image.css';

        #各種キー設定
        #デフォルトセキュリティキー
        if ( !defined $setting::security_key || $setting::security_key eq '' )
        {
            if ( defined $main::n && $main::n ne '' ) {
                $setting::security_key = 'revise';
            }
            else {
                $setting::security_key = 'input';
            }
        }

        #ページ設定キー
        $setting::key = 'editor'
            if !defined $setting::key || $setting::key eq '';

        #デフォルトキー
        $main::k = 'editor' if !defined $main::k || !$main::k;

        #クラス登録
        Common::class_register( 'editor', 'Editor' );

    }
}

1;
