package Ics;
require CMONOS::Calendar::CalCommon;
@ISA = qw(CalCommon);
use strict;
use utf8;
######################################################################
# icsファイル書き出しモジュール
# Ics.pm ver1.1.9
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-04-10
######################################################################
sub _tz_to_id ($$) {
    my ( $this, $tzname ) = @_;
    my %tzname_to_tzid = (
        'ACT'                          => 'Australia/Darwin',
        'AET'                          => 'Australia/Sydney',
        'AGT'                          => 'America/Buenos_Aires',
        'ART'                          => 'Africa/Cairo',
        'AST'                          => 'America/Anchorage',
        'America/Atka'                 => 'America/Adak',
        'America/Ensenada'             => 'America/Tijuana',
        'America/Fort_Wayne'           => 'America/Indianapolis',
        'America/Indiana/Indianapolis' => 'America/Indianapolis',
        'America/Kentucky/Louisville'  => 'America/Louisville',
        'America/Knox_IN'              => 'America/Indiana/Knox',
        'America/Porto_Acre'           => 'America/Rio_Branco',
        'America/Rosario'              => 'America/Cordoba',
        'America/Shiprock'             => 'America/Denver',
        'America/Virgin'               => 'America/St_Thomas',
        'Antarctica/South_Pole'        => 'Antarctica/McMurdo',
        'Arctic/Longyearbyen'          => 'Europe/Oslo',
        'Asia/Ashkhabad'               => 'Asia/Ashgabat',
        'Asia/Chungking'               => 'Asia/Chongqing',
        'Asia/Dacca'                   => 'Asia/Dhaka',
        'Asia/Istanbul'                => 'Europe/Istanbul',
        'Asia/Macao'                   => 'Asia/Macau',
        'Asia/Tel_Aviv'                => 'Asia/Jerusalem',
        'Asia/Thimbu'                  => 'Asia/Thimphu',
        'Asia/Ujung_Pandang'           => 'Asia/Makassar',
        'Asia/Ulan_Bator'              => 'Asia/Ulaanbaatar',
        'Australia/ACT'                => 'Australia/Sydney',
        'Australia/Canberra'           => 'Australia/Sydney',
        'Australia/LHI'                => 'Australia/Lord_Howe',
        'Australia/NSW'                => 'Australia/Sydney',
        'Australia/North'              => 'Australia/Darwin',
        'Australia/Queensland'         => 'Australia/Brisbane',
        'Australia/South'              => 'Australia/Adelaide',
        'Australia/Tasmania'           => 'Australia/Hobart',
        'Australia/Victoria'           => 'Australia/Melbourne',
        'Australia/West'               => 'Australia/Perth',
        'Australia/Yancowinna'         => 'Australia/Broken_Hill',
        'BET'                          => 'America/Sao_Paulo',
        'BST'                          => 'Asia/Dhaka',
        'Brazil/Acre'                  => 'America/Porto_Acre',
        'Brazil/DeNoronha'             => 'America/Noronha',
        'Brazil/East'                  => 'America/Sao_Paulo',
        'Brazil/West'                  => 'America/Manaus',
        'CAT'                          => 'Africa/Harare',
        'CNT'                          => 'America/St_Johns',
        'CST'                          => 'America/Chicago',
        'CST6CDT'                      => 'America/Chicago',
        'CTT'                          => 'Asia/Shanghai',
        'Canada/Atlantic'              => 'America/Halifax',
        'Canada/Central'               => 'America/Winnipeg',
        'Canada/East-Saskatchewan'     => 'America/Regina',
        'Canada/Eastern'               => 'America/Toronto',
        'Canada/Mountain'              => 'America/Edmonton',
        'Canada/Newfoundland'          => 'America/St_Johns',
        'Canada/Pacific'               => 'America/Vancouver',
        'Canada/Saskatchewan'          => 'America/Regina',
        'Canada/Yukon'                 => 'America/Whitehorse',
        'Chile/Continental'            => 'America/Santiago',
        'Chile/EasterIsland'           => 'Pacific/Easter',
        'Cuba'                         => 'America/Havana',
        'EAT'                          => 'Africa/Addis_Ababa',
        'ECT'                          => 'Europe/Paris',
        'EST'                          => 'America/Indianapolis',
        'EST5EDT'                      => 'America/New_York',
        'Egypt'                        => 'Africa/Cairo',
        'Eire'                         => 'Europe/Dublin',
        'Etc/GMT+0'                    => 'Etc/GMT',
        'Etc/GMT-0'                    => 'Etc/GMT',
        'Etc/GMT0'                     => 'Etc/GMT',
        'Etc/Greenwich'                => 'Etc/GMT',
        'Etc/Universal'                => 'Etc/UTC',
        'Etc/Zulu'                     => 'Etc/UTC',
        'Europe/Nicosia'               => 'Asia/Nicosia',
        'Europe/Tiraspol'              => 'Europe/Chisinau',
        'GB'                           => 'Europe/London',
        'GB-Eire'                      => 'Europe/London',
        'GMT'                          => 'Etc/GMT',
        'GMT+0'                        => 'Etc/GMT+0',
        'GMT-0'                        => 'Etc/GMT-0',
        'GMT0'                         => 'Etc/GMT0',
        'Greenwich'                    => 'Etc/Greenwich',
        'HST'                          => 'Pacific/Honolulu',
        'Hongkong'                     => 'Asia/Hong_Kong',
        'IET'                          => 'America/Indianapolis',
        'IST'                          => 'Asia/Calcutta',
        'Iceland'                      => 'Atlantic/Reykjavik',
        'Iran'                         => 'Asia/Tehran',
        'Israel'                       => 'Asia/Jerusalem',
        'JST'                          => 'Asia/Tokyo',
        'Jamaica'                      => 'America/Jamaica',
        'Japan'                        => 'Asia/Tokyo',
        'Kwajalein'                    => 'Pacific/Kwajalein',
        'Libya'                        => 'Africa/Tripoli',
        'MIT'                          => 'Pacific/Apia',
        'MST'                          => 'America/Phoenix',
        'MST7MDT'                      => 'America/Denver',
        'Mexico/BajaNorte'             => 'America/Tijuana',
        'Mexico/BajaSur'               => 'America/Mazatlan',
        'Mexico/General'               => 'America/Mexico_City',
        'Mideast/Riyadh87'             => 'Asia/Riyadh87',
        'Mideast/Riyadh88'             => 'Asia/Riyadh88',
        'Mideast/Riyadh89'             => 'Asia/Riyadh89',
        'NET'                          => 'Asia/Yerevan',
        'NST'                          => 'Pacific/Auckland',
        'NZ'                           => 'Pacific/Auckland',
        'NZ-CHAT'                      => 'Pacific/Chatham',
        'Navajo'                       => 'America/Denver',
        'PLT'                          => 'Asia/Karachi',
        'PNT'                          => 'America/Phoenix',
        'PRC'                          => 'Asia/Shanghai',
        'PRT'                          => 'America/Puerto_Rico',
        'PST'                          => 'America/Los_Angeles',
        'PST8PDT'                      => 'America/Los_Angeles',
        'Pacific/Samoa'                => 'Pacific/Pago_Pago',
        'Poland'                       => 'Europe/Warsaw',
        'Portugal'                     => 'Europe/Lisbon',
        'ROC'                          => 'Asia/Taipei',
        'ROK'                          => 'Asia/Seoul',
        'SST'                          => 'Pacific/Guadalcanal',
        'Singapore'                    => 'Asia/Singapore',
        'AST4'                         => 'America/Puerto_Rico',
        'AST4ADT'                      => 'America/Halifax',
        'CST6'                         => 'America/Regina',
        'CST6CDT'                      => 'America/Chicago',
        'EST5'                         => 'America/Indianapolis',
        'EST5EDT'                      => 'America/New_York',
        'HST10'                        => 'Pacific/Honolulu',
        'MST7'                         => 'America/Phoenix',
        'MST7MDT'                      => 'America/Denver',
        'PST8'                         => 'Pacific/Pitcairn',
        'PST8PDT'                      => 'America/Los_Angeles',
        'YST9'                         => 'Pacific/Gambier',
        'YST9YDT'                      => 'America/Anchorage',
        'Turkey'                       => 'Europe/Istanbul',
        'UCT'                          => 'Etc/UCT',
        'US/Alaska'                    => 'America/Anchorage',
        'US/Aleutian'                  => 'America/Adak',
        'US/Arizona'                   => 'America/Phoenix',
        'US/Central'                   => 'America/Chicago',
        'US/East-Indiana'              => 'America/Indianapolis',
        'US/Eastern'                   => 'America/New_York',
        'US/Hawaii'                    => 'Pacific/Honolulu',
        'US/Indiana-Starke'            => 'America/Indiana/Knox',
        'US/Michigan'                  => 'America/Detroit',
        'US/Mountain'                  => 'America/Denver',
        'US/Pacific'                   => 'America/Los_Angeles',
        'US/Pacific-New'               => 'America/Los_Angeles',
        'US/Samoa'                     => 'Pacific/Pago_Pago',
        'UTC'                          => 'Etc/UTC',
        'Universal'                    => 'Etc/Universal',
        'VST'                          => 'Asia/Saigon',
        'W-SU'                         => 'Europe/Moscow',
        'Zulu'                         => 'Etc/Zulu',
    );
    return
          ( $tzname_to_tzid{$tzname} ) ? $tzname_to_tzid{$tzname}
        : ( $tzname =~ /^([A-Z]{3}(?![0-9a-zA-Z_]))/ && $tzname_to_tzid{$1} )
        ? $tzname_to_tzid{$1}
        : $tzname;
}

# メイン
sub run ($) {
    no warnings qw(untie);
    my $this = shift;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #---------------------------------------------------------------------
    #書き出し

    #フィールド
    my $title_field
        = ( $field::for{'title'} ) ? $field::for{'title'}
        : ( $field::for{'key'} )  ? $field::for{'key'}
        : ( $field::for{'name'} ) ? $field::for{'name'}
        :                           "";
    my @field = ();
    my ( %dt, $pending_time, $preview_time );
    my %database = ();
    my $tzid     = '';
    foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
        my $id = $field::ORDER{$o};
        if (ref $field::obj{$id}
            && (   $id eq $title_field
                || $field::obj{$id}->{'class'} eq 'xcal_category'
                || ( defined $setting::security{'detail'}{'passwordCheck'}
                    && $setting::security{'detail'}{'passwordCheck'} eq
                    'nocheck' )
                || $this->is_one_of_owners
            )
            && !$field::obj{$id}->{'Hidden'}
            && ( $field::obj{$id}->{'Open'} || $this->is_secure('secret') )
            )
        {
            push( @field, $id );
            my $db = $field::obj{$id}->db;
            if (   defined $db
                && $db
                && !ref $database{$db}
                && !ref tied %{ $database{$db} } )
            {
                tie(%{ $database{$db} }, $env::dbm_file,
                    "$default::db/$db",  O_RDONLY,
                    $env::dbm_mode
                );
            }
        }
    }
    if ( !$database{'mtime'} ) {
        tie( %{ $database{'mtime'} },
            $env::dbm_file, "$default::db/mtime", O_RDONLY, $env::dbm_mode );
    }
    if (  !$database{'ctime'}
        && $setting::general{'pendingTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        tie( %{ $database{'ctime'} },
            $env::dbm_file, "$default::db/ctime", O_RDONLY, $env::dbm_mode );
        $pending_time = time - $setting::general{'pendingTime'};
    }
    if (   $setting::general{'previewTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        $preview_time
            = $this->time_id( time + $setting::general{'previewTime'} );
    }

    #HTTPヘッダ
    binmode STDOUT, ":utf8";
    $this->print_http_header('text/calendar; charset=utf-8;', 0, 'web' . $default::id . '.ics');
    print 'BEGIN:VCALENDAR',                          "\x0D\x0A";
    print 'PRODID:-//CMONOS Co. Ltd.//CMONOS.JP//JA', "\x0D\x0A";
    print 'METHOD:PUBLISH',                           "\x0D\x0A";
    print 'VERSION:2.0',                              "\x0D\x0A";
    if ( $ENV{'TZ'} ) {
        $tzid = $this->_tz_to_id( $ENV{'TZ'} );
        print 'BEGIN:VTIMEZONE',         "\x0D\x0A";
        print 'TZID:',                   $tzid, "\x0D\x0A";
        print 'BEGIN:STANDARD',          "\x0D\x0A";
        print 'DTSTART:19390101T000000', "\x0D\x0A";
        if ( $ENV{'TZ'}
            =~ /^([A-Z]{3}(?![0-9a-zA-Z_]))([\-+]?[0-9]*)([A-Z]{0,3}(?![0-9a-zA-Z_]))([\-+]?[0-9]*)/
            )
        {
            my ( $std, $std_offset, $dst, $dst_offset ) = ( $1, $2, $3, $4 );
            print 'TZOFFSETFROM:',
                $this->offset_format(
                ($dst_offset) ? $dst_offset : $std_offset ), "\x0D\x0A";
            print 'TZOFFSETTO:', $this->offset_format($std_offset),
                "\x0D\x0A";
        }
        print 'TZNAME:', $ENV{'TZ'}, "\x0D\x0A";
        print 'END:STANDARD',  "\x0D\x0A";
        print 'END:VTIMEZONE', "\x0D\x0A";
    }
    my $domain = $env::server_name;
    my $dirid  = $this->home_directory(1);
    $dirid =~ s!^/!!;
    $dirid =~ s/[^0-9a-zA-Z_\-]/\./g;
    $dirid .= ( ($dirid) ? '.' : '' ) . $default::id;

    tie( %dt, $env::dbm_file, $default::db . '/dt', O_RDONLY,
        $env::dbm_mode );
    if ( ref tied %dt ) {
    EVENT:
        while ( my ( $n, $dt_data ) = each %dt ) {
            next EVENT
                if $pending_time
                && $database{'ctime'}->{ $this->encode_utf8($n) }
                && $database{'ctime'}->{ $this->encode_utf8($n) }
                > $pending_time;
            my $category_id;
            if (   ref $database{'category'}
                && %setting::event_category
                && ref $setting::event_category{'ORDER'}
                && %{ $setting::event_category{'ORDER'} } )
            {
                $category_id = $this->decode_utf8(
                    $database{'category'}->{ $this->encode_utf8($n) } );
                next EVENT
                    if defined $category_id
                    && ref $setting::event_category{$category_id}
                    && $setting::event_category{$category_id}{'closed'}
                    && !$this->is_secure('closed');
            }
            my ( $dtspan,  $rrule ) = split( /\x09/, $dt_data );
            my ( $dtstart, $dtend ) = split( /-/,    $dtspan );
            if ($preview_time) {
                my $s = $dtstart;
                $s =~ s/T//;
                $s .= '000000' if length $s == 8;
                next EVENT if $preview_time < $s;
            }
            my $path        = $default::file . '/' . $n;
            my $description = '';
            print 'BEGIN:VEVENT', "\x0D\x0A";
            print 'UID:',
                $dirid
                . (
                ( $default::subdivided && $main::c ) ? '.' . $main::c : '' )
                . '.'
                . $n . '@'
                . $domain, "\x0D\x0A";
            print 'DTSTAMP:', $this->time_to_dtstamp($n), "\x0D\x0A";
            print $this->ical_data(
                'SUMMARY:'
                    . $field::obj{$title_field}->format_for_text(
                    $this->decode_utf8(
                        $database{ $field::obj{$title_field}->db }
                            ->{ $this->encode_utf8($n) }
                    ),
                    $path, $main::c, $n
                    )
                ),
                "\x0D\x0A"
                if defined $title_field
                && $title_field
                && grep( $_ eq $title_field, @field );
            if ((   defined $setting::security{'detail'}{'passwordCheck'}
                    && $setting::security{'detail'}{'passwordCheck'} eq
                    'nocheck'
                )
                || ( $this->is_secure('detail') && $this->is_staff )
                )
            {
                print $this->ical_data(
                    'LOCATION:'
                        . $field::obj{ $field::for{'location'} }
                        ->format_for_text(
                        $this->decode_utf8(
                            $database{
                                $field::obj{ $field::for{'location'} }->db
                            }->{ $this->encode_utf8($n) }
                        ),
                        $path, $main::c, $n
                        )
                    ),
                    "\x0D\x0A"
                    if defined $field::for{'location'}
                    && $field::for{'location'}
                    && grep( $_ eq $field::for{'location'}, @field );
                print $this->ical_data(
                    'CONTACT:'
                        . $field::obj{ $field::for{'name'} }->format_for_text(
                        $this->decode_utf8(
                            $database{ $field::obj{ $field::for{'name'} }
                                    ->db }->{ $this->encode_utf8($n) }
                        ),
                        $path, $main::c, $n
                        )
                    ),
                    "\x0D\x0A"
                    if defined $field::for{'name'}
                    && $field::for{'name'}
                    && grep( $_ eq $field::for{'name'}, @field );
                foreach my $id (@field) {
                    next
                        if $id eq $title_field
                        || $id eq $field::for{'location'}
                        || $id eq $field::for{'name'};
                    my $db   = $field::obj{$id}->db;
                    my $data = $field::obj{$id}->format_for_text(
                        $this->decode_utf8(
                            $database{$db}->{ $this->encode_utf8($n) }
                        ),
                        $path, $main::c, $n
                    );
                    $data =~ s/\x0A+$//;
                    $description
                        .= '['
                        . $field::obj{$id}->name . "]\x0A"
                        . $data
                        . "\x0A\x0A"
                        if $data =~ /[^\x09\x0A\x0C\x0D\x20]/;
                }
                print $this->ical_data( 'DESCRIPTION:' . $description ),
                    "\x0D\x0A"
                    if $description ne '';
            }
            print $this->ical_data(
                      'URL;VALUE=URI:'
                    . $this->my_url
                    . '?m=DetailViewer'
                    . (
                    ( $default::subdivided && $main::c )
                    ? '&c=' . $main::c
                    : ''
                    )
                    . '&n='
                    . $n
                ),
                "\x0D\x0A";
            print 'DTSTART', ( ($tzid) ? ';TZID=' . $tzid : '' ),
                ( ( $dtstart =~ /^[0-9]{8}$/ ) ? ';VALUE=DATE' : '' ), ':',
                $dtstart, "\x0D\x0A";
            print 'DTEND', ( ($tzid) ? ';TZID=' . $tzid : '' ),
                ( ( $dtend =~ /^[0-9]{8}$/ ) ? ';VALUE=DATE' : '' ), ':',
                $dtend, "\x0D\x0A";
            print 'RRULE:', $rrule, "\x0D\x0A" if $rrule;
            print $this->ical_data(
                'CATEGORIES:'
                    . (
                    (   defined $setting::event_category{$category_id}{'name'}
                            && $setting::event_category{$category_id}{'name'}
                            ne ''
                    )
                    ? $setting::event_category{$category_id}{'name'}
                    : $category_id
                    )
                ),
                "\x0D\x0A"
                if defined $category_id
                && ref $setting::event_category{$category_id};
            print 'TRANSP:TRANSPARENT', "\x0D\x0A";
            print 'PRIORITY:0',         "\x0D\x0A";
            print 'CLASS:PUBLIC',       "\x0D\x0A";
            print 'END:VEVENT',         "\x0D\x0A";
        }
    }
    untie %dt;
    while ( my ( $name, $db ) = each %database ) {
        untie %$db;
    }
    print 'END:VCALENDAR';
}

sub offset_format ($$) {
    my ( $this, $offset ) = @_;
    $offset *= -1;
    return ( ( $offset > 0 ) ? '+' : '' ) . sprintf( "%.2d", $offset ) . '00';
}

# 改行=\n
# ,;はエスケープが必要=\,\;
sub ical_data ($$) {
    my ( $this, $str ) = @_;
    $str = $this->detag($str);
    $str =~ s/,/\\,/g;
    $str =~ s/;/\\;/g;
    $str =~ s/^([0-9a-zA-Z_]+)\\;/$1;/s;
    $str =~ s/[\x0A\x0D]/\\n/g;
    my $ical_data = '';
    my $length    = 0;

    while ( $str =~ /\G(\\n|\\,|\\;|.)/sg ) {
        my $char = $1;
        my $l = ( bytes::length $char >= 2 ) ? 2 : 1;
        $length += $l;
        if ( $length > 75 ) {
            $ical_data .= "\x0D\x0A ";
            $length = 1 + $l;
        }
        $ical_data .= $char;
    }
    return $ical_data;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::deco     = 1;    #テキスト装飾設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み
    $setting::table    = 0;    #テーブル設定読み込み
    $setting::page     = 0;    #ページ設定読み込み

    $setting::field_db_name = 'inputfield';    #フィールド設定DB
    $setting::field         = 2
        ; #フィールド設定読み込み（2=オリジナルも読み込む）
    $setting::field_style_disabled = 0
        ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視
    $setting::field_tag_enabled
        = 'no'; #tag=タグのみ/all=全ての設定/no=タグをつけない

    #設定種キー設定
    $setting::security_key
        = 'browse';    #デフォルトセキュリティキー
    $setting::key = 'none';    #ページ設定キー

    #クッキー
    $cookie::id_enabled = 1;    #ID情報クッキー読み込み

    #デフォルトキー
    $main::k = 'ics';

    #クラス登録
    Common::class_register( 'ics', 'Ics' );

}

1;
