package EventCategoryEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# イベントカテゴリー設定モジュール
# EventCategoryEditor.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-10-29
######################################################################
# 実行
sub run ($) {
    my $this               = shift;
    my $eventcategory_pref = $default::preference_dir . '/event_category';

    #全体編集書き込み
    if ( defined $main::a && $main::a eq 'e' ) {
        no warnings qw(numeric);
        my ( $k, $v, %old );

        #不正なデータの除去
        while ( ( $k, $v ) = each %main::order ) {
            $main::data{ $k . '_closed' } = ''
                if !defined $main::data{ $k . '_closed' }
                || !$main::data{ $k . '_closed' };
            if (  !$main::data{ $k . '_color' }
                || $main::data{ $k . '_color' } eq 'random' )
            {
                $main::data{ $k . '_color' } = $this->rand_color;
            }
            elsif ($main::data{ $k . '_color' } ne ''
                && $this->is_invalid_color( $main::data{ $k . '_color' } ) )
            {
                push(
                    @{ $this->{'dialog'} },
                    $k
                        . ' のカラーを更新できませんでした。色指定を確認してください。'
                );
                delete $main::data{ $k . '_color' };
            }
            if ( !$main::data{ $k . '_fontcolor' } ) {
                $main::data{ $k . '_fontcolor' } = '#FFFFFF';
            }
            elsif ( $main::data{ $k . '_fontcolor' } eq 'random' ) {
                $main::data{ $k . '_fontcolor' } = $this->rand_color;
            }
            elsif ($main::data{ $k . '_fontcolor' } ne ''
                && $this->is_invalid_color( $main::data{ $k . '_fontcolor' } )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    $k
                        . ' の文字色を更新できませんでした。色指定を確認してください。'
                );
                delete $main::data{ $k . '_fontcolor' };
            }
            $this->detag( \$main::data{ $k . '_name' } );
            if ( !defined $main::data{ $k . '_name' }
                || $main::data{ $k . '_name' } eq '' )
            {
                push(
                    @{ $this->{'dialog'} },
                    $k
                        . ' のカテゴリー名を更新できませんでした。カテゴリー名が空です。'
                );
                delete $main::data{ $k . '_name' };
            }
            $main::data{ $k . '_overlapDisabled' } = ''
                if !$main::data{ $k . '_overlapDisabled' };
            if (   $main::data{ $k . '_eventMargin' } ne ''
                && $this->is_nan( $main::data{ $k . '_eventMargin' } ) )
            {
                push(
                    @{ $this->{'dialog'} },
                    $k
                        . ' の前後間隔を更新できませんでした。数を確認してください。'
                );
                delete $main::data{ $k . '_eventMargin' };
            }
            $main::data{ $k . '_availableTime' } = $this->available_time_fix(
                $main::data{ $k . '_availableTime' } );
            $main::data{ $k . '_availableWday' } = ''
                if !defined $main::data{ $k . '_availableWday' };
            if ( defined $main::data{ $k . '_admin' }
                && $main::data{ $k . '_admin' } ne '' )
            {
                my @addr = $this->get_address( $main::data{ $k . '_admin' } );
                if (@addr) {
                    $main::data{ $k . '_admin' } = join( ',', @addr );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        $k
                            . ' の担当者メールアドレスを更新できませんでした。メールアドレスがみつかりません。'
                    );
                    delete $main::data{ $k . '_admin' };
                }
            }
        }

        #表示順データ読み込み
        %old = $this->import_order( $eventcategory_pref, \%main::order );

        #データ削除
        push( @main::delete, $main::delete )
            if defined $main::delete && $main::delete;
        @main::delete = grep( $_ ne '', @main::delete );
        if ( @main::delete > 0 ) {

            #順序判定への影響を回避
            foreach (@main::delete) {
                delete $old{$_};
                delete $main::order{$_};
            }
        }

        #順序判定
        my $no = 0;
        foreach (
            sort {
                       $main::order{$a} <=> $main::order{$b}
                    || $old{$b} <=> $old{$a}
                    || $main::order{$a} cmp $main::order{$b}
                    || $a <=> $b
            } keys %old
            )
        {

            $no++;
            $main::data{ 'ORDER_' . $no } = $_;
            $main::data{ $_ . '_ORDER' } = $no;
        }

        #新規作成
        $this->detag( \$main::new{'name'} );
        if ( defined $main::new{'name'} && $main::new{'name'} ne '' ) {
            my $is_valid = 1;
            $main::new{'closed'} = ''
                if !defined $main::new{'closed'} || !$main::new{'closed'};
            if ( !$main::new{'color'} || $main::new{'color'} eq 'random' ) {
                $main::new{'color'} = $this->rand_color;
            }
            elsif ($main::new{'color'} ne ''
                && $this->is_invalid_color( $main::new{'color'} ) )
            {
                push(
                    @{ $this->{'dialog'} },
                    '新規カテゴリーを追加できませんでした。カラーを確認してください。'
                );
                delete $main::new{'color'};
                $is_valid = 0;
            }
            if ( !$main::new{'fontcolor'} ) {
                $main::new{'fontcolor'} = '#FFFFFF';
            }
            elsif ( $main::new{'fontcolor'} eq 'random' ) {
                $main::new{'fontcolor'} = $this->rand_color;
            }
            elsif ($main::new{'fontcolor'} ne ''
                && $this->is_invalid_color( $main::new{'fontcolor'} ) )
            {
                push(
                    @{ $this->{'dialog'} },
                    '新規カテゴリーを追加できませんでした。文字色を確認してください。'
                );
                delete $main::new{'fontcolor'};
                $is_valid = 0;
            }
            $this->detag( \$main::new{'name'} );
            if ( !defined $main::new{'name'} || $main::new{'name'} eq '' ) {
                push(
                    @{ $this->{'dialog'} },
                    '新規カテゴリーを追加できませんでした。カテゴリー名が空です。'
                );
                delete $main::new{'name'};
                $is_valid = 0;
            }
            $main::new{'overlapDisabled'} = ''
                if !$main::new{'overlapDisabled'};
            $main::new{'availableTime'}
                = $this->available_time_fix( $main::new{'availableTime'} );
            $main::new{'availableWday'} = ''
                if !defined $main::new{'availableWday'};
            if ( defined $main::new{'admin'} && $main::new{'admin'} ne '' ) {
                my @addr = $this->get_address( $main::new{'admin'} );
                if (@addr) {
                    $main::new{'admin'} = join( ',', @addr );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        '新規カテゴリーを追加できませんでした。担当者メールアドレスを確認してください。'
                    );
                    delete $main::new{'admin'};
                }
            }
            if ($is_valid) {
                my ( $k, $v, $n );
                while (1) {
                    $n++;
                    last if !defined $old{$n} || $old{$n} eq '';
                }
                $no++;
                $main::data{ 'ORDER_' . $no } = $n;
                $main::data{ $n . '_ORDER' } = $no;
                while ( ( $k, $v ) = each %main::new ) {
                    $main::data{ $n . '_' . $k } = $v;
                }
                @main::delete = grep( $_ ne $n, @main::delete );
                undef %main::new;
            }
        }

        undef %old;
        $this->write_db( $eventcategory_pref, \%main::data, 'ORDER',
            \@main::delete, \%setting::event_category, 1 );

        #CSS書き出し
        local (*CSS);
        if ( open( CSS, ">>$default::file/event-category.css" ) ) {
            binmode CSS, ":utf8";
            flock( CSS, LOCK_EX );
            truncate( CSS, 0 );
            seek( CSS, 0, 0 );
            foreach ( sort { $a <=> $b }
                keys %{ $setting::event_category{'ORDER'} } )
            {
                my $id = $setting::event_category{'ORDER'}{$_};
                if (   $setting::event_category{$id}{'color'}
                    || $setting::event_category{$id}{'fontcolor'} )
                {
                    my ($bordercolor);
                    $bordercolor
                        = $this->get_border_color(
                        $setting::event_category{$id}{'color'} )
                        if $setting::event_category{$id}{'color'};
                    print CSS
                        'table#daytable div.day-event-box a.event-category-'
                        . $id . ',' . "\x0A";
                    print CSS 'div.hourly-list div.event-category-'
                        . $id . ',' . "\x0A";
                    print CSS
                        'div#category-toggle-panel div#cltgl-color-sample-'
                        . $id . ' {' . "\x0A";
                    print CSS '  background-color: '
                        . $setting::event_category{$id}{'color'} . ';'
                        . "\x0A"
                        if $setting::event_category{$id}{'color'};
                    print CSS '  color: '
                        . $setting::event_category{$id}{'fontcolor'} . ';'
                        . "\x0A"
                        if $setting::event_category{$id}{'fontcolor'};
                    print CSS '  border-color: '
                        . $bordercolor . ';' . "\x0A"
                        if $bordercolor;
                    print CSS '}' . "\x0A";

                    if ($bordercolor) {
                        print CSS
                            'table#daytable ol.event-list a.event-category-'
                            . $id . ' {' . "\x0A";
                        print CSS '  color: ' . $bordercolor . ';' . "\x0A";
                        print CSS '}' . "\x0A";
                    }
                    if ($bordercolor) {
                        print CSS 'div.hourly-list div.event-category-'
                            . $id
                            . ' div.time-info {' . "\x0A";
                        print CSS '  background-color: '
                            . $bordercolor . ';' . "\x0A";
                        print CSS '}' . "\x0A";
                    }
                    if ( $setting::event_category{$id}{'fontcolor'} ) {
                        print CSS 'div.hourly-list div.event-category-'
                            . $id . ' a {' . "\x0A";
                        print CSS '  color: '
                            . $setting::event_category{$id}{'fontcolor'} . ';'
                            . "\x0A";
                        print CSS '}' . "\x0A";
                    }
                    print CSS "\x0A";
                }
            }
            close(CSS);
        }

    }
    else {
        $this->import_db( $eventcategory_pref, \%setting::event_category, 1 );

    }
    undef(%main::data);

    #ページヘッダ
    $this->set_css(
        'ol.checkbox li { display: inline; margin-right: 0.2em; }');
    $this->jump_menu('tools');
    $this->print_page_header('tools');
    print '<h1>イベントカテゴリーの設定</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="e" />';
    print '<fieldset>';

    #テーブル設定
    require CMONOS::CGI_Lib::Table;
    my $table = Table->new;
    $table->set(
        'tableclass' => 'tooltable',
        'trclass'    => 'odd,even',
        'class0'     => 'odd,even',
        'style0'     => 'white-space: nowrap;',
        'style0'     => 'text-align: center;',
        'classF'     => 'caution',
        'thenabled'  => 'true'
    );

    #表
    print $table->table;

    #ヘッダ
    print $table->tr;
    print $table->td('削除');
    print $table->td('表示順');
    print $table->td('ID');
    print $table->td('カテゴリー名');
    print $table->td('非公開');
    print $table->td('カラー');
    print $table->td('文字色');
    print $table->td('重複禁止[ゾーン](前後[分])');
    print $table->td('設定可能時間');
    print $table->td('設定可能曜日');
    print $table->td('担当者メールアドレス');
    print $table->_tr;

    #設定項目
    foreach ( sort { $a <=> $b } keys %{ $setting::event_category{'ORDER'} } )
    {
        my $id = $setting::event_category{'ORDER'}{$_};
        print $table->tr;
        print $table->td( '<input type="checkbox" name="delete" value="'
                . $id
                . '" onclick="cmonos.form.field(this).checkboxConfirm(\''
                . $setting::event_category{$id}{'name'}
                . ' 設定データは削除されますが、各イベントのカテゴリーIDは保持されます。本当に削除しますか？\')" />'
        );
        print $table->td( '<input type="number" name="order:'
                . $id
                . '" value="'
                . $_
                . '" size="2" maxlength="2" />' );
        print $table->td($id);
        print $table->td( '<input type="text" name="data:'
                . $id
                . '_name" value="'
                . $this->escape( $setting::event_category{$id}{'name'} )
                . '" size="20" />' );
        print $table->td(
                  '<input type="checkbox" name="data:'
                . $id
                . '_closed" value="1"'
                . (
                ( $setting::event_category{$id}{'closed'} )
                ? ' checked="checked"'
                : ''
                )
                . ' />'
        );
        print $table->td(
                  '<input type="text" name="data:'
                . $id
                . '_color" id="data:'
                . $id
                . '_color" value="'
                . $this->escape( $setting::event_category{$id}{'color'} )
                . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:'
                . $id
                . '_color\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
                . (
                ( $setting::event_category{$id}{'color'} )
                ? ' style="background-color: '
                    . $setting::event_category{$id}{'color'} . '"'
                : ''
                )
                . ' />'
                . ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:'
                . $id
                . '_color\')" ondblclick="cmonos.tool.colorPalette.open(\'data:'
                . $id
                . '_color\',\'window\')"><img src="'
                . $env::image_dir
                . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>'
                . ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:'
                . $id
                . '_color\')" ondblclick="cmonos.tool.colorSlider.open(\'data:'
                . $id
                . '_color\',\'window\')"><img src="'
                . $env::image_dir
                . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>'
        );
        print $table->td(
                  '<input type="text" name="data:'
                . $id
                . '_fontcolor" id="data:'
                . $id
                . '_fontcolor" value="'
                . $this->escape( $setting::event_category{$id}{'fontcolor'} )
                . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:'
                . $id
                . '_fontcolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
                . (
                ( $setting::event_category{$id}{'fontcolor'} )
                ? ' style="background-color: '
                    . $setting::event_category{$id}{'fontcolor'} . '"'
                : ''
                )
                . ' />'
                . ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:'
                . $id
                . '_fontcolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:'
                . $id
                . '_fontcolor\',\'window\')"><img src="'
                . $env::image_dir
                . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>'
                . ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:'
                . $id
                . '_fontcolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:'
                . $id
                . '_fontcolor\',\'window\')"><img src="'
                . $env::image_dir
                . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>'
        );
        print $table->td(
                  '<input type="checkbox" name="data:'
                . $id
                . '_overlapDisabled" value="1"'
                . (
                ( $setting::event_category{$id}{'overlapDisabled'} )
                ? ' checked="checked"'
                : ''
                )
                . ' /> '
                . $this->category_zone_menu( 'data:' . $id . '_zone',
                $setting::event_category{$id}{'zone'} )
                . ' <input type="text" name="data:'
                . $id
                . '_eventMargin" value="'
                . $this->escape(
                $setting::event_category{$id}{'eventMargin'} )
                . '" size="2" />分'
        );
        print $table->td(
                  '<input type="text" name="data:'
                . $id
                . '_availableTime" value="'
                . $this->escape(
                $setting::event_category{$id}{'availableTime'}
                )
                . '" size="12" title="hh:mm-hh:mm,hh:mm-hh:mm" />'
        );
        print $table->td(
            $this->available_wday_list(
                'data:' . $id . '_availableWday',
                $setting::event_category{$id}{'availableWday'}
            )
        );
        print $table->td( '<input type="email" name="data:'
                . $id
                . '_admin" value="'
                . $this->escape( $setting::event_category{$id}{'admin'} )
                . '" size="20" title="代表メールフィールドの値になります。" />'
        );
        print $table->_tr;
    }

    #新規作成
    print $table->tr;
    print $table->td('');
    print $table->td('');
    print $table->td('新規');
    print $table->td( '<input type="text" name="new:name" value="'
            . $this->escape( $main::new{'name'} )
            . '" size="20" />' );
    print $table->td( '<input type="checkbox" name="new:closed" value="1"'
            . ( ( $main::new{'closed'} ) ? ' checked="checked"' : '' )
            . ' />' );
    print $table->td(
              '<input type="text" name="new:color" id="new:color" value="'
            . $this->escape( $main::new{'color'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'new:color\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
            . (
            ( $main::new{'color'} )
            ? ' style="background-color: ' . $main::new{'color'} . '"'
            : ''
            )
            . ' />'
            . ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'new:color\')" ondblclick="cmonos.tool.colorPalette.open(\'new:color\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>'
            . ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'new:color\')" ondblclick="cmonos.tool.colorSlider.open(\'new:color\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>'
    );
    print $table->td(
        '<input type="text" name="new:fontcolor" id="new:fontcolor" value="'
            . $this->escape( $main::new{'fontcolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'new:fontcolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
            . (
            ( $main::new{'fontcolor'} )
            ? ' style="background-color: ' . $main::new{'fontcolor'} . '"'
            : ''
            )
            . ' />'
            . ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'new:fontcolor\')" ondblclick="cmonos.tool.colorPalette.open(\'new:fontcolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>'
            . ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'new:fontcolor\')" ondblclick="cmonos.tool.colorSlider.open(\'new:fontcolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>'
    );
    print $table->td(
        '<input type="checkbox" name="new:overlapDisabled" value="1"'
            . (
            ( $main::new{'overlapDisabled'} ) ? ' checked="checked"' : '' )
            . ' /> '
            . $this->category_zone_menu( 'new:zone', $main::new{'zone'} )
            . ' <input type="text" name="new:eventMargin" value="'
            . $this->escape( $main::new{'eventMargin'} )
            . '" size="2" />分' );
    print $table->td( '<input type="text" name="new:availableTime" value="'
            . $this->escape( $main::new{'availableTime'} )
            . '" size="12" title="hh:mm-hh:mm,hh:mm-hh:mm" />' );
    print $table->td(
        $this->available_wday_list(
            'new:availableWday', $main::new{'availableWday'}
        )
    );
    print $table->td( '<input type="email" name="new:admin" value="'
            . $this->escape( $main::new{'admin'} )
            . '" size="20" title="代表メールフィールドの値となります。" />'
    );
    print $table->_tr;

    print $table->_table;
    undef %$table;
    undef $table;
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

sub available_time_fix ($$) {
    my ( $this, $time ) = @_;
    if ( defined $time && $time ne '' ) {
        $time = $this->z_to_h($time);
        my %span = ();
        while ( $time =~ /([0-9]+):([0-9]+)-([0-9]+):([0-9]+)/sg ) {
            my ( $fh, $fm, $th, $tm ) = ( $1 + 0, $2 + 0, $3 + 0, $4 + 0 );
            $fh = 23 if $fh > 23;
            $fm = 59 if $fm > 59;
            if ( $th > 23 ) {
                $th = 24;
                $tm = 0;
            }
            elsif ( $tm > 59 ) {
                $tm = 59;
            }
            my $f = $fh * 100 + $fm;
            my $t = $th * 100 + $tm;
            if ( $f == $t ) {
                next;
            }
            elsif ( $f > $t ) {
                $span{'0'} = $t if !$span{'0'} || $span{'0'} < $t;
                $span{$f} = 2400 if !$span{$f} || $span{$f} < 2400;
                next;
            }
            $span{$f} = $t if !$span{$f} || $span{$f} < $t;
        }
        my ( $prevf, $prevt ) = ( 0, 0 );
        my $fixed_time = '';
        foreach my $f ( sort { $a <=> $b } keys %span ) {
            if ( $f < $prevt && $span{$f} < $prevt ) {
                delete $span{$f};
                next;
            }
            elsif ( $f < $prevt ) {
                $span{$prevf} = $prevt = $span{$f};
                delete $span{$f};
                next;
            }
            $prevf = $f;
            $prevt = $span{$f};
        }
        foreach my $f ( sort { $a <=> $b } keys %span ) {
            $fixed_time .= ',' if $fixed_time;
            $fixed_time .= sprintf( "%d:%.2d-%d:%.2d",
                ( ( !$f )        ? 0 : int( $f / 100 ) ),
                ( ( !$f )        ? 0 : ( $f % 100 ) ),
                ( ( !$span{$f} ) ? 0 : int( $span{$f} / 100 ) ),
                ( ( !$span{$f} ) ? 0 : ( $span{$f} % 100 ) ) );
        }
        return $fixed_time;
    }
    else {
        return '';
    }
}

sub available_wday_list ($$$) {
    my ( $this, $name, $value ) = @_;
    $this->scalar_to_array( \$value );
    my $list = '<ol class="checkbox">';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="0"'
        . ( ( grep( $_ eq '0', @$value ) ) ? ' checked="checked"' : '' )
        . ' />日</label></li>';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="1"'
        . ( ( grep( $_ eq '1', @$value ) ) ? ' checked="checked"' : '' )
        . ' />月</label></li>';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="2"'
        . ( ( grep( $_ eq '2', @$value ) ) ? ' checked="checked"' : '' )
        . ' />火</label></li>';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="3"'
        . ( ( grep( $_ eq '3', @$value ) ) ? ' checked="checked"' : '' )
        . ' />水</label></li>';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="4"'
        . ( ( grep( $_ eq '4', @$value ) ) ? ' checked="checked"' : '' )
        . ' />木</label></li>';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="5"'
        . ( ( grep( $_ eq '5', @$value ) ) ? ' checked="checked"' : '' )
        . ' />金</label></li>';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="6"'
        . ( ( grep( $_ eq '6', @$value ) ) ? ' checked="checked"' : '' )
        . ' />土</label></li>';
    $list
        .= '<li><label><input type="checkbox" name="'
        . $name
        . '" value="7"'
        . ( ( grep( $_ eq '7', @$value ) ) ? ' checked="checked"' : '' )
        . ' />休日は例外</label></li>';
    $list .= '</ol>';
    return $list;
}

sub category_zone_menu ($$$) {
    my ( $this, $name, $value ) = @_;
    my $menu = '<select name="' . $name . '">';
    $menu .= '<option value=""></option>';
    foreach my $z ( 'A' .. 'Z' ) {
        $menu
            .= '<option value="'
            . $z . '"'
            . ( ( $value && $z eq $value ) ? ' selected="selected"' : '' )
            . '>'
            . $z
            . '</option>';
    }
    $menu .= '</select>';
    return $menu;
}

sub import_order ($$$) {
    no warnings qw(untie);
    my ( $this, $db, $order ) = @_;
    my (%db);
    eval 'use ' . $env::dbm_file . ';';
    tie( %db, $env::dbm_file, $db, O_RDONLY, $env::dbm_mode );
    if ( ref tied %db ) {
        my ( $key, $value, %order );
        while ( ( $key, $value ) = each %db ) {
            utf8::decode($key)   if !utf8::is_utf8($key);
            if ( $key =~ /^ORDER_(.*)$/ ) {
                $order{$value} = $1;
                $$order{$value} = $1 unless $$order{$value} ne '';
            }
        }
        untie %db;
        return %order;
    }
    else {
        untie %db;
        my ($dbname);
        $dbname = '\n' . $1 if $db ne $env::register_db && $db =~ m!([^/]+)$!;
        push(
            @{ $this->{'dialog'} },
            'データベース読み込みに失敗しました。'
                . $dbname
        );
        return ();
    }
}

sub rand_color ($) {
    my $this = shift;
    my @h = ( 0 .. 9, 'a' .. 'f' );
    return
          '#'
        . $h[ int( rand(16) ) ]
        . $h[ int( rand(16) ) ]
        . $h[ int( rand(16) ) ]
        . $h[ int( rand(16) ) ]
        . $h[ int( rand(16) ) ]
        . $h[ int( rand(16) ) ];
}

sub get_border_color ($$) {
    my ( $this, $color ) = @_;
    if ( $color =~ /^#([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})/ ) {
        return
              '#'
            . $this->_border_rgb($1)
            . $this->_border_rgb($2)
            . $this->_border_rgb($3);
    }
    else {
        return '#0044BB';
    }
}

sub _border_rgb ($$) {
    my ( $this, $color ) = @_;
    $color = unpack( "C", pack( "H2", $color ) ) - 51;
    $color = ( $color > 255 ) ? 255 : ( $color < 0 ) ? 0 : $color;
    return unpack( "H2", pack( "C", $color ) );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) )
    {
        #クラス登録
        Common::class_register( 'c101', 'EventCategoryEditor' );

        #ツール登録
        $setting::jump_menu{'c101'} = 'イベントカテゴリー設定';
    }
}

1;
