package GeneralSetting;
require CMONOS::CGI_Lib::Tools::General;
@ISA = qw(General);
use strict;
use utf8;
no warnings qw(once);
######################################################################
# 一般設定モジュール
# GeneralSetting.pm ver1.0.46
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-12-09
######################################################################
# 設定チェック項目取得
sub get_sub_setting_items ($) {
    my $this = shift;
    $check::num{'rewindable'} = '更新の保存数';

    #チェックボックス
    push( @check::checkbox,
        'rebuildenabled',          'skip',
        'feedArticleRSS',          'feedSectionRSS',
        'sectionRSStitleRequired', 'addressProtector',
        'siteRSSEnabled',          'RSSEnabled',
        'rssChecked',              'pendingChecked',
        'keepMtimeEnabled',        'keepMtimeChecked',
        'autoPublishingEnabled' );
    push( @check::checkbox, 'masterRSSEnabled' ) if $default::subdivided;
    push( @check::checkbox, 'externalIndexDisabled' )
        if $default::external_index && -f $default::external_index;
    push( @check::checkbox, 'externalNaviDisabled' )
        if $default::external_navi && -f $default::external_navi;
    push( @check::checkbox, 'externalNavi2Disabled' )
        if $default::external_navi2 && -f $default::external_navi2;

    #タグを修正する
    push( @check::tagfix, 'description' );

    if ( defined $default::date && $default::date ) {

        #数字チェック
        $check::num{'yearStart'} = '「年」選択メニュー起点'
            if !defined $check::num{'yearStart'};
        $check::num{'yearEnd'} = '「年」選択メニュー終点'
            if !defined $check::num{'yearEnd'};
    }

    #アイコン画像
    $check::imagefile{'og'} = 'og:image';
    $check::ogimage{'og'}   = 1;
    if ( !defined $default::subdivided || !$default::subdivided ) {
        $check::imagefile{'icon'} = 'アイコン'
            if !defined $check::imagefile{'icon'};    #画像の名前

    }
    elsif ( defined $master::general{'subicon'}
        && $master::general{'subicon'} )
    {
        $check::imagefile{'icon'} = 'アイコン'
            if !defined $check::imagefile{'icon'};    #画像の名前
        $check::maxsize{'icon'}
            = ( $master::general{'iconMaxSize'} )
            ? $master::general{'iconMaxSize'}
            : 30000
            if !defined $check::maxsize{'icon'};  #ファイルサイズ制限
        $check::widthlimit{'icon'} = $master::general{'iconWidthLimit'}
            if !defined $check::widthlimit{'icon'};  #ファイル横幅制限
        $check::heightlimit{'icon'} = $master::general{'iconHeightLimit'}
            if !defined $check::heightlimit{'icon'}; #ファイル高さ制限
    }
    return;
}

######################################################################
# 設定環境2
sub is_database_cgi ($) {
    my $this = shift;
    return 1;
}

sub sub_set_preferences ($) {
    my $this = shift;
    $cgi::enctype_multipart = 1;

    # カテゴリーがない場合
    if ( !defined $default::subdivided || !$default::subdivided ) {
        $default::home = '.'
            if !defined $default::home || $default::home eq '';
        $preference::filedir{'og'} = $preference::filedir{'icon'}
            = $default::home;    #画像保存ディレクトリ

        # カテゴリーがある場合
    }
    elsif ( $default::subdivided && defined $main::c && $main::c ne '' ) {
        $default::home = '.'
            if !defined $default::home || $default::home eq '';
        $master::home = $default::home;
        $default::home .= '/' . $main::c;
        if ( defined $master::general{'subicon'}
            && $master::general{'subicon'} )
        {
            $preference::filedir{'icon'}
                = $default::home;    #画像保存ディレクトリ
        }
        $preference::filedir{'og'}
            = $default::home;        #画像保存ディレクトリ
    }
}

######################################################################
# サブ設定パネル
sub print_sub_preference_panel ($$) {
    my ( $this, $master ) = @_;

    # カテゴリーがある場合
    if ( !$default::subdivided || $master::general{'subicon'} ) {
        print
            '<fieldset><legend><label for="data:icon">アイコン</label></legend>';
        $this->print_image_upload_form( 'icon', 'icon', \%main::data );
        print '</fieldset>';
    }

    print '<fieldset><legend>og:image</legend>';
    $this->print_image_upload_form( 'og', 'og', \%main::data );
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:description">説明</label></legend>';
    print
        '<div class="hints">インデックスに表示するページの説明を設定します。ページが更新されるまで設定変更は表示に反映されません。</div>';
    print
        '<textarea name="data:description" id="data:description" cols="70" rows="6" wrap="soft" title="インデックスに表示するページの説明を設定します。">'
        . ( ( defined $main::data{'description'} )
        ? $main::data{'description'}
        : '' )
        . '</textarea>';
    print
        '<div class="extra-field"><label><input type="checkbox" name="data:addressProtector" value="1"'
        . (
        (   defined $main::data{'addressProtector'}
                && $main::data{'addressProtector'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> メールアドレスを JavaScript + CSS + HTML エンティティで保護する</label></div>';
    print
        '<div class="extra-field"><label><input type="checkbox" name="rebuild" value="1" /> 関連ページをすべて書き換える(処理が完了するまで時間がかかる場合があります)</label></div>';
    print '</fieldset>';

    print '<fieldset><legend>ページ情報入力</legend>';
    print
        '<label><input type="checkbox" name="data:skip" id="data:skip" value="1"'
        . ( ( defined $main::data{'skip'} && $main::data{'skip'} )
        ? ' checked="checked"'
        : '' )
        . ' /> ページ情報入力をスキップする</label>';
    print '</fieldset>';

    print '<fieldset><legend>全記事再構築</legend>';
    print
        '<label><input type="checkbox" name="data:rebuildenabled" id="data:rebuildenabled" value="1"'
        . (
        (   defined $main::data{'rebuildenabled'}
                && $main::data{'rebuildenabled'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> すべての記事を最新のテンプレートを使って書き換え可能にする</label>';
    print
        ' (<label title="再構築に失敗する場合は一度に書き換える記事の数を減らします。"><input type="number" size="3" name="data:rebuildinterval" id="data:rebuildinterval" value="'
        . $this->escape( $main::data{'rebuildinterval'} )
        . '" /> 件ずつ書き換える</label>)';
    print '</fieldset>';

    print '<fieldset><legend>日付</legend>';
    print
        '<table class="field-table"><tbody><tr><td class="field-name"><label for="data:datetype">標準日付書式</label></td>';
    print '<td class="field">';
    $this->print_date_type_menu( 'data:', $main::data{'datetype'},
        '日付の標準書式を設定します。' );
    print '</td></tr>';

    if ( defined $default::date && $default::date ) {
        print
            '<tr><td class="field-name"><span class="datatitle">「年」選択メニュー</span></td>';
        print '<td class="field">';
        print
            '<label><input type="text" name="data:yearStart" id="data:yearStart" size="4" maxlength="4" value="'
            . ( ( defined $main::data{'yearStart'} )
            ? $main::data{'yearStart'}
            : '' )
            . '" title="「年」選択メニューの起点を設定します。" />年〜</label>';
        print
            '<label><input type="text" name="data:yearEnd" size="4" maxlength="4" value="'
            . (
            ( defined $main::data{'yearEnd'} ) ? $main::data{'yearEnd'} : '' )
            . '" title="「年」選択メニューの終点を設定します。" />年</label>';
        print '</td></tr>';

        print
            '<tr><td class="field-name"><label for="data:historyType">履歴</label></td>';
        print '<td class="field">';
        print $this->mk_select_menu(
            'data:historyType',
            '履歴ページ１ページあたりの記事範囲を設定します。',
            '', $main::data{'historyType'},
            'm' => '月ごとにまとめる',
            'y' => '年ごとにまとめる',
            'd' => '毎日まとめる'
        );
        print
            '<div class="note">新しい設定は現在の設定の翌月/年/日から反映されます</div>';
        print '</td></tr>';
    }
    print '</tbody></table></fieldset>';

    print
        '<fieldset><legend>記事編集画面で他のページに移動したとき最初に表示する chapter</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="data:getNewChapter" value="1"'
        . ( ( $main::data{'getNewChapter'} ) ? ' checked="checked"' : '' )
        . ' /> 新規 chapter を最初に表示する</label></li>';
    print '<li><label><input type="radio" name="data:getNewChapter" value="0"'
        . ( ( !$main::data{'getNewChapter'} ) ? ' checked="checked"' : '' )
        . ' /> 最後の chapter を最初に表示する</label></li>';
    print '</ol>';
    print '</fieldset>';
    print
        '<fieldset><legend>記事編集画面で他のページに移動したとき最初に表示する section</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="data:getNewSection" value="1"'
        . ( ( $main::data{'getNewSection'} ) ? ' checked="checked"' : '' )
        . ' /> 新規 section を最初に表示する</label></li>';
    print '<li><label><input type="radio" name="data:getNewSection" value="0"'
        . ( ( !$main::data{'getNewSection'} ) ? ' checked="checked"' : '' )
        . ' /> 最後の section を最初に表示する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset><legend>更新の保存</legend>';
    print
        '<label>過去の更新を<input type="number" size="3" maxlength="3" name="data:rewindable" value="'
        . $this->escape( $main::data{'rewindable'} )
        . '" /> 件まで自動保存する。</label>';
    print '</fieldset>';

    #外部インデックス更新停止
    if (   ( $default::external_navi && -f $default::external_navi )
        || ( $default::external_navi2 && -f $default::external_navi2 )
        || ( $default::external_index && -f $default::external_index ) )
    {
        print
            '<fieldset><legend>外部インデックスの更新停止</legend>';
        print '<ol class="checkbox">';
        print
            '<li><label><input type="checkbox" name="data:externalIndexDisabled" value="1"'
            . ( ( $main::data{'externalIndexDisabled'} )
            ? ' checked="checked"'
            : '' )
            . ' /> '
            . $this->path_change( $default::external_index,
            $ENV{'SCRIPT_NAME'}, 'abs' )
            . ' を更新<strong>しない</strong></label></li>'
            if $default::external_index && -f $default::external_index;
        print
            '<li><label><input type="checkbox" name="data:externalNaviDisabled" value="1"'
            . ( ( $main::data{'externalNaviDisabled'} )
            ? ' checked="checked"'
            : '' )
            . ' /> '
            . $this->path_change( $default::external_navi,
            $ENV{'SCRIPT_NAME'}, 'abs' )
            . ' を更新<strong>しない</strong></label></li>'
            if $default::external_navi && -f $default::external_navi;
        print
            '<li><label><input type="checkbox" name="data:externalNavi2Disabled" value="1"'
            . ( ( $main::data{'externalNavi2Disabled'} )
            ? ' checked="checked"'
            : '' )
            . ' /> '
            . $this->path_change( $default::external_navi2,
            $ENV{'SCRIPT_NAME'}, 'abs' )
            . ' を更新<strong>しない</strong></label></li>'
            if $default::external_navi2 && -f $default::external_navi2;
        print '</ol>';
        print '</fieldset>';
    }

    #RSS更新設定
    print '<fieldset><legend>RSS更新設定</legend>';
    print '<fieldset><legend>更新のタイミング</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:feedArticleRSS" value="1"'
        . ( ( $main::data{'feedArticleRSS'} ) ? ' checked="checked"' : '' )
        . ' /> 「ページ情報」書き込み時にRSSを更新する(リンク先は記事の最初のページ)</label></li>';
    print
        '<li><label><input type="checkbox" name="data:feedSectionRSS" value="1"'
        . ( ( $main::data{'feedSectionRSS'} ) ? ' checked="checked"' : '' )
        . ' /> section 書き込み時ににRSSを更新する(リンク先は更新された section)</label>';
    print
        ' (<label><input type="checkbox" name="data:sectionRSStitleRequired" value="1"'
        . ( ( $main::data{'sectionRSStitleRequired'} )
        ? ' checked="checked"'
        : '' )
        . ' /> タイトルフィールドが空の時は更新しない</label>)</li>';
    print '</ol>';
    print '</fieldset>';
    print '<fieldset><legend>更新するRSS</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:siteRSSEnabled" value="1"'
        . ( ( $main::data{'siteRSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( -f '../../index.xml' )
        ? '<a href="../../index.xml" target="_blank">サイト全体の新着情報(RSS)</a>'
        : 'サイト全体の新着情報(RSS)' )
        . 'を更新する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:masterRSSEnabled" value="1"'
        . ( ( $main::data{'masterRSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( $master::home && -f $master::home . '/index.xml' )
        ? '<a href="'
            . $master::home
            . '/index.xml'
            . '" target="_blank">ページ全体の新着情報(RSS)</a>'
        : 'ページ全体の新着情報(RSS)' )
        . 'を更新する</label></li>'
        if $default::subdivided;
    print '<li><label><input type="checkbox" name="data:RSSEnabled" value="1"'
        . ( ( $main::data{'RSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( $default::home && -f $default::home . '/index.xml' )
        ? '<a href="'
            . $default::home
            . '/index.xml'
            . '" target="_blank">このページの新着情報(RSS)</a>'
        : 'このページの新着情報(RSS)' )
        . 'を更新する</label></li>';
    print '</ol>';
    print '</fieldset>';
    print '<fieldset><legend>デフォルトの状態</legend>';
    print '<label><input type="checkbox" name="data:rssChecked" value="1"'
        . ( ( $main::data{'rssChecked'} ) ? ' checked="checked"' : '' )
        . ' /> デフォルトで RSS を更新する(RSS 更新チェックボックスがチェック済み)</label>';
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>非公開</legend>';
    print
        '<fieldset><legend><label><input type="checkbox" name="data:autoPublishingEnabled" value="1"'
        . (
        ( $main::data{'autoPublishingEnabled'} ) ? ' checked="checked"' : '' )
        . ' onclick="cmonos.element.item(\'trrigers\').disappear(!this.checked)" />公開予約</label></legend>';
    print
        '<p class="warning">公開予約機能は、RSSリーダーによる定期的な更新チェックを自動公開のトリガーとして利用します。サイトに埋め込んだ 透明GIF画像 または 改行のみのJavaScript として CGI を呼び出すことで、訪問者のアクセスを自動公開のトリガーとすることもできます。</p>';
    print '<div id="trrigers">';
    print '<fieldset><legend>公開日チェック時刻</legend>';
    print
        '<label><select name="data:autoPublishingStartTime" id="data:autoPublishingStartTime">';

    foreach my $h ( 0 .. 23 ) {
        print '<option value="' . $h . '"';
        print ' selected="selected"'
            if $main::data{'autoPublishingStartTime'}
            && $h == $main::data{'autoPublishingStartTime'};
        print '>' . $h . '</option>';
    }
    print '</select> 時以降</label>';
    print '';
    print '</fieldset>';
    my $trriger_url
        = $this->my_url
        . '?m=Publish'
        . ( ($default::subdivided) ? '&c=' . $main::c : '' );
    print '<fieldset><legend>RSSリーダーに登録</legend>';
    print '<textarea cols="90" rows="1">' . $trriger_url . '</textarea>';
    print '</fieldset>';
    print
        '<fieldset><legend>透明GIFとしてページ(テンプレート)に埋め込む</legend>';
    print '<textarea cols="90" rows="1">&lt;img src=&quot;'
        . $this->escape( $trriger_url . '&res=img' )
        . '&quot; width=&quot;1&quot; height=&quot;1&quot; alt=&quot;&quot; /&gt;</textarea>';
    print '</fieldset>';
    print
        '<fieldset><legend>JavaScriptとしてページ(テンプレート)埋め込む</legend>';
    print
        '<textarea cols="90" rows="1">&lt;script type="text/javascript" src=&quot;'
        . $this->escape( $trriger_url . '&res=js' )
        . '&quot;&gt;&lt;/script&gt;</textarea>';
    print '</fieldset>';
    print '</div>';
    print $this->javascript(
        'cmonos.element.item(\'trrigers\').disappear(true)')
        if !$main::data{'autoPublishingEnabled'};
    print '</fieldset>';
    print '<label><input type="checkbox" name="data:pendingChecked" value="1"'
        . ( ( $main::data{'pendingChecked'} ) ? ' checked="checked"' : '' )
        . ' /> デフォルトで「非公開」チェックボックスをチェック済みにする</label>';
    print '</fieldset>';

    print '<fieldset><legend>前回更新時の保持</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:keepMtimeEnabled" value="1"'
        . ( ( $main::data{'keepMtimeEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> 「前回更新時の保持」を許可する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:keepMtimeChecked" value="1"'
        . ( ( $main::data{'keepMtimeChecked'} ) ? ' checked="checked"' : '' )
        . ' /> デフォルトで「前回更新時の保持」チェックボックスをチェック済みにする</label></li>';
    print '</ol>';
    print '</fieldset>';
}

######################################################################
# サブ設定チェック
sub _is_valid_setting ($) {
    my $this = shift;
    $main::data{'autoPublishingStartTime'} = 0
        if !$main::data{'autoPublishingStartTime'}
        || $this->is_nan( $main::data{'autoPublishingStartTime'} );
    my ( @rss, @exindex ) = ( (), () );
    push( @rss, '../../index.xml' )
        if $setting::general{'siteRSSEnabled'}
        && !$main::data{'siteRSSEnabled'};
    push( @rss, $master::home . '/index.xml' )
        if $default::subdivided
        && $setting::general{'masterRSSEnabled'}
        && !$main::data{'masterRSSEnabled'};
    unlink $default::home . '/index.xml'
        if $setting::general{'RSSEnabled'} && !$main::data{'RSSEnabled'};

    push( @exindex, $default::external_index )
        if !$setting::general{'externalIndexDisabled'}
        && $main::data{'externalIndexDisabled'}
        && -f $default::external_index;
    push( @exindex, $default::external_navi )
        if !$setting::general{'externalNaviDisabled'}
        && $main::data{'externalNaviDisabled'}
        && -f $default::external_navi;
    push( @exindex, $default::external_navi2 )
        if !$setting::general{'externalNavi2Disabled'}
        && $main::data{'externalNavi2Disabled'}
        && -f $default::external_navi2;

    if ( @rss || @exindex ) {
        require CMONOS::CGI_Lib::IndexSweeper;
        IndexSweeper->new(
            'index_files' => \@exindex,
            'rss_files'   => \@rss,
            )->sweep(
            [ $this->full_path($default::home) ],
            {   $default::id => (
                    (          $default::subdivided
                            && defined $main::c
                            && $main::c ne ''
                    ) ? [$main::c] : []
                )
            }
            );
    }
    if ($main::rebuild) {
        require CMONOS::CGI_Lib::ProgressBar;
        $cgi::ProgressBar = ProgressBar->new;
        $this->set_javascript( $cgi::ProgressBar->javascript );
    }
    return;
}

######################################################################
# アイコンと説明をリフレッシュ
sub print_progress_bar ($) {
    my $this = shift;
    $cgi::ProgressBar->print_progress_bar('ファイル') if $main::rebuild;
    return;
}

sub site_update ($) {
    my $this = shift;
    $this->refresh_icon_and_description if $main::rebuild;
    return;
}

sub refresh_icon_and_description ($$) {
    my $this = shift;
    return 1 if !defined $main::data{'description'};
    my @categories = ();
    my @indexes    = ();
    push( @indexes, $default::external_index )
        if $default::external_index && -f $default::external_index;
    push( @indexes, $default::external_navi )
        if $default::external_navi && -f $default::external_navi;
    push( @indexes, $default::external_navi2 )
        if $default::external_navi2 && -f $default::external_navi2;
    my @articles       = ();
    my $icon_attribute = '';
    my $description    = '';
    my $index_id
        = ( defined $default::id && $default::id ne '' )
        ? $default::id
        : 'homepage';
    my $c = ($default::subdivided) ? $main::c : 'main';
    my $metaindex_target = '';

    if ($default::id) {
        my $prefix = $default::id;
        $prefix =~ s/^_+//;
        $prefix =~ s/_/-/g;
        $metaindex_target
            = $prefix
            . (
            ( defined $main::c && $main::c ne '' ) ? '-' . $main::c : '' );
    }
    if ($default::subdivided) {
        my $master_home = $default::home;
        $master_home =~ s!/[0-9a-zA-Z_]+$!!;
        if ( opendir( DIR, $master_home ) ) {
            while ( my $file = readdir(DIR) ) {
                push( @indexes, $master_home . '/' . $file )
                    if $file !~ /^\./
                    && -f $master_home . '/' . $file
                    && $file =~ /\.(s?html?|ssi)$/i;
            }
        }
        closedir(DIR);
    }
    if ( opendir( DIR, $default::home ) ) {
        my @article_folders = ();
        my $history_dir     = $default::home . '/history';
        while ( my $file = readdir(DIR) ) {
            next if $file =~ /^\./ && $file ne 'calendar';
            if ( -d $default::home . '/' . $file ) {
                push( @article_folders, $default::home . '/' . $file );
            }
            elsif ( -f $default::home . '/' . $file
                && $file =~ /\.(s?html?|ssi)$/i )
            {
                push( @indexes, $default::home . '/' . $file );
            }
        }
        closedir(DIR);
        if (@article_folders) {
            foreach my $d (@article_folders) {
                if ( opendir( DIR, $d ) ) {
                    if ( $default::date && $d eq $history_dir ) {
                        push(
                            @indexes,
                            map( $d . '/' . $_,
                                grep( $_ =~ /^([0-9]+|index)\.s?html?$/i
                                        && -f $d . '/' . $_,
                                    readdir(DIR) ) )
                        );
                    }
                    else {
                        push(
                            @articles,
                            map( $d . '/' . $_,
                                grep( $_ =~ /^([0-9]+|index)\.s?html?$/i
                                        && -f $d . '/' . $_,
                                    readdir(DIR) ) )
                        );
                    }
                    closedir(DIR);
                }
            }
        }
    }
    if ( $main::data{'icon'} ) {
        $icon_attribute
            = ' width="'
            . $main::data{'iconwidth'}
            . '" height="'
            . $main::data{'iconheight'}
            . '" alt="'
            . $main::data{'cginame'}
            . '" border="0"';
    }
    elsif ( $setting::general{'icon'}
        && -f $default::home . '/' . $setting::general{'icon'} )
    {
        $icon_attribute
            = ' width="'
            . $setting::general{'iconwidth'}
            . '" height="'
            . $setting::general{'iconheight'}
            . '" alt="'
            . $setting::general{'cginame'}
            . '" border="0"';
    }
    $description
        = $this->autolink(
        $this->tag_fix( $this->autolist( $main::data{'description'} ), 2 ),
        0, $main::data{'addressProtector'} )
        if $main::data{'description'};
    my $total = @indexes + @articles;
    my $count = 0;
    if (@indexes) {
        foreach my $path (@indexes) {
            my ( $data, $icon, $done );
            $done = 0;
            $count++;
            $icon = '<img src="'
                . $this->base_change(
                $default::home . '/' . $main::data{'icon'},
                $path, ( ( $path =~ /\.ssi$/ ) ? 'abs' : '' ) )
                . '"'
                . $icon_attribute . ' />'
                if $icon_attribute;
            $data = $this->read_file( $path, 1, 1 );
            $data
                =~ s/(<!--index(?![0-9a-zA-Z_])[^>]*id="$index_id"[^>]*-->)(.*?)(<!--\/index-->)/"\x03" . $1 . $this->_index_replace($2,$c,$icon,$description,\$done) . $3 . "\x02"/sge
                if $c ne '';
            $data
                =~ s/(<!--metaindex(?![0-9a-zA-Z_])[^>]*-->)(.*?)(<!--\/metaindex-->)/"\x03" . $1 . $this->_metaindex_replace($2,$metaindex_target,$icon,$description,\$done) . $3 . "\x02"/sge
                if $metaindex_target;
            $data
                =~ s/(?:^|\x02)(.*?)(?:\x03|$)/$this->_replace_icon_and_description($1,$icon,$description,\$done)/sge
                if !$default::subdivided || $path =~ /^\Q$default::home\E/;
            $data =~ s/[\x02\x03]//sg;

            if ($done) {
                $this->write_file( $path, \$data, 1, $env::file_mode, 1 );
                $cgi::ProgressBar->update( $count, $total, 20 );
            }
        }
    }
    if (@articles) {
        foreach my $path (@articles) {
            my ( $data, $icon, $done );
            $done = 0;
            $count++;
            $icon = '<img src="'
                . $this->base_change(
                $default::home . '/' . $main::data{'icon'},
                $path, ( ( $path =~ /\.ssi$/ ) ? 'abs' : '' ) )
                . '"'
                . $icon_attribute . ' />'
                if $icon_attribute;
            $data = $this->read_file( $path, 1, 1 );
            $data = $this->_replace_icon_and_description( $data, $icon,
                $description, \$done );
            $data = $this->_replace_cginame( $data, $main::data{'cginame'},
                \$done );
            if ($done) {
                $this->write_file( $path, \$data, 1, $env::file_mode, 1 );
                $cgi::ProgressBar->update( $count, $total, 20 );
            }
        }
    }
    $cgi::ProgressBar->update( $total, $total );
    $cgi::ProgressBar->remove;
}

sub _index_replace ($$$$$$) {
    my ( $this, $data, $c, $icon, $description, $done_ref ) = @_;
    $data
        =~ s/(<!--subindex(?![0-9a-zA-Z_])[^>]*id="$c"[^>]*-->)(.*?)(<!--\/subindex-->)/$1 . $this->_subindex_replace($2,$icon,$description,$done_ref) . $3/sge;
    return $data;
}

sub _subindex_replace ($$$$$) {
    my ( $this, $data, $icon, $description, $done_ref ) = @_;
    my $article
        = ( $data
            =~ s/(<!--article(?![0-9a-zA-Z_])[^>]*-->.*<!--\/article-->)/<!--article--><!--\/article-->/sg
        )
        ? $this->_replace_icon_and_description( $1, $icon, $description,
        $done_ref, 'main' )
        : '';
    $data = $this->_replace_icon_and_description( $data, $icon, $description,
        $done_ref );
    $data =~ s/<!--article--><!--\/article-->/$article/sg;
    return $data;
}

sub _metaindex_replace ($$$$$$) {
    my ( $this, $data, $target, $icon, $description, $done_ref ) = @_;
    $data
        =~ s/(<!--article(?![0-9a-zA-Z_])[^>]*id="$target-[^"]+"[^>]*-->.*?<!--\/article-->)/$this->_replace_icon_and_description($1,$icon,$description,$done_ref,'main')/sge;
    return $data;
}

sub _replace_icon_and_description ($$$$$;$) {
    my ( $this, $data, $icon, $description, $done_ref, $prefix ) = @_;
    $prefix = '' if !defined $prefix;
    $$done_ref = 1
        if $data
        =~ s/<\!--(${prefix}icon(?![0-9a-zA-Z_]))([^>]*)-->(.*?)<\!--\/\1(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$icon,1)/gies;
    $$done_ref = 1
        if $data
        =~ s/<\!--(${prefix}description(?![0-9a-zA-Z_]))([^>]*)-->(.*?)<\!--\/\1(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$description,1)/gies;
    return $data;
}

sub _replace_cginame ($$$$) {
    my ( $this, $data, $cginame, $done_ref ) = @_;
    $$done_ref = 1
        if $data =~ s!<a href="../">[^<>]+</a>!<a href="../">$cginame</a>!gis;
    return $data;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    unless ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) )
    {
        #クラス登録
        Common::class_register( '0000', 'GeneralSetting' );

        #ツール登録
        $setting::jump_menu{'0000'} = '一般設定'
            if !defined $setting::jump_menu{'0000'};
    }
}

1;
