package FileDownload;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 添付ファイルダウンロードモジュール
# FileDownload.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-01-11
######################################################################
# 実行
sub run ($) {
    no warnings qw(numeric);
    my $this = shift;
    @main::delete = ($main::delete) if !@main::delete && $main::delete;
    my $file_dir = $default::data . '/files';

    #データ表示
    if ( defined $main::dir && $main::dir ne '' && $main::target ) {
        if (   $main::dir !~ /^\./
            && $main::target !~ /^\./
            && -f $file_dir . '/' . $main::dir . '/' . $main::target )
        {
            $this->print_stdout_download_file(
                $file_dir . '/' . $main::dir . '/' . $main::target,
                $main::target );
        }
        else {
            my $url = $cgi::path . '?m=Tools&amp;k=q201';
            $this->print_http_header;
            print '<html><head><meta http-equiv="Refresh" content="0; URL='
                . $this->escape($url) . '" />';
            print
                '<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" /></head>';
            print '<body onload="location.href=&quot;'
                . $this->escape($url)
                . '&quot;">';
            print '<a href="' . $this->escape($url) . '">エラー。</a>';
            print '</body></html>';
            exit;
        }
    }

    #データ削除
    if (@main::delete) {
        use File::Path;
        foreach my $folder (@main::delete) {
            rmtree $file_dir . '/' . $folder;
            push(
                @{ $this->{'dialog'} },
                'フォルダ「' . $folder . '」を削除しました。'
            );
        }
        undef @main::delete;
    }
    elsif ( $main::a && $main::a eq 'delete' ) {
        push(
            @{ $this->{'dialog'} },
            '削除するフォルダを指定してください。'
        );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>添付ファイルダウンロード/削除</h1>';

    my $has_attachment = 0;
    local (*DIR);
    if ( opendir( DIR, $file_dir ) ) {
        my @directories = sort { $a <=> $b || $a cmp $b }
            grep( $_ !~ /^\./ && -d $file_dir . '/' . $_, readdir(DIR) );
        closedir(DIR);
        if (@directories) {
            $has_attachment = 1;

            #フォーム開始
            print $this->form_start(
                'key'      => 'tools',
                'name'     => 'input',
                'id'       => 'input',
                'method'   => 'post',
                'required' => \@setting::form
            );
            print '<fieldset><legend>添付ファイル一覧</legend>';

            #テーブル設定
            require CMONOS::CGI_Lib::Table;
            my $table = Table->new;
            $table->set(
                'tableclass' => 'tooltable',
                'trclass'    => 'odd,even',
                'class0'     => 'odd,even',
                'classF'     => 'caution',
                'styleH'     => 'text-align: center',
                'style0' =>
                    'text-align: center,text-align: center,text-align: left,text-align: left,text-align: left',
                'thenabled' => 'true'
            );
            my $totalsize = 0;
            print $table->table;

            #ヘッダ
            print $table->tr;
            print $table->td('削除');
            print $table->td('No');
            print $table->td('名前');
            print $table->td('保存日時');
            print $table->td('添付ファイル');
            print $table->_tr;

            foreach my $dir (@directories) {
                my $folder = $file_dir . '/' . $dir;
                my $name   = $this->escape_except_html_entity(
                    $this->read_file( $folder . '/name', 1 ) );
                my $from = $this->escape_except_html_entity(
                    $this->read_file( $folder . '/from', 1 ) );
                if ($from) {
                    $this->autolink( \$from );
                    $name .= ( ($name) ? ' ' : '' ) . '&lt;' . $from . '&gt;';
                }
                print $table->tr;
                print $table->td(
                          '<input type="checkbox" name="delete" value="'
                        . $dir
                        . '" onclick="cmonos.form.field(this).checkboxConfirm(\'このフォルダの全てのファイルが削除されます。本当に削除しますか？\')" />'
                );
                print $table->td($dir);
                print $table->td($name);
                print $table->td(
                    $this->date(
                        ( $this->read_file( $folder . '/time', 1 ) || time ),
                        3,
                        'gmt/24'
                    )
                );
                print $table->td(
                    $this->attachment_file_list( $folder, $dir, \$totalsize )
                );
                print $table->_tr;
            }

            print $table->_table;
            print '<p>合計:' . $this->file_size($totalsize) . '</p>';
            print '</fieldset>';
            print '<fieldset class="delete">';
            print
                '<input type="submit" name="a=delete" value="削除する" onclick="return cmonos.tool.questionnaireDataExporter(true) && confirm(\'この操作は取り消せません。本当に削除しますか？\') &amp;&amp; cmonos.form.item(this.form).isNotSubmitted();" class="delete" />';
            print '</fieldset></form>';
        }
    }
    if ( !$has_attachment ) {
        print
            '<fieldset><legend>添付ファイル一覧</legend><p>保存された添付ファイルはありません。</p></fieldset>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

sub attachment_file_list ($$$$) {
    my ( $this, $file_dir, $dir, $totalsize_ref ) = @_;
    local (*DIR);
    my $list = '';
    if ( opendir( DIR, $file_dir ) ) {
        no warnings qw(numeric);
        my @files
            = sort { $a <=> $b || $a cmp $b }
            grep( $_ !~ /^\./
                && $_ =~ /\.[a-zA-Z0-9]+$/
                && -f $file_dir . '/' . $_,
            readdir(DIR) );
        closedir(DIR);
        if (@files) {
            $list .= '<ul>';
            foreach my $file (@files) {
                my $size = -s $file_dir . '/' . $file;
                $$totalsize_ref += $size;
                $list
                    .= '<li><a href="'
                    . $cgi::path . '/'
                    . $dir . '/'
                    . $file
                    . '?m=Tools&amp;k=q201&amp;dir='
                    . $dir
                    . '&amp;target='
                    . $file . '">'
                    . $file . '</a>('
                    . $this->file_size($size)
                    . ')</li>';
            }
            $list .= '</ul>';
        }
    }
    return $list;
}

sub file_size ($$) {
    my ( $this, $size ) = @_;
    if ( $size > 1000000000 ) {
        return ( int( $size / 10000000 ) / 100 ) . 'GB';
    }
    elsif ( $size > 1000000 ) {
        return ( int( $size / 10000 ) / 100 ) . 'MB';
    }
    elsif ( $size > 1000 ) {
        return ( int( $size / 10 ) / 100 ) . 'KB';
    }
    else {
        return $size . 'B';
    }
}

sub print_stdout_download_file ($$$) {
    my ( $this, $path, $file ) = @_;
    my $size = -s $path;
    my $type
        = (    $file
            && %Common::mime_type
            && $file =~ /\.([0-9a-zA-Z_]+)$/
            && $Common::mime_type{ lc $1 } )
        ? $Common::mime_type{ lc $1 }
        : 'application/octet-stream';
    binmode STDOUT;
    print "Content-Type: $type\n";
    print "Content-Length: $size\n";
    print "Content-Disposition: attachment\n";
    print "\n";
    local (*FILE);

    if ( open( FILE, $path ) ) {
        binmode FILE;
        flock( FILE, LOCK_SH );
        my $buf;
        while ( read( FILE, $buf, $env::buffer_size ) ) {
            print $buf;
        }
        close(FILE);
    }
    exit;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'q201', 'FileDownload' );

    #メニュー登録
    $setting::jump_menu{'q201'}
        = '添付ファイルダウンロード/削除';

}

1;
