package CSSUpdater;
require CMONOS::SetupAssistant::Tools::JavaScriptUpdater;
@ISA = qw(JavaScriptUpdater);
use strict;
use utf8;
######################################################################
# CSSアップデータモジュール
# CSSUpdater.pm ver1.0.3
# Copyright (C) 2008-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-11-29
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    $this->get_setup_env;
    my $updater_enabled
        = ( -d $env::template_dir . '/site/default/htdocs/lib/css' ) ? 1 : 0;
    if ( !@main::target && defined $main::target && $main::target ne '' ) {
        push( @main::target, $main::target );
        undef $main::target;
    }

    #アップデート
    $this->update if $updater_enabled && @main::target;

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>CSS アップデート</h1>';
    print '<p>"'
        . $this->get_path_for_log($env::template_dir)
        . '/site/default/htdocs/lib/css" 以下の CSSファイル のうち選択された CSSファイル を全てのサイトテンプレートおよび全てのサブディレクトリの css フォルダに<strong>上書き</strong>コピーします。</p>';
    if ( !$updater_enabled ) {
        print
            '<fieldset>アップデートに必要なテンプレートがみつかりません。</fieldset>';
    }
    else {
        print $this->form_start(
            'key'    => 'tools',
            'name'   => 'input',
            'method' => 'post',
            'onsubmit' =>
                'return confirm(\'新しい CSS で古い CSS を上書きします。この操作を実行後、古い CSS は復元できません。本当に変更しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
            'required' => \@setting::form
        );
        if ( $this->{'update_log'} ) {
            print
                '<fieldset><legend>CSS アップデートログ</legend><ol>';
            print $this->{'update_log'};
            print '</ol></fieldset>';
        }
        if (opendir( CSSDIR,
                $env::template_dir . '/site/default/htdocs/lib/css'
            )
            )
        {
            my @menu
                = grep(
                -f $env::template_dir . '/site/default/htdocs/lib/css/' . $_
                    && $_ =~ /^[0-9a-zA-Z_\-]+\.css$/
                    && $_ !~ /(^|\-)image\.css$/
                    && $_ ne 'site-navi.css',
                readdir(CSSDIR) );
            print
                '<fieldset><legend>アップデート対象 CSS ファイル</legend><ol class="checkbox">';
            foreach my $name ( sort @menu ) {
                print
                    '<li><label><input type="checkbox" name="target" value="'
                    . $name . '"'
                    . ( ( $name eq 'common.css' || $name eq 'tools.css' )
                    ? ' checked="checked"'
                    : '' )
                    . ' /> '
                    . $name
                    . '</label></li>';
            }
            print
                '<li><label><input type="checkbox" name="target" value="images" /> CSS 関連画像</label></li>';
            print '</ol></fieldset>';
        }
        print
            '<fieldset class="submit"><input type="submit" value="CSS をアップデートする" /></fieldset></form>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

sub update ($) {
    my $this = shift;
    local (*TEMP);
    foreach my $dir ( '', $this->sub_directory_scan($default::cgi_data_root) )
    {
        $this->css_update($dir);
        if ($dir
            && opendir( TEMP,
                $default::cgi_data_root . $dir . '/template/site'
            )
            )
        {
            my @template
                = grep( $_ ne 'default'
                    && $_ !~ /\./
                    && -d $default::cgi_data_root
                    . $dir
                    . '/template/site/'
                    . $_,
                readdir(TEMP) );
            closedir(TEMP);
            foreach my $temp_dir (@template) {
                $this->css_update( $temp_dir, 1,
                    $default::cgi_data_root . $dir . '/template' );
            }
        }
    }
    if ( opendir( TEMP, $env::template_dir . '/site' ) ) {
        my @template
            = grep( $_ ne 'default'
                && $_ !~ /\./
                && -d $env::template_dir . '/site/' . $_,
            readdir(TEMP) );
        closedir(TEMP);
        foreach my $dir (@template) {
            $this->css_update( $dir, 1 );
        }
    }
    return $this;
}

sub css_update ($$;$$) {
    my ( $this, $home_dir, $is_template, $template_dir ) = @_;
    $template_dir = $env::template_dir if !$template_dir;
    my $new_version = $env::template_dir . '/site/default/htdocs/lib/css';
    my $css_dir
        = ($is_template)
        ? $template_dir . '/site/' . $home_dir . '/htdocs/lib/css'
        : ( !$home_dir ) ? $env::css
        :                  '../..' . $home_dir . '/lib/css';
    return 0
        if !-d $css_dir || !-d $new_version || !-f $css_dir . '/common.css';
    my $css_path
        = ($is_template)
        ? $this->get_path_for_log($css_dir)
        : $this->path_change( $css_dir, $ENV{'SCRIPT_NAME'}, 'abs' );
    foreach my $f (@main::target) {
        if ($f =~ /^[0-9a-zA-Z_\-]+\.css$/
            && ( $main::updating
                || ( $f !~ /(^|\-)image\.css$/ && $f ne 'site-navi.css' ) )
            )
        {
            $this->file_copy( $new_version . '/' . $f, $css_dir . '/' . $f );
            $this->{'update_log'} .= '<li>' . $css_path . '/' . $f . '</li>';
        }
    }
    if ( grep( $_ eq 'images', @main::target ) ) {
        my $new_image_dir
            = $env::template_dir . '/site/default/htdocs/lib/image';
        my $css_image_dir
            = ($is_template)
            ? $template_dir . '/site/' . $home_dir . '/htdocs/lib/image'
            : ( !$home_dir && $env::css =~ /^(.*\/)[\/]+$/ ) ? $1 . 'image'
            :   '../..' . $home_dir . '/lib/image';
        $this->directory_copy( $new_image_dir, $css_image_dir );
        $this->{'update_log'} .= '<li>'
            . (
            ($is_template)
            ? $this->get_path_for_log($css_image_dir)
            : $this->path_change(
                $css_image_dir, $ENV{'SCRIPT_NAME'}, 'abs'
            )
            ) . '</li>';
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's304', 'CSSUpdater' );

    #メニュー登録
    $setting::jump_menu{'s304'} = 'CSS アップデート';

}

1;
