package MobileRedirect;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 携帯リダイレクトモジュール
# FTPAccess.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-03-06
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my $htaccess = $this->read_file( '../../.htaccess', 1 );
    my $redirect_ready
        = ( $htaccess =~ /# For Mobile - start -.*?# For Mobile - end -/s )
        ? 1
        : 0;
    if ($main::toggle) {
        if ($redirect_ready) {
            $htaccess
                =~ s/[\x09\x0A\x0C\x0D\x20]?# For Mobile - start -.*?# For Mobile - end -[\x09\x0A\x0C\x0D\x20]?//s;
            $redirect_ready = 0;
            push(
                @{ $this->{'dialog'} },
                '携帯リダイレクトを解除しました。'
            );
        }
        else {
            my $this_dir
                = $this->path_change( '../../', $ENV{'SCRIPT_NAME'}, 'abs' );
            $this_dir .= '/' if $this_dir !~ m!/$!;
            my $redirect_rule = <<REDIRECT;

# For Mobile - start -
<IfModule mod_rewrite.c>
RewriteEngine On
RewriteCond \%{HTTP_USER_AGENT} (DoCoMo|J-PHONE|Vodafone|SoftBank|KDDI|UP\\.Browser) [NC]
RewriteCond \%{HTTP_USER_AGENT} !MCONV
RewriteCond \%{REQUEST_URI} !/m\\.cgi
RewriteCond \%{REQUEST_URI} (/|\\.html|\\.htm|\\.shtml)\$
RewriteRule ^(.*)\$ ${this_dir}m.cgi/\$1 [L,R]
</IfModule>
# For Mobile - end -
REDIRECT
            $htaccess .= $redirect_rule
                unless $htaccess
                =~ s/[\x09\x0A\x0C\x0D\x20]?# For Mobile - start -.*?# For Mobile - end -[\x09\x0A\x0C\x0D\x20]?/$redirect_rule/s;
            $redirect_ready = 1;
            push(
                @{ $this->{'dialog'} },
                '携帯リダイレクトを設定しました。'
            );
        }
        $this->write_file( '../../.htaccess', \$htaccess, 1 );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>携帯リダイレクト</h1>';
    print
        '<p class="warning">このディレクトリ以下に旧式携帯ブラウザからアクセスがあったとき、旧式携帯ブラウザ向けに表示を変換する URL にリダイレクトするかどうか設定します。この設定は mod_rewrite が利用できる環境でのみ有効です。なお、mod_rewrite 非対応環境でリダイレクト設定を行うと全てのアクセスがエラーとなる場合がありますのでご注意ください。エラーからの復旧についてはマニュアルをご覧ください。</p>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print
        '<fieldset class="submit"><input type="submit" name="toggle=1" value="'
        . ( ($redirect_ready)
        ? '携帯リダイレクトを解除する'
        : '携帯リダイレクトを設定する' )
        . '" /></fieldset></form>';

    require CMONOS::CGI_Lib::HTTP_Client;
    my $mobile_url = $this->full_path('../../m.cgi/');
    my $res        = HTTP_Client->new->request(
        'GET' => 'http://chart.apis.google.com/chart?chs=160x160&cht=qr&chl='
            . $this->url_encode($mobile_url),
        {}, '', '', '',
        '../../qr.png'
    );

    if ( $res->is_success && -f '../../qr.png' ) {
        my $img
            = '<img src="'
            . $this->full_path('../../qr.png')
            . '" width="160" height="160" border="0" alt="'
            . $this->escape($mobile_url)
            . '" title="'
            . $this->escape($mobile_url) . '" />';
        print
            '<form action="\#"><fieldset><legend>QR コード (by <a href="http://code.google.com/intl/ja-JP/apis/chart/" target="_blank">Google Chart Tools</a>)</legend><p>'
            . $img
            . '</p><textarea rows="2" cols="90" />'
            . $this->escape($img)
            . '</textarea></fieldset></form>';
    }
    $res = HTTP_Client->new->request(
        'GET' => 'http://chart.apis.google.com/chart?chs=100x100&cht=qr&chl='
            . $this->url_encode($mobile_url),
        {}, '', '', '',
        '../../qr_mini.png'
    );
    if ( $res->is_success && -f '../../qr_mini.png' ) {
        my $img
            = '<img src="'
            . $this->full_path('../../qr_mini.png')
            . '" width="100" height="100" border="0" alt="'
            . $this->escape($mobile_url)
            . '" title="'
            . $this->escape($mobile_url) . '" />';
        print
            '<form action="\#"><fieldset><legend>QR コード (by <a href="http://code.google.com/intl/ja-JP/apis/chart/" target="_blank">Google Chart Tools</a>)</legend><p>'
            . $img
            . '</p><textarea rows="2" cols="90" />'
            . $this->escape($img)
            . '</textarea></fieldset></form>';
    }

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ($env::ownerpower) {
        no warnings qw(once);

        #クラス登録
        Common::class_register( '00s3', 'MobileRedirect' );

        #メニュー登録
        $setting::jump_menu{'00s3'} = '携帯リダイレクト';

    }
}

1;
