package SetupEnv;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# セットアップ情報モジュール
# SetupEnv.pm ver1.1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-01-09
######################################################################
# セットアップ情報取得
sub get_setup_env ($;$) {
    no warnings qw(once);
    my ( $this, $ver_only ) = @_;
######################################################################
    $cgi::cmonos_ver = 20230227;
######################################################################
    return $cgi::cmonos_ver if $ver_only;
    my $root
        = ($default::home_dir)
        ? ( '../' x ( $default::home_dir =~ tr!/!/! ) )
        : '';
    $default::cgi_data_name = 'cgi-data' if !$default::cgi_data_name;
    $default::cgi_data_root
        = $root
        . ( ( $default::cgi_data_name =~ /^\.\.\// ) ? '' : '../../../' )
        . $default::cgi_data_name;
    $default::cgi_data = $default::cgi_data_root . $default::home_dir;
    $default::shared_cgi_data_name = $default::cgi_data_name
        if !$default::shared_cgi_data_name;
    $default::shared_cgi_data_root
        = $root
        . (
        ( $default::shared_cgi_data_name =~ /^\.\.\// ) ? '' : '../../../' )
        . $default::shared_cgi_data_name;
}

sub get_perl_path ($) {
    my $this = shift;
    local (*ME);
    if ( !$default::perl_path ) {
        if ( open( ME, './start.cgi' ) ) {
            binmode ME, ":utf8";
            flock( ME, LOCK_EX );
            $default::perl_path = <ME>;
            chomp $default::perl_path;
            close(ME);
        }
        else {
            $default::perl_path = '#!/usr/bin/perl';
        }
    }
}

sub is_not_standard_env ($) {
    my $this = shift;
    return (   $env::spacer ne '../../image/spacer.gif'
            || $env::image_dir ne '../../image'
            || $env::css ne '../../lib/css'
            || $env::javascript ne '../../lib/javascript'
            || $env::externalfiles ne '../../lib/xfiles'
            || $this->base_change( '../', '../../' ) ne './cgi' ) ? 1 : 0;
}

sub get_home_dir_id ($;$) {
    my ( $this, $http_home_dir ) = @_;
    $http_home_dir
        = ( $http_home_dir && $http_home_dir ne '/' ) ? $http_home_dir
        : ( $default::http_home_dir && $default::http_home_dir ne '/' )
        ? $default::http_home_dir
        : '';
    if ($http_home_dir) {
        $http_home_dir =~ tr!/!_!;
        $http_home_dir =~ s/^_//g;
        $http_home_dir .= '_';
    }
    return $http_home_dir;
}

######################################################################
# ディレクトリ内のDBを復元
sub restore_default_settings ($$$;$$$$$) {
    my ( $this, $backup_path, $target_path, $find, $home_dir_id,
        $user_directory, $home_url, $conv_table )
        = @_;
    my ( @db, @restore );
    local (*DIR);
    return unless opendir( DIR, $backup_path );
    @db = readdir(DIR);
    closedir(DIR);
    foreach my $d (@db) {
        next if $d =~ /^\./;
        if ( $find && -d $backup_path . '/' . $d ) {
            $this->restore_default_settings(
                $backup_path . '/' . $d,
                $target_path . '/' . $d,
                $find, $home_dir_id, $user_directory, $home_url, $conv_table
            );
        }
        elsif ( $d =~ s/\.bak$// ) {
            push( @restore, $d );
        }
    }
    return unless $this->set_dir($target_path);
    $home_dir_id    = $this->get_home_dir_id   if !defined $home_dir_id;
    $user_directory = $this->home_directory(1) if !defined $user_directory;
    $home_url = $this->full_path( '../../', 1 ) if !defined $home_url;
    my $home_url_encoded = $this->url_encode($home_url);
    my $error            = 0;
    eval 'use ' . $env::dbm_file . ';';

    foreach my $bak (@restore) {
        my ( %db, $db_path, $bak_path );
        $db_path  = $target_path . '/' . $bak;
        $bak_path = $backup_path . '/' . $bak . '.bak';
        my $data = $this->read_file($bak_path);
        if ($data) {
            tie( %db, $env::dbm_file, $db_path, O_RDWR | O_CREAT,
                $env::dbm_mode );
            if ( ref tied %db ) {
                while ( $data
                    =~ /([^\x09\x0A\x0D]+)\x09([^\x0A\x0D]*)[\x0A\x0D]*/sg )
                {
                    my ( $k, $v ) = ( $1, $2 );
                    $k =~ s/\&#09;/\x09/gs;
                    $k =~ s/\x0B/\x0A/gs;
                    utf8::encode($k) if utf8::is_utf8($k);
                    if ( $bak eq 'record_id' && $k eq 'last' ) {
                        $v = '';
                    }
                    else {
                        $v =~ s/\&#09;/\x09/gs;
                        $v =~ s/\x0B/\x0A/gs;
                        $v =~ s/^_/_$home_dir_id/
                            if $home_dir_id
                            && $bak eq 'navi_db'
                            && $k =~ /_id$/;
                        $v =~ s!<home-dir-id>!$home_dir_id!gs;
                        $v =~ s!<user-directory>!$user_directory!gs;
                        $v =~ s!<home-url>!$home_url!gs;
                        $v =~ s/<home-url-encoded>/$home_url_encoded/gs;
                        $v = $this->replace_text( $v, $conv_table )
                            if $conv_table;
                        utf8::encode($v) if utf8::is_utf8($v);
                    }
                    eval '$db{$k} = $v;';
                    $error++ if defined $@ && $@;
                }
            }
            untie %db;
        }
    }
    push(
        @{ $this->{'dialog'} },
        '容量が大き過ぎる'
            . $error
            . '件のデータ復元に失敗しました。'
    ) if $error;
}

sub replace_text ($;$) {
    my ( $this, $text, $table ) = @_;
    if ($table) {
        while ( $table =~ /^(.+?)=>(.+?)$/gm ) {
            my ( $m, $r ) = ( $1, $2 );
            chomp $r;
            $text =~ s/\Q$m\E/$r/sg;
        }
    }
    return $text;
}

sub directory_suffix ($) {
    my $this = shift;
    my $dir = $default::home_dir || '/';
    $dir =~ s/([^0-9a-zA-Z_])/'-' . sprintf("%.2X", unpack("C", $1))/eg;
    return '@' . $dir;
}

sub get_sitetemplate_id_and_home_dir ($$) {
    my ( $this, $dir ) = @_;
    my $suffix = '';
    if ( $dir =~ /^([0-9a-zA-Z_]+)\@(.*)$/ ) {
        $dir    = $1;
        $suffix = $2;
        utf8::encode($suffix) if utf8::is_utf8($suffix);
        $suffix =~ s/-([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
    }
    return ( $dir, $suffix );
}

sub site_template_title ($$$$) {
    my ( $this, $name, $label ) = @_;
    return ( $label eq 'default' && !$name )
        ? 'デフォルトのサイトテンプレート'
        : ( $name && $name eq '[記事・データを含む]' )
        ? $label . $name
        : ($name) ? $this->escape($name) . '(' . $label . ')'
        :           $label;
}

sub rand_char_sample ($) {
    my $this = shift;
    my (@charset) = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9' );
    my ( $char, $num );
    $num = 6 + int( rand(11) );
    foreach ( 0 .. $num ) {
        $char .= $charset[ int( rand(62) ) ];
    }
    return $char;
}

sub env_escape ($$) {
    my ( $this, $str ) = @_;
    $$str =~ s/[\x0A\x0D]//sg;
    $$str =~ s/'/&#039;/sg;
    $$str =~ s/\$/&#036;/sg;
    $this->escape_except_html_entity($str);
}

sub subdir_options ($$) {
    my ( $this, $path ) = @_;
    my $data = $this->read_file( $path, 1 );
    my ( $multi_domain, $server_name, $ssl, $subdirectory_disabled,
        $user_site_template_dir, $user_default_dir );
    if ( $data
        =~ /^[\x20\x09]*\$multi_domain[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*([01]*);/m
        )
    {
        $multi_domain = $1;
    }
    if ( $data
        =~ /^[\x20\x09]*\$server_name[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*["']([^'"]*)["'];/m
        )
    {
        $server_name = $1;
    }
    if ( $data
        =~ /^[\x20\x09]*\$ssl[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*["']?([0-9]*)["']?;/m
        )
    {
        $ssl = $1;
    }
    if ( $data
        =~ /^[\x20\x09]*\$user_site_template_dir[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*["']([^'"]*)["'];/m
        )
    {
        $user_site_template_dir = $1;
    }
    if ( $data
        =~ /^[\x20\x09]*\$user_default_dir[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*["']([^'"]*)["'];/m
        )
    {
        $user_default_dir = $1;
    }
    if ( $data
        =~ /^[\x20\x09]*\$subdirectory_disabled[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*([01]*);/m
        )
    {
        $subdirectory_disabled = $1;
    }
    return ( $multi_domain, $server_name, $ssl, $subdirectory_disabled,
        $user_site_template_dir, $user_default_dir );
}

sub optimize_htaccess ($$$;$) {
    my ( $this, $home_path, $home_abs ) = @_;
    $home_abs = $this->path_change( $home_path, $ENV{'SCRIPT_NAME'}, 'abs' )
        if !$home_abs;
    $home_path =~ s!/$!!;
    $home_abs =~ s!/$!!;
    my $htaccess_path = $home_path . '/.htaccess';
    my $htaccess = $this->read_file($htaccess_path) || '';
    if ( -d $home_path . '/error' ) {
        $htaccess
            =~ s/(ErrorDocument\s+\d+\s+)\/error(\/\d+\.html\s*)/$1$home_abs\/error$2/sig;
    }
    else {
        $htaccess =~ s/ErrorDocument\s+\d+\s+\/error\/\d+\.html\s*//sig;
    }
    $this->write_file( $home_path . '/lib/xfiles/blank-page.ssi', '', 1 );
    my $redirect_rule = <<REDIRECT;

# For Bot - start -
<IfModule mod_rewrite.c>
RewriteEngine On
RewriteCond \%{HTTP_USER_AGENT} (Googlebot|Slurp|msnbot|http://) [NC]
RewriteRule /(navi|recent\\w+|metaindex|trackbacks|related)\\.ssi\$ $home_abs/lib/xfiles/blank-page.ssi
</IfModule>
# For Bot - end -
REDIRECT
    $htaccess .= $redirect_rule
        unless $htaccess
        =~ s/[\x09\x0A\x0C\x0D\x20]?# For Bot - start -.*?# For Bot - end -[\x09\x0A\x0C\x0D\x20]?/$redirect_rule/s;
    $this->write_file( $htaccess_path, \$htaccess, 1 );

}

1;
