package SiteTemplateApplier;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# サイトテンプレート適用モジュール
# SiteTemplateApplier.pm ver1.0.14
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-07-20
######################################################################
# 実行
sub run ($) {
    my $this       = shift;
    my $error_flag = 0;
    $this->get_setup_env;
    $this->get_perl_path;
    my ( $site_template_dir, $selected_template );
    if (   $main::template
        && $main::template =~ /^(user:|)([0-9a-zA-Z_\-\@]+)$/ )
    {
        my $user_template = $1;
        $selected_template = $2;
        my ( $tempid, $home_dir )
            = $this->get_sitetemplate_id_and_home_dir($selected_template);
        if (   $default::home_dir
            && $home_dir
            && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/ )
        {
            undef $selected_template;
        }
        elsif ( $user_template && $env::user_site_template_dir ) {
            $site_template_dir = $env::user_site_template_dir;
        }
        else {
            $site_template_dir = $env::template_dir . '/site';
        }
    }

    #入力チェック〜インストール処理
    if ( defined $selected_template
        && -d $site_template_dir . '/' . $selected_template )
    {
        my $temp_path = $site_template_dir . '/' . $selected_template;
        my ( $temp_name, $data_included ) = split(
            /\x09/,
            $this->read_file(
                $site_template_dir . '/' . $selected_template . '/name.txt'
            )
        );
        my $home_dir_id            = $this->get_home_dir_id;
        my $user_directory         = $this->home_directory(1);
        my $home_url               = $this->full_path( '../../', 1 );
        my $home_url_encoded       = $this->url_encode($home_url);
        my $user_directory_encoded = $this->url_encode($user_directory);

        #env.pl更新
        $this->update_env( $temp_path . '/env.bak', $main::conv_table );

        #公開ディレクトリ更新
        eval 'use File::Path';
        $this->update_htdocs(
            $temp_path . '/htdocs', $home_dir_id,
            $user_directory,        $user_directory_encoded,
            $home_url,              $home_url_encoded,
            $main::conv_table,      $data_included
        );

        #preferenceディレクトリ更新
        eval 'use ' . $env::dbm_file . ';';
        $this->update_db(
            $temp_path . '/preferences', $default::preferences,
            $home_dir_id,                $user_directory,
            $user_directory_encoded,     $home_url,
            $home_url_encoded,           $main::conv_table,
            $data_included,              $main::page_replaceable,
            $main::field_replaceable
        );

        #dataディレクトリ更新
        $this->update_db(
            $temp_path . '/data',    $default::data,
            $home_dir_id,            $user_directory,
            $user_directory_encoded, $home_url,
            $home_url_encoded,       $main::conv_table,
            1
        );

        #QRコード画像
        require CMONOS::CGI_Lib::HTTP_Client;
        my $res = HTTP_Client->new->request(
            'GET' =>
                'http://chart.apis.google.com/chart?chs=160x160&cht=qr&chl='
                . $this->url_encode( $this->full_path('../../m.cgi/') ),
            {}, '', '', '',
            '../../qr.png'
        );
        $res = HTTP_Client->new->request(
            'GET' =>
                'http://chart.apis.google.com/chart?chs=100x100&cht=qr&chl='
                . $this->url_encode( $this->full_path('../../m.cgi/') ),
            {}, '', '', '',
            '../../qr_mini.png'
        );

        #JavaScriptアップデート
        require CMONOS::SetupAssistant::Tools::JavaScriptUpdater;
        Common->new('s303')
            ->javascript_update( $default::home_dir, 0, '../..' );

        #CSSアップデート
        require CMONOS::SetupAssistant::Tools::CSSUpdater;
        {
            local @main::target = (
                'common.css',            'calendar-common.css',
                'tablemaker-common.css', 'tools.css',
                'images'
            );
            Common->new('s304')->css_update($default::home_dir);
        }

        push(
            @{ $this->{'dialog'} },
            '選択されたサイトテンプレート「'
                . (
                ( $selected_template eq 'default' )
                ? 'デフォルトのサイトテンプレート'
                : ($temp_name)
                ? $this->escape($temp_name) . '(' . $selected_template . ')'
                : $selected_template
                )
                . '」をこのディレクトリに適用しました。'
        );
        undef $main::page_replaceable;
        undef $main::field_replaceable;
        require CMONOS::CGI_Lib::NavUpdate;
        NavUpdate->new->scan;
        Common->new('s302')->update('imagedir');
        my $javascript = <<JAVA;
cmonos.tool.refreshNavi = function () {
	if (parent.tool_navi) {
		parent.tool_navi.location.reload();
	}
}
JAVA
        $this->set_onload('cmonos.tool.refreshNavi()');
        $this->set_javascript($javascript);
    }
    elsif ( defined $selected_template && $selected_template ne '' ) {
        push(
            @{ $this->{'dialog'} },
            '指定されたテンプレートは存在しません。'
        );
    }
    else {
        push(
            @{ $this->{'dialog'} },
            'このディレクトリは CGI の動作に必要なファイルまたはディレクトリが標準的な位置にありません。サイトテンプレートの適用には適していません。'
        ) if $this->is_not_standard_env;
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サイトテンプレート適用</h1>';

    print $this->form_start(
        'key'    => 'tools',
        'name'   => 'input',
        'method' => 'post',
        'onsubmit' =>
            'return confirm(\'この操作は取り消せません。本当にサイトテンプレートを適用しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
        'required' => \@setting::form
    );
    print '<fieldset>';
    print '<table class="field-table"><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="template">サイトテンプレート</label></td>';
    print
        '<td class="field"><select name="template"><option value="default">デフォルトのサイトテンプレート</option>';

    if ( opendir( TEMP, $env::template_dir . '/site' ) ) {
        ;
        no warnings qw(numeric);
        my @template
            = grep( $_ ne 'default'
                && $_ !~ /\./
                && -d $env::template_dir . '/site/' . $_,
            readdir(TEMP) );
        closedir(TEMP);
        foreach my $dir ( sort { $a <=> $b || $a cmp $b } @template ) {
            my ( $tempid, $home_dir, $name );
            ( $tempid, $home_dir )
                = $this->get_sitetemplate_id_and_home_dir($dir);
            if ($home_dir) {
                next
                    if $default::home_dir
                    && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/;
                $tempid = $this->escape( $tempid . '@~' . $home_dir );
            }
            ($name) = split(
                /\x09/,
                $this->read_file(
                    $env::template_dir . '/site/' . $dir . '/name.txt'
                )
            );
            $name = '' if !defined $name;
            print '<option value="'
                . $dir . '"'
                . ( ( $selected_template && $dir eq $selected_template )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $this->site_template_title( $name, $tempid )
                . ( ($env::user_site_template_dir) ? '&lt;共有&gt;' : '' )
                . '</option>';
        }
    }
    if ( $env::user_site_template_dir
        && opendir( TEMP, $env::user_site_template_dir ) )
    {
        ;
        no warnings qw(numeric);
        my @template
            = grep( $_ !~ /\./ && -d $env::user_site_template_dir . '/' . $_,
            readdir(TEMP) );
        closedir(TEMP);
        foreach my $dir ( sort { $a <=> $b || $a cmp $b } @template ) {
            my ($name) = split(
                /\x09/,
                $this->read_file(
                    $env::user_site_template_dir . '/' . $dir . '/name.txt'
                )
            );
            $name = '' if !defined $name;
            print '<option value="user:'
                . $dir . '"'
                . ( ( $selected_template && $dir eq $selected_template )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $this->site_template_title( $name, $dir )
                . '</option>';
        }
    }
    print '</select></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">初期設定上書き設定</span></td>';
    print '<td class="field"><ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="page_replaceable" id="page_replaceable"'
        . ( ($main::page_replaceable) ? ' checked="checked"' : '' )
        . ' /> 同名 CGI のページ設定を上書きする。</label></li>';
    print
        '<li><label><input type="checkbox" name="field_replaceable" id="field_replaceable"'
        . ( ($main::field_replaceable) ? ' checked="checked"' : '' )
        . ' /> 同名 CGI のフィールド設定を上書きする。</label></li>';
    print '</ol></td></tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="conv_table">文字変換表</label></td>';
    print
        '<td class="field"><textarea name="conv_table" id="conv_table" cols="72" rows="4" wrap="off" title="下記書式に従って文字変換表を設定します。">'
        . $this->escape($main::conv_table)
        . '</textarea>';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'conv_table\').setAutoFit(this.checked);" />フィット</label>';
    print
        '<div class="note">書式：変換対象文字列=&gt;変換後文字列[改行]</div>';
    print '</td>';
    print '</tr>';
    print '</tbody></table></fieldset>';
    print
        '<div class="warning">現在のディレクトリにサイトテンプレートを適用すると、画像ライブラリやナビゲーション設定がサイトテンプレートのものに置き替わります。現在の設定は破棄されます。後から復元することはできません。';
    print
        'このため、新しい画像ライブラリやナビゲーション設定が現在の設定と矛盾することがあります。';
    print
        'サイトテンプレート適用後、ページレイアウト等の設定を再確認してください。</div>';
    print
        '<fieldset class="delete"><input type="submit" value="選択したサイトテンプレートをこのディレクトリに適用する" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

#env.pl作成
sub update_env ($$;$) {
    my ( $this, $env_setting, $conv_table ) = @_;
    if ($env_setting) {
        no strict "refs";
        my $env_path = $default::cgi_data . '/lib/env.pl';
        my $setting  = $this->read_file($env_setting);
        if ($setting) {
            my $data = $this->read_file( $env_path, 1 );
            $data =~ s/package env;/package env;\x0Ause utf8;/s
                if $data !~ /use utf8;/;
            while ( $setting
                =~ /([^\x09\x0A\x0D]+)\x09([^\x0A\x0D]*)[\x0A\x0D]*/sg )
            {
                my ( $k, $v ) = ( $1, $2 );
                next if $k eq 'checkout' && $env::checkout && !$v;
                $v =~ s/\x0B/\x0A/sg;
                $v = $this->replace_text( $v, $conv_table ) if $conv_table;
                ${ 'env::' . $k } = $v;
                $v =~ s/\\/\\\\/sg;
                $v =~ s/'/\\'/sg;
                $v = "'" . $v . "'" if $v !~ /^\-?[0-9\.]+$/;

                if ( $v =~ /\x0A/ ) {
                    $v =~ s/^';/'&#59;/s;
                    $data =~ s!\$$k[\x20\x09]*=.*?[^\\]';!\$$k = $v;!gs;
                }
                else {
                    $data
                        =~ s!^[\x20\x09]*\$$k[\x20\x09]*=[^\x0A\x0D]*;!\$$k = $v;!gm;
                }
            }
            $this->write_file( $env_path, \$data, 1 );
        }
    }
}

sub update_htdocs ($$$$$$;$$) {
    my ($this,             $htdocs_path,            $home_dir_id,
        $user_directory,   $user_directory_encoded, $home_url,
        $home_url_encoded, $conv_table,             $data_included
    ) = @_;
    my ( @files, $javascript_dir, $home_site, $home_domain );
    local (*DIR);
    return unless opendir( DIR, $htdocs_path );
    @files = grep( $_ !~ /^\.\./ && $_ ne '.placeholder', readdir(DIR) );
    closedir(DIR);
    $home_site = $home_url;
    $home_site =~ s!^https?://!!i;
    $javascript_dir
        = $this->path_change( $env::javascript, $ENV{'SCRIPT_NAME'}, 'abs' );
    $home_domain
        = $env::server_name || ( ( $home_site =~ /^([\/:]+)/ ) ? $1 : '' );

    foreach my $file (@files) {
        if (   $file eq 'cgi'
            || $file eq 'lib'
            || $file eq 'image'
            || '../../' . $file eq $default::cgi_data_root
            || '../../' . $file eq $default::shared_cgi_data_root )
        {
            next;
        }
        elsif ( -f $htdocs_path . '/' . $file ) {
            if ( $data_included || !-f '../../' . $file ) {
                if (   $file =~ /\.(s?html?|ssi|xml|kml|dat|js|css)$/i
                    && $file
                    !~ /[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.dat$/ )
                {
                    my $data = $this->read_file( $htdocs_path . '/' . $file );
                    $data
                        =~ s!<meta\s*http-equiv="Content-Type"\s*content="text/html;\s*charset=EUC-JP"\s*/?>!<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />!sig;
                    $data
                        =~ s!<meta\s*charset="EUC-JP"\s*/?>!<meta charset="UTF-8" />!sig;
                    $data =~ s! charset="EUC-JP"! charset="UTF-8"!sig;
                    $data =~ s/\@charset "EUC-JP";/\@charset "UTF-8";/sig;
                    if (   defined $home_dir_id
                        && $home_dir_id ne ''
                        && (   $file =~ /^index\.s?html?$/
                            || $file eq 'frame.html' )
                        && $data =~ /<frameset/
                        )
                    {
                        $data =~ s/ name="main"/ name="${home_dir_id}main"/g;
                        $data =~ s/ name="navi"/ name="${home_dir_id}navi"/g;
                    }
                    my $icon_dir = $this->base_change( $env::icon_dir,
                        '../../' . $file );
                    my $is_cms = 0;
                    $is_cms = 1
                        if $data =~ s/<user-directory>/$user_directory/gs;
                    $data =~ s/<site-title>/$env::title/g;
                    $data =~ s/<site-description>/$env::description/g;
                    $is_cms = 1
                        if $data =~ s/<user-directory>/$user_directory/gs;
                    $data =~ s/<icon-dir>/$icon_dir/g;
                    $data =~ s/<home-dir>/$user_directory/g;
                    $is_cms = 1 if $data =~ s/<home-dir-id>/$home_dir_id/g;
                    $is_cms = 0
                        if $data
                        =~ m!<link rel="canonical" href="<home-url>/[/0-9a-zA-Z_]+/start.cgi[^"]*?" />!s;
                    $data =~ s/<home-url>/$home_url/g;
                    $data =~ s/<home-url-encoded>/$home_url_encoded/g;
                    $data =~ s/<home-site>/$home_site/gs;
                    $data =~ s/<home-domain>/$home_domain/gs;

                    if ($is_cms) {
                        if ( $file =~ /\.shtml?$/i && !$env::ssi ) {
                            $file =~ s/\.shtml?$/.html/i;
                        }
                        elsif ( $file =~ /\.html?$/i && $env::ssi ) {
                            $file =~ s/\.html?$/.$env::ssi_ext/i;
                        }
                    }
                    $data
                        =~ s/(<form(?=[\x09\x0A\x0D\x20])[^>]*?[\x09\x0A\x0D\x20]action="(?!http:\/\/)[^"]+"[^>]*?>)(?![\x09\x0A\x0D\x20]*<input type="hidden" name="_" value="＿"[^>]*?>)/$1<input type="hidden" name="_" value="＿" \/>/sig;
                    $data = $this->replace_text( $data, $conv_table )
                        if $conv_table;
                    $this->write_file(
                        '../../' . $file,
                        \$data,
                        1,
                        (   ( $file =~ /\.(cgi|pl)$/ )
                            ? $env::cgi_file_mode
                            : $env::file_mode
                        )
                    );
                }
                elsif ( $file eq 'm.cgi' ) {
                    if ( !-f '../../m.cgi' ) {
                        my $data
                            = $this->read_file( $htdocs_path . '/' . $file );
                        my $cgi_data_path
                            = $this->base_change( $default::cgi_data,
                            '../../m.cgi', 'rel', 1 );
                        my $shared_cgi_data_root
                            = $this->base_change(
                            $default::shared_cgi_data_root,
                            '../../m.cgi', 'rel', 1 );
                        $data =~ s/\A#!.*?$/$default::perl_path/m
                            if $default::perl_path;
                        $data
                            =~ s!<shared-cgi-data-root-from-top>!$shared_cgi_data_root!g;
                        $data =~ s/<cgi-data-path-from-top>/$cgi_data_path/g;
                        $this->write_file( '../../m.cgi', \$data, 1,
                            $env::cgi_file_mode );
                    }
                }
                elsif ( $file eq '.htaccess' ) {
                    my $gate_keeper     = '';
                    my $mobile_redirect = '';
                    if ( -f '../../' . $file ) {
                        my $old_data = $this->read_file( '../../' . $file );
                        $gate_keeper = $1
                            if $old_data
                            =~ /(# Gate Keeper - start -.*?# Gate Keeper - end -[\x09\x0A\x0C\x0D\x20]?)/s;
                        $mobile_redirect = $1
                            if $old_data
                            =~ /(# For Mobile - start -.*?# For Mobile - end -[\x09\x0A\x0C\x0D\x20]?)/s;
                    }
                    my $data = $this->read_file( $htdocs_path . '/' . $file );
                    if ($gate_keeper) {
                        $data .= "\x0A" . $gate_keeper
                            unless $data
                            =~ s/# Gate Keeper - start -.*?# Gate Keeper - end -[\x09\x0A\x0C\x0D\x20]?/$gate_keeper/s;
                    }
                    elsif ( $data =~ /# Gate Keeper - start -/ ) {
                        my $spath = $this->path_conv( '../../', 'path' );
                        $data
                            =~ s!(AuthUserFile[\x09\x20]+)[^\x0A]*(/\.htpasswd)!$1$spath$2!ig;
                        $data
                            =~ s!(AuthGroupFile[\x09\x20]+)[^\x0A]*(/\.htgroup)!$1$spath$2!ig;
                    }
                    if ($mobile_redirect) {
                        $data .= "\x0A" . $mobile_redirect
                            unless $data
                            =~ s/# For Mobile - start -.*?# For Mobile - end -[\x09\x0A\x0C\x0D\x20]?/$mobile_redirect/s;
                    }
                    elsif ( $data =~ /# For Mobile - start -/ ) {
                        my $this_dir = $this->path_change( '../../',
                            $ENV{'SCRIPT_NAME'}, 'abs' );
                        $this_dir .= '/' if $this_dir !~ m!/$!;
                        $data
                            =~ s![^\x09\x0A\x0C\x0D\x20]*/m\.cgi/!${this_dir}m\.cgi/!sg;
                    }
                    $this->write_file(
                        '../../' . $file,
                        \$data,
                        1,
                        (   ( $file =~ /\.(cgi|pl)$/ )
                            ? $env::cgi_file_mode
                            : $env::file_mode
                        )
                    );
                }
                else {
                    $this->file_copy(
                        $htdocs_path . '/' . $file,
                        '../../' . $file,
                        (   ( $file =~ /\.(cgi|pl)$/ )
                            ? $env::cgi_file_mode
                            : $env::file_mode
                        )
                    );
                }
            }
        }
        elsif ( $file !~ /^\./ && -d $htdocs_path . '/' . $file ) {
            $this->_update_htdocs(
                $htdocs_path . '/' . $file, '../../' . $file,
                $user_directory,            $user_directory_encoded,
                $javascript_dir,            $home_dir_id,
                $home_url,                  $home_url_encoded,
                $home_site,                 $conv_table,
                $data_included
            );
        }
    }
    $this->_update_htdocs(
        $htdocs_path . '/cgi', '../../cgi',
        $user_directory,       $user_directory_encoded,
        $javascript_dir,       $home_dir_id,
        $home_url,             $home_url_encoded,
        $home_site,            $conv_table,
        1
    );
    $this->_update_htdocs(
        $htdocs_path . '/lib', '../../lib',
        $user_directory,       $user_directory_encoded,
        $javascript_dir,       $home_dir_id,
        $home_url,             $home_url_encoded,
        $home_site,            $conv_table,
        1
    );
    $this->_update_htdocs(
        $htdocs_path . '/image',
        '../../image',
        $user_directory,
        $user_directory_encoded,
        $javascript_dir,
        $home_dir_id,
        $home_url,
        $home_url_encoded,
        $home_site,
        $conv_table,
        1,
        1,
        ( ($data_included) ? [] : ['top'] ),
        1,
        [ 'baobab', 'calendar', 'navi', 'order', 'tablemaker' ]
    );
}

sub _update_htdocs ($$$$$$$$$$$$;$$$) {
    my ($this,            $dir,                    $clone,
        $user_directory,  $user_directory_encoded, $javascript_dir,
        $home_dir_id,     $home_url,               $home_url_encoded,
        $home_site,       $conv_table,             $replaceable,
        $replaceable_dir, $skip,                   $replace_dir_skip,
        $parent_dir
    ) = @_;
    my ( @files, @directories, $file );
    local (*DIR);
    return unless opendir( DIR, $dir );
    while ( $file = readdir(DIR) ) {
        my $path = $dir . '/' . $file;
        if (   $clone . '/' . $file eq $default::cgi_data_root
            || $clone . '/' . $file eq $default::shared_cgi_data_root )
        {
            next;
        }
        elsif ( $file !~ /^\.\./ && $file ne '.placeholder' && -f $path ) {
            if ( $replaceable || !-f $clone . '/' . $file ) {
                push( @files, $file );
            }
        }
        elsif ( $file !~ /^\./ && -d $path ) {
            push( @directories, $file );
        }
    }
    closedir(DIR);
    return unless $this->set_dir($clone);
    if (@files) {
        my $meta_home_dir_id = $home_dir_id;
        $meta_home_dir_id =~ s/_/-/g;
        my $default_id = '';
        (   $default_id = (
                ( $default::http_home_dir && $default::http_home_dir ne '/' )
                ? $default::http_home_dir
                : ''
                )
                . '/'
                . $1
            ) =~ tr!/!_!
            if $dir =~ /([0-9a-zA-Z_]+)$/;
        my $home_domain = $env::server_name
            || ( ( $home_site =~ /^([\/:]+)/ ) ? $1 : '' );
        foreach $file (@files) {
            if ((      $file =~ /\.[0-9a-zA-Z_]+$/
                    && $file ne '.htaccess'
                    && $file !~ /\.(cgi|pl|s?html?|ssi|xml|kml|dat|js|css)$/
                    && $file ne 'name.txt'
                )
                || $file
                =~ /[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.dat$/
                || ( -s $dir . '/' . $file ) > 1000000
                )
            {
                if (   $clone =~ m!^\.\./\.\./image/!
                    && $clone !~ m!^\.\./\.\./image/top/!
                    && $clone !~ m!/(?:[0-9]+|misc)$!
                    && $file =~ /^([0-9]+)\.(?:jpg|gif|png)$/ )
                {
                    my $file_name = $1;
                    unlink( $clone . '/' . $file_name . '.jpg' )
                        || unlink( $clone . '/' . $file_name . '.gif' )
                        || unlink( $clone . '/' . $file_name . '.png' );
                }
                $this->file_copy(
                    $dir . '/' . $file,
                    $clone . '/' . $file,
                    (   ( $file =~ /\.(cgi|pl)$/ )
                        ? $env::cgi_file_mode
                        : $env::file_mode
                    )
                );
                next;
            }
            my $is_xfile = 0;
            my $data     = $this->read_file( $dir . '/' . $file );
            if ( $file eq '.htaccess' ) {
                if ( $data =~ /# Gate Keeper - start -/ ) {
                    my $spath = $this->path_conv( $clone, 'path' );
                    $data
                        =~ s!(AuthUserFile[\x09\x20]+)[^\x0A]*(/\.htpasswd)!$1$spath$2!ig;
                    $data
                        =~ s!(AuthGroupFile[\x09\x20]+)[^\x0A]*(/\.htgroup)!$1$spath$2!ig;
                }
                if ( $data =~ /# For Mobile - start -/ ) {
                    my $this_dir
                        = $this->path_change( $clone, $ENV{'SCRIPT_NAME'},
                        'abs' );
                    $this_dir .= '/' if $this_dir !~ m!/$!;
                    $data
                        =~ s![^\x09\x0A\x0C\x0D\x20]*/m\.cgi/!${this_dir}m\.cgi/!sg;
                }
            }
            elsif ( $file =~ /\.cgi$/ ) {
                $data =~ s/use strict;/use strict;\x0Ause utf8;/s
                    if $data !~ /use utf8;/;
                $data =~ s/\->decode\(1,\s*'euc'/->decode\(1,'utf8'/sg;
                $data =~ s/\A#!.*?$/$default::perl_path/m
                    if $default::perl_path;
                $data
                    =~ s/<shared-cgi-data-root>/$default::shared_cgi_data_root/g;
                $data =~ s/<cgi-data-root>/$default::cgi_data_root/g;
                $data =~ s/<cgi-data-path>/$default::cgi_data/g;
            }
            elsif ( $file eq 'default.pl' ) {
                $data =~ s/package default;/package default;\x0Ause utf8;/s
                    if $data !~ /use utf8;/;
                $data
                    =~ s/<shared-cgi-data-root>/$default::shared_cgi_data_root/g;
                $data =~ s/<cgi-data-root>/$default::cgi_data_root/g;
                $data =~ s/<cgi-data-path>/$default::cgi_data/g;
                if ( $dir =~ m!/gatekeeper$! ) {
                    if ($env::home_dir_id) {
                        $data =~ s/<web-home-dir>/$user_directory/g;
                        $data =~ s/<home-dir>/$user_directory/g;
                    }
                    else {
                        $data =~ s/<web-home-dir>//g;
                        $data =~ s/<home-dir>//g;
                    }
                }
                else {
                    $data =~ s/<web-home-dir>/$user_directory/g;
                    $data =~ s/<home-dir>/$default::home_dir/g;
                }
                $data =~ s/<home-dir-id>/$home_dir_id/g;
                $data =~ s/<default-id>/$default_id/g;
                $data =~ s/<cgi-name>/$file/g;
                if ( $dir =~ m!/setup$! ) {
                    $data
                        =~ s/^[\x20\x09]*\$cgi_data_name[\x20\x09]*=[^\x0A\x0D]*;/\$cgi_data_name = '$default::cgi_data_name';/gm;
                    $data
                        =~ s/^[\x20\x09]*\$shared_cgi_data_name[\x20\x09]*=[^\x0A\x0D]*;/\$shared_cgi_data_name = '$default::shared_cgi_data_name';/gm;
                }
            }
            elsif ( $file eq 'cmonos.js' ) {
                $data
                    =~ s!<meta\s*http-equiv="Content-Type"\s*content="text/html;\s*charset=EUC-JP"\s*/?>!<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />!sig;
                $data
                    =~ s!<meta\s*charset="EUC-JP"\s*/?>!<meta charset="UTF-8" />!sig;
                $data =~ s! charset="EUC-JP"! charset="UTF-8"!sig;
                $data =~ s!(\.\./\.\./|/)lib/javascript!$javascript_dir!g;
                if ( defined $home_dir_id && $home_dir_id ne '' ) {
                    $data =~ s/parent\.main/parent\.${home_dir_id}main/g;
                    $data =~ s/parent\.navi/parent\.${home_dir_id}navi/g;
                }
            }
            elsif ( $file =~ /\.(s?html?|ssi|xml|kml|dat|css)$/i
                || ( $file =~ /\.js$/i && $dir !~ m!/lib/javascript! ) )
            {
                next if $file =~ /\.js$/;
                if ( $file =~ s/^site-navi-utf8\.ssi$/site-navi-euc.ssi/ ) {
                    $data
                        =~ s!<meta\s*http-equiv="Content-Type"\s*content="text/html;\s*charset=UTF-8"\s*/?>!<meta http-equiv="Content-Type" content="text/html;charset=EUC-JP" />!sig;
                    $data
                        =~ s!<meta\s*charset="UTF-8"\s*/?>!<meta charset="EUC-JP" />!sig;
                }
                else {
                    $data
                        =~ s!<meta\s*http-equiv="Content-Type"\s*content="text/html;\s*charset=EUC-JP"\s*/?>!<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />!sig;
                    $data
                        =~ s!<meta\s*charset="EUC-JP"\s*/?>!<meta charset="UTF-8" />!sig;
                    $data =~ s! charset="EUC-JP"! charset="UTF-8"!sig;
                    $data =~ s/\@charset "EUC-JP";/\@charset "UTF-8";/sig;
                }
                my $title       = $env::title;
                my $description = $env::description;
                my $table       = $conv_table;
                my $icon_dir    = $this->base_change( $env::icon_dir,
                    $clone . '/' . $file );
                my $is_cms = 0;
                $data =~ s/<site-title>/$title/g;
                $data =~ s/<site-description>/$description/g;
                $data =~ s/<icon-dir>/$icon_dir/g;
                $is_cms = 1 if $data =~ s/<user-directory>/$user_directory/gs;
                $is_cms = 1
                    if $data
                    =~ s/<user-directory-encoded>/$user_directory_encoded/gs;
                $data =~ s/<home-dir>/$user_directory/g;
                $is_cms = 1 if $data =~ s/<home-dir-id>/$home_dir_id/g;
                $data =~ s/<meta-home-dir-id>/$meta_home_dir_id/g;
                $data =~ s/<home-url>/$home_url/g;
                $data =~ s/<home-url-encoded>/$home_url_encoded/g;
                $data =~ s/<home-site>/$home_site/gs;
                $data =~ s/<home-domain>/$home_domain/gs;
                $is_cms = 0
                    if $is_cms
                    && $file =~ /^([0-9]+)\./
                    && ( -f $dir . '/' . $1 . '.spch'
                    || $dir . '/' . $1 . '.dat' );

                if ($is_cms) {
                    if ( $file =~ /\.shtml?$/i && !$env::ssi ) {
                        $file =~ s/\.shtml?$/.html/i;
                    }
                    elsif ($file =~ /\.html?$/i
                        && $env::ssi
                        && $dir !~ m!/cgi/! )
                    {
                        $file =~ s/\.html?$/.$env::ssi_ext/i;
                    }
                }
                $data
                    =~ s/(<form(?=[\x09\x0A\x0D\x20])[^>]*?[\x09\x0A\x0D\x20]action="(?!http:\/\/)[^"]+"[^>]*?>)(?![\x09\x0A\x0D\x20]*<input type="hidden" name="_" value="＿"[^>]*?>)/$1<input type="hidden" name="_" value="＿" \/>/sig;
                $data = $this->replace_text( $data, $table ) if $table;
                if ( $file =~ /^site-navi-(sjis|euc)\.ssi$/ ) {
                    $this->jcode_convert( \$data, $1, 'utf8' );
                }
                $is_xfile = 1 if $file =~ /\.ssi$/;
            }
            $this->write_file(
                $clone . '/' . $file,
                \$data,
                1,
                (   ( $file =~ /\.(cgi|pl)$/ )
                    ? $env::cgi_file_mode
                    : $env::file_mode
                )
            );
            $this->mk_external_javascript( $clone . '/' . $file )
                if $is_xfile;
        }
    }
    if (@directories) {
        foreach my $directory (@directories) {
            next if ref $skip && grep( $directory eq $_, @$skip );
            if ((      $clone =~ m!^\.\./\.\./image/!
                    && $clone !~ m!^\.\./\.\./image/top/!
                    && $directory =~ /^[0-9]+$/
                )
                || (  !$replace_dir_skip
                    && $replaceable_dir
                    && -d $clone . '/' . $directory )
                )
            {
                rmtree( $clone . '/' . $directory );
            }
            $this->_update_htdocs(
                $dir . '/' . $directory,
                $clone . '/' . $directory,
                $user_directory,
                $user_directory_encoded,
                $javascript_dir,
                $home_dir_id,
                $home_url,
                $home_url_encoded,
                $home_site,
                $conv_table,
                $replaceable,
                $replaceable_dir,
                '',
                (   (   ref $parent_dir
                            && grep( $directory eq $_, @$parent_dir )
                    ) ? 1 : 0
                )
            );
        }
    }
}

sub update_db ($$$$$$$$;$$$$) {
    my ($this,            $backup_path,      $target_path,
        $home_dir_id,     $user_directory,   $user_directory_encoded,
        $home_url,        $home_url_encoded, $conv_table,
        $abs_replaceable, $page_replaceable, $field_replaceable,
        $all_replaceable
    ) = @_;
    my ( @db, @restore );
    local (*DIR);
    return unless opendir( DIR, $backup_path );
    @db = readdir(DIR);
    closedir(DIR);
    foreach my $d (@db) {
        next if $d =~ /^\./;
        if ( -d $backup_path . '/' . $d ) {
            $this->update_db(
                $backup_path . '/' . $d,
                $target_path . '/' . $d,
                $home_dir_id,
                $user_directory,
                $user_directory_encoded,
                $home_url,
                $home_url_encoded,
                $conv_table,
                $abs_replaceable,
                $page_replaceable,
                $field_replaceable,
                (   (          $all_replaceable
                            || $d eq 'common'
                            || $d eq 'navimaker'
                    ) ? 1 : 0
                )
            );
        }
        elsif ( $d =~ s/\.bak$// ) {
            push( @restore, $d );
        }
    }
    return unless $this->set_dir($target_path);
    my $error = 0;
    foreach my $bak (@restore) {
        my ( %db, $db_path, $bak_path, $skip, $is_replaceable );
        $db_path        = $target_path . '/' . $bak;
        $bak_path       = $backup_path . '/' . $bak . '.bak';
        $is_replaceable = (
                   $abs_replaceable
                || ( $bak ne 'general' && $all_replaceable )
                || ( $bak eq 'page'
                && ( $page_replaceable || $all_replaceable ) )
                || ( $bak =~ /field$/
                && ( $field_replaceable || $all_replaceable ) )
        ) ? 1 : 0;
        if ( !$is_replaceable ) {
            tie( %db, $env::dbm_file, $db_path, O_RDONLY, $env::dbm_mode );
            $skip = 1 if ref tied %db;
            untie %db;
        }
        if ( !$skip ) {
            tie( %db, $env::dbm_file, $db_path, O_RDWR | O_CREAT,
                $env::dbm_mode );
            if ( ref tied %db ) {
                if ( $bak eq 'navi_db' ) {
                    while ( my ( $k, $v ) = each %db ) {
                        utf8::decode($k) if !utf8::is_utf8($k);
                        utf8::decode($v) if !utf8::is_utf8($v);
                        $this->{'old_navi'}{$k} = $v
                            if $k =~ /_(default|tabcolor|tabfontcolor|icon)$/;
                    }
                }
                %db = ()
                    if $is_replaceable
                    && $backup_path !~ m!/navimaker$!
                    && $bak ne 'deco'
                    && $bak !~ /_icon$/
                    && ( $abs_replaceable || $bak ne 'page' );
                my $data = $this->read_file($bak_path);
                if ($data) {
                    while ( $data
                        =~ /([^\x09\x0A\x0D]+)\x09([^\x0A\x0D]*)[\x0A\x0D]*/sg
                        )
                    {
                        my ( $k, $v ) = ( $1, $2 );
                        $k =~ s/\&#09;/\x09/gs;
                        $k =~ s/\x0B/\x0A/gs;
                        utf8::encode($k) if utf8::is_utf8($k);
                        if ( $bak eq 'record_id' && $k eq 'last' ) {
                            $v = '';
                        }
                        else {
                            $v =~ s/\&#09;/\x09/gs;
                            $v =~ s/\x0B/\x0A/gs;
                            $v =~ s/^_/_$home_dir_id/
                                if $home_dir_id
                                && $bak eq 'navi_db'
                                && $k =~ /_id$/;
                            $v =~ s!<user-directory>!$user_directory!gs;
                            $v
                                =~ s!<user-directory-encoded>!$user_directory_encoded!gs;
                            $v =~ s!<home-url>!$home_url!gs;
                            $v =~ s/<home-url-encoded>/$home_url_encoded/gs;
                            $v =~ s!<home-dir-id>!$home_dir_id!gs;
                            $v = $this->replace_text( $v, $conv_table )
                                if $conv_table;
                            utf8::encode($v) if utf8::is_utf8($v);
                        }
                        eval '$db{$k} = $v;';
                        $error++ if defined $@ && $@;
                    }
                }
            }
            untie %db;
        }
    }
    push(
        @{ $this->{'dialog'} },
        '容量が大き過ぎる'
            . $error
            . '件の設定復元に失敗しました。'
    ) if $error;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's602', 'SiteTemplateApplier' );

    #メニュー登録
    $setting::jump_menu{'s602'} = 'サイトテンプレート適用';

}

1;
