package SiteTemplateMaker;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# サイトテンプレート作成モジュール
# SiteTemplateMaker.pm ver1.1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2013-09-19
######################################################################
# 無視するフォルダ
@SiteTemplateMaker::skip = (
    'cgi/imageLib',              'cgi/dummy\\.html',
    'cgi/dummy4href\\.html',     'cgi/servercheck\\.cgi',
    'image/top/[0-9]+',          'image/top/i',
    'image/top/media',           'image/top/archives\\.pending',
    'image/top/digital-content', 'error',
    'favicon\\.[0-9a-zA-Z_]+',   'icon',
    'qr.png',                    'qr_mini.png',
);

# 上書きする環境設定
@SiteTemplateMaker::replaceable_env = (
    'register_in_same_window', 'tax_rate',
    'tax_frac',                'tax_included',
    'checkout',                'checkout_window_type',
    'checkout_window_width',   'checkout_window_height',
    'checkout_window_top',     'checkout_window_left',
    'doctype',                 'html_header',
    'inline_navi',             'session_enabled',
    'session_security',        'google_analytics_embedded',
    'spacer',                  'image_dir',
    'css',                     'javascript',
    'externalfiles',
);
######################################################################
# 実行
sub run ($) {
    my $this       = shift;
    my $error_flag = 0;
    $this->get_setup_env;
    if ( $this->is_not_standard_env ) {
        $this->jump_menu('tools');
        $this->print_page_header('tools');
        print
            '<h1>サイトテンプレート作成</h1><p>このディレクトリは CGI の動作に必要ないくつかのファイルまたはディレクトリが標準的な位置にないので、この機能は使えません。</p>';
        $this->print_page_footer('tools');
        exit;
    }
    my $site_template_dir;
    if ($env::user_site_template_dir) {
        if ( $this->set_dir($env::user_site_template_dir) ) {
            $site_template_dir = $env::user_site_template_dir;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                'サイトテンプレート保存ディレクトリを作成できませんでした。'
            );
            undef $main::a;
        }
        undef $main::closed;
    }
    else {
        $site_template_dir = $env::template_dir . '/site';
    }

    my @targetable_directories
        = ($env::subdirectory_disabled)
        ? ()
        : $this->targetable_directories(
        $default::cgi_data_root . ( $default::home_dir || '' ) );
    if (@targetable_directories) {
        no warnings qw(numeric);
        @targetable_directories
            = sort { $a <=> $b || $a cmp $b } @targetable_directories;
    }

    #入力チェック
    $main::tempname =~ s/[\x0A\x0D\x09]+//gs if $main::tempname;
    $main::tempid = $this->z_to_h( $main::tempid, 'utf8' ) if $main::tempid;
    $main::maxsize = 1
        if !$main::maxsize
        || $main::maxsize =~ /[^0-9]/
        || $main::maxsize < 0
        || $main::maxsize > 1000;
    if ( defined $main::a && $main::a ) {
        my $tempid;
        if ($main::target_dir
            && (  !@targetable_directories
                || $main::target_dir !~ /^\//
                || $main::target_dir =~ /\/\.\.\//
                || !grep( $_ eq $main::target_dir, @targetable_directories )
                || !-d $default::cgi_data_root
                . $default::home_dir
                . $main::target_dir )
            )
        {
            push(
                @{ $this->{'dialog'} },
                'ディレクトリ指定を確認してください。'
            );
            $main::target_dir = '';
            $error_flag       = 1;
        }
        if ( !defined $main::tempid || $main::tempid eq '' ) {
            push(
                @{ $this->{'dialog'} },
                'サイトテンプレートフォルダ名を入力してください。'
            );
            $error_flag = 1;
        }
        elsif ( $main::tempid !~ /^[a-zA-Z][a-zA-Z0-9]{0,32}$/ ) {
            push(
                @{ $this->{'dialog'} },
                'サイトテンプレートフォルダ名は半角英字で始まる32文字以内の半角英数字列にしてください。'
            );
            $error_flag = 1;
        }
        else {
            $tempid = $main::tempid
                . ( ($main::closed) ? $this->directory_suffix : '' );
            if ( -d $site_template_dir . '/' . $tempid ) {
                if ( -f $site_template_dir . '/' . $tempid . '/locked.txt' ) {
                    push(
                        @{ $this->{'dialog'} },
                        'このサイトテンプレートフォルダ名は既に使用されています。また、同名サイトテンプレートフォルダを上書きすることが禁止されています。'
                    );
                    $error_flag = 1;
                }
                elsif ( !$main::replaceable ) {
                    push(
                        @{ $this->{'dialog'} },
                        'このサイトテンプレートフォルダ名は既に使用されています。既にある同名サイトテンプレートを上書きする場合は「同名のサイトテンプレートフォルダを上書きする」チェックボックスをチェックしてください。'
                    );
                    $error_flag = 1;
                }
                else {
                    eval 'use File::Path';
                    rmtree( $site_template_dir . '/' . $tempid );
                }
            }
        }

        #サイトテンプレート作成
        if ( !$error_flag ) {
            my $home_dir       = ( $default::home_dir || '' );
            my $home_dir_id    = $this->get_home_dir_id;
            my $user_directory = $this->home_directory(1);
            my $home_url       = $this->full_path( '../../', 1 );
            my $temp_path      = $site_template_dir . '/' . $tempid;
            my $pref_dir       = $default::preferences;
            my $data_dir       = $default::data;
            my %env            = $this->get_env($main::target_dir);
            if ($main::wholesite) {
                @SiteTemplateMaker::skip
                    = grep( $_ ne 'error' && $_ ne 'favicon\\.[0-9a-zA-Z_]+',
                    @SiteTemplateMaker::skip );
                push(
                    @SiteTemplateMaker::replaceable_env,
                    'title',                     'locale',
                    'copyright',                 'description',
                    'rss_marker',                'window_title_format',
                    'openid_enabled',            'password_threshold',
                    'password_latency',          'rss_maxitem',
                    'rss_description_maxlength', 'google_analytics_uacct',
                    'amazon_access_key',         'amazon_secret_access_key',
                    'amazon_associates_id',      'cookie_lifetime',
                    'noindex',
                );
            }
            my @skip
                = ($main::skiplist)
                ? split( /[\x09\x0A\x0C\x0D\x20]+/,
                $this->z_to_h( $main::skiplist, 'utf8' ) )
                : ();
            push( @skip, @SiteTemplateMaker::skip );
            if ($main::target_dir) {
                $home_dir .= $main::target_dir;
                $home_url .= $main::target_dir;
                my $http_home_dir = $home_dir;
                my ($multi_domain, $server_name, $ssl)
                    = $this->subdir_options( $default::cgi_data_root
                        . $default::home_dir
                        . $main::target_dir
                        . '/lib/env.pl' );
                if ($multi_domain) {
                    $http_home_dir =~ s!^/[^/]+!!;
                    $user_directory = $http_home_dir;
                    %env
                        = $this->get_env( $main::target_dir, $http_home_dir );
                    $home_url
                        = ( ( $ssl ) ? 'https://' : 'http://' )
                        . $server_name
                        . $http_home_dir;
                }
                else {
                    $user_directory .= $main::target_dir;
                    %env = $this->get_env($main::target_dir);
                }
                $home_dir_id = $this->get_home_dir_id($http_home_dir);
                $pref_dir =~ s!/preferences$!$main::target_dir/preferences!;
                $data_dir =~ s!/data$!$main::target_dir/data!;
                @skip
                    = map( '^\\.\\./\\.\\.'
                        . quotemeta($main::target_dir) . '/'
                        . $_ . '$',
                    @skip );
            }
            else {
                @skip = map( '^\\.\\./\\.\\./' . $_ . '$', @skip );
                $main::target_dir = '';
                %env              = $this->get_env($main::target_dir);
            }

            #サイトテンプレートディレクトリ作成
            $this->set_dir($temp_path);

            #上書き環境設定保存
            $this->mk_env_bak( $temp_path . '/env.bak', \%env );

            #公開ディレクトリテンプレート作成
            $this->mk_htdocs(
                $temp_path . '/htdocs', $main::target_dir,
                $home_dir_id,           $user_directory,
                $home_url,              \%env,
                \@skip,                 $main::wholesite,
                $main::maxsize
            );

            #preferenceディレクトリ作成
            eval 'use ' . $env::dbm_file . ';';
            $this->mk_bak( $pref_dir, $temp_path . '/preferences',
                $home_dir_id, $user_directory, $home_url, $main::noaddr,
                $main::wholesite, 1 );

            #dataディレクトリ作成
            if ($main::wholesite) {
                $this->mk_bak( $data_dir, $temp_path . '/data',
                    $home_dir_id, $user_directory, $home_url, $main::noaddr,
                    1 );
            }
            else {
                $this->mk_data_dir( $data_dir, $temp_path . '/data' );
            }

            #サイトテンプレート名保存
            $this->write_file(
                $temp_path . '/name.txt',
                $main::tempname
                    . (
                    ($main::wholesite)
                    ? '[記事・データを含む]' . "\x09" . '1'
                    : ''
                    ),
                1
            ) if defined $main::tempname && $main::tempid ne 'default';

            my $temp_label
                = $main::tempid
                . ( ($main::closed)
                ? ($home_dir)
                        ? '@~' . $this->escape($home_dir)
                        : '@~/'
                : '' );
            push(
                @{ $this->{'dialog'} },
                'このディレクトリを元にサイトテンプレート「'
                    . (
                    ( $main::tempid eq 'default' )
                    ? 'デフォルトのサイトテンプレート'
                    : ($main::tempname) ? $this->escape($main::tempname) . '('
                        . $temp_label . ')'
                    : $temp_label
                    )
                    . '」を作成しました。'
            );
            undef $main::tempid;
            undef $main::tempname;
            undef $main::replaceable;
            my %general;
            tie( %general, $env::dbm_file,
                $default::preference_dir . '/general',
                O_RDWR, $env::dbm_mode );

            if ( ref tied %general ) {
                $general{'STMaxsize'} = $this->encode_utf8($main::maxsize)
                    if defined $main::maxsize;
                $general{'STSkipList'} = $this->encode_utf8($main::skiplist)
                    if defined $main::skiplist;
                $general{'STNoAddr'}    = ($main::noaddr)    ? 1 : '';
                $general{'STWholeSite'} = ($main::wholesite) ? 1 : '';
                $general{'STClosed'}    = ($main::closed)    ? 1 : '';
            }
            untie %general;
        }
    }
    else {
        $main::maxsize = $setting::general{'STMaxsize'}
            if defined $setting::general{'STMaxsize'};
        $main::skiplist = $setting::general{'STSkipList'}
            if defined $setting::general{'STSkipList'};
        $main::noaddr    = ( $setting::general{'STNoAddr'} )    ? 1 : '';
        $main::wholesite = ( $setting::general{'STWholeSite'} ) ? 1 : '';
        $main::closed    = ( $setting::general{'STClosed'} )    ? 1 : '';
    }

    #ページヘッダ
    if ( -f $site_template_dir . '/default/locked.txt' ) {
        $this->set_javascript(
            <<'SCRIPT'
cmonos.validate.asTemplateId = function () {
	cmonos.validate.field().bailOut();
	if (this.currentField.value && this.currentField.value == "default") {
		alert('この名前でテンプレートを作成することはできません。');
		this.currentField.focus(); 
		this.currentField.select(); 
		this.isInvalid();
	} else if (this.currentField.value && (this.currentField.value.search(/^[a-zA-Z0-9]+$/) == -1)) {
		alert('半角英数字にしてください。');
		this.currentField.focus(); 
		this.currentField.select(); 
		this.isInvalid();
	}
}
SCRIPT
        );
    }
    else {
        $this->set_javascript(
            <<'SCRIPT'
cmonos.validate.asTemplateId = function () {
	this.bailOut();
	var template_name_field = document.getElementById('tempname');
	if (this.currentField.value && this.currentField.value == "default") {
		if (confirm('この名前でテンプレートを作成するとデフォルトの設定が上書きされます。')) {
			template_name_field.value = "";
			template_name_field.disabled = true;
		} else {
			this.currentField.value = "";
			this.currentField.focus(); 
			this.currentField.select(); 
		}
	} else {
		template_name_field.disabled = false;
		if (this.currentField.value && (this.currentField.value.search(/^[a-zA-Z0-9]+$/) == -1)) {
			alert("半角英数字にしてください。");
			this.currentField.focus(); 
			this.currentField.select(); 
			this.isInvalid();
		}
	}
}
SCRIPT
        );
    }
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サイトテンプレート作成</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted();',
        'required' => \@setting::form
    );
    if ( $this->{'ignored_files'} ) {
        print
            '<fieldset><legend>下記ファイルは容量が大き過ぎるので無視されました。</legend><ol>';
        print $this->{'ignored_files'};
        print '</ol></fieldset>';
        delete $this->{'ignored_files'};
    }

    print '<input type="hidden" name="a" value="1" />';
    print '<fieldset>';
    print '<table class="field-table"><tbody>';
    if (@targetable_directories) {
        print '<tr id="target-directory">';
        print
            '<td class="field-name"><label for="target_dir">対象ディレクトリ</label></td>';
        print '<td class="field">';
        print
            '<select name="target_dir" id="target_dir" onchange="document.getElementById(\'samplepage\').href = \'../..\'+this.value+\'/\'">';
        print '<option value="">このディレクトリ</option>';
        foreach my $dir (@targetable_directories) {
            my $d = $dir;
            $d =~ s!^/!!;
            print '<option value="'
                . $dir . '"'
                . ( ( defined $main::target_dir && $main::target_dir eq $dir )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $d
                . '</option>';
        }
        print '</select> <a href="../..'
            . ( $main::target_dir || '' )
            . '/" id="samplepage" target="sample">[確認]</a>';
        print '</td>';
        print '</tr>';
    }
    print '<tr>';
    print
        '<td class="field-name"><label for="tempid">サイトテンプレートフォルダ名</label></td>';
    print '<td><input type="text" name="tempid" id="tempid" size="20" value="'
        . $this->escape($main::tempid)
        . '" onchange="cmonos.validate.field(this).asTemplateId()" />（半角英数字）</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="tempname">サイトテンプレート名</label></td>';
    print
        '<td class="field"><input type="text" name="tempname" id="tempname" size="40" value="'
        . $this->escape($main::tempname)
        . '" /></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="skiplist">無視するフォルダのリスト</span></td>';
    print
        '<td class="field"><textarea name="skiplist" id="skiplist" cols="40" rows="4" wrap="hard">'
        . $this->escape($main::skiplist)
        . '</textarea><div class="hints">フォルダごとに[改行]</div></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">ファイル容量制限</span></td>';
    print
        '<td class="field"><label>容量が <select name="maxsize" id="maxsize">';

    foreach my $n ( 1 .. 10, 100, 500, 1000 ) {
        print '<option value="'
            . $n . '"'
            . ( ( $main::maxsize && $main::maxsize == $n )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $n
            . '</option>';
    }
    print '</select> MB 以上のファイルを無視する。</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">上書き設定</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="replaceable" id="replaceable" value="1"'
        . ( ($main::replaceable) ? ' checked="checked"' : '' )
        . ' /> 同名のサイトテンプレートフォルダを上書きする。</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">初期設定保存設定</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="noaddr" id="noaddr" value="1"'
        . ( ($main::noaddr) ? ' checked="checked"' : '' )
        . ' /> メールアドレスおよび管理人名を初期設定から削除する。</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">記事/データ</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="wholesite" id="wholesite" value="1"'
        . ( ($main::wholesite) ? ' checked="checked"' : '' )
        . ' /> サイトテンプレートに記事およびデータを含む。</label></td>';
    print '</tr>';

    if ( !$env::user_site_template_dir ) {
        print '<tr>';
        print
            '<td class="field-name"><span class="datatitle">公開設定</span></td>';
        print
            '<td class="field"><label><input type="checkbox" name="closed" id="closed" value="1"'
            . ( ($main::closed) ? ' checked="checked"' : '' )
            . ' /> 下位ディレクトリからこのサイトテンプレートを隠す。</label></td>';
        print '</tr>';
    }
    print '</tbody></table></fieldset>';
    print
        '<fieldset class="submit"><input type="submit" value="現在のディレクトリを元にサイトテンプレートを作成する" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

sub targetable_directories ($$) {
    my ( $this, $dir, $home_dir ) = @_;
    my ( @directories, @sub_directories, $entry, $d );
    @directories = @sub_directories = ();
    local (*DIR);
    return unless opendir( DIR, $dir );
    $home_dir = '' if !defined $home_dir;
    while ( $entry = readdir(DIR) ) {
        next if $entry =~ /^\./;
        my $path = $dir . '/' . $entry;
        if (   -d $dir . '/' . $entry
            && -f $dir . '/' . $entry . '/lib/env.pl' )
        {
            push( @directories, $entry );
        }
    }
    closedir(DIR);
    foreach $d (@directories) {
        push(
            @sub_directories,
            $home_dir . '/' . $d,
            $this->targetable_directories(
                $dir . '/' . $d,
                $home_dir . '/' . $d
            )
        );
    }
    return @sub_directories;
}

sub get_env ($;$$) {
    my ( $this, $target_dir, $http_home_dir ) = @_;
    my @param = (
        'title', 'description', 'ssl', 'javascript', 'icon_dir',
        'inline_navi', @SiteTemplateMaker::replaceable_env
    );
    my %env = ();
    if ($target_dir) {
        my $data = $this->read_file(
            $default::cgi_data_root
                . $default::home_dir
                . $target_dir
                . '/lib/env.pl',
            1
        );
        foreach my $p (@param) {
            if (   $p eq 'html_header'
                && $data
                =~ /\$$p[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*(.*?[^\\]');/s )
            {
                $env{$p} = eval $1;
            }
            elsif ( $data
                =~ /^[\x20\x09]*\$$p[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/m
                )
            {
                $env{$p} = eval $1;
            }
        }
        $env{'javascript_dir'} = $this->un_base_change(
            $env{'javascript'},
            ( $http_home_dir || '../..' . $target_dir )
                . '/cgi/setup/start.cgi',
            'abs'
        );
        $env{'icon_dir_path'} = $this->un_base_change( $env{'icon_dir'},
            ( $http_home_dir || '../..' . $target_dir )
                . '/cgi/setup/start.cgi' );
    }
    else {
        no strict "refs";
        foreach my $p (@param) {
            $env{$p} = ${ 'env::' . $p };
        }
        $env{'javascript_dir'}
            = $this->path_change( $env::javascript, $ENV{'SCRIPT_NAME'},
            'abs' );
        $env{'icon_dir_path'} = $env{'icon_dir'};
    }
    return %env;
}

#上書き環境設定保存
sub mk_env_bak ($$$) {
    my ( $this, $env_bak_path, $env ) = @_;
    local (*BAK);
    if ( $env_bak_path && open( BAK, ">>$env_bak_path" ) ) {
        binmode BAK, ":utf8";
        flock( BAK, LOCK_EX );
        truncate( BAK, 0 );
        seek( BAK, 0, 0 );
        foreach my $p (@SiteTemplateMaker::replaceable_env) {
            my $v = $env->{$p} || '';
            $v =~ s/[\x0A\x0D]/\x0B/sg;
            print BAK $p;
            print BAK "\x09";
            print BAK $v;
            print BAK "\x0A";
        }
        close(BAK);
        chmod( $env::file_mode, $env_bak_path );
    }
}

sub mk_htdocs ($$$$$$$;$$$) {
    my ($this,           $htdocs_path, $target_dir, $home_dir_id,
        $user_directory, $home_url,    $env,        $skip,
        $wholesite,      $maxsize
    ) = @_;
    my ( $target_htdocs, $cgi_data, $cgi_data_root, $cgi_data_path, @files,
        @cgi_folders, @sub_dir );
    local (*DIR);
    $target_htdocs = '../..' . ( $target_dir || '' ) . '/';
    return
        unless $this->set_dir($htdocs_path) && opendir( DIR, $target_htdocs );
    $cgi_data_path = $default::cgi_data;
    $cgi_data      = $default::cgi_data;
    $cgi_data_root = $default::cgi_data_root;

    if ($target_dir) {
        $cgi_data_path .= $target_dir;
        my $rel = '../' x ( $target_dir =~ tr!/!/! );
        $cgi_data      = $rel . $cgi_data . $target_dir;
        $cgi_data_root = $rel . $cgi_data_root;
    }
    if ($wholesite) {
        @files = grep( $_ !~ /^\.\./, readdir(DIR) );
    }
    else {
        @files = grep( $_ !~ /^\./, readdir(DIR) );
    }
    closedir(DIR);
    if ( opendir( DIR, $cgi_data_path . '/preferences' ) ) {
        @cgi_folders
            = grep( $_ !~ /^\./ && -d $cgi_data_path . '/preferences/' . $_,
            readdir(DIR) );
        closedir(DIR);
    }
    if ( opendir( DIR, $cgi_data_path ) ) {
        @sub_dir
            = grep( $_ !~ /^\./
                && -f $cgi_data_path . '/' . $_ . '/lib/env.pl',
            readdir(DIR) );
        closedir(DIR);
    }
    $maxsize = $maxsize * 1000000;
    foreach my $file (@files) {
        my $path = $target_htdocs . $file;
        if (   grep( $path =~ /$_/, @$skip )
            || ( @sub_dir && grep( $file eq $_, @sub_dir ) )
            || $file eq 'cgi'
            || $file eq 'lib'
            || $path eq $default::cgi_data_root
            || $path eq $default::shared_cgi_data_root )
        {
            next;
        }
        elsif ( -f $path ) {
            if ( $file eq 'm.cgi' ) {
                my $data = $this->read_file( $path, 1 );
                my $cgi_data_path
                    = $this->base_change( $cgi_data_path, $path, 'rel', 1 );
                $data
                    =~ s!use lib ".*?/lib"!use lib "<shared-cgi-data-root-from-top>/lib"!g;
                $data =~ s/$cgi_data_path/<cgi-data-path-from-top>/g;
                $this->write_file( $htdocs_path . '/' . $file, \$data, 1 );
            }
            elsif ($wholesite) {
                if ( ( -s $path ) > $maxsize ) {
                    $this->{'ignored_files'} .= '<li>'
                        . $this->path_change( $path, $ENV{'SCRIPT_NAME'},
                        'abs' )
                        . '</li>';
                }
                elsif ( $file eq '.htaccess' ) {
                    my $data = $this->read_file( $path, 1 );
                    if ( $data =~ /# Gate Keeper - start -/ ) {
                        $data
                            =~ s!(AuthUserFile[\x09\x20]+)[^\x0A]*(/\.htpasswd)!$1$2!ig;
                        $data
                            =~ s!(AuthGroupFile[\x09\x20]+)[^\x0A]*(/\.htgroup)!$1$2!ig;
                    }
                    if ( $data =~ /# For Mobile - start -/ ) {
                        $data
                            =~ s![^\x09\x0A\x0C\x0D\x20]*/m\.cgi/!/m\.cgi/!sg;
                    }
                    $this->write_file( $htdocs_path . '/' . $file, \$data,
                        1 );
                }
                elsif ($file =~ /\.(s?html?|ssi|xml|kml|dat|js|css)$/i
                    && $file
                    !~ /[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.dat$/ )
                {
                    $this->write_file(
                        $htdocs_path . '/' . $file,
                        $this->replace_home_info(
                            $this->read_file( $path, 1 ),
                            $path,
                            $home_dir_id,
                            $user_directory,
                            $home_url,
                            $env->{'icon_dir_path'}
                        ),
                        1
                    );
                }
                else {
                    $this->file_copy( $path, $htdocs_path . '/' . $file );
                }
            }
            elsif (
                (      $env->{'inline_navi'}
                    && $file =~ /^(index|frame|navi|top)\.s?html?$/
                )
                || (  !$env->{'inline_navi'}
                    && $file =~ /^(frame|top)\.s?html?$/ )
                || $file =~ /\.xml$/
                )
            {
                $this->write_file( $htdocs_path . '/' . $file, "\x0A", 1 );
            }
            elsif ( ( -s $path ) > $maxsize ) {
                $this->{'ignored_files'}
                    .= '<li>'
                    . $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' )
                    . '</li>';
            }
            else {
                $this->file_copy( $path, $htdocs_path . '/' . $file );
            }
        }
        elsif ( $file !~ /^\./ && -d $path ) {
            if ( @cgi_folders && grep( $_ eq $file, @cgi_folders ) ) {
                if ($wholesite) {
                    $this->_mk_htdocs(
                        $path,          $htdocs_path . '/' . $file,
                        $file,          $user_directory,
                        $home_dir_id,   $home_url,
                        $env,           $cgi_data,
                        $cgi_data_root, $skip,
                        1,              $maxsize
                    );
                }
                else {
                    $this->set_dir( $htdocs_path . '/' . $file );
                    $this->write_file(
                        $htdocs_path . '/' . $file . '/.placeholder',
                        "", 1 );
                }
            }
            else {
                my @sizeover = ();
                $this->directory_copy( $path, $htdocs_path . '/' . $file,
                    1, $skip, $maxsize, \@sizeover, 1 );
                $this->{'ignored_files'} .= join(
                    '',
                    map( '<li>'
                            . $this->path_change( $_, $ENV{'SCRIPT_NAME'},
                            'abs' )
                            . '</li>',
                        @sizeover )
                ) if @sizeover;
            }
        }
    }
    require CMONOS::CGI_Lib::IndexSweeper;
    $this->{'index_sweeper'} = new IndexSweeper;
    $this->_mk_htdocs(
        $target_htdocs . 'cgi', $htdocs_path . '/cgi',
        'cgi',                  $user_directory,
        $home_dir_id,           $home_url,
        $env,                   $cgi_data,
        $cgi_data_root,         $skip,
        $wholesite,             $maxsize
    );
    $this->_mk_htdocs(
        $target_htdocs . 'lib', $htdocs_path . '/lib',
        'lib',                  $user_directory,
        $home_dir_id,           $home_url,
        $env,                   $cgi_data,
        $cgi_data_root,         $skip,
        $wholesite,             $maxsize
    );
    if ($wholesite) {
        my $target_htdocs_quotemeta = '^\\.\\./\\.\\.'
            . ( ($target_dir) ? quotemeta($target_dir) : '' ) . '/';
        @$skip
            = grep( $_ ne $target_htdocs_quotemeta
                . 'image/top/[0-9]+$'
                && $_ ne $target_htdocs_quotemeta
                . 'image/top/i$'
                && $_ ne $target_htdocs_quotemeta
                . 'image/top/media$'
                && $_ ne $target_htdocs_quotemeta
                . 'image/top/archives\\.pending$'
                && $_ ne $target_htdocs_quotemeta
                . 'image/top/digital-content$',
            @$skip );
        $this->_mk_htdocs(
            $target_htdocs . 'image/top', $htdocs_path . '/image/top',
            'top',                        $user_directory,
            $home_dir_id,                 $home_url,
            $env,                         $cgi_data,
            $cgi_data_root,               $skip,
            $wholesite,                   $maxsize
        );
    }
}

sub _mk_htdocs ($$$$$$$$$;$$$) {
    my ($this,           $dir,           $clone,    $folder,
        $user_directory, $home_dir_id,   $home_url, $env,
        $cgi_data,       $cgi_data_root, $skip,     $wholesite,
        $maxsize
    ) = @_;
    my ( @files, @directories );
    local (*DIR);
    return unless opendir( DIR, $dir );
    while ( my $file = readdir(DIR) ) {
        next
            if ( !$wholesite && $file =~ /^\./ )
            || ( $wholesite
            && ( $file =~ /^\.\./ || $file eq '.placeholder' ) );
        my $path = $dir . '/' . $file;
        if (   grep( $path =~ /$_/, @$skip )
            || $path eq $default::cgi_data_root
            || $path eq $default::shared_cgi_data_root )
        {
            next;
        }
        elsif ( -f $path ) {
            push( @files, $file );
        }
        elsif ($file !~ /^\./
            && -d $path
            && ( !$main::share_flag || $path !~ /pswd/ ) )
        {
            push( @directories, $file );
        }
    }
    closedir(DIR);
    return unless $this->set_dir($clone);
    my $is_cgi_folder = 0;
    foreach my $file (@files) {
        my $s = -s $dir . '/' . $file;
        if ( $s > $maxsize ) {
            $this->{'ignored_files'} .= '<li>'
                . $this->path_change( $dir . '/' . $file,
                $ENV{'SCRIPT_NAME'}, 'abs' )
                . '</li>';
            next;
        }
        elsif (
            $s > 1000000
            || (   $file =~ /\.[0-9a-zA-Z_]+$/
                && $file !~ /\.(cgi|pl|s?html?|ssi|xml|kml|dat|js|css)$/ )
            || (   $wholesite
                && $file
                =~ /[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.dat$/ )
            )
        {
            $this->file_copy( $dir . '/' . $file, $clone . '/' . $file );
            next;
        }
        my $data = $this->read_file( $dir . '/' . $file );
        if ( $file eq '.htaccess' ) {
            if ( $data =~ /# Gate Keeper - start -/ ) {
                $data
                    =~ s!(AuthUserFile[\x09\x20]+)[^\x0A]*(/\.htpasswd)!$1$2!ig;
                $data
                    =~ s!(AuthGroupFile[\x09\x20]+)[^\x0A]*(/\.htgroup)!$1$2!ig;
            }
            if ( $data =~ /# For Mobile - start -/ ) {
                $data =~ s![^\x09\x0A\x0C\x0D\x20]*/m\.cgi/!/m\.cgi/!sg;
            }
        }
        elsif ( $file =~ /\.cgi$/ ) {
            $data
                =~ s!use lib ".*?/lib"!use lib "<shared-cgi-data-root>/lib"!g;
            $data =~ s/$cgi_data/<cgi-data-path>/g;
            $data =~ s/$cgi_data_root/<cgi-data-root>/g;
            $is_cgi_folder = 1 if $file eq 'start.cgi';
        }
        elsif ( $file eq 'default.pl' ) {
            $data =~ s/$cgi_data/<cgi-data-path>/g;
            $data =~ s/$cgi_data_root/<cgi-data-root>/g;
            $data =~ s/\$id = .*?;/\$id = '<default-id>';/g;
            if ( $folder eq 'gatekeeper' ) {
                $data =~ s/\$target = .*?;/\$target = '<web-home-dir>';/g;
            }
            elsif ( $folder eq 'station' ) {
                $data =~ s/\$home_dir = .*?;//g;
            }
            elsif ( $folder eq 'setup' ) {
                $data =~ s/\$home_dir = .*?;/\$home_dir = '<home-dir>';/g;
                $data
                    =~ s!\$preferences_ftp = .*?;!\$preferences_ftp = \$cgi_data_name . '<home-dir>/preferences';!g;
                $data
                    =~ s!\$data_ftp = .*?;!\$data_ftp = \$cgi_data_name . '<home-dir>/data';!g;
                $data
                    =~ s!\$zipcode_ftp = .*?;!\$zipcode_ftp = \$cgi_data_name . '<home-dir>/zipcode';!g;
                $data
                    =~ s!\$cgi_data_name = .*?;!\$cgi_data_name = 'cgi-data';!g;
                $data
                    =~ s!\$shared_cgi_data_name = .*?;!\$cgi_data_name = 'shared_cgi_data_name';!g;
            }
        }
        elsif ( $file eq 'cmonos.js' ) {
            $data =~ s!$env->{'javascript_dir'}!/lib/javascript!g;
            if ($home_dir_id) {
                $data =~ s/parent\.${home_dir_id}main/parent\.main/g;
                $data =~ s/parent\.${home_dir_id}navi/parent\.navi/g;
            }
        }
        elsif ( ( $file =~ /^index\.s?html?$/ || $file eq 'frame.html' )
            && $data =~ /<frameset/ )
        {
            if ($home_dir_id) {
                $data =~ s/ name="${home_dir_id}main"/ name="main"/g;
                $data =~ s/ name="${home_dir_id}navi"/ name="navi"/g;
            }
        }
        elsif (
            (   $wholesite
                && ( $file =~ /\.(s?html?|ssi|xml|kml|dat|css)$/i
                    || ( $file =~ /\.js$/i && $dir !~ m!/lib/javascript! ) )
            )
            || $file =~ /^index\.s?html?$/
            || $file =~ /^site-navi(-sjis|-euc|)\.(ssi|js)$/
            || $file eq 'site-navi.css'
            || $file
            =~ /^(metaindex|recentcomments|recenttrackbacks)\.(ssi|js)$/
            || $dir =~ m!/lib/xfiles$!
            )
        {
            if ( $file =~ /\.js$/ ) {
                next;
            }
            elsif (
                $file =~ /\.ssi$/
                && ( $file
                    =~ /^(metaindex|recentcomments|recenttrackbacks)\.ssi/
                    || $data
                    =~ /<!--([0-9a-zA-Z_]+)(?![0-9a-zA-Z_])[^>]*-->.*<!--\/\1-->/s
                )
                )
            {
                $data = $this->{'index_sweeper'}->_mk_template_from($data)
                    if !$wholesite;
                $data = $this->replace_home_info( $data, $dir . '/' . $file,
                    $home_dir_id, $user_directory, $home_url,
                    $env->{'icon_dir_path'} );
            }
            elsif ( !$wholesite && $file =~ /^site-navi-?(sjis|euc|)\.ssi$/ )
            {
                my $ocode       = $1;
                my $title       = $env->{'title'};
                my $description = $env->{'description'};
                unless ( $data
                    =~ s/(<[0-9a-zA-Z_]+ id="site-title"><a [^>]*>).*?(<\/a><\/[0-9a-zA-Z_]+>)/$1<site-title>$2/
                    || $data
                    =~ s/(<p class="siteName" id="siteName"><a [^>]*>).*?(<\/a><\/p>)/$1<site-title>$2/
                    )
                {
                    $data =~ s/$title/<site-title>/
                        if defined $title && $title ne '';
                }
                unless ( $data
                    =~ s/(<p id="site-subtitle">).*?(<\/p>)/$1<site-description>$2/
                    || $data
                    =~ s/<p class="description">.*?<\/p>/<p class="description"><site-description><\/p>/
                    )
                {
                    $data =~ s/$description/<site-description>/
                        if defined $description && $description ne '';
                }
                $data
                    =~ s!<script src="http://www.google.com/jsapi.*?writeGSearch\(\);.*?</script>[\x09\x0A\x0C\x0D\x20]*!!is;
                $data = $this->replace_home_info( $data, $dir . '/' . $file,
                    $home_dir_id, $user_directory, $home_url,
                    $env->{'icon_dir_path'} );
                if ($ocode) {
                    $this->jcode_convert( \$data, $ocode, 'utf8' );
                }
            }
            else {
                $data
                    =~ s!(\x09)\Q$user_directory\E(?=/image/imagemenu/)!$1<user-directory>!g
                    if $file =~ /^[0-9]+\.dat$/;
                $data = $this->replace_home_info( $data, $dir . '/' . $file,
                    $home_dir_id, $user_directory, $home_url,
                    $env->{'icon_dir_path'} );
            }
        }
        elsif ( $file =~ /\.(s?html?|ssi|xml|kml|dat|css)$/i ) {
            $data
                =~ s!(\x09)\Q$user_directory\E(?=/image/imagemenu/)!$1<user-directory>!g
                if $file =~ /^[0-9]+\.dat$/;
            $data = $this->replace_home_info( $data, $dir . '/' . $file,
                $home_dir_id, $user_directory, $home_url,
                $env->{'icon_dir_path'} );
        }
        $this->write_file( $clone . '/' . $file, \$data, 1 );
    }
    if (@directories) {
        foreach my $directory (@directories) {
            if ( !$wholesite && $is_cgi_folder && $directory eq 'files' ) {
                $this->set_dir( $clone . '/files' );
                $this->file_copy(
                    $dir . '/files/event-category.css',
                    $clone . '/files/event-category.css'
                ) if -f $dir . '/files/event-category.css';
                $this->write_file(
                    $clone . '/files/index.html',
                    '<html><head><meta http-equiv="Refresh" content="0;URL=../start.cgi" /></head><body></body></html>',
                    1
                );
            }
            else {
                $this->_mk_htdocs(
                    $dir . '/' . $directory, $clone . '/' . $directory,
                    $directory,              $user_directory,
                    $home_dir_id,            $home_url,
                    $env,                    $cgi_data,
                    $cgi_data_root,          $skip,
                    $wholesite,              $maxsize
                );
            }
        }
    }
}

sub replace_home_info ($$$$$$$) {
    my ( $this, $data, $path, $home_dir_id, $user_directory, $home_url,
        $icon_dir )
        = @_;
    my $meta_home_dir_id = $home_dir_id;
    $meta_home_dir_id =~ s/_/-/g;
    my $home_url_encoded       = $this->url_encode($home_url);
    my $user_directory_encoded = $this->url_encode($user_directory);
    my $home_site              = $home_url;
    $home_site =~ s!^https?://!!i;
    $home_site =~ s!/$!!;
    my $home_domain
        = $env::server_name || ( ( $home_site =~ /^([\/:]+)/ ) ? $1 : '' );
    $icon_dir = $this->base_change( $icon_dir, $path );
    $data =~ s/\Q$home_url\E/<home-url>/gs;
    $data =~ s/\Q$home_url_encoded\E/<home-url-encoded>/gs;
    $data
        =~ s/(itpc:\/\/|feed:\/\/|['"])\Q$home_site\E(?=[\/'"])/$1<home-site>/gs;
    $data =~ s/'\Q$home_domain\E(?=[\/'])/'<home-domain>/gs;
    $data
        =~ s!([\x09\x0A\x0C\x0D\x20]+src=["'])\Q$icon_dir\E(?=/)!$1<icon-dir>!gi;
    $data
        =~ s!($user_directory|\.)(/lib/(?:xfiles|javascript|css)/)!(($1 && $1 eq '.') ? $1 . $2 : '<user-directory>' . $2)!ge;

    if ($home_dir_id) {
        $data =~ s!\Q$user_directory\E(?=["'/])!<user-directory>!gs;
        $data
            =~ s!\Q$user_directory_encoded\E(?=%2F)!<user-directory-encoded>!gis;
        1 while $data
            =~ s!((?:<home-url>|<user-directory>|\.)(?:/\w+)*)<user-directory>!$1$user_directory!gs;
        $data =~ s/#document_$home_dir_id/#document_<home-dir-id>/g;
        $data
            =~ s/(<body(?![0-9a-zA-Z_])[^>]*id="document_)$home_dir_id([0-9a-zA-Z_]+"[^>]*>)/$1<home-dir-id>$2/gsi;
        $data =~ s/(a[0-9]{8}_)$home_dir_id/$1<home-dir-id>/g;
        $data
            =~ s/(<!--article(?![0-9a-zA-Z_])[^>]*id=")$meta_home_dir_id([^"]+"[^>]*-->)/$1<meta-home-dir-id>$2/g;
        $data
            =~ s/(<!--index(?![0-9a-zA-Z_])[^>]*id="_)$home_dir_id([^"]+"[^>]*-->)/$1<home-dir-id>$2/g;
        $data
            =~ s/(&lt;!--article(?![0-9a-zA-Z_]).*?id=&quot;)$meta_home_dir_id(.*?&quot;.*?--&gt;)/$1<meta-home-dir-id>$2/g;
        $data
            =~ s/(&lt;!--index(?![0-9a-zA-Z_]).*?id=&quot;_)$home_dir_id(.*?&quot;.*?--&gt;)/$1<home-dir-id>$2/g;
    }
    else {
        $data =~ s/#document_(?!_)/#document_<home-dir-id>/g;
        $data
            =~ s/(<body(?![0-9a-zA-Z_])[^>]*id="document_(?!_))([0-9a-zA-Z_]+"[^>]*>)/$1<home-dir-id>$2/gsi;
        $data =~ s/(a[0-9]{8}_)([0-9a-zA-Z_]+)/$1<home-dir-id>$2/g;
        $data
            =~ s/([\x09\x0A\x0C\x0D\x20]+(?:src|href|background|virtual|action)=(?:&quot;|["'])(?!\/\/))\Q$user_directory\E(?=(?:[\/"']|&quot;))/$1<user-directory>/gi;
        $data
            =~ s/([\x09\x0A\x0C\x0D\x20]+(?:path|thumbnail)=(?=%2F))\Q$user_directory_encoded\E/$1<user-directory-encoded>/gi;
        $data
            =~ s!(<option value=")\Q$user_directory\E(?=/image/imagemenu/)!$1<user-directory>!gi;
        $data
            =~ s!(['"])\Q$user_directory\E(?=/cgi/\w+/\w+\.cgi)!$1<user-directory>!gi;
        $data
            =~ s/(<!--article(?![0-9a-zA-Z_])[^>]*id=")([0-9a-zA-Z_]+-[^"]+"[^>]*-->)/$1<meta-home-dir-id>$2/g;
        $data
            =~ s/(<!--index(?![0-9a-zA-Z_])[^>]*id="_)([0-9a-zA-Z_]+"[^>]*-->)/$1<home-dir-id>$2/g;
        $data
            =~ s/(&lt;!--article(?![0-9a-zA-Z_]).*?id=&quot;)([0-9a-zA-Z_]+-[0-9a-zA-Z_\-]*?&quot;.*?--&gt;)/$1<meta-home-dir-id>$2/g;
        $data
            =~ s/(&lt;!--index(?![0-9a-zA-Z_]).*?id=&quot;_)([0-9a-zA-Z_]+&quot;.*?--&gt;)/$1<home-dir-id>$2/g;
    }
    return $data;
}

sub _path_conv_for_temp ($$) {
    my ( $this, $str ) = @_;
    $str =~ s!'/!'<home-dir>/!sg;
    return $str;
}

######################################################################
# 初期設定テンプレートを作成
sub mk_bak ($$$$$$;$$$) {
    my ($this,        $db_path,        $bak_path,
        $home_dir_id, $user_directory, $home_url,
        $no_addr,     $wholesite,      $is_pref_root
    ) = @_;
    my ( @files, $db, %db, $has_db );
    return unless opendir( DIR, $db_path );
    @files = readdir(DIR);
    closedir(DIR);
    return unless $this->set_dir($bak_path);
    foreach $db (@files) {
        next
            if $db =~ /^\./
            || $db =~ /\.(bak|default|txt|html|htm|gif|jpg|png)$/i;
        if ( -d $db_path . '/' . $db ) {
            $has_db = 1;
            if (  !$wholesite
                && $db eq 'register'
                && $is_pref_root
                && $this->set_dir( $bak_path . '/' . $db ) )
            {
                $this->_mk_bak(
                    $db_path . '/' . $db . '/preference',
                    $bak_path . '/' . $db . '/preference',
                    $home_dir_id,
                    $user_directory,
                    $home_url,
                    $no_addr,
                    $wholesite
                );
            }
            else {
                $this->mk_bak(
                    $db_path . '/' . $db,
                    $bak_path . '/' . $db,
                    $home_dir_id, $user_directory, $home_url, $no_addr,
                    $wholesite
                );
            }
        }
        elsif ( -f $db_path . '/' . $db ) {
            $db =~ s/\.[^\/\.]+$//;
            $db{$db} = 1
                if $wholesite
                || ( $db ne 'pending'
                && $db ne 'terms'
                && $db ne 'url_to_title'
                && $db ne 'referer_log' );
        }
    }
    foreach $db ( keys %db ) {
        $has_db = 1
            if $this->_mk_bak(
            $db_path . '/' . $db,
            $bak_path . '/' . $db,
            $home_dir_id, $user_directory, $home_url, $no_addr, $wholesite
            );
    }
    $this->write_file(
        $bak_path . '/index.html',
        "<html><head><title>404 Not Found</title></head><body><h1>Not Found</h1> The requested object does not exist on this server.</body></html>\x0A",
        1
    ) if !$has_db;
}

sub _mk_bak ($$$$$$;$$) {
    my ( $this, $db_path, $bak_path, $home_dir_id, $user_directory,
        $home_url, $no_addr, $wholesite )
        = @_;
    my $home_url_encoded = $this->url_encode($home_url);
    my $done             = 0;
    my (%db_hash);
    tie( %db_hash, $env::dbm_file, $db_path, O_RDONLY, $env::dbm_mode );
    if ( ref tied %db_hash ) {
        local (*BAK);
        $bak_path .= '.bak';
        if ( open( BAK, ">>$bak_path" ) ) {
            binmode BAK, ":utf8";
            flock( BAK, LOCK_EX );
            truncate( BAK, 0 );
            seek( BAK, 0, 0 );
            my $is_navi_db = ( $db_path =~ /\/navi_db$/ )        ? 1 : 0;
            my $is_setup   = ( $db_path =~ /\/setup\/general$/ ) ? 1 : 0;
            my ( $k, $v );
            while ( eval { ( $k, $v ) = each %db_hash } ) {
                utf8::decode($k) if !utf8::is_utf8($k);
                utf8::decode($v) if !utf8::is_utf8($v);
                next
                    if (
                    !$wholesite
                    && grep( $k eq $_,
                        'latest',        'count', 'counterReset',
                        'mastercginame', 'navi_gajaxkey' )
                    )
                    || ( $is_setup && $k eq 'ver' );
                if ( defined $k && $k ne '' ) {
                    $v = '' if !defined $v;
                    $k =~ s/\x09/&#09;/gs;
                    $k =~ s/(\x0D\x0A|\x0D|\x0A)/\x0B/gs;
                    if ($no_addr) {
                        if (   $k eq 'mastername'
                            || $k eq 'master'
                            || $k eq 'owners'
                            || $k eq 'staff' )
                        {
                            $v = '';
                        }
                        else {
                            $v
                                =~ s!<?[0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+>?!!g;
                        }
                    }
                    $v =~ s/^.*(_[a-zA-Z0-9]+)$/$1/
                        if $is_navi_db && $k =~ /_id$/;
                    $v
                        =~ s!($user_directory|\.)(/lib/(?:xfiles|javascript|css)/)!(($1 && $1 eq '.') ? $1 . $2 : '<user-directory>' . $2)!ge;
                    if ($home_dir_id) {
                        $v
                            =~ s!\Q$user_directory\E(?=["'/])!<user-directory>!gs;
                        $v
                            =~ s/#document_$home_dir_id/#document_<home-dir-id>/g;
                    }
                    else {
                        $v
                            =~ s/([\x09\x0A\x0C\x0D\x20]+(?:src|href|background|virtual|action)=(?:&quot;|["'])(?!\/\/))\Q$user_directory\E(?=(?:[\/"']|&quot;))/$1<user-directory>/gi;
                        $v =~ s/#document_(?!_)/#document_<home-dir-id>/g;
                    }
                    $v =~ s!$home_url!<home-url>!g;
                    $v =~ s/$home_url_encoded/<home-url-encoded>/g;
                    $v =~ s/\x09/&#09;/gs;
                    $v =~ s/(\x0D\x0A|\x0D|\x0A)/\x0B/gs;
                    print BAK $k;
                    print BAK "\x09";
                    print BAK $v;
                    print BAK "\x0A";
                }
                undef $k;
                undef $v;
            }
            close(BAK);
            $done = 1;
        }
    }
    untie %db_hash;
    chmod( $env::file_mode, $bak_path );
    return $done;
}

######################################################################
# データディレクトリテンプレートを作成
sub mk_data_dir ($$$;$) {
    my ( $this, $data_path, $temp_path, $wholesite ) = @_;
    my (@dir);
    local (*DIR);
    return unless $this->set_dir($temp_path) && opendir( DIR, $data_path );
    @dir = grep( $_ !~ /^\./ && -d $data_path . '/' . $_, readdir(DIR) );
    closedir(DIR);
    foreach my $d (@dir) {
        $this->set_dir( $temp_path . '/' . $d );
        $this->write_file(
            $temp_path . '/' . $d . '/index.html',
            "<html><head><title>404 Not Found</title></head><body><h1>Not Found</h1> The requested object does not exist on this server.</body></html>\x0A",
            1
        );
        $this->mk_data_dir( $data_path . '/' . $d, $temp_path . '/' . $d );
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's600', 'SiteTemplateMaker' );

    #メニュー登録
    $setting::jump_menu{'s600'} = 'サイトテンプレート作成';

}

1;
