package SiteTemplateRemover;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
######################################################################
# サイトテンプレート削除モジュール
# SiteTemplateRemover.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-04-18
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( @site_template_list, $site_template_dir );
    $this->get_setup_env;
    if ( !@main::target && defined $main::target && $main::target ne '' ) {
        push( @main::target, $main::target );
        undef $main::target;
    }
    $site_template_dir
        = $env::user_site_template_dir || $env::template_dir . '/site';

    #サイトテンプレートスキャン
    local (*DIR);
    if ( opendir( DIR, $site_template_dir ) ) {
        @site_template_list
            = grep( $_ !~ /^\./
                && $_ ne 'default'
                && -d $site_template_dir . '/' . $_,
            readdir(DIR) );
        closedir(DIR);
    }

    #アンインストール処理
    if (@main::target) {
        eval 'use File::Path';
        foreach my $dir (@main::target) {
            if ( @site_template_list
                && grep( $_ eq $dir, @site_template_list ) )
            {
                my ( $tempid, $home_dir, $name );
                $tempid = $dir;
                if ( !$env::user_site_template_dir ) {
                    ( $tempid, $home_dir )
                        = $this->get_sitetemplate_id_and_home_dir($dir);
                    if ($home_dir) {
                        next
                            if $default::home_dir
                            && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/;
                        $tempid = $this->escape( $tempid . '@~' . $home_dir );
                    }
                }
                ($name) = split(
                    /\x09/,
                    $this->read_file(
                        $site_template_dir . '/' . $dir . '/name.txt'
                    )
                );
                if ( -f $site_template_dir . '/' . $dir . '/locked.txt' ) {
                    push(
                        @{ $this->{'dialog'} },
                        '「'
                            . (
                            ($name)
                            ? $this->escape($name) . '(' . $tempid . ')'
                            : $tempid
                            )
                            . '」は削除できません。管理人によって保護されています。'
                    );
                }
                else {
                    rmtree( $site_template_dir . '/' . $dir );
                    push(
                        @{ $this->{'dialog'} },
                        '「'
                            . (
                            ($name)
                            ? $this->escape($name) . '(' . $tempid . ')'
                            : $tempid
                            )
                            . '」を削除しました。'
                    );
                    @site_template_list
                        = grep( $_ ne $dir, @site_template_list );
                }
            }
            elsif ( $dir eq 'default' ) {
                push(
                    @{ $this->{'dialog'} },
                    'デフォルトのサイトテンプレートは削除できません。'
                );
            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    '「'
                        . $this->escape($dir)
                        . '」は存在しません。'
                );
            }
        }
        undef @main::target;
    }
    @site_template_list
        = grep( !-f $site_template_dir . '/' . $_ . '/locked.txt',
        @site_template_list )
        if @site_template_list;

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サイトテンプレート削除</h1>';

    if (@site_template_list) {
        no warnings qw(numeric);
        print $this->form_start(
            'key'    => 'tools',
            'name'   => 'input',
            'method' => 'post',
            'onsubmit' =>
                'return confirm(\'この操作は取り消せません。本当に削除しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
            'required' => \@setting::form
        );
        print
            '<fieldset><legend>削除するサイトテンプレートを選択</legend>';
        @site_template_list
            = sort { $a <=> $b || $a cmp $b } @site_template_list;
        print '<select name="target" multiple="multiple" size="'
            . ( scalar @site_template_list ) . '">';
        foreach my $dir (@site_template_list) {
            my ( $tempid, $home_dir, $name );
            $tempid = $dir;
            if ( !$env::user_site_template_dir ) {
                ( $tempid, $home_dir )
                    = $this->get_sitetemplate_id_and_home_dir($dir);
                if ($home_dir) {
                    next
                        if $default::home_dir
                        && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/;
                    $tempid = $this->escape( $tempid . '@~' . $home_dir );
                }
            }
            ($name) = split(
                /\x09/,
                $this->read_file(
                    $site_template_dir . '/' . $dir . '/name.txt'
                )
            );
            $name = '' if !defined $name;
            print '<option value="'
                . $dir . '">'
                . $this->site_template_title( $name, $tempid )
                . '</option>';
        }
        print '</select>';
        print '</fieldset>';
        print
            '<fieldset class="delete"><input type="submit" value="削除" /></fieldset></form>';
    }
    else {
        print
            '<fieldset><div class="hints">削除可能なサイトテンプレートが存在しません。</div></fieldset>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's601', 'SiteTemplateRemover' );

    #メニュー登録
    $setting::jump_menu{'s601'} = 'サイトテンプレート削除';

}

1;
