package SubDirInstaller;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# サブディレクトリ増設モジュール
# SubDirInstaller.pm ver1.2.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2013-07-20
######################################################################
# 実行
sub run ($) {
    my $this             = shift;
    my $error_flag       = 0;
    my $is_document_root = 1 if !$cgi::input->home_directory(1);
    undef $main::multi_domain if $env::multi_domain || !$is_document_root;
    undef $main::share_flag if $main::multi_domain;
    $this->get_setup_env;
    $this->get_perl_path;
    my ( $multi_domain, $server_name, $ssl, $site_template_dir,
        $selected_template );

    if (   $main::template
        && $main::template =~ /^(user:|)([0-9a-zA-Z_\-\@]+)$/ )
    {
        my $user_template = $1;
        $selected_template = $2;
        my ( $tempid, $home_dir )
            = $this->get_sitetemplate_id_and_home_dir($selected_template);
        if (   $default::home_dir
            && $home_dir
            && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/ )
        {
            undef $selected_template;
        }
        elsif ( $user_template && $env::user_site_template_dir ) {
            $site_template_dir = $env::user_site_template_dir;
        }
        else {
            $site_template_dir = $env::template_dir . '/site';
        }
    }
    my @targetable_directories = $this->targetable_directories(
        $default::cgi_data_root . ( $default::home_dir || '' ) );

    #入力チェック
    if ( defined $main::a && $main::a ) {
        $main::name = $this->z_to_h( $main::name, 'utf8' )
            if defined $main::name && $main::name;
        $main::target_dir = '' if $main::multi_domain;
        if ($main::target_dir) {
            if (  !@targetable_directories
                || $main::target_dir !~ /^\//
                || $main::target_dir =~ /\/\.\.\//
                || !grep( $_ eq $main::target_dir, @targetable_directories )
                || !-d $default::cgi_data_root
                . $default::home_dir
                . $main::target_dir )
            {
                push(
                    @{ $this->{'dialog'} },
                    '増設先ディレクトリ指定を確認してください。'
                );
                $main::target_dir = '';
                $error_flag       = 1;
            }
            else {
                my ( $subdirectory_disabled, $user_site_template_dir,
                    $user_default_dir );
                (   $multi_domain, $server_name, $ssl,
                    $subdirectory_disabled, $user_site_template_dir,
                    $user_default_dir
                    )
                    = $this->subdir_options( $default::cgi_data_root
                        . $default::home_dir
                        . $main::target_dir
                        . '/lib/env.pl' );
                if ($subdirectory_disabled) {
                    $main::user_site_template
                        = ($user_site_template_dir)
                        ? 1
                        : $main::user_site_template;
                    $main::user_default
                        = ($user_default_dir) ? 1 : $main::user_default;
                    $main::subdirectory_disabled = 1;
                    undef $multi_domain if $env::multi_domain;
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        '増設先ディレクトリ指定を確認してください。'
                    );
                    $main::target_dir = '';
                    $error_flag       = 1;
                }
            }
        }
        if ( !defined $main::name || $main::name eq '' ) {
            push(
                @{ $this->{'dialog'} },
                'フォルダ名を入力してください。'
            );
            $error_flag = 1;
        }
        elsif ($main::multi_domain) {
            if ( $main::name !~ /^[0-9a-zA-Z_][0-9a-zA-Z_\.\-]+$/ ) {
                push(
                    @{ $this->{'dialog'} },
                    'フォルダ名は半角英字で始まる20文字以内の半角英数字列(a-z,A-Z,.,-,_)にしてください。'
                );
                $error_flag = 1;
            }
            elsif ( !$main::domain ) {
                push(
                    @{ $this->{'dialog'} },
                    'ドメイン名を入力してください。'
                );
                $error_flag = 1;
            }
            elsif ( $main::domain !~ /^[0-9a-zA-Z_\.\-\%]+\.[0-9a-zA-Z_]+$/ )
            {
                push(
                    @{ $this->{'dialog'} },
                    '英数字ドメイン以外を指定することは出来ません。'
                );
                $error_flag = 1;
            }
            elsif ( !-d '../..'
                . ( $main::target_dir || '' ) . '/'
                . $main::name )
            {
                push(
                    @{ $this->{'dialog'} },
                    '指定された別ドメイン用フォルダが存在しません。'
                );
                $error_flag = 1;
            }
            elsif (
                $this->name_is_unavailable(
                    $main::target_dir, $main::name, 1
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    '指定された別ドメイン用フォルダは既に使用されています。'
                );
                $error_flag = 1;
            }
        }
        elsif ( $main::name !~ /^[a-zA-Z][a-zA-Z0-9]{0,20}$/ ) {
            push(
                @{ $this->{'dialog'} },
                'フォルダ名は半角英字で始まる20文字以内の半角英数字列にしてください。'
            );
            $error_flag = 1;
        }
        elsif ( $this->name_is_unavailable( $main::target_dir, $main::name ) )
        {
            push(
                @{ $this->{'dialog'} },
                '入力されたフォルダ名は既に使用されています。'
            );
            $error_flag = 1;
        }
        if ( defined $main::password
            && $main::password =~ /^[0-9a-zA-Z_]{6,}$/ )
        {
            if ( !defined $main::password2
                || $main::password ne $main::password2 )
            {
                push(
                    @{ $this->{'dialog'} },
                    '確認のためのパスワードが一致しません。'
                );
                $error_flag = 1;
            }
        }
        else {
            push(
                @{ $this->{'dialog'} },
                'パスワードは6文字以上の半角英数字にしてください。'
            );
            $error_flag = 1;
        }
        $main::emailaddr = $this->z_to_h($main::emailaddr);
        if ( $this->is_invalid_email_address($main::emailaddr) ) {
            push(
                @{ $this->{'dialog'} },
                '管理人メールアドレスを確認してください。'
            );
            $error_flag = 1;
        }
        $this->env_escape( \$main::title );
        if ( $main::title ne '' && $main::title !~ /[^\x09\x0A\x0C\x0D\x20]/ )
        {
            push(
                @{ $this->{'dialog'} },
                'サイトタイトルを入力してください。'
            );
            $error_flag = 1;
        }
        $this->env_escape( \$main::description );
        if (   $main::description ne ''
            && $main::description !~ /[^\x09\x0A\x0C\x0D\x20]/ )
        {
            push(
                @{ $this->{'dialog'} },
                'サイトの説明を入力してください。'
            );
            $error_flag = 1;
        }
        $this->env_escape( \$main::copyright );
        if ( !defined $main::template || $main::template eq '' ) {
            $selected_template = $main::template = 'default';
        }
        elsif (!-d $site_template_dir . '/' . $selected_template
            || !-f $site_template_dir . '/'
            . $selected_template
            . '/htdocs/cgi/setup/start.cgi' )
        {
            push(
                @{ $this->{'dialog'} },
                '指定されたサイトテンプレートは存在しません。'
            );
            $error_flag = 1;
        }

        #インストール処理
        if ( !$error_flag ) {
            my ($target_dir,     $target_rel,
                $cgi_data_path,  $shared_cgi_data_root,
                $home_dir,       $http_home_dir,
                $home_dir_id,    $cgi_data_root,
                $env_path,       $home_url,
                $home_site,      $default_id,
                $javascript_dir, $user_directory
            );
            $target_dir = $main::target_dir || '';
            $target_rel
                = ($target_dir) ? ( '../' x ( $target_dir =~ tr!/!/! ) ) : '';
            $cgi_data_path
                = ( ( defined $default::cgi_data ) ? $default::cgi_data : '' )
                . $target_dir . '/'
                . $main::name;
            if ($main::multi_domain) {
                $home_dir       = '/' . $main::name;
                $http_home_dir  = '';
                $home_dir_id    = '';
                $user_directory = '';
                $home_url       = ( ($main::ssl) ? 'https://' : 'http://' )
                    . $main::domain;
                $javascript_dir = '/lib/javascript';
            }
            elsif ($multi_domain) {
                my $target = $target_dir;
                $target =~ s!^/[^/]+!!;
                $home_dir      = $target_dir . '/' . $main::name;
                $http_home_dir = $user_directory
                    = $target . '/' . $main::name;
                ( $home_dir_id = $http_home_dir ) =~ tr!/!_!;
                $home_dir_id =~ s/^_//g;
                $home_dir_id .= '_';
                $home_url
                    = ( ($ssl) ? 'https://' : 'http://' )
                    . $server_name
                    . $user_directory;
                $javascript_dir = $user_directory . '/lib/javascript';
            }
            else {
                $home_dir
                    = (
                    ( defined $default::home_dir ) ? $default::home_dir : '' )
                    . $target_dir . '/'
                    . $main::name;
                $http_home_dir
                    = ( ( defined $default::http_home_dir )
                    ? $default::http_home_dir
                    : '' )
                    . $target_dir . '/'
                    . $main::name;
                ( $home_dir_id = $http_home_dir ) =~ tr!/!_!;
                $home_dir_id =~ s/^_//g;
                $home_dir_id .= '_';
                $user_directory
                    = $this->home_directory(1)
                    . $target_dir . '/'
                    . $main::name;
                $home_url
                    = $this->full_path( '../../', 1 )
                    . $target_dir . '/'
                    . $main::name;
                $javascript_dir = $user_directory . '/lib/javascript';
            }
            $cgi_data_root = '../' . $target_rel . $default::cgi_data_root;
            $shared_cgi_data_root
                = '../' . $target_rel . $default::shared_cgi_data_root;

            #env.pl
            $this->set_dir( $cgi_data_path . '/lib' );
            $this->mk_env(
                $cgi_data_path,
                $cgi_data_root,
                $shared_cgi_data_root,
                $home_dir,
                $http_home_dir,
                $home_dir_id,
                $user_directory,
                $site_template_dir . '/' . $selected_template . '/env.bak',
                $main::conv_table,
                $multi_domain,
                $server_name,
                $ssl,
                (   ($env::user_site_template_dir) ? 1
                    : $main::user_site_template
                ),
                ( ($env::user_default_dir) ? 1 : $main::user_default ),
                (   ($env::subdirectory_disabled) ? 1
                    : $main::subdirectory_disabled
                ),
            );

            #サブディレクトリ増設
            $home_site = $home_url;
            $home_site =~ s!^https?://!!i;
            $this->mk_sub_directory(
                $site_template_dir . '/' . $selected_template . '/htdocs',
                '../..' . $target_dir . '/' . $main::name,
                '../' . $target_rel . $cgi_data_path,
                $cgi_data_root,
                $shared_cgi_data_root,
                $home_dir,
                $http_home_dir,
                $home_dir_id,
                $home_url,
                $this->url_encode($home_url),
                $home_site,
                $javascript_dir,
                $user_directory,
                $this->url_encode($user_directory),
                $main::conv_table
            );

            #preferenceディレクトリ復元
            $this->restore_default_settings(
                $site_template_dir . '/'
                    . $selected_template
                    . '/preferences',
                $cgi_data_path . '/preferences',
                1,
                $home_dir_id,
                $user_directory,
                $home_url,
                $main::conv_table
            );

            #dataディレクトリ作成
            $this->restore_default_settings(
                $site_template_dir . '/' . $selected_template . '/data',
                $cgi_data_path . '/data',
                1,
                $home_dir_id,
                $user_directory,
                $home_url,
                $main::conv_table
            );

            #QRコード画像
            require CMONOS::CGI_Lib::HTTP_Client;
            my $res = HTTP_Client->new->request(
                'GET' =>
                    'http://chart.apis.google.com/chart?chs=160x160&cht=qr&chl='
                    . $this->url_encode( $home_url . '/m.cgi/' ),
                {}, '', '', '',
                '../..' . $target_dir . '/' . $main::name . '/qr.png'
            );
            $res = HTTP_Client->new->request(
                'GET' =>
                    'http://chart.apis.google.com/chart?chs=100x100&cht=qr&chl='
                    . $this->url_encode( $home_url . '/m.cgi/' ),
                {}, '', '', '',
                '../..' . $target_dir . '/' . $main::name . '/qr_mini.png'
            );

            #JavaScriptアップデート
            require CMONOS::SetupAssistant::Tools::JavaScriptUpdater;
            Common->new('s303')
                ->javascript_update( $home_dir, 0,
                '../..' . $target_dir . '/' . $main::name );

            #CSSアップデート
            require CMONOS::SetupAssistant::Tools::CSSUpdater;
            {
                local @main::target = (
                    'common.css',            'calendar-common.css',
                    'tablemaker-common.css', 'tools.css',
                    'images'
                );
                Common->new('s304')->css_update($home_dir);
            }

            if ( $main::multi_domain || $multi_domain ) {
                if ($main::multi_domain) {
                    $this->directory_copy(
                        $env::image_lib,
                        '../..'
                            . $target_dir . '/'
                            . $main::name
                            . '/cgi/imageLib',
                        1
                    );
                    $this->directory_copy( $env::icon_dir,
                        '../..' . $target_dir . '/' . $main::name . '/icon',
                        1 );
                    $this->directory_copy(
                        '../../error',
                        '../..' . $target_dir . '/' . $main::name . '/error',
                        1
                    );
                    $this->file_copy( $env::dummy_window,
                              '../..'
                            . $target_dir . '/'
                            . $main::name
                            . '/cgi/dummy4href.html' );
                    $this->file_copy( $env::download_window,
                              '../..'
                            . $target_dir . '/'
                            . $main::name
                            . '/cgi/dummy.html' );
                    $this->optimize_htaccess(
                        '../..' . $target_dir . '/' . $main::name, '/' );
                }
                if ($target_dir) {
                    $target_dir =~ s/^\///;
                    push(
                        @{ $this->{'dialog'} },
                        'サブディレクトリ「'
                            . ( ($target_dir) ? $target_dir . '/' : '' )
                            . $main::name
                            . '」を別ドメイン'
                            . (
                            ($multi_domain)
                            ? '「' . $server_name . '」'
                            : ''
                            )
                            . 'に増設しました。'
                    );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'サブディレクトリ「'
                            . $main::name
                            . '」を別ドメインとして増設しました。'
                    );
                }

            }
            else {

                #増設通知
                if ( defined $main::notification && $main::notification ) {
                    my %installmsg;
                    $this->import_db(
                        $default::preference_dir . '/installmsg',
                        \%installmsg );
                    $installmsg{'subject'}
                        = '【お知らせ】サブディレクトリを増設しました。'
                        if !$installmsg{'subject'};
                    $installmsg{'msg'}
                        = '○URL' . "\x0A" . '<uri>'
                        . "\x0A\x0A"
                        . '○メンテナンスステーション' . "\x0A"
                        . '<tool>' . "\x0A"
                        if !$installmsg{'msg'};
                    my $uri  = $home_url . '/';
                    my $tool = $home_url . '/cgi/';
                    $installmsg{'msg'} =~ s/<(uri|url)>/$uri/igs;
                    $installmsg{'msg'} =~ s/<tool>/$tool/igs;
                    $installmsg{'msg'} =~ s/<password>/$main::password/igs;

                    unless ( ref $cgi::SendMail ) {
                        require CMONOS::CGI_Lib::SendMail;
                        $cgi::SendMail = SendMail->new( $env::sendmail,
                            $env::master_email, '', $cgi::input, 1 );
                    }
                    $cgi::SendMail->send_mail(
                        'To'   => $main::emailaddr,
                        'From' => (
                            ( $setting::general{'master'} )
                            ? ( ( $setting::general{'mastername'} )
                                ? $setting::general{'mastername'} . ' <'
                                    . $setting::general{'master'} . '>'
                                : $setting::general{'master'} )
                            : $env::master_email
                        ),
                        'Subject' => $installmsg{'subject'},
                        'message' => $installmsg{'msg'} . "\x0A\x0A"
                    );
                }
                if ($target_dir) {
                    $target_dir =~ s/^\///;
                    $target_dir .= '/';
                }
                $this->set_javascript(
                    <<SCRIPT
function jumpConfirmation () {
	if (confirm('「${main::name}」をインストールしました。\\n「${target_dir}${main::name}」設定ページに移動しますか？')) {
		top.document.location.href = '../../$target_dir$main::name/cgi/';
	}
}
SCRIPT
                );
                $this->set_onload('jumpConfirmation();');
            }
            undef $main::share_flag;
            undef $main::name;
            undef $main::title;
            undef $main::copyright;
            undef $main::description;
            undef $main::emailaddr;
            undef $main::notification;
        }
    }

    #ページヘッダ
    if ( !$env::multi_domain && $is_document_root ) {
        if ($main::multi_domain) {
            $this->set_onload(
                'cmonos.element.item(\'target-directory\',\'register-setting\',\'send-mail\').disappear(true);cmonos.element.item(\'new-domain\').disappear(false);'
            );
        }
        else {
            $this->set_onload(
                'cmonos.element.item(\'new-domain\').disappear(true);');
        }
    }
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サブディレクトリ増設</h1>';

    print $this->form_start(
        'key'    => 'tools',
        'name'   => 'input',
        'method' => 'post',
        'onsubmit' =>
            'return confirm(\'この操作は取り消せません。本当にインストールしますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="1" />';
    print '<fieldset><table class="field-table"><tbody>';
    if (@targetable_directories) {
        print '<tr id="target-directory">';
        print
            '<td class="field-name"><label for="target_dir">増設先ディレクトリ</label></td>';
        print '<td class="field">';
        print
            '<select name="target_dir" id="target_dir" onchange="document.getElementById(\'samplepage\').href = \'../..\'+this.value+\'/\'">';
        print '<option value="">このディレクトリ</option>';
        foreach my $dir (@targetable_directories) {
            my $d = $dir;
            $d =~ s!^/!!;
            print '<option value="'
                . $dir . '"'
                . ( ( defined $main::target_dir && $main::target_dir eq $dir )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $d
                . '</option>';
        }
        print '</select> <a href="../..'
            . ( $main::target_dir || '' )
            . '/" id="samplepage" target="sample">[確認]</a><div class="hints">増設先ディレクトリを選択するとオプションは増設先ディレクトリの設定に準じます。</div>';
        print '</td>';
        print '</tr>';
    }
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="name">フォルダ名</label></td>';
    print
        '<td class="field"><input type="text" name="name" id="name" size="20" value="'
        . $this->escape($main::name)
        . '" required="required" />';
    print
        ' <label title="このフォルダを HTTP 公開ディレクトリ(ドキュメントルート)として増設します。"><input type="checkbox" name="multi_domain" value="1"'
        . ( ($main::multi_domain) ? ' checked="checked"' : '' )
        . ' onclick="cmonos.element.item(\'target-directory\',\'register-setting\',\'send-mail\').disappear(this.checked);cmonos.element.item(\'new-domain\').disappear(!this.checked);" /> 別ドメインとして増設する</label>'
        if !$env::multi_domain && $is_document_root;
    print '</td>';
    print '</tr>';
    print '</tbody><tbody>';

    if ( !$env::multi_domain && $is_document_root ) {
        print '<tr id="new-domain">';
        print
            '<td class="field-name"><label for="title">ドメイン</label></td>';
        print '<td class="field">';
        print $this->mk_select_menu(
            'ssl', '', '', $main::ssl,
            '0' => 'http://',
            '1' => 'https://',
        );
        print '<input type="text" name="domain" id="title" size="20" value="'
            . $this->escape($main::domain) . '" />/';
        print '</td>';
        print '</tr>';
    }
    print '<tr>';
    print
        '<td class="field-name"><label for="title">サイトタイトル</label></td>';
    print
        '<td class="field"><input type="text" name="title" id="title" size="20" value="'
        . $this->escape( ( defined $main::title && $main::title ne '' )
        ? $main::title
        : $env::title )
        . '" required="required" /></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="copyright">サイトの著作権者</label></td>';
    print
        '<td class="field"><input type="text" name="copyright" id="copyright" size="30" value="'
        . $this->escape(
        ( defined $main::copyright && $main::copyright ne '' )
        ? $main::copyright
        : $env::copyright )
        . '" />';
    print
        '<div class="hints">RSSのサイトの著作権表示などに使用されます。</div>';
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="description">サイトの説明</label></td>';
    print
        '<td class="field"><textarea name="description" id="description" cols="60" rows="2" required="required">'
        . $this->escape(
        ( defined $main::description && $main::description ne '' )
        ? $main::description
        : $env::description )
        . '</textarea>';
    print
        '<div class="hints">RSSのサイトの説明などに使用されます。</div>';
    print '</td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr id="register-setting">';
    print
        '<td class="field-name"><span class="datatitle">登録パスワード情報</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="share_flag" id="share_flag"'
        . ( ($main::share_flag) ? ' checked="checked"' : '' )
        . ' /> 最上層と共有する</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="password">マスターパスワード</label></td>';
    print
        '<td class="field"><input type="password" name="password" id="password" size="20" required="required" /></td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="password2">マスターパスワード（確認）</label></td>';
    print
        '<td class="field"><input type="password" name="password2" id="password2" size="20" required="required" />';
    print '<div class="hints">パスワードの例 "'
        . $this->rand_char_sample
        . '"</div></td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="emailaddr">管理人アドレス</label></td>';
    print
        '<td class="field"><input type="email" name="emailaddr" id="emailaddr" size="30" value="'
        . $this->escape($main::emailaddr)
        . '" required="required" />';
    print
        '<div class="note" id="send-mail"><label><input type="checkbox" name="notification" value="1"'
        . ( ($main::notification) ? ' checked="checked"' : '' )
        . ' /> 新規サブディレクトリの管理人にサブディレクトリ増設通知を送る</label></div>';
    print '</td>';
    print '</tr>';

    if (   !$env::user_site_template_dir
        || !$env::user_default_dir
        || !$env::subdirectory_disabled )
    {
        print '</tbody><tbody>';
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><span class="datatitle">オプション</span></td>';
        print '<td class="field">';
        print '<ol class="checkbox">';
        print
            '<li><label><input type="checkbox" name="user_site_template" value="1"'
            . ( ($main::user_site_template) ? ' checked="checked"' : '' )
            . ' /> サイトテンプレートをサブディレクトリごとに保存する</label></li>'
            if !$env::user_site_template_dir;
        print
            '<li><label><input type="checkbox" name="user_default" value="1"'
            . ( ($main::user_default) ? ' checked="checked"' : '' )
            . ' /> 共有設定をサブディレクトリごとに保存する</label></li>'
            if !$env::user_default_dir;
        print
            '<li><label><input type="checkbox" name="subdirectory_disabled" value="1"'
            . ( ($main::subdirectory_disabled) ? ' checked="checked"' : '' )
            . ' /> サブディレクトリ増設を禁止する</label></li>'
            if !$env::subdirectory_disabled;
        print '</ol>';
        print
            '<div class="warning">上記オプションはサブディレクトリインストール後変更できません。</div>';
        print '</td>';
        print '</tr>';
    }
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="template">サイトテンプレート</label></td>';
    print
        '<td class="field"><select name="template"><option value="default">デフォルトのサイトテンプレート</option>';
    if ( opendir( TEMP, $env::template_dir . '/site' ) ) {
        ;
        no warnings qw(numeric);
        my @template
            = grep( $_ ne 'default'
                && $_ !~ /\./
                && -d $env::template_dir
                . '/site/'
                . $_
                && -f $env::template_dir
                . '/site/'
                . $_
                . '/htdocs/cgi/setup/start.cgi',
            readdir(TEMP) );
        closedir(TEMP);
        foreach my $dir ( sort { $a <=> $b || $a cmp $b } @template ) {
            my ( $tempid, $home_dir, $name );
            $tempid = $dir;
            ( $tempid, $home_dir )
                = $this->get_sitetemplate_id_and_home_dir($dir);
            if ($home_dir) {
                next
                    if $default::home_dir
                    && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/;
                $tempid = $this->escape( $tempid . '@~' . $home_dir );
            }
            ($name) = split(
                /\x09/,
                $this->read_file(
                    $env::template_dir . '/site/' . $dir . '/name.txt'
                )
            );
            $name = '' if !defined $name;
            print '<option value="'
                . $dir . '"'
                . ( ( $selected_template && $selected_template eq $dir )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $this->site_template_title( $name, $tempid )
                . ( ($env::user_site_template_dir) ? '&lt;共有&gt;' : '' )
                . '</option>';
        }
    }
    if ( $env::user_site_template_dir
        && opendir( TEMP, $env::user_site_template_dir ) )
    {
        ;
        no warnings qw(numeric);
        my @template
            = grep( $_ !~ /\./
                && -d $env::user_site_template_dir . '/'
                . $_
                && -f $env::user_site_template_dir . '/'
                . $_
                . '/htdocs/cgi/setup/start.cgi',
            readdir(TEMP) );
        closedir(TEMP);
        foreach my $dir ( sort { $a <=> $b || $a cmp $b } @template ) {
            my ($name) = split(
                /\x09/,
                $this->read_file(
                    $env::user_site_template_dir . '/' . $dir . '/name.txt'
                )
            );
            $name = '' if !defined $name;
            print '<option value="user:'
                . $dir . '"'
                . ( ( $selected_template && $dir eq $selected_template )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $this->site_template_title( $name, $dir )
                . '</option>';
        }
    }
    print '</select></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="conv_table">文字変換表</label></td>';
    print
        '<td class="field"><textarea name="conv_table" id="conv_table" cols="72" rows="4" wrap="off" title="下記書式に従って文字変換表を設定します。">'
        . $this->escape($main::conv_table)
        . '</textarea>';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'conv_table\').setAutoFit(this.checked);" />フィット</label>';
    print
        '<div class="note">書式：変換対象文字列=&gt;変換後文字列[改行]</div>';
    print '</td>';
    print '</tr>';
    print '</tbody></table></fieldset>';
    print
        '<fieldset class="submit"><input type="submit" value="サブディレクトリインストール" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

#フォルダ名重複チェック
sub name_is_unavailable ($$;$$) {
    my ( $this, $target_dir, $name, $multidomain ) = @_;
    return (
               $name =~ /^\./
            || ( $env::user_default_dir       && $name eq 'default' )
            || ( $env::user_site_template_dir && $name eq 'template' )
            || $name eq 'clipboard'
            || ( !$multidomain
            && -e '../..' . ( $target_dir || '' ) . '/' . $name )
            || -e $default::cgi_data . ( $target_dir || '' ) . '/' . $name
    ) ? 1 : 0;
}

#env.pl作成
sub mk_env ($$$$$$$$$$$$$) {
    my ($this,                 $env_path,           $cgi_data_root,
        $shared_cgi_data_root, $home_dir,           $http_home_dir,
        $home_dir_id,          $user_directory,     $env_setting,
        $conv_table,           $multi_domain,       $server_name,
        $ssl,                  $user_site_template, $user_default,
        $subdirectory_disabled
    ) = @_;
    my ( $image_lib, $icon_dir, $download_window, $dummy_window, $data,
        $master_password_encrypted, $master_password, $sub_cgi_data_root );
    $master_password = $main::password;
    if ($env::session_enabled) {
        $master_password_encrypted = 1;
        $master_password = $cgi::Register->encrypt($master_password);
    }
    $env_path .= '/lib/env.pl';
    $data = $this->read_file( $default::cgi_data_root . '/lib/env.pl' );
    $data =~ s/package env;/package env;\x0Ause utf8;/s
        if $data !~ /use utf8;/;
    if (   $env::multi_domain
        && $default::cgi_data =~ m!^($default::cgi_data_root/[^/]+)! )
    {
        $sub_cgi_data_root = '../' . $1;
    }
    elsif ($multi_domain) {
        $sub_cgi_data_root = $shared_cgi_data_root
            . ( ( $home_dir =~ m!^(/[^/]+)! ) ? $1 : '' );
    }
    else {
        $sub_cgi_data_root = $cgi_data_root;
    }
    if ($main::multi_domain) {
        $icon_dir        = '../../icon';
        $image_lib       = '../imageLib';
        $download_window = '../dummy.html';
        $dummy_window    = '../dummy4href.html';
    }
    elsif ($multi_domain) {
        my $rel = '../' x ( $http_home_dir =~ tr!/!/! );
        $icon_dir        = $rel . '../../icon';
        $image_lib       = $rel . '../../cgi/imageLib';
        $download_window = $rel . '../../cgi/dummy.html';
        $dummy_window    = $rel . '../../cgi/dummy4href.html';
    }
    else {
        $image_lib = $this->base_change( $env::image_lib,
            $user_directory . '/cgi/cgi/start.cgi' );
        $icon_dir = $this->base_change( $env::icon_dir,
            $user_directory . '/cgi/cgi/start.cgi' );
        $download_window = $this->base_change( $env::download_window,
            $user_directory . '/cgi/cgi/start.cgi' );
        $dummy_window = $this->base_change( $env::dummy_window,
            $user_directory . '/cgi/cgi/start.cgi' );
    }
    if ( $default::cgi_data_name =~ /^\.\.\// ) {
        $data =~ s!(['"])$default::cgi_data_name!$1$cgi_data_root!g;
    }
    else {
        $data
            =~ s!(['"])\.\./\.\./\.\./$default::cgi_data_name!$1$cgi_data_root!g;
    }
    if ( $default::cgi_data_name ne $default::shared_cgi_data_name ) {
        if ( $default::shared_cgi_data_name =~ /^\.\.\// ) {
            $data
                =~ s!(['"])$default::shared_cgi_data_name!$1$shared_cgi_data_root!g;
        }
        else {
            $data
                =~ s!(['"])\.\./\.\./\.\./$default::shared_cgi_data_name!$1$shared_cgi_data_root!g;
        }
    }
    $data
        =~ s/^[\x20\x09]*\$master_password[\x20\x09]*=[^\x0A\x0D]*;/\$master_password = '$master_password';/gm;
    $data
        =~ s/^[\x20\x09]*\$master_password_encrypted[\x20\x09]*=[^\x0A\x0D]*;/\$master_password_encrypted = 1;/gm
        if $master_password_encrypted;
    $data
        =~ s/^[\x20\x09]*\$master_email[\x20\x09]*=[^\x0A\x0D]*;/\$master_email = '$main::emailaddr';/gm;
    $data
        =~ s/^[\x20\x09]*\$title[\x20\x09]*=[^\x0A\x0D]*;/\$title = '$main::title';/gm;
    $data
        =~ s/^[\x20\x09]*\$description[\x20\x09]*=[^\x0A\x0D]*;/\$description = '$main::description';/gm;
    $data
        =~ s/^[\x20\x09]*\$copyright[\x20\x09]*=[^\x0A\x0D]*;/\$copyright = '$main::copyright';/gm;
    $data
        =~ s/^[\x20\x09]*\$home_dir_id[\x20\x09]*=[^\x0A\x0D]*;/\$home_dir_id = '$home_dir_id';/gm;

    if ($main::share_flag) {
        my $cookie_path
            = ($multi_domain)
            ? ''
            : $this->user_directory(1)
            . ( ($env::top_dir) ? $env::top_dir : '' );
        $cookie_path = '/' if !$cookie_path;
        my $top_cgi_path = $image_lib;
        $top_cgi_path =~ s/\/[^\/]+$//;
        $data =~ s/\/(preferences|data(?!\/session_store))/$home_dir\/$1/g;
        $data
            =~ s!^[\x20\x09]*\$register_cgi[\x20\x09]*=[^\x0A\x0D]*;!\$register_cgi = '$top_cgi_path/pswd/start.cgi';!gm;
        $data
            =~ s!^[\x20\x09]*\$register_db[\x20\x09]*=[^\x0A\x0D]*;!\$register_db = '$sub_cgi_data_root/preferences/register';!gm;
        $data
            =~ s!^[\x20\x09]*\$session_store[\x20\x09]*=[^\x0A\x0D]*;!\$session_store = '$sub_cgi_data_root/data/session_store';!gm;
        $data
            =~ s/^[\x20\x09]*\$cookie_path[\x20\x09]*=[^\x0A\x0D]*;/\$cookie_path = '$cookie_path';/gm;
    }
    else {
        my $cookie_path = $user_directory || '/';
        $data =~ s!/(preferences|data)!$home_dir/$1!g;
        $data
            =~ s/^[\x20\x09]*\$cookie_path[\x20\x09]*=[^\x0A\x0D]*;/\$cookie_path = '$cookie_path';/gm;
    }
    $data
        =~ s/^[\x20\x09]*\$page_header_path[\x20\x09]*=[^\x0A\x0D]*;/\$page_header_path = '';/gm;
    $data
        =~ s/^[\x20\x09]*\$page_footer_path[\x20\x09]*=[^\x0A\x0D]*;/\$page_footer_path = '';/gm;
    $data
        =~ s!^[\x20\x09]*\$image_lib[\x20\x09]*=[^\x0A\x0D]*;!\$image_lib = '$image_lib';!gm;
    $data
        =~ s!^[\x20\x09]*\$icon_dir[\x20\x09]*=[^\x0A\x0D]*;!\$icon_dir = '$icon_dir';!gm;
    $data
        =~ s!^[\x20\x09]*\$download_window[\x20\x09]*=[^\x0A\x0D]*;!\$download_window = '$download_window';!gm;
    $data
        =~ s!^[\x20\x09]*\$dummy_window[\x20\x09]*=[^\x0A\x0D]*;!\$dummy_window = '$dummy_window';!gm;
    $data
        =~ s!^[\x20\x09]*\$google_analytics_uacct[\x20\x09]*=[^\x0A\x0D]*;!\$google_analytics_uacct = '$env::google_analytics_uacct';!gm
        if $env::google_analytics_uacct;
    $data
        =~ s!^[\x20\x09]*\$amazon_access_key[\x20\x09]*=[^\x0A\x0D]*;!\$amazon_access_key = '$env::amazon_access_key';!gm
        if $env::amazon_access_key;
    $data
        =~ s!^[\x20\x09]*\$amazon_associates_id[\x20\x09]*=[^\x0A\x0D]*;!\$amazon_associates_id = '$env::amazon_associates_id';!gm
        if $env::amazon_associates_id;

    if ($main::multi_domain) {
        my $ssl = ($main::ssl) ? 1 : 0;
        $data =~ s!^[\x20\x09]*1;!\$multi_domain = 1;\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$multi_domain[\x20\x09]*=[^\x0A\x0D]*;!\$multi_domain = 1;!gm;
        $data
            =~ s!^[\x20\x09]*1;!\$server_name = '$main::domain';\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$server_name[\x20\x09]*=[^\x0A\x0D]*;!\$server_name = '$main::domain';!gm;
        $data =~ s!^[\x20\x09]*1;!\$ssl = $ssl;\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$ssl[\x20\x09]*=[^\x0A\x0D]*;!\$ssl = $ssl;!gm;
    }
    elsif ($multi_domain) {
        $ssl = ( !defined $ssl || $ssl eq '' ) ? "''" : ($ssl) ? 1 : 0;
        $data =~ s!^[\x20\x09]*1;!\$multi_domain = 1;\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$multi_domain[\x20\x09]*=[^\x0A\x0D]*;!\$multi_domain = 1;!gm;
        $data =~ s!^[\x20\x09]*1;!\$server_name = '$server_name';\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$server_name[\x20\x09]*=[^\x0A\x0D]*;!\$server_name = '$server_name';!gm;
        $data =~ s!^[\x20\x09]*1;!\$ssl = $ssl;\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$ssl[\x20\x09]*=[^\x0A\x0D]*;!\$ssl = $ssl;!gm;
    }
    else {
        my $multi_domain = ($env::multi_domain) ? 1 : 0;
        my $server_name = $env::server_name || '';
        my $ssl
            = ( !defined $env::ssl || $env::ssl eq '' ) ? "''"
            : ($env::ssl) ? 1
            :               0;
        $data =~ s!^[\x20\x09]*1;!\$multi_domain = $multi_domain;\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$multi_domain[\x20\x09]*=[^\x0A\x0D]*;!\$multi_domain = $multi_domain;!gm;
        $data =~ s!^[\x20\x09]*1;!\$server_name = '$server_name';\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$server_name[\x20\x09]*=[^\x0A\x0D]*;!\$server_name = '$server_name';!gm;
        $data =~ s!^[\x20\x09]*1;!\$ssl = $ssl;\x0A\x0A1;!gm
            unless $data
            =~ s!^[\x20\x09]*\$ssl[\x20\x09]*=[^\x0A\x0D]*;!\$ssl = $ssl;!gm;
    }
    if ($user_site_template) {
        my $user_site_template_dir
            = $cgi_data_root . $home_dir . '/template/site';
        $data
            =~ s/^[\x20\x09]*\$user_site_template_dir[\x20\x09]*=[^\x0A\x0D]*;/\$subdirectory_disabled = '$user_site_template_dir';/gm
            || $data
            =~ s/^[\x20\x09]*1;$/\$user_site_template_dir = '$user_site_template_dir';\x0A\x0A1;/gm;
    }
    if ($user_default) {
        my $user_default_dir = $cgi_data_root . $home_dir . '/default';
        $data
            =~ s/^[\x20\x09]*\$user_default_dir[\x20\x09]*=[^\x0A\x0D]*;/\$subdirectory_disabled = '$user_default_dir';/gm
            || $data
            =~ s/^[\x20\x09]*1;$/\$user_default_dir = '$user_default_dir';\x0A\x0A1;/gm;
    }
    if ($subdirectory_disabled) {
        $data
            =~ s/^[\x20\x09]*\$subdirectory_disabled[\x20\x09]*=[^\x0A\x0D]*;/\$subdirectory_disabled = 1;/gm
            || $data
            =~ s/^[\x20\x09]*1;$/\$subdirectory_disabled = 1;\x0A\x0A1;/gm;
    }
    if ($env_setting) {
        my $setting = $this->read_file($env_setting);
        if ($setting) {
            while ( $setting
                =~ /([^\x09\x0A\x0D]+)\x09([^\x0A\x0D]*)[\x0A\x0D]*/sg )
            {
                my ( $k, $v ) = ( $1, $2 );
                $v =~ s/\x0B/\x0A/sg;
                $v = $this->replace_text( $v, $conv_table ) if $conv_table;
                $v =~ s/\\/\\\\/sg;
                $v =~ s/'/\\'/sg;
                $v = "'" . $v . "'" if $v !~ /^\-?[0-9\.]+$/;
                if ( $v =~ /\x0A/ ) {
                    $v =~ s/^';/'&#59;/s;
                    $data =~ s!\$$k[\x20\x09]*=.*?[^\\]';!\$$k = $v;!gs;
                }
                else {
                    $data
                        =~ s!^[\x20\x09]*\$$k[\x20\x09]*=[^\x0A\x0D]*;!\$$k = $v;!gm;
                }
            }
        }
    }
    $this->write_file( $env_path, \$data, 1 );
}

sub mk_sub_directory ($$$$$$$$$$$$$) {
    my ($this,                   $dir,
        $clone,                  $cgi_data_path,
        $cgi_data_root,          $shared_cgi_data_root,
        $home_dir,               $http_home_dir,
        $home_dir_id,            $home_url,
        $home_url_encoded,       $home_site,
        $javascript_dir,         $user_directory,
        $user_directory_encoded, $conv_table
    ) = @_;
    my ( @files, @directories, $default_id );
    local (*DIR);
    return unless opendir( DIR, $dir );
    while ( my $file = readdir(DIR) ) {
        my $path = $dir . '/' . $file;
        if (   $clone . '/' . $file eq $default::cgi_data_root
            || $clone . '/' . $file eq $default::shared_cgi_data_root )
        {
            next;
        }
        elsif ( $file !~ /^\.\./ && $file ne '.placeholder' && -f $path ) {
            push( @files, $file );
        }
        elsif ( $file !~ /^\./ && -d $path ) {
            push( @directories, $file );
        }
    }
    closedir(DIR);
    return unless $this->set_dir($clone);
    if ($main::multi_domain) {
        $default_id = '_' . $1 if $dir =~ /([0-9a-zA-Z_]+)$/;
    }
    else {
        ( $default_id = $http_home_dir . '/' . $1 ) =~ tr!/!_!
            if $http_home_dir
            && $http_home_dir ne '/'
            && $dir =~ /([0-9a-zA-Z_]+)$/;
    }
    my $meta_home_dir_id = $home_dir_id;
    $meta_home_dir_id =~ s/_/-/g;
    my $cgi_data_path_from_top = $cgi_data_path;
    $cgi_data_path_from_top =~ s!^\.\./\.\./!!;
    my $shared_cgi_data_root_from_top = $shared_cgi_data_root;
    $shared_cgi_data_root_from_top =~ s!^\.\./\.\./!!;
    my $home_domain
        = $env::server_name || ( ( $home_site =~ /^([\/:]+)/ ) ? $1 : '' );

    foreach my $file (@files) {
        if ((      $file =~ /\.[0-9a-zA-Z_]+$/
                && $file ne '.htaccess'
                && $file !~ /\.(cgi|pl|s?html?|ssi|xml|kml|dat|js|css)$/
                && $file ne 'name.txt'
            )
            || $file =~ /[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.dat$/
            || ( -s $dir . '/' . $file ) > 1000000
            )
        {
            $this->file_copy(
                $dir . '/' . $file,
                $clone . '/' . $file,
                (   ( $file =~ /\.(cgi|pl)$/ )
                    ? $env::cgi_file_mode
                    : $env::file_mode
                )
            );
            next;
        }
        my $is_xfile = 0;
        my $data     = $this->read_file( $dir . '/' . $file );
        if ( $file eq '.htaccess' ) {
            if ( $data =~ /# Gate Keeper - start -/ ) {
                my $spath = $this->path_conv( $clone, 'path' );
                $data
                    =~ s!(AuthUserFile[\x09\x20]+)[^\x0A]*(/\.htpasswd)!$1$spath$2!ig;
                $data
                    =~ s!(AuthGroupFile[\x09\x20]+)[^\x0A]*(/\.htgroup)!$1$spath$2!ig;
            }
            if ( $data =~ /# For Mobile - start -/ ) {
                my $this_dir
                    = $this->path_change( $clone, $ENV{'SCRIPT_NAME'},
                    'abs' );
                $this_dir .= '/' if $this_dir !~ m!/$!;
                $data
                    =~ s![^\x09\x0A\x0C\x0D\x20]*/m\.cgi/!${this_dir}m\.cgi/!sg;
            }
        }
        elsif ( $file =~ /\.cgi$/ ) {
            $data =~ s/use strict;/use strict;\x0Ause utf8;/sg
                if $data !~ /use utf8;/;
            $data =~ s/\->decode\(1,\s*'euc'/->decode\(1,'utf8'/sg;
            $data =~ s/\A#!.*?$/$default::perl_path/m if $default::perl_path;
            $data =~ s/<shared-cgi-data-root>/$shared_cgi_data_root/g;
            $data =~ s/<cgi-data-root>/$cgi_data_root/g;
            $data =~ s/<cgi-data-path>/$cgi_data_path/g;
            $data
                =~ s/<shared-cgi-data-root-from-top>/$shared_cgi_data_root_from_top/g;
            $data =~ s/<cgi-data-path-from-top>/$cgi_data_path_from_top/g;
        }
        elsif ( $file eq 'default.pl' ) {
            $data =~ s/package default;/package default;\x0Ause utf8;/s
                if $data !~ /use utf8;/;
            $data =~ s/<shared-cgi-data-root>/$shared_cgi_data_root/g;
            $data =~ s/<cgi-data-root>/$cgi_data_root/g;
            $data =~ s/<cgi-data-path>/$cgi_data_path/g;
            $data =~ s/<web-home-dir>/$user_directory/g;
            if ( $dir =~ m!/gatekeeper$! ) {
                $data =~ s/<home-dir>/$user_directory/g;
            }
            else {
                $data =~ s/<home-dir>/$home_dir/g;
            }
            $data =~ s/<home-dir-id>/$home_dir_id/g;
            $data =~ s/<default-id>/$default_id/g;
            $data =~ s/<cgi-name>/$file/g;
            if ( $dir =~ m!/setup$! ) {
                $data
                    =~ s/^[\x20\x09]*\$cgi_data_name[\x20\x09]*=[^\x0A\x0D]*;/\$cgi_data_name = '$default::cgi_data_name';/gm;
                $data
                    =~ s/^[\x20\x09]*\$shared_cgi_data_name[\x20\x09]*=[^\x0A\x0D]*;/\$shared_cgi_data_name = '$default::shared_cgi_data_name';/gm;
            }
        }
        elsif ( $file eq 'cmonos.js' ) {
            $data =~ s!(\.\./\.\./|/)lib/javascript!$javascript_dir!g;
            if ( defined $home_dir_id && $home_dir_id ne '' ) {
                $data =~ s/parent\.main/parent\.${home_dir_id}main/g;
                $data =~ s/parent\.navi/parent\.${home_dir_id}navi/g;
            }
        }
        elsif ( ( $file =~ /^index\.s?html?$/ || $file eq 'frame.html' )
            && $data =~ /<frameset/ )
        {
            $data
                =~ s!<meta\s*http-equiv="Content-Type"\s*content="text/html;\s*charset=EUC-JP"\s*/?>!<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />!sig;
            $data
                =~ s!<meta\s*charset="EUC-JP"\s*/?>!<meta charset="UTF-8" />!sig;
            $data =~ s! charset="EUC-JP"! charset="UTF-8"!sig;
            if ( defined $home_dir_id && $home_dir_id ne '' ) {
                $data =~ s/ name="main"/ name="${home_dir_id}main"/g;
                $data =~ s/ name="navi"/ name="${home_dir_id}navi"/g;
            }
            $data = $this->replace_text( $data, $conv_table ) if $conv_table;
        }
        elsif ( $file =~ /\.(s?html?|ssi|xml|kml|dat|css)$/i
            || ( $file =~ /\.js$/i && $dir !~ m!/lib/javascript! ) )
        {
            next if $file =~ /\.js$/;
            if ( $file =~ s/^site-navi-utf8\.ssi$/site-navi-euc.ssi/ ) {
                $data
                    =~ s!<meta\s*http-equiv="Content-Type"\s*content="text/html;\s*charset=UTF-8"\s*/?>!<meta http-equiv="Content-Type" content="text/html;charset=EUC-JP" />!sig;
                $data
                    =~ s!<meta\s*charset="UTF-8"\s*/?>!<meta charset="EUC-JP" />!sig;
            }
            else {
                $data
                    =~ s!<meta\s*http-equiv="Content-Type"\s*content="text/html;\s*charset=EUC-JP"\s*/?>!<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />!sig;
                $data
                    =~ s!<meta\s*charset="EUC-JP"\s*/?>!<meta charset="UTF-8" />!sig;
                $data =~ s! charset="EUC-JP"! charset="UTF-8"!sig;
                $data =~ s/\@charset "EUC-JP";/\@charset "UTF-8";/sig;
            }
            my $title       = $main::title;
            my $description = $main::description;
            my $table       = $conv_table;
            my $icon_dir
                = $this->base_change( $env::icon_dir, $clone . '/' . $file );
            my $is_cms = 0;
            $data =~ s/<site-title>/$title/g;
            $data =~ s/<site-description>/$description/g;
            $data =~ s/<icon-dir>/$icon_dir/g;
            $is_cms = 1 if $data =~ s/<user-directory>/$user_directory/gs;
            $is_cms = 1
                if $data
                =~ s/<user-directory-encoded>/$user_directory_encoded/gs;
            $data =~ s/<home-dir>/$user_directory/g;
            $is_cms = 1 if $data =~ s/<home-dir-id>/$home_dir_id/g;
            $is_cms = 0
                if $data
                =~ m!<link rel="canonical" href="<home-url>/[/0-9a-zA-Z_]+/start.cgi[^"]*?" />!s;
            $data =~ s/<meta-home-dir-id>/$meta_home_dir_id/g;
            $data =~ s/<home-url>/$home_url/g;
            $data =~ s/<home-url-encoded>/$home_url_encoded/g;
            $data =~ s/<home-site>/$home_site/gs;
            $data =~ s/<home-domain>/$home_domain/gs;
            $is_cms = 0
                if $is_cms
                && $file =~ /^([0-9]+)\./
                && ( -f $dir . '/' . $1 . '.spch'
                || $dir . '/' . $1 . '.dat' );

            if ($is_cms) {
                if ( $file =~ /\.shtml?$/i && !$env::ssi ) {
                    $file =~ s/\.shtml?$/.html/i;
                }
                elsif ( $file =~ /\.html?$/i && $env::ssi ) {
                    $file =~ s/\.html?$/.$env::ssi_ext/i;
                }
            }
            $data
                =~ s/(<form(?=[\x09\x0A\x0D\x20])[^>]*?[\x09\x0A\x0D\x20]action="(?!http:\/\/)[^"]+"[^>]*?>)(?![\x09\x0A\x0D\x20]*<input type="hidden" name="_" value="＿"[^>]*?>)/$1<input type="hidden" name="_" value="＿" \/>/sig;
            $data = $this->replace_text( $data, $table ) if $table;
            if ( $file =~ /^site-navi-(sjis|euc)\.ssi$/ ) {
                $this->jcode_convert( \$data, $1, 'utf8' );
            }
            $is_xfile = 1 if $file =~ /\.ssi$/;
        }
        $this->write_file(
            $clone . '/' . $file,
            \$data,
            1,
            (   ( $file =~ /\.(cgi|pl)$/ )
                ? $env::cgi_file_mode
                : $env::file_mode
            )
        );
        $this->mk_external_javascript( $clone . '/' . $file,
            '', '', '', $home_url )
            if $is_xfile;
    }
    if (@directories) {
        foreach my $directory (@directories) {
            $this->mk_sub_directory(
                $dir . '/' . $directory, $clone . '/' . $directory,
                $cgi_data_path,          $cgi_data_root,
                $shared_cgi_data_root,   $home_dir,
                $http_home_dir,          $home_dir_id,
                $home_url,               $home_url_encoded,
                $home_site,              $javascript_dir,
                $user_directory,         $user_directory_encoded,
                $conv_table
            );
        }
    }
}

sub targetable_directories ($$$) {
    my ( $this, $dir, $home_dir ) = @_;
    my ( @directories, @sub_directories );
    @directories = @sub_directories = ();
    local (*DIR);
    return unless opendir( DIR, $dir );
    $home_dir = '' if !defined $home_dir;
    while ( my $entry = readdir(DIR) ) {
        next if $entry =~ /^\./;
        my $path = $dir . '/' . $entry;
        if (   -d $dir . '/' . $entry
            && -f $dir . '/' . $entry . '/lib/env.pl' )
        {
            my ( $multi_domain, $server_name, $ssl, $subdirectory_disabled,
                $user_site_template_dir, $user_default_dir )
                = $this->subdir_options(
                $dir . '/' . $entry . '/lib/env.pl' );
            push( @directories, $entry ) if $subdirectory_disabled;
        }
    }
    closedir(DIR);
    foreach my $d (@directories) {
        push(
            @sub_directories,
            $home_dir . '/' . $d,
            $this->targetable_directories(
                $dir . '/' . $d,
                $home_dir . '/' . $d
            )
        );
    }
    return @sub_directories;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$env::subdirectory_disabled ) {

        #クラス登録
        Common::class_register( 's500', 'SubDirInstaller' );

        #メニュー登録
        $setting::jump_menu{'s500'} = 'サブディレクトリ増設';
    }
}

1;
