package SCAmountOfPayment;
require CMONOS::TableMaker::Field::Sum;
@ISA = qw(Sum);
use strict;
use utf8;
######################################################################
# 購入額モジュール
# SCAmountOfPayment.pm 1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-02-23
######################################################################
#	(0)-3 フィールドDB
sub db ($) {
    my $this = shift;
    return 'order';
}

sub _set ($$) {
    my ( $this, $setting ) = @_;
    $this->{'unit'}         = '円';
    $this->{'monetaryunit'} = '&yen;' if !$this->{'monetaryunit'};
    $this->{'sumEnabled'}   = 1 if !defined $this->{'sumEnabled'};
    $this->{'cols'}         = 8;
    return;
}

sub name ($) {
    my $this = shift;
    if ( $main::m eq 'TableViewer' && $this->{'sumEnabled'} ) {
        return ( ( defined $this->{'name'} ) ? $this->{'name'}
            : '[' . $this->{'id'} . ']' )
            . ( ( $this->{'br'} ) ? '<br />' : ' ' )
            . ( ( defined $default::english && $default::english )
            ? '([ ] is the sum)'
            : '([ ]内は合計額)' );
    }
    else {
        return ( ( defined $this->{'name'} )
            ? $this->{'name'}
            : '[' . $this->{'id'} . ']' );
    }
}

######################################################################
#	(1) obj->match(str);
sub number_from_data ($$) {
    my ( $this, $data ) = @_;
    my $amount;
    if ( $data =~ s/^(.*?)\x0A// ) {
        my ($total,           $tax,            $fee,
            $carriage,        $carriage_total, $download_key,
            $ptime,           $transaction_id, $status,
            $noshipping_only, $expense,        $dc_included,
            $discount
        ) = split( /\x09/, $1 );
        $amount
            = (   !$this->{'amountType'}
                || $this->{'amountType'} eq 'total'
                || !$total ) ? $total
            : ( $this->{'amountType'} eq 'subtotal' )
            ? ( $total - ( $carriage || 0 ) - ( $fee || 0 ) - ( $tax || 0 ) )
            : ( $this->{'amountType'} eq 'withouttax' )
            ? ( $total - ( $carriage || 0 ) - ( $fee || 0 ) )
            : $total;
    }
    return $amount;
}

######################################################################
#	(2) obj->form;
sub form {
    my ( $this, $status, $original, $path ) = @_;
    return '';
}

######################################################################
#	(3) obj->is_valid_data;
sub input_enabled ($) {
    my $this = shift;
    return 0;
}

sub is_valid_data ($;$) {
    my $this = shift;
    return 1;
}

######################################################################
#	(4) obj->usual_format(str,path,category,no);
sub usual_format {
    my ( $this, $str, $path, $c, $n ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    $str = $this->number_from_data($str);
    my $num = $str;
    $str
        = ( $this->{'numtype'} )
        ? $this->kansuuji( $this->{'numtype'}, $str )
        : $this->digit($str);
    $str = (
        ( $this->{'monetaryunit'} !~ /[^\x20-\x7F]/ )
        ? '<span class="unit">'
            . (
            ( $this->{'monetaryunit'} eq '&yen;' )
            ? '￥'
            : $this->{'monetaryunit'}
            )
            . '</span>'
        : ''
        )
        . $str
        . ( ( $this->{'monetaryunit'} =~ /[^\x20-\x7F]/ )
        ? '<span class="unit">' . $this->{'monetaryunit'} . '</span>'
        : '' )
        if $this->{'monetaryunit'};
    if ( $this->{'sumEnabled'} && $main::m eq 'TableViewer' ) {
        $this->{'sum'} += ( $num || 0 );
        $str
            .= ( ( $this->{'br'} ) ? '<div class="sum-info">' : ' ' ) . '['
            . $this->digit( $this->{'sum'} ) . ']'
            . ( ( $this->{'br'} ) ? '</div>' : '' );
    }
    return ( defined $str && $str ne '' )
        ? $this->style( $str, $this->{'style'}, $this->{'fontsize'},
        $this->{'fontcolor'}, $this->{'fontface'} )
        : '';
}

#---------------------------------------------------------------------
#	(4)-2 obj->format_for_mail(str);
sub format_for_mail_enabled ($) {
    my $this = shift;
    return ( $this->{'mailDisabled'} ) ? 0 : 1;
}

sub format_for_mail {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    $str = $this->number_from_data($str);
    return '' if !defined $str || $str eq '';
    my $num = $str;
    $str
        = ( $this->{'numtype'} )
        ? $this->kansuuji( $this->{'numtype'}, $str )
        : $this->digit($str);
    $str
        = ( ( $this->{'monetaryunit'} !~ /[^\x20-\x7F]/ )
        ? ( $this->{'monetaryunit'} eq '&yen;' )
                ? '￥'
                : $this->{'monetaryunit'}
        : '' )
        . $str
        . ( ( $this->{'monetaryunit'} =~ /[^\x20-\x7F]/ )
        ? $this->{'monetaryunit'}
        : '' )
        if $this->{'monetaryunit'};
    if ( $this->{'sumEnabled'} && $main::m eq 'TableViewer' ) {
        $this->{'sum'} += ( $num || 0 );
        $str .= '[' . $this->digit( $this->{'sum'} ) . ']';
    }
    return '[' . $this->name . "]\x0A" . $str . "\x0A\x0A";
}

#---------------------------------------------------------------------
#	(4)-3 obj->format_for_detail(str);
sub format_for_detail_enabled ($) {
    my $this = shift;
    return ( $this->{'detailDisabled'} ) ? 0 : 1;
}

#---------------------------------------------------------------------
#	(4)-4 obj->format_for_text(str);
sub format_for_text {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    $str = $this->number_from_data($str);
    return ( defined $str && $str ne '' ) ? $str : '';
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my $this = shift;
    my $name = 'data:' . $this->{'id'};
    $this->print_field_info_form($name);

    print '<fieldset><legend>購入額の定義</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="'
        . $name
        . '_amountType" value="total"'
        . ( ( !$this->{'amountType'} || $this->{'amountType'} eq 'total' )
        ? 'checked="checked"'
        : '' )
        . ' /> 税込み合計（小計+手数料+送料+消費税）</label></li>';
    print '<li><label><input type="radio" name="'
        . $name
        . '_amountType" value="withouttax"'
        . ( ( $this->{'amountType'} && $this->{'amountType'} eq 'withouttax' )
        ? 'checked="checked"'
        : '' )
        . ' />  税抜き合計（小計+手数料+送料）</label></li>';
    print '<li><label><input type="radio" name="'
        . $name
        . '_amountType" value="subtotal"'
        . ( ( $this->{'amountType'} && $this->{'amountType'} eq 'subtotal' )
        ? 'checked="checked"'
        : '' )
        . ' /> 小計</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset><legend>表示設定</legend>';
    print '<ol class="checkbox">';

    print '<fieldset><legend>表示画面</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_detailDisabled" value="1"'
        . ( ( $this->{'detailDisabled'} ) ? 'checked="checked"' : '' )
        . ' /> 詳細画面では表示しない</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_mailDisabled" value="1"'
        . ( ( $this->{'mailDisabled'} ) ? 'checked="checked"' : '' )
        . ' /> メールには記載しない</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset><legend>表示レコード合計額表示</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_sumEnabled"'
        . ( ( $this->{'sumEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> テーブル表示時に、表示レコードの合計額を表示する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name . '_br"'
        . ( ( $this->{'br'} ) ? ' checked="checked"' : '' )
        . ' /> 合計額表示の前に改行を入れる</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset><legend><label for="' . $name
        . '_monetaryunit">価格単位</label></legend>';
    print $this->mk_select_menu(
        $name . '_monetaryunit', '', '', $this->{'monetaryunit'},
        '円'   => '円',
        '&yen;' => '&yen;'
    );
    print '</fieldset>';

    print '</fieldset>';

    print '<fieldset>';
    print '<legend>検索設定</legend>';
    print '<label><input type="checkbox" name="'
        . $name
        . '_searchOption" value="1"'
        . ( ( defined $this->{'searchOption'} && $this->{'searchOption'} )
        ? ' checked="checked"'
        : '' )
        . ' /> より詳しい検索</label>';
    print '</fieldset>';
    $this->print_font_setting_form($name);
    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this = shift;
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };

    $main::data{ $this->{'id'} . '_searchOption' } = ''
        if !defined $main::data{ $this->{'id'} . '_searchOption' }
        || $main::data{ $this->{'id'} . '_searchOption' } eq '';
    $main::data{ $this->{'id'} . '_br' } = ''
        if !defined $main::data{ $this->{'id'} . '_br' }
        || $main::data{ $this->{'id'} . '_br' } eq '';
    $main::data{ $this->{'id'} . '_sumEnabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_sumEnabled' }
        || $main::data{ $this->{'id'} . '_sumEnabled' } eq '';
    $main::data{ $this->{'id'} . '_detailDisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_detailDisabled' }
        || $main::data{ $this->{'id'} . '_detailDisabled' } eq '';
    $main::data{ $this->{'id'} . '_mailDisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_mailDisabled' }
        || $main::data{ $this->{'id'} . '_mailDisabled' } eq '';
    return $this->is_valid_font_setting;
}

######################################################################
#	(8) obj->sort_enabled;
sub sample ($$) {
    my ( $this, $str ) = @_;
    return $this->number_from_data($str);
}

######################################################################
#	(9) obj->cookie_enabled;
sub cookie_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(10) obj->required_enabled;
sub required_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);
    if (   defined $default::type
        && $default::type eq 'order'
        && defined $env::checkout
        && $env::checkout
        && !$setting::field_for_mail )
    {
        Field::class_register( 'yorder_amount', 'SCAmountOfPayment',
            '購入額' );
    }
}

1;
