package Inquiry;
require CMONOS::TableMaker::TMCommon;
@ISA = qw(TMCommon);
use strict;
use utf8;
######################################################################
# 問い合わせモジュール
# Inquiry.pm ver1.1.8
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-02-05
######################################################################
# メイン
sub run ($) {
    my $this = shift;

    #---------------------------------------------------------------------
    #要求チェック
    unless ( defined $main::c
        && $main::c ne ''
        && $main::c !~ /[^0-9a-zA-Z_]/
        && defined $main::n
        && $main::n ne ''
        && $main::n !~ /[^0-9a-zA-Z_]/
        && -d "$default::db/$main::c"
        && ref $field::obj{$main::mf} )
    {
        $this->print_dialog( 'inq',
            'エラー : データ指定に不正があります。(101)' );
    }

    #メールフィールドオブジェクト読み込み
    $cgi::confirmation_disabled = $cgi::enctype_multipart = 0;
    $setting::field_for_mail = 1;
    $this->use_field( 'inquiry', $default::field_pref_dir . '/inquiryfield',
        2, 2, 'no' );

    #---------------------------------------------------------------------
    #フィールドチェック
    my (%db,      $user,        $name,   $key,
        $title,   $name_db,     $key_db, $title_db,
        %setting, $mailsetting, @mailto
    );
    eval 'use ' . $env::dbm_file . ';';
    use Fcntl qw(:DEFAULT);
    tie( %db, $env::dbm_file, "$default::db/$main::c/$main::mf",
        O_RDONLY, $env::dbm_mode );
    $mailsetting = $this->decode_utf8( $db{ $this->encode_utf8($main::n) } )
        if ref tied %db;
    untie %db;
    while ( $mailsetting =~ s/^([0-9a-zA-Z_]+)\:([^\x0A\x0D]*)\x0A//s ) {
        $setting{$1} = $2;
        $setting{$1} =~ s/&#10;/\x0A/sig;
    }

    #メール送信設定のチェック
    if (!$setting{'mailto'}
        || (   !$field::obj{$main::mf}->{'mailtoMaster'}
            && !$field::obj{$main::mf}->{'mailtoOwners'}
            && !$field::obj{$main::mf}->{'mailtoStaff'}
            && (   !$field::obj{$main::mf}->{'mailtoUser'}
                || !$setting{'mailto'} )
        )
        )
    {
        $this->print_dialog( 'inq',
            'エラー : データ指定に不正があります。' );
    }

    #---------------------------------------------------------------------
    #アクセス権チェック
    return if $this->is_pending_record_detail( $main::c, $main::n );

    #期間チェック
    if (   $field::obj{$main::mf}->{'period'}
        && ref $field::obj{ $field::obj{$main::mf}->{'period'} }
        && $field::obj{ $field::obj{$main::mf}->{'period'} }->{'class'}
        && $field::obj{ $field::obj{$main::mf}->{'period'} }->{'class'} eq
        'time_span' )
    {
        my $pdb = $field::obj{ $field::obj{$main::mf}->{'period'} }->db;
        my $period;
        tie( %db, $env::dbm_file, "$default::db/$main::c/$pdb", O_RDONLY,
            $env::dbm_mode );
        $period = $this->decode_utf8( $db{ $this->encode_utf8($main::n) } )
            if ref tied %db;
        untie %db;
        use Time::Local;
        if (   $period
            && $period
            =~ m!^([0-9]{0,4})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2}) \- ([0-9]{0,4})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})!
            )
        {
            my $start = ($1)
                ? timelocal(
                ( $6 || 0 ),
                ( $5 || 0 ),
                ( $4 || 0 ),
                ( $3 || 1 ),
                ( ($2) ? $2 - 1 : 0 ), $1
                )
                : 0;
            my $end = ($7)
                ? timelocal(
                ( $12 || 0 ),
                ( $11 || 0 ),
                ( $10 || 0 ),
                ( $9  || 1 ),
                ( ($8) ? $8 - 1 : 0 ), $7
                )
                : 0;
            my $now = time;
            if ( $start > $now || ( $end && $end < $now ) ) {
                $this->print_dialog(
                    'inq',
                    (   $field::obj{$main::mf}->{'unavailableMessage'}
                            || 'エラー : データ指定に不正があります。'
                    )
                );
            }
        }
    }

    #---------------------------------------------------------------------
    #名前フィールド
    $name_db = $field::obj{ $field::for{'name'} }->db
        if $field::for{'name'} && ref $field::obj{ $field::for{'name'} };
    if ( defined $name_db && $name_db ) {
        my %name;
        tie( %name, $env::dbm_file, "$default::db/$main::c/$name_db",
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %name ) {
            $name = $this->detag_text(
                $field::obj{ $field::for{'name'} }->usual_format(
                    $this->decode_utf8(
                        $name{ $this->encode_utf8($main::n) }
                    )
                )
            );
        }
        untie %name;
    }

    #キーフィールド
    $key_db = $field::obj{ $field::for{'key'} }->db
        if $field::for{'key'} && ref $field::obj{ $field::for{'key'} };
    if ( defined $key_db && $key_db ) {
        my %key;
        tie( %key, $env::dbm_file, "$default::db/$main::c/$key_db",
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %key ) {
            $key = $this->detag_text(
                $field::obj{ $field::for{'key'} }->usual_format(
                    $this->decode_utf8(
                        $key{ $this->encode_utf8($main::n) }
                    )
                )
            );
        }
        untie %key;
    }

    #タイトルフィールド
    $title_db = $field::obj{ $field::for{'title'} }->db
        if $field::for{'title'} && ref $field::obj{ $field::for{'title'} };
    if ( defined $title_db && $title_db ) {
        my %title;
        tie( %title, $env::dbm_file, "$default::db/$main::c/$title_db",
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %title ) {
            $title = $this->detag_text(
                $field::obj{ $field::for{'title'} }->usual_format(
                    $this->decode_utf8(
                        $title{ $this->encode_utf8($main::n) }
                    )
                )
            );
        }
        untie %title;
    }

    #メールフィールド
    if ( defined $setting{'mailto'} && $setting{'mailto'} !~ /[^0-9]/ ) {
        my %mail;
        tie( %mail, $env::dbm_file,
            "$default::db/$main::c/$setting{'mailto'}",
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %mail ) {
            my $addr
                = $this->decode_utf8( $mail{ $this->encode_utf8($main::n) } );
            if ($addr) {
                push( @mailto, $addr );
                $user = $addr
                    if !$field::obj{ $setting{'mailto'} }->{'Hidden'};
            }
        }
        untie %mail;
    }
    push( @mailto, ( $setting::general{'master'} || $env::master_email ) )
        if $field::obj{$main::mf}->{'mailtoMaster'}
        && ( $setting::general{'master'} || $env::master_email );
    push( @mailto, $setting::general{'owners'} )
        if $field::obj{$main::mf}->{'mailtoOwners'}
        && $setting::general{'owners'};
    push( @mailto, $setting::general{'staff'} )
        if $field::obj{$main::mf}->{'mailtoStaff'}
        && $setting::general{'staff'};
    $this->print_dialog( 'inq',
        'エラー : データ指定に不正があります。(103)' )
        if !@mailto;
    $this->{'title'} = $title || $key || $name;

    #---------------------------------------------------------------------
    #リンク元
    $this->query_optimize;
    my $referer_url = $this->{'referer_url'} = $this->referer_url;
    my $samewindow = ( ref $field::obj{$main::mf}
            && $field::obj{$main::mf}->{'sameWindow'} ) ? 1 : 0;

    #---------------------------------------------------------------------
    #メール送信

    #入力のチェック
    my ( $input_is_valid, $msg, $from_name );
    if ( defined $main::a && $main::a eq 'm' ) {
        $input_is_valid = 1;
        foreach my $fid ( sort { $a <=> $b } keys %inquiry::ORDER ) {
            $input_is_valid = 0
                unless ref $inquiry::obj{ $inquiry::ORDER{$fid} }
                && $inquiry::obj{ $inquiry::ORDER{$fid} }->is_valid_data;
        }

        #連続チェック
        if (   defined $field::obj{$main::mf}->{'ipStopper'}
            && $field::obj{$main::mf}->{'ipStopper'}
            && $setting::general{'latestmail'} )
        {
            my ( $mtime, $ip )
                = split( /:/, $setting::general{'latestmail'} );
            if (   $ip
                && $mtime
                && $ip eq (
                           $ENV{'REMOTE_HOST'}
                        || $this->to_host( $ENV{'REMOTE_ADDR'} )
                )
                && time - $mtime <= $field::obj{$main::mf}->{'ipStopper'}
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    '連続送信は制限されています。しばらくしてから送信しなおしてください。'
                );
                $input_is_valid = 0;
            }
        }

        #メール送信
        if ( $input_is_valid && !$main::cancel ) {
            if ( $cgi::confirmation_disabled || $main::ok ) {

                #入力通知メール
                my ( $subject, $from, $url );
                $url = $this->permalink_of_detailview( $main::c, $main::n );
                $subject
                    = $inquiry::obj{ $inquiry::for{'subject'} }
                    ->format_for_text(
                    $main::data{ $inquiry::for{'subject'} }{'str'} )
                    if defined $inquiry::for{'subject'}
                    && ref $inquiry::obj{ $inquiry::for{'subject'} };
                $subject = $field::obj{$main::mf}->{'subject'}
                    if !defined $subject || $subject eq '';
                $this->detag( \$subject );

                $from = $main::data{ $inquiry::for{'from'} }{'str'}
                    if defined $main::data{ $inquiry::for{'from'} }{'str'}
                    && $main::data{ $inquiry::for{'from'} }{'str'};
                $from_name = $main::data{ $inquiry::for{'name'} }{'str'}
                    if defined $from && $from ne '';
                $this->detag( \$from_name )
                    if defined $from_name && $from_name ne '';
                $from = $from_name . ' <' . $from . '>'
                    if defined $from && $from ne '';

                $msg = $this->notice_mail(
                    $subject,
                    '発信元レコード:'
                        . $url
                        . (
                          ($key)   ? ' (' . $key . ')'
                        : ($title) ? ' (' . $title . ')'
                        : ($name)  ? ' (' . $name . ')'
                        :            ''
                        ),
                    \%inquiry::obj,
                    \%main::data,
                    $from, @mailto
                );

                #自動返信メール
                if ($from && $field::obj{$main::mf}->{'resmsg'}
                    || (   $field::obj{$main::mf}->{'autores'}
                        && $setting{'resmsg'} )
                    )
                {
                    my ( $res_subject, $res_msg );
                    $res_msg
                        = ( ( $setting{'resmsg'} )
                        ? $setting{'resmsg'}
                        : $field::obj{$main::mf}->{'resmsg'} );
                    $res_subject
                        = ( ( $setting{'ressubject'} )
                        ? $setting{'ressubject'}
                        : $field::obj{$main::mf}->{'ressubject'} );

                    $res_subject =~ s/<name>/$from_name/sig;
                    $res_msg =~ s/<name>/$from_name/sig;
                    $res_msg =~ s/<msg>/$msg/sig;
                    $res_msg =~ s/<url>/$url/sig;

                    $cgi::SendMail->send_mail(
                        'To'   => $from,
                        'From' => (
                            ($user)
                            ? $user
                            : ( ( $setting::general{'master'} )
                                ? ( ( $setting::general{'mastername'} )
                                    ? $setting::general{'mastername'} . ' <'
                                        . $setting::general{'master'} . '>'
                                    : $setting::general{'master'} )
                                : $env::master_email
                            )
                        ),
                        'Subject' => $res_subject,
                        'message' => $res_msg . "\x0A\x0A",
                        'info'    => 0
                    );
                }

                #後処理
                foreach my $fid ( sort { $a <=> $b } keys %inquiry::ORDER ) {
                    $inquiry::obj{ $inquiry::ORDER{$fid} }->settle_up
                        if ref $inquiry::obj{ $inquiry::ORDER{$fid} };
                }
                $cgi::input->clear_parameters_trash;
                $cgi::input->clear_clipboard;

                #一般情報更新
                my (%general);
                tie(%general, $env::dbm_file,
                    $default::preference_dir . '/general',
                    O_RDWR | O_CREAT,
                    $env::dbm_mode
                );
                if ( ref tied %general ) {
                    $general{'latestmail'} = $this->encode_utf8(
                        time . ':'
                            . (
                                   $ENV{'REMOTE_HOST'}
                                || $this->to_host( $ENV{'REMOTE_ADDR'} )
                            )
                    );
                }
                untie %general;

                #書き込みお礼
                my ( $thanks, $thankstitle );

                #ユーザー指定
                if (   $field::obj{$main::mf}->{'usrthanks'}
                    && $setting{'thanks'} )
                {
                    $thanks = $setting{'thanks'};
                    $thankstitle
                        = ( $setting{'thankstitle'} )
                        ? $setting{'thankstitle'}
                        : ( $field::obj{$main::mf}->{'thankstitle'} )
                        ? $field::obj{$main::mf}->{'thankstitle'}
                        : '';

                }
                elsif ( defined $field::obj{$main::mf}->{'thanks'}
                    && $field::obj{$main::mf}->{'thanks'} )
                {
                    $thanks      = $field::obj{$main::mf}->{'thanks'};
                    $thankstitle = $field::obj{$main::mf}->{'thankstitle'}
                        if defined $field::obj{$main::mf}->{'thankstitle'}
                        && $field::obj{$main::mf}->{'thankstitle'};
                }
                if ( defined $thankstitle && $thankstitle ) {
                    $thankstitle =~ s/<name>/$from_name/sig;
                    $this->escape( \$thankstitle );
                    $this->push_window_title($thankstitle);
                }

                #ページヘッダ
                $this->print_page_header('inq');

                #ユーザー指定
                if ( defined $thanks && $thanks ) {
                    $this->escape( \$from_name );
                    $this->escape( \$msg );

                    #$from_name =~ s/(\x0A|\x0D|\x0D\x0A)/<br \/>/sg;
                    #$msg =~ s/(\x0A|\x0D|\x0D\x0A)/<br \/>/sg;
                    $thanks =~ s/<name>/$from_name/sig;
                    $thanks =~ s/<msg>/$msg/sig;
                    $thanks =~ s/<url>/$url/sig;
                    print $this->tag_fix( $thanks, 2, 0, 0, 1 );

                    #英語表記
                }
                elsif ( defined $default::english && $default::english ) {
                    print '<p id="tm-thanks">Thank you for your e-mail.</p>';

                    #日本語
                }
                else {
                    print '<p id="tm-thanks">送信しました。</p>';
                }

                #入力確認画面
            }
            else {
                $this->print_page_header('inq');
                $this->print_inquiry_message( $name, $key, $title );
                print $this->form_start(
                    'key'    => 'inq',
                    'name'   => 'input',
                    'method' => 'post',
                    'onsubmit' =>
                        'return cmonos.form.item(this).isNotSubmitted()',
                    'required' => \@setting::form
                );
                print '<input type="hidden" name="a" value="m" />';
                print '<p id="tm-confirmation">'
                    . ( ($default::english)
                    ? 'Confirmation.'
                    : '下記内容に問題がなければ「送信」ボタンを押してください。'
                    ) . '</p>';
                print '<table class="field-table"><tbody>';
                foreach ( sort { $a <=> $b } keys %inquiry::ORDER ) {
                    my $id = $inquiry::ORDER{$_};
                    next
                        unless ref $inquiry::obj{$id}
                        && $inquiry::obj{$id}->format_for_mail_enabled;
                    $inquiry::obj{$id}
                        ->print_confirmation_form_table_row( '', $main::c,
                        $main::n );
                }
                print '</tbody><tbody>';
                print
                    '<tr class="tbody-start"><td class="submit-box" colspan="2"><input type="submit" name="cancel=1" value="'
                    . ( ($default::english) ? 'Cancel' : 'キャンセル' );
                print
                    '" class="cancel" /><input type="submit" name="ok=1" value="'
                    . ( ($default::english) ? 'OK' : '送信' )
                    . '" class="save" /></td></tr>';
                print '</tbody></table></form>';
            }
            $this->print_close_button( $referer_url, $samewindow );

            #ページフッタ
            $this->print_page_footer('inq');
            return;

        }
    }

    #---------------------------------------------------------------------
    #書き出し
    my ( $template, $temp, %temp, $path );

    #フォーム
    my ($formcheck);

    $this->print_page_header('inq');
    $this->print_inquiry_message( $name, $key, $title );
    print $this->form_start(
        'key'      => 'inq',
        'name'     => 'input',
        'multi'    => $cgi::enctype_multipart,
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="m" />';

    #ファイル格納ディレクトリ
    $path = "$default::file/$main::c/$main::n";
    $key
        = ( defined $default::english && $default::english )
        ? 'This'
        : 'これ'
        if !defined $key || $key eq '';
    $title
        = ( defined $default::english && $default::english )
        ? 'This'
        : 'これ'
        if !defined $title || $title eq '';
    $name
        = ( defined $default::english && $default::english )
        ? 'You'
        : 'あなた'
        if !defined $name || $name eq '';

    print '<table class="field-table"><tbody>';
    my $cookie_enabled = 0;
    foreach my $i ( sort { $a <=> $b } keys %inquiry::ORDER ) {
        my $field = $inquiry::obj{ $inquiry::ORDER{$i} };
        next unless ref $field && $field->format_for_mail_enabled;
        if ( defined $field->{'default'} ) {
            $field->{'default'} =~ s/<name>/$name/sgi;
            $field->{'default'} =~ s/<key>/$key/sgi;
            $field->{'default'} =~ s/<title>/$title/sgi;
        }
        print $field->form( '', '', $path );
        $cookie_enabled = 1 if $field->cookie_enabled && $field->{'cookie'};
        $formcheck = 1;
    }

    if ( defined $formcheck && $formcheck ) {
        print '</tbody><tbody>';
        print '<tr class="tbody-start"><td class="submit-box" colspan="2">';
        if ( defined $cookie_enabled && $cookie_enabled ) {
            print '<label><input type="checkbox" name="set_cookie" value="1"'
                . ( ( $cookie::read{$cookie::name} || $main::set_cookie )
                ? ' checked="checked"'
                : '' )
                . ' />'
                . ( ( defined $default::english && $default::english )
                ? 'Remember Me'
                : '情報を記憶する' )
                . '</label> ';
        }
        print '<input type="submit" value="'
            . ( ( defined $default::english && $default::english )
            ? 'Send'
            : '送信' )
            . '" /></td></tr>';
    }

    print '</tbody></table></form>';
    $this->print_close_button( $referer_url, $samewindow );

    #ページフッタ
    $this->print_page_footer('inq');
}

sub has_opener ($) {
    my $this = shift;
    return 1;
}

sub push_optional_title ($$) {
    my ( $this, $title_path ) = @_;
    $this->import_db( $default::preference_dir . '/category',
        \%setting::category, 1 )
        if !%setting::category;
    if (   ref $setting::category{'ORDER'}
        && keys %{ $setting::category{'ORDER'} } > 1
        && ref $setting::category{$main::c}
        && defined $setting::category{$main::c}{'name'}
        && $setting::category{$main::c}{'name'} ne '' )
    {
        push( @$title_path, $setting::category{$main::c}{'name'} );
    }
    push( @$title_path, $this->{'title'} )
        if defined $this->{'title'} && $this->{'title'} ne '';
    return 1;
}

sub push_optional_topicpath ($$) {
    my ( $this, $topicpath_pair ) = @_;
    $this->import_db( $default::preference_dir . '/category',
        \%setting::category, 1 )
        if !%setting::category;
    if (   ref $setting::category{'ORDER'}
        && keys %{ $setting::category{'ORDER'} } > 1
        && ref $setting::category{$main::c}
        && defined $setting::category{$main::c}{'name'}
        && $setting::category{$main::c}{'name'} ne '' )
    {
        push( @$topicpath_pair,
                  './start.cgi?'
                . ( ($main::mod) ? 'm=' . $main::mod . '&' : '' ) . 'c='
                . $main::c => $setting::category{$main::c}{'name'} );
    }
    push( @$topicpath_pair, $this->{'referer_url'} => $this->{'title'} )
        if defined $this->{'title'} && $this->{'title'} ne '';
    return 1;
}

sub print_close_button ($;$$) {
    my ( $this, $referer_url, $samewindow ) = @_;
    print '<div id="close-button">';
    if ($samewindow) {
        $this->use_deco($cgi::output);
        print $cgi::Deco->button( 'return', $setting::page{'backButton'},
            $referer_url, ( ($default::english) ? 'Return.' : '戻る' ) );
    }
    else {
        print $this->close_button(
            $referer_url,
            $setting::page{'closeButton'},
            $setting::page{'backButton'}
        );
    }
    print '</div>';
}

sub print_inquiry_message ($$$$) {
    my ( $this, $name, $key, $title ) = @_;
    return
        if !defined $setting::page{'inq'}{'template'}
        || $setting::page{'inq'}{'template'} eq '';
    $key
        = ( defined $default::english && $default::english )
        ? 'This'
        : 'これ'
        if !defined $key || $key eq '';
    $title
        = ( defined $default::english && $default::english )
        ? 'This'
        : 'これ'
        if !defined $title || $title eq '';
    $name
        = ( defined $default::english && $default::english )
        ? 'You'
        : 'あなた'
        if !defined $name || $name eq '';
    my $template = $setting::page{'inq'}{'template'};
    $template =~ s/<name>/$name/sig;
    $template =~ s/<key>/$key/sig;
    $template =~ s/<title>/$title/sig;
    print $this->tag_fix( $template, 2, 0, 0, 1 );
}

sub set_rss_link ($;$) {
    my ( $this, $output ) = @_;

#$this->import_db($default::preference_dir . '/category',\%setting::category,1) if !%setting::category;
#$this->_set_rss_link($default::file . '/' . $main::c,((ref $setting::category{$main::c} && defined $setting::category{$main::c}{'name'} && $setting::category{$main::c}{'name'} ne '') ? $setting::category{$main::c}{'name'} : ''));
    $this->_set_rss_link(
        $default::file,
        (   (   defined $setting::general{'cginame'}
                    && $setting::general{'cginame'} ne ''
            ) ? $setting::general{'cginame'} : ''
        ),
        $output
    );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::deco     = 1;    #テキスト装飾設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み
    $setting::table    = 1;    #テーブル設定読み込み
    $setting::page     = 1;    #ページ設定読み込み

    $setting::field_db_name = 'inputfield';    #フィールド設定DB
    $setting::field         = 2
        ; #フィールド設定読み込み（2=オリジナルも読み込む）
    $setting::field_style_disabled = 2
        ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視
    $setting::field_tag_enabled
        = 'no'; #tag=タグのみ/all=全ての設定/no=タグをつけない
    $setting::field_for_mail = 0;

    #設定種キー設定
    $setting::security_key = 'inq';   #デフォルトセキュリティキー
    $setting::key          = 'inq';   #ページ設定キー

    #クッキー
    $cookie::id_enabled = 1;          #ID情報クッキー読み込み
    $cookie::default_enabled
        = 1;    #ユーザー入力情報クッキー読み込み
    $cookie::name = 'INQ';
    $cookie::path = $ENV{'SCRIPT_NAME'};

    #デフォルトキー
    $main::k = 'inq' if !defined $main::k || !$main::k;

    #必須フォーム
    @setting::form = (
        'm',  'c', 'n',   'mf',     's',   'w',   'f', 'u',
        'fv', 'q', 'and', 'search', 'mod', 'uni', 'ct'
    );
    if ( $ENV{'QUERY_STRING'}
        || ( $ENV{'REQUEST_METHOD'} eq 'POST' && !$cgi::input->referer_is_me )
        )
    {
        undef $main::a;
    }

    #イメージCSS
    $setting::image_css = 'order-image.css'
        if $default::type && $default::type eq 'order';

    #クラス登録
    Common::class_register( 'inq', 'Inquiry' );
}

1;
