package Mailer;
require CMONOS::TableMaker::TMCommon;
@ISA = qw(TMCommon);
use strict;
use utf8;
######################################################################
# メール配信モジュール
# Mailer.pm ver1.1.7
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-04-10
######################################################################
# メイン
sub run ($) {
    my $this = shift;
    $main::a = '' if !defined $main::a;

    #メール配信設定読み込み
    $this->import_db( $default::preference_dir . '/mail', \%setting::mail,
        0 );
    if ( defined $main::data{'send'} && $main::data{'send'} eq 'to' ) {
        $setting::mail{'limiter'} = 1;
    }
    elsif ( !$setting::mail{'limiter'} || $setting::mail{'limiter'} > 100 ) {
        $setting::mail{'limiter'} = 100;
    }

    #メールフィールド取得
    my ( $mail_fields, @temp, @mail_fields );
    $this->scalar_to_array( \$setting::mail{'mailFields'} );
    $this->scalar_to_array( \$main::data{'mailFields'} );
    @{ $main::data{'mailFields'} } = grep {
        my $temp = $_;
        grep( $_ eq $temp, @{ $setting::mail{'mailFields'} } )
        } @{ $main::data{'mailFields'} }
        if ref $setting::mail{'mailFields'}
        && @{ $setting::mail{'mailFields'} };

    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        my $id = $field::ORDER{$i};
        next unless ref $field::obj{$id} && $field::obj{$id}->{'e-mail'};
        next
            unless !ref $setting::mail{'mailFields'}
            || !@{ $setting::mail{'mailFields'} }
            || grep( $_ == $id, @{ $setting::mail{'mailFields'} } );
        $mail_fields
            .= '<option value="'
            . $id . '"'
            . (
            (          !ref $main::data{'mailFields'}
                    || !@{ $main::data{'mailFields'} }
                    || grep( $_ eq $id, @{ $main::data{'mailFields'} } )
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $field::obj{$id}->name
            . ( ( $field::obj{$id}->{'authorityName'} )
            ? $field::obj{$id}->{'authorityName'}
            : '' )
            . '</option>';
        push( @mail_fields, $id )
            if grep( $_ eq $id, @{ $main::data{'mailFields'} } );
        push( @temp, $id );
    }
    if ( !defined $mail_fields || $mail_fields eq '' ) {
        $this->print_page_header($setting::key);
        print
            '「メール配信設定」に問題があります。「宛先フィールド」が設定されていないので、メール配信機能は利用できません。';
        $this->print_page_footer($setting::key);
        return;
    }
    @mail_fields = @temp if !@mail_fields;

    #カテゴリー指定がない場合カテゴリー1を指定
    push( @main::c, ( ( defined $main::c ) ? $main::c : 1 ) ) if !@main::c;
    undef $main::c;
    $this->query_optimize;

    #---------------------------------------------------------------------
    #入力チェック
    if ( $main::a eq 'send' ) {
        if (   !ref $main::data{'mailFields'}
            || !@{ $main::data{'mailFields'} } )
        {
            push(
                @{ $this->{'dialog'} },
                '宛先フィールドが選択されていません。'
            );
            $main::a = '';
        }
        if ( $this->is_invalid_email_address_header( $main::data{'email'} ) )
        {
            $main::data{'email'} = ($main::email) ? $main::email : '';
            if ($this->is_invalid_email_address_header(
                    $main::data{'email'}
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    '発信者メールアドレスを確認してください。'
                );
                $main::a = '';
            }
        }
        if ( !defined $main::data{'subject'} || $main::data{'subject'} eq '' )
        {
            push(
                @{ $this->{'dialog'} },
                'メールの表題が入力されていません。'
            );
            $main::a = '';
        }
        if ( !defined $main::data{'comment'} || $main::data{'comment'} eq '' )
        {
            push(
                @{ $this->{'dialog'} },
                'メールの本文が入力されていません。'
            );
            $main::a = '';
        }
        if ( defined $main::data{'attachment'}
            && $main::data{'attachment'} ne '' )
        {
            if ( !$setting::mail{'attachment'} ) {
                push(
                    @{ $this->{'dialog'} },
                    'ファイルは添付できません。'
                );
                delete $main::data{'attachment'};
            }
            elsif (
                $cgi::input->get_posted_file_size_from_param(
                    'data:attachment') > $setting::mail{'attachment'}
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    '添付ファイルのファイルサイズが大きすぎます。'
                );
                delete $main::data{'attachment'};
                $main::a = '';
            }
            else {
                $cgi::input->save_file(
                    'data:attachment',
                    (   ( $main::data{'attachment'} =~ /[^0-9a-zA-Z_\-\.,]/ )
                        ? '!'
                        : '<client>'
                    ),
                );
            }
        }
        if ( $main::data{'name'} ) {
            $main::data{'name'} =~ s/\@(?=[0-9a-zA-Z_])/ \@ /g;
            $main::data{'name'} =~ s/<//g;
            $main::data{'name'} =~ s/>//g;
        }

    }
    elsif ( $main::a eq 'pswd' ) {
        if (   !ref $main::data{'mailFields'}
            || !@{ $main::data{'mailFields'} } )
        {
            push(
                @{ $this->{'dialog'} },
                '登録メールアドレスフィールドが選択されていません。'
            );
            $main::a = '';
        }
    }

    #---------------------------------------------------------------------
    #メール送信
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #---------------------------------------------------------------------
    #ヘッダ
    if ( $main::a eq 'send' ) {
        $this->print_page_header($setting::key);
        print
            '<div id="progress"><span id="scount">0</span>件 送信...</div>'
            . $this->javascript(
            "\x0Avar c = 0; var counter = document.getElementById('scount');"
            );
    }

    #---------------------------------------------------------------------
    #検索〜処理
    my ($from,   $to,       $name,            $key,
        $title,  $ctime,    $mtime,           $name_db,
        $key_db, $title_db, $count,           $scount,
        %addr,   %sent,     $attachment_path, $attachment,
        $info,   $error
    );
    $scount = $count = 0;
    $name_db = $field::obj{ $field::for{'name'} }->db
        if $field::for{'name'} && ref $field::obj{ $field::for{'name'} };
    $key_db = $field::obj{ $field::for{'key'} }->db
        if $field::for{'key'} && ref $field::obj{ $field::for{'key'} };
    $title_db = $field::obj{ $field::for{'title'} }->db
        if $field::for{'title'} && ref $field::obj{ $field::for{'title'} };
    if ( $main::a eq 'send' && !ref $cgi::SendMail ) {
        require CMONOS::CGI_Lib::SendMail;
        $cgi::SendMail
            = SendMail->new( $env::sendmail, $env::master_email, '',
            $cgi::input, 1 );
    }
    $from = ( ( $setting::mail{'fromOption'} || $main::email eq '' )
            && $main::data{'email'} ) ? $main::data{'email'} : $main::email;
    $from = $main::data{'name'} . ' <' . $from . '>'
        if defined $main::data{'name'}
        && $main::data{'name'} ne ''
        && !$this->is_invalid_email_address($from);

    #添付
    if ( $main::clipboard_id && !$main::data{'attachment'} ) {
        $cgi::input->inherit_posted_file_properties('data:attachment');
    }
    $attachment_path
        = $cgi::input->get_posted_file_path_from_param('data:attachment');
    delete $main::data{'delete_attachment'} if $main::data{'attachment'};
    $attachment
        = ( !$main::data{'delete_attachment'} && $attachment_path )
        ? '<data:attachment>'
        : '';

    #オーナー発信の場合発信者情報を付加しない
    $info = ( $this->is_staff ) ? 0 : 1;

    $this->sort_and_search;
    no warnings qw(untie numeric);
    foreach my $no (
        sort {
            $this->sort_by( \$this->{'match'}{$b}, \$this->{'match'}{$a} )
                || $this->sort_by( \$this->{'sort'}{$a},
                \$this->{'sort'}{$b}, $main::w )
                || $a <=> $b
                || $a cmp $b
        } keys %{ $this->{'sort'} }
        )
    {
        my ( $n, $c ) = split( /\@/, $no );
        my $dir = "$default::db/$c/";
        $name
            = ( $setting::mail{'defaultName'} )
            ? $setting::mail{'defaultName'}
            : '皆さん';
        $key
            = ( $setting::mail{'defaultKey'} )
            ? $setting::mail{'defaultKey'}
            : 'これ';
        $title
            = ( $setting::mail{'defaultTitle'} )
            ? $setting::mail{'defaultTitle'}
            : 'これ';
        $to = $mtime = $ctime = '';

        #名前の取得
        if ( $main::a eq 'send' && $setting::mail{'limiter'} == 1 ) {

            #名前フィールド
            if ( defined $name_db && $name_db ) {
                unless ( ref tied %{ $this->{'DB'}{$c}{$name_db} } ) {
                    tie(%{ $this->{'DB'}{$c}{$name_db} },
                        $env::dbm_file, $dir . $name_db,
                        O_RDONLY,       $env::dbm_mode
                    );
                }
                $name
                    = $this->decode_utf8(
                    $this->{'DB'}{$c}{$name_db}{ $this->encode_utf8($n) } )
                    if ref tied %{ $this->{'DB'}{$c}{$name_db} };
            }
            if ( defined $key_db && $key_db ) {
                unless ( ref tied %{ $this->{'DB'}{$c}{$key_db} } ) {
                    tie(%{ $this->{'DB'}{$c}{$key_db} },
                        $env::dbm_file, $dir . $key_db,
                        O_RDONLY,       $env::dbm_mode
                    );
                }
                $key
                    = $this->decode_utf8(
                    $this->{'DB'}{$c}{$key_db}{ $this->encode_utf8($n) } )
                    if ref tied %{ $this->{'DB'}{$c}{$key_db} };
            }
            if ( defined $title_db && $title_db ) {
                unless ( ref tied %{ $this->{'DB'}{$c}{$title_db} } ) {
                    tie(%{ $this->{'DB'}{$c}{$title_db} },
                        $env::dbm_file, $dir . $title_db,
                        O_RDONLY,       $env::dbm_mode
                    );
                }
                $title
                    = $this->decode_utf8(
                    $this->{'DB'}{$c}{$title_db}{ $this->encode_utf8($n) } )
                    if ref tied %{ $this->{'DB'}{$c}{$title_db} };
            }
            unless ( ref tied %{ $this->{'DB'}{$c}{'ctime'} } ) {
                tie(%{ $this->{'DB'}{$c}{'ctime'} },
                    $env::dbm_file, $dir . 'ctime',
                    O_RDONLY,       $env::dbm_mode
                );
            }
            $ctime
                = $this->decode_utf8(
                $this->{'DB'}{$c}{'ctime'}{ $this->encode_utf8($n) } )
                if ref tied %{ $this->{'DB'}{$c}{'ctime'} };
            unless ( ref tied %{ $this->{'DB'}{$c}{'mtime'} } ) {
                tie(%{ $this->{'DB'}{$c}{'mtime'} },
                    $env::dbm_file, $dir . 'mtime',
                    O_RDONLY,       $env::dbm_mode
                );
            }
            $mtime
                = $this->decode_utf8(
                $this->{'DB'}{$c}{'mtime'}{ $this->encode_utf8($n) } )
                if ref tied %{ $this->{'DB'}{$c}{'mtime'} };
            $to = 'RCPT TO';
        }

        foreach my $m (@mail_fields) {
            my ( @address, $d );
            $d = $field::obj{$m}->db;
            unless ( ref tied %{ $this->{'DB'}{$c}{$d} } ) {
                tie( %{ $this->{'DB'}{$c}{$d} },
                    $env::dbm_file, $dir . $d, O_RDONLY, $env::dbm_mode );
            }
            @address = $this->get_address(
                $this->decode_utf8(
                    $this->{'DB'}{$c}{$d}{ $this->encode_utf8($n) }
                )
            );
            if (@address) {
                $scount += @address;
                if ( $main::a eq 'send' || $main::a eq 'pswd' ) {
                    foreach my $m (@address) {
                        $addr{$m}++ if !%sent || !$sent{$m};
                    }
                }
            }
        }

        #メール配信
        if ( $main::a eq 'send' ) {
            my $addr_count = scalar keys %addr;
            if ( $addr_count >= $setting::mail{'limiter'} ) {
                $addr_count = $this->send_out(
                    \%addr, \%sent,      $name,  $key,
                    $title, $ctime,      $mtime, $from,
                    $to,    $attachment, $info
                );
                if ($addr_count) {
                    $count += $addr_count;
                    print $this->javascript(
                        "c += $addr_count; counter.innerHTML = c;");
                }
                else {
                    $error = 1;
                }
            }
            %addr = () if $setting::mail{'limiter'} == 1;
        }
    }
    $this->untie_all_dbm;

    if (%addr) {

        #送信し残したアドレス
        if ( $main::a eq 'send' ) {
            while (%addr) {
                my $addr_count = $this->send_out(
                    \%addr, \%sent,      $name, $key,
                    $title, '',          '',    $from,
                    '',     $attachment, $info
                );
                if ($addr_count) {
                    $count += $addr_count;
                    print $this->javascript(
                        "c += $addr_count; counter.innerHTML = c;");
                }
                else {
                    $error = 1;
                }
            }

            #パスワード発行
        }
        elsif ( $setting::mail{'pswd'} && $main::a eq 'pswd' ) {
            my ( $addr, $pcount, $allcount, $skip );
            $this->print_dialog( 'pswd',
                '設定エラー！ : パスワード管理データベースディレクトリのパスが設定されていません。'
            ) if !$this->use_register;
            $pcount = $allcount = $skip = 0;
            while ( ($addr) = each %addr ) {
                $allcount++;
                if ( $main::data{'skip'} && $cgi::Register->exist($addr) ) {
                    $skip++;
                }
                elsif ( $cgi::Register->register($addr) ) {
                    $pcount++;
                }
            }
            push(
                @{ $this->{'dialog'} },
                $allcount
                    . ' 件中 '
                    . $pcount
                    . ' 件のメールアドレスを一括登録しました。'
                    . (
                    ($skip)
                    ? $skip . ' 件は既に登録されています。'
                    : ''
                    )
            );
            undef %addr;
        }
    }

    #確認のために発信者に送信
    if ( $main::a eq 'send' && $main::data{'cc'} ) {
        my $addr_count = 0;
        $addr{ $this->get_address( $main::data{'email'} ) } = 1;
        $addr_count = $this->send_out(
            \%addr,                         \%sent,
            $main::data{'name'},            $setting::mail{'defaultKey'},
            $setting::mail{'defaultTitle'}, time,
            time,                           $from,
            $to,                            $attachment,
            $info
        ) if %addr;
        print $this->javascript("c += $addr_count; counter.innerHTML = c;")
            if $addr_count;
    }
    $cgi::input->clear_clipboard
        if $main::data{'delete_attachment'} || $main::a eq 'send';
    if ( $main::a eq 'send' ) {
        print $this->javascript(
            "document.getElementById('progress').style.display = \"none\";");
    }
    else {
        $this->print_page_header($setting::key);
    }

    #---------------------------------------------------------------------
    #該当者無し
    if ( $this->{'mcount'} == 0 ) {
        print
            '<p>条件に一致するレコードがみつかりませんでした。</p>';

        #---------------------------------------------------------------------
        #確認ページ
    }
    elsif ( $scount == 0 ) {
        print '<p>条件に一致する '
            . $this->{'mcount'}
            . '件のいずれにも、メールアドレスがみつかりませんでした。</p>';
    }

    #---------------------------------------------------------------------
    #確認ページ
    if ( $main::a eq 'send' ) {
        if ( defined $error && $error ) {
            print '<p>条件に一致する '
                . $this->{'mcount'}
                . '件に登録された';
            print $count
                . '件のメールアドレスに、メッセージを配信しましたが、';
            print
                '下記アドレスへの送信に失敗しました。</p><ol>';
            foreach my $m ( @{ $cgi::SendMail->{'bad_address'} } ) {
                print '<li><a href="&#109;&#97;&#105;&#108;&#116;&#111;&#58;'
                    . $m . '">'
                    . $m
                    . '</a></li>';
            }
            print '</ol>';
        }
        else {
            print '<p>条件に一致する '
                . $this->{'mcount'}
                . '件に登録された ';
            print $count
                . '件のメールアドレスに、メッセージを配信しました。</p>';
        }

        #---------------------------------------------------------------------
        #入力ページ
    }
    else {

        print '<p>条件に一致する '
            . $this->{'mcount'}
            . '件に登録された ';
        print $scount
            . '件のメールアドレスに、メッセージを配信します。</p>';

        #フォーム
        print $this->form_start(
            'key'    => 'mail',
            'name'   => 'input',
            'method' => 'post',
            'multi'  => (
                (   defined $setting::mail{'attachment'}
                        && $setting::mail{'attachment'}
                ) ? 1 : 0
            ),
            'onsubmit' =>
                'return confirm(\'この操作は取り消せません。本当にこの内容で送信しますか？\') && cmonos.form.item(this).isNotSubmitted()',
            'required' => \@setting::form
        );
        print '<input type="hidden" name="a" value="send" />';

        print '<table class="field-table"><tbody>';

        print
            '<tr><td class="field-name"><label for="data:mailFields">宛先フィールド</label></td>';
        print
            '<td class="field"><select name="data:mailFields" id="data:mailFields" multiple="multiple">';
        print $mail_fields;
        print '</select></td></tr>';

        #処理時間予測
        my ( $sleep, $bcc_count, $bcc_min, $to_min, $bcc_sec, $to_sec );
        $sleep
            = (   !$setting::mail{'sleepTime'}
                || $setting::mail{'sleepTime'} < 1 )
            ? 0.5
            : $setting::mail{'sleepTime'};
        $bcc_count
            = ( $setting::mail{'limiter'} > $scount )
            ? $scount
            : $setting::mail{'limiter'};
        $bcc_sec
            = int( ( 100 * $sleep * $scount / ( $bcc_count || 1 ) ) + 0.5 )
            / 100;
        $bcc_min = int( $bcc_sec / 60 );
        $bcc_sec = $bcc_sec - ( $bcc_min * 60 );
        $to_sec  = int( ( 100 * $sleep * $scount ) + 0.5 ) / 100;
        $to_min  = int( $to_sec / 60 );
        $to_sec  = $to_sec - ( $to_min * 60 );

        if ( $setting::mail{'limiter'} == 1 ) {
            print
                '<tr class="tbody-end"><td class="field-name"><label for="data:send">配信方法</label></td><td class="field">';
            print '個別に送信 [' . $to_min . '分' . $to_sec . '秒]';
        }
        else {
            print
                '<tr class="tbody-end"><td class="field-name"><span class="datatitle">配信方法</span></td><td class="field">';
            print $this->mk_select_menu(
                'data:send', '', '', $main::data{'send'},
                'bcc' => 'Bccで配信 ['
                    . $bcc_min . '分'
                    . $bcc_sec . '秒]',
                'to' => '個別に送信 ['
                    . $to_min . '分'
                    . $to_sec . '秒]'
            );
        }
        print
            ' <span class="note">（[ ]内は作業時間予測）</span></td></tr>';
        print
            '<tr><td class="field-name"><span class="datatitle">重複送信の禁止</span></td><td class="field"><label for="data:avoidRedundancy"><input type="checkbox" name="data:avoidRedundancy" id="data:avoidRedundancy" value="1"'
            . (
            (   $main::data{'avoidRedundancy'}
                    || ( !defined $main::data{'subject'}
                    && $setting::mail{'avoidRedundancy'} )
            ) ? ' checked="checked"' : ''
            )
            . ' />一度送信したメールアドレスへの重複送信を禁止する</label></td></tr>';

        print '</tbody><tbody>';

        $main::data{'name'} = $setting::mail{'defaultFrom'}
            if !defined $main::data{'name'} || $main::data{'name'} eq '';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:name">発信者名</label></td>';
        print
            '<td class="field"><input type="text" name="data:name" id="data:name" size="50" value="'
            . $this->escape( $main::data{'name'} )
            . '" maxlength="50" /></td>';
        print '</tr>';

        if (   $setting::mail{'fromOption'}
            || !defined $main::email
            || $main::email eq '' )
        {
            $main::data{'email'} = ''
                if $this->is_invalid_email_address( $main::data{'email'} );
            print
                '<tr class="tbody-end"><td class="field-name"><label for="data:subject">発信者メールアドレス</label></td><td class="field">';
            print
                '<input type="text" name="data:email" id="data:email" size="50" value="'
                . $this->escape( $main::data{'email'} )
                . '" /></td></tr>';
        }
        else {
            print
                '<tr class="tbody-end"><td class="field-name"><span class="datatitle">発信者メールアドレス</span></td><td class="field">';
            print $this->escape($main::email);
            print
                '<input type="hidden" name="data:email" id="data:email" value="'
                . $this->escape($main::email)
                . '" /></td></tr>';
        }

        print '</tbody><tbody>';

        $main::data{'subject'}
            = ( $setting::mail{'defaultSubject'} )
            ? $setting::mail{'defaultSubject'}
            : ''
            if !defined $main::data{'subject'}
            || $main::data{'subject'} eq '';
        print
            '<tr><td class="field-name"><label for="data:subject">メールの表題</label></td>';
        print
            '<td class="field"><input type="text" name="data:subject" id="data:subject" size="72" value="'
            . $this->escape( $main::data{'subject'} )
            . '" /></td></tr>';

        $main::data{'comment'}
            = ( $setting::mail{'defaultComment'} )
            ? $setting::mail{'defaultComment'}
            : ''
            if !defined $main::data{'comment'}
            || $main::data{'comment'} eq '';
        print
            '<tr><td class="field-name"><label for="data:comment">メールの本文</label></td>';
        print
            '<td class="field"><textarea name="data:comment" id="data:comment" cols="72" rows="20" wrap="soft">'
            . $this->escape( $main::data{'comment'} )
            . '</textarea>';
        print '<div class="toolbox">';
        print
            '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:comment\').quotationByMenu(this)" title="これらのタグはメール送信時に対応する値に変換されます。">';
        print '<option value="">▼挿入する</option>';
        print
            '<option value="&lt;name&gt;">&lt;name&gt;[名前フィールド('
            . (
            ( $field::for{'name'} && ref $field::obj{ $field::for{'name'} } )
            ? $field::obj{ $field::for{'name'} }->name
            : '&lt;name&gt;のデフォルト' )
            . ')の値]</option>';
        print '<option value="&lt;key&gt;">&lt;key&gt;[キーフィールド('
            . (
            ( $field::for{'key'} && ref $field::obj{ $field::for{'key'} } )
            ? $field::obj{ $field::for{'key'} }->name
            : '&lt;key&gt;のデフォルト' )
            . ')の値]</option>';
        print
            '<option value="&lt;title&gt;">&lt;title&gt;[タイトルフィールド('
            . (
            (   $field::for{'title'}
                    && ref $field::obj{ $field::for{'title'} }
            )
            ? $field::obj{ $field::for{'title'} }->name
            : '&lt;title&gt;のデフォルト'
            ) . ')の値]</option>';
        print
            '<option value="&lt;ctime&gt;">&lt;ctime&gt;[作成時]</option>';
        print
            '<option value="&lt;mtime&gt;">&lt;mtime&gt;[更新時]</option>';
        print '</select>';
        print '</div>';
        print '</td></tr>';

        if ( defined $setting::mail{'attachment'}
            && $setting::mail{'attachment'} )
        {
            print
                '<tr><td class="field-name"><label for="data:attachment">添付ファイル</label></td>';
            print
                '<td class="field"><span id="data:attachment-wrapper"><input type="file" name="data:attachment" id="data:attachment" value="'
                . $this->escape( $main::data{'attachment'} )
                . '" /></span><input type="button" value="&#139; リセット" onclick="cmonos.form.item(this.form).fieldname(\'data:attachment\').reset()">';
            if ( $attachment_path && !$main::data{'delete_attachment'} ) {
                my %properties
                    = $cgi::input->get_posted_file_properties_from_param(
                    'data:attachment');
                unless ( ref $cgi::Image ) {
                    require CMONOS::CGI_Lib::Image;
                    $cgi::Image = Image->new;
                }
                my ( $w, $h ) = $cgi::Image->info($attachment_path);
                print '<div id="attachment-sample">';
                if ( $w && $h ) {
                    print $cgi::Image->image_tag( $attachment_path, $w, $h,
                        0, $properties{'filename'}, '', 560 );
                }
                else {
                    print '<a href="'
                        . $attachment_path
                        . '" target="_blank">'
                        . $this->escape( $properties{'filename'} ) . '</a>';
                }
                print
                    ' <label><input type="checkbox" name="data:delete_attachment" value="1" />削除</label></div>';
            }
            print '</td></tr>';
        }

        print '</tbody><tbody>';
        print '<tr class="tbody-start"><td class="submit-box" colspan="2">';

        print '<label><input type="checkbox" name="data:cc" value="1"'
            . ( ( defined $main::data{'cc'} && $main::data{'cc'} )
            ? ' checked="checked"'
            : '' )
            . ' />発信者にもコピーを送る</label> ';
        print '<input type="submit" value="送信" class="save" /></td></tr>';

        print '</tbody></table></form>';

        #パスワード一括発行フォーム
        if ( defined $setting::mail{'pswd'} && $setting::mail{'pswd'} ) {

            #フォーム
            print $this->form_start(
                'key'    => 'mail',
                'name'   => 'pswd',
                'id'     => 'tm-register',
                'method' => 'post',
                'onsubmit' =>
                    'return confirm(\'この操作は取り消せません。本当に一括登録しますか？\') && cmonos.form.item(this).isNotSubmitted()',
                'required' => \@setting::form
            );
            print
                '<fieldset><legend>パスワード一括発行</legend><p>条件に一致する '
                . $this->{'mcount'}
                . '件中、メールアドレスの見つかった ';
            print $scount
                . '件にパスワードを自動発行します。</p>';
            print '<input type="hidden" name="a" value="pswd" />';

            print '<table class="field-table"><tbody>';

            print
                '<tr><td class="field-name"><label for="register-mailFields">登録メールアドレスフィールド</label></td>';
            print
                '<td class="field"><select name="data:mailFields" id="register-mailFields" multiple="multiple">';
            print $mail_fields;
            print '</select></td></tr>';

            print
                '<tr><td class="submit-box" colspan="2"><label><input type="checkbox" name="data:skip" value="1" checked="checked" />登録済みメールアドレスをスキップする</label> ';
            print ' <input type="submit" value="一括登録" /></td></tr>';

            print '</tbody></table></fieldset></form>';
        }
    }

    $main::mod = 'TableViewer' if !$main::mod;
    print '<div class="panel">';
    print $cgi::Deco->button( 'return', $setting::page{'returnButton'},
        $this->referer_url, ( ($default::english) ? 'Return' : '戻る' ) );
    print '</div>';
    $this->print_page_footer($setting::key);
    return;
}

######################################################################
# メール配信
sub send_out ($$$$$$$$) {
    my ($this,  $addr,  $sent, $name, $key,        $title,
        $ctime, $mtime, $from, $to,   $attachment, $info
    ) = @_;
    my ( $subject, $comment, @addr, $check );

    $name
        = ( $setting::mail{'defaultName'} )
        ? $setting::mail{'defaultName'}
        : 'みな'
        if !defined $name || !$name;
    $key
        = ( $setting::mail{'defaultKey'} )
        ? $setting::mail{'defaultKey'}
        : 'これ'
        if !defined $key || !$key;
    $title
        = ( $setting::mail{'defaultTitle'} )
        ? $setting::mail{'defaultTitle'}
        : 'これ'
        if !defined $title || !$title;
    $ctime = $this->date( $ctime, 0, 'gmt' . $setting::mail{'datetype'} )
        if $ctime;
    $mtime = $this->date( $mtime, 0, 'gmt' . $setting::mail{'datetype'} )
        if $mtime;

    $subject = ( $main::data{'subject'} ) ? $main::data{'subject'} : '';
    $comment = ( $main::data{'comment'} ) ? $main::data{'comment'} : '';
    $subject =~ s/<name>/$name/sgi;
    $comment =~ s/<name>/$name/sgi;
    $subject =~ s/<key>/$key/sgi;
    $comment =~ s/<key>/$key/sgi;
    $subject =~ s/<title>/$title/sgi;
    $comment =~ s/<title>/$title/sgi;
    $subject =~ s/<ctime>/$ctime/sgi;
    $comment =~ s/<ctime>/$ctime/sgi;
    $subject =~ s/<mtime>/$mtime/sgi;
    $comment =~ s/<mtime>/$mtime/sgi;

    @addr = keys(%$addr);
    @addr = splice( @addr, 0, 100 );
    foreach my $m (@addr) {
        delete $$addr{$m};
        $sent->{$m}++ if $main::data{'avoidRedundancy'};
    }

    sleep( $setting::mail{'sleepTime'} ) if $setting::mail{'sleepTime'};
    $check = $cgi::SendMail->send_mail(
        'RCPT TO' => \@addr,
        'To'      => $to,
        'From'    => $from,
        'Subject' => $subject,
        'message' => $comment,
        'info'    => $info,
        'file'    => $attachment
    );
    push( @{ $this->{'dialog'} }, $cgi::SendMail->dialog );
    $check = @addr if $check;
    return $check;
}

sub set_rss_link ($;$) {
    my ( $this, $output ) = @_;

#$this->import_db($default::preference_dir . '/category',\%setting::category,1) if !%setting::category;
#$this->_set_rss_link($default::file . '/' . $main::c,((ref $setting::category{$main::c} && defined $setting::category{$main::c}{'name'} && $setting::category{$main::c}{'name'} ne '') ? $setting::category{$main::c}{'name'} : ''));
    $this->_set_rss_link(
        $default::file,
        (   (   defined $setting::general{'cginame'}
                    && $setting::general{'cginame'} ne ''
            ) ? $setting::general{'cginame'} : ''
        ),
        $output
    );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::deco     = 1;    #テキスト装飾設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み
    $setting::table    = 1;    #テーブル設定読み込み
    $setting::page     = 1;    #ページ設定読み込み

    $setting::field_db_name = 'inputfield';    #フィールド設定DB
    $setting::field         = 2
        ; #フィールド設定読み込み（2=オリジナルも読み込む）
    $setting::field_style_disabled = 2
        ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視
    $setting::field_tag_enabled
        = 'no'; #tag=タグのみ/all=全ての設定/no=タグをつけない

    #設定種キー設定
    $setting::security_key = 'mail';  #デフォルトセキュリティキー

    #クッキー
    $cookie::id_enabled = 1;          #ID情報クッキー読み込み

    #必須フォーム
    @setting::form = (
        'm',  'c', 's',   'w',      'f',   'u',
        'fv', 'q', 'and', 'search', 'mod', 'uni'
    );
    push( @setting::form, 'order_item' )
        if $default::type && $default::type eq 'order';
    $main::mod = $main::m if !$main::m || $main::m ne 'Mailer';
    if ($ENV{'QUERY_STRING'}
        || (   $ENV{'REQUEST_METHOD'}
            && $ENV{'REQUEST_METHOD'} eq 'POST'
            && !$cgi::input->referer_is_me )
        )
    {
        undef $main::a;
    }

    #設定種キー設定
    $setting::key = 'mail';    #ページ設定キー

    #デフォルトキー
    $main::k = 'mail' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'mail', 'Mailer' );

    #イメージCSS
    $setting::image_css = 'order-image.css'
        if $default::type && $default::type eq 'order';
}

1;
