package TileViewer;
require CMONOS::TableMaker::Viewer;
@ISA = qw(Viewer);
use strict;
use utf8;
######################################################################
# タイルビューワモジュール
# TileViewer.pm ver1.1.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-01-21
######################################################################
# 表示フィールド取得
sub get_required_fields ($) {
    my $this = shift;
    return
        if ref $this->{'required_fields'} && @{ $this->{'required_fields'} };
    $this->get_required_fields_for_template;
    if ( $setting::page{'newIconLimit'} || $setting::page{'upIconLimit'} ) {
        $this->{'key_field'} = (
            $field::for{'key'} && grep( $_ eq $field::for{'key'},
                @{ $this->{'required_fields'} } )
            )
            ? $field::for{'key'}
            : (
            $field::for{'title'}
                && grep( $_ eq $field::for{'title'},
                @{ $this->{'required_fields'} } )
            ) ? $field::for{'title'}
            : '';
        if (   $setting::page{$setting::key}{'template'}
            && $this->{'key_field'} )
        {
            $setting::page{$setting::key}{'template'}
                =~ s/(<!--field(?=\s)[^>]*\s+id="$this->{'key_field'}"[^>]*-->.*?<!--\/field(?!\w)[^>]*-->)/$this->add_field_footer($1,'<newicon>')/seig;
        }
    }
}

######################################################################
# コンテンツヘッダ
sub print_contents_header ($) {
    my $this = shift;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    $main::uni = $main::u if !$main::uni;

    #ツールボックス
    $setting::page{$setting::key}{'toolboxTop'} = 1
        if !defined $setting::page{$setting::key}{'toolboxTop'}
        && !defined $setting::page{$setting::key}{'toolboxBottom'};
    $this->print_toolbox('top')
        if defined $setting::page{$setting::key}{'toolboxTop'}
        && $setting::page{$setting::key}{'toolboxTop'};

    #前後ページへのジャンプ
    $this->print_page_jump_form;

    #タイルフレーム調整
    $this->{'tile_frame_start'} = $this->{'tile_frame_end'} = '';
    my $cell_style = '';
    if ( $setting::page{$setting::key}{'tileFrame'}
        && ref $frame::obj{'frame'}
        { $setting::page{$setting::key}{'tileFrame'} } )
    {
        my ( $temp, $contents, @frame_setting );
        @frame_setting = (
            'framewidth'  => $setting::page{$setting::key}{'tileWidth'},
            'frameheight' => $setting::page{$setting::key}{'tileHeight'},
        );
        push( @frame_setting,
            'contentsalign', $setting::page{$setting::key}{'tileAlign'} )
            if $setting::page{$setting::key}{'tileAlign'};
        push( @frame_setting,
            'contentsvalign', $setting::page{$setting::key}{'tileValign'} )
            if $setting::page{$setting::key}{'tileHeight'}
            && $setting::page{$setting::key}{'tileValign'};
        $contents = '_C_';
        $temp
            = $cgi::Deco->frame( 'frame',
            $setting::page{$setting::key}{'tileFrame'},
            \$contents, '', 'no', '', 0, @frame_setting );
        ( $this->{'tile_frame_start'}, $this->{'tile_frame_end'} )
            = split( /_C_/, $temp );
        $cell_style
            .= 'vertical-align:'
            . $setting::page{$setting::key}{'tileValign'} . ';'
            if !$setting::page{$setting::key}{'tileHeight'}
            && $setting::page{$setting::key}{'tileValign'};

    }
    elsif ($setting::page{$setting::key}{'tileWidth'}
        || $setting::page{$setting::key}{'tileHeight'} )
    {
        $cell_style
            .= 'width:' . $setting::page{$setting::key}{'tileWidth'} . 'px;'
            if $setting::page{$setting::key}{'tileWidth'};
        $cell_style
            .= 'height:'
            . $setting::page{$setting::key}{'tileHeight'} . 'px;'
            if $setting::page{$setting::key}{'tileHeight'};
        $cell_style
            .= 'text-align:'
            . $setting::page{$setting::key}{'tileAlign'} . ';'
            if $setting::page{$setting::key}{'tileAlign'};
        $cell_style
            .= 'vertical-align:'
            . $setting::page{$setting::key}{'tileValign'} . ';'
            if $setting::page{$setting::key}{'tileValign'};
    }

    my $handle = $this->{'filehandle'};
    if ( !defined $setting::page{$setting::key}{'tileTableEnabled'}
        || $setting::page{$setting::key}{'tileTableEnabled'} )
    {

        #一ページ記事数
        $setting::page{$setting::key}{'tableCols'} = 3
            if !$setting::page{$setting::key}{'tableCols'};
        $main::u
            = int(
            $main::u / $setting::page{$setting::key}{'tableCols'} + 0.99999 )
            * $setting::page{$setting::key}{'tableCols'};
        $main::uni = $main::u if !$main::uni;

        $this->{'Table'} = Table->new;
        if ( ref $table::obj{ $setting::page{$setting::key}{'table'} } ) {
            %{ $this->{'Table'} }
                = %{ $table::obj{ $setting::page{$setting::key}{'table'} } };
        }
        else {
            $this->{'Table'}->set(
                'tableclass' => 'tile-table',
                'class0'     => 'tile-cell',
                ( ($cell_style) ? ( 'style0' => $cell_style ) : () )
            );
        }

        #テーブル開始
        print $handle ( $this->{'Table'}->table );

        #列制限
        $this->{'Table'}->limit( $setting::page{$setting::key}{'tableCols'} );

    }
    else {
        $this->{'li_attributes'}
            = ($cell_style) ? ' style="' . $cell_style . '"' : '';
        print $handle '<ul class="tile">';
    }
}

######################################################################
# コンテンツフッタ
sub print_contents_footer ($) {
    my $this = shift;

    #テーブル終了
    my $handle = $this->{'filehandle'};
    if ( ref $this->{'Table'} ) {
        print $handle ( $this->{'Table'}->trtd( '', 1 ) );
        print $handle ( $this->{'Table'}->_table );
    }
    else {
        print $handle '</ul>';
    }

    #前後ページへのジャンプ
    $this->print_page_jump_form;

    $this->print_toolbox('bottom')
        if defined $setting::page{$setting::key}{'toolboxBottom'}
        && $setting::page{$setting::key}{'toolboxBottom'};
    $this->print_download_form;
    $this->print_page_return_form;

}

######################################################################
# レコード
sub print_record ($$$) {
    my ( $this, $c, $n ) = @_;
    my ( $temp, %temp, $path, $id );
    $path = "$default::file/$c/$n";
    $this->set_title_link( $c, $n );
    foreach $id ( @{ $this->{'required_fields'} } ) {
        next
            unless ref $field::obj{$id}
            && $field::obj{$id}->usual_format_enabled;
        my $db = $field::obj{$id}->db;
        unless ( !$db || ref tied %{ $this->{'DB'}{$c}{$db} } ) {
            tie( %{ $this->{'DB'}{$c}{$db} },
                $env::dbm_file, "$default::db/$c/$db", O_RDONLY,
                $env::dbm_mode );
        }
        if ( $this->{'has_template'} ) {
            $temp{$id}
                = ($db)
                ? $this->decode_utf8(
                $this->{'DB'}{$c}{$db}{ $this->encode_utf8($n) } )
                : '';
        }
        else {
            $temp
                .= '<div>'
                . (
                ( defined $field::for{'key'} && $id eq $field::for{'key'} )
                ? '<strong>'
                : '' )
                . $field::obj{$id}->usual_format(
                (   ($db) ? $this->decode_utf8(
                        $this->{'DB'}{$c}{$db}{ $this->encode_utf8($n) }
                    ) : ''
                ),
                $path, $c, $n
                )
                . (
                ( defined $field::for{'key'} && $id eq $field::for{'key'} )
                ? '</strong>'
                : '' )
                . ( ( $this->{'key_field'} && $id eq $this->{'key_field'} )
                ? $this->new_item_icon( $c, $n )
                : '' )
                . '</div>';
        }
    }
    if ( $this->{'has_template'} ) {
        $temp = $setting::page{$setting::key}{'template'};
        my $newicon = $this->new_item_icon( $c, $n );
        $temp =~ s/<newicon>/$newicon/sig;
        $newicon = $this->escape($newicon);
        $temp =~ s/&lt;newicon&gt;/$newicon/sig;
        $temp = $this->{'template'}
            ->replace( $temp, \%temp, '', $path, $c, $n );
    }
    $this->clear_title_link;

    return if !defined $temp || $temp eq '';
    my $handle = $this->{'filehandle'};
    print $handle (
        ( ref $this->{'Table'} )
        ? $this->{'Table'}->trtd(
            $this->{'tile_frame_start'} . $temp . $this->{'tile_frame_end'}
            )
        : '<li'
            . $this->{'li_attributes'} . '>'
            . $this->{'tile_frame_start'}
            . $temp
            . $this->{'tile_frame_end'} . '</li>'
    );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    $setting::field_tag_enabled
        = 'no'; #tag=タグのみ/all=全ての設定/no=タグをつけない
    $setting::field_style_disabled = 2
        ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視

    #設定種キー設定
    $setting::key = 'tile'
        if !defined $setting::key || !$setting::key;    #ページ設定キー

    #デフォルトキー
    $main::k = 'tile' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'tile', 'TileViewer' );
}

1;
