package InputPage;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
no warnings qw(once);
######################################################################
# 入力ページ設定モジュール
# InputPage.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox, 'headerNeighborLink', 'footerNeighborLink' );
    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    my $this = shift;
    $preference::id   = 'input';                          # データキー
    $preference::name = '入力ページレイアウト'; # 設定パネル名
    $preference::title
        = '入力ページのレイアウトについて設定します。';
}

sub print_sub_preference_panel ($;$$) {
    my ( $this, $id, $key ) = @_;
    print '<fieldset>';
    print '<legend>入力画面設定</legend>';

    print '<fieldset>';
    print '<legend>前後レコードへのリンク</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="data:'
        . $id
        . 'headerNeighborLink" id="data:'
        . $id
        . 'headerNeighborLink" value="1"'
        . (
        (   defined $main::data{ $key . 'headerNeighborLink' }
                && $main::data{ $key . 'headerNeighborLink' }
        ) ? ' checked="checked"' : ''
        ) . ' /> ページ上部に表示する</label></li>';
    print '<li><label><input type="checkbox" name="data:'
        . $id
        . 'footerNeighborLink" id="data:'
        . $id
        . 'footerNeighborLink" value="1"'
        . (
        (   defined $main::data{ $key . 'footerNeighborLink' }
                && $main::data{ $key . 'footerNeighborLink' }
        ) ? ' checked="checked"' : ''
        ) . ' /> ページ下部に表示する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>ウィンドウサイズ</legend>';
    print
        '<span id="windowsize"><label>幅 <input type="text" name="data:inputWindowWidth" id="data:inputWindowWidth" value="'
        . $this->escape( $main::data{'inputWindowWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span></label> ';
    print
        '&times; <label>高さ <input type="text" name="data:inputWindowHeight" id="data:inputWindowHeight" value="'
        . $this->escape( $main::data{'inputWindowHeight'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span></label></span>';
    print
        ' <label><input type="checkbox" name="data:sameWindowNewDataButton" id="data:sameWindowNewDataButton" value="1"'
        . ( ( $main::data{'sameWindowNewDataButton'} )
        ? ' checked="checked"'
        : '' )
        . ' onclick="cmonos.element.item(\'windowsize\').disappear(this.checked)" /> 入力画面を同じウィンドウで開く</label>';
    print '</fieldset>';
    print $this->javascript(
        'cmonos.element.item(\'windowsize\').disappear(true)')
        if $main::data{'sameWindowNewDataButton'};

    print '</fieldset>';
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    $main::data{'sameWindowNewDataButton'} = ''
        if !defined $main::data{'sameWindowNewDataButton'}
        || $main::data{'sameWindowNewDataButton'} eq '';
    if (   defined $main::data{'inputWindowWidth'}
        && $main::data{'inputWindowWidth'} ne ''
        && $this->is_nan( $main::data{'inputWindowWidth'} ) )
    {
        push(
            @{ $this->{'dialog'} },
            'ウィンドウサイズ(幅)を更新できませんでした。半角数字にしてください。'
        );
        delete $main::data{'inputWindowWidth'};
    }
    if (   defined $main::data{'inputWindowHeight'}
        && $main::data{'inputWindowHeight'} ne ''
        && $this->is_nan( $main::data{'inputWindowHeight'} ) )
    {
        push(
            @{ $this->{'dialog'} },
            'ウィンドウサイズ(高さ)を更新できませんでした。半角数字にしてください。'
        );
        delete $main::data{'inputWindowHeight'};
    }
    return;
}

######################################################################
#sub import {
BEGIN {
    #クラス登録
    Common::class_register( 't020', 'InputPage' );

    #ツール登録
    $setting::sub_jump_menu{'page'}{'t020'} = '入力ページ';
    $setting::selected_jump_menu{'t020'} = 't000'
        ; #サブメニューがあるときメインメニューのデフォルトキー

    $preference::page{'t020'} = 'input';
}

1;
