package MailerSetting;
require CMONOS::CGI_Lib::Tools::Preference;
@ISA = qw(Preference);
use strict;
use utf8;
######################################################################
# メール配信設定モジュール
# MailerSetting.pm ver1.1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2013-04-10
######################################################################
# 設定チェック項目取得
sub get_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #タグ除去
    $check::detag{'defaultFrom'}    = '発信者名のデフォルト';
    $check::detag{'defaultSubject'} = '表題のデフォルト';
    $check::detag{'defaultComment'} = '本文のデフォルト';
    $check::detag{'defaultName'}    = '&lt;name&gt;のデフォルト';
    $check::detag{'defaultKey'}     = '&lt;name&gt;のデフォルト';

    #チェックボックス
    push( @check::checkbox, 'fromOption', 'pswd', 'avoidRedundancy' );

    #数値
    $check::num{'limiter'}    = '同時送信数制限';
    $check::num{'sleepTime'}  = '処理間隔';
    $check::num{'attachment'} = 'ファイル添付制限';
}

######################################################################
# 設定環境
sub set_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::db = 'mail';    # 記録用データベース名
    $preference::name = 'メール配信設定';    # 設定パネル名
    $preference::title = 'メール配信機能の設定をします。';
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;

    #フィールド設定読み込み
    $this->use_field( 'field', $default::field_pref_dir . '/inputfield', 2 );

    my ($mail_fields);
    $this->scalar_to_array( \$main::data{'mailFields'} );
    foreach ( sort { $a <=> $b } keys %field::ORDER ) {
        my $id = $field::ORDER{$_};
        next unless ref $field::obj{$id} && $field::obj{$id}->{'e-mail'};
        $mail_fields
            .= '<option value="'
            . $id . '"'
            . (
            (          !ref $main::data{'mailFields'}
                    || !@{ $main::data{'mailFields'} }
                    || grep( $_ == $id, @{ $main::data{'mailFields'} } )
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $field::obj{$id}->name
            . ( ( $field::obj{$id}->{'authorityName'} )
            ? $field::obj{$id}->{'authorityName'}
            : '' )
            . '</option>';
    }
    if ( !defined $mail_fields || $mail_fields eq '' ) {
        print
            'メールアドレスフィールドが存在しないので、メール配信機能は利用できません。';
        return;
    }

    print '<fieldset><table class="field-table"><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:mailFields">宛先フィールド</label></td>';
    print
        '<td class="field"><select name="data:mailFields" id="data:mailFields" multiple="multiple">';
    print $mail_fields;
    print '</select></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:limiter">同時送信数制限</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:limiter', '', '', $main::data{'limiter'},
        '100' => '100件ずつ送信',
        '1'   => '個別に送信'
    );
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:sleepTime">処理間隔</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:sleepTime', '', '', $main::data{'sleepTime'},
        '1' => '1秒',
        '2' => '2秒',
        '4' => '4秒',
        '8' => '8秒',
        '0' => '0秒'
    );
    print '</td>';
    print '</tr>';

    print
        '<tr><td class="field-name"><span class="datatitle">重複送信の禁止</span></td><td class="field"><label for="data:avoidRedundancy"><input type="checkbox" name="data:avoidRedundancy" id="data:avoidRedundancy" value="1"'
        . (
        (   defined $main::data{'avoidRedundancy'}
                && $main::data{'avoidRedundancy'}
        ) ? ' checked="checked"' : ''
        )
        . ' />デフォルトで一度送信したメールアドレスへの重複送信を禁止する</label></td></tr>';
    print
        '<tr><td class="field-name"><span class="datatitle">差出人アドレス</span></td><td class="field"><label for="data:fromOption"><input type="checkbox" name="data:fromOption" id="data:fromOption" value="1"'
        . ( ( defined $main::data{'fromOption'} && $main::data{'fromOption'} )
        ? ' checked="checked"'
        : '' )
        . ' />差出人アドレスの変更を許可する</label></td></tr>';

    print
        '<tr class="tbody-end"><td class="field-name"><label for="data:attachment">ファイル添付制限</label></td>';
    print
        '<td class="field">最大 <input type="text" name="data:attachment" name="data:attachment" size="10" value="'
        . $main::data{'attachment'}
        . '" /> byte</td></tr>';

    print '</tbody><tbody>';

    $main::data{'defaultName'} = 'みな'
        if !defined $main::data{'defaultName'}
        || $main::data{'defaultName'} eq '';
    print
        '<tr><td class="field-name"><label for="data:defaultName">&lt;name&gt;のデフォルト</label></td>';
    print
        '<td class="field"><input type="text" name="data:defaultName" id="data:defaultName" size="20" value="'
        . $this->escape( $main::data{'defaultName'} )
        . '" /></td></tr>';

    $main::data{'defaultKey'} = 'これ'
        if !defined $main::data{'defaultKey'}
        || $main::data{'defaultKey'} eq '';
    print
        '<tr><td class="field-name"><label for="data:defaultKey">&lt;key&gt;のデフォルト</label></td>';
    print
        '<td class="field"><input type="text" name="data:defaultKey" id="data:defaultKey" size="20" value="'
        . $this->escape( $main::data{'defaultKey'} )
        . '" /></td></tr>';

    $main::data{'defaultTitle'} = 'これ'
        if !defined $main::data{'defaultTitle'}
        || $main::data{'defaultTitle'} eq '';
    print
        '<tr><td class="field-name"><label for="data:defaultTitle">&lt;title&gt;のデフォルト</label></td>';
    print
        '<td class="field"><input type="text" name="data:defaultTitle" id="data:defaultTitle" size="20" value="'
        . $this->escape( $main::data{'defaultTitle'} )
        . '" /></td></tr>';

    print
        '<tr><td class="field-name"><label for="data:defaultFrom">発信者名のデフォルト</label></td>';
    print
        '<td class="field"><input type="text" name="data:defaultFrom" id="data:defaultFrom" size="20" value="'
        . $this->escape( $main::data{'defaultFrom'} )
        . '" maxlength="50" /></td></tr>';

    print
        '<tr class="tbody-end"><td class="field-name"><label for="data:datetype">日時の表示形式</label></td><td class="field">';
    $this->print_date_type_menu( 'data:', $main::data{'datetype'} );
    print '</td></tr>';

    print '</tbody><tbody>';

    print
        '<tr><td class="field-name"><label for="data:defaultSubject">表題のデフォルト</label></td>';
    print
        '<td class="field"><input type="text" name="data:defaultSubject" id="data:defaultSubject" size="72" value="'
        . $this->escape( $main::data{'defaultSubject'} )
        . '" /></td></tr>';

    print
        '<tr class="tbody-end"><td class="field-name"><label for="data:defaultComment">本文のデフォルト</label></td>';
    print
        '<td class="field"><textarea name="data:defaultComment" id="data:defaultComment" cols="72" rows="20" wrap="soft">'
        . $this->escape( $main::data{'defaultComment'} )
        . '</textarea>';

    print '<div class="toolbox">';
    print
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:defaultComment\').quotationByMenu(this)" title="これらのタグはメール送信時に対応する値に変換されます。">';
    print '<option value="">▼挿入する</option>';
    print '<option value="&lt;name&gt;">&lt;name&gt;[名前フィールド('
        . ( ( $field::for{'name'} && ref $field::obj{ $field::for{'name'} } )
        ? $field::obj{ $field::for{'name'} }->name
        : '&lt;name&gt;のデフォルト' )
        . ')の値]</option>';
    print '<option value="&lt;key&gt;">&lt;key&gt;[キーフィールド('
        . ( ( $field::for{'key'} && ref $field::obj{ $field::for{'key'} } )
        ? $field::obj{ $field::for{'key'} }->name
        : '&lt;key&gt;のデフォルト' )
        . ')の値]</option>';
    print
        '<option value="&lt;title&gt;">&lt;title&gt;[タイトルフィールド('
        . (
        ( $field::for{'title'} && ref $field::obj{ $field::for{'title'} } )
        ? $field::obj{ $field::for{'title'} }->name
        : '&lt;title&gt;のデフォルト' )
        . ')の値]</option>';
    print '<option value="&lt;ctime&gt;">&lt;ctime&gt;[作成時]</option>';
    print '<option value="&lt;mtime&gt;">&lt;mtime&gt;[更新時]</option>';
    print '</select>';
    print '</div>';
    print '</td></tr>';

    print '</tbody><tbody>';

    print
        '<tr><td class="field-name"><span class="datatitle">メールアドレス一括登録</span></td><td class="field"><label for="data:pswd"><input type="checkbox" name="data:pswd" id="data:pswd" value="1"'
        . ( ( defined $main::data{'pswd'} && $main::data{'pswd'} )
        ? ' checked="checked"'
        : '' )
        . ' />メールアドレス一括登録フォームを表示する</label></td></tr>';

    print '</tbody></table></fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't201', 'MailerSetting' );

    #ツール登録
    $setting::jump_menu{'t201'} = 'メール配信設定';
}

1;
