package VCardViewer;
require CMONOS::TableMaker::Viewer;
@ISA = qw(Viewer);
use strict;
use utf8;
######################################################################
# vCardダウンロードモジュール
# VCardViewer.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# コンテンツタイプ
sub print_viewer_header ($) {
    my $this = shift;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    $main::u = 0;
    binmode STDOUT;
    $this->print_http_header('text/x-vcard');
}

######################################################################
# コンテンツヘッダ
sub print_contents_header ($) {
    my $this = shift;
    $this->field_to_vcard;
}

######################################################################
# フィールド分析
sub field_to_vcard ($) {
    my $this = shift;
    my ($id);
    foreach $id ( @{ $this->{'required_fields'} } ) {
        next if !ref $field::obj{$id};
        my $name  = $field::obj{$id}->name;
        my $class = $field::obj{$id}->{'class'};
        my $type  = '';
        my $key   = '';
        $name = $this->z_to_h( $name, 'utf8' );

        #名前
        if ( $name =~ /(?:name|氏名|名前|なまえ|ナマエ)/i ) {
            if ( $name
                =~ /(?:ふりがな|フリガナ|フリカナ|ナマエ|なまえ)/
                )
            {
                $key = 'SORT-STRING';
            }
            else {
                $key = 'FN';
            }

            #姓
        }
        elsif ( $name
            =~ /(?:family name|surname|姓|名字|みょうじ|ミョウジ)/i
            || $name =~ /^(?:セイ|せい)$/ )
        {
            if (   $name =~ /(?:ふりがな|フリガナ|フリカナ)/
                || $name =~ /^(?:セイ|せい)$/ )
            {
                $key = 'SORT-STRING2';
            }
            else {
                $key = 'N2';
            }

            #名
        }
        elsif ($name =~ /(?:first name|given name)/i
            || $name =~ /^(?:名|メイ|めい)$/ )
        {
            if ( $name
                =~ /(?:ふりがな|フリガナ|フリカナ|ナマエ|なまえ)/
                || $name =~ /^(?:名|メイ|めい)$/ )
            {
                $key = 'SORT-STRING1';
            }
            else {
                $key = 'N1';
            }

            #フリガナ
        }
        elsif ( $name =~ /(?:sort|ふりがな|フリガナ|フリカナ)/i )
        {
            $key = 'SORT-STRING';

            #ニックネーム
        }
        elsif ( $name
            =~ /(?:nickname|handle name|ニックネーム|ペンネーム|愛称|ハンドル|あだ名|別称|別名)/i
            )
        {
            $key = 'NICKNAME';

            #誕生日
        }
        elsif ($class eq 'time_date'
            && $name
            =~ /(?:birthday|誕生日|バースデイ|たんじょう|生年|生まれた)/i
            )
        {
            $key = 'BDAY';

            #役職
        }
        elsif ( $name
            =~ /(?:official title|official position|post|役職|肩書|ポジション)/i
            )
        {
            $key = 'title';

            #役割
        }
        elsif ( $name =~ /(?:role|部署|配属)/i ) {
            $key = 'ROLE';

            #所属
        }
        elsif ( $name =~ /(?:role|所属)/i ) {
            $key = 'ORG';

            #更新時
        }
        elsif ( $class eq 'user_xtime' ) {
            $key = 'REV' if $field::obj{$id}->{'timetype'} eq 'mtime';

            #その他
        }
        else {

            #勤務先
            if ( $name =~ /(?:work|office|business|会社|勤務|営業)/i ) {
                $type = 'WORK';

                #自宅用
            }
            elsif ( $name =~ /(?:home|自宅|家族)/i ) {
                $type = 'HOME';

                #郵便配達用
            }
            elsif ( $name =~ /(?:postal|郵送|〒|郵便|送付)/i ) {
                $type = 'POSTAL';
                $key  = 'ADR';

                #宅配用
            }
            elsif ( $name =~ /(?:parcel|配送|宅配|宅急便|納入)/i ) {
                $type = 'PARCEL';
                $key  = 'ADR';

                #自宅
            }
            else {
                $type = 'HOME';
            }

            #Eメール
            if ( $class =~ /(?:email|メール)/i ) {
                $type .= ',INTERNET';
                $key = 'EMAIL';

                #郵便番号
            }
            elsif ( $class eq 'addr_zipcode' ) {
                $key = 'ADR5';

                #国
            }
            elsif ( $name =~ /(?:country|国)/i ) {
                $key = 'ADR6';

                #住所
            }
            elsif ( $class eq 'addr_j' ) {
                $key = 'ADR';

                #携帯電話
            }
            elsif ($class eq 'num_number'
                && $name =~ /(?:cellular|PHS|mobile|携帯|ケータイ)/i )
            {
                $type .= ',CELL';
                $key = 'TEL';

                #ファックス
            }
            elsif ($class eq 'num_number'
                && $name =~ /(?:fax|ファックス)/i )
            {
                $type .= ',FAX';
                $key = 'TEL';

                #電話
            }
            elsif ($class eq 'num_number'
                && $name =~ /(?:tel|phone|電話|でんわ)/i )
            {
                $type .= ',VOICE';
                $key = 'TEL';

                #URL
            }
            elsif ( $class =~ /banner/i || $class =~ /url/ ) {
                $key = 'URL';
            }
        }
        if ( defined $key && $key ne '' ) {

     #		$this->{'vcard'}{$type}{$key} = $id if !$this->{'vcard'}{$type}{$key};
            $this->{'vcard'}{$type}{$key}{$id} = 1;
        }
        elsif ($class =~ /^text/
            || $class eq 'select_menu'
            || $class eq 'select_box' )
        {
            #		$this->{'vcard'}{''}{'NOTE'} = $id;
            $this->{'vcard'}{''}{'NOTE'}{$id} = 1;
        }
    }

#$this->{'vcard'}{''}{'FN'} = $field::for{'name'} if $field::for{'name'} && ref $field::obj{$field::for{'name'}};
    $this->{'vcard'}{''}{'FN'}{ $field::for{'name'} } = 1
        if $field::for{'name'} && ref $field::obj{ $field::for{'name'} };

    #if ($ENV{'TZ'} =~ /([+\-])([0-9]+)$/) {
    #	$this->{'vcard-TZ'} = $1 . sprintf("%.2d",$2) . ':00';
    #}
}

######################################################################
# ページフッタ
sub print_viewer_footer ($) {
    my $this = shift;
}

######################################################################
# レコード
sub print_record ($$$) {
    my ( $this, $c, $n ) = @_;
    my (%data);
    foreach my $id ( @{ $this->{'required_fields'} } ) {
        next
            unless ref $field::obj{$id}
            && $field::obj{$id}->format_for_text_enabled;
        my $db = $field::obj{$id}->db;
        $field::obj{$id}->{'datetype'} = 'v'
            if $field::obj{$id}->{'class'} eq 'user_xtime'
            || $field::obj{$id}->{'class'} eq 'time_date';
        if ( defined $db && $db ) {
            unless ( ref tied %{ $this->{'DB'}{$c}{$db} } ) {
                tie( %{ $this->{'DB'}{$c}{$db} },
                    $env::dbm_file, "$default::db/$c/$db", O_RDONLY,
                    $env::dbm_mode );
            }
            $data{$id} = $field::obj{$id}->format_for_text(
                $this->decode_utf8(
                    $this->{'DB'}{$c}{$db}{ $this->encode_utf8($n) }
                ),
                "$default::file/$c/$n",
                $c, $n
            );
        }
        else {
            $data{$id} = $field::obj{$id}->format_for_text('');
        }
    }

    #書き出し
    $this->print_vcard( 'BEGIN', 'VCARD' );
    $this->print_vcard_header;

    my ($type);
TYPE:
    foreach $type ( sort keys %{ $this->{'vcard'} } ) {
        my ( $key, @types, %done );
        @types = split( /,/, $type ) if defined $type && $type ne '';
        map( $_ = 'type=' . $_, @types );
    KEY:
        foreach $key ( sort keys %{ $this->{'vcard'}{$type} } ) {
            my ($fid);
        ID:
            foreach $fid ( sort keys %{ $this->{'vcard'}{$type}{$key} } ) {
                my $value = $data{$fid};
                next ID if !defined $value || $value eq '';
                next KEY if defined $done{$key} && $done{$key};
                if ( $key eq 'FN' ) {
                    if ((   !defined $this->{'vcard'}{$type}{'N1'}{$fid}
                            || $this->{'vcard'}{$type}{'N1'}{$fid} eq ''
                        )
                        && ( !defined $this->{'vcard'}{$type}{'N2'}{$fid}
                            || $this->{'vcard'}{$type}{'N2'}{$fid} eq '' )
                        )
                    {
                        my (@name);
                        @name = split( /\s+/, $value, 3 );
                        @name = ( $name[0], $name[2], $name[1] )
                            if defined $name[2] && $name[2] ne '';
                        @name = ( $name[1], $name[0], $name[2] );
                        $this->print_vcard( 'N', \@name, \@types );
                    }
                }
                elsif ( $key eq 'N1' || $key eq 'N2' ) {
                    next KEY if defined $done{'N'} && $done{'N'};
                    my $firstname
                        = (    ref $this->{'vcard'}{$type}
                            && ref $this->{'vcard'}{$type}{'N1'}
                            && %{ $this->{'vcard'}{$type}{'N1'} } )
                        ? $data{ ( keys %{ $this->{'vcard'}{$type}{'N1'} } )
                            [0] }
                        : '';
                    my $familyname
                        = (    ref $this->{'vcard'}{$type}
                            && ref $this->{'vcard'}{$type}{'N2'}
                            && %{ $this->{'vcard'}{$type}{'N2'} } )
                        ? $data{ ( keys %{ $this->{'vcard'}{$type}{'N2'} } )
                            [0] }
                        : '';
                    if ( !defined $this->{'vcard'}{$type}{'FN'}{$fid}
                        || $this->{'vcard'}{$type}{'FN'}{$fid} eq '' )
                    {
                        my ($name);
                        if ( $firstname !~ /[^0-9a-zA-Z_\x09\x0A\x0C\x0D\x20]/
                            && $firstname
                            !~ /[^0-9a-zA-Z_\x09\x0A\x0C\x0D\x20]/ )
                        {
                            $name = $firstname . ' ' . $familyname;
                        }
                        else {
                            $name = $familyname . ' ' . $firstname;
                        }
                        $this->print_vcard( 'FN', $name, \@types );
                    }
                    my @name = ( $familyname, $firstname );
                    $done{'N'} = 1;
                    $this->print_vcard( 'N', \@name, \@types );
                    next KEY;
                }
                elsif ( $key eq 'SORT-STRING1' || $key eq 'SORT-STRING2' ) {
                    next KEY
                        if $done{'SORT-STRING'}
                        || $this->{'vcard'}{$type}{'SORT-STRING'}{$fid};
                    my $firstname
                        = (    ref $this->{'vcard'}{$type}
                            && ref $this->{'vcard'}{$type}{'SORT-STRING1'}
                            && %{ $this->{'vcard'}{$type}{'SORT-STRING1'} } )
                        ? $data{ (
                            keys %{ $this->{'vcard'}{$type}{'SORT-STRING1'} }
                            )[0] }
                        : '';
                    my $familyname
                        = (    ref $this->{'vcard'}{$type}
                            && ref $this->{'vcard'}{$type}{'SORT-STRING2'}
                            && %{ $this->{'vcard'}{$type}{'SORT-STRING2'} } )
                        ? $data{ (
                            keys %{ $this->{'vcard'}{$type}{'SORT-STRING2'} }
                            )[0] }
                        : '';
                    $value = $familyname . ' ' . $firstname;
                    $done{'SORT-STRING'} = 1;
                    $this->print_vcard( 'SORT-STRING', $value, \@types );
                    next;
                }
                elsif ( $key eq 'ADR5' ) {
                    $done{'ADR5'} = 1;
                    next KEY;
                }
                elsif ( $key eq 'ADR6' ) {
                    $done{'ADR6'} = 1;
                    next KEY;
                }
                elsif ( $key eq 'ADR' ) {
                    $done{'ADR5'} = 1;
                    $done{'ADR6'} = 1;
                    $done{'ADR'}  = 1;
                    my @addr = split( /\x0A/, $value );
                    @addr = (
                        '', '',
                        ( ( defined $addr[2] ) ? $addr[2] : '' )
                            . ( ( defined $addr[3] ) ? $addr[3] : '' )
                            . ( ( defined $addr[4] ) ? $addr[4] : '' ),
                        ( ( defined $addr[1] ) ? $addr[1] : '' ),
                        ( ( defined $addr[0] ) ? $addr[0] : '' ),
                        (   ref $this->{'vcard'}{$type}{'ADR5'}
                                && %{ $this->{'vcard'}{$type}{'ADR5'} }
                            ) ? $data{
                            ( keys %{ $this->{'vcard'}{$type}{'ADR5'} } )[0]
                            } : '',
                        (   ref $this->{'vcard'}{$type}{'ADR6'}
                                && %{ $this->{'vcard'}{$type}{'ADR6'} }
                            ) ? $data{
                            ( keys %{ $this->{'vcard'}{$type}{'ADR6'} } )[0]
                            } : ''
                    );
                    $this->print_vcard( 'ADR', \@addr, \@types );
                    next KEY;
                }
                $this->print_vcard( $key, $value, \@types );
            }
        }
    }
    $this->print_vcard( 'END', 'VCARD' );
}

sub print_vcard_header ($) {
    my $this = shift;
    $this->print_vcard( 'VERSION', '3.0' );

#$this->print_vcard('TZ',$this->{'vcard-TZ'}) if defined $this->{'vcard-TZ'} && $this->{'vcard-TZ'};
}

sub print_vcard ($$$;$) {
    my ( $this, $key, $value, $types ) = @_;
    if ( ref $value eq 'ARRAY' ) {
        map {
            $_ = ( defined $_ ) ? $_ : '';
            $_ =~ s/(\x0D\x0A|\x0D|\x0A)/\\n/gs;
            $_ =~ s/,/\\,/g;
            $_ =~ s/;/\\;/g;

            #		$_ =~ s/:/\\:/g;
        } @$value;
        $value = join( "\x3B", @$value );
    }
    elsif ( $key ne 'REV' && $key ne 'BDAY' ) {
        $value =~ s/(\x0D\x0A|\x0D|\x0A)/\\n/gs;

        #	$value =~ s/,/\\,/g;
        $value =~ s/;/\\;/g;

        #	$value =~ s/:/\\:/g;
    }
    $key = join( ';', $key, @$types ) if ref $types;
    $this->_print_vcard( $key, $value );
}

sub _print_vcard ($$$) {
    my ( $this, $key, $value ) = @_;
    print $this->jcode_convert( \$key, 'ucs2', 'utf8' );
    print "\x00\x3A";
    print $this->jcode_convert( \$value, 'ucs2', 'utf8' );
    print "\x00\x0D\x00\x0A";
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定種キー設定
    $setting::key = 'none';    #ページ設定キー

    #デフォルトキー
    $main::k = 'vcard' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'vcard', 'VCardViewer' );
}

1;
