package YahooSearch;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# Yahoo検索モジュール
# YahooSearch.pm ver1.0.1
# Copyright (C) 2007-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-02-21
######################################################################
$SiteSearch::yahoo_web_search_url
    = 'http://search.yahooapis.jp/WebSearchService/V2/webSearch';
######################################################################
# メイン
sub run ($) {
    my $this = shift;
    my ( $result, $total, $start, $msg, $cloud, @query );
    if ( !$setting::general{'appid'} ) {
        $this->print_dialog( 'search',
            '<p><a href="http://developer.yahoo.co.jp/faq/#appid" target="_blank">Yahoo! アプリケーションID</a> が設定されていないので、検索機能を利用できません。<br />まだアプリケーションIDを登録していない場合は、<a href="http://api.search.yahoo.co.jp/webservices/register_application">アプリケーションID を登録</a>してください。</p>'
        );
    }
    if ( $ENV{'HTTP_USER_AGENT'} =~ /bot/i ) {
        $this->error404;
    }
    $this->set_og_url( $this->my_url );

    $setting::general{'results'} = 10 if !$setting::general{'results'};
    $setting::general{'rank'} = 4
        if !$setting::general{'rank'} || $setting::general{'rank'} > 26;
    $setting::general{'maxsize'}   = 3   if !$setting::general{'maxsize'};
    $setting::general{'minsize'}   = 0.7 if !$setting::general{'minsize'};
    $setting::general{'necessary'} = 16  if !$setting::general{'necessary'};
    $setting::general{'pickup'}    = 5   if !$setting::general{'pickup'};
    $setting::general{'datelimit'} = 3   if !$setting::general{'datelimit'};

    #---------------------------------------------------------------------
    #検索結果取得
    if ( $main::query && $main::query =~ /\S/ ) {
        @query = grep( $_ ne '', split( /\s+/, $main::query ) );
        require CMONOS::CGI_Lib::HTTP_Client;
        my $res = HTTP_Client->new->request(
            'POST' => $SiteSearch::yahoo_web_search_url,
            {},
            {   'appid'      => $setting::general{'appid'},
                'similar_ok' => 1,
                'site'       => $env::server_name,
                'query'      => $main::query,
                'results'    => $setting::general{'results'},
                'start'      => (
                    $setting::general{'results'} * (
                        ( !$main::p || $this->is_nan($main::p) )
                        ? 0
                        : ( $main::p - 1 )
                    ) + 1
                ),
            },
        );
        if ( $res->is_success ) {
            my $hash = $res->hash('utf8');
            if ( $hash->{'ResultSet'}->{'totalResultsAvailable'}
                && ref $hash->{'ResultSet'}->{'Result'} )
            {
                $total = $hash->{'ResultSet'}->{'totalResultsAvailable'};
                $start = $hash->{'ResultSet'}->{'firstResultPosition'};
                $msg
                    = '"'
                    . $this->escape($main::query)
                    . '" の検索結果 約 '
                    . $total
                    . ' 件中 '
                    . $start . ' - '
                    . (
                    $start + $hash->{'ResultSet'}->{'totalResultsReturned'}
                        - 1 )
                    . ' 件目';
                $this->scalar_to_array( \$hash->{'ResultSet'}->{'Result'} );
                $result = $hash->{'ResultSet'}->{'Result'};
            }
        }
        else {
            push( @{ $this->{'dialog'} }, 'ERROR:' . $res->code );
        }
    }

    if ( ref $result && !$main::p ) {
        my (%words);
        use Fcntl qw(:DEFAULT);
        eval 'use ' . $env::dbm_file . ';';
        tie(%words, $env::dbm_file,
            $default::db . '/words',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %words ) {
            no warnings qw(numeric);
            my ( $tip, $time, $count, $max, $url, @rank );
            $time  = time;
            $count = 0;
            foreach my $w (@query) {
                next if bytes::length $w < 3 || $w !~ /[^0-9]/;
                my $c = 1;
                if (   $words{ $this->encode_utf8($w) }
                    && $this->decode_utf8( $words{ $this->encode_utf8($w) } )
                    =~ /\x09([0-9]+)$/ )
                {
                    $c += $1;
                    $words{'MAX VALUE'} = $this->encode_utf8($c)
                        if !$words{'MAX VALUE'} || $c > $words{'MAX VALUE'};
                }
                $words{ $this->encode_utf8($w) }
                    = $this->encode_utf8( $time . "\x09" . $c );
            }
            $max
                = (    $words{'MAX VALUE'}
                    && $words{'MAX VALUE'} > $setting::general{'necessary'} )
                ? $words{'MAX VALUE'}
                : $setting::general{'necessary'};
            $cloud
                = ( $setting::general{'vicunaCompatible'} )
                ? '<ul class="tagCloud">'
                : '<ul id="searchword-cloud">';
            $tip  = '<div id="searchword-tip"><span>';
            $url  = $cgi::path . '?query=';
            @rank = ( 'a' .. 'z' );
            foreach my $w ( sort { $words{$b} <=> $words{$a} } keys %words ) {
                next if $w eq 'MAX VALUE';
                if ( $words{$w} && $words{$w} =~ /\x09([0-9]+)$/ ) {
                    utf8::decode($w) if !utf8::is_utf8($w);
                    $count++;
                    last
                        if $setting::general{'limit'}
                        && $count > $setting::general{'limit'};
                    my $c = $1;
                    my $u = $this->escape( $url . $this->url_encode($w) );
                    my $r = int(
                        ( 1 - ( $c / $max ) ) * $setting::general{'rank'} );
                    $r = $setting::general{'rank'} - 1
                        if $r >= $setting::general{'rank'};
                    $this->escape( \$w );

                    if ( $setting::general{'vicunaCompatible'} ) {
                        $cloud
                            .= '<li class="level'
                            . ( $r + 1 )
                            . '"><a href="'
                            . $u
                            . '" title="'
                            . $c
                            . '" rel="nofollow">'
                            . $w
                            . '</a></li>' . "\x0A";
                    }
                    else {
                        $cloud
                            .= '<li class="searchword-rank-'
                            . $rank[$r]
                            . '"><a href="'
                            . $u
                            . '" title="'
                            . $c
                            . '" rel="nofollow">'
                            . $w
                            . '</a></li>' . "\x0A";
                    }
                    if ( $count <= $setting::general{'pickup'} ) {
                        $tip
                            .= '<a href="'
                            . $u
                            . '" title="'
                            . $c
                            . '" rel="nofollow" class="searchword-rank-'
                            . $rank[$r]
                            . '" style="margin: 0 3px;font-size: '
                            . (
                            int((         $setting::general{'maxsize'}
                                        - $setting::general{'minsize'}
                                )
                                * ( $setting::general{'rank'} - 1 - $r )
                                    * 100 / ( $setting::general{'rank'} - 1 )
                                    + 0.5
                            ) / 100 + $setting::general{'minsize'}
                            )
                            . 'em">'
                            . $w . '</a>';
                    }
                }
            }
            $cloud .= '</ul>';
            $tip   .= '</span></div>';
            $this->write_file( "$env::externalfiles/ysearch-cloud.ssi",
                \$cloud, 1 );
            $this->mk_external_javascript(
                "$env::externalfiles/ysearch-cloud.ssi",
                '', $cloud );
            $this->write_file( "$env::externalfiles/ysearch-tip.ssi",
                \$tip, 1 );
            $this->mk_external_javascript(
                "$env::externalfiles/ysearch-tip.ssi",
                '', $tip );
        }
        untie %words;
    }

    #---------------------------------------------------------------------
    #ページヘッダsub push_title ($$;$) {
    if ($main::query) {
        my $q = $this->escape($main::query);
        $setting::page{'search'}{'title'} .= ' - '
            if defined $setting::page{'search'}{'title'}
            && $setting::page{'search'}{'title'} ne '';
        $setting::page{'search'}{'title'} .= $q . ' の検索結果';
        if ( defined $setting::page{'search'}{'titleText'}
            && $setting::page{'search'}{'titleText'} ne '' )
        {
            my $tag = ( $setting::page{$setting::key}{'titleText'}
                    =~ s/((<\/[^>]+>)+)$// ) ? $1 : '';
            $setting::page{'search'}{'titleText'} .= ' (' . $q . ')' . $tag;
        }
    }
    $this->set_style_sheet_link("$env::css/ysearch-cloud.css")
        if !$setting::general{'vicunaCompatible'};
    $this->print_page_header('search');

    print( ($cloud)
        ? $cloud
        : $this->read_file( "$env::externalfiles/ysearch-cloud.ssi", 1 ) );

    print $this->form_start(
        'key'      => 'search',
        'name'     => 'ysearch',
        'id'       => 'ysearch',
        'required' => \@setting::form
    );
    print
        '<input type="text" size="32" name="query" id="ysearch-query" value="'
        . $this->escape($main::query) . '" />';
    print '<input type="submit" id="ysearch-submit" value="検索" />';
    print '</form>';

    if ($main::query) {
        if ( ref $result ) {
            print '<p id="ysearch-message">' . $msg . '</p>';
            print '<ol id="ysearch-result" start="' . $start . '">';
            foreach my $item (@$result) {
                print '<li><div class="ttl"><a class="yschttl" href="'
                    . $item->{'ClickUrl'} . '">'
                    . ( ( $item->{'Title'} )
                    ? $this->emphasize( $item->{'Title'}, \@query )
                    : $item->{'Url'} )
                    . '</a></div>';
                print '<div class="yschabstr">'
                    . $this->emphasize( $item->{'Summary'}, \@query )
                    . '</div>';
                print '<div class="yschmtime">最終更新日:'
                    . $this->date(
                    $item->{'ModificationDate'},
                    $setting::general{'datelimit'},
                    'gmt>d' . $setting::general{'datetype'}
                    ) . '</div>';
                print '</li>';
            }
            print '</ol>';
            my $url
                = $cgi::path
                . '?query='
                . $this->url_encode($main::query)
                . '&amp;p=';
            my $left = $total;
            my $p    = 0;
            $main::p = 1 if !$main::p || $this->is_nan($main::p);
            print '<p id="ysearch-navi">';
            print '<a href="'
                . $url
                . ( $main::p - 1 )
                . '">&lt; 前のページ</a>'
                if $main::p > 1;

            while ( $left > 0 ) {
                my $f = ( $setting::general{'results'} * $p++ ) + 1;
                if ( $main::p && $main::p == $p ) {
                    print ' <strong>' . $p . '</strong>';
                }
                else {
                    my $t = $setting::general{'results'} * $p;
                    print ' <a href="'
                        . $url
                        . $p
                        . '" title="結果 '
                        . $f . ' - '
                        . ( ( $t > $total ) ? $total : $t ) . '">'
                        . $p . '</a>';
                }
                $left -= $setting::general{'results'};
            }
            print ' <a href="'
                . $url
                . ( $main::p + 1 )
                . '">次のページ &gt;</a>'
                if $main::p < $p;
            print '</p>';

        }
        else {
            print '<p id="ysearch-error">"'
                . $this->escape($main::query)
                . '" に該当するページが見つかりませんでした。</p>';
        }
    }

    print <<'YIMG';
<p id="yahoo-attribution-snippet">
<!-- Begin Yahoo! JAPAN Web Services Attribution Snippet -->
<a href="http://developer.yahoo.co.jp/about"><img src="http://i.yimg.jp/images/yjdn/yjdn_attbtn2_105_17.gif" width="105" height="17" title="Webサービス by Yahoo! JAPAN" alt="Webサービス by Yahoo! JAPAN" border="0" style="margin:15px 15px 15px 15px" /></a>
<!-- End Yahoo! JAPAN Web Services Attribution Snippet -->
</p>
YIMG

    #ページフッタ
    $this->print_page_footer('search');
}

######################################################################
#検索にマッチした部分を強調
sub emphasize ($$$) {
    my ( $this, $text, $keywords ) = @_;
    return '' if !defined $text;
    $this->escape( \$text ) if $text =~ /[<>]/;
    return $text if !ref $keywords || !@$keywords;
    my $n;
    foreach my $w (@$keywords) {
        $n++;
        $text =~ s/\Q$w\E/<strong class="yschq$n">$w<\/strong>/ig;
    }
    return $text;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::deco     = 1;    #テキスト装飾設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み
    $setting::page     = 1;    #ページ設定読み込み

    #設定種キー設定
    $setting::security_key
        = 'search';            #デフォルトセキュリティキー
    $setting::key = 'search';  #ページ設定キー

    #クッキー
    $cookie::id_enabled = 1;    #ID情報クッキー読み込み

    #デフォルトキー
    $main::k = 'search' if !defined $main::k || !$main::k;

    #必須フォーム
    @setting::form = ( 'm', 'k' );

    #クラス登録
    Common::class_register( 'search', 'YahooSearch' );
}

1;
