//	gmap.js (+ gmap.html + gmap/search.gif) for Google Maps API v3
// Copyright (C) 2010-2018 CMONOS Co. Ltd.
//	Version: 2.3.2
//	License: MIT License
//	※ Google Maps API および各種マーカー画像に関する全ての権利は米 Google, Inc が保有しています。
//
//	URI: https://cmonos.jp/
//	Last Modified: 2017-10-04
//	Author: CMONOS Co.Ltd.
//
//	【ストリートビュー関連コードのベースとしたデモ】
//	Street View Data Querying (Example by Tom Manshreck)
//	http://gmaps-samples.googlecode.com/svn/trunk/streetview/streetview_layer.html
//
//	【使い方】
//	次のような <iframe> タグで、地図を埋め込みます。
//
//	<iframe class="gmap-frame" id="gmap-canvas" src="./gmap.html?key=API_KEY&om=1&z=4&c=36.315125,137.504883"  width="500" height="400" scrolling="no" marginwidth="0" marginheight="0" frameborder="0">
//	<a href="./gmap.html?key=API_KEY&om=1&z=4&c=36.315125,137.504883" target="_blank">新しいウィンドウで地図を開く</a>
//	</iframe>
//
//	【地図呼び出しオプション】
//	key		Google API Key
//	om		1=概観地図(右下に表示される小さな四角い概観地図)を表示する場合
//	z		拡大倍率(0-17)
//	t		地図のタイプ(k=衛星写真/h=地図と衛星の合成/p=地形/設定なし=通常の地図)
//	c		中心座標の緯度,中心座標の経度
//	m		中心座標コメント
//	mw		中心座標コメントを表示する情報ウィンドウの幅
//	mk		カスタムマーカー画像URL
//	mks		カスタムマーカー影画像URL
//	mka		カスタムマーカーが地図に固定される水平位置（画像に対して left center right で指定します）
//	mkv		カスタムマーカーが地図に固定される垂直位置（画像に対して top center bottom で指定します）
//	s		1=地図検索欄を表示する
//	sv		1=ストリートビューを表示する
//	url		kml ファイルの URL
//	fit		1=kml ファイルに合わせて中心座標と拡大倍率を調整する
//	d		kml ファイルに記述された線分の距離を表示するかどうか
//			(1=始点から終点までの総延長を表示する/2=総延長に加えて始点と終点の直線距離を表示する)
//	width	地図の幅(ピクセル)
//	height	地図の高さ(ピクセル)
//	tr		情報ウィンドウ内リンクのターゲット p=_parent/s=_self/b=_blank/t=_top
//	i		1=index 用KMLを表示する(リンクつきのPlacemarkerのみで構成されたKML)
//	ns		1=index 用KMLを表示する際、マーカーを親HTMLのリンクと連動させない
//	opn		地図を埋め込んだ記事の URL/URLが一致したマーカーの情報ウィンドウを開きます
//	r		1=地図を移動または拡大縮小する度に kml を再読み込みする(必ず fit を無効化します)
//	sf		kml 再検索フォームのID
//	lb		表示している Placemarker の一覧を表示するリスト(<ul> or <ol>)を含むブロック(リストが空の時非表示にします)
//	li		表示している Placemarker の一覧を表示するリスト(<ul> or <ol>)
//
//	【以下は呼びだし元ページのフォームに地図の情報を書き込む場合に使用するオプションです】
//	z_field		表示中の地図の拡大倍率を収める入力欄のID
//	t_field		表示中の地図の地図のタイプを収める入力欄のID
//	c_field		表示中の地図の中心座標を収める入力欄のID
//	m_field		表示中の地図の中心座標コメントが収められている入力欄のID
//	mk_field	カスタムマーカー画像URLが収められている入力欄のID
//	mk_disabled	カスタムマーカー画像URLが収められている入力欄を有効化するチェックボックスのID(オプション)
//	fit_field	表示中の地図を KML ファイルにあわせて調整するかどうかの設定が収められているチェックボックスのID
//	addr_field	地図検索欄に読み込む住所が収められている住所入力欄のIDのカンマ区切りリスト(都道府県,市区群,番地,ビル)

var gmap = {

	//	外部ドメインからのアクセスを許可する場合は下記を false にします。
	domain_check : true,

	//	デフォルトのマーカーアイコン
	prepareMarkerImage : function () {
		this.readImage("//maps.google.com/mapfiles/ms/micons/red.png");
		this.readImage("//maps.google.com/mapfiles/ms/micons/msmarker.shadow.png");
	},
	getMarkerImage : function () {
		this.togglablemarker_disabled = false;
		return this.getMarkerImageFrom(
			"//maps.google.com/mapfiles/ms/micons/red.png",
			32,
			32,
			16,
			32,
			"//maps.google.com/mapfiles/ms/micons/msmarker.shadow.png",
			59,
			32
		);
	},

	//	定規アイコン
	getMeasureImage : function () {
		return this.getMarkerImageFrom(
			"//maps.gstatic.com/mapfiles/kml/pal5/icon5.png",
			32,
			32,
			16,
			32,
			"//maps.gstatic.com/mapfiles/kml/pal5/icon5s.png",
			59,
			32
		);
	},

	//	選択マーカー画像
	prepareTogglableMarkerImage : function () {
		this.readImage("//maps.google.com/mapfiles/ms/micons/yellow-dot.png");
	},
	toggleLinkMarkerImage : function (this_marker,is_selected) {
		this_marker.setIcon(((is_selected) ? "//maps.google.com/mapfiles/ms/micons/yellow-dot.png" : "//maps.google.com/mapfiles/ms/micons/red.png"));
	},

	_caller : ((parent && parent != self) ? parent : (opener) ? opener : self),
	icon_cache : [],
	geocoder_client : null,
	togglablemarker_disabled : false,

	prepare : function () {
		this.q = this.urlDecode(self.document.location.search);
		this.api_enabled = ((this.domain_check && this._caller.document.domain != self.document.domain)
			 || (navigator.userAgent.match(/MSIE\s*(\d+\.\d+)/) && parseFloat(RegExp.$1) < 7.0) || (navigator.userAgent.match(/Safari\/(\d+)/) && parseFloat(RegExp.$1) < 400)) ? false : true;
		this.map = null;
		this.map_center = null;
		this.center_marker = null;
		this.measure_icon = null;
		this.kml_refreshing = false;
		this.default_icon = null;
		this.streetview_window = null;
		this.streetview_window_is_opened = null;
		this.streetview_holder = null;
		this.streetview_node = null;
		this.streetview_canvas = null;
		this.streetview_panorama = null;
		this.streetview_last_position = null;
		this.kml_overlay = null;
		this.kml_objects = null;
		if (this.api_enabled) self.document.write('<script type="text/javascript" src="//maps.google.com/maps/api/js?' + ((this.q["key"]) ? 'key=' + this.q["key"] + '&amp;' : '') + ((this.q["lang"] != null && this.q["lang"] != "") ? '&amp;language=' + this.q["lang"] : '') + '"></script>');
		this.z_field = (this.q["z_field"] != null && this.q["z_field"] != "") ? this.getCallerFieldsById(this.q["z_field"]) : null;
		this.t_field = (this.q["t_field"] != null && this.q["t_field"] != "") ? this.getCallerFieldsById(this.q["t_field"]) : null;
		this.c_field = (this.q["c_field"] != null && this.q["c_field"] != "") ? this.getCallerFieldsById(this.q["c_field"]) : null;
		this.m_field = (this.q["m_field"] != null && this.q["m_field"] != "" && this._caller.document.getElementById(this.q["m_field"])) ? this._caller.document.getElementById(this.q["m_field"]) : null;
		this.mk_field = (this.q["mk_field"] != null && this.q["mk_field"] != "" && this._caller.document.getElementById(this.q["mk_field"])) ? this._caller.document.getElementById(this.q["mk_field"]) : null;
		this.mkc_field = (this.q["mkc_field"] != null && this.q["mkc_field"] != "" && this._caller.document.getElementById(this.q["mkc_field"])) ? this._caller.document.getElementById(this.q["mkc_field"]) : null;
		this.mka_field = (this.q["mka_field"] != null && this.q["mka_field"] != "" && this._caller.document.getElementById(this.q["mka_field"])) ? this._caller.document.getElementById(this.q["mka_field"]) : null;
		this.mkv_field = (this.q["mkv_field"] != null && this.q["mkv_field"] != "" && this._caller.document.getElementById(this.q["mkv_field"])) ? this._caller.document.getElementById(this.q["mkv_field"]) : null;
		this.fit_field = (this.q["fit_field"] != null && this.q["fit_field"] != "" && this._caller.document.getElementById(this.q["fit_field"])) ? this._caller.document.getElementById(this.q["fit_field"]) : null;
		this.addr_field_ids = (this.q["addr_field"] != null && this.q["addr_field"] != "") ? this.addr_field_ids = this.q["addr_field"].split(/,/) : null;
		this.mw = (this.q["mw"] != null && this.q["mw"] > 100) ? this.q["mw"] : 280;
		this.kml_url = null;
		this.base_url = null;
		this.search_form = null;
		this.list_box = (this.q["lb"] != null && this.q["lb"] != "" && this._caller.document.getElementById(this.q["lb"])) ? this._caller.document.getElementById(this.q["lb"]) : null;
		this.list = (this.q["li"] != null && this.q["li"] != "" && this._caller.document.getElementById(this.q["li"])) ? this._caller.document.getElementById(this.q["li"]) : null;
		this.kml_by_myself = (this.q["i"] != null && parseInt(this.q["i"]) > 0) ? true : false;
		this.nosync = (this.q["ns"] != null && parseInt(this.q["ns"]) > 0) ? true : false;
		if (this.q["r"] != null && this.q["r"] == 1) this.q["fit"] = null;
		if (this.q["mk"] != null) this.readImage(this.q["mk"]);
		if (this.q["mks"] != null) this.readImage(this.q["mks"]);
		if (this.mk_field != null) {
			if (this.mk_field.type.search(/select/) != -1) {
				for (var i=0; i < this.mk_field.options.length; i++) {
					if (this.mk_field.options[i].value != null && this.mk_field.options[i].value != "") {
						this.readImage(this.mk_field.options[i].value);
					}
				}
			} else if (this.mk_field.value != null && this.mk_field.value != "") {
				this.readImage(this.mk_field.value);
			}
		};
		this.dw = (!this.q["dw"] || isNaN(this.q["dw"])) ? 640 : parseInt(this.q["dw"]);
		this.dh = (!this.q["dh"] || isNaN(this.q["dh"])) ? 480 : parseInt(this.q["dh"]);
		this.prepareMarkerImage();
		this.prepareTogglableMarkerImage();
	},

	getCallerFieldsById : function (text) {
		var list = text.split(/,/);
		var fields = new Array();
		for (var i=0; i < list.length; i++) {
			if (this._caller.document.getElementById(list[i])) {
				fields[fields.length] = this._caller.document.getElementById(list[i]);
			}
		}
		return (fields.length > 0) ? fields : null;
	},

	readImage : function (this_url) {
		var i = this.icon_cache.length;
		this.icon_cache[i] = new Image();
		this.icon_cache[i].src = this_url;
	},

	initialize : function () {
		var field_enabled = (this.z_field != null || this.t_field != null || this.c_field != null || this.m_field != null || this.fit_field != null) ? true : false;
		var w = ((self.innerWidth) ? self.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientWidth : self.document.body.clientWidth));
		var h = ((self.innerHeight) ? self.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientHeight : self.document.body.clientHeight))
		if (this.api_enabled) {
			var z = 4;
			var center = new Array(36.315125,137.504883);
			var map_canvas = self.document.getElementById("map");
			var this_gmap = this;
			if (this.q["c"] != null && this.q["c"] != "") center = this.q["c"].split(/,/);
			if (this.q["z"] != null && this.q["z"] != "" && !isNaN(this.q["z"])) z = parseInt(this.q["z"]);
			this.map_center = new google.maps.LatLng(center[0],center[1]);
			this.map = new google.maps.Map(map_canvas,{
				zoom		: z,
				center		: this.map_center,
				mapTypeId	: ((this.q["t"] == null || this.q["t"] == "") ? google.maps.MapTypeId.ROADMAP : (this.q["t"] == "k") ? google.maps.MapTypeId.SATELLITE : (this.q["t"] == "h") ? google.maps.MapTypeId.HYBRID : (this.q["t"] == "p") ? google.maps.MapTypeId.TERRAIN : google.maps.MapTypeId.ROADMAP),
				streetViewControl	: ((this.q["sv"] != null && this.q["sv"] == 1) ? true : false),
				overviewMapControl	: ((this.q["om"] != null && this.q["om"] == 1) ? true : false)
			});

			if (this.q["sv"] != null && this.q["sv"] == 1) {
				this.streetview_node = document.createElement('div');
				this.streetview_canvas = document.createElement('div');

				this.streetview_holder = document.createElement('div');
				this.streetview_holder.style.width = w + 'px';
				this.streetview_holder.style.height = h + 'px';
				this.streetview_holder.style.position = 'absolute';
				this.streetview_holder.style.overflow = 'hidden';
				this.streetview_holder.style.top = '-' + w + 'px';
				this.streetview_holder.style.left = '-' + h + 'px';
				this.streetview_holder.appendChild(this.streetview_canvas);
				self.document.body.appendChild(this.streetview_holder);

				this.streetview_panorama = new google.maps.StreetViewPanorama(this.streetview_canvas,{
					visible		: false
				});

				this.streetview_window = new google.maps.InfoWindow({ content : this.streetview_node, pixelOffset : new google.maps.Size(0, -33) });
				this.setStreetViewCanvasSize(w,h);

				google.maps.event.addDomListener(
					self,
					'resize',
					function() {
						w = ((self.innerWidth) ? self.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientWidth : self.document.body.clientWidth));
						h = ((self.innerHeight) ? self.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientHeight : self.document.body.clientHeight))
						this_gmap.setStreetViewCanvasSize(w,h);
					}
				);
				google.maps.event.addListener(
					this.streetview_window,
					'domready',
					function() {
						this_gmap.streetview_node.appendChild(this_gmap.streetview_canvas);
					}
				);
				google.maps.event.addListener(
					this.streetview_window,
					'closeclick',
					function() {
						if (this_gmap.streetview_window_is_opened) {
							this_gmap.streetview_holder.appendChild(this_gmap.streetview_canvas);
							this_gmap.streetview_panorama.setVisible(false);
							this_gmap.streetview_window_is_opened = false;
							this_gmap.streetview_last_position = null;
						}
					}
				);
				google.maps.event.addListener(
					this.streetview_panorama,
					'visible_changed',
					function() {
						if (this_gmap.streetview_panorama.getVisible()) {
							if (!this_gmap.streetview_window_is_opened) {
								this_gmap.streetview_window.setPosition(this_gmap.streetview_panorama.getPosition());
								this_gmap.streetview_window.open(this_gmap.map);
								this_gmap.streetview_window_is_opened = true;
							}
						} else if (this_gmap.streetview_window_is_opened) {
							this_gmap.streetview_holder.appendChild(this_gmap.streetview_canvas);
							this_gmap.streetview_window.close();
							this_gmap.streetview_window_is_opened = false;
							this_gmap.streetview_last_position = null;
						}
					}
				);
				google.maps.event.addListener(
					this.streetview_panorama,
					'position_changed',
					function() {
						var current_position = this_gmap.streetview_panorama.getPosition();
						this_gmap.streetview_window.setPosition(current_position);
						if (this_gmap.streetview_last_position != null && !current_position.equals(this_gmap.streetview_last_position)) {
							this_gmap.map_center = this_gmap.map.getCenter();
							var map_bounds = this_gmap.map.getBounds()
							if (map_bounds.contains(this_gmap.streetview_last_position)) {
								if (
										((Math.abs(this_gmap.streetview_last_position.lat()-current_position.lat()) < 0.0004) && (Math.abs(this_gmap.streetview_last_position.lng()-current_position.lng()) < 0.0004))
										|| (this_gmap.streetview_last_position.temp && (Math.abs(this_gmap.streetview_last_position.temp.lat()-current_position.lat()) < 0.0002) && (Math.abs(this_gmap.streetview_last_position.temp.lng()-current_position.lng()) < 0.0002))
									) {
									this_gmap.map.panTo(new google.maps.LatLng((this_gmap.map_center.lat() + current_position.lat() - this_gmap.streetview_last_position.lat()),(this_gmap.map_center.lng() + current_position.lng() - this_gmap.streetview_last_position.lng())));
								} else {
									this_gmap.streetview_last_position.temp = current_position;
									return;
								}
							} else {
								s = map_bounds.getSouthWest().lat();
								current_position = new google.maps.LatLng((((map_bounds.getNorthEast().lat() - s) / 10) + s),this_gmap.map_center.lng());
							}
						}
						this_gmap.streetview_last_position = current_position;
					}
				);
			}

			if ((this.m_field != null) || (this.q["m"] != null && this.q["m"] != "")) {
				if (this.m_field != null) {
					this.makeCenterMarker(true);
					if (this.mk_field != null) this.addEventListener(this.mk_field,'change',function () { this_gmap.makeCenterMarker(true); });
					if (this.mkc_field != null) this.addEventListener(this.mkc_field,'click',function () { this_gmap.makeCenterMarker(true); });
					if (this.mka_field != null) this.addEventListener(this.mka_field,'change',function () { this_gmap.makeCenterMarker(true); });
					if (this.mkv_field != null) this.addEventListener(this.mkv_field,'change',function () { this_gmap.makeCenterMarker(true); });
				} else {
					var center_marker_image = (this.q["mk"] != null && this.q["mk"] != "") ? this.getCustomMarkerImage(this.q["mk"],this.q["mka"],this.q["mkv"],this.q["mks"]) :  this.getMarkerImage();
					this.center_marker = new google.maps.Marker({
						position	: this.map_center,
						map			: this.map,
						icon		: center_marker_image.icon,
						shadow		: center_marker_image.shadow,
						flat		: ((center_marker_image.shadow == null) ? true : false)
					});
					google.maps.event.addListener(
						this.center_marker,
						"click",
						function () {
							if (!this_gmap.center_marker.info_window) this_gmap.center_marker.info_window = new google.maps.InfoWindow({
								content: '<div style="font-size: 14px">' + this_gmap.q["m"] + '</div>',
								maxWidth: this_gmap.mw
							});
							this_gmap.center_marker.info_window.open(this_gmap.map,this_gmap.center_marker);
							this_gmap.changeLinkTarget();
						}
					);
				}
			}
			if (this.q["url"] != null && this.q["url"] != "") {
				this.kml_url = this.q["url"];
				this.base_url = this.q["url"].replace(/\?.*$/,'');
			} else if (this.q["sf"] != null && this.q["sf"] != "" && this._caller.document.getElementById(this.q["sf"])) {
				this.search_form = this._caller.document.getElementById(this.q["sf"]);
				this.base_url = this.search_form.action;
				this.kml_url = this.base_url + '?' + this.urlEncode(this.getHashFromForm(this.search_form));
			}
			if (this.kml_url != null) {
				google.maps.event.addListenerOnce( this.map, "tilesloaded", function () { this_gmap.refreshKml(); } );
				if (this.search_form != null) {
					this.addEventListener(this.search_form,'submit',function (e) {
						this_gmap.kml_url = this_gmap.base_url + '?' + this_gmap.urlEncode(this_gmap.getHashFromForm(this_gmap.search_form));
						this_gmap.refreshKml();
						e = e || event || window.event;
						if (e.preventDefault) {
							e.preventDefault();
						} else {
							e.returnValue = false;
						}
						return false;
					});
				}
				if (this.q["r"] != null && this.q["r"] == 1) {
					google.maps.event.addListener( this.map, 'dragend', function () { this_gmap.refreshKml(); } );
					google.maps.event.addListener( this.map, 'zoom_changed', function () { google.maps.event.addListenerOnce( this_gmap.map, "tilesloaded", function () { this_gmap.refreshKml(); } ); } );
					google.maps.event.addDomListener( self, 'resize', function () { this_gmap.refreshKml(); } );
				}
			}
			if (this.q["wp"] != null) {
				if (this.q["wp"].constructor == String) this.q["wp"] = [this.q["wp"]];
				for (var i = 0; i < this.q["wp"].length; i++) {
					(function (wp) { google.maps.event.addListenerOnce( this_gmap.map, "tilesloaded", function () { this_gmap.setWaypoints(wp); } ) })(this.q["wp"][i]);
				}
			}
			if (field_enabled || (this.q["s"] != null && this.q["s"] == 1)) {
				self.document.getElementById("geocoder-tool").innerHTML = '<a style="display: block; font-size: 1px; text-indent: -200px; overflow: hidden; cursor: pointer; border: none; margin: 0; padding: 0; position: absolute; top: 0; left: 0; width: 15px; height: 15px; z-index: 10000; background-image: url(\'./gmap/search.gif\')" onclick="gmap.toggleSearchBox()" title="住所検索">住所検索</a><form id="geocoder-search" action="#" onsubmit="gmap.geocoderSearch(); return false;" style="display: none; position: absolute; top: 24px; left: 76px; float: left"><input type="search" placeholder="住所検索" results="5" autosave="geocodersearch" size="24" id="geocoderkey" value="" style="font-size: 13px" /><input type="button" onclick="gmap.geocoderSearch()" value="検索" /><img style="border: 0px none ; margin-left: 3px; width: 14px; height: 13px; -moz-user-select: none; cursor: pointer;" src="//maps.gstatic.com/mapfiles/close.gif" onclick="gmap.toggleSearchBox()" title="住所検索欄を閉じる" /></form>';
				self.document.getElementById("geocoder-search").style.display = "none";
				if (field_enabled) {
					google.maps.event.addListener( this.map, 'dragend', function () { this_gmap.centerMarker(); this_gmap.saveMap(); } );
					google.maps.event.addListener( this.map, 'zoom_changed', function () { google.maps.event.addListenerOnce( this_gmap.map, "tilesloaded", function () { this_gmap.centerMarker(); this_gmap.saveMap(); } ); } );
					google.maps.event.addListener( this.map, 'maptypeid_changed', function () {  this_gmap.saveMap(); } );
				}
			}
		} else {
			if (w > 512) w = 512;
			if (h > 512) h = 512;
			var gmap_img = document.createElement('img');
			gmap_img.style.width = w + "px";
			gmap_img.style.height = h + "px";
			gmap_img.style.border = "none";
			if (this.q["m"] != null && this.q["m"] != "") gmap_img.title = this.q["m"].replace(/<.*?>/g,'');
			var center = ((this.q["c"] != null && this.q["c"] != "") ? this.q["c"] : '36.315125,137.504883');
			gmap_img.src	= '//maps.google.com/maps/api/staticmap?center=' + center
							+ '&zoom=' + ((this.q["z"] != null && this.q["z"] != "" && !isNaN(this.q["z"])) ? parseInt(this.q["z"]) : 4)
							+ '&size=' + w + 'x' + h
							+ ((this.q["m"] != null && this.q["m"] != "") ? '&markers=' + center : '');
			self.document.getElementById("map").appendChild(gmap_img);
			var gmap_msg = document.createElement('p');
			gmap_msg.style.position = 'absolute';
			gmap_msg.style.backgroundColor = '#FFF';
			gmap_msg.style.color = '#000';
			gmap_msg.style.top = 0;
			gmap_msg.style.left = 0;
			gmap_msg.innerHTML = 'Google マップを表示できませんでした。';
			self.document.getElementById("map").appendChild(gmap_msg);
		}
	},

	distanceFrom : function(new_latlng,new_altitude,prev_latlng,prev_altitude) {
		var R = 6371000;
		var lat1 = new_latlng.lat();
		var lon1 = new_latlng.lng();
		var lat2 = prev_latlng.lat();
		var lon2 = prev_latlng.lng();
		var dLat = (lat2-lat1) * Math.PI / 180;
		var dLon = (lon2-lon1) * Math.PI / 180;
		var a = Math.sin(dLat/2) * Math.sin(dLat/2) + Math.cos(lat1 * Math.PI / 180 ) * Math.cos(lat2 * Math.PI / 180 ) * Math.sin(dLon/2) * Math.sin(dLon/2);
		var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
		var d = R * c;
		if (new_altitude != null && prev_altitude != null) {
			d = Math.sqrt(Math.pow(d,2)+Math.pow(Math.abs(new_altitude-prev_altitude),2))
		}
		return d;
	},

	changeLinkTarget : function  () {
		var self_links = document.getElementsByTagName('a');
		var this_gmap = this;
		for (var i=0; i<self_links.length; i++) {
			if (
					typeof self_links[i].getAttribute("href") == "string"
					&& self_links[i].getAttribute("href").search(/^https?/i) != -1
					&& self_links[i].getAttribute("href").search(/^https?:\/\/\w+\.google/i) == -1
					&& (self_links[i].target == null || self_links[i].target == "")
			) {
				if (this.q["tr"] != null && this.q["tr"] == "p") {
					self_links[i].target = "_parent";
				} else if (this.q["tr"] != null && this.q["tr"] == "t") {
					self_links[i].target = "_top";
				} else if (this.q["tr"] != null && this.q["tr"] == "s") {
					self_links[i].target = "_self";
				} else if (this.q["tr"] != null && this.q["tr"] == "d") {
					if (self_links[i].getAttribute("href").search(/m=(DetailViewer|Inquiry)&/) != -1) {
						self_links[i].target = "detailViewer";
						this.addEventListener(self_links[i],'click',function () { window.open('','detailViewer','width='+this_gmap.dw+',height='+this_gmap.dh+',scrollbars=yes,resizable=yes'); });
					} else {
						self_links[i].target = "_blank";
					}
				} else {
					self_links[i].target = "_blank";
				}
			}
		}
		var self_forms = document.getElementsByTagName('form');
		for (var i=0; i<self_forms.length; i++) {
			if (
					typeof self_forms[i].getAttribute("action") == "string"
					&& self_forms[i].getAttribute("action").search(/^https?/i) != -1
					&& self_forms[i].getAttribute("action").search(/^https?:\/\/\w+\.google/i) == -1
					&& (this.kml_by_myself || self_forms[i].target == null || self_forms[i].target == "")
			) {
				if (this.q["tr"] != null && this.q["tr"] == "p") {
					self_forms[i].target = "_parent";
				} else if (this.q["tr"] != null && this.q["tr"] == "t") {
					self_forms[i].target = "_top";
				} else if (this.q["tr"] != null && this.q["tr"] == "s") {
					self_forms[i].target = "_self";
				} else {
					self_forms[i].target = "_blank";
				}
			}
		}
	},

	setStreetViewCanvasSize : function (w,h) {
		var svw = parseInt(w * 2 / 3);
		var svh = parseInt(h * 3 / 5);
		svw = (svw > 640) ? 640 : (svw > (w - 83) || svw < 232) ? (w - 68) : svw;
		svh = (svh > 615) ? 615 : (svh > (h - 180) || svw < 202) ? (h - 168) : svh;
		this.streetview_node.style.width = svw + 'px';
		this.streetview_node.style.height = svh + 'px';
		this.streetview_canvas.style.width = svw + 'px';
		this.streetview_canvas.style.height = svh + 'px';
		if (svw < 232 || svh < 202) {
			this.map.setStreetView(null);
		} else {
			this.map.setStreetView(this.streetview_panorama);
		}
	},

	makeCenterMarker : function (make_from_form) {
		if (this.center_marker != null) this.center_marker.setMap(null);
		var center_marker_image = null;
		if (make_from_form) {
			var mk;
			var mka;
			var mkv;
			var mks;
			if (this.mk_field != null && (this.mkc_field == null || this.mkc_field.checked)) {
				if (this.mk_field.type.search(/select/) != -1) {
					mk = this.mk_field.options[this.mk_field.options.selectedIndex].value;
				} else if (this.mk_field.value != null && this.mk_field.value != "") {
					mk = this.mk_field.value;
				}
			}
			if (this.mka_field != null) {
				if (this.mka_field.type.search(/select/) != -1) {
					mka = this.mka_field.options[this.mka_field.options.selectedIndex].value;
				} else if (this.mk_field.value != null && this.mk_field.value != "") {
					mka = this.mka_field.value;
				}
			}
			if (this.mkv_field != null) {
				if (this.mkv_field.type.search(/select/) != -1) {
					mkv = this.mkv_field.options[this.mkv_field.options.selectedIndex].value;
				} else if (this.mk_field.value != null && this.mk_field.value != "") {
					mkv = this.mkv_field.value;
				}
			}
			if (mk != null && mk != "" && mk.search(/^https?:\/\/.*\.(gif|png|jpg|jpeg)$/i) != -1) {
				this.togglablemarker_disabled = true;
				center_marker_image = this.getCustomMarkerImage(mk,mka,mkv,null);
			} else if (this.q["mk"] != null && this.q["mk"] != "") {
				center_marker_image = this.getCustomMarkerImage(this.q["mk"],((this.mkc_field != null) ? mka : this.q["mka"]),((this.mkc_field != null) ? mkv : this.q["mkv"]),this.q["mks"]);
			} else {
				this.togglablemarker_disabled = false;
				center_marker_image = this.getMarkerImage();
			}
		} else if (this.q["mk"] != null && this.q["mk"] != "") {
			center_marker_image = this.getCustomMarkerImage(this.q["mk"],this.q["mka"],this.q["mkv"],this.q["mks"]);
		} else {
			center_marker_image =  this.getMarkerImage();
		}
		this.center_marker = new google.maps.Marker({
			position	: this.map_center,
			map			: this.map,
			icon		: center_marker_image.icon,
			shadow		: center_marker_image.shadow,
			flat		: ((center_marker_image.shadow == null) ? true : false),
			title		: 'この地図の中心'
		});
		var this_gmap = this;
		google.maps.event.addListener(
			this.center_marker,
			"click",
			function () {
				this_gmap.map.setOptions({draggable : false});
				this_gmap.map_center = this_gmap.map.getCenter();
				if (this_gmap.m_field.value != "") {
					this_gmap.center_marker.info_window = new google.maps.InfoWindow({
						content: '<div style="font-size: 14px">' + this_gmap.m_field.value + '</div>',
						maxWidth: this_gmap.mw
					});
					this_gmap.center_marker.info_window.open(this_gmap.map,this_gmap.center_marker);
					google.maps.event.addListener(
						this_gmap.center_marker.info_window,
						"closeclick",
						function () {
							this_gmap.map.setOptions({draggable : true});
							this_gmap.map.panTo(this_gmap.map_center);
						}
					);
					this_gmap.changeLinkTarget();
				}
			}
		);
		google.maps.event.addListener(
			map,
			"drag",
			function () {
				this_gmap.center_marker.setPosition(this_gmap.map.getCenter());
			}
		);
		this.centerMarker();
	},

	getCustomMarkerImage : function (url,mka,mkv,surl) {
		var g_image = new Image();
		g_image.src = url;
		var w = g_image.width;
		var h = g_image.height;
		if (surl != null && surl != "") {
			var sg_image = new Image();
			sg_image.src = surl;
			var sw = sg_image.width;
			var sh = sg_image.height;
		}
		if (w != null && w > 0) {
			if (mka == null) mka = "center";
			if (mkv == null) mkv = "bottom";
			var a = (mka == "left") ? 0 : (mka == "right") ? w : parseInt(w/2);
			var v = (mkv == "top") ? 0 : (mkv == "center") ? parseInt(h/2) : (h+1);
			this.togglablemarker_disabled = true;
			return this.getMarkerImageFrom(url,w,h,a,v,surl,sw,sh);
		} else {
			return this.getMarkerImage();
		}
	},

	getMarkerImageFrom : function (url,w,h,a,v,surl,sw,sh) {
		var icon = new google.maps.MarkerImage(url,
			new google.maps.Size(w, h),
			new google.maps.Point(0, 0),
			new google.maps.Point(a, v),
			new google.maps.Size(w, h)
		);
		icon.image_url = url;
		if (surl) {
			sw = (sw != null && sw > 0) ? sw : w*2;
			sh = (sh != null && sh > 0) ? sh : h;
			shadow = new google.maps.MarkerImage(surl,
				new google.maps.Size(sw, sh),
				new google.maps.Point(0, 0),
				new google.maps.Point(a, v),
				new google.maps.Size(sw, sh)
			);
			shadow.image_url = surl;
		} else {
			shadow = null;
		}
		return {
			icon	:	icon,
			shadow	:	shadow
		};
	},

	getCustomMarkerImageFromKml : function (icon_style) {
		var url = this.getNodeValue(icon_style.getElementsByTagName("href")[0]);
		var w = parseInt(this.getNodeValue(icon_style.getElementsByTagName("width")[0]));
		var h = parseInt(this.getNodeValue(icon_style.getElementsByTagName("height")[0]));
		var a = parseInt(this.getNodeValue(icon_style.getElementsByTagName("anchorx")[0]));
		var v = parseInt(this.getNodeValue(icon_style.getElementsByTagName("anchory")[0]));
		var surl = this.getNodeValue(icon_style.getElementsByTagName("shadow")[0]);
		var sw = parseInt(this.getNodeValue(icon_style.getElementsByTagName("swidth")[0]));
		var sh = parseInt(this.getNodeValue(icon_style.getElementsByTagName("sheight")[0]));
		if (url != "" && w > 0) {
			return this.getMarkerImageFrom(url,w,h,a,v,surl,sw,sh);
		} else {
			return {
				icon	:	null,
				shadow	:	null
			};
		}
	},

	refreshKml : function () {
		if (this.kml_refreshing) return;
		this.kml_refreshing = true;
		var kml_refresh_url = this.kml_url;
		var this_gmap = this;
		if ((this.q["r"] != null && this.q["r"] == 1) || this.kml_overlay != null || this.kml_objects != null) {
			if (this.map.getBounds() != null) {
				kml_refresh_url += ((kml_refresh_url.search(/\?/) == -1) ? '?' : '&') + 'gmap_sw=' + this.map.getBounds().getSouthWest().lat() + ',' + this.map.getBounds().getSouthWest().lng() + '&gmap_ne=' + this.map.getBounds().getNorthEast().lat() + ',' + this.map.getBounds().getNorthEast().lng();
			} else {
				this.kml_refreshing = false;
				return;
			}
			if (this.kml_by_myself && this.kml_objects != null) {
				for (var i=0; i<this.kml_objects.length; i++) {
					if (this.kml_objects[i].info_window) this.kml_objects[i].info_window.close();
					this.kml_objects[i].setMap(null);
				}
				this.kml_objects = new Array();
			} else if (!this.kml_by_myself && this.kml_overlay != null) {
				this.kml_overlay.setMap(null);
			}
		}
		if (!this.kml_by_myself) {
			this.kml_overlay = new google.maps.KmlLayer(kml_refresh_url,{ map : this.map, preserveViewport : ((this.q["fit"] != null && this.q["fit"] == 1) ? false : true) });
			google.maps.event.addListener(this.kml_overlay,'click',function() { setTimeout(function() { this_gmap.changeLinkTarget(); }, 1000 ); });
			if (this.q["fit"] != null && this.q["fit"] == 1) this.saveMap();
		}
		if (this.kml_by_myself || (this.q["d"] != null && this.q["d"] >= 1)) {
			var request = this.createXmlHttpRequest();
			request.open("GET", kml_refresh_url, true);
			request.onreadystatechange = function () {
				if (request.readyState == 4) {
					var kml = this_gmap.xmlParse(request.responseText);
					if ((this_gmap.kml_by_myself && this_gmap.q["fit"] != null && this_gmap.q["fit"] == 1) || (this_gmap.q["d"] != null && this_gmap.q["d"] >= 1)) {
						var coordinates = kml.documentElement.getElementsByTagName("coordinates");
						var n;
						var e;
						var s;
						var w;
						for (var i=0; i<coordinates.length; i++) {
							var points = this_gmap.getNodeValue(coordinates[i]).replace(/^\s+/,'').replace(/\s+$/,'').split(/\s+/);
							var len = points.length;
							var center_n = Math.floor(len/2);
							var center_point = null;
							var start_point = null;
							var start_altitude = null;
							var prev_point = null;
							var prev_altitude = null;
							var distance = 0;
							for (var j=0; j<len; j++) {
								var lnglat = points[j].split(/,/);
								if (lnglat[0] != null && lnglat[1] != null) {
									var lng = parseFloat(lnglat[0]);
									var lat = parseFloat(lnglat[1]);
									var altitude = (lnglat[2] != null) ? parseFloat(lnglat[2]) : null;
									if (this_gmap.kml_by_myself && this_gmap.q["fit"] != null && this_gmap.q["fit"] == 1) {
										if (n == null || lat > n) n = lat;
										if (s == null || lat < s) s = lat;
										if (e == null || lng > e) e = lng;
										if (w == null || lng < w) w = lng;
									}
									if (this_gmap.q["d"] != null && this_gmap.q["d"] >= 1) {
										var here = new google.maps.LatLng(lat,lng);
										if (start_point == null) start_point = here;
										if (start_altitude == null) start_altitude = altitude;
										if (j == center_n && len > 1) center_point = here;
										if (prev_point != null) distance += this_gmap.distanceFrom(here,altitude,prev_point,prev_altitude);
										prev_point = here;
										prev_altitude = altitude;
									}
								}
							}
							if (this_gmap.q["d"] != null && this_gmap.q["d"] >= 1 && center_point != null) {
								var distance_info = this_gmap.formattedDistance(distance);
								if (len == 2) {
									center_point = new google.maps.LatLng(parseFloat((center_point.lat() + start_point.lat()) / 2),parseFloat((center_point.lng() + start_point.lng()) / 2));
								} else if (this_gmap.q["d"] == 2) {
									var shortest = this_gmap.distanceFrom(prev_point,prev_altitude,start_point,start_altitude);
									if (shortest != distance) distance_info += '<div style="font-size: 12px">(直線距離:' + this_gmap.formattedDistance(shortest) + ')</div>';
								}
								if (this_gmap.measure_icon == null) this_gmap.measure_icon = this_gmap.getMeasureImage();
								this_gmap.makeMarker(center_point,this_gmap.measure_icon,distance_info);
							}
						}
					}
					if (this_gmap.kml_by_myself) {
						if (this_gmap.list != null) {
							if (this_gmap.list_box != null) this_gmap.list_box.style.display = "none";
							while (this_gmap.list.hasChildNodes()) {
								this_gmap.list.removeChild(this_gmap.list.lastChild)
							}
						}
						if (this_gmap.kml_objects == null) this_gmap.kml_objects = new Array();
						if (this_gmap.default_icon == null) {
							if (kml.documentElement.getElementsByTagName("Style")[0] != null) {
								var first_style = kml.documentElement.getElementsByTagName("Style")[0];
								if ((first_style.getAttribute("id") == "defaultIcon" || (this_gmap.nosync && first_style.getAttribute("id") == "temporaryIcon")) && first_style.getElementsByTagName("IconStyle")[0] != null) {
									this_gmap.default_icon = this_gmap.getCustomMarkerImageFromKml(first_style.getElementsByTagName("IconStyle")[0]);
									this_gmap.togglablemarker_disabled = true;
								}
							}
							if (this_gmap.default_icon == null) this_gmap.default_icon = (this_gmap.q["mk"] != null && this_gmap.q["mk"] != "") ? this_gmap.getCustomMarkerImage(this_gmap.q["mk"],this_gmap.q["mka"],this_gmap.q["mkv"],this_gmap.q["mks"]) : this_gmap.getMarkerImage();
						}
						var network_links = kml.documentElement.getElementsByTagName("NetworkLink");
						var kml_layers = [];
						for (var i=0; i<network_links.length; i++) {
							var this_kml_url = this_gmap.getNodeValue(network_links[i].getElementsByTagName("href")[0]);
							if (this_gmap.isNewKml(this_kml_url)) {
								var this_kml = this_gmap.setKmlLayer(this_kml_url,true);
								if (network_links[i].getElementsByTagName("atom:link")[0] != null) kml_layers[this_gmap.getNodeValue(network_links[i].getElementsByTagName("atom:link")[0])] = this_kml;
								if (this_gmap.q["d"] != null && this_gmap.q["d"] >= 1) this_gmap.setDistance(this_kml_url,true);
							}
						}
						var placemarks = kml.documentElement.getElementsByTagName("Placemark");
						var center_placemark = null;
						for (var i=0; i<placemarks.length; i++) {
							var placemarker;
							var placemark_icon = null;
							var placemark_name = this_gmap.getNodeValue(placemarks[i].getElementsByTagName("name")[0]);
							var placemark_description = this_gmap.getNodeValue(placemarks[i].getElementsByTagName("description")[0]);
							var placemark_link = (placemarks[i].getElementsByTagName("atom:link")[0] != null) ? placemarks[i].getElementsByTagName("atom:link")[0].getAttribute("href") : (placemarks[i].getElementsByTagName("link")[0] != null) ? placemarks[i].getElementsByTagName("link")[0].getAttribute("href") : null;
							var placemark_id = (placemarks[i].getElementsByTagName("atom:id")[0] != null) ? this_gmap.getNodeValue(placemarks[i].getElementsByTagName("atom:id")[0]) : (placemarks[i].getElementsByTagName("id")[0] != null) ? this_gmap.getNodeValue(placemarks[i].getElementsByTagName("id")[0]) : null;
							var placemark_lnglat = this_gmap.getNodeValue(placemarks[i].getElementsByTagName("coordinates")[0]).split(/,/);
							var placemark_point = (placemark_lnglat.length <= 1) ? this_gmap.map.getCenter() : new google.maps.LatLng(parseFloat(placemark_lnglat[1]),parseFloat(placemark_lnglat[0]));
							if (placemarks[i].getElementsByTagName("IconStyle")[0] != null) placemark_icon = this_gmap.getCustomMarkerImageFromKml(placemarks[i].getElementsByTagName("Style")[0]);
							placemarker = this_gmap.makeMarker(placemark_point,((placemark_icon == null) ? this_gmap.default_icon : placemark_icon),placemark_name,placemark_description,((kml_layers[placemark_link] != null) ? [kml_layers[placemark_link]] : null));
							if (!this_gmap.nosync && placemark_link != null && placemark_link != "") {
								this_gmap.setMouseOverImage(
									placemark_link,
									placemarker,
									placemark_name,
									placemark_description,
									((placemark_icon == null) ? this_gmap.togglablemarker_disabled : true)
								);
							}
							this_gmap.kml_objects.push(placemarker);
							if (this_gmap.q["opn"] != null && (this_gmap.q["opn"] == placemark_id || this_gmap.q["opn"] == placemark_link)) {
								center_placemark = {
									"link"				:	placemark_link,
									"marker"			:	placemarker,
									"name"				:	placemark_name,
									"description"		:	placemark_description,
									"is_custom_icon"	:	((placemark_icon == null) ? this_gmap.togglablemarker_disabled : true)
								};
								this_gmap.q["opn"] = null;
							}
							if (this_gmap.list != null && placemark_link != null) {
								var list_a = this_gmap._caller.document.createElement("a");
								list_a.href = placemark_link;
								if (this_gmap.q["tr"] != null && this_gmap.q["tr"] == "p") {
									list_a.target = "_self";
								} else if (this_gmap.q["tr"] != null && this_gmap.q["tr"] == "t") {
									list_a.target = "_top";
								} else if (this_gmap.q["tr"] != null && this_gmap.q["tr"] == "s") {
									list_a.target = "_self";
								} else if (this_gmap.q["tr"] != null && this_gmap.q["tr"] == "d") {
									list_a.target = "detailViewer";
									this_gmap.addEventListener(list_a,'click',function () { window.open('','detailViewer','width='+this_gmap.dw+',height='+this_gmap.dh+',scrollbars=yes,resizable=yes'); });
								} else {
									list_a.target = "_blank";
								}
								this_gmap.setMouseOverImage(
									placemark_link,
									placemarker,
									placemark_name,
									placemark_description,
									((placemark_icon == null) ? this_gmap.togglablemarker_disabled : true),
									[list_a]
								);

								var list_img = this_gmap._caller.document.createElement('img');
								list_img.src = (placemark_icon) ? placemark_icon.icon.image_url : this_gmap.default_icon.icon.image_url;
								list_a.appendChild(list_img);
								list_a.appendChild(this_gmap._caller.document.createTextNode(placemark_name));
								var list_li = this_gmap._caller.document.createElement('li');
								list_li.appendChild(list_a);
								this_gmap.list.appendChild(list_li);
							}
						}
						var screen_overlays = kml.documentElement.getElementsByTagName("ScreenOverlay");
						if (screen_overlays && screen_overlays[0]) {
							var screen_overlay = screen_overlays[0];
							if (screen_overlay.getElementsByTagName("Icon")[0] != null) {
								var screen_overlay_icon = this_gmap.getNodeValue(screen_overlay.getElementsByTagName("Icon")[0].getElementsByTagName("href")[0]);
								var w = ((self.innerWidth) ? self.innerWidth : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientWidth : self.document.body.clientWidth));
								var h = ((self.innerHeight) ? self.innerHeight : ((document.compatMode != null && document.compatMode != "BackCompat") ? self.document.documentElement.clientHeight : self.document.body.clientHeight))
								var screen_overlay_name = this_gmap.getNodeValue(screen_overlay.getElementsByTagName("name")[0]);
								var screen_overlay_description = this_gmap.getNodeValue(screen_overlay.getElementsByTagName("description")[0]);
								if (screen_overlay.getElementsByTagName("size")[0] != null) {
									var thisSize = screen_overlay.getElementsByTagName("size")[0];
									var screen_overlay_width = parseInt(thisSize.getAttribute("width"));
									var screen_overlay_height = parseInt(thisSize.getAttribute("height"));
								} else {
									var screen_overlay_width = parseInt(w/2);
									var screen_overlay_height = parseInt(h/2);
								}
							}
							var overlay_msg = document.createElement('div');
							overlay_msg.style.position = 'absolute';
							overlay_msg.style.top = parseInt((h - screen_overlay_height) / 2) + 'px';
							overlay_msg.style.left = parseInt((w - screen_overlay_width) / 2) + 'px';
							overlay_msg.style.width = screen_overlay_width + 'px';
							overlay_msg.style.height = screen_overlay_height + 'px';
							if (screen_overlay_icon) overlay_msg.style.backgroundImage = 'url("' + screen_overlay_icon + '")';
							self.document.getElementsByTagName("body")[0].appendChild(overlay_msg);
							overlay_msg.setMap = function() { this.parentNode.removeChild(this) };
							this_gmap.kml_objects.push(overlay_msg);
						}
						if (this_gmap.q["fit"] != null && this_gmap.q["fit"] == 1 && n != null && s != null && e != null && w != null) {
							this_gmap.map.fitBounds(new google.maps.LatLngBounds(new google.maps.LatLng(s,w),new google.maps.LatLng(n,e)));
							this_gmap.saveMap();
						}
						if (center_placemark != null) {
							if (!center_placemark.is_custom_icon) this_gmap.toggleLinkMarkerImage(center_placemark.marker,true);
							if ((center_placemark.description != null && center_placemark.description != "") || (center_placemark.title != null && center_placemark.title != "")) {
								if (!center_placemark.marker.info_window) center_placemark.marker.info_window = new google.maps.InfoWindow({
									content		: ((center_placemark.description != null && center_placemark.description != "") ? center_placemark.description : center_placemark.title),
									maxWidth	: this_gmap.mw
								});
								center_placemark.marker.info_window.open(this_gmap.map,center_placemark.marker);
								setTimeout(function() { this_gmap.changeLinkTarget(); }, 1000 );
							}
						}
						if (this_gmap.list != null && this_gmap.list.hasChildNodes() && this_gmap.list_box != null) this_gmap.list_box.style.display = "";
					}
					this_gmap.kml_refreshing = false;
				}
			}
			request.send(null);
		}
	},

	setWaypoints : function (wp) {
		if (wp != null) {
			var this_gmap = this;
			var request = this.createXmlHttpRequest();
			request.open("GET", wp, true);
			request.onreadystatechange = function () {
				if (request.readyState == 4) {
					var default_icon = this_gmap.getMarkerImage();
					var kml = this_gmap.xmlParse(request.responseText);
					var placemarks = kml.documentElement.getElementsByTagName("Placemark");
					for (var i=0; i<placemarks.length; i++) {
						var placemarker;
						var placemark_icon = null;
						var placemark_name = this_gmap.getNodeValue(placemarks[i].getElementsByTagName("name")[0]);
						var placemark_description = this_gmap.getNodeValue(placemarks[i].getElementsByTagName("description")[0]);
						var placemark_link = (placemarks[i].getElementsByTagName("atom:link")[0] != null) ? placemarks[i].getElementsByTagName("atom:link")[0].getAttribute("href") : (placemarks[i].getElementsByTagName("link")[0] != null) ? placemarks[i].getElementsByTagName("link")[0].getAttribute("href") : null;
						var placemark_id = (placemarks[i].getElementsByTagName("atom:id")[0] != null) ? this_gmap.getNodeValue(placemarks[i].getElementsByTagName("atom:id")[0]) : (placemarks[i].getElementsByTagName("id")[0] != null) ? this_gmap.getNodeValue(placemarks[i].getElementsByTagName("id")[0]) : null;
						var placemark_lnglat = this_gmap.getNodeValue(placemarks[i].getElementsByTagName("coordinates")[0]).split(/,/);
						var placemark_point = (placemark_lnglat.length <= 1) ? this_gmap.map.getCenter() : new google.maps.LatLng(parseFloat(placemark_lnglat[1]),parseFloat(placemark_lnglat[0]));
						if (placemarks[i].getElementsByTagName("IconStyle")[0] != null) placemark_icon = this_gmap.getCustomMarkerImageFromKml(placemarks[i].getElementsByTagName("Style")[0]);
						placemarker = this_gmap.makeMarker(placemark_point,((placemark_icon == null) ? default_icon : placemark_icon),placemark_name,placemark_description);
					}
					var network_links = kml.documentElement.getElementsByTagName("NetworkLink");
					for (var i=0; i<network_links.length; i++) {
						var url = this_gmap.getNodeValue(network_links[i].getElementsByTagName("href")[0]);
						if (this_gmap.isNewKml(url)) {
							this_gmap.setKmlLayer(url);
							if (this_gmap.q["d"] != null && this_gmap.q["d"] >= 1) this_gmap.setDistance(url,false);
						}
					}
				}
			}
			request.send(null);
		}
	},

	isNewKml : function (url) {
		if ((this.q["url"] && url == this.q["url"]) || (this.kml_url && url == this.kml_url)) return false;
		if (this.q["wp"] != null) {
			for (var i = 0; i < this.q["wp"].length; i++) {
				if (this.q["wp"][i] == url) return false;
			}
		}
		return true;
	},

	setDistance : function (url,register) {
		var this_gmap = this;
		var request = this.createXmlHttpRequest();
		request.open("GET", url, true);
		request.onreadystatechange = function () {
			if (request.readyState == 4) {
				var kml = this_gmap.xmlParse(request.responseText);
				var coordinates = kml.documentElement.getElementsByTagName("coordinates");
				for (var i=0; i<coordinates.length; i++) {
					var points = this_gmap.getNodeValue(coordinates[i]).replace(/^\s+/,'').replace(/\s+$/,'').split(/\s+/);
					var len = points.length;
					var center_n = Math.floor(len/2);
					var center_point = null;
					var start_point = null;
					var start_altitude = null;
					var prev_point = null;
					var prev_altitude = null;
					var distance = 0;
					for (var j=0; j<len; j++) {
						var lnglat = points[j].split(/,/);
						if (lnglat[0] != null && lnglat[1] != null) {
							var lng = parseFloat(lnglat[0]);
							var lat = parseFloat(lnglat[1]);
							var altitude = (lnglat[2] != null) ? parseFloat(lnglat[2]) : null;
							var here = new google.maps.LatLng(lat,lng);
							if (start_point == null) start_point = here;
							if (start_altitude == null) start_altitude = altitude;
							if (j == center_n && len > 1) center_point = here;
							if (prev_point != null) distance += this_gmap.distanceFrom(here,altitude,prev_point,prev_altitude);
							prev_point = here;
							prev_altitude = altitude;
						}
					}
					if (center_point != null) {
						var distance_info = this_gmap.formattedDistance(distance);
						if (len == 2) {
							center_point = new google.maps.LatLng(parseFloat((center_point.lat() + start_point.lat()) / 2),parseFloat((center_point.lng() + start_point.lng()) / 2));
						} else if (this_gmap.q["d"] == 2) {
							var shortest = this_gmap.distanceFrom(prev_point,prev_altitude,start_point,start_altitude);
							if (shortest != distance) distance_info += '<div style="font-size: 12px">(直線距離:' + this_gmap.formattedDistance(shortest) + ')</div>';
						}
						if (this_gmap.measure_icon == null) this_gmap.measure_icon = this_gmap.getMeasureImage();
						var measure_point = this_gmap.makeMarker(center_point,this_gmap.measure_icon,distance_info);
						if (register) this_gmap.kml_objects.push(measure_point);
					}
				}
			}
		}
		request.send(null);
	},

	setKmlLayer : function (kml_link,is_kml_object) {
		if (kml_link != null) {
			var this_gmap = this;
			var kml_layer = new google.maps.KmlLayer(kml_link,{ preserveViewport : true });
			kml_layer.setMap(this.map);
			google.maps.event.addListener(kml_layer,'click',function() { setTimeout(function() { this_gmap.changeLinkTarget(); }, 1000 ); });
			if (is_kml_object) this.kml_objects.push(kml_layer);
			return kml_layer;
		}
		return null;
	},

	setMouseOverImage : function (placemark_link,placemarker,marker_title,marker_description,is_custom_icon,caller_links) {
		if (!caller_links) caller_links = this._caller.document.getElementsByTagName('a');
		for (var i=0; i<caller_links.length; i++) {
			if (caller_links[i].href == placemark_link) {
				var this_gmap = this;
				google.maps.event.addDomListener(
					caller_links[i],
					'mouseover',
					function () {
						if (!is_custom_icon) this_gmap.toggleLinkMarkerImage(placemarker,true);
						if ((marker_description != null && marker_description != "") || (marker_title != null && marker_title != "")) {
							if (!placemarker.info_window) placemarker.info_window = new google.maps.InfoWindow({
								content		: ((marker_description != null && marker_description != "") ? marker_description : marker_title),
								maxWidth	: this_gmap.mw
							});
							placemarker.info_window.open(this_gmap.map,placemarker);
							this_gmap.changeLinkTarget();
						}
					}
				);
				if (!is_custom_icon) {
					google.maps.event.addDomListener(
						caller_links[i],
						'mouseout',
						function () {
							this_gmap.toggleLinkMarkerImage(placemarker,false);
						}
					);
				}
			}
		}
	},

	formattedDistance : function (meter) {
		return (meter > 1000) ? (parseInt(meter) / 1000) + 'km' : parseInt(meter) + 'm';
	},

	makeMarker : function (point,marker_image,marker_title,marker_description,related_objects){
		var marker = new google.maps.Marker({
			position	: point,
			map			: this.map,
			icon		: marker_image.icon,
			shadow		: marker_image.shadow,
			flat		: ((marker_image.shadow == null) ? true : false),
			title		: marker_title.replace(/<.*?>/g,'')
		});
		var this_gmap = this;
		var func = function () {
			if (!marker.info_window) marker.info_window = new google.maps.InfoWindow({
				content		: ((marker_description != null && marker_description != "") ? marker_description : marker_title),
				maxWidth	: this_gmap.mw
			});
			marker.info_window.open(this_gmap.map,marker);
			this_gmap.changeLinkTarget();
		};
		google.maps.event.addListener(marker,"click",func);
		if (related_objects != null) {
			for (var i=0; i<related_objects.length; i++) {
				google.maps.event.addListener(related_objects[i],"click",func);
			}
		}
		return marker;
	},

	geocoderSearch : function () {
		var keyword = self.document.getElementById('geocoderkey').value;
		if (keyword != null && keyword != "") {
			if (this.geocoder_client == null) this.geocoder_client = new google.maps.Geocoder();
			var this_gmap = this;
			this.geocoder_client.geocode(
				{ address : keyword },
				function (results, status) {
					if (status == google.maps.GeocoderStatus.OK) {
						var z = this_gmap.map.getZoom();
						this_gmap.map.setCenter(results[0].geometry.location);
						this_gmap.map.setZoom((z < 10) ? 15 : z);
						this_gmap.saveMap();
						this_gmap.centerMarker();
					} else {
						alert("条件に該当する地図は見つかりませんでした");
					}
				}
			);
		}
	},

	toggleSearchBox : function () {
		var search_box = self.document.getElementById("geocoder-search");
		if (search_box.style.display == null || search_box.style.display == 'none') {
			var addr = '';
			if (this.addr_field_ids != null && this.addr_field_ids.constructor == Array && this.addr_field_ids.length > 0) {
				for (var i in this.addr_field_ids) {
					if (this._caller.document.getElementById(this.addr_field_ids[i])) {
						var addr_field = this._caller.document.getElementById(this.addr_field_ids[i])
						if (addr_field.type != null && addr_field.type == 'select') {
							addr += addr_field.options[addr_field.options.selectedIndex].value;
						} else {
							addr += addr_field.value;
						}
					}
				}
				if (addr != "") self.document.getElementById("geocoderkey").value = addr;
			}
			search_box.style.display = 'block';
			self.document.getElementById("geocoderkey").focus();
			if (addr == "") self.document.getElementById("geocoderkey").select();
		} else {
			search_box.style.display = 'none';
		}
	},

	saveMap : function () {
		if (this.z_field != null) this.setFieldValue(this.z_field,this.map.getZoom());
		if (this.t_field != null) {
			var map_type = this.map.getMapTypeId();
			this.setFieldValue(this.t_field,((map_type == google.maps.MapTypeId.SATELLITE) ? 'k' : (map_type == google.maps.MapTypeId.HYBRID) ? 'h' : (map_type == google.maps.MapTypeId.TERRAIN) ? 'p' : ''));
		}
		if (this.c_field != null) this.setFieldValue(this.c_field,this.map.getCenter().lat() + ',' + this.map.getCenter().lng());
		if (this.fit_field != null) this.fit_field.checked = false;
	},

	setFieldValue : function (e,v) {
		for (var i=0; i < e.length; i++) {
			if (e[i]) e[i].value = v;
		}
	},

	centerMarker : function () {
		if (this.center_marker != null) this.center_marker.setPosition(this.map.getCenter());
	},

	addEventListener : function (element,type,func,capture) {
		if (capture == null) capture = false;
		if (typeof element.addEventListener =='function') {
			element.addEventListener(type, func, capture);
		} else if (typeof element.attachEvent == 'object') {
			element.attachEvent("on" + type, func);
		}
		return func;
	},

	urlDecode : function (query) {
		var hash = new Array();
		if (query == null || query == "" || query.search("=") == -1) return hash;
		query = query.replace(/^\?/,'').split('&');
		for (var i=0; i<query.length; i++) {
			var p = query[i].split('=');
			if (window.decodeURIComponent) {
				p[0] = decodeURIComponent(p[0].replace(/\+/g,' '));
				p[1] = decodeURIComponent(p[1].replace(/\+/g,' '));
			} else {
				p[0] = unescape(p[0].replace(/\+/g,' '));
				p[1] = unescape(p[1].replace(/\+/g,' '));
			}
			if (p[0] == 'bg') p[0] = 'wp';
			if (hash[p[0]] == null) {
				hash[p[0]] = p[1];
			} else {
				if (hash[p[0]].constructor == Array) {
					hash[p[0]].push(p[1]);
				} else {
					hash[p[0]] = new Array(hash[p[0]],p[1]);
				}
			}
		}
		return hash;
	},

	urlEncode : function (hash) {
		var query = '';
		for (var key in hash) {
			if (hash[key] != null) {
				var encoded_key = ((window.encodeURIComponent) ? encodeURIComponent(key) : escape(key)).replace(/ /g,'+')
				if (hash[key].constructor == Array) {
					for (var i=0; i<hash[key].length; i++) {
						if ((typeof hash[key][i] == "string" || typeof hash[key][i] == "number") && hash[key][i] != null) query += ((query != "") ? "&" : "") + encoded_key + "=" + ((window.encodeURIComponent) ? encodeURIComponent(hash[key][i]) : escape(hash[key][i])).replace(/ /g,'+');
					}
				} else if (typeof hash[key] == "string" || typeof hash[key] == "number") {
					query += ((query != "") ? "&" : "") + encoded_key + "=" + ((window.encodeURIComponent) ? encodeURIComponent(hash[key]) : escape(hash[key])).replace(/ /g,'+');
				}
			}
		}
		return query;
	},

	getHashFromForm : function (this_form) {
		var hash = new Array();
		for (var i=0; i < this_form.elements.length; i++) {
			if (typeof this_form.elements[i].name != "undefined" && this_form.elements[i].name != "" && (typeof this_form.elements[i].value == "string" || typeof this_form.elements[i].value == "number")) {
				var name = this_form.elements[i].name;
				if (this_form.elements[i].type == null || typeof this_form.elements[i].type == "undefined") {
					hash[name] = this._setHashValue(hash[name],this_form.elements[i].value);
				} else if (this_form.elements[i].type == 'text' || this_form.elements[i].type == 'password' || this_form.elements[i].type == 'textarea' || this_form.elements[i].type == 'hidden' || this_form.elements[i].type == 'submit' || this_form.elements[i].type == 'image' || this_form.elements[i].type == 'search') {
					if (this_form.elements[i].type == 'hidden' && this_form.elements[i].getAttribute('alt') && typeof this_form.elements[i].getAttribute('alt') == 'string') {
						hash[name] = this._setHashValue(hash[name],this_form.elements[i].getAttribute('alt'));
					} else {
						hash[name] = this._setHashValue(hash[name],this_form.elements[i].value);
					}
				} else if (this_form.elements[i].type == 'checkbox' || this_form.elements[i].type == 'radio') {
					if (this_form.elements[i].checked) hash[name] = this._setHashValue(hash[name],this_form.elements[i].value);
				} else if (this_form.elements[i].type.search(/select/) != -1) {
					for (var o=0; o < this_form.elements[i].options.length; o++) {
						if (this_form.elements[i].options[o].selected) hash[name] = this._setHashValue(hash[name],this_form.elements[i].options[o].value);
					}
				}
			}
		}
		return hash;
	},

	_setHashValue : function (p,v) {
		if (p != null) {
			if (p.constructor == Array) {
				p.push(v);
			} else {
				p = new Array(p,v);
			}
		} else {
			p = v;
		}
		return p;
	},

	createXmlHttpRequest : function () {
		try {
			if (typeof ActiveXObject != 'undefined') {
				return new ActiveXObject('Microsoft.XMLHTTP');
			} else if (window["XMLHttpRequest"]) {
				return new XMLHttpRequest();
			}
		} catch (e) {
			changeStatus(e);
		}
		return null;
	},

	xmlParse : function (str) {
		if (typeof ActiveXObject != 'undefined' && typeof GetObject != 'undefined') {
			var doc = new ActiveXObject('Microsoft.XMLDOM');
			doc.loadXML(str);
			return doc;
		}

		if (typeof DOMParser != 'undefined') {
			return (new DOMParser()).parseFromString(str, 'text/xml');
		}

		return createElement('div', null);
	},

	getNodeValue : function (node){
		if (!node) {
			return"";
		}
		var retStr="";
		if (node.nodeType==3 || node.nodeType==4 || node.nodeType==2) {
			retStr += node.nodeValue;
		} else if(node.nodeType==1 || node.nodeType==9 || node.nodeType==11) {
			for (var i=0; i<node.childNodes.length; ++i) {
				retStr += arguments.callee(node.childNodes[i]);
			}
		}
		return retStr;
	}
};

gmap.prepare();
